#include <sdk_version.h>
#include <cellstatus.h>
#include <cell/cell_fs.h>
#include <cell/rtc.h>
#include <cell/gcm.h>
#include <cell/pad.h>
#include <sys/vm.h>
#include <sysutil/sysutil_common.h>

#include <sys/prx.h>
#include <sys/ppu_thread.h>
#include <sys/event.h>
#include <sys/syscall.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/memory.h>
#include <sys/timer.h>
#include <sys/process.h>

#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <netex/net.h>
#include <netex/errno.h>
#include <netex/libnetctl.h>
#include <netex/sockinfo.h>

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

//#define USE_DEBUG 1
//#define CCAPI 1		// uncomment for ccapi release
#define COBRA_ONLY 1	// comment out for ccapi/non-cobra release

#include "types.h"
#include "common.h"
#include "printf.h"
#include "cobra/storage.h"

#ifdef COBRA_ONLY
#include "cobra/cobra.h"
#include "cobra/netiso.h"
#endif

SYS_MODULE_INFO(WWWD, 0, 1, 0);
SYS_MODULE_START(wwwd_start);
SYS_MODULE_STOP(wwwd_stop);

#define WM_VERSION			"1.30.7 MOD"						// webMAN version
#define MM_ROOT_STD			"/dev_hdd0/game/BLES80608/USRDIR"	// multiMAN root folder
#define MM_ROOT_SSTL		"/dev_hdd0/game/NPEA00374/USRDIR"	// multiman SingStar® Stealth root folder
#define MM_ROOT_STL			"/dev_hdd0/tmp/game_repo/main"		// stealthMAN root folder

#define WMCONFIG			"/dev_hdd0/tmp/wmconfig.bin"		// webMAN config file
#define WMTMP				"/dev_hdd0/tmp/wmtmp"				// webMAN work/temp folder

#define THREAD_NAME 		"wwwdt"
#define THREAD_NAME_FTP 	"ftpdt"
#define THREAD_NAME_NET 	"netiso"
#define THREAD_NAME_NTFS 	"ntfsd"

#define STOP_THREAD_NAME 	"wwwds"

#define WWWPORT			(80)
#define FTPPORT			(21)

#define ssend(socket, str) send(socket, str, strlen(str), 0)
#define getPort(p1x, p2x) ((p1x * 256) + p2x)

static u32 BUFFER_SIZE_FTP	= ( 128*1024);

static u32 BUFFER_SIZE		= ( 448*1024);
static u32 BUFFER_SIZE_PSX	= ( 160*1024);
static u32 BUFFER_SIZE_PSP	= (  32*1024);
static u32 BUFFER_SIZE_PS2	= (  64*1024);
static u32 BUFFER_SIZE_DVD	= ( 192*1024);
static u32 BUFFER_SIZE_ALL	= ( 896*1024);

static sys_ppu_thread_t thread_id_ntfs = -1;
static sys_ppu_thread_t thread_id_net	=-1;
static sys_ppu_thread_t thread_id_poll	=-1;
static sys_ppu_thread_t thread_id_ftp	=-1;
static sys_ppu_thread_t thread_id		=-1;

#define MIN(a, b)	((a) <= (b) ? (a) : (b))
#define ABS(a)		(((a) < 0) ? -(a) : (a))

#define CD_CACHE_SIZE			(64)

#define ATA_HDD				0x101000000000007ULL
#define BDVD_DRIVE			0x101000000000006ULL
#define PATA0_HDD_DRIVE		0x101000000000008ULL
#define PATA0_BDVD_DRIVE	BDVD_DRIVE
#define PATA1_HDD_DRIVE		ATA_HDD
#define PATA1_BDVD_DRIVE	0x101000000000009ULL
#define BUILTIN_FLASH		0x100000000000001ULL
#define MEMORY_STICK		0x103000000000010ULL
#define SD_CARD				0x103000100000010ULL
#define COMPACT_FLASH		0x103000200000010ULL

#define USB_MASS_STORAGE_1(n)	(0x10300000000000AULL+n) /* For 0-5 */
#define USB_MASS_STORAGE_2(n)	(0x10300000000001FULL+(n-6)) /* For 6-127 */

#define	HDD_PARTITION(n)	(ATA_HDD | ((uint64_t)n<<32))
#define FLASH_PARTITION(n)	(BUILTIN_FLASH | ((uint64_t)n<<32))

#define DEVICE_TYPE_PS3_DVD	0xFF70
#define DEVICE_TYPE_PS3_BD	0xFF71
#define DEVICE_TYPE_PS2_CD	0xFF60
#define DEVICE_TYPE_PS2_DVD	0xFF61
#define DEVICE_TYPE_PSX_CD	0xFF50
#define DEVICE_TYPE_BDROM	0x40
#define DEVICE_TYPE_BDMR_SR	0x41 /* Sequential record */
#define DEVICE_TYPE_BDMR_RR 0x42 /* Random record */
#define DEVICE_TYPE_BDMRE	0x43
#define DEVICE_TYPE_DVD		0x10 /* DVD-ROM, DVD+-R, DVD+-RW etc, they are differenced by booktype field in some scsi command */
#define DEVICE_TYPE_CD		0x08 /* CD-ROM, CD-DA, CD-R, CD-RW, etc, they are differenced somehow with scsi commands */


enum STORAGE_COMMAND
{
	CMD_READ_ISO,
	CMD_READ_DISC,
	CMD_READ_CD_ISO_2352,
	CMD_FAKE_STORAGE_EVENT,
	CMD_GET_PSX_VIDEO_MODE
};

#ifdef COBRA_ONLY
typedef struct
{
	char server[0x40];
	char path[0x420];
	uint32_t emu_mode;
	uint32_t numtracks;
	uint16_t port;
	uint8_t pad[6];
	ScsiTrackDescriptor tracks[1];
} __attribute__((packed)) netiso_args;

typedef struct
{
	uint64_t device;
        uint32_t emu_mode;
        uint32_t num_sections;
        uint32_t num_tracks;
} __attribute__((packed)) rawseciso_args;


static uint32_t *sections, *sections_size;
static uint32_t num_sections;
uint32_t handle = -1;
static sys_event_queue_t command_queue_ntfs = -1;
static u8 netiso_loaded=0;
static u8 rawseciso_loaded=0;
#endif

typedef struct {
	uint32_t total;
	uint32_t avail;
} _meminfo;

static u8 loading_html=0;
static u8 loading_games=0;
static u8 init_running=0;
static int g_socket = -1;
static sys_event_queue_t command_queue = -1;

static uint64_t discsize;
static int is_cd2352;
static uint8_t *cd_cache;
static uint32_t cached_cd_sector=0x80000000;

#define MIN_FANSPEED	(25)
#define MAX_FANSPEED	(0xE6)
#define MY_TEMP 		(80)
static u8 fan_speed=0x33;
static u8 old_fan=0x33;
static u32 max_temp=MY_TEMP;

#define MAX_LAST_GAMES (5)
typedef struct
{
	uint8_t last;
	char game[MAX_LAST_GAMES][512];
} __attribute__((packed)) _lastgames;
static _lastgames lastgames;

#ifdef USE_DEBUG
static int debug_s=-1;
static char debug[256];
#endif

static volatile u8 working = 1;
static u8 cobra_mode=0;
static u8 max_mapped=0;

static float c_firmware=0.0f;
static u8 dex_mode=0;

#ifndef COBRA_ONLY
static u64 SYSCALL_TABLE = SYSCALL_TABLE_450;

typedef struct
{
	char src[384];
	char dst[384];
} redir_files_struct;
static redir_files_struct file_to_map[8];
#endif


typedef struct
{
	uint8_t usb0;
	uint8_t usb1;
	uint8_t usb2;
	uint8_t usb3;
	uint8_t usb6;
	uint8_t usb7;
	uint8_t netd0;
	uint8_t lastp;
	uint8_t autob;
	uint8_t delay;
	uint8_t bootd;
	uint8_t boots;
	uint8_t blind;
	uint8_t nogrp;
	uint8_t noset;
	uint8_t cmask;
	uint32_t netp0;
	char neth0[16];
	uint8_t poll;
	uint8_t ftpd;
	uint8_t warn;
	uint8_t fanc;
	uint8_t temp1;
	uint8_t fanm;	//unused
	uint8_t bind;
	uint8_t refr;
	uint8_t manu;
	uint8_t temp0;
	uint8_t netd1;
	uint32_t netp1;
	char neth1[16];
	uint8_t foot;
	uint8_t nopad;
	uint8_t nocov;
	uint8_t nospoof;
	uint8_t ps2temp;
	uint8_t pspl;
	uint8_t minfan;
	uint16_t combo;
	uint8_t sidps;
	uint8_t spsid;
	uint8_t spp;
	uint8_t lang;
	char vIDPS1[17];
	char vIDPS2[17];
	char vPSID1[17];
	char vPSID2[17];
	uint8_t tid;
} __attribute__((packed)) WebmanCfg;

#define FAIL_SAFE (1<<0)
#define SHOW_TEMP (1<<1)
#define PREV_GAME (1<<2)
#define NEXT_GAME (1<<3)
#define SHUT_DOWN (1<<4)
#define RESTARTPS (1<<5)
#define UNLOAD_WM (1<<6)
#define MANUALFAN (1<<7)
#define SHOW_IDPS (1<<8)
#define DISABLESH (1<<9)
#define DISABLEFC (1<<10)
#define MINDYNFAN (1<<11)

void reset_settings(void);
void save_settings(void);
static u64 backup[6];

static u8 wmconfig[sizeof(WebmanCfg)];
static WebmanCfg *webman_config = (WebmanCfg*) wmconfig;

static char smonth[12][4]={"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

static char drives[10][16]={"/dev_hdd0", "/dev_usb000", "/dev_usb001", "/dev_usb002", "/dev_usb003", "/dev_usb006", "/dev_usb007", "/net0", "/net1", "/ext"};
static char paths [11][16]={"GAMES", "GAMEZ", "PS3ISO", "BDISO", "DVDISO", "PS2ISO", "PSXISO", "PSXGAMES", "PSPISO", "ISO", "video"};

static bool covers_exist[5];

uint64_t convertH(char *val);
uint64_t find_syscall();
uint64_t search64(uint64_t val);
uint64_t find_syscall_table();
void remove_cfw_syscalls();
void clean_history();
bool language(const char *file_str, char *default_str);
uint64_t IDPS[2] = {0,0};
uint64_t PSID[2] = {0,0};
int lang_pos;

char STR_TRADBY[150]		= "";

char STR_FILES[30]			= "Files";
char STR_GAMES[30]			= "Games";
char STR_SETUP[30]			= "Setup";
char STR_EJECT[50]			= "Eject";
char STR_INSERT[50]			= "Insert";
char STR_UNMOUNT[50]		= "Unmount";
char STR_REFRESH[50]		= "Refresh";
char STR_SHUTDOWN[50]		= "Shutdown";
char STR_RESTART[50]		= "Restart";

char STR_BYTE[10]			= "b";
char STR_KILOBYTE[10]		= "KB";
char STR_MEGABYTE[10]		= "MB";
char STR_GIGABYTE[10]		= "GB";

char STR_SCAN1[100]			= "Scan these devices";
char STR_SCAN2[100]			= "Scan for content";
char STR_PSPL[100]			= "Show PSP Launcher";
char STR_VIDLG[30]			= "Video";
char STR_LPG[100]			= "Load last-played game on startup";
char STR_AUTOB[150]			= "Check for /dev_hdd0/PS3ISO/AUTOBOOT.ISO on startup";
char STR_DELAYAB[200]		= "Delay loading of AUTOBOOT.ISO/last-game (Disc Auto-start)";
char STR_DEVBL[150]			= "Enable /dev_blind (writable /dev_flash) on startup";
char STR_CONTSCAN[150]		= "Disable content scan on startup";
char STR_USBPOLL[100]		= "Disable USB polling";
char STR_FTPSVC[100]		= "Disable FTP service";
char STR_COMBOS[100]		= "Disable all PAD shortcuts";
char STR_MMCOVERS[100]		= "Disable multiMAN covers";
char STR_ACCESS[100]		= "Disable remote access to FTP/WWW services";
char STR_NOSETUP[150]		= "Disable webMAN Setup entry in \"My Games\"";
char STR_NOSPOOF[100]		= "Disable firmware version spoofing";
char STR_NOGRP[100]			= "Disable grouping of content in \"My Games\"";
char STR_TITLEID[100]		= "Include the ID as part of the title of the game";
char STR_FANCTRL[100]		= "Enable dynamic fan control";
char STR_NOWARN[100]		= "Disable temperature warnings";
char STR_AUTOAT[100]		= "Auto at";
char STR_LOWEST[30]			= "Lowest";
char STR_FANSPEED[80]		= "fan speed";
char STR_MANUAL[30]			= "Manual";
char STR_PS2EMU[100]		= "PS2 Emulator";
char STR_LANGAMES[100]		= "Scan for LAN games/videos";
char STR_ANYUSB[100]		= "Wait for any USB device to be ready";
char STR_ADDUSB[150]		= "Wait additionally for each selected USB device to be ready";
char STR_SPOOFID[150]		= "Change idps and psid in lv2 memory at system startup";
char STR_DELCFWSYS[200]		= "Disable lv1&lv2 peek&poke syscalls (6,7,9,10,36) and delete history files at system startup";
char STR_MEMUSAGE[100]		= "Plugin memory usage";
char STR_PLANG[100]			= "Plugin language";
char STR_COMBOS2[100]		= "XMB/In-Game PAD SHORTCUTS";
char STR_FAILSAFE[100]		= "FAIL SAFE";
char STR_SHOWTEMP[100]		= "SHOW TEMP";
char STR_SHOWIDPS[100]		= "SHOW IDPS";
char STR_PREVGAME[100]		= "PREV GAME";
char STR_NEXTGAME[100]		= "NEXT GAME";
char STR_SHUTDOWN2[100]		= "SHUTDOWN ";
char STR_RESTART2[100]		= "RESTART&nbsp; ";
char STR_DELCFWSYS2[100] 	= "DEL CFW SYSCALLS";
char STR_UNLOADWM[100]		= "UNLOAD WM";
char STR_FANCTRL2[100]		= "CTRL FAN";
char STR_FANCTRL4[100]		= "CTRL DYN FAN";
char STR_FANCTRL5[100]		= "CTRL MIN FAN";
char STR_UPDN[20]			= "&#8593;/&#8595;"; //↑/↓
char STR_LFRG[20]			= "&#8592;/&#8594;"; //←/→
char STR_SAVE[30]			= "Save";
char STR_SETTINGSUPD[250]	= "Settings updated.<br><br>Click <a href=\"/restart.ps3\">here</a> to restart your PLAYSTATIONR3 system.";
char STR_ERROR[30]			= "Error!";

char STR_MYGAMES[50]		= "My Games";
char STR_LOADGAMES[80]		= "Load games with webMAN";

char STR_WMSETUP[50]		= "webMAN Setup";
char STR_WMSETUP2[50]		= "Setup webMAN options";

char STR_EJECTDISC[50]		= "Eject Disc";
char STR_UNMOUNTGAME[100]	= "Unmount current game";

char STR_WMSTART[50]		= "webMAN loaded!";
char STR_WMUNL[80]			= "webMAN unloaded!";
char STR_CFWSYSALRD[130]	= "CFW Syscalls already disabled";
char STR_CFWSYSRIP[130]		= "Removal History files & CFW Syscalls in progress...";
char STR_RMVCFWSYS[130]		= "History files & CFW Syscalls deleted OK!";
char STR_RMVCFWSYSF[130]	= "Failed to remove CFW Syscalls";

char STR_RMVWMCFG[130]		= "webMAN config reset in progress...";
char STR_RMVWMCFGOK[130]	= "Done! Restart within 3 seconds";

char STR_PS3FORMAT[50]		= "PS3 format games";
char STR_PS2FORMAT[50]		= "PS2 format games";
char STR_PS1FORMAT[50]		= "PSOne format games";
char STR_PSPFORMAT[50]		= "PSP\xE2\x84\xA2 format games";

char STR_VIDFORMAT[50]		= "Blu-ray\xE2\x84\xA2 and DVD";
char STR_VIDEO[50]			= "Video content";

char STR_LAUNCHPSP[100]		= "Launch PSP ISO mounted through webMAN or mmCM";

char STR_GAMEUM[50]			= "Game unmounted.";

char STR_EJECTED[50]		= "Disc ejected.";
char STR_LOADED[50]			= "Disc inserted.";

char STR_GAMETOM[50]		= "Game to mount";
char STR_GAMELOADED[250]	= "Game loaded successfully. Start the game from the disc icon<br>or from <b>/app_home</b>&nbsp;XMB entry.<hr/>Click <a href=\"/mount.ps3/unmount\">here</a> to unmount the game.";
char STR_PSPLOADED[230]		= "Game loaded successfully. Start the game using <b>PSP Launcher</b>.<hr/>Click <a href=\"/mount.ps3/unmount\">here</a> to unmount the game.";
char STR_LOADED2[50]		= "loaded   ";

char STR_MOVIETOM[50]		= "Movie to mount";
char STR_MOVIELOADED[250]	= "Movie loaded successfully. Start the movie from the disc icon<br>under the Video column.<hr/>Click <a href=\"/mount.ps3/unmount\">here</a> to unmount the movie.";

char STR_XMLRF[200]			= "Game list refreshed (<a href=\"/dev_hdd0/xmlhost/game_plugin/mygames.xml\">mygames.xml</a>).<br>Click <a href=\"/restart.ps3\">here</a> to restart your PLAYSTATION®3 system now.";

char STR_STORAGE[50]		= "System storage";
char STR_MEMORY[50]			= "Memory available";
char STR_MBFREE[50]			= "MB free";
char STR_KBFREE[50]			= "KB free";

char STR_FANCTRL3[50]		= "Fan control:";
char STR_ENABLED[50]		= "Enabled";
char STR_DISABLED[50]		= "Disabled";

char STR_FANCH0[50]			= "Fan setting changed:";
char STR_FANCH1[50]			= "MAX TEMP: ";
char STR_FANCH2[50]			= "FAN SPEED: ";
char STR_FANCH3[50]			= "MIN FAN SPEED: ";

char STR_OVERHEAT[100]		= "System overheat warning!";
char STR_OVERHEAT2[100]		= "  OVERHEAT DANGER!\r\nFAN SPEED INCREASED!";


int wwwd_start(uint64_t arg);
int wwwd_stop(void);

static inline void get_temperature(u32 _dev, u32 *_temp);
void fan_control(u8 temp0, u8 maxtemp);
static inline void led(u64 color, u64 mode);
void restore_fan(u8 settemp);
int do_umount_iso(void);

void absPath(char* absPath_s, const char* path, const char* cwd);
int isDir(const char* path);
int ssplit(const char* str, char* left, int lmaxlen, char* right, int rmaxlen);
int slisten(int port, int backlog);
void sclose(int *socket_e);

int my_atoi(const char *c);

static void mount_with_mm(const char *_path, u8 do_eject);
void eject_insert(u8 eject, u8 insert);

int do_startup_msg = 1;
void show_msg(char* msg);

//int (*_cellGcmIoOffsetToAddress)(uint32_t, void**) = NULL;
int (*vshtask_notify)(int, const char *) = NULL;
/*
int (*vshmain_is_ss_enabled)() = NULL;
int (*set_SSHT_)(int) = NULL;
int (*BgmPlaybackEnable)(int,void *) = NULL;
int opd[2] = {0,0};
*/
void * getNIDfunc(const char * vsh_module, uint32_t fnid);


void * getNIDfunc(const char * vsh_module, uint32_t fnid)
{
	// 0x10000 = ELF
	// 0x10080 = segment 2 start
	// 0x10200 = code start

	uint32_t table = (*(uint32_t*)0x1008C) + 0x984; // vsh table address
//	uint32_t table = (*(uint32_t*)0x1002C) + 0x214 - 0x10000; // vsh table address
//	uint32_t table = 0x63A9D4;


	while(((uint32_t)*(uint32_t*)table) != 0)
	{
		uint32_t* export_stru_ptr = (uint32_t*)*(uint32_t*)table; // ptr to export stub, size 2C, "sys_io" usually... Exports:0000000000635BC0 stru_635BC0:    ExportStub_s <0x1C00, 1, 9, 0x39, 0, 0x2000000, aSys_io, ExportFNIDTable_sys_io, ExportStubTable_sys_io>

		const char* lib_name_ptr =  (const char*)*(uint32_t*)((char*)export_stru_ptr + 0x10);

		if(strncmp(vsh_module, lib_name_ptr, strlen(lib_name_ptr))==0)
		{
			// we got the proper export struct
			uint32_t lib_fnid_ptr = *(uint32_t*)((char*)export_stru_ptr + 0x14);
			uint32_t lib_func_ptr = *(uint32_t*)((char*)export_stru_ptr + 0x18);
			uint16_t count = *(uint16_t*)((char*)export_stru_ptr + 6); // number of exports
			for(int i=0;i<count;i++)
			{
				if(fnid == *(uint32_t*)((char*)lib_fnid_ptr + i*4))
				{
					// take adress from OPD
					return (void**)*((uint32_t*)(lib_func_ptr) + i);
				}
			}
		}
		table=table+4;
	}
	return 0;
}


void show_msg(char* msg)
{
	if(!vshtask_notify)
		vshtask_notify = (void*)((int)getNIDfunc("vshtask", 0xA02D46E7));

	if(vshtask_notify)
		vshtask_notify(0, msg);
}

#ifndef COBRA_ONLY
void string_to_lv2(char* path, u64 addr);
void add_to_map(char *path1, char *path2);
#endif

uint64_t peekq(uint64_t addr);
void pokeq( uint64_t addr, uint64_t val);
void poke_lv1( uint64_t addr, uint64_t val);

u32 lv2peek32(u64 addr);
void lv2poke32(u64 addr, u32 value);

u32 lv2peek32(u64 addr)
{
    u32 ret = (u32) (peekq(addr) >> 32ULL);
    return ret;
}

void lv2poke32(u64 addr, u32 value)
{
    pokeq(addr, (((u64) value) <<32) | (peekq(addr) & 0xffffffffULL));
}

#define PS3 (1<<0)
#define PS2 (1<<1)
#define PS1 (1<<2)
#define PSP (1<<3)
#define BLU (1<<4)
#define DVD (1<<5)

#define RED		0
#define GREEN	1
#define YELLOW	2

#define OFF		0
#define ON		1
#define BLINK	2

static inline void led(u64 color, u64 mode)
{
	system_call_2(386, (u64)color, (u64)mode);
}

static inline void get_temperature(u32 _dev, u32 *_temp)
{
	system_call_2(383, (u64)_dev, (u64)_temp);
}

static int sys_sm_set_fan_policy(u8 arg0, u8 arg1, u8 arg2)
{
    system_call_3(389, (u64) arg0, (u64) arg1, (u64) arg2);
    return_to_user_prog(int);
}

static int sys_sm_get_fan_policy(u8 id, u8 *st, u8 *mode, u8 *speed, u8 *unknown)
{
    system_call_5(409, (u64) id, (u64) st, (u64) mode, (u64) speed, (u64) unknown);
    return_to_user_prog(int);
}

static inline void _sys_ppu_thread_exit(uint64_t val)
{
	system_call_1(41, val);
}

static inline sys_prx_id_t prx_get_module_id_by_address(void *addr)
{
	system_call_1(461, (uint64_t)(uint32_t)addr);
	return (int)p1;
}

/*
static u32 in_cobra(u32 *mode)
{
	system_call_2(8, (uint32_t) 0x7000, (uint32_t)mode);
	return_to_user_prog(uint32_t);
}
*/

int my_atoi(const char *c)
{
    int previous_result=0, result=0;
    int multiplier=1;

    if (c && *c == '-')
    {
        multiplier = -1;
        c++;
    }
    else
    {
        multiplier = 1;
    }

    while (*c)
    {
        if (*c < '0' || *c > '9')
        {
            return result * multiplier;
        }
        result *= 10;
        if (result < previous_result)
        {

            return(0);
        }
        else
        {
            previous_result *= 10;
        }

        result += *c - '0';

        if (result < previous_result)
        {

            return(0);
        }
        else
        {
            previous_result += *c - '0';
        }
        c++;
    }
    return(result * multiplier);
}

static int connect_to_server(char *server, uint16_t port)
{
	struct sockaddr_in sin;
	unsigned int temp;
	int s;

	if ((temp = inet_addr(server)) != (unsigned int)-1)
	{
		sin.sin_family = AF_INET;
		sin.sin_addr.s_addr = temp;
	}
	else {
		struct hostent *hp;

		if ((hp = gethostbyname(server)) == NULL)
		{
			return -1;
		}

		sin.sin_family = hp->h_addrtype;
		memcpy(&sin.sin_addr, hp->h_addr, hp->h_length);
	}

	sin.sin_port = htons(port);
	s = socket(AF_INET, SOCK_STREAM, 0);
	if (s < 0)
	{
		return -1;
	}

	struct timeval tv;
	tv.tv_usec = 0;

	tv.tv_sec = 3;
	//setsockopt(s, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv));
	setsockopt(s, SOL_SOCKET, SO_SNDTIMEO, &tv, sizeof(tv));

	if (connect(s, (struct sockaddr *)&sin, sizeof(sin)) < 0)
	{
		return -1;
	}

	tv.tv_sec = 60;
	//setsockopt(s, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv));
	setsockopt(s, SOL_SOCKET, SO_SNDTIMEO, &tv, sizeof(tv));

	return s;
}


#ifdef COBRA_ONLY
static int remote_stat(int s, char *path, int *is_directory, int64_t *file_size, uint64_t *mtime, uint64_t *ctime, uint64_t *atime, int *abort_connection)
{
	netiso_stat_cmd cmd;
	netiso_stat_result res;
	int len;

	*abort_connection = 0;

	len = strlen(path);
	memset(&cmd, 0, sizeof(cmd));
	cmd.opcode = (NETISO_CMD_STAT_FILE);
	cmd.fp_len = (len);

	if (send(s, &cmd, sizeof(cmd), 0) != sizeof(cmd))
	{
		//DPRINTF("send failed (remote_stat) (errno=%d)!\n", get_network_error());
		*abort_connection = 1;
		return -1;
	}

	if (send(s, path, len, 0) != len)
	{
		//DPRINTF("send failed (remote_stat) (errno=%d)!\n", get_network_error());
		*abort_connection = 1;
		return -1;
	}

	if (recv(s, &res, sizeof(res), MSG_WAITALL) != sizeof(res))
	{
		//DPRINTF("recv failed (remote_stat) (errno=%d)!\n", get_network_error());
		*abort_connection = 1;
		return -1;
	}

	*file_size = (res.file_size);
	if (*file_size == -1)
		return -1;

	*is_directory = res.is_directory;
	*mtime = (res.mtime);
	*ctime = (res.ctime);
	*atime = (res.atime);

	return 0;
}

static int read_remote_file(int s, void *buf, uint64_t offset, uint32_t size, int *abort_connection)
{
	netiso_read_file_cmd cmd;
	netiso_read_file_result res;

	*abort_connection = 0;

	memset(&cmd, 0, sizeof(cmd));
	cmd.opcode = (NETISO_CMD_READ_FILE);
	cmd.offset = (offset);
	cmd.num_bytes = (size);

	if (send(s, &cmd, sizeof(cmd), 0) != sizeof(cmd))
	{
		//DPRINTF("send failed (read_remote_file) (errno=%d)!\n", get_network_error());
		*abort_connection = 1;
		return -1;
	}

	if (recv(s, &res, sizeof(res), MSG_WAITALL) != sizeof(res))
	{
		//DPRINTF("recv failed (read_remote_file) (errno=%d)!\n", get_network_error());
		*abort_connection = 1;
		return -1;
	}

	int bytes_read = (res.bytes_read);
	if (bytes_read <= 0)
		return bytes_read;

	if (recv(s, buf, bytes_read, MSG_WAITALL) != bytes_read)
	{
		//DPRINTF("recv failed (read_remote_file) (errno=%d)!\n", get_network_error());
		*abort_connection = 1;
		return -1;
	}

	return bytes_read;
}

static int64_t open_remote_file_2(int s, char *path, int *abort_connection)
{
	netiso_open_cmd cmd;
	netiso_open_result res;
	int len;

	*abort_connection = 0;

	len = strlen(path);
	memset(&cmd, 0, sizeof(cmd));
	cmd.opcode = BE16(NETISO_CMD_OPEN_FILE);
	cmd.fp_len = BE16(len);

	if (send(s, &cmd, sizeof(cmd), 0) != sizeof(cmd))
	{
		//DPRINTF("send failed (open_remote_file) (errno=%d)!\n", get_network_error());
		*abort_connection = 1;
		return -1;
	}

	if (send(s, path, len, 0) != len)
	{
		//DPRINTF("send failed (open_remote_file) (errno=%d)!\n", get_network_error());
		*abort_connection = 1;
		return -1;
	}

	if (recv(s, &res, sizeof(res), MSG_WAITALL) != sizeof(res))
	{
		//DPRINTF("recv failed (open_remote_file) (errno=%d)!\n", get_network_error());
		*abort_connection = 1;
		return -1;
	}

	return (res.file_size);
}

static int64_t open_remote_file(char *path)
{
	netiso_open_cmd cmd;
	netiso_open_result res;
	int len;

	len = strlen(path);

	memset(&cmd, 0, sizeof(cmd));
	cmd.opcode = NETISO_CMD_OPEN_FILE;
	cmd.fp_len = len;

	if (send(g_socket, &cmd, sizeof(cmd), 0) != sizeof(cmd))
	{
		DPRINTF("send failed (open_file) (errno=%d)!\n", sys_net_errno);
		return -1;
	}

	if (send(g_socket, path, len, 0) != len)
	{
		DPRINTF("send failed (open_file) (errno=%d)!\n", sys_net_errno);
		return -1;
	}

	if (recv(g_socket, &res, sizeof(res), MSG_WAITALL) != sizeof(res))
	{
		DPRINTF("recv failed (open_file) (errno=%d)!\n", sys_net_errno);
		return -1;
	}

	if (res.file_size == -1)
	{
		DPRINTF("Remote file %s doesn't exist!\n", path);
		return -1;
	}

	DPRINTF("Remote file %s opened. Size = %x%08lx bytes\n", path, (res.file_size>>32), res.file_size&0xFFFFFFFF);
	return res.file_size;
}


static int read_remote_file_critical(uint64_t offset, void *buf, uint32_t size)
{
	netiso_read_file_critical_cmd cmd;

	memset(&cmd, 0, sizeof(cmd));
	cmd.opcode = NETISO_CMD_READ_FILE_CRITICAL;
	cmd.num_bytes = size;
	cmd.offset = offset;

	if (send(g_socket, &cmd, sizeof(cmd), 0) != sizeof(cmd))
	{
		DPRINTF("send failed (read file) (errno=%d)!\n", sys_net_errno);
		return -1;
	}

	if (recv(g_socket, buf, size, MSG_WAITALL) != (int)size)
	{
		DPRINTF("recv failed (recv file)  (errno=%d)!\n", sys_net_errno);
		return -1;
	}

	return 0;
}

static int process_read_iso_cmd(uint8_t *buf, uint64_t offset, uint32_t size)
{
	uint64_t read_end;

	//DPRINTF("read iso: %p %lx %x\n", buf, offset, size);
	read_end = offset + size;

	if (read_end >= discsize)
	{
		DPRINTF("Read beyond limits: %llx %x (discsize=%llx)!\n", offset, size, discsize);

		if (offset >= discsize)
		{
			memset(buf, 0, size);
			return 0;
		}

		memset(buf+(discsize-offset), 0, read_end-discsize);
		size = discsize-offset;
	}

	return read_remote_file_critical(offset, buf, size);
}

static int process_read_cd_2048_cmd(uint8_t *buf, uint32_t start_sector, uint32_t sector_count)
{
	netiso_read_cd_2048_critical_cmd cmd;

	memset(&cmd, 0, sizeof(cmd));
	cmd.opcode = NETISO_CMD_READ_CD_2048_CRITICAL;
	cmd.start_sector = start_sector;
	cmd.sector_count = sector_count;

	if (send(g_socket, &cmd, sizeof(cmd), 0) != sizeof(cmd))
	{
		DPRINTF("send failed (read 2048) (errno=%d)!\n", sys_net_errno);
		return -1;
	}

	if (recv(g_socket, buf, sector_count*2048, MSG_WAITALL) != (int)(sector_count*2048))
	{
		DPRINTF("recv failed (read 2048)  (errno=%d)!\n", sys_net_errno);
		return -1;
	}

	return 0;
}

static int process_read_cd_2352_cmd(uint8_t *buf, uint32_t sector, uint32_t remaining)
{
	int cache = 0;

	if (remaining <= CD_CACHE_SIZE)
	{
		int dif = (int)cached_cd_sector-sector;

		if (ABS(dif) < CD_CACHE_SIZE)
		{
			uint8_t *copy_ptr = NULL;
			uint32_t copy_offset = 0;
			uint32_t copy_size = 0;

			if (dif > 0)
			{
				if (dif < (int)remaining)
				{
					copy_ptr = cd_cache;
					copy_offset = dif;
					copy_size = remaining-dif;
				}
			}
			else
			{

				copy_ptr = cd_cache+((-dif)*2352);
				copy_size = MIN((int)remaining, CD_CACHE_SIZE+dif);
			}

			if (copy_ptr)
			{
				memcpy(buf+(copy_offset*2352), copy_ptr, copy_size*2352);

				if (remaining == copy_size)
				{
					return 0;
				}

				remaining -= copy_size;

				if (dif <= 0)
				{
					uint32_t newsector = cached_cd_sector + CD_CACHE_SIZE;
					buf += ((newsector-sector)*2352);
					sector = newsector;
				}
			}
		}

		cache = 1;
	}

	if (!cache)
	{
		return process_read_iso_cmd(buf, sector*2352, remaining*2352);
	}

	if (!cd_cache)
	{
		sys_addr_t addr;

		int ret = sys_memory_allocate(192*1024, SYS_MEMORY_PAGE_SIZE_64K, &addr);
		if (ret != 0)
		{
			DPRINTF("sys_memory_allocate failed: %x\n", ret);
			return ret;
		}

		cd_cache = (uint8_t *)addr;
	}

	if (process_read_iso_cmd(cd_cache, sector*2352, CD_CACHE_SIZE*2352) != 0)
		return -1;

	memcpy(buf, cd_cache, remaining*2352);
	cached_cd_sector = sector;
	return 0;
}


static int sys_storage_ext_mount_discfile_proxy(sys_event_port_t result_port, sys_event_queue_t command_queue, int emu_type, uint64_t disc_size_bytes, uint32_t read_size, unsigned int trackscount, ScsiTrackDescriptor *tracks)
{
	system_call_8(8, SYSCALL8_OPCODE_MOUNT_DISCFILE_PROXY, result_port, command_queue, emu_type, disc_size_bytes, read_size, trackscount, (uint64_t)(uint32_t)tracks);
	return (int)p1;
}

static int fake_insert_event(uint64_t disctype)
{
	uint64_t param = (uint64_t)(disctype) << 32ULL;
	sys_storage_ext_fake_storage_event(7, 0, BDVD_DRIVE);
	return sys_storage_ext_fake_storage_event(3, param, BDVD_DRIVE);
}

static int fake_eject_event(void)
{
	sys_storage_ext_fake_storage_event(4, 0, BDVD_DRIVE);
	return sys_storage_ext_fake_storage_event(8, 0, BDVD_DRIVE);
}



static void netiso_thread(uint64_t arg)
{
	netiso_args *args;
	sys_event_port_t result_port;
	sys_event_queue_attribute_t queue_attr;
	unsigned int real_disctype;
	int64_t ret64;
	int ret;
	ScsiTrackDescriptor *tracks;
	int emu_mode, numtracks;

	args = (netiso_args *)(uint32_t)arg;

	//DPRINTF("Hello VSH\n");

	g_socket = connect_to_server(args->server, args->port);
	if (g_socket < 0)
	{
		sys_memory_free((sys_addr_t)args);
		sys_ppu_thread_exit(0);
	}

	ret64 = open_remote_file(args->path);
	if (ret64 < 0)
	{
		sys_memory_free((sys_addr_t)args);
		sys_ppu_thread_exit(0);
	}

	discsize = (uint64_t)ret64;

	ret = sys_event_port_create(&result_port, 1, SYS_EVENT_PORT_NO_NAME);
	if (ret != 0)
	{
		//DPRINTF("sys_event_port_create failed: %x\n", ret);
		sys_memory_free((sys_addr_t)args);
		sys_ppu_thread_exit(ret);
	}

	sys_event_queue_attribute_initialize(queue_attr);
	ret = sys_event_queue_create(&command_queue, &queue_attr, 0, 1);
	if (ret != 0)
	{
		//DPRINTF("sys_event_queue_create failed: %x\n", ret);
		sys_memory_free((sys_addr_t)args);
		sys_ppu_thread_exit(ret);
	}


	sys_storage_ext_get_disc_type(&real_disctype, NULL, NULL);

	if (real_disctype != 0)
	{
		fake_eject_event();
	}

	emu_mode = args->emu_mode;
	if (emu_mode == EMU_PSX)
	{
		numtracks = args->numtracks;
		tracks = &args->tracks[0];
		is_cd2352 = 1;
	}
	else
	{
		numtracks = 0;
		tracks = NULL;
	}

	ret = sys_storage_ext_mount_discfile_proxy(result_port, command_queue, emu_mode, discsize, 256*1024, numtracks, tracks);
	//DPRINTF("mount = %x\n", ret);

	sys_memory_free((sys_addr_t)args);
	fake_insert_event(real_disctype);

	if (ret != 0)
	{
		sys_event_port_destroy(result_port);
		sys_ppu_thread_exit(0);
	}

	netiso_loaded=1;
	while (netiso_loaded)
	{
		sys_event_t event;

		ret = sys_event_queue_receive(command_queue, &event, 0);
		if (ret != 0)
		{
			//DPRINTF("sys_event_queue_receive failed: %x\n", ret);
			break;
		}

		void *buf = (void *)(uint32_t)(event.data3>>32ULL);
		uint64_t offset = event.data2;
		uint32_t size = event.data3&0xFFFFFFFF;

		switch (event.data1)
		{
			case CMD_READ_ISO:
			{
				if (is_cd2352)
				{
					ret = process_read_cd_2048_cmd(buf, offset/2048, size/2048);
				}
				else
				{
					ret = process_read_iso_cmd(buf, offset, size);
				}
			}
			break;

			case CMD_READ_CD_ISO_2352:
			{
				ret = process_read_cd_2352_cmd(buf, offset/2352, size/2352);
			}
			break;
		}

		ret = sys_event_port_send(result_port, ret, 0, 0);
		if (ret != 0)
		{
			//DPRINTF("sys_event_port_send failed: %x\n", ret);
			break;
		}
	}

	sys_storage_ext_get_disc_type(&real_disctype, NULL, NULL);
	fake_eject_event();
	sys_storage_ext_umount_discfile();

	if (real_disctype != 0)
	{
		fake_insert_event(real_disctype);
	}

	if (cd_cache)
	{
		sys_memory_free((sys_addr_t)cd_cache);
	}

	if (g_socket >= 0)
	{
		shutdown(g_socket, SHUT_RDWR);
		socketclose(g_socket);
		g_socket = -1;
	}

	sys_event_port_disconnect(result_port);
	if (sys_event_port_destroy(result_port) != 0)
	{
		DPRINTF("Error destroyng result_port\n");
	}

	//DPRINTF("Exiting main thread!\n");
	netiso_loaded=0;
	sys_ppu_thread_exit(0);
}

static void netiso_stop_thread(uint64_t arg)
{
	uint64_t exit_code;

	if (g_socket >= 0)
	{
		shutdown(g_socket, SHUT_RDWR);
		socketclose(g_socket);
		g_socket = -1;
	}

	if (command_queue != (sys_event_queue_t)-1)
	{
		if (sys_event_queue_destroy(command_queue, SYS_EVENT_QUEUE_DESTROY_FORCE) != 0)
		{
			DPRINTF("Failed in destroying command_queue\n");
		}
	}

	if (thread_id_net != (sys_ppu_thread_t)-1)
	{
		sys_ppu_thread_join(thread_id_net, &exit_code);
	}

	netiso_loaded=0;

	sys_ppu_thread_exit(0);
}

static inline void get_next_read(uint64_t discoffset, uint64_t bufsize, uint64_t *offset, uint64_t *readsize, int *idx)
{
	uint64_t base = 0;
	*idx = -1;
	*readsize = bufsize;
	*offset = 0;

	for (uint32_t i = 0; i < num_sections; i++)
	{
		uint64_t last = base + ((uint64_t)sections_size[i] * 512);

		if (discoffset >= base && discoffset < last)
		{
			uint64_t maxfileread = last-discoffset;

			if (bufsize > maxfileread)
				*readsize = maxfileread;
			else
				*readsize = bufsize;

			*idx = i;
			*offset = discoffset-base;
			return;
		}

		base += ((uint64_t)sections_size[i] * 512);
	}

	// We can be here on video blu-ray
	DPRINTF("Offset or size out of range  %lx%08lx   %lx!!!!!!!!\n", discoffset>>32, discoffset, bufsize);
}

static int process_read_iso_cmd_iso(uint8_t *buf, uint64_t offset, uint64_t size)
{
	uint64_t remaining;

	//DPRINTF("read iso: %p %lx %lx\n", buf, offset, size);
	remaining = size;

	while (remaining > 0)
	{
		uint64_t pos, readsize;
		int idx;
		int ret;
		uint8_t tmp[512];
		uint32_t sector;
		uint32_t r;

		get_next_read(offset, remaining, &pos, &readsize, &idx);

		if (idx == -1 || sections[idx] == 0xFFFFFFFF)
		{
			memset(buf, 0, readsize);
			buf += readsize;
			offset += readsize;
			remaining -= readsize;
			continue;
		}

		if (pos & 511)
		{
			uint64_t csize;

			sector = sections[idx] + pos/512;
			ret = sys_storage_read(handle, 0, sector, 1, tmp, &r, 0);
			if (ret != 0 || r != 1)
			{
				DPRINTF("sys_storage_read failed: %x 1 -> %x\n", sector, ret);
				return -1;
			}

			csize = 512-(pos&511);

			if (csize > readsize)
				csize = readsize;

			memcpy(buf, tmp+(pos&511), csize);
			buf += csize;
			offset += csize;
			pos += csize;
			remaining -= csize;
			readsize -= csize;
		}

		if (readsize > 0)
		{
			uint32_t n = readsize / 512;

			if (n > 0)
			{
				uint64_t s;

				sector = sections[idx] + pos/512;
				ret = sys_storage_read(handle, 0, sector, n, buf, &r, 0);
				if (ret != 0 || r != n)
				{
					DPRINTF("sys_storage_read failed: %x %x -> %x\n", sector, n, ret);
					return -1;
				}

				s = n * 512;
				buf += s;
				offset += s;
				pos += s;
				remaining -= s;
				readsize -= s;
			}

			if (readsize > 0)
			{
				sector = sections[idx] + pos/512;
				ret = sys_storage_read(handle, 0, sector, 1, tmp, &r, 0);
				if (ret != 0 || r != 1)
				{
					DPRINTF("sys_storage_read failed: %x 1 -> %x\n", sector, ret);
					return -1;
				}

				memcpy(buf, tmp, readsize);
				buf += readsize;
				offset += readsize;
				remaining -= readsize;
			}
		}
	}

	return 0;
}

static inline void my_memcpy(uint8_t *dst, uint8_t *src, int size)
{
	for (int i = 0; i < size; i++)
		dst[i] = src[i];
}

static int process_read_cd_2048_cmd_iso(uint8_t *buf, uint32_t start_sector, uint32_t sector_count)
{
	uint32_t capacity = sector_count*2048;
	uint32_t fit = capacity/2352;
	uint32_t rem = (sector_count-fit);
	uint32_t i;
	uint8_t *in = buf;
	uint8_t *out = buf;

	if (fit > 0)
	{
		process_read_iso_cmd_iso(buf, start_sector*2352, fit*2352);

		for (i = 0; i < fit; i++)
		{
			my_memcpy(out, in+24, 2048);
			in += 2352;
			out += 2048;
			start_sector++;
		}
	}

	for (i = 0; i < rem; i++)
	{
		process_read_iso_cmd_iso(out, (start_sector*2352)+24, 2048);
		out += 2048;
		start_sector++;
	}

	return 0;
}

static int process_read_cd_2352_cmd_iso(uint8_t *buf, uint32_t sector, uint32_t remaining)
{
	int cache = 0;

	if (remaining <= CD_CACHE_SIZE)
	{
		int dif = (int)cached_cd_sector-sector;

		if (ABS(dif) < CD_CACHE_SIZE)
		{
			uint8_t *copy_ptr = NULL;
			uint32_t copy_offset = 0;
			uint32_t copy_size = 0;

			if (dif > 0)
			{
				if (dif < (int)remaining)
				{
					copy_ptr = cd_cache;
					copy_offset = dif;
					copy_size = remaining-dif;
				}
			}
			else
			{

				copy_ptr = cd_cache+((-dif)*2352);
				copy_size = MIN((int)remaining, CD_CACHE_SIZE+dif);
			}

			if (copy_ptr)
			{
				memcpy(buf+(copy_offset*2352), copy_ptr, copy_size*2352);

				if (remaining == copy_size)
				{
					return 0;
				}

				remaining -= copy_size;

				if (dif <= 0)
				{
					uint32_t newsector = cached_cd_sector + CD_CACHE_SIZE;
					buf += ((newsector-sector)*2352);
					sector = newsector;
				}
			}
		}

		cache = 1;
	}

	if (!cache)
	{
		return process_read_iso_cmd_iso(buf, sector*2352, remaining*2352);
	}

	if (!cd_cache)
	{
		sys_addr_t addr;

		int ret = sys_memory_allocate(192*1024, SYS_MEMORY_PAGE_SIZE_64K, &addr);
		if (ret != 0)
		{
			DPRINTF("sys_memory_allocate failed: %x\n", ret);
			return ret;
		}

		cd_cache = (uint8_t *)addr;
	}

	if (process_read_iso_cmd_iso(cd_cache, sector*2352, CD_CACHE_SIZE*2352) != 0)
		return -1;

	memcpy(buf, cd_cache, remaining*2352);
	cached_cd_sector = sector;
	return 0;
}

static void rawseciso_stop_thread(uint64_t arg)
{
	uint64_t exit_code;
	rawseciso_loaded = 1;

	if (command_queue_ntfs != (sys_event_queue_t)-1)
	{
		if (sys_event_queue_destroy(command_queue_ntfs, SYS_EVENT_QUEUE_DESTROY_FORCE) != 0)
		{
			DPRINTF("Failed in destroying command_queue\n");
		}
	}

	if (thread_id_ntfs != (sys_ppu_thread_t)-1)
	{
		sys_ppu_thread_join(thread_id_ntfs, &exit_code);
	}

	rawseciso_loaded=0;
	sys_ppu_thread_exit(0);
}

static void rawseciso_thread(uint64_t arg)
{
	rawseciso_args *args;
	sys_event_port_t result_port;
	sys_event_queue_attribute_t queue_attr;
	unsigned int real_disctype;
	int ret;
	ScsiTrackDescriptor *tracks;
	int emu_mode, num_tracks;

	args = (rawseciso_args *)(uint32_t)arg;

	num_sections = args->num_sections;
	sections = (uint32_t *)(args+1);
	sections_size = sections + num_sections;

	discsize = 0;

	for (uint32_t i = 0; i < num_sections; i++)
	{
		discsize += sections_size[i];
	}

	discsize = discsize * 512;

	emu_mode = args->emu_mode;
	if (emu_mode == EMU_PSX)
        {
		num_tracks = args->num_tracks;
		tracks = (ScsiTrackDescriptor *)(sections_size + num_sections);
		is_cd2352 = 1;
	}
	else
        {
		num_tracks = 0;
		tracks = NULL;
	}

	if (is_cd2352)
	{
		if (discsize % 2352)
		{
			discsize = discsize - (discsize % 2352);
		}
	}

	DPRINTF("discsize = %lx%08lx\n", discsize>>32, discsize);

	ret = sys_storage_open(args->device, 0, &handle, 0);
	if (ret != 0)
	{
		DPRINTF("sys_storage_open failed: %x\n", ret);
		sys_memory_free((sys_addr_t)args);
		sys_ppu_thread_exit(ret);
	}

	ret = sys_event_port_create(&result_port, 1, SYS_EVENT_PORT_NO_NAME);
	if (ret != 0)
	{
		DPRINTF("sys_event_port_create failed: %x\n", ret);
                sys_storage_close(handle);
		sys_memory_free((sys_addr_t)args);
		sys_ppu_thread_exit(ret);
	}

	sys_event_queue_attribute_initialize(queue_attr);
	ret = sys_event_queue_create(&command_queue_ntfs, &queue_attr, 0, 1);
	if (ret != 0)
	{
		DPRINTF("sys_event_queue_create failed: %x\n", ret);
		sys_event_port_destroy(result_port);
		sys_storage_close(handle);
		sys_memory_free((sys_addr_t)args);
		sys_ppu_thread_exit(ret);
	}

	sys_storage_ext_get_disc_type(&real_disctype, NULL, NULL);

	if (real_disctype != 0)
	{
		fake_eject_event();
	}

	ret = sys_storage_ext_mount_discfile_proxy(result_port, command_queue_ntfs, emu_mode, discsize, 256*1024, num_tracks, tracks);
	DPRINTF("mount = %x\n", ret);

	fake_insert_event(real_disctype);

	if (ret != 0)
	{
		sys_event_port_disconnect(result_port);
		// Queue destroyed in stop thread sys_event_queue_destroy(command_queue_ntfs);
		sys_event_port_destroy(result_port);
		sys_storage_close(handle);
		sys_memory_free((sys_addr_t)args);
		sys_ppu_thread_exit(0);
	}

	rawseciso_loaded=1;
	while (rawseciso_loaded)
	{
		sys_event_t event;

		ret = sys_event_queue_receive(command_queue_ntfs, &event, 0);
		if (ret != 0)
		{
			//DPRINTF("sys_event_queue_receive failed: %x\n", ret);
			break;
		}

		void *buf = (void *)(uint32_t)(event.data3>>32ULL);
		uint64_t offset = event.data2;
		uint32_t size = event.data3&0xFFFFFFFF;

		switch (event.data1)
		{
			case CMD_READ_ISO:
			{
				if (is_cd2352)
				{
					ret = process_read_cd_2048_cmd_iso(buf, offset/2048, size/2048);
				}
				else
				{
					ret = process_read_iso_cmd_iso(buf, offset, size);
				}
			}
			break;

			case CMD_READ_CD_ISO_2352:
			{
				ret = process_read_cd_2352_cmd_iso(buf, offset/2352, size/2352);
			}
			break;
		}

		ret = sys_event_port_send(result_port, ret, 0, 0);
		if (ret != 0)
		{
			DPRINTF("sys_event_port_send failed: %x\n", ret);
			break;
		}
	}

	sys_memory_free((sys_addr_t)args);

	sys_storage_ext_get_disc_type(&real_disctype, NULL, NULL);
	fake_eject_event();
	sys_storage_ext_umount_discfile();

	if (real_disctype != 0)
	{
		fake_insert_event(real_disctype);
	}

	if (cd_cache)
	{
		sys_memory_free((sys_addr_t)cd_cache);
	}

	sys_storage_close(handle);

	sys_event_port_disconnect(result_port);
	if (sys_event_port_destroy(result_port) != 0)
	{
		DPRINTF("Error destroyng result_port\n");
	}

	// queue destroyed in stop thread

	DPRINTF("Exiting main thread!\n");
	rawseciso_loaded=0;
	sys_ppu_thread_exit(0);
}
#endif

void absPath(char* absPath_s, const char* path, const char* cwd)
{
	if(path[0] == '/')
	{
		strcpy(absPath_s, path);
	}
	else
	{
		strcpy(absPath_s, cwd);

		if(cwd[strlen(cwd) - 1] != '/')
		{
			strcat(absPath_s, "/");
		}

		strcat(absPath_s, path);
	}
}

int isDir(const char* path)
{
	struct CellFsStat s;
	if(cellFsStat(path, &s)==CELL_FS_SUCCEEDED)
		return ((s.st_mode & CELL_FS_S_IFDIR) != 0);
	else
		return 0;
}


void pokeq( uint64_t addr, uint64_t val)
{
	system_call_2(7, addr, val);
}

void poke_lv1( uint64_t addr, uint64_t val)
{
	system_call_2(9, addr, val);
}

uint64_t peekq(uint64_t addr)
{
	system_call_1(6, addr);
	return_to_user_prog(uint64_t);
}

bool language(const char *file_str, char *default_str)
{
	uint64_t siz = 0;
	uint8_t i;
	int f=0;
	char temp[1];
	const char *lang_path = 0;

    bool do_retry=true;

	if(webman_config->lang==0) {
		lang_path = "/dev_hdd0/tmp/LANG_EN.TXT";
	} else if(webman_config->lang==1) {
		lang_path = "/dev_hdd0/tmp/LANG_FR.TXT";
	} else if(webman_config->lang==2) {
		lang_path = "/dev_hdd0/tmp/LANG_IT.TXT";
	} else if(webman_config->lang==3) {
		lang_path = "/dev_hdd0/tmp/LANG_ES.TXT";
	} else if(webman_config->lang==4) {
		lang_path = "/dev_hdd0/tmp/LANG_DE.TXT";
	} else if(webman_config->lang==5) {
		lang_path = "/dev_hdd0/tmp/LANG_NL.TXT";
	} else if(webman_config->lang==6) {
		lang_path = "/dev_hdd0/tmp/LANG_PT.TXT";
	} else if(webman_config->lang==7) {
		lang_path = "/dev_hdd0/tmp/LANG_RU.TXT";
	} else if(webman_config->lang==8) {
		lang_path = "/dev_hdd0/tmp/LANG_HU.TXT";
	} else if(webman_config->lang==9) {
		lang_path = "/dev_hdd0/tmp/LANG_PL.TXT";
	} else if(webman_config->lang==10) {
		lang_path = "/dev_hdd0/tmp/LANG_GR.TXT";
	} else if(webman_config->lang==11) {
		lang_path = "/dev_hdd0/tmp/LANG_IN.TXT";
	} else if(webman_config->lang==12) {
		lang_path = "/dev_hdd0/tmp/LANG_TR.TXT";
	} else if(webman_config->lang==13) {
		lang_path = "/dev_hdd0/tmp/LANG_AR.TXT";
	} else if(webman_config->lang==14) {
		lang_path = "/dev_hdd0/tmp/LANG_CN.TXT";
	} else if(webman_config->lang==15) {
		lang_path = "/dev_hdd0/tmp/LANG_KR.TXT";
	} else if(webman_config->lang==16) {
		lang_path = "/dev_hdd0/tmp/LANG_JP.TXT";
	} else if(webman_config->lang==99) {
		lang_path = "/dev_hdd0/tmp/LANG_XX.TXT";
	} else return false;

	if(cellFsOpen(lang_path, CELL_FS_O_RDONLY, &f, 0,0) != CELL_FS_SUCCEEDED) return false;

retry:
	cellFsLseek(f, lang_pos, CELL_FS_SEEK_SET, &siz);

	do {
		cellFsRead(f, (void *)&temp, 0x01, &siz);
		lang_pos++;
		for(i=0; i < strlen(file_str) ; i++) {
			if(temp[0] != file_str[i]) {
				break;
			} else {
				if (i==strlen(file_str)-1 ) {
					while(temp[0] != '[') {
						cellFsRead(f, (void *)&temp, 0x01, &siz);
						lang_pos++;
					}
					int str_len = 0;
					while(1) {
						cellFsRead(f, (void *)&temp, 0x01, &siz);
						lang_pos++;
						if(temp[0] == ']') {
							default_str[str_len] = NULL;
							cellFsClose(f);
							return true;
						}
						default_str[str_len] = temp[0];
						str_len++;
					}
				} else {
					cellFsRead(f, (void *)&temp, 0x01, &siz);
					lang_pos++;
				}
			}
		}
	} while(siz != 0) ;

	if(do_retry) {do_retry=false; lang_pos=0; goto retry;}

	cellFsClose(f);

	return true;
}

void update_language()
{
	lang_pos=0;
	if(language("STR_TRADBY", STR_TRADBY))
	{
		language("STR_FILES", STR_FILES);
		language("STR_GAMES", STR_GAMES);
		language("STR_SETUP", STR_SETUP);
		language("STR_EJECT", STR_EJECT);
		language("STR_INSERT", STR_INSERT);
		language("STR_UNMOUNT", STR_UNMOUNT);
		language("STR_REFRESH", STR_REFRESH);
		language("STR_SHUTDOWN", STR_SHUTDOWN);
		language("STR_RESTART", STR_RESTART);

		language("STR_BYTE", STR_BYTE);
		language("STR_KILOBYTE", STR_KILOBYTE);
		language("STR_MEGABYTE", STR_MEGABYTE);
		language("STR_GIGABYTE", STR_GIGABYTE);

		language("STR_SCAN1", STR_SCAN1);
		language("STR_SCAN2", STR_SCAN2);
		language("STR_PSPL", STR_PSPL);
		language("STR_VIDLG", STR_VIDLG	);
		language("STR_LPG", STR_LPG);
		language("STR_AUTOB", STR_AUTOB);
		language("STR_DELAYAB", STR_DELAYAB);
		language("STR_DEVBL", STR_DEVBL);
		language("STR_CONTSCAN", STR_CONTSCAN);
		language("STR_USBPOLL", STR_USBPOLL);
		language("STR_FTPSVC", STR_FTPSVC);
		language("STR_COMBOS", STR_COMBOS);
		language("STR_MMCOVERS", STR_MMCOVERS);
		language("STR_ACCESS", STR_ACCESS);
		language("STR_NOSETUP", STR_NOSETUP);
		language("STR_NOSPOOF", STR_NOSPOOF);
		language("STR_NOGRP", STR_NOGRP);
		language("STR_TITLEID", STR_TITLEID);
		language("STR_FANCTRL", STR_FANCTRL);
		language("STR_NOWARN", STR_NOWARN);
		language("STR_AUTOAT", STR_AUTOAT);
		language("STR_LOWEST", STR_LOWEST);
		language("STR_FANSPEED", STR_FANSPEED);
		language("STR_MANUAL", STR_MANUAL);
		language("STR_PS2EMU", STR_PS2EMU);
		language("STR_LANGAMES", STR_LANGAMES);
		language("STR_ANYUSB", STR_ANYUSB);
		language("STR_ADDUSB", STR_ADDUSB);
		language("STR_SPOOFID", STR_SPOOFID);
		language("STR_DELCFWSYS", STR_DELCFWSYS);
		language("STR_MEMUSAGE", STR_MEMUSAGE);
		language("STR_PLANG", STR_PLANG);
		language("STR_COMBOS2", STR_COMBOS2);
		language("STR_FAILSAFE", STR_FAILSAFE);
		language("STR_SHOWTEMP", STR_SHOWTEMP);
		language("STR_SHOWIDPS", STR_SHOWIDPS);
		language("STR_PREVGAME", STR_PREVGAME);
		language("STR_NEXTGAME", STR_NEXTGAME);
		language("STR_SHUTDOWN2", STR_SHUTDOWN2);
		language("STR_RESTART2", STR_RESTART2);
		language("STR_DELCFWSYS2", STR_DELCFWSYS2);
		language("STR_UNLOADWM", STR_UNLOADWM);
		language("STR_FANCTRL2", STR_FANCTRL2);
		language("STR_FANCTRL4", STR_FANCTRL4);
		language("STR_FANCTRL5", STR_FANCTRL5);
		language("STR_UPDN", STR_UPDN);
		language("STR_LFRG", STR_LFRG);
		language("STR_SAVE", STR_SAVE);
		language("STR_SETTINGSUPD", STR_SETTINGSUPD);
		language("STR_ERROR", STR_ERROR);

		language("STR_MYGAMES", STR_MYGAMES);
		language("STR_LOADGAMES", STR_LOADGAMES);

		language("STR_WMSETUP", STR_WMSETUP);
		language("STR_WMSETUP2", STR_WMSETUP2);

		language("STR_EJECTDISC", STR_EJECTDISC);
		language("STR_UNMOUNTGAME", STR_UNMOUNTGAME);

        language("STR_WMSTART", STR_WMSTART);
		language("STR_WMUNL", STR_WMUNL);
		language("STR_CFWSYSALRD", STR_CFWSYSALRD);
		language("STR_CFWSYSRIP", STR_CFWSYSRIP);
		language("STR_RMVCFWSYS", STR_RMVCFWSYS);
		language("STR_RMVCFWSYSF", STR_RMVCFWSYSF);

		language("STR_RMVWMCFG", STR_RMVWMCFG);
		language("STR_RMVWMCFGOK", STR_RMVWMCFGOK);

		language("STR_PS3FORMAT", STR_PS3FORMAT);
		language("STR_PS2FORMAT", STR_PS2FORMAT);
		language("STR_PS1FORMAT", STR_PS1FORMAT);
		language("STR_PSPFORMAT", STR_PSPFORMAT);

		language("STR_VIDFORMAT", STR_VIDFORMAT);
		language("STR_VIDEO", STR_VIDEO);

		language("STR_LAUNCHPSP", STR_LAUNCHPSP);

		language("STR_GAMEUM", STR_GAMEUM);

		language("STR_EJECTED", STR_EJECTED);
		language("STR_LOADED", STR_LOADED);

		language("STR_GAMETOM", STR_GAMETOM);
		language("STR_GAMELOADED", STR_GAMELOADED);
		language("STR_PSPLOADED", STR_PSPLOADED);
		language("STR_LOADED2", STR_LOADED2);

		language("STR_MOVIETOM", STR_MOVIETOM);
		language("STR_MOVIELOADED", STR_MOVIELOADED);

		language("STR_XMLRF", STR_XMLRF);

		language("STR_STORAGE", STR_STORAGE);
		language("STR_MEMORY", STR_MEMORY);
		language("STR_MBFREE", STR_MBFREE);
		language("STR_KBFREE", STR_KBFREE);

		language("STR_FANCTRL3", STR_FANCTRL3);
		language("STR_ENABLED", STR_ENABLED);
		language("STR_DISABLED", STR_DISABLED);

		language("STR_FANCH0", STR_FANCH0);
		language("STR_FANCH1", STR_FANCH1);
		language("STR_FANCH2", STR_FANCH2);
		language("STR_FANCH3", STR_FANCH3);

		language("STR_OVERHEAT", STR_OVERHEAT);
		language("STR_OVERHEAT2", STR_OVERHEAT2);
	}
}

uint64_t convertH(char *val)
{
    uint8_t buff;
    uint64_t ret=0;
    int i;

    for(i = 0; i < 16; i++) {
        if(val[i]=='0'               ) buff=0x00; else
        if(val[i]=='1'               ) buff=0x10; else
        if(val[i]=='2'               ) buff=0x20; else
        if(val[i]=='3'               ) buff=0x30; else
        if(val[i]=='4'               ) buff=0x40; else
        if(val[i]=='5'               ) buff=0x50; else
        if(val[i]=='6'               ) buff=0x60; else
        if(val[i]=='7'               ) buff=0x70; else
        if(val[i]=='8'               ) buff=0x80; else
        if(val[i]=='9'               ) buff=0x90; else
        if(val[i]=='A' || val[i]=='a') buff=0xA0; else
        if(val[i]=='B' || val[i]=='b') buff=0xB0; else
        if(val[i]=='C' || val[i]=='c') buff=0xC0; else
        if(val[i]=='D' || val[i]=='d') buff=0xD0; else
        if(val[i]=='E' || val[i]=='e') buff=0xE0; else
        if(val[i]=='F' || val[i]=='f') buff=0xF0; else
        return 0;

        i++;

        if(val[i]=='0'               ) buff+=0x00; else
        if(val[i]=='1'               ) buff+=0x01; else
        if(val[i]=='2'               ) buff+=0x02; else
        if(val[i]=='3'               ) buff+=0x03; else
        if(val[i]=='4'               ) buff+=0x04; else
        if(val[i]=='5'               ) buff+=0x05; else
        if(val[i]=='6'               ) buff+=0x06; else
        if(val[i]=='7'               ) buff+=0x07; else
        if(val[i]=='8'               ) buff+=0x08; else
        if(val[i]=='9'               ) buff+=0x09; else
        if(val[i]=='A' || val[i]=='a') buff+=0x0A; else
        if(val[i]=='B' || val[i]=='b') buff+=0x0B; else
        if(val[i]=='C' || val[i]=='c') buff+=0x0C; else
        if(val[i]=='D' || val[i]=='d') buff+=0x0D; else
        if(val[i]=='E' || val[i]=='e') buff+=0x0E; else
        if(val[i]=='F' || val[i]=='f') buff+=0x0F; else
        return 0;

        ret = (ret << 8) | buff;
    }

    return ret;
}

uint64_t find_syscall()
{
	uint64_t i = 0x8000000000600000ULL;
	while(i>0x8000000000000000ULL)	{
		if(peekq(i) == 0x3C60800160630003ULL)
			if(((peekq(i+8) >> 32) & 0xFFFFFFFF) == 0x4E800020)	return i;
		i-=4;
	}
	return 0;
}

uint64_t search64(uint64_t val)
{
	uint64_t i;
	for(i=0x8000000000080200ULL;0x8000000000600000ULL;i+=4)
	{
		if(peekq(i) == val)
			return i;
	}
	return 0;
}

uint64_t find_syscall_table()
{
	uint64_t sc, opd_sc;
	sc = find_syscall();
	opd_sc = search64(sc);
	return search64(opd_sc);
}

void remove_cfw_syscalls()
{
    u64 syscall_table, syscall_not_impl;

    u64 CEX=0x4345580000000000ULL;
    u64 DEX=0x4445580000000000ULL;

    if(peekq(0x80000000002E79C8ULL)==DEX) {dex_mode=2; c_firmware=3.41f;}	else
    if(peekq(0x80000000002CFF98ULL)==CEX) {dex_mode=0; c_firmware=3.41f;}	else
    if(peekq(0x80000000002EFE20ULL)==DEX) {dex_mode=2; c_firmware=3.55f;}	else
    if(peekq(0x80000000002D83D0ULL)==CEX) {dex_mode=0; c_firmware=3.55f;}	else
    if(peekq(0x8000000000302D88ULL)==DEX) {dex_mode=2; c_firmware=4.21f;}	else
    if(peekq(0x80000000002E8610ULL)==CEX) {dex_mode=0; c_firmware=4.21f;}	else
    if(peekq(0x80000000002E9F08ULL)==CEX) {dex_mode=0; c_firmware=4.30f;}	else
    if(peekq(0x8000000000304630ULL)==DEX) {dex_mode=2; c_firmware=4.30f;}	else
    if(peekq(0x80000000002E9F18ULL)==CEX) {dex_mode=0; c_firmware=4.31f;}	else
    if(peekq(0x80000000002EA488ULL)==CEX) {dex_mode=0; c_firmware=4.40f;}	else
    if(peekq(0x80000000002EA498ULL)==CEX) {dex_mode=0; c_firmware=4.41f;}	else
    if(peekq(0x8000000000304EF0ULL)==DEX) {dex_mode=2; c_firmware=4.41f;}	else
    if(peekq(0x80000000002EA9B8ULL)==CEX) {dex_mode=0; c_firmware=4.46f;}	else
    if(peekq(0x8000000000305410ULL)==DEX) {dex_mode=2; c_firmware=4.46f;}	else
    if(peekq(0x80000000002E9BE0ULL)==CEX) {dex_mode=0; c_firmware=4.50f;}	else
    if(peekq(0x8000000000309698ULL)==DEX) {dex_mode=2; c_firmware=4.50f;}	else
    if(peekq(0x80000000002E9D70ULL)==CEX) {dex_mode=0; c_firmware=4.53f;}	else
	if(peekq(0x80000000002EC5E0ULL)==CEX) {dex_mode=0; c_firmware=4.55f;}	else
    if(peekq(0x80000000002ED850ULL)==CEX) {dex_mode=0; c_firmware=4.60f;}	else
    c_firmware=0.00f;

    if(c_firmware==4.21f && !dex_mode) syscall_table = SYSCALL_TABLE_421;	else
	if(c_firmware==4.21f && dex_mode)  syscall_table = SYSCALL_TABLE_421D;	else
    if(c_firmware==4.30f && !dex_mode) syscall_table = SYSCALL_TABLE_430;	else
    if(c_firmware==4.30f && dex_mode)  syscall_table = SYSCALL_TABLE_430D;	else
    if(c_firmware==4.31f && !dex_mode) syscall_table = SYSCALL_TABLE_431;	else
    if(c_firmware==4.40f && !dex_mode) syscall_table = SYSCALL_TABLE_440;	else
    if(c_firmware==4.41f && !dex_mode) syscall_table = SYSCALL_TABLE_441;	else
    if(c_firmware==4.41f && dex_mode)  syscall_table = SYSCALL_TABLE_441D;	else
    if(c_firmware==4.46f && !dex_mode) syscall_table = SYSCALL_TABLE_446;	else
    if(c_firmware==4.50f && !dex_mode) syscall_table = SYSCALL_TABLE_450;	else
    if(c_firmware==4.46f && dex_mode)  syscall_table = SYSCALL_TABLE_446D;	else
	if(c_firmware==4.50f && dex_mode)  syscall_table = SYSCALL_TABLE_450D;	else
	if(c_firmware==4.53f && !dex_mode) syscall_table = SYSCALL_TABLE_453;	else
	if(c_firmware==4.55f && !dex_mode) syscall_table = SYSCALL_TABLE_455;	else
	if(c_firmware==4.60f && !dex_mode) syscall_table = SYSCALL_TABLE_460;	else

	syscall_table = find_syscall_table();
	syscall_not_impl = peekq(syscall_table);

	if(c_firmware==4.55f || c_firmware==4.60f) {
		IDPS[0] = peekq(0x8000000000474F1CULL  );
		IDPS[1] = peekq(0x8000000000474F1CULL+8);
		PSID[0] = peekq(0x8000000000474F34ULL  );
		PSID[1] = peekq(0x8000000000474F34ULL+8);
    }
    pokeq(syscall_table + (8*6), syscall_not_impl);
	pokeq(syscall_table + (8*7), syscall_not_impl);
    pokeq(syscall_table + (8*8), syscall_not_impl);
    pokeq(syscall_table + (8*9), syscall_not_impl);
    pokeq(syscall_table + (8*10), syscall_not_impl);
    pokeq(syscall_table + (8*35), syscall_not_impl);
    pokeq(syscall_table + (8*36), syscall_not_impl);

    u64 sc_not_impl_pt = peekq(syscall_not_impl);
    u64 sc6 = peekq(syscall_table + (8*6));
	u64 sc7 = peekq(syscall_table + (8*7));
    //u64 sc8 = peekq(syscall_table + (8*8));
    u64 sc9 = peekq(syscall_table + (8*9));
    u64 sc10 = peekq(syscall_table + (8*10));
    //u64 sc35 = peekq(syscall_table + (8*35));
    u64 sc36 = peekq(syscall_table + (8*36));

    if(sc6!=syscall_not_impl) pokeq(sc6, sc_not_impl_pt);
	if(sc7!=syscall_not_impl) pokeq(sc7, sc_not_impl_pt);
    //if(sc8!=syscall_not_impl) pokeq(sc8, sc_not_impl_pt);
    if(sc9!=syscall_not_impl) pokeq(sc9, sc_not_impl_pt);
    if(sc10!=syscall_not_impl) pokeq(sc10, sc_not_impl_pt);
    //if(sc35!=syscall_not_impl) pokeq(sc35, sc_not_impl_pt);
    if(sc36!=syscall_not_impl) pokeq(sc36, sc_not_impl_pt);
}

 void delete_history()
{
	char path[128];
	int fd;
	u64 read;
	CellFsDirent dir;

	if(cellFsOpendir("/dev_hdd0/home", &fd) == CELL_FS_SUCCEEDED) {
		read = sizeof(CellFsDirent);
		while(!cellFsReaddir(fd, &dir, &read)) {
			if(!read) break;
			sprintf(path, "/dev_hdd0/home/%s/etc/boot_history.dat", dir.d_name);
			cellFsUnlink(path);
			sprintf(path, "/dev_hdd0/home/%s/etc/community/CI.TMP", dir.d_name);
			cellFsUnlink(path);
			sprintf(path, "/dev_hdd0/home/%s/community/MI.TMP", dir.d_name);
			cellFsUnlink(path);
			sprintf(path, "/dev_hdd0/home/%s/community/PTL.TMP", dir.d_name);
			cellFsUnlink(path);
		}
		cellFsClosedir(fd);
	}

	cellFsUnlink("/dev_hdd0/vsh/pushlist/game.dat");
	cellFsUnlink("/dev_hdd0/vsh/pushlist/patch.dat");

	cellFsRmdir("/dev_hdd0/GAMES");
	cellFsRmdir("/dev_hdd0/GAMEZ");
	cellFsRmdir("/dev_hdd0/PS3ISO");
	cellFsRmdir("/dev_hdd0/PS2ISO");
	cellFsRmdir("/dev_hdd0/PSXISO");
	cellFsRmdir("/dev_hdd0/PSXGAMES");
	cellFsRmdir("/dev_hdd0/PSPISO");
	cellFsRmdir("/dev_hdd0/ISO");
	cellFsRmdir("/dev_hdd0/BDISO");
	cellFsRmdir("/dev_hdd0/DVDISO");
	cellFsRmdir("/dev_hdd0/PKG");
}

int ssplit(const char* str, char* left, int lmaxlen, char* right, int rmaxlen)
{
	int ios = strcspn(str, " ");
	int ret = (ios < (int)strlen(str) - 1);
	int lmax = (ios < lmaxlen) ? ios : lmaxlen;

	strncpy(left, str, lmax);
	left[lmax] = '\0';

	if(ret)
	{
		strncpy(right, str + ios + 1, rmaxlen);
		right[rmaxlen] = '\0';
	}
	else
	{
		right[0] = '\0';
	}

	return ret;
}

int slisten(int port, int backlog)
{
	int list_s = socket(AF_INET, SOCK_STREAM, 0);
	if(list_s<0) return list_s;

	//int optval = 1;
	//setsockopt(list_s, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(optval));

	struct sockaddr_in sa;
	socklen_t sin_len = sizeof(sa);
	memset(&sa, 0, sin_len);

	sa.sin_family = AF_INET;
	sa.sin_port = htons(port);
	sa.sin_addr.s_addr = htonl(INADDR_ANY);

	bind(list_s, (struct sockaddr *)&sa, sin_len);
	listen(list_s, backlog);

	return list_s;
}

void sclose(int *socket_e)
{
	if(*socket_e != -1)
	{
		shutdown(*socket_e, SHUT_RDWR);
		socketclose(*socket_e);
		*socket_e = -1;
	}
}




/*
u64 ssssocket(void)
{
	system_call_3(713, 2, 1, 0);			//socket
	return_to_user_prog(uint64_t);
}


static int connect_to_server2(char *server, uint16_t port)
{
	struct sockaddr_in sin;
	unsigned int temp;
	int s;

	if ((temp = inet_addr(server)) != (unsigned int)-1)
	{
		sin.sin_family = AF_INET;
		sin.sin_addr.s_addr = temp;
	}
	else {
		struct hostent *hp;

		if ((hp = gethostbyname(server)) == NULL)
		{
			return -1;
		}

		sin.sin_family = hp->h_addrtype;
		memcpy(&sin.sin_addr, hp->h_addr, hp->h_length);
	}

	sin.sin_port = htons(port);

	memcpy(&sin_global, &sin, 16);

	sprintf(netbuf, "Empty buffer...\r\n");

	pokeq(0x80000000007FD0A0ULL, (u64)&sin_global);
	pokeq(0x80000000007FD0A8ULL, (u64)&netbuf);
	pokeq(0x80000000007FD0B0ULL, (u64)&statbuf);
	pokeq(0x80000000007FD0B8ULL, (u64)&direntry);

	//peek(0x80000000007FD608): 0x0002947AC0A864D1 | ...z..d.

	s = ssssocket();
	{system_call_3(702, s, (u64)&sin, sizeof(sin));}		//connect
	{system_call_6(710, s, (u64)&sin, sizeof(sin), 0, (u64)&sin, sizeof(sin));}	//sendto
	//{system_call_6(707, s, (u64)&sin, sizeof(sin), 8, (u64)&sin, sizeof(sin));}	//recvfrom
	{system_call_1(714, s);}				//close
	{system_call_2(712, s, 2);}				//shutdown

	return 0;
}


static uint64_t syscall_837(const char *device, const char *format, const char *point, u32 a, u32 b, u32 c, void *buffer, u32 len)
{
	system_call_8(837, (u64)device, (u64)format, (u64)point, a, b, c, (u64)buffer, len);
	return_to_user_prog(uint64_t);
}
*/
#ifdef COBRA_ONLY
static int open_remote_dir(int s, char *path, int *abort_connection)
{
	netiso_open_dir_cmd cmd;
	netiso_open_dir_result res;
	int len;

	*abort_connection = 0;

	len = strlen(path);
	memset(&cmd, 0, sizeof(cmd));
	cmd.opcode = (NETISO_CMD_OPEN_DIR);
	cmd.dp_len = (len);

	if (send(s, &cmd, sizeof(cmd), 0) != sizeof(cmd))
	{
		//DPRINTF("send failed (open_remote_dir) (errno=%d)!\n", get_network_error());
		*abort_connection = 1;
		return -1;
	}

	if (send(s, path, len, 0) != len)
	{
		//DPRINTF("send failed (open_remote_dir) (errno=%d)!\n", get_network_error());
		*abort_connection = 1;
		return -1;
	}

	if (recv(s, &res, sizeof(res), MSG_WAITALL) != sizeof(res))
	{
		//DPRINTF("recv failed (open_remote_dir) (errno=%d)!\n", get_network_error());
		*abort_connection = 1;
		return -1;
	}

	return (res.open_result);
}

static int read_remote_dir(int s, sys_addr_t *data /*netiso_read_dir_result_data **data*/, int *abort_connection)
{
	netiso_read_dir_entry_cmd cmd;
	netiso_read_dir_result res;
	int len;

	*abort_connection = 0;

	memset(&cmd, 0, sizeof(cmd));
	cmd.opcode = (NETISO_CMD_READ_DIR);
//MM_LOG("Sending request...(%i) ", s);
	if (send(s, &cmd, sizeof(cmd), 0) != sizeof(cmd))
	{
//MM_LOG("FAILED!\n");
		*abort_connection = 1;
		return -1;
	}
//MM_LOG("Receiving response...");
	if (recv(s, &res, sizeof(res), MSG_WAITALL) != sizeof(res))
	{
//MM_LOG("FAILED!\n");
		*abort_connection = 1;
		return -1;
	}
//MM_LOG("OK (%i entries)\n", res.dir_size );
	if (res.dir_size > 0)
	{
		if(res.dir_size>866) res.dir_size=866;
		len = (sizeof(netiso_read_dir_result_data)*res.dir_size);
		sys_addr_t data1=0;
		int len2= ((len+65536)/65536)*65536;
		if(sys_memory_allocate(len2, SYS_MEMORY_PAGE_SIZE_64K, &data1)==0)
		{
			*data=data1;
			u8 *data2=(u8*)data1;
			//MM_LOG("dir_size=%i (%i bytes) (%i)\n", res.dir_size, len, s);

			if (recv(s, data2, len, MSG_WAITALL) != len)
			{
				*abort_connection = 1;
				sys_memory_free(data1);
				*data=NULL;
				return -1;
			}
		}
		else
			*data=NULL;

		//MM_LOG("Receive done.\n");
	}
	else
		*data=NULL;

	return (res.dir_size);
}
#endif

static void addradiobutton(const char *name, const char *value, const char *id, const char *label, const char *sufix, bool checked, char *buffer)
{
	char templn[256];
    sprintf(templn, "<label><input type=\"radio\" name=\"%s\" value=\"%s\" id=\"%s\" %s/> %s%s</label>", name, value, id, checked?(char*)"checked=\"checked\"" : (char*)"", label, (!sufix)?"<br>":sufix);
    strcat(buffer, templn);
}

static void addcheckbox(const char *name, const char *value, const char *label, const char *sufix, bool checked, char *buffer)
{
	char templn[256];
    sprintf(templn, "<label><input type=\"checkbox\" name=\"%s\" value=\"%s\" %s/> %s%s</label>", name, value, checked?(char*)"checked=\"checked\"" : (char*)"", label, (!sufix)?"<br>":sufix);
    strcat(buffer, templn);
}

static void addoptionitem(const char *value, const char *label, bool selected, char *buffer)
{
	char templn[256];
    sprintf(templn, "<option value=\"%s\" %s/>%s</option>", value, selected?(char*)"selected=\"selected\"" : (char*)"", label);
    strcat(buffer, templn);
}

static void parse_param_sfo(unsigned char *mem, char *titleID, char *title)
{
	int indx=0;
	int fcount=0;

	unsigned pos, str;

	str=(mem[0x8]+(mem[0x9]<<8));
	pos=(mem[0xc]+(mem[0xd]<<8));

	while(str<4090)
	{
		if(mem[str]==0) break;

		if(!strcmp((char *) &mem[str], "TITLE"))
		{
			memset(title, 0, 63);
			strncpy(title, (char *) &mem[pos], 63);
            fcount++;
		}
		if(!strcmp((char *) &mem[str], "TITLE_ID"))
		{
			memset(titleID, 0, 16);
			strncpy(titleID, (char *) &mem[pos], 9);
            fcount++;
		}
		if(fcount>=2) break;

		while(mem[str]) str++;str++;
		pos+=(mem[0x1c+indx]+(mem[0x1d+indx]<<8));
		indx+=16;
	}

	if(webman_config->tid)
	{
		strcat(title, " [");
		strcat(title, titleID);
		strcat(title, "]");
	}
}

static void get_icon(char *icon, char *titleid)
{
	struct CellFsStat s;

	if(covers_exist[0])
	{
        sprintf(icon, "%s/covers/%s.JPG", MM_ROOT_STD, titleid); if(cellFsStat(icon, &s)==CELL_FS_SUCCEEDED) return;
        sprintf(icon, "%s/covers/%s.PNG", MM_ROOT_STD, titleid); if(cellFsStat(icon, &s)==CELL_FS_SUCCEEDED) return;
	}

	if(covers_exist[1])
	{
		sprintf(icon, "%s/covers/%s.JPG", MM_ROOT_STL, titleid); if(cellFsStat(icon, &s)==CELL_FS_SUCCEEDED) return;
		sprintf(icon, "%s/covers/%s.PNG", MM_ROOT_STL, titleid); if(cellFsStat(icon, &s)==CELL_FS_SUCCEEDED) return;
	}

	if(covers_exist[2])
	{
		sprintf(icon, "%s/covers/%s.JPG", MM_ROOT_SSTL, titleid); if(cellFsStat(icon, &s)==CELL_FS_SUCCEEDED) return;
		sprintf(icon, "%s/covers/%s.PNG", MM_ROOT_SSTL, titleid); if(cellFsStat(icon, &s)==CELL_FS_SUCCEEDED) return;
	}

	if(covers_exist[3])
	{
		sprintf(icon, "/dev_hdd0/GAMES/covers/%s.JPG", titleid); if(cellFsStat(icon, &s)==CELL_FS_SUCCEEDED) return;
		sprintf(icon, "/dev_hdd0/GAMES/covers/%s.PNG", titleid); if(cellFsStat(icon, &s)==CELL_FS_SUCCEEDED) return;
	}

	if(covers_exist[4])
	{
		sprintf(icon, "/dev_hdd0/GAMEZ/covers/%s.JPG", titleid); if(cellFsStat(icon, &s)==CELL_FS_SUCCEEDED) return;
		sprintf(icon, "/dev_hdd0/GAMEZ/covers/%s.PNG", titleid); if(cellFsStat(icon, &s)==CELL_FS_SUCCEEDED) return;
	}

	sprintf(icon, "%s/%s.JPG", WMTMP, titleid); if(cellFsStat(icon, &s)==CELL_FS_SUCCEEDED) return;
	sprintf(icon, "%s/%s.PNG", WMTMP, titleid); if(cellFsStat(icon, &s)==CELL_FS_SUCCEEDED) return;

	icon[0]=0;
}

static void get_default_icon(char *icon, char *param, char *param2)
{
	struct CellFsStat s;

    if(icon[0]==0 || cellFsStat(icon, &s)!=CELL_FS_SUCCEEDED) {
		if(strstr(param, "/PS2ISO"))
			strcpy(icon, "/dev_flash/vsh/resource/explore/user/025.png");
		else if(strstr(param, "/PSXISO") || strstr(param, "/PSXGAMES") || strstr(param2, ".ntfs[PSXISO]") || strstr(param2, ".ntfs[PSXGAMES]"))
			strcpy(icon, "/dev_flash/vsh/resource/explore/user/026.png");
		else if(strstr(param, "/PSPISO") || strstr(param, "/ISO"))
			strcpy(icon, "/dev_flash/vsh/resource/explore/user/022.png");
		else if(strstr(param, "/BDISO") || strstr(param, "/DVDISO") || strstr(param2, ".ntfs[BDISO]") || strstr(param2, ".ntfs[DVDISO]"))
			strcpy(icon, "/dev_flash/vsh/resource/explore/user/023.png");
		else
			strcpy(icon, "/dev_flash/vsh/resource/explore/user/024.png");
	}
}

static void handleclient(u64 conn_s_p)
{
	int conn_s = (int)conn_s_p; // main communications socket

	u8 is_binary = 0;
	u64 c_len = 0;

	sys_addr_t sysmem=0;

	//char buffer[BUFFER_SIZE];
	//char myxml[BUFFER_SIZE];

	char templn[1024];
	char tempstr[4096];

	char cmd[16], param[512];
	char header[512];
	struct CellFsStat buf;
	int fd;
	u8 served=0;	// served http requests
	int fdxml=0;

	CellRtcDateTime rDate;
	CellRtcTick pTick;

	if(conn_s_p==0xC0FEBABE || conn_s_p==0xC0FEBAB0)
	{
		init_running=1;
		//reset_settings(webman_config);

#ifdef COBRA_ONLY
		//if(cobra_mode)
		{
			u8 cconfig[15];
			CobraConfig *cobra_config = (CobraConfig*) cconfig;
			memset(cobra_config, 0, 15);
			cobra_read_config(cobra_config);

			if(webman_config->nospoof)
			{
				cobra_config->spoof_version=0;
				cobra_config->spoof_revision=0;
			}
			else
			{
				if(!(c_firmware==4.53f || c_firmware==4.60f))
				{
					cobra_config->spoof_version=0x0460;
					cobra_config->spoof_revision=63910;
				}
			}


			if(cobra_config->ps2softemu==0 && cobra_get_ps2_emu_type()==PS2_EMU_SW)
				cobra_config->ps2softemu=1;

			cobra_write_config(cobra_config);

		}

#endif
		update_language();

		if(webman_config->spsid)
		{
			uint64_t j, newPSID[2] = {0,0};

			newPSID[0] = convertH(webman_config->vPSID1);
			newPSID[1] = convertH(webman_config->vPSID2);

			if(newPSID[0] != 0 && newPSID[1] != 0) {
				if(c_firmware==4.55f || c_firmware==4.60f) {
					pokeq(0x8000000000474F34ULL, newPSID[0]);
                    pokeq(0x8000000000474F34ULL+8, newPSID[1]);
				}
				else {
					system_call_1(872, (uint64_t) PSID);
					for(j = 0x8000000000000000ULL; j < 0x8000000000600000ULL; j++) {
						if((peekq(j) == PSID[0]) && (peekq(j+8) == PSID[1])) {
							pokeq(j, newPSID[0]);
							j+=8;
							pokeq(j, newPSID[1]);
						}	j+=8;
					}
				}
			}
		}

		if(webman_config->sidps) {
			uint64_t j, newIDPS[2] = {0,0};

			newIDPS[0] = convertH(webman_config->vIDPS1);
			newIDPS[1] = convertH(webman_config->vIDPS2);

			if(newIDPS[0] != 0 && newIDPS[1] != 0) {
				if(c_firmware==4.60f) {
					pokeq(0x80000000003E2BB0ULL, newIDPS[0]);
					pokeq(0x80000000003E2BB0ULL+8, newIDPS[1]);
					pokeq(0x8000000000474F1CULL, newIDPS[0]);
					pokeq(0x8000000000474F1CULL+8, newIDPS[1]);
				}
				else if(c_firmware==4.55f) {
					pokeq(0x80000000003E17B0ULL, newIDPS[0]);
					pokeq(0x80000000003E17B0ULL+8, newIDPS[1]);
					pokeq(0x8000000000474F1CULL, newIDPS[0]);
					pokeq(0x8000000000474F1CULL+8, newIDPS[1]);
				}
				else {
					system_call_1(870, (uint64_t) IDPS);
					for(j = 0x8000000000000000ULL; j < 0x8000000000600000ULL; j++) {
						if((peekq(j) == IDPS[0]) && (peekq(j+8) == IDPS[1])) {
							pokeq(j, newIDPS[0]);
							j+=8;
							pokeq(j, newIDPS[1]);
							j+=8;
						}
					}
				}
			}
		}

	if(c_firmware==4.55f || c_firmware==4.60f) {
		IDPS[0] = peekq(0x8000000000474F1CULL  );
		IDPS[1] = peekq(0x8000000000474F1CULL+8);
		PSID[0] = peekq(0x8000000000474F34ULL  );
		PSID[1] = peekq(0x8000000000474F34ULL+8);
    }

	if(webman_config->spp) {
		sys_timer_sleep(5);

		remove_cfw_syscalls();
		delete_history();
	}

	if(conn_s_p==0xC0FEBABE && webman_config->refr==1)
	{
		//show webMAN startup message
		if(do_startup_msg)
		{
			do_startup_msg = 0;
			if(strlen(STR_WMSTART)>0)
			{
				sys_timer_sleep(10);
				show_msg((char*)STR_WMSTART);
            }
		}

		//identify covers folders to be scanned
		sprintf(templn, "%s/covers", MM_ROOT_STD) ; covers_exist[0]=(cellFsStat(templn, &buf) ==CELL_FS_SUCCEEDED);
		sprintf(templn, "%s/covers", MM_ROOT_STL) ; covers_exist[1]=(cellFsStat(templn, &buf) ==CELL_FS_SUCCEEDED);
		sprintf(templn, "%s/covers", MM_ROOT_SSTL); covers_exist[2]=(cellFsStat(templn, &buf) ==CELL_FS_SUCCEEDED);
												    covers_exist[3]=(cellFsStat("/dev_hdd0/GAMES/covers", &buf)==CELL_FS_SUCCEEDED);
												    covers_exist[4]=(cellFsStat("/dev_hdd0/GAMEZ/covers", &buf)==CELL_FS_SUCCEEDED);

#ifdef COBRA_ONLY

		u8 do_delay=0;

		sprintf(tempstr, "/dev_hdd0/PS3ISO/AUTOBOOT.ISO");
		if(cobra_mode && webman_config->autob && cellFsStat(tempstr, &buf)==CELL_FS_SUCCEEDED)
			do_delay=1;

		else
		{
			sprintf(tempstr, WMTMP "/last_game.txt");
			if(cobra_mode && webman_config->lastp && cellFsStat(tempstr, &buf)==CELL_FS_SUCCEEDED)
			{
				int fd=0;
				if(cellFsOpen(tempstr, CELL_FS_O_RDONLY, &fd, NULL, 0) == CELL_FS_SUCCEEDED)
				{
					u64 read_e = 0;
					if(cellFsRead(fd, (void *)&tempstr, 512, &read_e)==CELL_FS_SUCCEEDED)
					{
						cellFsClose(fd);
						tempstr[read_e]=0;
						u8 retries1=0;
						if(strlen(tempstr)>10 || strstr(tempstr, "/net") || strstr(tempstr, ".ntfs["))
						{
again1:
							if(strstr(tempstr, "/dev_usb") && cellFsStat(tempstr, &buf)!=CELL_FS_SUCCEEDED)
							{
								sys_timer_sleep(1);
								retries1++;
								if(retries1<10) goto again1;
							}
							if(strstr(tempstr, "/net") || cellFsStat(tempstr, &buf)==CELL_FS_SUCCEEDED)
								do_delay=1;
							if(strstr(tempstr, ".ntfs[") && !webman_config->usb0 && !webman_config->usb1 && !webman_config->usb2 &&
								!webman_config->usb3 && !webman_config->usb6 && !webman_config->usb7) do_delay=0;
						}
					}
					else
						cellFsClose(fd);
				}
			}
		}

		if(do_delay)
		{
			if(webman_config->delay)
			{
				sys_timer_sleep(30);
				/*
				if(!webman_config->bootd || !webman_config->boots)
					sys_timer_sleep(20);
				else
					sys_timer_sleep(10);
					*/
			}
			else
				if(strstr(tempstr, "/net") || strstr(tempstr, ".ntfs[")) sys_timer_sleep(10);

			mount_with_mm(tempstr, 0);
		}
#endif
		init_running=0;
		sys_ppu_thread_exit(0);
	}

		_meminfo meminfo;
		{system_call_1(352, (uint64_t) &meminfo);}
		if((meminfo.avail)<( (BUFFER_SIZE_ALL) + 256*1024)) //leave if less than 1024+256KB memory
		{
			init_running=0;
			sys_ppu_thread_exit(0);
		}

#ifdef USE_VM
		if(sys_vm_memory_map(32*1024*1024, 1024*1024, SYS_MEMORY_CONTAINER_ID_INVALID, SYS_MEMORY_PAGE_SIZE_64K, SYS_VM_POLICY_AUTO_RECOMMENDED, &sysmem)!=CELL_OK)
		{
			init_running=0;
			sys_ppu_thread_exit(0);
		}
#else
		if(sys_memory_allocate((BUFFER_SIZE_ALL), SYS_MEMORY_PAGE_SIZE_64K, &sysmem)!=0)
		{
			init_running=0;
			sys_ppu_thread_exit(0);
		}
#endif

		sys_addr_t sysmem1=sysmem +(BUFFER_SIZE);
		sys_addr_t sysmem2=sysmem1+(BUFFER_SIZE_PSX)+(BUFFER_SIZE_PSP);
		sys_addr_t sysmem3=sysmem2+(BUFFER_SIZE_PS2);

		char *myxml_ps3	= (char*)sysmem;
		char *myxml_psx = NULL;
		char *myxml_psp = NULL;
		char *myxml_ps2 = NULL;
		char *myxml_dvd = NULL;
		char *myxml		= NULL;
		char *myxml_items=NULL;

		myxml_psx = (char*)sysmem1;
		myxml_psp = (char*)sysmem1+(BUFFER_SIZE_PSX);
		myxml_ps2 = (char*)sysmem2;

		myxml_dvd	= (char*)sysmem3;
		myxml		= (char*)sysmem+(BUFFER_SIZE)-4300;
		myxml_items = (char*)sysmem3;

		/*
		char ps3icon[128];
		char ps2icon[128];
		char psxicon[128];
		char dvdicon[128];

		strcpy(ps3icon, "/dev_flash/vsh/resource/explore/user/024.png");
		strcpy(ps2icon, "/dev_flash/vsh/resource/explore/user/025.png");
		strcpy(psxicon, "/dev_flash/vsh/resource/explore/user/026.png");
		strcpy(dvdicon, "/dev_flash/vsh/resource/explore/user/023.png");
		*/

		cellFsMkdir((char*)WMTMP, 0777);
		cellFsMkdir((char*)"/dev_hdd0/xmlhost", 0777);
		cellFsMkdir((char*)"/dev_hdd0/xmlhost/game_plugin", 0777);
		u32 key=0;

		char xml[128];
		char skey[1024][10];
		char swap[16];
		u8 is_net=0;

		sprintf(xml, "/dev_hdd0/xmlhost/game_plugin/fb.xml");
		cellFsUnlink(xml);
		cellFsOpen(xml, CELL_FS_O_CREAT | CELL_FS_O_TRUNC | CELL_FS_O_WRONLY, &fdxml, NULL, 0);
		sprintf(myxml, "<?xml version=\"1.0\" encoding=\"UTF-8\"?><XMBML version=\"1.0\"><View id=\"seg_fb\"><Attributes><Table key=\"mgames\"><Pair key=\"icon_rsc\"><String>item_tex_ps3util</String></Pair><Pair key=\"icon_notation\"><String>WNT_XmbItemSavePS3</String></Pair><Pair key=\"title\"><String>%s</String></Pair><Pair key=\"info\"><String>%s</String></Pair></Table></Attributes><Items><Query class=\"type:x-xmb/folder-pixmap\" key=\"mgames\" attr=\"mgames\" src=\"xmb://localhost/dev_hdd0/xmlhost/game_plugin/mygames.xml#seg_mygames\"/></Items></View></XMBML>", STR_MYGAMES, STR_LOADGAMES);
		cellFsWrite(fdxml, (char*)myxml, strlen(myxml), NULL);
		cellFsClose(fdxml);
		cellFsChmod(xml, 0666);
		myxml[0]=0;

		sprintf(xml, "/dev_hdd0/xmlhost/game_plugin/mygames.xml");
		cellFsUnlink(xml);

		key=0;

		if(!(webman_config->nogrp))
		{
			if(!(webman_config->cmask & PS3)) strcpy(myxml_ps3, "<View id=\"seg_mygames_ps3_items\"><Attributes>");
#ifdef COBRA_ONLY
			{
				if(!(webman_config->cmask & PS2)) strcpy(myxml_ps2, "<View id=\"seg_mygames_ps2_items\"><Attributes>");
				if(!(webman_config->cmask & PS1)) strcpy(myxml_psx, "<View id=\"seg_mygames_psx_items\"><Attributes>");
				if(!(webman_config->cmask & PSP)) {
					strcpy(myxml_psp, "<View id=\"seg_mygames_psp_items\"><Attributes>");
					if(webman_config->pspl && cobra_mode && cellFsStat((char*)"/dev_hdd0/game/PSPC66820", &buf)==CELL_FS_SUCCEEDED) {
						sprintf(templn, "<Table key=\"cobra_psp_launcher\"><Pair key=\"icon\"><String>/dev_hdd0/game/PSPC66820/ICON0.PNG</String></Pair><Pair key=\"icon_notation\"><String>WNT_XmbItemBrowser</String></Pair><Pair key=\"title\"><String>PSP Launcher</String></Pair><Pair key=\"info\"><String>%s</String></Pair></Table>", STR_LAUNCHPSP);
						strcat(myxml_psp, templn);
					}
				}
				if(!(webman_config->cmask & DVD) || !(webman_config->cmask & BLU)) strcpy(myxml_dvd, "<View id=\"seg_mygames_dvd_items\"><Attributes>");
			}
#endif
		}

		cellRtcGetCurrentTick(&pTick);

		int ns=-2;
		u8 retries=0;

		led(YELLOW, BLINK);

		for(u8 f0=0; f0<10; f0++)
		{
			for(u8 f1=0; f1<11; f1++)
			{
				if(!cobra_mode && (f1>1 && f1<10)) break;

				if(key>1020) break;

				cellRtcGetCurrentTick(&pTick);

				if(f0==9 && f1>6) break;   //ntfs
				if(f1==10 && f0!=0) break; //video
				if(f0==7 && (!webman_config->netd0 || f1>6 || !cobra_mode)) break;
				if(f0==8 && (!webman_config->netd1 || f1>6 || !cobra_mode)) break;
				if(f0==7 || f0==8) is_net=1; else is_net=0;
#ifdef COBRA_ONLY
				if(ns==-2 && is_net &&
					( (f0==7 && webman_config->netp0 && webman_config->neth0[0]) ||
					(  f0==8 && webman_config->netp1 && webman_config->neth1[0]) )
					)
				{
reconnect:
					if(f0==7)
						ns=connect_to_server(webman_config->neth0, webman_config->netp0);
					else
						ns=connect_to_server(webman_config->neth1, webman_config->netp1);
					if(ns<0)
					{
						if(retries<10)
						{
							retries++;
							sys_timer_usleep(500000);
							goto reconnect;
						}
					}
				}
#endif

				if(ns<0 && is_net) break;

				if(is_net)
					sprintf(param, "/%s", paths[f1]);
				else
				{
					if(f0==9)//ntfs
						strcpy(param, WMTMP);
					else
						sprintf(param, "%s/%s", drives[f0], paths[f1]);
				}

				if(f1==5 && f0!=0) continue; // PS2ISO only on /dev_hdd0 supported

				//if(cobra_mode)
				{
					if(f0==1 && (f1==0 || f1==1 || f1==10) && webman_config->bootd)
					{
						for(u8 z=0; z<((webman_config->bootd)*2); z++) // 5 seconds
						{
							if(cellFsStat((char*)"/dev_usb", &buf)==CELL_FS_SUCCEEDED)
								break;
							else
								sys_timer_usleep(500000);
						}
					}

					if(f0>=1 && f0<=6 && (f1==0 || f1==1 || f1==10)) // usb000->006
					{

						if(((webman_config->usb0 && f0==1) ||
							(webman_config->usb1 && f0==2) ||
							(webman_config->usb2 && f0==3) ||
							(webman_config->usb3 && f0==4) ||
							(webman_config->usb6 && f0==5) ||
							(webman_config->usb7 && f0==6)
							) && webman_config->boots )
						{
							for(u8 z=0; z<((webman_config->boots)*2); z++) // 3 seconds per usb drive
							{
								if(cellFsStat(drives[f0], &buf)==CELL_FS_SUCCEEDED)
									break;
								else
									sys_timer_usleep(500000);
							}
						}
					}
				}

				if(f0==9 && !webman_config->usb0 && !webman_config->usb1 && !webman_config->usb2 &&
					!webman_config->usb3 && !webman_config->usb6 && !webman_config->usb7) continue;

				if(!webman_config->usb0 && strstr(param, drives[1])) continue;
				if(!webman_config->usb1 && strstr(param, drives[2])) continue;
				if(!webman_config->usb2 && strstr(param, drives[3])) continue;
				if(!webman_config->usb3 && strstr(param, drives[4])) continue;
				if(!webman_config->usb6 && strstr(param, drives[5])) continue;
				if(!webman_config->usb7 && strstr(param, drives[6])) continue;

				if( (webman_config->cmask & PS3) && (f1< 3 || f1==10)) continue;
				if( (webman_config->cmask & BLU) && f1==3) continue;
				if( (webman_config->cmask & DVD) && f1==4) continue;
				if( (webman_config->cmask & PS2) && f1==5) continue;
				if( (webman_config->cmask & PS1) && f1==6) continue;
				if( (webman_config->cmask & PS1) && f1==7) continue;
				if( (webman_config->cmask & PSP) && f1==8) continue;
				if( (webman_config->cmask & PSP) && f1==9) continue;

				if(!is_net && cellFsOpendir( param, &fd) != CELL_FS_SUCCEEDED) continue;

				int abort_connection=0;
#ifdef COBRA_ONLY
				if(is_net && open_remote_dir(ns, param, &abort_connection) < 0) continue;
#endif
				//led(YELLOW, ON);
				{
					CellFsDirent entry;
					u64 read_e;
					int fdw, fs;
					uint64_t msiz = 0;
					u8 is_iso=0;
					char icon[512];
					char tempID[16];
					unsigned char *mem = (u8*)tempstr;
					sys_addr_t data2=0;
					int v3_entries=0;
					int v3_entry=0;
					int is_directory=0;
					int64_t file_size;
					u64 mtime, ctime, atime;
					int bytes_read=0;
#ifdef COBRA_ONLY
					netiso_read_dir_result_data *data=NULL;

					if(is_net)
					{
						v3_entries = read_remote_dir(ns, &data2, &abort_connection);
						if(data2==NULL) continue;
						data=(netiso_read_dir_result_data*)data2;
					}
#endif
					while((!is_net && cellFsReaddir(fd, &entry, &read_e) == 0 && read_e > 0)
						|| (is_net && v3_entry<v3_entries)
						)
					{
						if(key>1020) break;
						cellRtcGetCurrentTick(&pTick);
#ifdef COBRA_ONLY
						if(is_net)
						{
							icon[0]=0;

							if(!data[v3_entry].is_directory)
							{
								if(!strstr(data[v3_entry].name, ".ISO") && !strstr(data[v3_entry].name, ".iso") && !strstr(data[v3_entry].name, ".BIN") && !strstr(data[v3_entry].name, ".bin")) {v3_entry++; continue;}
								if(data[v3_entry].name[strlen(data[v3_entry].name)-4]!='.') {v3_entry++; continue;}
							}
							else
								if(!strstr(param, "/GAME")) {v3_entry++; continue;}

							if((f1< 3 || f1==10))//PS3 games only
							{

								sprintf(templn, WMTMP "/%s.SFO", data[v3_entry].name);
								if(!data[v3_entry].is_directory)
								{
									templn[strlen(templn)-7]='S';
									templn[strlen(templn)-6]='F';
									templn[strlen(templn)-5]='O';
									templn[strlen(templn)-4]=0;
								}

								if(data[v3_entry].is_directory && cellFsStat(templn, &buf)!=CELL_FS_SUCCEEDED)
								{
									sprintf(templn, "%s/%s/PS3_GAME/PARAM.SFO", param, data[v3_entry].name);
									if(remote_stat(ns, templn, &is_directory, &file_size, &mtime, &ctime, &atime, &abort_connection)!=0) {v3_entry++; continue;}
									sprintf(templn, WMTMP "/%s.SFO", data[v3_entry].name);

									if(file_size && cellFsOpen(templn, CELL_FS_O_CREAT|CELL_FS_O_RDWR|CELL_FS_O_TRUNC, &fdw, NULL, 0)==CELL_FS_SUCCEEDED)
									{
										sprintf(templn, "%s/%s/PS3_GAME/PARAM.SFO", param, data[v3_entry].name);
										open_remote_file_2(ns, templn, &abort_connection);

										int boff=0;
										while(boff<file_size)
										{
											bytes_read = read_remote_file(ns, (char*)tempstr, boff, 3072, &abort_connection);
											if(bytes_read)
												cellFsWrite(fdw, (char*)tempstr, bytes_read, NULL);
											boff+=bytes_read;
											if(bytes_read<3072 || boff>=file_size) break;
										}
										cellFsClose(fdw);
									}
									sprintf(templn, WMTMP "/%s.SFO", data[v3_entry].name);
									cellFsChmod(templn, 0666);
								}

								if(cellFsOpen(templn, CELL_FS_O_RDONLY, &fdw, 0, 0)==CELL_FS_SUCCEEDED)
								{
									//cellFsOpen(templn, CELL_FS_O_RDONLY, &fdw, 0, 0);
									cellFsLseek(fdw, 0, CELL_FS_SEEK_SET, &msiz);
									cellFsRead(fdw, (void *)&tempstr, 4096, &msiz);
									cellFsClose(fdw);
									sprintf(templn, "%s", data[v3_entry].name);
									if(msiz>256)
									{
										parse_param_sfo(mem, tempID, templn);

										icon[0]=0;
										if(tempID[0] && !webman_config->nocov) get_icon(icon, tempID);
									}
								}
								else
									strcpy(templn, data[v3_entry].name);
							}
							else
								strcpy(templn, data[v3_entry].name);

							if(templn[strlen(templn)-4]=='.') templn[strlen(templn)-4]=0;

							if(icon[0]==0)
							{
								sprintf(icon, WMTMP "/%s.PNG", data[v3_entry].name);
								if(!data[v3_entry].is_directory)
								{
									icon[strlen(icon)-7]='P';
									icon[strlen(icon)-6]='N';
									icon[strlen(icon)-5]='G';
									icon[strlen(icon)-4]=0;
								} else goto go3;

								if(cellFsStat(icon, &buf)!=CELL_FS_SUCCEEDED)
								{
									icon[strlen(icon)-3]='J';
									icon[strlen(icon)-2]='P';
								} else goto c_load2;

								if(cellFsStat(icon, &buf)!=CELL_FS_SUCCEEDED)
								{
									icon[strlen(icon)-3]='j';
									icon[strlen(icon)-2]='p';
									icon[strlen(icon)-1]='g';
								} else goto c_load2;

								if(cellFsStat(icon, &buf)!=CELL_FS_SUCCEEDED)
								{
									icon[strlen(icon)-3]='p';
									icon[strlen(icon)-2]='n';
								} else goto c_load2;
go3:
								if(cellFsStat(icon, &buf)!=CELL_FS_SUCCEEDED)
								{
									if(data[v3_entry].is_directory)
									{
										sprintf(tempstr, "%s/%s/PS3_GAME/ICON0.PNG", param, data[v3_entry].name);
										icon[strlen(icon)-3]='P';
										icon[strlen(icon)-2]='N';
										icon[strlen(icon)-1]='G';
									}
									else
									{
										sprintf(tempstr, "%s/%s", param, data[v3_entry].name);
										icon[strlen(icon)-3]='j';
										icon[strlen(icon)-2]='p';
										icon[strlen(icon)-1]='g';

										tempstr[strlen(tempstr)-3]='j';
										tempstr[strlen(tempstr)-2]='p';
										tempstr[strlen(tempstr)-1]='g';
									}

									abort_connection=0;
									if(remote_stat(ns, tempstr, &is_directory, &file_size, &mtime, &ctime, &atime, &abort_connection)!=0)
									{
										tempstr[strlen(tempstr)-3]='p';
										tempstr[strlen(tempstr)-2]='n';
										tempstr[strlen(tempstr)-1]='g';
										icon[strlen(icon)-3]='P';
										icon[strlen(icon)-2]='N';
										icon[strlen(icon)-1]='G';
										if(remote_stat(ns, tempstr, &is_directory, &file_size, &mtime, &ctime, &atime, &abort_connection)!=0)
										{
											icon[0]=0;
											goto c_load2;
										}
									}

									if(file_size && open_remote_file_2(ns, tempstr, &abort_connection)>0 && !abort_connection)
									{
										if(cellFsOpen(icon, CELL_FS_O_CREAT|CELL_FS_O_RDWR|CELL_FS_O_TRUNC, &fdw, NULL, 0)==CELL_FS_SUCCEEDED)
										{
											int boff=0;
											while(boff<file_size)
											{
												bytes_read = read_remote_file(ns, (char*)tempstr, boff, 3072, &abort_connection);
												if(bytes_read)
													cellFsWrite(fdw, (char*)tempstr, bytes_read, NULL);
												boff+=bytes_read;
												if(bytes_read<3072 || abort_connection || boff>=file_size) break;
											}
											cellFsClose(fdw);
											if(boff<1 || abort_connection) cellFsUnlink(icon);
											open_remote_file_2(ns, (char*)"/CLOSEFILE", &bytes_read);
										}
									}
								}
							}
c_load2:
							get_default_icon(icon, param, data[v3_entry].name);

							sprintf(tempstr, "<Table key=\"%04i\"><Pair key=\"icon\"><String>%s</String></Pair><Pair key=\"title\"><String>%s</String></Pair><Pair key=\"module_name\"><String>webbrowser_plugin</String></Pair><Pair key=\"module_action\"><String>http://127.0.0.1/mount_ps3/net%i%s/%s?random=%i</String></Pair><Pair key=\"info\"><String>/net%i%s</String></Pair></Table>",
								key, icon, templn, (f0-7), param, data[v3_entry].name, (int)pTick.tick, (f0-7), param);

							v3_entry++;

							if(strlen(templn)<5) strcat(templn, "     ");
							sprintf(skey[key], "3%c%c%c%c%04i", templn[0], templn[1], templn[2], templn[3], key);
							if( !(webman_config->nogrp))
							{
								/*if(strstr(param, "/PSPISO") && strlen(myxml_psp)<(BUFFER_SIZE_PSP-1024))
								{strcat(myxml_psp, tempstr); skey[key][0]='4';}
								else*/
								if(strstr(param, "/PSX") && strlen(myxml_psx)<(BUFFER_SIZE_PSX-1024))
								{strcat(myxml_psx, tempstr); skey[key][0]='1';}
								else
								if((strstr(param, "/BDISO") || strstr(param, "/DVDISO")) && strlen(myxml_dvd)<(BUFFER_SIZE_DVD-1024))
								{strcat(myxml_dvd, tempstr); skey[key][0]='0';}
								else
								if(strlen(myxml_ps3)<(BUFFER_SIZE-5000))
									strcat(myxml_ps3, tempstr);
							}
							else
							{
								if(strlen(myxml_ps3)<(BUFFER_SIZE-5000))
									strcat(myxml_ps3, tempstr);
							}
							key++;
						}
						else
#endif
						{
							sprintf(templn, "%s/%s/PS3_GAME/PARAM.SFO", param, entry.d_name);
							is_iso=((strstr(param, "/PS3ISO") || strstr(param, "/PS2ISO") ||
									strstr(param, "/PSPISO") || strstr(param, "/ISO")||
									strstr(param, "/PSX") || strstr(param, "/DVDISO") || strstr(param, "/BDISO") ||
									strstr(param, WMTMP))
								&&
								(
								  (strstr(entry.d_name, ".ISO") || strstr(entry.d_name, ".iso") ||
									(strstr(param, "/PSX") &&
										(strstr(entry.d_name, ".CUE") || strstr(entry.d_name, ".cue"))
									)
								   )
								&&
								  ((entry.d_name[strlen(entry.d_name)-1]=='0' && entry.d_name[strlen(entry.d_name)-2]=='.') || entry.d_name[strlen(entry.d_name)-1]=='o' || entry.d_name[strlen(entry.d_name)-1]=='O' || entry.d_name[strlen(entry.d_name)-1]=='e' || entry.d_name[strlen(entry.d_name)-1]=='E')
								)) || strstr(entry.d_name, ".ntfs[");
							if(is_iso || cellFsStat(templn, &buf)==CELL_FS_SUCCEEDED)
							{
								msiz=0;
								tempID[0]=0;
								if(!is_iso)
								{
									cellFsOpen(templn, CELL_FS_O_RDONLY, &fdw, 0, 0);
									cellFsLseek(fdw, 0, CELL_FS_SEEK_SET, &msiz);
									cellFsRead(fdw, (void *)&tempstr, ((buf.st_size>4096)?4096:buf.st_size), &msiz);
									cellFsClose(fdw);
									sprintf(templn, "%s", entry.d_name);
									if(msiz>256)
									{
										parse_param_sfo(mem, tempID, templn);
									}
								}
								else
								{
									tempID[0]=0;
									if((strstr(param, "/PS3ISO") && f0<9) || (f0==9 && f1==2 && strstr(entry.d_name, "ntfs[PS3ISO]")))
									{
										sprintf(templn, WMTMP "/%s", entry.d_name);
										if(strstr(templn, "ntfs[PS3ISO]")) templn[strlen(templn)-13]=0;
										if(templn[strlen(templn)-2]=='.') templn[strlen(templn)-2]=0;
										templn[strlen(templn)-3]='S';
										templn[strlen(templn)-2]='F';
										templn[strlen(templn)-1]='O';

										if(cellFsOpen(templn, CELL_FS_O_RDONLY, &fdw, 0, 0)==CELL_FS_SUCCEEDED)
										{
											cellFsLseek(fdw, 0, CELL_FS_SEEK_SET, &msiz);
											cellFsRead(fdw, (void *)&tempstr, ((buf.st_size>4096)?4096:buf.st_size), &msiz);
											cellFsClose(fdw);
											sprintf(templn, "%s", entry.d_name);
											if(strstr(templn, "ntfs[PS3ISO]")) templn[strlen(templn)-13]=0;
											if(templn[strlen(templn)-2]=='.') templn[strlen(templn)-2]=0;
											if(templn[strlen(templn)-4]=='.') templn[strlen(templn)-4]=0;
											if(msiz>256)
											{
												parse_param_sfo(mem, tempID, templn);
											}
										}
										else
										{
											sprintf(templn, "%s", entry.d_name);
											if(strstr(templn, "ntfs[PS3ISO]")) templn[strlen(templn)-13]=0;
											if(templn[strlen(templn)-2]=='.') templn[strlen(templn)-2]=0;
											if(templn[strlen(templn)-4]=='.') templn[strlen(templn)-4]=0;
										}
									}
									else
									{
										sprintf(templn, "%s", entry.d_name);
										if(f0==9)
										{
											if(f1< 2 || f1>6) continue;
											if(f1==2 && !strstr(entry.d_name, "ntfs[PS3ISO]")) continue;
											if(f1==3 && !strstr(entry.d_name, "ntfs[BDISO]")) continue;
											if(f1==4 && !strstr(entry.d_name, "ntfs[DVDISO]")) continue;
											if(f1==6 && !strstr(entry.d_name, "ntfs[PSXISO]")) continue;

											if(f1==3 && strstr(entry.d_name, "ntfs[BDISO]")) templn[strlen(templn)-12]=0;
											if(f1==4 && strstr(entry.d_name, "ntfs[DVDISO]")) templn[strlen(templn)-13]=0;
											if(f1==6 && strstr(entry.d_name, "ntfs[PSXISO]")) templn[strlen(templn)-13]=0;
										}

										if(templn[strlen(templn)-2]=='.') templn[strlen(templn)-2]=0;
										if(templn[strlen(templn)-4]=='.') templn[strlen(templn)-4]=0;
									}

									if(strstr(param, "/PS3ISO") && tempID[0]==0 && f0<9)
									{
										sprintf(icon, "%s/%s", param, entry.d_name);
										if (cellFsOpen(icon, CELL_FS_O_RDONLY, &fs, NULL, 0) == CELL_FS_SUCCEEDED)
										{
											if(cellFsLseek(fs, 0x810, CELL_FS_SEEK_SET, &msiz) == CELL_FS_SUCCEEDED)
											{
												if(cellFsRead(fs, (void *)&tempID, 11, &msiz) == CELL_FS_SUCCEEDED)
												{
													tempID[4]=tempID[5];
													tempID[5]=tempID[6];
													tempID[6]=tempID[7];
													tempID[7]=tempID[8];
													tempID[8]=tempID[9];
													tempID[9]=0;
												}
											}
											cellFsClose(fs);
										}
									}
								}
		//title_foundx:
								icon[0]=0;
								if(tempID[0] && !webman_config->nocov) get_icon(icon, tempID);

								if(is_iso)
								{
									if(icon[0]==0 || cellFsStat(icon, &buf)!=CELL_FS_SUCCEEDED)
									{
										sprintf(icon, "%s/%s", param, entry.d_name);
										if(strstr(icon, "ntfs[PS3ISO]") || strstr(icon, "ntfs[DVDISO]") || strstr(icon, "ntfs[PSXISO]")) icon[strlen(icon)-13]=0;
										if(strstr(icon, "ntfs[BDISO]")) icon[strlen(icon)-12]=0;
										if(icon[strlen(icon)-4]=='.')
										{
											icon[strlen(icon)-3]='p'; icon[strlen(icon)-2]='n'; icon[strlen(icon)-1]='g';
											if(cellFsStat(icon, &buf)!=CELL_FS_SUCCEEDED)
												{icon[strlen(icon)-3]='P'; icon[strlen(icon)-2]='N'; icon[strlen(icon)-1]='G';}
										}
										else
										if(icon[strlen(icon)-2]=='.')
										{
											icon[strlen(icon)-5]='p'; icon[strlen(icon)-4]='n'; icon[strlen(icon)-3]='g';  icon[strlen(icon)-2]=0;
										}

										if(cellFsStat(icon, &buf)!=CELL_FS_SUCCEEDED)
											{icon[strlen(icon)-3]='j'; icon[strlen(icon)-2]='p'; icon[strlen(icon)-1]='g';}
									}

									get_default_icon(icon, param, entry.d_name);
								}
								else if(icon[0]==0 || cellFsStat(icon, &buf)!=CELL_FS_SUCCEEDED)
									sprintf(icon, "%s/%s/PS3_GAME/ICON0.PNG", param, entry.d_name);

								sprintf(tempstr, "<Table key=\"%04i\"><Pair key=\"icon\"><String>%s</String></Pair><Pair key=\"title\"><String>%s</String></Pair><Pair key=\"module_name\"><String>webbrowser_plugin</String></Pair><Pair key=\"module_action\"><String>http://127.0.0.1/mount_ps3%s/%s?random=%i</String></Pair><Pair key=\"info\"><String>%s</String></Pair></Table>",
									key, icon, templn, param, entry.d_name, (int)pTick.tick, (f0==9?(char*)" ":param));

								if(strlen(templn)<5) strcat(templn, "     ");
								sprintf(skey[key], "3%c%c%c%c%04i", templn[0], templn[1], templn[2], templn[3], key);
#ifdef COBRA_ONLY
								if( !(webman_config->nogrp) && cobra_mode)
								{
									if(strstr(param, "/PS2ISO") && strlen(myxml_ps2)<(BUFFER_SIZE_PS2-1024))
									{strcat(myxml_ps2, tempstr); skey[key][0]='2';}
									else
									if(strstr(param, "/PSPISO") && strlen(myxml_psp)<(BUFFER_SIZE_PSP-1024))
									{strcat(myxml_psp, tempstr); skey[key][0]='4';}
									else
									if(strstr(param, "/ISO") && strlen(myxml_psp)<(BUFFER_SIZE_PSP-1024))
									{strcat(myxml_psp, tempstr); skey[key][0]='4';}
									else
									if((strstr(param, "/PSX") || strstr(entry.d_name, "ntfs[PSXISO]")) && strlen(myxml_psx)<(BUFFER_SIZE_PSX-1024))
									{strcat(myxml_psx, tempstr); skey[key][0]='1';}
									else
									if((strstr(param, "/BDISO") || strstr(param, "/DVDISO") || strstr(entry.d_name, "ntfs[DVDISO]") || strstr(entry.d_name, "ntfs[BDISO]")) && strlen(myxml_dvd)<(BUFFER_SIZE_DVD-1024))
									{strcat(myxml_dvd, tempstr); skey[key][0]='0';}
									else
									if(strlen(myxml_ps3)<(BUFFER_SIZE-5000))
										strcat(myxml_ps3, tempstr);
								}
								else
#endif
								{
									if(strlen(myxml_ps3)<(BUFFER_SIZE-5000))
										strcat(myxml_ps3, tempstr);
								}

								key++;
							}
						}
					}
					if(!is_net) cellFsClosedir(fd);
					if(is_net && data2!=NULL) sys_memory_free(data2);
				}
			}
			if(is_net && ns>=0) {shutdown(ns, SHUT_RDWR); socketclose(ns); ns=-2;}
		}

		if( !(webman_config->nogrp))
		{
			if(!(webman_config->cmask & PS3)) strcat(myxml_ps3, "</Attributes><Items>");
#ifdef COBRA_ONLY
			{
				if(!(webman_config->cmask & PS2)) strcat(myxml_ps2, "</Attributes><Items>");
				if(!(webman_config->cmask & PS1)) strcat(myxml_psx, "</Attributes><Items>");
				if(!(webman_config->cmask & PSP)) {strcat(myxml_psp, "</Attributes><Items>"); if(webman_config->pspl && cobra_mode && cellFsStat((char*)"/dev_hdd0/game/PSPC66820", &buf)==CELL_FS_SUCCEEDED) strcat(myxml_psp, "<Query class=\"type:x-xmb/folder-pixmap\" key=\"cobra_psp_launcher\" attr=\"cobra_psp_launcher\" src=\"xcb://localhost/query?limit=1&cond=Ae+Game:Game.titleId PSPC66820\"/>");}
				if(!(webman_config->cmask & DVD) || !(webman_config->cmask & BLU)) strcat(myxml_dvd, "</Attributes><Items>");
			}
#endif
		}
		else
			myxml_dvd[0]=0;

		led(YELLOW, OFF);
		led(GREEN, ON);
	/*
	int data_s=connect_to_server((char*)"192.168.100.209", 38008);
	for(u32 n=0; n<key; n++)
	{
		sprintf(tempstr, "%i) [%s] (%i)\r\n", n, skey[n], strlen(skey[n]));
		send(data_s, tempstr, strlen(tempstr), 0);
	}

	for(u32 n=0; n<key; n++)
	{
		sprintf(tempstr, "%i] [%s] (%i)\r\n", n, skey[n], strlen(skey[n]));
		send(data_s, tempstr, strlen(tempstr), 0);
	}
	sclose(&data_s)
	*/
		if(key)
		{
			for(u32 n=0; n<(key-1); n++)
			{
				for(u32 m=(n+1); m<key; m++)
				{
					if( (webman_config->nogrp))
					{
						if(strcasecmp(skey[n]+1, skey[m]+1)>0)
						{
							strcpy(swap, skey[n]);
							strcpy(skey[n], skey[m]);
							strcpy(skey[m], swap);
						}
					}

					else
					{
						if(strcasecmp(skey[n], skey[m])>0)
						{
							strcpy(swap, skey[n]);
							strcpy(skey[n], skey[m]);
							strcpy(skey[m], swap);
						}
					}
				}
			}
		}

		if( (webman_config->nogrp))
		{
			strcat(myxml_items, "<Item class=\"type:x-xmb/module-action\" key=\"eject\" attr=\"eject\"/>");
			if(!webman_config->noset) strcat(myxml_items, "<Item class=\"type:x-xmb/module-action\" key=\"setup\" attr=\"setup\"/>");
		}

		for(u32 a=0; a<key; a++)
		{
			sprintf(tempstr, "<Item class=\"type:x-xmb/module-action\" key=\"%s\" attr=\"%s\"/>", skey[(a)]+5, skey[(a)]+5);
			if( !(webman_config->nogrp))
			{
#ifdef COBRA_ONLY
				//if(cobra_mode)
				{
					if(skey[(a)][0]=='2' && strlen(myxml_ps2)<(BUFFER_SIZE_PS2-128))
						strcat(myxml_ps2, tempstr);
					else
					if(skey[(a)][0]=='4' && strlen(myxml_psp)<(BUFFER_SIZE_PSP-128))
						strcat(myxml_psp, tempstr);
					else
					if(skey[(a)][0]=='1' && strlen(myxml_psx)<(BUFFER_SIZE_PSX-128))
						strcat(myxml_psx, tempstr);
					else
					if(skey[(a)][0]=='0' && strlen(myxml_dvd)<(BUFFER_SIZE_DVD-128))
						strcat(myxml_dvd, tempstr);
					else
					if(strlen(myxml_ps3)<(BUFFER_SIZE-5000))
						strcat(myxml_ps3, tempstr);
				}
#else
				if(strlen(myxml_ps3)<(BUFFER_SIZE-5000))
					strcat(myxml_ps3, tempstr);
#endif
			}
			else
				if(strlen(myxml_dvd)<(BUFFER_SIZE-1000))
					strcat(myxml_items, tempstr);
		}

		if( !(webman_config->nogrp))
		{
			if(!(webman_config->cmask & PS3)) strcat(myxml_ps3, "</Items></View>");
#ifdef COBRA_ONLY
			{
				if(!(webman_config->cmask & PS2)) strcat(myxml_ps2, "</Items></View>");
				if(!(webman_config->cmask & PS1)) strcat(myxml_psx, "</Items></View>");
				if(!(webman_config->cmask & PSP)) strcat(myxml_psp, "</Items></View>");
				if(!(webman_config->cmask & DVD) || !(webman_config->cmask & BLU)) strcat(myxml_dvd, "</Items></View>");
			}
#endif
		}

		//sprintf(myxml, "<?xml version=\"1.0\" encoding=\"UTF-8\"?><XMBML version=\"1.0\"><View id=\"seg_mygames\"><Attributes><Table key=\"eject\"><Pair key=\"icon\"><String>/dev_flash/vsh/resource/explore/icon/icon_home.png</String></Pair><Pair key=\"title\"><String>Eject Disc</String></Pair><Pair key=\"module_name\"><String>webbrowser_plugin</String></Pair><Pair key=\"module_action\"><String>http://127.0.0.1/mount_ps3/unmount?random=-1059561539</String></Pair><Pair key=\"info\"><String>Unmount current game</String></Pair></Table><Table key=\"mygames_ps3\"><Pair key=\"icon\"><String>%s</String></Pair><Pair key=\"title\"><String>PLAYSTATION\xC2\xAE\x33</String></Pair><Pair key=\"info\"><String>PS3 format games                              </String></Pair></Table><Table key=\"mygames_ps2\"><Pair key=\"icon\"><String>%s</String></Pair><Pair key=\"title\"><String>PLAYSTATION\xC2\xAE\x32</String></Pair><Pair key=\"info\"><String>PS2 format games                              </String></Pair></Table><Table key=\"mygames_psx\"><Pair key=\"icon\"><String>%s</String></Pair><Pair key=\"title\"><String>PLAYSTATION\xC2\xAE</String></Pair><Pair key=\"info\"><String>PSOne format games                              </String></Pair></Table><Table key=\"mygames_dvd\"><Pair key=\"icon\"><String>%s</String></Pair><Pair key=\"title\"><String>Blu-ray\xE2\x84\xA2 and DVD</String></Pair><Pair key=\"info\"><String>Video content                              </String></Pair></Table></Attributes><Items><Item class=\"type:x-xmb/module-action\" key=\"eject\" attr=\"eject\"/><Query class=\"type:x-xmb/folder-pixmap\" key=\"mygames_ps3\" attr=\"mygames_ps3\" src=\"#seg_mygames_ps3_items\"/><Query class=\"type:x-xmb/folder-pixmap\" key=\"mygames_ps2\" attr=\"mygames_ps2\" src=\"#seg_mygames_ps2_items\"/><Query class=\"type:x-xmb/folder-pixmap\" key=\"mygames_psx\" attr=\"mygames_psx\" src=\"#seg_mygames_psx_items\"/><Query class=\"type:x-xmb/folder-pixmap\" key=\"mygames_dvd\" attr=\"mygames_dvd\" src=\"#seg_mygames_dvd_items\"/></Items></View>", ps3icon, ps2icon, psxicon, dvdicon);
		sprintf(templn, "<?xml version=\"1.0\" encoding=\"UTF-8\"?><XMBML version=\"1.0\"><View id=\"seg_mygames\"><Attributes><Table key=\"eject\"><Pair key=\"icon\"><String>/dev_flash/vsh/resource/explore/icon/icon_home.png</String></Pair><Pair key=\"title\"><String>%s</String></Pair><Pair key=\"module_name\"><String>webbrowser_plugin</String></Pair><Pair key=\"module_action\"><String>http://127.0.0.1/mount_ps3/unmount?random=-1059561539</String></Pair><Pair key=\"info\"><String>%s</String></Pair></Table>", STR_EJECTDISC, STR_UNMOUNTGAME); strcpy(myxml, templn);
		if( !(webman_config->nogrp))
		{
			if( !(webman_config->cmask & PS3)) {sprintf(templn, "<Table key=\"mygames_ps3\"><Pair key=\"icon\"><String>/dev_flash/vsh/resource/explore/user/024.png</String></Pair><Pair key=\"title\"><String>PLAYSTATION\xC2\xAE\x33</String></Pair><Pair key=\"info\"><String>%s</String></Pair><Pair key=\"str_noitem\"><String>msg_error_no_content</String></Pair></Table>", STR_PS3FORMAT); strcat(myxml, templn);}
#ifdef COBRA_ONLY
			{
				if( !(webman_config->cmask & PS2)) {sprintf(templn, "<Table key=\"mygames_ps2\"><Pair key=\"icon\"><String>/dev_flash/vsh/resource/explore/user/025.png</String></Pair><Pair key=\"title\"><String>PLAYSTATION\xC2\xAE\x32</String></Pair><Pair key=\"info\"><String>%s</String></Pair><Pair key=\"str_noitem\"><String>msg_error_no_content</String></Pair></Table>", STR_PS2FORMAT); strcat(myxml, templn);}
				if( !(webman_config->cmask & PS1)) {sprintf(templn, "<Table key=\"mygames_psx\"><Pair key=\"icon\"><String>/dev_flash/vsh/resource/explore/user/026.png</String></Pair><Pair key=\"title\"><String>PLAYSTATION\xC2\xAE</String></Pair><Pair key=\"info\"><String>%s</String></Pair><Pair key=\"str_noitem\"><String>msg_error_no_content</String></Pair></Table>", STR_PS1FORMAT);strcat(myxml, templn);}
				if( !(webman_config->cmask & PSP)) {sprintf(templn, "<Table key=\"mygames_psp\"><Pair key=\"icon\"><String>/dev_flash/vsh/resource/explore/user/022.png</String></Pair><Pair key=\"title\"><String>PLAYSTATION\xC2\xAEPORTABLE</String></Pair><Pair key=\"info\"><String>%s</String></Pair><Pair key=\"str_noitem\"><String>msg_error_no_content</String></Pair></Table>", STR_PSPFORMAT);strcat(myxml, templn);}
				if( !(webman_config->cmask & DVD) || !(webman_config->cmask & BLU)) {sprintf(templn, "<Table key=\"mygames_dvd\"><Pair key=\"icon\"><String>/dev_flash/vsh/resource/explore/user/023.png</String></Pair><Pair key=\"title\"><String>%s</String></Pair><Pair key=\"info\"><String>%s</String></Pair><Pair key=\"str_noitem\"><String>msg_error_no_content</String></Pair></Table>", STR_VIDFORMAT, STR_VIDEO );strcat(myxml, templn);}
			}
#endif
		}

		bool add_xmbm_plus = cellFsStat("/dev_hdd0/game/XMBMANPLS/USRDIR/FEATURES/webMAN.xml", &buf)==CELL_FS_SUCCEEDED;

		if(!webman_config->noset)
			{sprintf(templn, "<Table key=\"setup\"><Pair key=\"icon\"><String>%s</String></Pair><Pair key=\"title\"><String>%s</String></Pair><Pair key=\"module_name\"><String>webbrowser_plugin</String></Pair><Pair key=\"info\"><String>%s</String></Pair>%s</Table>", add_xmbm_plus ? "/dev_hdd0/game/XMBMANPLS/USRDIR/IMAGES/multiman.png" : "/dev_flash/vsh/resource/explore/icon/icon_home.png", STR_WMSETUP, STR_WMSETUP2, add_xmbm_plus ? "<Pair key=\"child\"><String>segment</String></Pair>" : "<Pair key=\"module_action\"><String>http://127.0.0.1/setup.ps3</String></Pair>");strcat(myxml, templn);}

		if( !(webman_config->nogrp))
		{
			strcat(myxml, "</Attributes><Items><Item class=\"type:x-xmb/module-action\" key=\"eject\" attr=\"eject\"/>");
			if(!(webman_config->cmask & PS3)) strcat(myxml, "<Query class=\"type:x-xmb/folder-pixmap\" key=\"mygames_ps3\" attr=\"mygames_ps3\" src=\"#seg_mygames_ps3_items\"/>");
#ifdef COBRA_ONLY
			{
				if(!(webman_config->cmask & PS2)) strcat(myxml, "<Query class=\"type:x-xmb/folder-pixmap\" key=\"mygames_ps2\" attr=\"mygames_ps2\" src=\"#seg_mygames_ps2_items\"/>");
				if(!(webman_config->cmask & PS1)) strcat(myxml, "<Query class=\"type:x-xmb/folder-pixmap\" key=\"mygames_psx\" attr=\"mygames_psx\" src=\"#seg_mygames_psx_items\"/>");
				if(!(webman_config->cmask & PSP)) strcat(myxml, "<Query class=\"type:x-xmb/folder-pixmap\" key=\"mygames_psp\" attr=\"mygames_psp\" src=\"#seg_mygames_psp_items\"/>");
				if(!(webman_config->cmask & DVD) || !(webman_config->cmask & BLU) ) strcat(myxml, "<Query class=\"type:x-xmb/folder-pixmap\" key=\"mygames_dvd\" attr=\"mygames_dvd\" src=\"#seg_mygames_dvd_items\"/>");
			}
#endif
		}

		if(!(webman_config->nogrp) && !(webman_config->noset))
        {
			if(add_xmbm_plus)
				strcat(myxml, "<Query class=\"type:x-xmb/folder-pixmap\" key=\"setup\" attr=\"setup\" src=\"xmb://localhost/dev_hdd0/game/XMBMANPLS/USRDIR/FEATURES/webMAN.xml#seg_webman_links_items\"/>");
			else
				strcat(myxml, "<Item class=\"type:x-xmb/module-action\" key=\"setup\" attr=\"setup\"/>");
        }

		if( !(webman_config->nogrp)) strcat(myxml, "</Items></View>");

		cellFsOpen(xml, CELL_FS_O_CREAT | CELL_FS_O_TRUNC | CELL_FS_O_WRONLY, &fdxml, NULL, 0);
		cellFsWrite(fdxml, (char*)myxml, strlen(myxml), NULL);
		if( (webman_config->nogrp))
		{
			cellFsWrite(fdxml, (char*)myxml_ps3, strlen(myxml_ps3), NULL);
			cellFsWrite(fdxml, (char*)"</Attributes><Items>", 20, NULL);
			cellFsWrite(fdxml, (char*)myxml_items, strlen(myxml_items), NULL);
		}
		else
		{
			cellFsWrite(fdxml, (char*)myxml_ps3, strlen(myxml_ps3), NULL);
#ifdef COBRA_ONLY
			{
				cellFsWrite(fdxml, (char*)myxml_ps2, strlen(myxml_ps2), NULL);
				cellFsWrite(fdxml, (char*)myxml_psx, strlen(myxml_psx), NULL);
				cellFsWrite(fdxml, (char*)myxml_psp, strlen(myxml_psp), NULL);
				cellFsWrite(fdxml, (char*)myxml_dvd, strlen(myxml_dvd), NULL);
			}
#endif
		}
		if( (webman_config->nogrp))
		{
			sprintf(myxml, "</Items></View></XMBML>\r\n");
		}
		else
			sprintf(myxml, "</XMBML>\r\n");
		cellFsWrite(fdxml, (char*)myxml, strlen(myxml), NULL);
		cellFsClose(fdxml);
		cellFsChmod(xml, 0666);
		u32 xmlsize=BUFFER_SIZE;

		if(cellFsOpen(xml, CELL_FS_O_RDONLY, &fdxml, NULL, 0) == CELL_FS_SUCCEEDED)
		{
			u64 read_e = 0;
			cellFsRead(fdxml, (void *)myxml_ps3, xmlsize, &read_e);
			cellFsClose(fdxml);
			for(u32 n=0;n<xmlsize;n++) if(myxml_ps3[n]=='&') myxml_ps3[n]='^';

			cellFsOpen((char*)"/dev_hdd0/xmlhost/game_plugin/mygames.droid", CELL_FS_O_CREAT | CELL_FS_O_TRUNC | CELL_FS_O_WRONLY, &fdxml, NULL, 0);
			cellFsWrite(fdxml, (char*)myxml_ps3, strlen(myxml_ps3), NULL);
			cellFsClose(fdxml);
		}

#ifdef COBRA_ONLY
		u8 do_delay=0;

		sprintf(myxml, "/dev_hdd0/PS3ISO/AUTOBOOT.ISO");
		if(cobra_mode && webman_config->autob && cellFsStat(myxml, &buf)==CELL_FS_SUCCEEDED)
			do_delay=1;

		else
		{
			sprintf(myxml, WMTMP "/last_game.txt");
			if(cobra_mode && webman_config->lastp && cellFsStat(myxml, &buf)==CELL_FS_SUCCEEDED)
			{
				int fd=0;
				if(cellFsOpen(myxml, CELL_FS_O_RDONLY, &fd, NULL, 0) == CELL_FS_SUCCEEDED)
				{
					u64 read_e = 0;
					if(cellFsRead(fd, (void *)myxml, 512, &read_e)==CELL_FS_SUCCEEDED)
					{
						cellFsClose(fd);
						myxml[read_e]=0;
						if(strlen(myxml)>10 || strstr(myxml, "/net") || strstr(myxml, ".ntfs["))
						{
							if(strstr(myxml, "/net") || cellFsStat(myxml, &buf)==CELL_FS_SUCCEEDED)
							{
								do_delay=1;
								if(strstr(myxml, ".ntfs[") && !webman_config->usb0 && !webman_config->usb1 && !webman_config->usb2 &&
									!webman_config->usb3 && !webman_config->usb6 && !webman_config->usb7) do_delay=0;
								else
								if(strstr(myxml, ".ntfs[") && !webman_config->delay) sys_timer_sleep(10);
							}
						}
					}
					cellFsClose(fd);
				}
			}
		}
		strcpy(tempstr, myxml);
#endif
		led(GREEN, ON);

#ifdef USE_VM
		sys_vm_unmap(sysmem);
#else
		sys_memory_free(sysmem);
#endif

#ifdef COBRA_ONLY
		if(do_delay)
		{
			if(webman_config->delay)
			{
				if(!webman_config->bootd || !webman_config->boots)
					sys_timer_sleep(20);
				else
					sys_timer_sleep(10);
			}
			mount_with_mm(tempstr, 0);
		}
#endif
		init_running=0;
		sys_ppu_thread_exit(0);
	}

#ifdef USE_DEBUG
	ssend(debug_s, "waiting...");
#endif
	if(loading_html>10) loading_html=0;
	//while((init_running/* || loading_html>3*/) && working) sys_timer_usleep(10000);

	sys_net_sockinfo_t conn_info_main;
	sys_net_get_sockinfo(conn_s, &conn_info_main, 1);
	if( webman_config->bind && (conn_info_main.local_adr.s_addr != conn_info_main.remote_adr.s_addr))
	{
		sclose(&conn_s);
		loading_html--;
		sys_ppu_thread_exit(0);
	}

	_meminfo meminfo;
	u8 retries=0;
again3:
	{system_call_1(352, (uint64_t) &meminfo);}
	if((meminfo.avail)<( (BUFFER_SIZE) + 256*1024))
	{
#ifdef USE_DEBUG
	ssend(debug_s, "!!! NOT ENOUGH MEMORY!\r\n");
#endif
		retries++;
		sys_timer_sleep(1);
		if(retries<3) goto again3;
		init_running=0;
		sclose(&conn_s);
		loading_html--;
		sys_ppu_thread_exit(0);
	}


	char buffer1[512];
	u8 is_ps3_http=0;

	struct timeval tv;
	tv.tv_usec = 0;
	tv.tv_sec = 3;
	setsockopt(conn_s, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv));

	while(!served && working)
	{
		served++;
		buffer1[0]=0;

#ifdef USE_DEBUG
	ssend(debug_s, "ACC - ");
#endif
		if(recv(conn_s, buffer1, 512, 0) > 0 && buffer1[0]=='G' && buffer1[4]=='/') // serve only GET /xxx requests
		{
			if(strstr(buffer1, "Mozilla/5.0 (PLAYSTATION 3;")) is_ps3_http=1; else is_ps3_http=0;
			buffer1[strcspn(buffer1, "\n")] = '\0';
			buffer1[strcspn(buffer1, "\r")] = '\0';

			ssplit(buffer1, cmd, 15, header, 511);
			ssplit(header, param, 511, cmd, 15);

#ifdef USE_DEBUG
	ssend(debug_s, param);
	ssend(debug_s, "\r\n");
#endif
			if(strstr(param, "%"))
			{
				strcpy(buffer1, param);
				int pos=0;
				for(u32 i=0;i<strlen(buffer1);i++)
				{
					if(buffer1[i]!='%')
						param[pos]=buffer1[i];
					else
					{
						if(buffer1[i+2]>='0' && buffer1[i+2]<='9') param[pos]=buffer1[i+2]-0x30;
						else
							if(buffer1[i+2]>='A' && buffer1[i+2]<='F') param[pos]=buffer1[i+2]-55;

						if(buffer1[i+1]>='0' && buffer1[i+1]<='9') param[pos]+=(buffer1[i+1]-0x30)*16;
						else
							if(buffer1[i+1]>='A' && buffer1[i+1]<='F') param[pos]+=(buffer1[i+1]-55)*16;

						i+=2;
					}
					pos++;
					param[pos]=0;
				}
			}
			if(strstr(param, "popup.ps3"))
			{
				if(strlen(param)>10)
				{
					sprintf(buffer1, "HTTP/1.1 200 OK\r\nContent-Type: text/html\r\nContent-Length: %i\r\n\r\n<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\"><html xmlns=\"http://www.w3.org/1999/xhtml\"><meta http-equiv=\"Content-type\" content=\"text/html;charset=UTF-8\"><META HTTP-EQUIV=\"CACHE-CONTROL\" CONTENT=\"NO-CACHE\"><html><body>Message sent: %s</body></html>", strlen(param)+312, param+11);
					ssend(conn_s, buffer1);
					show_msg(param+11);
				}
				sclose(&conn_s);
				loading_html--;

				sys_ppu_thread_exit(0);
			}

			if(strstr(param, "quit.ps3"))
			{
				show_msg((char*)STR_WMUNL);
				//restore_fan(1);
				working=0;
				sclose(&conn_s);
				//sys_memory_free(sysmem);
				loading_html=0;
				wwwd_stop();
				sys_ppu_thread_exit(0);
				break;
			}
			if(strstr(param, "shutdown.ps3"))
			{
				working=0;
				sclose(&conn_s);
				//sys_memory_free(sysmem);
				cellFsUnlink((char*)"/dev_hdd0/tmp/turnoff");
				{system_call_4(379,0x1100,0,0,0);}
				sys_ppu_thread_exit(0);
			}
			if(strstr(param, "restart.ps3"))
			{
				working=0;
				sclose(&conn_s);
				//sys_memory_free(sysmem);
				cellFsUnlink((char*)"/dev_hdd0/tmp/turnoff");
				{system_call_4(379,0x1200,0,0,0);}
				sys_ppu_thread_exit(0);
			}

			if(strstr(param, "index.ps3")
				|| strstr(param, "mount.ps3/")
				|| strstr(param, "mount_ps3/")
				|| strstr(param, "eject.ps3")
				|| strstr(param, "insert.ps3")
				|| strstr(param, "refresh.ps3")
				|| strstr(param, "setup.ps3")
				|| strstr(param, "cpursx.ps3")
				)
				is_binary=0;
			else
			{
				is_binary=1;
				if(cellFsStat(param, &buf)==CELL_FS_SUCCEEDED)
				{
					c_len=buf.st_size;
					if((buf.st_mode & S_IFDIR) != 0) is_binary=2;
				}
				else
				{
					if(param[1]=='n')//net0/net1
					{
						is_binary=2;
					}
					else
					{
						c_len=0;
						is_binary=0;
						sprintf(buffer1, "HTTP/1.1 200 OK\r\nX-PS3-Info: [%s]\r\nContent-Type: text/plain\r\nContent-Length: 5\r\n\r\nError", param);
						ssend(conn_s, buffer1);
						sclose(&conn_s);
						loading_html--;
						sys_ppu_thread_exit(0);
					}
				}
			}

			header[0]=0;
			strcpy(header, "HTTP/1.1 200 OK\r\nContent-Type: ");
			if(is_binary==1)
			{
				if(strstr(param, ".JPG") || strstr(param, ".jpg") || strstr(param, ".JPEG") || strstr(param, ".jpeg") || strstr(param, ".STH"))
					strcat(header, "image/jpeg");
				else
				if(strstr(param, ".PNG") || strstr(param, ".png"))
					strcat(header, "image/png");
				else
				if(strstr(param, ".GIF") || strstr(param, ".gif"))
					strcat(header, "image/gif");
				else
				if(strstr(param, ".TXT") || strstr(param, ".txt") || strstr(param, ".LOG") || strstr(param, ".log") || strstr(param, ".INI") || strstr(param, ".ini") || strstr(param, ".HIP") || strstr(param, ".HIS"))
					strcat(header, "text/plain");
				else
				if(strstr(param, ".css"))
					strcat(header, "text/css");
				else
				if(strstr(param, ".htm") || strstr(param, ".html"))
					strcat(header, "text/html");
#ifndef NO_EXTRA_CONTENT
				else
				if(strstr(param, ".AVI") || strstr(param, ".avi"))
					strcat(header, "video/x-msvideo");
				else
				if(strstr(param, ".MKV") || strstr(param, ".mkv"))
					strcat(header, "video/x-matroska");
				else
				if(strstr(param, ".MP4") || strstr(param, ".mp4"))
					strcat(header, "video/mp4");
				else
				if(strstr(param, ".MPG") || strstr(param, ".mpg") || strstr(param, ".MPE") || strstr(param, ".mpe") || strstr(param, ".MP2") || strstr(param, ".mp2"))
					strcat(header, "video/mpeg");
				else
				if(strstr(param, ".VOB") || strstr(param, ".vob"))
					strcat(header, "video/vob");
				else
				if(strstr(param, ".WMV") || strstr(param, ".wmv"))
					strcat(header, "video/x-ms-wmv");
				else
				if(strstr(param, ".MOV") || strstr(param, ".mov"))
					strcat(header, "video/quicktime");
				else
				if(strstr(param, ".MP3") || strstr(param, ".mp3"))
					strcat(header, "audio/mpeg");
				else
				if(strstr(param, ".WAV") || strstr(param, ".wav"))
					strcat(header, "audio/x-wav");
				else
				if(strstr(param, ".BMP") || strstr(param, ".bmp"))
					strcat(header, "image/bmp");
				else
				if(strstr(param, ".TIF") || strstr(param, ".tif"))
					strcat(header, "image/tiff");
				else
				if(strstr(param, ".ZIP") || strstr(param, ".zip"))
					strcat(header, "application/zip");
				else
				if(strstr(param, ".PDF") || strstr(param, ".pdf"))
					strcat(header, "application/pdf");
				else
				if(strstr(param, ".SWF") || strstr(param, ".swf"))
					strcat(header, "application/x-shockwave-flash");
#endif
				else
					strcat(header, "application/octet-stream");
			}
			else
				strcat(header, "text/html");

			strcat(header, "\r\n");

			if(strlen(param)>1 && param[strlen(param)-1]=='/') param[strlen(param)-1]=0;
			//sprintf(templn, "X-PS3-Info: %llu [%s]\r\n", (unsigned long long)c_len, param); strcat(header, templn);

			if(is_binary==1) //file
			{
				sprintf(templn, "Content-Length: %llu\r\n\r\n", (unsigned long long)c_len); strcat(header, templn);
				ssend(conn_s, header);
				if(sys_memory_allocate(65536, SYS_MEMORY_PAGE_SIZE_64K, &sysmem)!=0)
				{
					sclose(&conn_s);
					loading_html--;
					sys_ppu_thread_exit(0);
				}
				char *buffer= (char*)sysmem;
				if(cellFsOpen(param, CELL_FS_O_RDONLY, &fd, NULL, 0) == CELL_FS_SUCCEEDED)
				{
					u64 read_e = 0, pos;
					cellFsLseek(fd, 0, CELL_FS_SEEK_SET, &pos);

					while(working)
					{
						//sys_timer_usleep(500);
						if(cellFsRead(fd, (void *)buffer, 65536, &read_e)==CELL_FS_SUCCEEDED)
						{
							if(read_e>0)
							{
								if(send(conn_s, buffer, (size_t)read_e, 0)<0) break;
							}
							else
								break;
						}
						else
							break;
					}
					cellFsClose(fd);
				}
				sys_memory_free(sysmem);
				sclose(&conn_s);
				loading_html--;
				sys_ppu_thread_exit(0);

			}
			if(strstr(param, "cpursx.ps3"))
			{
				if(sys_memory_allocate(65536, SYS_MEMORY_PAGE_SIZE_64K, &sysmem)!=0)
				{
					sclose(&conn_s);
					loading_html--;
					sys_ppu_thread_exit(0);
				}
			}
			else
			{
				if(sys_memory_allocate((BUFFER_SIZE), SYS_MEMORY_PAGE_SIZE_64K, &sysmem)!=0)
				{
					sclose(&conn_s);
					loading_html--;
					sys_ppu_thread_exit(0);
				}
			}

			char *buffer= (char*)sysmem;
			//else	// text page
			{

				if(is_binary!=2 && strstr(param, "setup.ps3?"))
				{
					memset(webman_config, 0, sizeof(WebmanCfg));

					if(strstr(param, "usb0"))  webman_config->usb0=1;
					if(strstr(param, "usb1"))  webman_config->usb1=1;
					if(strstr(param, "usb2"))  webman_config->usb2=1;
					if(strstr(param, "usb3"))  webman_config->usb3=1;
					if(strstr(param, "usb6"))  webman_config->usb6=1;
					if(strstr(param, "usb7"))  webman_config->usb7=1;
					if(strstr(param, "lastp")) webman_config->lastp=1;
					if(strstr(param, "autob")) webman_config->autob=1;
					if(strstr(param, "delay")) webman_config->delay=1;
					if(strstr(param, "b=5"))   webman_config->bootd=5;
					if(strstr(param, "b=9"))   webman_config->bootd=9;
					if(strstr(param, "s=3"))   webman_config->boots=3;
					if(strstr(param, "s=5"))   webman_config->boots=5;
					if(strstr(param, "s=10"))  webman_config->boots=10;
					if(strstr(param, "blind")) webman_config->blind=1;
					if(webman_config->blind)
						{system_call_8(837, (u64)(char*)"CELL_FS_IOS:BUILTIN_FLSH1", (u64)(char*)"CELL_FS_FAT", (u64)(char*)"/dev_blind", 0, 0, 0, 0, 0);}
						else
						{system_call_3(838, (u64)(char*)"/dev_blind", 0, 1);}


					if(strstr(param, "netd0"))  webman_config->netd0=1;
					if(strstr(param, "netd1"))  webman_config->netd1=1;

					if(strstr(param, "noset")) webman_config->noset=1;
					if(strstr(param, "nogrp") || !cobra_mode) webman_config->nogrp=1;


					webman_config->cmask=0;
					if(!strstr(param, "pst")) webman_config->cmask|=PS3;
					if(!strstr(param, "ps2")) webman_config->cmask|=PS2;
					if(!strstr(param, "ps1")) webman_config->cmask|=PS1;
					if(!strstr(param, "psp")) webman_config->cmask|=PSP;
					if(!strstr(param, "blu")) webman_config->cmask|=BLU;
					if(!strstr(param, "dvd")) webman_config->cmask|=DVD;

					if(strstr(param, "psl")) webman_config->pspl=1;

					webman_config->combo=0;
					if(!strstr(param, "failsaf")) webman_config->combo|=FAIL_SAFE;
					if(!strstr(param, "showtem")) webman_config->combo|=SHOW_TEMP;
					if(!strstr(param, "prevgam")) webman_config->combo|=PREV_GAME;
					if(!strstr(param, "nextgam")) webman_config->combo|=NEXT_GAME;
					if(!strstr(param, "shutdow")) webman_config->combo|=SHUT_DOWN;
					if(!strstr(param, "restart")) webman_config->combo|=RESTARTPS;
					if(!strstr(param, "unloadw")) webman_config->combo|=UNLOAD_WM;
					if(!strstr(param, "manualf")) webman_config->combo|=MANUALFAN;
					if(!strstr(param, "showidp")) webman_config->combo|=SHOW_IDPS;
					if(!strstr(param, "dsbcfws")) webman_config->combo|=DISABLESH;
					if(!strstr(param, "dsbcfan")) webman_config->combo|=DISABLEFC;
					if(!strstr(param, "mincfan")) webman_config->combo|=MINDYNFAN;


					if(strstr(param, "tid")) webman_config->tid=1;
					if(strstr(param, "poll")) webman_config->poll=1;
					if(strstr(param, "ftpd")) webman_config->ftpd=1;
					if(strstr(param, "nopad")) webman_config->nopad=1;
					if(strstr(param, "nocov")) webman_config->nocov=1;
					if(strstr(param, "nospf")) webman_config->nospoof=1;
                    if(!(c_firmware==4.46f || c_firmware==4.55f)) webman_config->nospoof=1;

					if(strstr(param, "fanc")) webman_config->fanc=1;

					webman_config->temp1=MY_TEMP;

					/*
					webman_config->fanm=3;

					if(strstr(param, "fanm=0")) webman_config->fanm=0;
					if(strstr(param, "fanm=1")) webman_config->fanm=1;
					if(strstr(param, "fanm=2")) webman_config->fanm=2;
					if(strstr(param, "fanm=3")) webman_config->fanm=3;
					*/

					webman_config->minfan=MIN_FANSPEED;
					if(strstr(param, "mfan="))
					{
						char *pos=strstr(param, "mfan=") + 5;
						char mytemp[3];
						for(u8 n=0;n<2;n++)
						{
							if(pos[n]!='&') {mytemp[n]=pos[n]; mytemp[n+1]=0;}
							else break;
						}
						webman_config->minfan=my_atoi(mytemp);
						if(webman_config->minfan<MIN_FANSPEED) webman_config->minfan=MIN_FANSPEED;
						if(webman_config->minfan>98) webman_config->minfan=99;
					}

					webman_config->bind=0;
					if(strstr(param, "bind")) webman_config->bind=1;

					webman_config->refr=0;
					if(strstr(param, "refr")) webman_config->refr=1;

					webman_config->manu=35;
					webman_config->temp0=0;

					if(strstr(param, "step="))
					{
						char *pos=strstr(param, "step=") + 5;
						char mytemp[3];
						for(u8 n=0;n<2;n++)
						{
							if(pos[n]!='&') {mytemp[n]=pos[n]; mytemp[n+1]=0;}
							else break;
						}
						webman_config->temp1=my_atoi(mytemp);
						if(webman_config->temp1<40) webman_config->temp1=40;
						if(webman_config->temp1>83) webman_config->temp1=83;
					}

					if(strstr(param, "fsp0="))
					{
						char *pos=strstr(param, "fsp0=") + 5;
						char mytemp[3];
						for(u8 n=0;n<2;n++)
						{
							if(pos[n]!='&') {mytemp[n]=pos[n]; mytemp[n+1]=0;}
							else break;
						}
						webman_config->ps2temp=my_atoi(mytemp);
						if(webman_config->ps2temp<20) webman_config->ps2temp=20;
						if(webman_config->ps2temp>99) webman_config->ps2temp=99;
					}

					if(strstr(param, "manu="))
					{
						char *pos=strstr(param, "manu=") + 5;
						char mytemp[3];
						for(u8 n=0;n<2;n++)
						{
							if(pos[n]!='&') {mytemp[n]=pos[n]; mytemp[n+1]=0;}
							else break;
						}
						webman_config->manu=my_atoi(mytemp);
					}
					if(webman_config->manu<20) webman_config->manu=20;

					if(strstr(param, "temp=1"))
						webman_config->temp0= (u8)(((float)webman_config->manu * 255.f)/100.f);
					else
						webman_config->temp0=0;

					max_temp=0;
					if(webman_config->fanc)
					{
						if(webman_config->temp0==0) max_temp=webman_config->temp1; else max_temp=0;
						fan_control(webman_config->temp0, 0);
					}
					else
						restore_fan(0);

					webman_config->warn=0;
					if(strstr(param, "warn=1")) webman_config->warn=1;

					webman_config->foot=0;
					if(strstr(param, "fp=1")) webman_config->foot=1;
					if(strstr(param, "fp=2")) webman_config->foot=2;

					if(strstr(param, "sidps"))  webman_config->sidps=1;
					if(strstr(param, "spsid"))  webman_config->spsid=1;
					if(strstr(param, "spp"))  webman_config->spp=1;
					if(strstr(param, "vIDPS1=")) {
						char *pos=strstr(param, "vIDPS1=") + 7;
						for(u8 n=0;n<17;n++) {
							if(pos[n]!='&') {webman_config->vIDPS1[n]=pos[n];webman_config->vIDPS1[n+1]=0;}
							else break;
						}
					}

					if(strstr(param, "vIDPS2=")) {
						char *pos=strstr(param, "vIDPS2=") + 7;
						for(u8 n=0;n<17;n++) {
							if(pos[n]!='&') {webman_config->vIDPS2[n]=pos[n];webman_config->vIDPS2[n+1]=0;}
							else break;
						}
					}
					if(strstr(param, "vPSID1=")) {
						char *pos=strstr(param, "vPSID1=") + 7;
						for(u8 n=0;n<17;n++) {
							if(pos[n]!='&') {webman_config->vPSID1[n]=pos[n];webman_config->vPSID1[n+1]=0;}
							else break;
						}
					}
					if(strstr(param, "vPSID2=")) {
						char *pos=strstr(param, "vPSID2=") + 7;
						for(u8 n=0;n<17;n++) {
							if(pos[n]!='&') {webman_config->vPSID2[n]=pos[n];webman_config->vPSID2[n+1]=0;}
							else break;
						}
					}

					webman_config->lang=0;

					// Europe
					if(strstr(param, "l=0"))   webman_config->lang=0;  // English
					if(strstr(param, "l=1"))   webman_config->lang=1;  // French
					if(strstr(param, "l=2"))   webman_config->lang=2;  // Italian
					if(strstr(param, "l=3"))   webman_config->lang=3;  // Spanish
					if(strstr(param, "l=4"))   webman_config->lang=4;  // German
					if(strstr(param, "l=5"))   webman_config->lang=5;  // Dutch
					if(strstr(param, "l=6"))   webman_config->lang=6;  // Portuguese
					if(strstr(param, "l=7"))   webman_config->lang=7;  // Russian
					if(strstr(param, "l=8"))   webman_config->lang=8;  // Hungarian
					if(strstr(param, "l=9"))   webman_config->lang=9;  // Polish
					if(strstr(param, "l=10"))  webman_config->lang=10; // Greek

					// Asia
					if(strstr(param, "l=11"))  webman_config->lang=11; // Indonesian
					if(strstr(param, "l=12"))  webman_config->lang=12; // Turkish
					if(strstr(param, "l=13"))  webman_config->lang=13; // Arabian
					if(strstr(param, "l=14"))  webman_config->lang=14; // Chinese
					if(strstr(param, "l=15"))  webman_config->lang=15; // Korean
					if(strstr(param, "l=16"))  webman_config->lang=16; // Japanese

					// Other
					if(strstr(param, "l=99"))  webman_config->lang=99;	// Unknown LANG_XX.TXT

					update_language();

					webman_config->neth0[0]=0;
					webman_config->neth1[0]=0;
#ifdef COBRA_ONLY
					//if(cobra_mode)
					{
						char *pos=strstr(param, "neth0=") + 6;
						char netp[7];
						if(strstr(param, "neth0="))
						{
							for(u8 n=0;n<16;n++)
							{
								if(pos[n]!='&') {webman_config->neth0[n]=pos[n];webman_config->neth0[n+1]=0;}
								else break;
							}

							pos=strstr(param, "netp0=") + 6;
							if(pos!=NULL)
							{
								for(u8 n=0;n<6;n++)
								{
									if(pos[n]!='&') {netp[n]=pos[n]; netp[n+1]=0;}
									else break;
								}
								webman_config->netp0=my_atoi(netp);
							}
						}
						if(strstr(param, "netp1="))
						{
							pos=strstr(param, "neth1=") + 6;
							for(u8 n=0;n<16;n++)
							{
								if(pos[n]!='&') {webman_config->neth1[n]=pos[n];webman_config->neth1[n+1]=0;}
								else break;
							}

							pos=strstr(param, "netp1=") + 6;
							if(pos!=NULL)
							{
								for(u8 n=0;n<6;n++)
								{
									if(pos[n]!='&') {netp[n]=pos[n]; netp[n+1]=0;}
									else break;
								}
								webman_config->netp1=my_atoi(netp);
							}
						}
					}
#endif
				}

				strcpy(buffer, "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\"><html xmlns=\"http://www.w3.org/1999/xhtml\"><meta http-equiv=\"Content-type\" content=\"text/html;charset=UTF-8\"><META HTTP-EQUIV=\"CACHE-CONTROL\" CONTENT=\"NO-CACHE\">");
				if(strstr(param, "cpursx.ps3"))
					strcat(buffer, "<meta http-equiv=\"refresh\" content=\"3\">");
				strcat(buffer, "<head><title>PS3 webMAN</title>");
				strcat(buffer, "<style type=\"text/css\">\r\n<!--\r\na:visited {color: #A0A0A0; text-decoration: none;}\r\na:hover {color: #C0C0C0; text-decoration: none;}\r\na:active {color: #F0F0F0; text-decoration: none;}\r\na:link {color: #909090;	text-decoration: none;}a.f:visited {color: #E0E0E0; text-decoration: none;}a.f:hover {color: #F0F0F0; text-decoration: none;}a.f:active {color: #F8F8F8; text-decoration: none;}a.f:link {color: #D0D0D0;	text-decoration: none;}\r\nbody,td,th {color: #f0f0f0;}\r\n.propfont {font-family: \"Courier New\", Courier, monospace;}");
				strcat(buffer, ".gc{float: left;overflow: hidden;position: relative;text-align: center;width: 280px;height: 260px;margin: 3px;border: 1px dashed grey;}");
				strcat(buffer, ".ic{position: absolute;top: 5px;right: 5px;left: 5px;bottom: 40px;}");

				if(is_ps3_http)
					strcat(buffer, ".gi{height: 210px;width: 267px;");
				else
					strcat(buffer, ".gi{position: absolute;max-height: 210px;max-width: 260px;");

				strcat(buffer, "position: absolute;bottom: 0px;top: 0px;left: 0px;right: 0px;margin: auto;}");

				strcat(buffer, ".gn{position: absolute;height: 38px;bottom: 0px;right: 7px;left: 7px;text-align: center;} --></style></head>");
				strcat(buffer, "<body bgcolor=\"#101010\"><font face=\"Courier New\"><b>");
				//if(strstr(param, "mount_ps3"))
				//	strcat(buffer, " onload=\"window.close();\"");
				//if(cobra_mode) strcat(buffer, "[Cobra] ");
				sprintf(templn, "PS3 webMAN " WM_VERSION " %s [<a href=\"/\">%s</a>] [<a href=\"/index.ps3\">%s</a>] [<a href=\"/setup.ps3\">%s</a>]</b>", STR_TRADBY, STR_FILES, STR_GAMES, STR_SETUP ); strcat(buffer, templn);
				u32 t1=0, t2=0, t1f=0, t2f=0;
				get_temperature(0, &t1); // 3E030000 -> 3E.03'C -> 62.(03/256)'C
				get_temperature(1, &t2);
				t1=t1>>24;
				t2=t2>>24;
				t1f=(1.8f*(float)t1+32.f);
				t2f=(1.8f*(float)t2+32.f);
				if(strstr(param, "cpursx.ps3"))
				{
					_meminfo meminfo;
					{system_call_1(352, (uint64_t) &meminfo);}

					if(!webman_config->fanc)
					{
						u8 st, mode, unknown;
						if(c_firmware==4.55f || c_firmware==4.60f)
						{
							backup[5]=peekq(0x8000000000009E38ULL);
							lv2poke32(0x8000000000009E38ULL, 0x38600001); // sys 409 get_fan_policy
							sys_sm_get_fan_policy(0, &st, &mode, &fan_speed, &unknown);
							pokeq(0x8000000000009E38ULL, backup[5]);
						}
						else
						{
							backup[5]=peekq(0x8000000000009E28ULL);
							lv2poke32(0x8000000000009E28ULL, 0x38600001); // sys 409 get_fan_policy
							sys_sm_get_fan_policy(0, &st, &mode, &fan_speed, &unknown);
							pokeq(0x8000000000009E28ULL, backup[5]);
						}
					}

					uint64_t eid0_idps[2], buffr[0x40], start_sector;
					uint32_t read;
					sys_device_handle_t source;
					if(sys_storage_open(0x100000000000004ULL, 0, &source, 0)!=0)
					{
						start_sector = 0x204;
						sys_storage_close(source);
						sys_storage_open(0x100000000000001ULL, 0, &source, 0);
					}
					else start_sector = 0x178;
					sys_storage_read(source, 0, start_sector, 1, buffr, &read, 0);
					sys_storage_close(source);

					eid0_idps[0]=buffr[0x0E];
					eid0_idps[1]=buffr[0x0F];

					if(!(c_firmware==4.55f || c_firmware==4.60f)) {
						{system_call_1(870, IDPS);}
						{system_call_1(872, PSID);}
					} else if(peekq(0x8000000000003000ULL)!=0xFFFFFFFF80010003ULL) {
						IDPS[0] = peekq(0x8000000000474F1CULL  );
						IDPS[1] = peekq(0x8000000000474F1CULL+8);
						PSID[0] = peekq(0x8000000000474F34ULL  );
						PSID[1] = peekq(0x8000000000474F34ULL+8);
					}

					sprintf(templn, "<hr><font size=42px><b>CPU: %i°C (MAX: %i°C)<br>RSX: %i°C<hr>CPU: %i°F (MAX: %i°F)<br>RSX: %i°F<hr>MEM: %iKB<hr>FAN SPEED: 0x%X (%i%%)<hr>PSID LV2 : %llX%llX<hr>IDPS EID0: %llX%llX<br>IDPS LV2 : %llX%llX</b></font><hr>", t1, max_temp, t2, t1f, (int)(1.8f*(float)max_temp+32.f), t2f, (meminfo.avail>>10), fan_speed, (int)((int)fan_speed*100)/255, PSID[0], PSID[1], eid0_idps[0], eid0_idps[1], IDPS[0], IDPS[1]);
					strcat(buffer, templn);



					//CellGcmConfig config; cellGcmGetConfiguration(&config);
					//sprintf(templn, "localAddr: %x", (u32) config.localAddress); strcat(buffer, templn);

					strcat(buffer, "</font></body></html>");
					sprintf(templn, "Content-Length: %i\r\n\r\n", strlen(buffer)); strcat(header, templn);
					ssend(conn_s, header);
					ssend(conn_s, buffer);
					buffer[0]=0;
					continue;
				}
				else
				{
					sprintf(templn, " [<a href=\"/cpursx.ps3\">CPU: %i°C | RSX: %i°C</a>]<hr/>", t1, t2);
//					sprintf(templn, " <a href=\"/cpursx.ps3\">CPU: %i°C | RSX: %i°C (CPU: %i°F | RSX: %i°F)</a><hr/>", t1, t2, t1f, t2f );
					//[<a href=\"/eject.ps3\">Eject</a>] [<a href=\"/insert.ps3\">Insert</a>] [<a href=\"/refresh.ps3\">Refresh</a>]
					strcat(buffer, templn);
					sprintf(templn, "<form action=\"\"> <input type=\"button\" value=\"%s\" onclick=\"window.location.href='/eject.ps3';\"> <input type=\"button\" value=\"%s\" onclick=\"window.location.href='/insert.ps3';\"> <input type=\"button\" value=\"%s\" onclick=\"window.location.href='/mount.ps3/unmount';\"> <input type=\"button\" value=\"%s XML\" onclick=\"window.location.href='/refresh.ps3';\">  <input type=\"button\" value=\"%s HTML\" onclick=\"window.location.href='/index.ps3?html';\"> <input type=\"button\" value=\"%s\" onclick=\"window.location.href='/shutdown.ps3';\">  <input type=\"button\" value=\"%s\" onclick=\"window.location.href='/restart.ps3';\"> </form><hr/>", STR_EJECT, STR_INSERT, STR_UNMOUNT, STR_REFRESH, STR_REFRESH, STR_SHUTDOWN, STR_RESTART); strcat(buffer, templn);
				}

				if(is_binary==2) // folder listing
				{
						u8 is_net = (param[1]=='n');

						if(is_net || cellFsOpendir( param, &fd) == CELL_FS_SUCCEEDED)
						{
							CellFsDirent entry;
							u64 read_e;
							unsigned long long sz=0;
							char sf[8];
							char fsize[128];
							char ename[8];
							char swap[1024];
							u16 idx=0;
							u32 tlen=0;
							sys_addr_t sysmem_html=0;
							if(sys_memory_allocate((BUFFER_SIZE), SYS_MEMORY_PAGE_SIZE_64K, &sysmem_html)!=0)
							{
								if(!is_net) cellFsClosedir(fd);
								sclose(&conn_s);
								if(sysmem) sys_memory_free(sysmem);
								loading_html--;
								sys_ppu_thread_exit(0);
								break;
							}

							typedef struct
							{
								char 	path[512];
							}
							t_line_entries;
							t_line_entries *line_entry	= (t_line_entries *)sysmem_html;
							u16 max_entries=((BUFFER_SIZE))/512;


							strcat(buffer, "<table class=\"propfont\"><tr><td>");

							strcpy(templn, param);
							while(strchr(templn+1, '/'))
                            {
								templn[strchr(templn+1, '/')-templn]=0;
								tlen+=strlen(templn)+1;

								strcpy(swap, param);
								swap[tlen]=0;

								strcat(buffer, "<a class=\"f\" href=\"");
								strcat(buffer, swap);

								sprintf(swap, "\">%s</a>/", templn);
								strcat(buffer, swap);
								strcpy(templn, param+tlen);
							}
							sprintf(swap, "%s:</td><td></td><td></td></tr>", templn);
							strcat(buffer, swap);
							tlen=0;

#ifdef COBRA_ONLY
							if(is_net)
							{
								int ns=-1;
								int abort_connection=0;
								if(param[4]=='0')
									ns=connect_to_server(webman_config->neth0, webman_config->netp0);
								if(param[4]=='1')
									ns=connect_to_server(webman_config->neth1, webman_config->netp1);
								if(ns>=0)
								{
									strcat(param, "/");
									if(open_remote_dir(ns, param+5, &abort_connection)>=0)
									{
										strcpy(templn, param); if(templn[strlen(templn)-1]=='/') templn[strlen(templn)-1]=0;
										if(strrchr(templn, '/')) templn[strrchr(templn, '/')-templn]=0; if(strlen(templn)<6 && strlen(param)<8) {templn[0]='/'; templn[1]=0;}
										sprintf(tempstr, "!00000<tr><td><a class=\"f\" href=\"%s\">..</a></td><td align=right>&nbsp; &lt;dir&gt; &nbsp;</td><td>11-Nov-2006 11:11</td></tr>", templn);
										strncpy(line_entry[idx].path, tempstr, 512);
										idx++;
										tlen+=strlen(tempstr);

										sys_addr_t data2=0;
										netiso_read_dir_result_data *data=NULL;
										int v3_entries=0;
										v3_entries = read_remote_dir(ns, &data2, &abort_connection);
										if(data2==NULL) goto just_leave;
										data=(netiso_read_dir_result_data*)data2;

										for(int n=0;n<v3_entries;n++)
										{
											if(data[n].name[0]=='.' && data[n].name[1]==0) continue;
											if(strlen(param)<2)
												sprintf(templn, "/%s", data[n].name);
											else
											{
												sprintf(templn, "%s%s", param, data[n].name);
												if(templn[strlen(templn)-1]=='/') templn[strlen(templn)-1]=0;
											}
											if(tlen>(BUFFER_SIZE-1024)) break;
											if(idx>=(max_entries-3)) break;

											cellRtcSetTime_t(&rDate, data[n].mtime);
											sz=(unsigned long long)data[n].file_size;
											if(sz<10240) sprintf(sf, "%s", STR_BYTE);
											else if(sz>=10240 && sz<2097152) {sprintf(sf, "%s", STR_KILOBYTE); sz>>=10;}
											else if(sz>=2097152 && sz<2147483648U) {sprintf(sf, "%s", STR_MEGABYTE); sz>>=20;}
											else if(sz>=2147483648U) {sprintf(sf, "%s", STR_GIGABYTE); sz>>=30;}
											if(data[n].is_directory)
												sprintf(fsize, "<a href=\"/mount.ps3%s\">&lt;dir&gt;</a>", templn);
											else
												sprintf(fsize, "%llu %s", sz, sf);
											snprintf(ename, 6, "%s    ", data[n].name);

											sprintf(tempstr, "%c%c%c%c%c%c<tr><td><a%shref=\"%s\">%s</a></td><td align=right>&nbsp; %s &nbsp;</td><td>%02i-%s-%04i %02i:%02i</td></tr>", //<td> %s%s%s%s%s%s%s%s%s</td>
											(data[n].is_directory) ? '0' : '1',
											ename[0], ename[1], ename[2], ename[3], ename[4],
											((data[n].is_directory) != 0) ? " class=\"f\" " : " ",
											templn, data[n].name,
											fsize,
											rDate.day, smonth[rDate.month-1], rDate.year,
											rDate.hour, rDate.minute);
											strncpy(line_entry[idx].path, tempstr, 512);
											idx++;
											tlen+=strlen(tempstr);

											if(!working) break;
										}
										sys_memory_free(data2);
									}
									else //may be a file
									{
										if(param[strlen(param)-1]=='/') param[strlen(param)-1]=0;
										int is_directory=0, bytes_read=0;
										int64_t file_size;
										u64 mtime, ctime, atime;
										if(remote_stat(ns, param+5, &is_directory, &file_size, &mtime, &ctime, &atime, &abort_connection)==0)
										{
											if(file_size && !is_directory)
											{
												if(open_remote_file_2(ns, param+5, &abort_connection)>0)
												{
													strcpy(tempstr, "application/octet-stream");
													if(strstr(param, ".JPG") || strstr(param, ".jpg") || strstr(param, ".JPEG") || strstr(param, ".jpeg") || strstr(param, ".STH"))
														strcpy(tempstr, "image/jpeg");
													else
													if(strstr(param, ".PNG") || strstr(param, ".png"))
														strcpy(tempstr, "image/png");
													else
													if(strstr(param, ".GIF") || strstr(param, ".gif"))
														strcpy(tempstr, "image/gif");
													else
													if(strstr(param, ".TXT") || strstr(param, ".txt") || strstr(param, ".LOG") || strstr(param, ".log") || strstr(param, ".INI") || strstr(param, ".ini"))
														strcpy(tempstr, "text/plain");
													else
													if(strstr(param, ".css"))
														strcpy(tempstr, "text/css");
													else
													if(strstr(param, ".htm"))
														strcpy(tempstr, "text/html");

													header[0]=0;
													sprintf(header, "HTTP/1.1 200 OK\r\nContent-Type: %s\r\nContent-Length: %llu\r\n\r\n", tempstr, (unsigned long long)file_size);
													ssend(conn_s, header);
													int boff=0;
													while(boff<file_size)
													{
														bytes_read = read_remote_file(ns, (char*)tempstr, boff, 3072, &abort_connection);
														if(bytes_read)
														{
															if(send(conn_s, tempstr, bytes_read, 0)<0) break;
														}
														boff+=bytes_read;
														if(bytes_read<3072 || boff>=file_size) break;
													}
													open_remote_file_2(ns, (char*)"/CLOSEFILE", &abort_connection);
													shutdown(ns, SHUT_RDWR); socketclose(ns);
													sclose(&conn_s);
													if(sysmem) sys_memory_free(sysmem);
													loading_html--;
													sys_ppu_thread_exit(0);
												}
											}
										}
									}
									shutdown(ns, SHUT_RDWR); socketclose(ns);
								}
							}
							else
#endif
							{
								while(cellFsReaddir(fd, &entry, &read_e) == 0 && read_e > 0)
								{
									if(entry.d_name[0]=='.' && entry.d_name[1]==0) continue;
									if(strlen(param)<2)
										sprintf(templn, "/%s", entry.d_name);
									else
									{
										sprintf(templn, "%s/%s", param, entry.d_name);
										if(templn[strlen(templn)-1]=='/') templn[strlen(templn)-1]=0;
									}
									if(tlen>(BUFFER_SIZE-1024)) break;
									if(idx>=(max_entries-3)) break;

									cellFsStat(templn, &buf);
									cellRtcSetTime_t(&rDate, buf.st_mtime);
									sz=(unsigned long long)buf.st_size;
									if(sz<10240) sprintf(sf, "%s", STR_BYTE);
									else if(sz>=10240 && sz<2097152) {sprintf(sf, "%s", STR_KILOBYTE); sz>>=10;}
									else if(sz>=2097152 && sz<2147483648U) {sprintf(sf, "%s", STR_MEGABYTE); sz>>=20;}
									else if(sz>=2147483648U) {sprintf(sf, "%s", STR_GIGABYTE); sz>>=30;}
									if((buf.st_mode & S_IFDIR) != 0)
										sprintf(fsize, "<a href=\"/mount.ps3%s\">&lt;dir&gt;</a>", templn);
									else
										sprintf(fsize, "%llu %s", sz, sf);
									snprintf(ename, 6, "%s    ", entry.d_name);

									sprintf(tempstr, "%c%c%c%c%c%c<tr><td><a%shref=\"%s\">%s</a></td><td align=right>&nbsp; %s &nbsp;</td><td>%02i-%s-%04i %02i:%02i</td></tr>", //<td> %s%s%s%s%s%s%s%s%s</td>
									((buf.st_mode & S_IFDIR) != 0) ? '0' : '1',
									ename[0], ename[1], ename[2], ename[3], ename[4],
									((buf.st_mode & S_IFDIR) != 0) ? " class=\"f\" " : " ",
									templn, entry.d_name,
									fsize,
									rDate.day, smonth[rDate.month-1], rDate.year,
									rDate.hour, rDate.minute);
									strncpy(line_entry[idx].path, tempstr, 512);
									idx++;
									tlen+=strlen(tempstr);

									if(!working) break;
								}
								cellFsClosedir(fd);
							}
just_leave:
							if(strlen(param)<4)
							{
								if(webman_config->netd0 && webman_config->neth0[0] && webman_config->netp0)
								{
									sprintf(tempstr, "0net0 <tr><td><a class=\"f\" href=\"/net0\">net0 (%s:%i)</a></td><td align=right>&nbsp; &lt;dir&gt; &nbsp;</td><td>11-Nov-2006 11:11</td></tr>", webman_config->neth0, webman_config->netp0);
									strncpy(line_entry[idx].path, tempstr, 512);
									idx++;
									tlen+=strlen(tempstr);
								}
								if(webman_config->netd1 && webman_config->neth1[0] && webman_config->netp1)
								{
									sprintf(tempstr, "0net1 <tr><td><a class=\"f\" href=\"/net1\">net1 (%s:%i)</a></td><td align=right>&nbsp; &lt;dir&gt; &nbsp;</td><td>11-Nov-2006 11:11</td></tr>", webman_config->neth1, webman_config->netp1);
									strncpy(line_entry[idx].path, tempstr, 512);
									idx++;
									tlen+=strlen(tempstr);
								}
							}

		//sclose(&data_s);
							if(idx)
							{
								for(u16 n=0; n<(idx-1); n++)
								{
									for(u16 m=(n+1); m<idx; m++)
									{
										if(strcasecmp(line_entry[n].path, line_entry[m].path)>0)
										{
											strcpy(swap, line_entry[n].path);
											strcpy(line_entry[n].path, line_entry[m].path);
											strcpy(line_entry[m].path, swap);
										}
									}
								}
							}

							for(u16 m=0;m<idx;m++)
							{
								strcat(buffer, (line_entry[m].path)+6);
							}

							if(sysmem_html) sys_memory_free(sysmem_html);
							strcat(buffer, "</table>");

							if(strlen(param)>6)
							{
								uint32_t blockSize;
								uint64_t freeSize;
								if(strchr(param+1, '/'))
									param[strchr(param+1, '/')-param]=0;
								cellFsGetFreeSize(param, &blockSize, &freeSize);
								sprintf(templn, "<hr><b><a href=\"%s\">%s</a>: %i %s</b><br>", param, param, (int)((blockSize*freeSize)>>20), STR_MBFREE);
								strcat(buffer, templn);
							}
							else
								strcat(buffer, "<hr>PS3 webMAN - Simple Web Server<br>");
						}

				}
				else
				{
					//reset_settings(webman_config);

					if(strstr(param, "refresh.ps3") && init_running==0)
					{
						init_running=1;
						sys_ppu_thread_t id3;
						sys_ppu_thread_create(&id3, handleclient, (u64)0xC0FEBAB0, -0x1d8, 0x20000, 0, "wwwd2");
						while(init_running && working) sys_timer_usleep(300000);
						sprintf(templn,  "<br>%s", STR_XMLRF); strcat(buffer, templn);
					}
					else
					if(strstr(param, "eject.ps3"))
					{
						eject_insert(1, 0);
						strcat(buffer, STR_EJECTED);
					}
					else
					if(strstr(param, "insert.ps3"))
					{
						eject_insert(0, 1);
						strcat(buffer, STR_LOADED);
					}
					else
					if(strstr(param, "setup.ps3?"))
					{
						int fdwm;
						u64 written;
						if(cellFsOpen(WMCONFIG, CELL_FS_O_CREAT|CELL_FS_O_WRONLY, &fdwm, NULL, 0) == CELL_FS_SUCCEEDED)
						{
							cellFsWrite(fdwm, (void *)wmconfig, sizeof(WebmanCfg), &written);
							cellFsClose(fdwm);
							sprintf(templn, "<br> %s", STR_SETTINGSUPD); strcat(buffer, templn);
						}
						else
							strcat(buffer, STR_ERROR);
					}
					else
					if(strstr(param, "setup.ps3"))
					{
						sprintf(templn, "<form action=\"/setup.ps3\" method=\"get\" enctype=\"application/x-www-form-urlencoded\" target=\"_self\"><font face=\"Courier New\"><table width=\"760\" border=\"0\" cellspacing=\"2\" cellpadding=\"0\"><tr><td width=\"220\"><u>%s:</u><br>", STR_SCAN1); strcat(buffer, templn);
						addcheckbox("u0", "usb0", drives[1], NULL, (webman_config->usb0), buffer);
						addcheckbox("u1", "usb1", drives[2], NULL, (webman_config->usb1), buffer);
						addcheckbox("u2", "usb2", drives[3], NULL, (webman_config->usb2), buffer);
						addcheckbox("u3", "usb3", drives[4], NULL, (webman_config->usb3), buffer);
						addcheckbox("u6", "usb6", drives[5], NULL, (webman_config->usb6), buffer);
						addcheckbox("u7", "usb7", drives[6], NULL, (webman_config->usb7), buffer);

						sprintf(templn, "<td width=\"520\"><u>%s:</u><br>", STR_SCAN2); strcat(buffer, templn);

						sprintf(templn, "<input name=\"p0\" type=\"checkbox\" value=\"pst\" %s/> PLAYSTATION\xC2\xAE\x33<br>", (!(webman_config->cmask & PS3))?(char*)"checked=\"checked\"" : (char*)" "); strcat(buffer, templn);
#ifdef COBRA_ONLY
						addcheckbox("p1", "ps2", "PLAYSTATION\xC2\xAE\x32"    , NULL     , !(webman_config->cmask & PS2), buffer);
						addcheckbox("p2", "ps1", "PLAYSTATION\xC2\xAE"        , NULL     , !(webman_config->cmask & PS1), buffer);
                        addcheckbox("p3", "psp", "PLAYSTATION\xC2\xAEPORTABLE", " ("     , !(webman_config->cmask & PSP), buffer);
                        addcheckbox("p6", "psl", STR_PSPL                     , ")<br>"  ,  (webman_config->pspl)       , buffer);
						addcheckbox("p4", "blu", "Blu-ray\xE2\x84\xA2"        , NULL     , !(webman_config->cmask & BLU), buffer);
						addcheckbox("p5", "dvd", "DVD "                       , STR_VIDLG, !(webman_config->cmask & DVD), buffer);
#endif

						strcat(buffer, "</td></tr></table><hr color=\"#0099FF\"/>");

#ifdef COBRA_ONLY
						addcheckbox("l", "lastp", STR_LPG    , NULL, (webman_config->lastp), buffer);
						addcheckbox("a", "autob", STR_AUTOB  , NULL, (webman_config->autob), buffer);
						addcheckbox("d", "delay", STR_DELAYAB, NULL, (webman_config->delay), buffer);
#endif
						addcheckbox("bl", "blind", STR_DEVBL   , NULL, (webman_config->blind), buffer);
						addcheckbox("rf", "refr",  STR_CONTSCAN, NULL, (webman_config->refr) , buffer);
						addcheckbox("pl", "poll",  STR_USBPOLL,  NULL, (webman_config->poll) , buffer);
						addcheckbox("ft", "ftpd",  STR_FTPSVC,   NULL, (webman_config->ftpd) , buffer);
						addcheckbox("np", "nopad", STR_COMBOS,   NULL, (webman_config->nopad), buffer);
						addcheckbox("nc", "nocov", STR_MMCOVERS, NULL, (webman_config->nocov), buffer);
						addcheckbox("ip", "bind",  STR_ACCESS,   NULL, (webman_config->bind) , buffer);
						addcheckbox("ws", "noset", STR_NOSETUP,  NULL, (webman_config->noset), buffer);

#ifdef COBRA_ONLY
						if(!(c_firmware==4.53f || c_firmware==4.60f))
       						addcheckbox("sp", "nospf", STR_NOSPOOF, NULL, (webman_config->nospoof), buffer);
#endif
						addcheckbox("gr", "nogrp", STR_NOGRP, NULL, (webman_config->nogrp), buffer);
						addcheckbox("ti", "tid", STR_TITLEID, NULL, (webman_config->tid), buffer);

						strcat(buffer, "<hr color=\"#0099FF\"/><table width=\"760\" border=\"0\" cellspacing=\"2\" cellpadding=\"0\"><tr class=\"propfont\"><td width=\"320\">");

						addcheckbox("fc", "fanc", STR_FANCTRL, " </td><td>", (webman_config->fanc), buffer);
						addcheckbox("warn", "1" , STR_NOWARN, " </td></tr>", (webman_config->warn), buffer);

						strcat(buffer, "<tr class=\"propfont\"><td>");
						addradiobutton("temp", "0", "t_0", STR_AUTOAT , " : ", (webman_config->temp0==0), buffer);
                        sprintf(templn, "<input name=\"step\" type=\"text\" value=\"%i\" size=\"3\" maxlength=\"2\" /> °C</td><td><input type=\"checkbox\" %s/> %s : <input name=\"mfan\" type=\"text\" value=\"%i\" size=\"3\" maxlength=\"2\" /> %% %s </td></tr>", webman_config->temp1, (webman_config->fanc && webman_config->temp0==0)?(char*)"checked=\"checked\"" : (char*)" ", STR_LOWEST, webman_config->minfan, STR_FANSPEED); strcat(buffer, templn);

						strcat(buffer, "<tr class=\"propfont\"><td>");
						addradiobutton("temp", "1", "t_1", STR_MANUAL , " : ", (webman_config->temp0!=0), buffer);
						sprintf(templn, "<input name=\"manu\" type=\"text\" value=\"%i\" size=\"3\" maxlength=\"2\" /> %% %s </td><td> %s : <input name=\"fsp0\" type=\"text\" value=\"%i\" size=\"3\" maxlength=\"2\" /> %% %s </td></tr></table>", (webman_config->manu), STR_FANSPEED, STR_PS2EMU, webman_config->ps2temp, STR_FANSPEED); strcat(buffer, templn);


#ifdef COBRA_ONLY
						strcat(buffer, "<hr color=\"#0099FF\"/>");
						addcheckbox("nd0", "netd0", STR_LANGAMES,  " &nbsp;&nbsp; PS3NETSRV#1 IP:", (webman_config->netd0), buffer);
						sprintf(templn, "<input name=\"neth0\" type=\"text\" value=\"%s\" size=\"20\" maxlength=\"15\" />:<input name=\"netp0\" type=\"text\" value=\"%i\" size=\"8\" maxlength=\"5\" /><br>", webman_config->neth0, webman_config->netp0); strcat(buffer, templn);
						addcheckbox("nd1", "netd1", STR_LANGAMES,  " &nbsp;&nbsp; PS3NETSRV#2 IP:", (webman_config->netd1), buffer);
						sprintf(templn, "<input name=\"neth1\" type=\"text\" value=\"%s\" size=\"20\" maxlength=\"15\" />:<input name=\"netp1\" type=\"text\" value=\"%i\" size=\"8\" maxlength=\"5\" />", webman_config->neth1, webman_config->netp1); strcat(buffer, templn);
#endif

						sprintf(templn, "<hr color=\"#0099FF\"/><u> %s:</u><br>", STR_ANYUSB); strcat(buffer, templn);

						addradiobutton("b", "0", "b_0", "0 sec" , NULL, (webman_config->bootd==0), buffer);
						addradiobutton("b", "5", "b_1", "5 sec" , NULL, (webman_config->bootd==5), buffer);
						addradiobutton("b", "9", "b_2", "10 sec", NULL, (webman_config->bootd==9), buffer);

						sprintf(templn, "<hr color=\"#0099FF\"/><u> %s:</u><br>", STR_ADDUSB); strcat(buffer, templn);

						addradiobutton("s", "0",  "s_0", "0 sec" , NULL, (webman_config->boots==0), buffer);
						addradiobutton("s", "3",  "s_1", "3 sec" , NULL, (webman_config->boots==3), buffer);
						addradiobutton("s", "5",  "s_2", "5 sec",  NULL, (webman_config->boots==5), buffer);
						addradiobutton("s", "10", "s_3", "10 sec", NULL, (webman_config->boots==10), buffer);

						sprintf(templn, "<hr color=\"#0099FF\"/><u> %s:</u><br>", STR_SPOOFID); strcat(buffer, templn);

						addcheckbox("id1", "sidps", "IDPS", ":", (webman_config->sidps), buffer);
						sprintf(templn, "<input name=\"vIDPS1\" type=\"text\" value=\"%s\" size=\"22\" maxlength=\"16\" />", webman_config->vIDPS1); strcat(buffer, templn);
						sprintf(templn, "<input name=\"vIDPS2\" type=\"text\" value=\"%s\" size=\"22\" maxlength=\"16\" /> <br>", webman_config->vIDPS2); strcat(buffer, templn);

						addcheckbox("id2", "spsid", "PSID", ":", (webman_config->spsid), buffer);
						sprintf(templn, "<input name=\"vPSID1\" type=\"text\" value=\"%s\" size=\"22\" maxlength=\"16\" />", webman_config->vPSID1); strcat(buffer, templn);
						sprintf(templn, "<input name=\"vPSID2\" type=\"text\" value=\"%s\" size=\"22\" maxlength=\"16\" /><br><br>", webman_config->vPSID2); strcat(buffer, templn);

						addcheckbox("pp", "spp", STR_DELCFWSYS, NULL, (webman_config->spp), buffer);

						sprintf(templn, "<hr color=\"#0099FF\"/> %s: ", STR_MEMUSAGE); strcat(buffer, templn);

						addradiobutton("fp", "0", "fo_0", "Standard (896KB)", ",  ", (webman_config->foot==0), buffer);
						addradiobutton("fp", "1", "fo_1", "Min (320KB)"     , ",  ", (webman_config->foot==1), buffer);
						addradiobutton("fp", "2", "fo_2", "Max (1280KB)"    , NULL , (webman_config->foot==2), buffer);

						sprintf(templn, "<br> %s: <select name=\"l\">", STR_PLANG); strcat(buffer, templn);

						addoptionitem("0" , "English"                                                 , (webman_config->lang==0) , buffer);
						addoptionitem("1" , "Fran\xC3\xA7\x61is"                                      , (webman_config->lang==1) , buffer);
						addoptionitem("2" , "Italiano"                                                , (webman_config->lang==2) , buffer);
						addoptionitem("3" , "Espa\xC3\xB1ol"                                          , (webman_config->lang==3) , buffer);
						addoptionitem("4" , "Deutsch"                                                 , (webman_config->lang==4) , buffer);
						addoptionitem("5" , "Nederlands"                                              , (webman_config->lang==5) , buffer);
						addoptionitem("6" , "Portugu\xC3\xAAs"                                        , (webman_config->lang==6) , buffer);
						addoptionitem("7" , "\xD0\xA0\xD1\x83\xD1\x81\xD1\x81\xD0\xBA\xD0\xB8\xD0\xB9", (webman_config->lang==7) , buffer);
						addoptionitem("8" , "Magyar"                                                  , (webman_config->lang==8) , buffer);
						addoptionitem("9" , "Polski"                                                  , (webman_config->lang==9) , buffer);

						addoptionitem("10", "\xCE\x95\xCE\xBB\xCE\xBB\xCE\xB7\xCE\xBD\xCE\xB9\xCE\xBA\xCF\x8E\xCE\xBD", (webman_config->lang==10), buffer);
						addoptionitem("11", "Indonesian"															  , (webman_config->lang==11), buffer);
						addoptionitem("12", "T\xC3\xBCrk\xC3\xA7\x65"												  , (webman_config->lang==12), buffer);
						addoptionitem("13", "\xD8\xA7\xD9\x84\xD8\xB9\xD8\xB1\xD8\xA8\xD9\x8A\xD8\xA9"				  , (webman_config->lang==13), buffer);
						addoptionitem("14", "\xE4\xB8\xAD\xE6\x96\x87"												  , (webman_config->lang==14), buffer);
						addoptionitem("15", "\xED\x95\x9C\xEA\xB5\xAD\xEC\x96\xB4"									  , (webman_config->lang==15), buffer);
						addoptionitem("16", "\xE6\x97\xA5\xE6\x9C\xAC\xE8\xAA\x9E"									  , (webman_config->lang==16), buffer);
						addoptionitem("99", "Unknown"																  , (webman_config->lang==99), buffer);

						sprintf(templn, "</select><hr color=\"#0099FF\"/><b><u> %s :</u></b><br><table width=\"760\" border=\"0\" cellspacing=\"2\" cellpadding=\"0\"><tr><td nowrap>", STR_COMBOS2); strcat(buffer, templn);

						addcheckbox("c0" , "failsaf", STR_FAILSAFE,   " : <b>SELECT+L3+L2+R2</b><br>"   , !(webman_config->combo & FAIL_SAFE), buffer);
						addcheckbox("c1" , "showtem", STR_SHOWTEMP,   " : <b>SELECT+R3</b><br>"         , !(webman_config->combo & SHOW_TEMP), buffer);
						addcheckbox("c2" , "prevgam", STR_PREVGAME,   " : <b>SELECT+L1</b><br>"         , !(webman_config->combo & PREV_GAME), buffer);
						addcheckbox("c3" , "nextgam", STR_NEXTGAME,   " : <b>SELECT+R1</b><br>"         , !(webman_config->combo & NEXT_GAME), buffer);
						addcheckbox("c4" , "dsbcfan", STR_FANCTRL4,   " : <b>L3+R2+START</b><br>"       , !(webman_config->combo & DISABLEFC), buffer);
						addcheckbox("c5" , "showidp", STR_SHOWIDPS,   " : <b>R2+O</b></td><td>"         , !(webman_config->combo & SHOW_IDPS), buffer);
						addcheckbox("c6" , "shutdow", STR_SHUTDOWN2,  " : <b>L3+R2+X</b><br>"           , !(webman_config->combo & SHUT_DOWN), buffer);
						addcheckbox("c7" , "restart", STR_RESTART2,   " : <b>L3+R2+O</b><br>"           , !(webman_config->combo & RESTARTPS), buffer);
						addcheckbox("c8" , "unloadw", STR_UNLOADWM,   " : <b>L3+R2+R3</b><br>"          , !(webman_config->combo & UNLOAD_WM), buffer);
						addcheckbox("c9" , "manualf", STR_FANCTRL2,   " : <b>SELECT+"                   , !(webman_config->combo & MANUALFAN), buffer); sprintf(templn, "%s</b><br>", STR_UPDN); strcat(buffer, templn);
						addcheckbox("c10", "mincfan", STR_FANCTRL5,   " : <b>SELECT+"                   , !(webman_config->combo & MINDYNFAN), buffer); sprintf(templn, "%s</b><br>", STR_LFRG); strcat(buffer, templn);
						addcheckbox("c11", "dsbcfws", STR_DELCFWSYS2, " : <b>R2+&#8710;</b></td></tr></table>", !(webman_config->combo & DISABLESH), buffer);

						sprintf(templn, "<hr color=\"#FF0000\"/><input name=\"save\" type=\"submit\" value=\" %s \"/></form>", STR_SAVE); strcat(buffer, templn);
						strcat(buffer, "<hr color=\"#FF0000\"/><a href=\"http://www.deanbg.com/prepNTFS.pkg\">prepNTFS - Prepare NTFS drives for webMAN access</a><br>");
#ifndef COBRA_ONLY
						strcat(buffer, "<a href=\"http://www.deanbg.com/webftp_server_noncobra.sprx\">webMAN (LATEST for non-COBRA7 CFW) - Latest version of webMAN</a><br>");
#else
						strcat(buffer, "<a href=\"http://www.deanbg.com/webftp_server.sprx\">webMAN (LATEST for COBRA7 CFW) - Latest version of webMAN</a><br>");
#endif
						strcat(buffer, "<a href=\"http://www.deanbg.com/webftp_server_ccapi.sprx\">webMAN (CCAPI) - ControlConsoleAPI compatible version of webMAN</a><br><hr color=\"#0099FF\"/>");

/*
	//vshmain_is_ss_enabled	= (void*)((int)getNIDfunc("vshmain", 0x981D7E9F));
	BgmPlaybackEnable		= (void*)((int)getNIDfunc("vshmain", 0xEDAB5E5E));
	if(BgmPlaybackEnable)
	{
		//set_SSHT_ = (uint32_t*)&opd;
		//memcpy(set_SSHT_, vshmain_is_ss_enabled, 8);
		//opd[0] -= 0x2C; // Sub before vshmain_981D7E9F sets Screenshot Flag
		//set_SSHT_(1);	// enable screenshot

		int * arg2;
		BgmPlaybackEnable(0, &arg2);
		show_msg((char*)"BGM enabled");
	}


						#define VSH_GCM_OBJ			0x70A8A8 // 4.53cex
						//#define VSH_GCM_OBJ		0x71A5F8 // 4.46dex

						uint32_t *gcm_obj0 = VSH_GCM_OBJ + ((uint32_t) 0 << 4);
						uint32_t *gcm_obj1 = VSH_GCM_OBJ + ((uint32_t) 1 << 4); // offset, pitch, width, height


						_cellGcmIoOffsetToAddress = (void*)((int)getNIDfunc("sdk", 0x2a6fba9c));

						void *buf_adr[2];

						if(_cellGcmIoOffsetToAddress)
						{
							_cellGcmIoOffsetToAddress(gcm_obj0[0], &buf_adr[0]);
							_cellGcmIoOffsetToAddress(gcm_obj1[0], &buf_adr[1]); //0x37ee5ac
						}

						sprintf(templn, "OFFSET#0: %x, P: %i, W: %i, H: %i, E: %x <br>",
							gcm_obj0[0], gcm_obj0[1], gcm_obj0[2], gcm_obj0[3], buf_adr[0]); strcat(buffer, templn);

						sprintf(templn, "OFFSET#1: %x, P: %i, W: %i, H: %i, E: %x <br>",
							gcm_obj1[0], gcm_obj1[1], gcm_obj1[2], gcm_obj1[3], buf_adr[1]); strcat(buffer, templn);
*/
					}
					else
					if(strstr(param, "mount.ps3") || strstr(param, "mount_ps3"))
					{
						//mount game
						if(strstr(param, "ps3/unmount"))
						{
							cellFsUnlink((char*)WMTMP "/last_game.txt");
#ifdef COBRA_ONLY
							//if(cobra_mode)
							{
								do_umount_iso();
								sys_timer_usleep(20000);

								{sys_map_path((char*)"/dev_bdvd", NULL);}
								{sys_map_path((char*)"/app_home", (char*)"/dev_hdd0/packages");}

								{
									sys_ppu_thread_t t;
									uint64_t exit_code;
									sys_ppu_thread_create(&t, netiso_stop_thread, 0, 0, 0x2000, SYS_PPU_THREAD_CREATE_JOINABLE, STOP_THREAD_NAME);
									sys_ppu_thread_join(t, &exit_code);

									sys_ppu_thread_create(&t, rawseciso_stop_thread, 0, 0, 0x2000, SYS_PPU_THREAD_CREATE_JOINABLE, STOP_THREAD_NAME);
									sys_ppu_thread_join(t, &exit_code);
								}

								while(netiso_loaded || rawseciso_loaded) {sys_timer_usleep(100000);}
							}
#else
							{
								pokeq(0x8000000000000000ULL+MAP_ADDR, 0x0000000000000000ULL);
								pokeq(0x8000000000000008ULL+MAP_ADDR, 0x0000000000000000ULL);
								//eject_insert(1, 1);

								if(cellFsStat((char*)"/dev_flash/pkg", &buf)==CELL_FS_SUCCEEDED)
									mount_with_mm((char*)"/dev_flash/pkg", 0);
							}
#endif
							strcat(buffer, STR_GAMEUM);
							if(strstr(param, "mount_ps3"))
							{
								//sys_timer_sleep(1);
								strcat(buffer, "<script type=\"text/javascript\">window.close(this);</script>");
							}
						}
						else
						{
							if(strstr(param, "?random="))
								param[strchr(param, '?')-param]=0;

							if(strstr(param, "mount.ps3"))
							{
								sprintf(tempstr, "%s/PS3_GAME/ICON0.PNG", param+10);
								if(cellFsStat(tempstr, &buf)!=CELL_FS_SUCCEEDED)
								{
									sprintf(tempstr, WMTMP "/%s", (strrchr(param, '/')+1));
									if(tempstr[strlen(tempstr)-2]=='.') tempstr[strlen(tempstr)-2]=0;
									if(tempstr[strlen(tempstr)-4]=='.') tempstr[strlen(tempstr)-4]=0;
									strcat(tempstr, ".PNG");
								}

								if(cellFsStat(tempstr, &buf)!=CELL_FS_SUCCEEDED)
								{
									sprintf(tempstr, "%s", param+10);
									if(tempstr[strlen(tempstr)-2]=='.') tempstr[strlen(tempstr)-2]=0;
									if(tempstr[strlen(tempstr)-4]=='.') tempstr[strlen(tempstr)-4]=0;
									strcat(tempstr, ".png");
									if(cellFsStat(tempstr, &buf)!=CELL_FS_SUCCEEDED)
									{
										tempstr[strlen(tempstr)-3]='j';
										tempstr[strlen(tempstr)-2]='p';
									}
									get_default_icon(tempstr, param, param);
								}

								if(strstr(param, "/PSPISO") || strstr(param, "/ISO"))
									sprintf(templn, "%s: %s<hr/><img src=\"%s\"><hr/>%s", STR_GAMETOM, param+10, tempstr, STR_PSPLOADED);
								else if(strstr(param, "/BDISO") || strstr(param, "/DVDISO") || strstr(param, ".ntfs[BDISO]") || strstr(param, ".ntfs[DVDISO]"))
									sprintf(templn, "%s: %s<hr/><img src=\"%s\"><hr/>%s" , STR_MOVIETOM,param+10, tempstr, STR_MOVIELOADED);
								else
									sprintf(templn, "%s: %s<hr/><img src=\"%s\"><hr/>%s", STR_GAMETOM, param+10, tempstr, STR_GAMELOADED);
								strcat(buffer, templn);
							}

							if(strstr(param, "mount_ps3"))
							{
								strcat(buffer, "<script type=\"text/javascript\">window.close(this);</script>");
								strcat(buffer, "</font></body></html>");

								sprintf(templn, "Content-Length: %i\r\n\r\n", strlen(buffer)); strcat(header, templn);
								ssend(conn_s, header);
								ssend(conn_s, buffer);
								buffer[0]=0;
							}

							max_mapped=0;
							mount_with_mm(param+10, 1);
							is_binary=1;

							if(strstr(param, "mount_ps3")) continue;
						}
					}
					else
					{

						//if(cobra_mode) strcat(buffer, "[Cobra] ");
						//strcat(buffer, "PS3 Game List:<br>");
						strcat(buffer, " <br>");

						while(loading_games && working) sys_timer_usleep(20000);

						CellRtcTick pTick, pTick2;
						cellRtcGetCurrentTick(&pTick);
						int upd_time=0;

						if(cellFsStat((char*)WMTMP "/games.html", &buf)==CELL_FS_SUCCEEDED)
							upd_time=buf.st_mtime;

						CellRtcDateTime rDate;
						cellRtcSetTime_t(&rDate, upd_time);
						cellRtcGetTick(&rDate, &pTick2);

						//sprintf(templn, "[%ull %ull %i ]<br>", pTick2, pTick, (pTick.tick-pTick2.tick)/1000000);
						//strcat(buffer, templn);
						u32 buf_len=strlen(buffer);

						if(strstr(param, "/index.ps3?html") || ((pTick.tick-pTick2.tick)/1000000)>43200) cellFsUnlink((char*)WMTMP "/games.html");
						if(cellFsStat((char*)WMTMP "/games.html", &buf)==CELL_FS_SUCCEEDED)
						{
							upd_time=1;
							int fdu;
							if(cellFsOpen((char*)WMTMP "/games.html", CELL_FS_O_RDONLY, &fdu, 0, 0)==CELL_FS_SUCCEEDED)
							{
								cellFsRead(fdu, (char*)(buffer+buf_len), buf.st_size, NULL);
								cellFsClose(fdu);
							}
						}
						else upd_time=0;

						if(upd_time==0)
						{
							loading_games=1;

							int ns=-2;
#ifdef COBRA_ONLY
							u8 retries=0;
							int abort_connection=0;
#endif
							u8 is_net=0;

							char ename[8];
							char swap[1024];
							u16 idx=0;
							u32 tlen=0;
							sys_addr_t sysmem_html=0;
							if(sys_memory_allocate((BUFFER_SIZE), SYS_MEMORY_PAGE_SIZE_64K, &sysmem_html)!=0)
							{
								sclose(&conn_s);
								if(sysmem) sys_memory_free(sysmem);
								loading_games=0;
								loading_html--;
								sys_ppu_thread_exit(0);
								break;
							}

							typedef struct
							{
								char 	path[512];
							}
							t_line_entries;
							t_line_entries *line_entry	= (t_line_entries *)sysmem_html;
							u16 max_entries=((BUFFER_SIZE))/512;


							for(u8 f0=0; f0<10; f0++)
							{
								for(u8 f1=0; f1<11; f1++)
								{

									if(tlen>(BUFFER_SIZE-1024)) break;
									if(idx>=(max_entries-1)) break;

									cellRtcGetCurrentTick(&pTick);

									if(!cobra_mode && (f1>1 && f1<10)) break;
									if(f0==9 && f1>6) break; //ntfs
									if(f1==10 && f0!=0) break; //video
									if(f0==7 && (!webman_config->netd0 || f1>6 || !cobra_mode)) break;
									if(f0==8 && (!webman_config->netd1 || f1>6 || !cobra_mode)) break;
									if(f1==5 && f0!=0) continue; // PS2ISO only from /dev_hdd0 supported

									if( (webman_config->cmask & PS3) && (f1< 3 || f1==10)) continue;
									if( (webman_config->cmask & BLU) && f1==3) continue;
									if( (webman_config->cmask & DVD) && f1==4) continue;
									if( (webman_config->cmask & PS2) && f1==5) continue;
									if( (webman_config->cmask & PS1) && f1==6) continue;
									if( (webman_config->cmask & PS1) && f1==7) continue;
									if( (webman_config->cmask & PSP) && f1==8) continue;
									if( (webman_config->cmask & PSP) && f1==9) continue;

									if(f0==7 || f0==8) is_net=1; else is_net=0;
#ifdef COBRA_ONLY
									if(ns==-2 && is_net &&
										( (f0==7 && webman_config->netp0 && webman_config->neth0[0])
										|| (f0==8 && webman_config->netp1 && webman_config->neth1[0]) )
										)
									{
		reconnect2:
										if(f0==7)
											ns=connect_to_server(webman_config->neth0, webman_config->netp0);
										else
											ns=connect_to_server(webman_config->neth1, webman_config->netp1);

										if(ns<0)
										{
											if(retries<5)
											{
												retries++;
												sys_timer_sleep(1);
												goto reconnect2;
											}
										}
									}
#endif
									if(ns<0 && is_net) break;

									if(is_net)
										sprintf(param, "/%s", paths[f1]);
									else
									{
										if(f0==9)//ntfs
											strcpy(param, WMTMP);
										else
											sprintf(param, "%s/%s", drives[f0], paths[f1]);
									}

									if(f0==9 && !webman_config->usb0 && !webman_config->usb1 && !webman_config->usb2 &&
										!webman_config->usb3 && !webman_config->usb6 && !webman_config->usb7) continue;

									if(!webman_config->usb0 && strstr(param, "/dev_usb000")) continue;
									if(!webman_config->usb1 && strstr(param, "/dev_usb001")) continue;
									if(!webman_config->usb2 && strstr(param, "/dev_usb002")) continue;
									if(!webman_config->usb3 && strstr(param, "/dev_usb003")) continue;
									if(!webman_config->usb6 && strstr(param, "/dev_usb006")) continue;
									if(!webman_config->usb7 && strstr(param, "/dev_usb007")) continue;
#ifdef COBRA_ONLY
									if(is_net && open_remote_dir(ns, param, &abort_connection) < 0) continue;
#endif
									CellFsDirent entry;
									u64 read_e;
									int fdw;
									uint64_t msiz = 0;
									u8 is_iso=0;
									char icon[512];
									char tempID[16];
									unsigned char *mem = (u8*)tempstr;
									sys_addr_t data2=0;
									int v3_entries=0;
									int v3_entry=0;
									int is_directory=0;
									int64_t file_size;
									u64 mtime, ctime, atime;
									int bytes_read=0;
#ifdef COBRA_ONLY
									netiso_read_dir_result_data *data=NULL;
									if(is_net)
									{
										v3_entries = read_remote_dir(ns, &data2, &abort_connection);
										if(data2==NULL) continue;
										data=(netiso_read_dir_result_data*)data2;
									}
#endif
									if(!is_net && cellFsOpendir( param, &fd) != CELL_FS_SUCCEEDED) continue;

									while((!is_net && cellFsReaddir(fd, &entry, &read_e) == 0 && read_e > 0)
										|| (is_net && v3_entry<v3_entries)
										)
									{
#ifdef COBRA_ONLY
										if(is_net)
										{
											icon[0]=0;

											if(!data[v3_entry].is_directory)
											{
												if(!strstr(data[v3_entry].name, ".ISO") && !strstr(data[v3_entry].name, ".iso") && !strstr(data[v3_entry].name, ".BIN") && !strstr(data[v3_entry].name, ".bin")) {v3_entry++; continue;}
												if(data[v3_entry].name[strlen(data[v3_entry].name)-4]!='.') {v3_entry++; continue;}
											}
											else
												if(!strstr(param, "/GAME")) {v3_entry++; continue;}

											if((f1< 3 || f1==10))//PS3 games only
											{

												sprintf(templn, WMTMP "/%s.SFO", data[v3_entry].name);
												if(!data[v3_entry].is_directory)
												{
													templn[strlen(templn)-7]='S';
													templn[strlen(templn)-6]='F';
													templn[strlen(templn)-5]='O';
													templn[strlen(templn)-4]=0;
												}

												if(data[v3_entry].is_directory && cellFsStat(templn, &buf)!=CELL_FS_SUCCEEDED)
												{
													sprintf(templn, "%s/%s/PS3_GAME/PARAM.SFO", param, data[v3_entry].name);
													if(remote_stat(ns, templn, &is_directory, &file_size, &mtime, &ctime, &atime, &abort_connection)!=0) {v3_entry++; continue;}
													sprintf(templn, WMTMP "/%s.SFO", data[v3_entry].name);

													if(cellFsOpen(templn, CELL_FS_O_CREAT|CELL_FS_O_RDWR|CELL_FS_O_TRUNC, &fdw, NULL, 0)==CELL_FS_SUCCEEDED)
													{
														sprintf(templn, "%s/%s/PS3_GAME/PARAM.SFO", param, data[v3_entry].name);
														open_remote_file_2(ns, templn, &abort_connection);
														int boff=0;
														while(1)
														{
															bytes_read = read_remote_file(ns, (char*)tempstr, boff, 3072, &abort_connection);
															if(bytes_read>0)
															{
															cellFsWrite(fdw, (char*)tempstr, bytes_read, NULL);
																boff+=bytes_read;
															}
															if(bytes_read<3072 || abort_connection) break;
														}
														open_remote_file_2(ns, (char*)"/CLOSEFILE", &bytes_read);
														cellFsClose(fdw);
													}
													sprintf(templn, WMTMP "/%s.SFO", data[v3_entry].name);
													cellFsChmod(templn, 0666);
												}

												if(cellFsOpen(templn, CELL_FS_O_RDONLY, &fdw, 0, 0)==CELL_FS_SUCCEEDED)
												{
													//cellFsOpen(templn, CELL_FS_O_RDONLY, &fdw, 0, 0);
													cellFsLseek(fdw, 0, CELL_FS_SEEK_SET, &msiz);
													cellFsRead(fdw, (void *)&tempstr, 4096, &msiz);
													cellFsClose(fdw);
													sprintf(templn, "%s", data[v3_entry].name);
													if(msiz>256)
													{
														parse_param_sfo(mem, tempID, templn);

														icon[0]=0;
														if(tempID[0] && !webman_config->nocov) get_icon(icon, tempID);
													}
												}
												else
													strcpy(templn, data[v3_entry].name);
											}
											else
												strcpy(templn, data[v3_entry].name);

											if(templn[strlen(templn)-4]=='.') templn[strlen(templn)-4]=0;

											if(icon[0]==0)
											{
												sprintf(icon, WMTMP "/%s.PNG", data[v3_entry].name);
												if(!data[v3_entry].is_directory)
												{
													icon[strlen(icon)-7]='P';
													icon[strlen(icon)-6]='N';
													icon[strlen(icon)-5]='G';
													icon[strlen(icon)-4]=0;
												} else goto go2;

												if(cellFsStat(icon, &buf)!=CELL_FS_SUCCEEDED)
												{
													icon[strlen(icon)-3]='J';
													icon[strlen(icon)-2]='P';
												} else goto c_load;

												if(cellFsStat(icon, &buf)!=CELL_FS_SUCCEEDED)
												{
													icon[strlen(icon)-3]='j';
													icon[strlen(icon)-2]='p';
													icon[strlen(icon)-1]='g';
												} else goto c_load;

												if(cellFsStat(icon, &buf)!=CELL_FS_SUCCEEDED)
												{
													icon[strlen(icon)-3]='p';
													icon[strlen(icon)-2]='n';
												} else goto c_load;
go2:
												if(cellFsStat(icon, &buf)!=CELL_FS_SUCCEEDED)
												{
													if(data[v3_entry].is_directory)
													{
														sprintf(tempstr, "%s/%s/PS3_GAME/ICON0.PNG", param, data[v3_entry].name);
														icon[strlen(icon)-3]='P';
														icon[strlen(icon)-2]='N';
														icon[strlen(icon)-1]='G';
													}
													else
													{
														sprintf(tempstr, "%s/%s", param, data[v3_entry].name);
														icon[strlen(icon)-3]='j';
														icon[strlen(icon)-2]='p';
														icon[strlen(icon)-1]='g';

														tempstr[strlen(tempstr)-3]='j';
														tempstr[strlen(tempstr)-2]='p';
														tempstr[strlen(tempstr)-1]='g';
													}

													abort_connection=0;
													if(remote_stat(ns, tempstr, &is_directory, &file_size, &mtime, &ctime, &atime, &abort_connection)!=0)
													{
														tempstr[strlen(tempstr)-3]='p';
														tempstr[strlen(tempstr)-2]='n';
														tempstr[strlen(tempstr)-1]='g';
														icon[strlen(icon)-3]='P';
														icon[strlen(icon)-2]='N';
														icon[strlen(icon)-1]='G';
														if(remote_stat(ns, tempstr, &is_directory, &file_size, &mtime, &ctime, &atime, &abort_connection)!=0)
														{
															icon[0]=0;
															goto c_load;
														}
													}

													if(file_size && open_remote_file_2(ns, tempstr, &abort_connection)>0 && !abort_connection)
													{
														if(cellFsOpen(icon, CELL_FS_O_CREAT|CELL_FS_O_RDWR|CELL_FS_O_TRUNC, &fdw, NULL, 0)==CELL_FS_SUCCEEDED)
														{
															int boff=0;
															while(boff<file_size)
															{
																bytes_read = read_remote_file(ns, (char*)tempstr, boff, 3072, &abort_connection);
																if(bytes_read)
																	cellFsWrite(fdw, (char*)tempstr, bytes_read, NULL);
																boff+=bytes_read;
																if(bytes_read<3072 || abort_connection || boff>=file_size) break;
															}
															cellFsClose(fdw);
															if(boff<1 || abort_connection) cellFsUnlink(icon);
															open_remote_file_2(ns, (char*)"/CLOSEFILE", &bytes_read);
														}
													}
												}
											}
c_load:
											get_default_icon(icon, param, data[v3_entry].name);

											snprintf(ename, 6, "%s    ", templn);
											sprintf(tempstr, "%c%c%c%c<div class=\"gc\"><div class=\"ic\"><a href=\"/mount.ps3/net%i%s/%s?random=%i\"><img src=\"%s\" class=\"gi\"></a></div><div class=\"gn\">%s</div></div>",
												ename[0], ename[1], ename[2], ename[3],
												(f0-7), param, data[v3_entry].name, (int)pTick.tick, icon, templn);

											v3_entry++;

											strncpy(line_entry[idx].path, tempstr, 512);
											idx++;
											tlen+=strlen(tempstr);
											if(tlen>(BUFFER_SIZE-1024)) break;
										}
										else
#endif
										{
											sprintf(templn, "%s/%s/PS3_GAME/PARAM.SFO", param, entry.d_name);

											is_iso=((strstr(param, "/PS3ISO") || strstr(param, "/PS2ISO") ||
													strstr(param, "/PSPISO") || strstr(param, "/ISO")||
													strstr(param, "/PSX") || strstr(param, "/DVDISO") || strstr(param, "/BDISO") ||
													strstr(param, WMTMP))
												&&
												(
												  (strstr(entry.d_name, ".ISO") || strstr(entry.d_name, ".iso") ||
													(strstr(param, "/PSX") &&
														(strstr(entry.d_name, ".CUE") || strstr(entry.d_name, ".cue"))
													)
												   )
												&&
												  ((entry.d_name[strlen(entry.d_name)-1]=='0' && entry.d_name[strlen(entry.d_name)-2]=='.') || entry.d_name[strlen(entry.d_name)-1]=='o' || entry.d_name[strlen(entry.d_name)-1]=='O' || entry.d_name[strlen(entry.d_name)-1]=='e' || entry.d_name[strlen(entry.d_name)-1]=='E')
												)) || strstr(entry.d_name, ".ntfs[");

											if(is_iso || cellFsStat(templn, &buf)==CELL_FS_SUCCEEDED)
											{

												icon[0]=0;
												tempID[0]=0;

												if(is_iso)
												{
													sprintf(templn, "%s", entry.d_name);
													if((strstr(param, "/PS3ISO") && f0<9) || (f0==9 && f1==2 && strstr(entry.d_name, "ntfs[PS3ISO]")))
													{
														int fs=0;
														sprintf(templn, WMTMP "/%s", entry.d_name);

														if(strstr(templn, "ntfs[PS3ISO]")) templn[strlen(templn)-13]=0;

														if(templn[strlen(templn)-2]=='.') templn[strlen(templn)-2]=0;
														templn[strlen(templn)-3]='S';
														templn[strlen(templn)-2]='F';
														templn[strlen(templn)-1]='O';

														if(cellFsOpen(templn, CELL_FS_O_RDONLY, &fs, 0, 0)==CELL_FS_SUCCEEDED)
														{
															cellFsLseek(fs, 0, CELL_FS_SEEK_SET, &msiz);
															cellFsRead(fs, (void *)&tempstr, ((buf.st_size>4096)?4096:buf.st_size), &msiz);
															cellFsClose(fs);
															sprintf(templn, "%s", entry.d_name);
															if(strstr(templn, "ntfs[PS3ISO]")) templn[strlen(templn)-13]=0;
															if(templn[strlen(templn)-2]=='.') templn[strlen(templn)-2]=0;
															if(templn[strlen(templn)-4]=='.') templn[strlen(templn)-4]=0;
															if(msiz>256)
															{
																parse_param_sfo(mem, tempID, templn);
															}
														}
														else
														{
															sprintf(templn, "%s", entry.d_name);
															if(strstr(templn, "ntfs[PS3ISO]"))
																templn[strlen(templn)-17]=0;
															else
															{
																sprintf(icon, "%s/%s", param, entry.d_name);
																if (cellFsOpen(icon, CELL_FS_O_RDONLY, &fs, NULL, 0) == CELL_FS_SUCCEEDED)
																{
																	if(cellFsLseek(fs, 0x810, CELL_FS_SEEK_SET, &msiz) == CELL_FS_SUCCEEDED)
																	{
																		if(cellFsRead(fs, (void *)&tempID, 11, &msiz) == CELL_FS_SUCCEEDED)
																		{
																			tempID[4]=tempID[5];
																			tempID[5]=tempID[6];
																			tempID[6]=tempID[7];
																			tempID[7]=tempID[8];
																			tempID[8]=tempID[9];
																			tempID[9]=0;
																		}
																	}
																	cellFsClose(fs);
																}
																if(templn[strlen(templn)-2]=='.') templn[strlen(templn)-2]=0;
																if(templn[strlen(templn)-4]=='.') templn[strlen(templn)-4]=0;
															}
														}
													}
													else
													{
														if(f0==9)
														{
															if(f1< 2 || f1>6) continue;
															if(f1==2 && !strstr(entry.d_name, "ntfs[PS3ISO]")) continue;
															if(f1==3 && !strstr(entry.d_name, "ntfs[BDISO]")) continue;
															if(f1==4 && !strstr(entry.d_name, "ntfs[DVDISO]")) continue;
															if(f1==6 && !strstr(entry.d_name, "ntfs[PSXISO]")) continue;

															if(f1==3 && strstr(entry.d_name, "ntfs[BDISO]")) templn[strlen(templn)-12]=0;
															if(f1==4 && strstr(entry.d_name, "ntfs[DVDISO]")) templn[strlen(templn)-13]=0;
															if(f1==6 && strstr(entry.d_name, "ntfs[PSXISO]")) templn[strlen(templn)-13]=0;
														}
													}
												}
												else
												{
													msiz=0;
													cellFsOpen(templn, CELL_FS_O_RDONLY, &fdw, 0, 0);
													cellFsLseek(fdw, 0, CELL_FS_SEEK_SET, &msiz);
													cellFsRead(fdw, (void *)&tempstr, ((buf.st_size>4096)?4096:buf.st_size), &msiz);
													cellFsClose(fdw);
													sprintf(templn, "%s", entry.d_name);
													if(msiz>256)
													{
														parse_param_sfo(mem, tempID, templn);
													}
												}

												if(templn[strlen(templn)-2]=='.') templn[strlen(templn)-2]=0;
												if(templn[strlen(templn)-4]=='.') templn[strlen(templn)-4]=0;

												if(!is_iso) sprintf(icon, "%s/%s/PS3_GAME/ICON0.PNG", param, entry.d_name);

												if(tempID[0])
												{
													icon[0]=0;
													if(!webman_config->nocov) get_icon(icon, tempID);

													if(!is_iso && icon[0]==0) sprintf(icon, "%s/%s/PS3_GAME/ICON0.PNG", param, entry.d_name);
												}

												if(icon[0]==0)
												{
													sprintf(icon, "%s/%s", param, entry.d_name);
													if(strstr(icon, "ntfs[PS3ISO]") || strstr(icon, "ntfs[DVDISO]") || strstr(icon, "ntfs[PSXISO]")) icon[strlen(icon)-13]=0;
													if(strstr(icon, "ntfs[BDISO]")) icon[strlen(icon)-12]=0;
													if(icon[strlen(icon)-4]=='.')
													{
														icon[strlen(icon)-3]='p'; icon[strlen(icon)-2]='n'; icon[strlen(icon)-1]='g';
														if(cellFsStat(icon, &buf)!=CELL_FS_SUCCEEDED)
														{
															icon[strlen(icon)-3]='P'; icon[strlen(icon)-2]='N'; icon[strlen(icon)-1]='G';
														}
													}
													else
													if(icon[strlen(icon)-2]=='.')
													{
														icon[strlen(icon)-5]='p'; icon[strlen(icon)-4]='n'; icon[strlen(icon)-3]='g';  icon[strlen(icon)-2]=0;
													}

													if(cellFsStat(icon, &buf)!=CELL_FS_SUCCEEDED)
														{icon[strlen(icon)-3]='j'; icon[strlen(icon)-2]='p'; icon[strlen(icon)-1]='g';}

													get_default_icon(icon, param, entry.d_name);
												}

												snprintf(ename, 6, "%s    ", templn);
												if(is_iso)
												{
													sprintf(tempstr, "%c%c%c%c<div class=\"gc\"><div class=\"ic\"><a href=\"/mount.ps3%s/%s\"><img src=\"%s\" class=\"gi\"></a></div><div class=\"gn\"><a href=\"%s\">%s</a></div></div>",
														ename[0], ename[1], ename[2], ename[3],
														param, entry.d_name, icon, param, templn);
												}
												else
												{
													sprintf(tempstr, "%c%c%c%c<div class=\"gc\"><div class=\"ic\"><a href=\"/mount.ps3%s/%s\"><img src=\"%s\" class=\"gi\"></a></div><div class=\"gn\"><a href=\"%s/%s\">%s</a></div></div>",
														ename[0], ename[1], ename[2], ename[3],
														param, entry.d_name, icon, param, entry.d_name, templn);
												}

												strncpy(line_entry[idx].path, tempstr, 512);
												idx++;
												tlen+=strlen(tempstr);
												if(tlen>(BUFFER_SIZE-1024)) break;

												cellRtcGetCurrentTick(&pTick);
											}
										}

										if(strlen(buffer)>(BUFFER_SIZE-1024)) break;
									}
									if(!is_net) cellFsClosedir(fd);
									if(is_net && data2!=NULL) sys_memory_free(data2);
								}
								if(is_net && ns>=0) {shutdown(ns, SHUT_RDWR); socketclose(ns); ns=-2;}
							}


							if(idx)
							{
								for(u16 n=0; n<(idx-1); n++)
								{
									for(u16 m=(n+1); m<idx; m++)
									{
										if(strcasecmp(line_entry[n].path, line_entry[m].path)>0)
										{
											strcpy(swap, line_entry[n].path);
											strcpy(line_entry[n].path, line_entry[m].path);
											strcpy(line_entry[m].path, swap);
										}
									}
								}
							}

							for(u16 m=0;m<idx;m++)
							{
								strcat(buffer, (line_entry[m].path)+4);
								if(strlen(buffer)>(BUFFER_SIZE-1024)) break;
							}
							if(sysmem_html) sys_memory_free(sysmem_html);
							loading_games=0;
							int fdu=0;
							if(cellFsOpen((char*)WMTMP "/games.html", CELL_FS_O_CREAT|CELL_FS_O_RDWR|CELL_FS_O_TRUNC, &fdu, NULL, 0)==CELL_FS_SUCCEEDED)
							{
								cellFsWrite(fdu, (char*)(buffer+buf_len), (strlen(buffer)-buf_len), NULL);
								cellFsClose(fdu);
							}
						}
					}
				}

				strcat(buffer, "</font></body></html>");

				sprintf(templn, "Content-Length: %i\r\n\r\n", strlen(buffer)); strcat(header, templn);
				ssend(conn_s, header);
				ssend(conn_s, buffer);
				buffer[0]=0;
			}
		}
		//else break;

		//if(served>125 || is_binary==1 || !working)
		break;

		//sys_timer_usleep(3200);
	}

#ifdef USE_DEBUG
	ssend(debug_s, "Request served.\r\n");
#endif
	sclose(&conn_s);
	if(sysmem) sys_memory_free(sysmem);
	loading_html--;
	sys_ppu_thread_exit(0);
}

static void handleclient_ftp(u64 conn_s_ftp_p)
{

	int conn_s_ftp = (int)conn_s_ftp_p; // main communications socket
	int data_s = -1;			// data socket
	int data_ls = -1;

	int connactive = 1;			// whether the ftp connection is active or not
	int dataactive = 0;			// prevent the data connection from being closed at the end of the loop
	int loggedin = 0;			// whether the user is logged in or not

	char rnfr[512];				// stores the path/to/file for the RNFR command

	char cwd[512];				// Current Working Directory
	int rest = 0;				// for resuming file transfers

	char buffer[2048];
	char filename[384];
	char tempcwd[512];
	char cmd[16], param[384];
	struct CellFsStat buf;
	int fd;

	int p1x = 0;
	int p2x = 0;

	CellRtcDateTime rDate;
	CellRtcTick pTick;

	sys_net_sockinfo_t conn_info;
	sys_net_get_sockinfo(conn_s_ftp, &conn_info, 1);

	char ip_address[16];
	char pasv_output[56];
	sprintf(ip_address, "%s", inet_ntoa(conn_info.local_adr));
	for(u8 n=0;n<strlen(ip_address);n++) if(ip_address[n]=='.') ip_address[n]=',';

	ssend(conn_s_ftp, "220-VSH ftpd\r\n");
	if(webman_config->bind && (conn_info.local_adr.s_addr!=conn_info.remote_adr.s_addr))
	{
		sprintf(buffer, "%i Access Denied. Use SETUP to allow remote connections.\r\n", 500); ssend(conn_s_ftp, buffer);
		sclose(&conn_s_ftp);
		sys_ppu_thread_exit(0);
	}

	sprintf(buffer, "%i webMAN ftpd " WM_VERSION "\r\n", 220); ssend(conn_s_ftp, buffer);

	strcpy(cwd, "/");

	while(connactive == 1 && working)
	{

		if(recv(conn_s_ftp, buffer, 2047, 0) > 0)
		{
			buffer[strcspn(buffer, "\n")] = '\0';
			buffer[strcspn(buffer, "\r")] = '\0';

			int split = ssplit(buffer, cmd, 15, param, 511);

			if(loggedin == 1)
			{
				if(strcasecmp(cmd, "CWD") == 0)
				{

					strcpy(tempcwd, cwd);

					if(split == 1)
					{
						absPath(tempcwd, param, cwd);
					}

					if(isDir(tempcwd))
					{
						strcpy(cwd, tempcwd);
						ssend(conn_s_ftp, "250 OK\r\n");
					}
					else
					{
						ssend(conn_s_ftp, "550 Error\r\n");
					}
				}
				else
				if(strcasecmp(cmd, "CDUP") == 0)
				{
					int pos = strlen(cwd) - 2;

					for(int i = pos; i > 0; i--)
					{
						if(i < pos && cwd[i] == '/')
						{
							break;
						}
						else
						{
							cwd[i] = '\0';
						}
					}
					ssend(conn_s_ftp, "250 OK\r\n");
				}
				else
				if(strcasecmp(cmd, "PWD") == 0)
				{
					sprintf(buffer, "257 \"%s\"\r\n", cwd);
					ssend(conn_s_ftp, buffer);
				}
				else
				if(strcasecmp(cmd, "TYPE") == 0)
				{
					ssend(conn_s_ftp, "200 TYPE OK\r\n");
					dataactive = 1;
				}
				else
				if(strcasecmp(cmd, "REST") == 0)
				{
					if(split == 1)
					{
						ssend(conn_s_ftp, "350 REST command successful\r\n");
						rest = my_atoi(param);
						dataactive = 1;
					}
					else
					{
						ssend(conn_s_ftp, "501 No restart point\r\n");
					}
				}
				else
				if(strcasecmp(cmd, "QUIT") == 0 || strcasecmp(cmd, "BYE") == 0)
				{
					ssend(conn_s_ftp, "221 BYE\r\n");
					connactive = 0;
				}
				else
				if(strcasecmp(cmd, "FEAT") == 0)
				{
					ssend(conn_s_ftp, "211-Ext:\r\n");
					ssend(conn_s_ftp, " SIZE\r\n");
					ssend(conn_s_ftp, " MDTM\r\n");
					ssend(conn_s_ftp, " PORT\r\n");
					ssend(conn_s_ftp, " CDUP\r\n");
					ssend(conn_s_ftp, " ABOR\r\n");
					ssend(conn_s_ftp, " REST STREAM\r\n");
					ssend(conn_s_ftp, " PASV\r\n");
					ssend(conn_s_ftp, " LIST\r\n");
					ssend(conn_s_ftp, " MLSD\r\n");
					ssend(conn_s_ftp, " MLST type*;size*;modify*;UNIX.mode*;UNIX.uid*;UNIX.gid*;\r\n");
					ssend(conn_s_ftp, "211 End\r\n");
				}
				else
				if(strcasecmp(cmd, "PORT") == 0)
				{
					rest = 0;

					if(split == 1)
					{
						char data[6][4];
						int i = 0;
						u8 k=0;

						for(u8 j=0;j<=strlen(param);j++)
						{
							if(param[j]!=',' && param[j]!=0) { data[i][k]=param[j]; k++; }
							else {data[i][k]=0; i++; k=0;}
							if(i>=6) break;
						}

						if(i == 6)
						{
							char ipaddr[16];
							sprintf(ipaddr, "%s.%s.%s.%s", data[0], data[1], data[2], data[3]);

							data_s=connect_to_server(ipaddr, getPort(my_atoi(data[4]), my_atoi(data[5])));

							if(data_s>=0)
							{
								ssend(conn_s_ftp, "200 OK\r\n");
								dataactive = 1;
							}
							else
							{
								ssend(conn_s_ftp, "451 Error\r\n");
							}
						}
						else
						{
							ssend(conn_s_ftp, "501 Error\r\n");
						}
					}
					else
					{
						ssend(conn_s_ftp, "501 Error\r\n");
					}
				}
				else
				if(strcasecmp(cmd, "SITE") == 0)
				{
					if(split == 1)
					{
						split = ssplit(param, cmd, 31, filename, 511);

						if(strcasecmp(cmd, "HELP") == 0)
						{
							ssend(conn_s_ftp, "214-CMDs:\r\n");
							ssend(conn_s_ftp, " SITE SHUTDOWN\r\n");
							ssend(conn_s_ftp, " SITE RESTART\r\n");
							ssend(conn_s_ftp, "214 End\r\n");

						}
						else
						if(strcasecmp(cmd, "SHUTDOWN") == 0)
						{
							ssend(conn_s_ftp, "221 OK\r\n");

							cellFsUnlink((char*)"/dev_hdd0/tmp/turnoff");
							{system_call_4(379,0x1100,0,0,0);}
							sys_ppu_thread_exit(0);
						}
						else
						if(strcasecmp(cmd, "RESTART") == 0)
						{
							ssend(conn_s_ftp, "221 OK\r\n");

							cellFsUnlink((char*)"/dev_hdd0/tmp/turnoff");
							{system_call_4(379,0x1200,0,0,0);}
							sys_ppu_thread_exit(0);
						}
						else
						{
							ssend(conn_s_ftp, "500 Error\r\n");
						}
					}
					else
					{
						ssend(conn_s_ftp, "501 Error\r\n");
					}
				}
				else
				if(strcasecmp(cmd, "NOOP") == 0)
				{
					ssend(conn_s_ftp, "200 NOOP\r\n");
				}
				else
				if(strcasecmp(cmd, "MLSD") == 0 || strcasecmp(cmd, "LIST") == 0 || strcasecmp(cmd, "MLST") == 0)
				{
					if(data_s > 0)
					{
						int nolist = (strcasecmp(cmd, "MLSD") == 0 || strcasecmp(cmd, "MLST") == 0);

						strcpy(tempcwd, cwd);

						if(split == 1)
						{
							absPath(tempcwd, param, cwd);
						}

						if(cellFsOpendir( (isDir(tempcwd) ? tempcwd : cwd), &fd) == CELL_FS_SUCCEEDED)
						{
							ssend(conn_s_ftp, "150 OK\r\n");

							CellFsDirent entry;
							u64 read_e;

							while(cellFsReaddir(fd, &entry, &read_e) == 0 && read_e > 0)
							{
								if(!strcmp(entry.d_name, "app_home") || !strcmp(entry.d_name, "host_root")) continue;

								absPath(filename, entry.d_name, cwd);

								cellFsStat(filename, &buf);
								cellRtcSetTime_t(&rDate, buf.st_mtime);
								if(nolist)
								{

									char dirtype[2];
									if(strcmp(entry.d_name, ".") == 0)
									{
										dirtype[0] = 'c';
									}
									else
									if(strcmp(entry.d_name, "..") == 0)
									{
										dirtype[0] = 'p';
									}
									else
									{
										dirtype[0] = '\0';
									}

									dirtype[1] = '\0';

									if(strcasecmp(cmd, "MLSD") == 0)
									sprintf(buffer, "type=%s%s;siz%s=%llu;modify=%04i%02i%02i%02i%02i%02i;UNIX.mode=0%i%i%i;UNIX.uid=root;UNIX.gid=root; %s\r\n",
										dirtype,
										((buf.st_mode & S_IFDIR) != 0) ? "dir" : "file",
										((buf.st_mode & S_IFDIR) != 0) ? "d" : "e", (unsigned long long)buf.st_size, rDate.year, rDate.month, rDate.day, rDate.hour, rDate.minute, rDate.second,
										(((buf.st_mode & S_IRUSR) != 0) * 4 + ((buf.st_mode & S_IWUSR) != 0) * 2 + ((buf.st_mode & S_IXUSR) != 0) * 1),
										(((buf.st_mode & S_IRGRP) != 0) * 4 + ((buf.st_mode & S_IWGRP) != 0) * 2 + ((buf.st_mode & S_IXGRP) != 0) * 1),
										(((buf.st_mode & S_IROTH) != 0) * 4 + ((buf.st_mode & S_IWOTH) != 0) * 2 + ((buf.st_mode & S_IXOTH) != 0) * 1),
										entry.d_name);
									else
										sprintf(buffer, " type=%s%s;siz%s=%llu;modify=%04i%02i%02i%02i%02i%02i;UNIX.mode=0%i%i%i;UNIX.uid=root;UNIX.gid=root; %s\r\n",
											dirtype,
											((buf.st_mode & S_IFDIR) != 0) ? "dir" : "file",
											((buf.st_mode & S_IFDIR) != 0) ? "d" : "e", (unsigned long long)buf.st_size, rDate.year, rDate.month, rDate.day, rDate.hour, rDate.minute, rDate.second,
											(((buf.st_mode & S_IRUSR) != 0) * 4 + ((buf.st_mode & S_IWUSR) != 0) * 2 + ((buf.st_mode & S_IXUSR) != 0) * 1),
											(((buf.st_mode & S_IRGRP) != 0) * 4 + ((buf.st_mode & S_IWGRP) != 0) * 2 + ((buf.st_mode & S_IXGRP) != 0) * 1),
											(((buf.st_mode & S_IROTH) != 0) * 4 + ((buf.st_mode & S_IWOTH) != 0) * 2 + ((buf.st_mode & S_IXOTH) != 0) * 1),
											entry.d_name);
								}
								else
									sprintf(buffer, "%s%s%s%s%s%s%s%s%s%s   1 root  root        %llu %s %02i %02i:%02i %s\r\n",
									((buf.st_mode & S_IFDIR) != 0) ? "d" : "-",
									((buf.st_mode & S_IRUSR) != 0) ? "r" : "-",
									((buf.st_mode & S_IWUSR) != 0) ? "w" : "-",
									((buf.st_mode & S_IXUSR) != 0) ? "x" : "-",
									((buf.st_mode & S_IRGRP) != 0) ? "r" : "-",
									((buf.st_mode & S_IWGRP) != 0) ? "w" : "-",
									((buf.st_mode & S_IXGRP) != 0) ? "x" : "-",
									((buf.st_mode & S_IROTH) != 0) ? "r" : "-",
									((buf.st_mode & S_IWOTH) != 0) ? "w" : "-",
									((buf.st_mode & S_IXOTH) != 0) ? "x" : "-",
									(unsigned long long)buf.st_size, smonth[rDate.month-1], rDate.day,
									rDate.hour, rDate.minute, entry.d_name);

								if(ssend(data_s, buffer)<0) break;
								sys_timer_usleep(1000);
							}

							cellFsClosedir(fd);
							if(strlen(tempcwd)>6)
							{
								uint32_t blockSize;
								uint64_t freeSize;
								char tempstr[128];
								if(strchr(tempcwd+1, '/'))
									tempcwd[strchr(tempcwd+1, '/')-tempcwd]=0;
								cellFsGetFreeSize(tempcwd, &blockSize, &freeSize);
								sprintf(tempstr, "226 [%s] [ %i %s ]\r\n", tempcwd, (int)((blockSize*freeSize)>>20), STR_MBFREE);
								ssend(conn_s_ftp, tempstr);
							}
							else
							{
								ssend(conn_s_ftp, "226 OK\r\n");
							}
						}
						else
						{
							ssend(conn_s_ftp, "550 Error\r\n");
						}
					}
					else
					{
						ssend(conn_s_ftp, "425 Error\r\n");
					}
				}
				else
				if(strcasecmp(cmd, "PASV") == 0)
				{
					u8 pasv_retry=0;
					rest = 0;
pasv_again:
					if(!p1x)
					{
						cellRtcGetCurrentTick(&pTick);
						p1x = ( ( (pTick.tick & 0xfe0000) >> 16) & 0xff) | 0x80; // use ports 32768 -> 65279 (0x8000 -> 0xFEFF)
						p2x = ( ( (pTick.tick & 0x00ff00) >>  8) & 0xff);
					}
					data_ls = slisten(getPort(p1x, p2x), 1);

					if(data_ls >= 0)
					{
						sprintf(pasv_output, "227 Entering Passive Mode (%s,%i,%i)\r\n", ip_address, p1x, p2x);
						ssend(conn_s_ftp, pasv_output);

						if((data_s = accept(data_ls, NULL, NULL)) > 0)
						{
							dataactive = 1;
						}
						else
						{
							ssend(conn_s_ftp, "451 Error\r\n");
						}

					}
					else
					{
						p1x=0;
						if(pasv_retry<10)
						{
							pasv_retry++;
							goto pasv_again;
						}
						ssend(conn_s_ftp, "451 Error\r\n");
					}
				}
				else
				if(strcasecmp(cmd, "RETR") == 0)
				{
					if(data_s > 0)
					{
						if(split == 1)
						{
							absPath(filename, param, cwd);

							//if(cellFsStat(filename, &buf)==CELL_FS_SUCCEEDED)
							{
								int rr=-4;

								if(cellFsOpen(filename, CELL_FS_O_RDONLY, &fd, NULL, 0) == CELL_FS_SUCCEEDED)
								{
									sys_addr_t sysmem=0;
									if(sys_memory_allocate(BUFFER_SIZE_FTP, SYS_MEMORY_PAGE_SIZE_64K, &sysmem)==0)
									{
										char *buffer2= (char*)sysmem;

										u64 read_e = 0, pos; //, write_e

										cellFsLseek(fd, rest, CELL_FS_SEEK_SET, &pos);
										rest = 0;
										//int optval = BUFFER_SIZE_FTP;
										//setsockopt(data_s, SOL_SOCKET, SO_SNDBUF, &optval, sizeof(optval));

										ssend(conn_s_ftp, "150 OK\r\n");
										rr=0;

										while(working)
										{
											//sys_timer_usleep(1668);
											if(cellFsRead(fd, (void *)buffer2, BUFFER_SIZE_FTP, &read_e)==CELL_FS_SUCCEEDED)
											{
												if(read_e>0)
												{
													if(send(data_s, buffer2, (size_t)read_e, 0)<0) {rr=-3; break;}
												}
												else
													break;
											}
											else
												{rr=-2;break;}
										}
										sys_memory_free(sysmem);
									}
									cellFsClose(fd);
								}

								if( rr == 0)
									ssend(conn_s_ftp, "226 OK\r\n");

								else if( rr == -4)
									ssend(conn_s_ftp, "550 Error\r\n");
								else
									ssend(conn_s_ftp, "451 Error\r\n");

							}
							//else ssend(conn_s_ftp, "550 Error\r\n");
							}
							else
							ssend(conn_s_ftp, "501 Error\r\n");
					}
					else
					{
						ssend(conn_s_ftp, "425 Error\r\n");
					}
				}
				else
				if(strcasecmp(cmd, "DELE") == 0)
				{
					if(split == 1)
					{

						absPath(filename, param, cwd);

						if(cellFsUnlink(filename) == 0)
						{
							ssend(conn_s_ftp, "250 OK\r\n");
						}
						else
						{
							ssend(conn_s_ftp, "550 Error\r\n");
						}
					}
					else
					{
						ssend(conn_s_ftp, "501 Error\r\n");
					}
				}
				else
				if(strcasecmp(cmd, "MKD") == 0)
				{
					if(split == 1)
					{

						absPath(filename, param, cwd);

						if(cellFsMkdir(filename, 0777) == 0)
						{
							sprintf(buffer, "257 \"%s\" OK\r\n", param);
							ssend(conn_s_ftp, buffer);
						}
						else
						{
							ssend(conn_s_ftp, "550 Error\r\n");
						}
					}
					else
					{
						ssend(conn_s_ftp, "501 Error\r\n");
					}
				}
				else
				if(strcasecmp(cmd, "RMD") == 0)
				{
					if(split == 1)
					{

						absPath(filename, param, cwd);

						if(cellFsRmdir(filename) == 0)
						{
							ssend(conn_s_ftp, "250 OK\r\n");
						}
						else
						{
							ssend(conn_s_ftp, "550 Error\r\n");
						}
					}
					else
					{
						ssend(conn_s_ftp, "501 Error\r\n");
					}
				}
				else
				if(strcasecmp(cmd, "STOR") == 0)
				{
					if(data_s > 0)
					{
						if(split == 1)
						{
							absPath(filename, param, cwd);

							int rr=-1;
							u64 pos=0;

							if(cellFsOpen(filename, CELL_FS_O_CREAT|CELL_FS_O_WRONLY, &fd, NULL, 0) == CELL_FS_SUCCEEDED)
							{

								sys_addr_t sysmem=0;
								if(sys_memory_allocate(BUFFER_SIZE_FTP, SYS_MEMORY_PAGE_SIZE_64K, &sysmem)==0)
								{
									char *buffer2= (char*)sysmem;
									u64 read_e = 0;

									if(rest)
										cellFsLseek(fd, rest, CELL_FS_SEEK_SET, &pos);
									else
										cellFsFtruncate(fd, 0);

									rest = 0;
									rr = 0;

									ssend(conn_s_ftp, "150 OK\r\n");
									//int optval = BUFFER_SIZE_FTP;
									//setsockopt(data_s, SOL_SOCKET, SO_RCVBUF, &optval, sizeof(optval));
									while(working)
									{
										//sys_timer_usleep(1668);
										if((read_e = (u64)recv(data_s, buffer2, BUFFER_SIZE_FTP, MSG_WAITALL)) > 0)
										{
											if(cellFsWrite(fd, buffer2, read_e, NULL)!=CELL_FS_SUCCEEDED) {rr=-1;break;}
										}
										else
											break;
									}
									sys_memory_free(sysmem);
								}
								cellFsClose(fd);
								cellFsChmod(filename, 0666);
								if(!working || rr!=0) cellFsUnlink(filename);
							}

							if(rr == 0)
							{
								ssend(conn_s_ftp, "226 OK\r\n");
							}
							else
							{
								ssend(conn_s_ftp, "451 Error\r\n");
							}
						}
						else
						{
							ssend(conn_s_ftp, "501 Error\r\n");
						}
					}
					else
					{
						ssend(conn_s_ftp, "425 Error\r\n");
					}
				}
				else
				if(strcasecmp(cmd, "SIZE") == 0)
				{
					if(split == 1)
					{
						absPath(filename, param, cwd);
						if(cellFsStat(filename, &buf)==CELL_FS_SUCCEEDED)
						{
							sprintf(buffer, "213 %llu\r\n", (unsigned long long)buf.st_size);
							ssend(conn_s_ftp, buffer);
							dataactive = 1;
						}
						else
						{
							ssend(conn_s_ftp, "550 Error\r\n");
						}
					}
					else
					{
						ssend(conn_s_ftp, "501 Error\r\n");
					}
				}
				else
				if(strcasecmp(cmd, "SYST") == 0)
				{
					ssend(conn_s_ftp, "215 UNIX Type: L8\r\n");
				}
				else
				if(strcasecmp(cmd, "MDTM") == 0)
				{
					if(split == 1)
					{
						absPath(filename, param, cwd);
						if(cellFsStat(filename, &buf)==CELL_FS_SUCCEEDED)
						{
							cellRtcSetTime_t(&rDate, buf.st_mtime);
							sprintf(buffer, "213 %04i%02i%02i%02i%02i%02i\r\n", rDate.year, rDate.month, rDate.day, rDate.hour, rDate.minute, rDate.second);
							ssend(conn_s_ftp, buffer);
						}
						else
						{
							ssend(conn_s_ftp, "550 Error\r\n");
						}
					}
					else
					{
						ssend(conn_s_ftp, "501 Error\r\n");
					}
				}
				else
				if(strcasecmp(cmd, "ABOR") == 0)
				{
					sclose(&data_s);
					ssend(conn_s_ftp, "226 ABOR OK\r\n");
				}

				else
				if(strcasecmp(cmd, "RNFR") == 0)
				{
					if(split == 1)
					{
						absPath(rnfr, param, cwd);

						if(cellFsStat(rnfr, &buf)==CELL_FS_SUCCEEDED)
						{
							ssend(conn_s_ftp, "350 RNFR OK\r\n");
						}
						else
						{
							rnfr[0]=0;
							ssend(conn_s_ftp, "550 RNFR Error\r\n");
						}
					}
					else
					{
						rnfr[0]=0;
						ssend(conn_s_ftp, "501 Error\r\n");
					}
				}

				else
				if(strcasecmp(cmd, "RNTO") == 0)
				{
					if(split == 1 && rnfr[0]=='/')
					{
						absPath(filename, param, cwd);

						if(cellFsRename(rnfr, filename) == CELL_FS_SUCCEEDED)
						{
							ssend(conn_s_ftp, "250 OK\r\n");
						}
						else
						{
							ssend(conn_s_ftp, "550 Error\r\n");
						}
					}
					else
					{
						ssend(conn_s_ftp, "501 Error\r\n");
					}
					rnfr[0]=0;
				}

				else
				if(strcasecmp(cmd, "USER") == 0 || strcasecmp(cmd, "PASS") == 0)
				{
					ssend(conn_s_ftp, "230 Already in\r\n");
				}
				else
				if(strcasecmp(cmd, "OPTS") == 0
				|| strcasecmp(cmd, "REIN") == 0 || strcasecmp(cmd, "ADAT") == 0
				|| strcasecmp(cmd, "AUTH") == 0 || strcasecmp(cmd, "CCC") == 0
				|| strcasecmp(cmd, "CONF") == 0 || strcasecmp(cmd, "ENC") == 0
				|| strcasecmp(cmd, "EPRT") == 0 || strcasecmp(cmd, "EPSV") == 0
				|| strcasecmp(cmd, "LANG") == 0 || strcasecmp(cmd, "LPRT") == 0
				|| strcasecmp(cmd, "LPSV") == 0 || strcasecmp(cmd, "MIC") == 0
				|| strcasecmp(cmd, "PBSZ") == 0 || strcasecmp(cmd, "PROT") == 0
				|| strcasecmp(cmd, "SMNT") == 0 || strcasecmp(cmd, "STOU") == 0
				|| strcasecmp(cmd, "XRCP") == 0 || strcasecmp(cmd, "XSEN") == 0
				|| strcasecmp(cmd, "XSEM") == 0 || strcasecmp(cmd, "XRSQ") == 0
				|| strcasecmp(cmd, "STAT") == 0)
				{
					ssend(conn_s_ftp, "502 Not implemented\r\n");
				}
				else
				{
					ssend(conn_s_ftp, "502 Error\r\n");
				}

				if(dataactive == 1)
				{
					dataactive = 0;
				}
				else
				{
					sclose(&data_s);
					if(data_ls>0) {sclose(&data_ls); data_ls=-1;}
					rest = 0;
				}
			}
			else
			{
				// available commands when not logged in
				if(strcasecmp(cmd, "USER") == 0)
				{
					if(split == 1)
					{
						ssend(conn_s_ftp, "331 OK\r\n");
					}
					else
					{
						ssend(conn_s_ftp, "501 Error\r\n");
					}
				}
				else
				if(strcasecmp(cmd, "PASS") == 0)
				{
					if(split == 1)
					{
						//#define DISABLE_PASS	(1)
						//if(DISABLE_PASS)// || (strcmp(D_USER, user) == 0 && strcmp(userpass, param) == 0))
						{
							ssend(conn_s_ftp, "230 OK\r\n");
							loggedin = 1;
						}
						//else ssend(conn_s_ftp, "430 Error\r\n");
					}
					else
					{
						ssend(conn_s_ftp, "501 Error\r\n");
					}
				}
				else
				if(strcasecmp(cmd, "QUIT") == 0 || strcasecmp(cmd, "BYE") == 0)
				{
					ssend(conn_s_ftp, "221 OK\r\n");
					connactive = 0;
				}
				else
				{
					ssend(conn_s_ftp, "530 Error\r\n");
				}
			}
		}
		else
		{
			connactive = 0;
			break;
		}

		sys_timer_usleep(1668);
	}

	sclose(&conn_s_ftp);
	sclose(&data_s);

	sys_ppu_thread_exit(0);
}



static void ftpd_thread(uint64_t arg)
{
	int list_s=-1;
relisten:
	list_s = slisten(FTPPORT, 4);

	if(list_s<0)
	{
		sys_timer_sleep(3);
		goto relisten;
	}

	if(list_s >= 0)
	{
		while(working)
		{
			sys_timer_usleep(1668);
			int conn_s_ftp;
			if((conn_s_ftp = accept(list_s, NULL, NULL)) > 0)
			{
				sys_ppu_thread_t id;
				sys_ppu_thread_create(&id, handleclient_ftp, (u64)conn_s_ftp, -0x1d8, 0x10000, 0, "ftpd");
			}
			else
				if(sys_net_errno==SYS_NET_EBADF || sys_net_errno==SYS_NET_ENETDOWN)
				{
					sclose(&list_s);
					list_s=-1;
					goto relisten;
				}
		}

		sclose(&list_s);
	}

	sys_ppu_thread_exit(0);
}


/*
uint64_t peek(uint64_t addr) //lv1
{
	system_call_1(8, (uint64_t) addr);
	return (uint64_t) p1;
}

void poke(uint64_t addr, uint64_t val) //lv1
{
	system_call_2(9, (uint64_t) addr, (uint64_t) val);
}

static uint64_t peek_chunk(uint64_t start, uint64_t size, uint8_t* buf) // read from lv1
{
	uint64_t i = 0, t = 0;
	for(i = 0; i < size; i += 8){
		t = peek(start + i);
		memcpy(buf + i, &t, 8);
	}

	return 0;
}
*/

/*
	u64 sa=0x03300000ULL;
	u64 offset=0;
	for(u64 i=0;i<0x7700000;i+=8)
	{
		if( peek(0x8000000000000000ULL+i+sa	 )==0xFFFFFF00FFFFFF00ULL &&
			peek(0x8000000000000000ULL+i+sa+8)==0xFFFFFF00FFFFFF00ULL )
		{
			show_msg((char*)"Found match!");
			offset=i+sa+16;
			for(u64 i2=offset;i2<(offset+320*4*90);i2+=8)
				poke(0x8000000000000000ULL+i2, 0x30405060708090A0ULL);
		}
	}

	int fp, i;
	uint64_t sw;
	uint32_t mem_size = (512 * 1024);
	sys_addr_t sys_mem = 0;
	uint64_t start = 0x8000000000000000ULL;

	sys_memory_allocate(mem_size, SYS_MEMORY_PAGE_SIZE_64K, &sys_mem);
	uint8_t *mem_buf	= (uint8_t*)sys_mem;

	if(cellFsOpen((char*)"/dev_hdd0/memory.bin", CELL_FS_O_CREAT | CELL_FS_O_TRUNC | CELL_FS_O_WRONLY, &fp, NULL, 0)==CELL_FS_SUCCEEDED)
	{
		for(i = 0; i < 512; i++)
		{
			peek_chunk(start + (i * mem_size), mem_size, mem_buf);
			cellFsWrite(fp, mem_buf, mem_size, &sw);
		}
		cellFsClose(fp);
	}
	sys_memory_free((sys_addr_t)sys_mem);
	show_msg((char*)"Memory dump completed!");
*/

static void poll_thread(uint64_t poll)
{
	u8 to=0;
	u8 sec=0;
	u8 step=3;
	u32 t1=0, t2=0;
	u8 lasttemp=0;
	old_fan=0;
	u8 stall=0;
	u8 step_up=5;
	//u8 step_down=2;
	u8 smoothstep=0;
	int delta=0;
	uint32_t usb_handle = -1;
	uint8_t tmp[2048];
	uint32_t r;

	CellPadData data;

	old_fan=0;
	while(working)
	{
		if(max_temp)
		{
			t1=0;
			get_temperature(0, &t1); // 3E030000 -> 3E.03'C -> 62.(03/256)'C
			t2=t1;
			sys_timer_usleep(300000);

			//if(webman_config->fanm)
			{
				get_temperature(1, &t2); // 3E030000 -> 3E.03'C -> 62.(03/256)'C
				sys_timer_usleep(200000);
			}
			t1=t1>>24;
			t2=t2>>24;

			//if( webman_config->fanm==1) t1=t2;
			//else if(webman_config->fanm==2) {t1+=t2; t1/=2;}
			//else if(webman_config->fanm==3)
			{if(t2>t1) t1=t2;}

			if(!lasttemp) lasttemp=t1;

			delta=(lasttemp-t1);

			lasttemp=t1;

			if(t1>=max_temp || t1>84)
			{
				if(delta< 0) fan_speed+=2;
				if(delta==0 && t1!=(max_temp-1)) fan_speed++;
				if(delta==0 && t1>=(max_temp+1)) fan_speed+=(2+(t1-max_temp));
				if(delta> 0)
				{
					smoothstep++;
					if(smoothstep>1)
					{
						fan_speed--;
						smoothstep=0;
					}
				}
				if(t1>84)	 fan_speed+=step_up;
				if(delta< 0 && (t1-max_temp)>=2) fan_speed+=step_up;
			}
			else
			{
				if(delta< 0 && t1>=(max_temp-1)) fan_speed+=2;
				if(delta==0 && t1<=(max_temp-2))
				{
					smoothstep++;
					if(smoothstep>1)
					{
						fan_speed--;
						if(t1<=(max_temp-5)) fan_speed--;
						if(t1<=(max_temp-8)) fan_speed--;
						smoothstep=0;
					}
				}
				//if(delta==0 && t1>=(max_temp-1)) fan_speed++;
				if(delta> 0)
				{
					smoothstep++;
					if(smoothstep)
					{
						fan_speed--;
						smoothstep=0;
					}
				}
			}

			if(t1>76 && old_fan<0x43) fan_speed++;
			if(t1>84 && fan_speed<0xB0) {old_fan=0; fan_speed=0xB0;}

			if(fan_speed<((webman_config->minfan*255)/100)) fan_speed=(webman_config->minfan*255)/100;
			if(fan_speed>MAX_FANSPEED) fan_speed=MAX_FANSPEED;

			//sprintf(debug, "OFAN: %x | CFAN: %x | TEMP: %i | STALL: %i\r\n", old_fan, fan_speed, t1, stall);	ssend(data_s, mytxt);
			//if(abs(old_fan-fan_speed)>=0x0F || stall>35 || (abs(old_fan-fan_speed) /*&& webman_config->aggr*/))
			if(old_fan!=fan_speed || stall>35)
			{
				//if(t1>76 && fan_speed<0x50) fan_speed=0x50;
				//if(t1>77 && fan_speed<0x58) fan_speed=0x58;
				if(t1>78 && fan_speed<0x50) fan_speed+=2;
				if(old_fan!=fan_speed)
				{
				old_fan=fan_speed;
				fan_control(fan_speed, 1);
				//sprintf(debug, "OFAN: %x | CFAN: %x | TEMP: %i | SPEED APPLIED!\r\n", old_fan, fan_speed, t1); ssend(data_s, mytxt);
				stall=0;
			}
			}
			else
				if( old_fan>fan_speed && (old_fan-fan_speed)>8 && t1<(max_temp-3) )
					stall++;
		}

/*
FAIL SAFE : SELECT+L3+L2+R2
RESET SAFE: SELECT+R3+L2+R2
SHOW TEMP : SELECT+R3
PREV GAME : SELECT+L1
NEXT GAME : SELECT+R1
SHUTDOWN  : L3+R2+X
RESTART   : L3+R2+O
FAN CNTRL : L3+R2+START
UNLOAD WM : L3+R2+R3
SYSCALLS  : R2+∆
SHOW IDPS : R2+O
*/
		for(u8 n=0;n<10;n++)
		{
			if(!webman_config->nopad)
			{
				data.len=0;
				if (cellPadGetData(0, &data) != CELL_PAD_OK)
					if (cellPadGetData(1, &data) != CELL_PAD_OK)
						if (cellPadGetData(2, &data) != CELL_PAD_OK) {sys_timer_usleep(300000); continue;}
				if (data.len > 0)
				{

					if ((data.button[CELL_PAD_BTN_OFFSET_DIGITAL1] & CELL_PAD_CTRL_SELECT))
					{
						if( !(webman_config->combo & FAIL_SAFE)
							&& (data.button[CELL_PAD_BTN_OFFSET_DIGITAL1] & CELL_PAD_CTRL_L3)
							&& (data.button[CELL_PAD_BTN_OFFSET_DIGITAL2] & CELL_PAD_CTRL_L2) // fail-safe mode
							&& (data.button[CELL_PAD_BTN_OFFSET_DIGITAL2] & CELL_PAD_CTRL_R2) // SELECT+L3+L2+R2
							)
						{
							cellFsUnlink((char*)"/dev_hdd0/boot_plugins.txt");
							cellFsUnlink((char*)"/dev_hdd0/tmp/turnoff");
							working=0;
							{system_call_4(379,0x1200,0,0,0);}
							sys_ppu_thread_exit(0);
						}
						else
						if( (data.button[CELL_PAD_BTN_OFFSET_DIGITAL1] & CELL_PAD_CTRL_R3)
							&& (data.button[CELL_PAD_BTN_OFFSET_DIGITAL2] & CELL_PAD_CTRL_L2) // reset-safe mode
							&& (data.button[CELL_PAD_BTN_OFFSET_DIGITAL2] & CELL_PAD_CTRL_R2) // SELECT+R3+L2+R2
							)
						{
							cellFsUnlink(WMCONFIG);
							system_call_3(392, 0x1004, 0x4, 0x6);
							show_msg(STR_RMVWMCFG);
							sys_timer_sleep(2);
							show_msg(STR_RMVWMCFGOK);
							sys_timer_sleep(3);
							cellFsUnlink((char*)"/dev_hdd0/tmp/turnoff");
							working=0;
							{system_call_4(379,0x200,0,0,0);}
							sys_ppu_thread_exit(0);
						}
#ifdef COBRA_ONLY
						else
						if( (data.button[CELL_PAD_BTN_OFFSET_DIGITAL2] & CELL_PAD_CTRL_L2) // Clone ps2emu habib's switcher
							&& (data.button[CELL_PAD_BTN_OFFSET_DIGITAL2] & CELL_PAD_CTRL_R2) // SELECT+L2+R2
							&& (c_firmware==4.53f || c_firmware==4.55f || c_firmware==4.60f) )
						{
								struct CellFsStat s;
								{system_call_8(837, (u64)(char*)"CELL_FS_IOS:BUILTIN_FLSH1", (u64)(char*)"CELL_FS_FAT", (u64)(char*)"/dev_blind", 0, 0, 0, 0, 0);}
								show_msg((char*)"Swapping ps2emu activated!");
								sys_timer_sleep(2);

								if(cellFsStat((char*)"/dev_blind/ps2emu/ps2_netemu.self.sp", &s)==CELL_FS_SUCCEEDED)
								{
									show_msg((char*)"Original ps2emu is detected!\r\nSwitch to custom emu...");

									cellFsRename("/dev_blind/ps2emu/ps2_netemu.self", "/dev_blind/ps2emu/ps2_netemu.tmp");
									cellFsRename("/dev_blind/ps2emu/ps2_netemu.self.sp", "/dev_blind/ps2emu/ps2_netemu.self");

									cellFsRename("/dev_blind/ps2emu/ps2_gxemu.self", "/dev_blind/ps2emu/ps2_gxemu.tmp");
									cellFsRename("/dev_blind/ps2emu/ps2_gxemu.self.sp", "/dev_blind/ps2emu/ps2_gxemu.self");

									cellFsRename("/dev_blind/ps2emu/ps2_emu.self", "/dev_blind/ps2emu/ps2_emu.tmp");
									cellFsRename("/dev_blind/ps2emu/ps2_emu.self.sp", "/dev_blind/ps2emu/ps2_emu.self");
								}
								else if(cellFsStat("/dev_blind/ps2emu/ps2_netemu.tmp", &s)==CELL_FS_SUCCEEDED)
								{
									show_msg((char*)"Custom ps2emu is detected!\r\nSwitch to original emu...");

									cellFsRename("/dev_blind/ps2emu/ps2_netemu.self", "/dev_blind/ps2emu/ps2_netemu.self.sp");
									cellFsRename("/dev_blind/ps2emu/ps2_netemu.tmp", "/dev_blind/ps2emu/ps2_netemu.self");

									cellFsRename("/dev_blind/ps2emu/ps2_gxemu.self", "/dev_blind/ps2emu/ps2_gxemu.self.sp");
									cellFsRename("/dev_blind/ps2emu/ps2_gxemu.tmp", "/dev_blind/ps2emu/ps2_gxemu.self");

									cellFsRename("/dev_blind/ps2emu/ps2_emu.self", "/dev_blind/ps2emu/ps2_emu.self.sp");
									cellFsRename("/dev_blind/ps2emu/ps2_emu.tmp", "/dev_blind/ps2emu/ps2_emu.self");
                                }
									sys_timer_sleep(2);
									show_msg((char*)"Swapping is done! Reboot now");
									sys_timer_sleep(2);
									{system_call_3(838, (u64)(char*)"/dev_blind", 0, 1);}
									cellFsUnlink((char*)"/dev_hdd0/tmp/turnoff");
									working=0;
									{system_call_4(379,0x200,0,0,0);}
									sys_ppu_thread_exit(0);
						}
#endif
						else
						if(!(webman_config->combo & SHOW_TEMP) && (data.button[CELL_PAD_BTN_OFFSET_DIGITAL1] & CELL_PAD_CTRL_R3) ) // SELECT+R3 show temperatures
						{
							get_temperature(0, &t1);
							get_temperature(1, &t2);

							uint32_t blockSize;
							uint64_t freeSize;
							cellFsGetFreeSize((char*)"/dev_hdd0", &blockSize, &freeSize);

							u8 st, mode, speed, unknown;
							if(!webman_config->fanc)
							{
								if(c_firmware==4.55f || c_firmware==4.60f)
								{
									backup[5]=peekq(0x8000000000009E38ULL);
									lv2poke32(0x8000000000009E38ULL, 0x38600001); // sys 409 get_fan_policy
								}
								else
								{
								backup[5]=peekq(0x8000000000009E28ULL);
								lv2poke32(0x8000000000009E28ULL, 0x38600001); // sys 409 get_fan_policy
								}
							}

							sys_sm_get_fan_policy(0, &st, &mode, &speed, &unknown);

							if(!webman_config->fanc)
							{
								if(c_firmware==4.55f || c_firmware==4.60f)
									pokeq(0x8000000000009E38ULL, backup[5]);
								else
									pokeq(0x8000000000009E28ULL, backup[5]);
							}
							_meminfo meminfo;
							{system_call_1(352, (uint64_t) &meminfo);}

							sprintf((char*)tmp, "CPU: %i°C  RSX: %i°C  FAN: %i%%   \r\n%s: %i %s\r\n%s: %i %s", t1>>24, t2>>24, (int)(((int)speed*100)/255), STR_STORAGE, (int)((blockSize*freeSize)>>20), STR_MBFREE, STR_MEMORY, meminfo.avail>>10, STR_KBFREE);
							show_msg((char*)tmp);
							sys_timer_sleep(2);
						}
						else
						if(webman_config->fanc && !(webman_config->combo & MANUALFAN) && (data.button[CELL_PAD_BTN_OFFSET_DIGITAL1] & CELL_PAD_CTRL_UP) ) // SELECT+UP increase TEMP/FAN
						{
							if(max_temp) //auto mode
							{
								max_temp+=5;
								if(max_temp>85) max_temp=85;
								webman_config->temp1=max_temp;
								sprintf((char*)tmp, "%s\r\n%s %i°C", STR_FANCH0, STR_FANCH1, max_temp);
							}
							else
							{
								webman_config->manu+=5;
								webman_config->temp0= (u8)(((float)webman_config->manu * 255.f)/100.f);
								if(webman_config->temp0<0x33) webman_config->temp0=0x33;
								if(webman_config->temp0>MAX_FANSPEED) webman_config->temp0=MAX_FANSPEED;
								fan_control(webman_config->temp0, 0);
								sprintf((char*)tmp, "%s\r\n%s %i%%", STR_FANCH0, STR_FANCH2, webman_config->manu);
							}
							save_settings();
							show_msg((char*)tmp);
							sys_timer_sleep(2);
						}
						else
						if(webman_config->fanc && !(webman_config->combo & MANUALFAN) && (data.button[CELL_PAD_BTN_OFFSET_DIGITAL1] & CELL_PAD_CTRL_DOWN) ) // SELECT+DOWN increase TEMP/FAN
						{
							if(max_temp) //auto mode
							{
								if(max_temp>30) max_temp-=5;
								webman_config->temp1=max_temp;
								sprintf((char*)tmp, "%s\r\n%s %i°C", STR_FANCH0, STR_FANCH1, max_temp);
							}
							else
							{
								if(webman_config->manu>20) webman_config->manu-=5;
								webman_config->temp0= (u8)(((float)webman_config->manu * 255.f)/100.f);
								if(webman_config->temp0<0x33) webman_config->temp0=0x33;
								if(webman_config->temp0>MAX_FANSPEED) webman_config->temp0=MAX_FANSPEED;
								fan_control(webman_config->temp0, 0);
								sprintf((char*)tmp, "%s\r\n%s %i%%", STR_FANCH0, STR_FANCH2, webman_config->manu);
							}
							save_settings();
							show_msg((char*)tmp);
							sys_timer_sleep(2);
						}
						else
						if(webman_config->minfan && !(webman_config->combo & MINDYNFAN) && (data.button[CELL_PAD_BTN_OFFSET_DIGITAL1] & CELL_PAD_CTRL_LEFT) ) // SELECT+LEFT decrease Minfan
						{
							if(webman_config->minfan-5>=MIN_FANSPEED) webman_config->minfan-=5;
							sprintf((char*)tmp, "%s\r\n%s %i%%", STR_FANCH0, STR_FANCH3, webman_config->minfan);

							save_settings();
							show_msg((char*)tmp);
							sys_timer_sleep(2);
						}
						else
						if(webman_config->minfan && !(webman_config->combo & MINDYNFAN) && (data.button[CELL_PAD_BTN_OFFSET_DIGITAL1] & CELL_PAD_CTRL_RIGHT) ) // SELECT+RIGHT increase Minfan
						{
							if(webman_config->minfan+5<100) webman_config->minfan+=5;
							sprintf((char*)tmp, "%s\r\n%s %i%%", STR_FANCH0, STR_FANCH3, webman_config->minfan);

							save_settings();
							show_msg((char*)tmp);
							sys_timer_sleep(2);
						}
						else
						if(!(webman_config->combo & PREV_GAME) && (data.button[CELL_PAD_BTN_OFFSET_DIGITAL2] & CELL_PAD_CTRL_L1) ) // SELECT+L1 (previous title)
						{
							led(GREEN, BLINK);
							mount_with_mm((char*)"_prev", 1);
							sys_timer_sleep(3);
							led(GREEN, ON);
						}
						else
						if(!(webman_config->combo & NEXT_GAME) && (data.button[CELL_PAD_BTN_OFFSET_DIGITAL2] & CELL_PAD_CTRL_R1) ) // SELECT+R1 (next title)
						{
							led(GREEN, BLINK);
							mount_with_mm((char*)"_next", 1);
							sys_timer_sleep(3);
							led(GREEN, ON);
						}
					}
					else
					if ((data.button[CELL_PAD_BTN_OFFSET_DIGITAL1] & CELL_PAD_CTRL_L3) && (data.button[CELL_PAD_BTN_OFFSET_DIGITAL2] & CELL_PAD_CTRL_R2))
					{
						if(!(webman_config->combo & SHUT_DOWN) && (data.button[CELL_PAD_BTN_OFFSET_DIGITAL2] & CELL_PAD_CTRL_CROSS) ) // L3+R2+X (shutdown)
						{
							// power off
							cellFsUnlink((char*)"/dev_hdd0/tmp/turnoff");
							working=0;
							{system_call_4(379,0x1100,0,0,0);}
							sys_ppu_thread_exit(0);
						}
						else if(!(webman_config->combo & RESTARTPS) && (data.button[CELL_PAD_BTN_OFFSET_DIGITAL2] & CELL_PAD_CTRL_CIRCLE) ) // L3+R2+O (restart)
						{
							// reboot
							cellFsUnlink((char*)"/dev_hdd0/tmp/turnoff");
							working=0;
							{system_call_4(379,0x1200,0,0,0);}
							sys_ppu_thread_exit(0);
						}
						else if(!(webman_config->combo & UNLOAD_WM) && (data.button[CELL_PAD_BTN_OFFSET_DIGITAL1] & CELL_PAD_CTRL_R3) ) // L3+R3+R2 (quit webMAN)
                        {
							restore_fan(1);
							show_msg((char*)STR_WMUNL);
							working=0;
							wwwd_stop();
							sys_ppu_thread_exit(0);
							break;
						}
						else if(!(webman_config->combo & DISABLEFC) && (data.button[CELL_PAD_BTN_OFFSET_DIGITAL1] & CELL_PAD_CTRL_START) ) // L3+R2+START (enable/disable fancontrol)
						{
							if(webman_config->fanc) webman_config->fanc=0; else webman_config->fanc=1;
							max_temp=0;
							if(webman_config->fanc)
							{
								if(webman_config->temp0==0) max_temp=webman_config->temp1; else max_temp=0;
								fan_control(webman_config->temp0, 0);
								sprintf((char*)tmp, "%s %s", STR_FANCTRL3, STR_ENABLED);
							}
							else
							{
								restore_fan(0);
								sprintf((char*)tmp, "%s %s", STR_FANCTRL3, STR_DISABLED);
							}
							save_settings();
							show_msg((char*)tmp);
							sys_timer_sleep(2);
						}
					}
					else
					if ((data.button[CELL_PAD_BTN_OFFSET_DIGITAL1] & CELL_PAD_CTRL_L3) && (data.button[CELL_PAD_BTN_OFFSET_DIGITAL2] & CELL_PAD_CTRL_R1))
					{
						if(data.button[CELL_PAD_BTN_OFFSET_DIGITAL2] & CELL_PAD_CTRL_CIRCLE) // L3+R1+O (soft restart)
						{
							// soft reboot lv2
							cellFsUnlink((char*)"/dev_hdd0/tmp/turnoff");
							working=0;
							{system_call_4(379,0x200,0,0,0);}
							sys_ppu_thread_exit(0);
						}
					}
					else
					if(data.button[CELL_PAD_BTN_OFFSET_DIGITAL2] & CELL_PAD_CTRL_R2)
					{
						if(!(webman_config->combo & DISABLESH) && (data.button[CELL_PAD_BTN_OFFSET_DIGITAL2] & CELL_PAD_CTRL_TRIANGLE) ) // R2+∆ Disable CFW Sycalls
                        {
							if(peekq(0x8000000000003000ULL)==0xFFFFFFFF80010003ULL) {
                                system_call_3(392, 0x1004, 0x7, 0x36);
								show_msg(STR_CFWSYSALRD);
                                sys_timer_sleep(2);
								} else {
                                show_msg(STR_CFWSYSRIP);
								remove_cfw_syscalls();
                                delete_history();
                                if(peekq(0x8000000000003000ULL)==0xFFFFFFFF80010003ULL) {
									system_call_3(392, 0x1004, 0x4, 0x6);
									show_msg(STR_RMVCFWSYS);
                                    sys_timer_sleep(2);
                                } else {
                                    system_call_3(392, 0x1004, 0x7, 0x36);
									show_msg(STR_RMVCFWSYSF);
                                    sys_timer_sleep(2);
								}
							}
						}
						else if(!(webman_config->combo & SHOW_IDPS) && (data.button[CELL_PAD_BTN_OFFSET_DIGITAL2] & CELL_PAD_CTRL_CIRCLE) ) // R2+O Show IDPS EID0+LV2
						{
							uint64_t eid0_idps[2], buffer[0x40], start_sector;
							uint32_t read;
							sys_device_handle_t source;
							if(sys_storage_open(0x100000000000004ULL, 0, &source, 0)!=0)
							{
								start_sector = 0x204;
								sys_storage_close(source);
								sys_storage_open(0x100000000000001ULL, 0, &source, 0);
							}
							else start_sector = 0x178;
							sys_storage_read(source, 0, start_sector, 1, buffer, &read, 0);
							sys_storage_close(source);

							eid0_idps[0]=buffer[0x0E];
							eid0_idps[1]=buffer[0x0F];

							if(c_firmware<4.55f) {
								system_call_1(870, (uint64_t) IDPS);
							} else if((c_firmware==4.55f || c_firmware==4.60f) && peekq(0x8000000000003000ULL)!=0xFFFFFFFF80010003ULL) {
								IDPS[0] = peekq(0x8000000000474F1CULL  );
								IDPS[1] = peekq(0x8000000000474F1CULL+8);
							}

							sprintf(tmp, "EID0 : %llX%llX\r\nLV2  : %llX%llX", eid0_idps[0], eid0_idps[1], IDPS[0], IDPS[1]);
							show_msg(tmp);
							sys_timer_sleep(2);
						}
					}
				}
			}
			//sys_timer_sleep(step);
			sys_timer_usleep(300000);
		}
		to++;
		if(to==20)
		{
			get_temperature(0, &t1);
			get_temperature(1, &t2);
			t1>>=24; t2>>=24;
			if(t1>83 || t2>83)
			{
				if(!webman_config->warn)
				{
					sprintf((char*)tmp, "%s\r\n CPU: %i°C   RSX: %i°C", STR_OVERHEAT, t1, t2);
				show_msg((char*)tmp);
				sys_timer_sleep(2);
				}
				if(t1>85 || t2>85)
				{
					if(!max_temp) max_temp=82;
					if(fan_speed<0xB0)
					fan_speed=0xB0;
					else
						if(fan_speed<MAX_FANSPEED) fan_speed+=8;

					old_fan=fan_speed;
					fan_control(fan_speed, 0);
					show_msg((char*)STR_OVERHEAT2);
				}
			}
		}
		if(to>40) to=0;

		if(poll==0 && sec>=120) // check USB drives each 120 seconds
		{
			for(u8 f0=0; f0<8; f0++)
			{
				if(sys_storage_open(((f0<6)?USB_MASS_STORAGE_1(f0):USB_MASS_STORAGE_2(f0)), 0, &usb_handle, 0)==0)
				{
					sys_storage_read(usb_handle, 0, to, 1, tmp, &r, 0);
					sys_storage_close(usb_handle);
					//sprintf(tmp, "/dev_usb00%i: Read %i sectors @ %i offset", f0, r, to); show_msg(tmp);
				}
			}
			sec=0;
		}
		sec+=step;
	}
	sys_ppu_thread_exit(0);
}

void restore_fan(u8 settemp)
{
	if(backup[0]==1)
	{
		//pokeq(backup[0] + (u64) (130 * 8), backup[1]);
		//pokeq(backup[0] + (u64) (138 * 8), backup[2]);
		//pokeq(backup[0] + (u64) (379 * 8), backup[3]);

		if(webman_config->ps2temp<20) webman_config->ps2temp=20;
		sys_sm_set_fan_policy(0, 1, ((webman_config->ps2temp*255)/100));
		if(settemp) sys_sm_set_fan_policy(0, 2, ((webman_config->ps2temp*255)/100));

		if(c_firmware==4.55f || c_firmware==4.60f)
		{
			pokeq(0x800000000000A334ULL, backup[4]);
			pokeq(0x8000000000009E38ULL, backup[5]);
		}
		else
		{
			pokeq(0x800000000000A324ULL, backup[4]);
			pokeq(0x8000000000009E28ULL, backup[5]);
		}
		backup[0]=0;
	}
}

void fan_control(u8 temp0, u8 initial)
{
	if(!dex_mode && (c_firmware==4.60f || c_firmware==4.55f || c_firmware==4.53f || c_firmware==4.46f || c_firmware==4.21f || c_firmware==4.30f || c_firmware==4.31f || c_firmware==4.40f || c_firmware==4.41f || c_firmware==4.50f) )
	{
		if(!initial)
		{
			if(backup[0]==0)
			{
				backup[0]=1;
				//backup[1]=peekq(syscall_base + (u64) (130 * 8));
				//backup[2]=peekq(syscall_base + (u64) (138 * 8));
				//backup[3]=peekq(syscall_base + (u64) (379 * 8));
				if(c_firmware==4.55f || c_firmware==4.60f)
				{
					backup[4]=peekq(0x800000000000A334ULL);
					backup[5]=peekq(0x8000000000009E38ULL);
					lv2poke32(0x8000000000009E38ULL, 0x38600001); // sys 409 get_fan_policy
					lv2poke32(0x800000000000A334ULL, 0x38600001); // sys 389 set_fan_policy
				}
				else
				{
				backup[4]=peekq(0x800000000000A324ULL);
				backup[5]=peekq(0x8000000000009E28ULL);
				lv2poke32(0x8000000000009E28ULL, 0x38600001); // sys 409 get_fan_policy
				lv2poke32(0x800000000000A324ULL, 0x38600001); // sys 389 set_fan_policy
				}

			    sys_sm_set_fan_policy(0, 2, 0x33);
		    }
		}

		if(temp0<0x33)
		{
			u8 st, mode, unknown;
			u8 fan_speed8=0;
			sys_sm_get_fan_policy(0, &st, &mode, &fan_speed8, &unknown);
			if(fan_speed8<0x33) return;
			fan_speed=fan_speed8;
		}
		else
			fan_speed=temp0;

		if(fan_speed<0x33 || fan_speed>0xFC)
		{
			fan_speed=0x48;
			sys_sm_set_fan_policy(0, 2, fan_speed);
			sys_timer_sleep(2);
		}
		old_fan=fan_speed;
		sys_sm_set_fan_policy(0, 2, fan_speed);
	}
}

void save_settings()
{
	u64 written;
	int fdwm=0;
	if(cellFsOpen(WMCONFIG, CELL_FS_O_CREAT|CELL_FS_O_WRONLY, &fdwm, NULL, 0) == CELL_FS_SUCCEEDED)
	{
		cellFsWrite(fdwm, (void *)wmconfig, sizeof(WebmanCfg), &written);
		cellFsClose(fdwm);
	}
}

void reset_settings()
{
	memset(webman_config, 0, sizeof(WebmanCfg));

	webman_config->usb0=1;
	webman_config->usb1=0;
	webman_config->usb2=0;
	webman_config->usb3=0;
	webman_config->usb6=0;
	webman_config->usb7=0;

	webman_config->netd0=0;
	webman_config->lastp=1;
	webman_config->autob=0;
	webman_config->delay=0;
	webman_config->bootd=0;
	webman_config->boots=3;
	webman_config->nogrp=0;
	webman_config->noset=0;
	webman_config->cmask=0;

	webman_config->poll=1;
	webman_config->nopad=0;
	webman_config->nocov=0;
	webman_config->nospoof=1;

	webman_config->fanc=0;
	webman_config->temp1=MY_TEMP;
//	webman_config->fanm=3;			//unused
	webman_config->ps2temp=37;

	webman_config->minfan=MIN_FANSPEED;

	webman_config->bind=0;
	webman_config->refr=0;
	webman_config->manu=35;
	webman_config->temp0=0;

	webman_config->neth0[0]=0;
	webman_config->netp0=38008;

	webman_config->netd1=0;
	webman_config->neth1[0]=0;
	webman_config->netp1=38008;

	webman_config->foot=1;
	webman_config->nospoof=1;
	webman_config->pspl=0;

	webman_config->sidps=0;
	webman_config->spsid=0;
	webman_config->spp=0;
	webman_config->vIDPS1[0]=0;
	webman_config->vIDPS2[0]=0;
	webman_config->vPSID1[0]=0;
	webman_config->vPSID2[0]=0;
	webman_config->lang=0;

	int fdwm=0;
	if(cellFsOpen(WMCONFIG, CELL_FS_O_RDONLY, &fdwm, NULL, 0) == CELL_FS_SUCCEEDED)
	{
		cellFsRead(fdwm, (void *)wmconfig, sizeof(WebmanCfg), NULL);
		cellFsClose(fdwm);
	}
	else
	{
		u64 written;
		if(cellFsOpen(WMCONFIG, CELL_FS_O_CREAT|CELL_FS_O_WRONLY, &fdwm, NULL, 0) == CELL_FS_SUCCEEDED)
		{
			cellFsWrite(fdwm, (void *)wmconfig, sizeof(WebmanCfg), &written);
			cellFsClose(fdwm);
		}
	}

	if(webman_config->warn>1) webman_config->warn=0;
	if(webman_config->minfan<MIN_FANSPEED) webman_config->minfan=MIN_FANSPEED;
}

static void wwwd_thread(uint64_t arg)
{

//	sys_timer_sleep(8);
//	u32 mode=0;
//	if(in_cobra(&mode)==0) cobra_mode=1;

	backup[0]=0;

	u64 CEX=0x4345580000000000ULL;
#ifndef COBRA_ONLY
	u64 DEX=0x4445580000000000ULL;
#endif
	dex_mode=0;
	c_firmware=0.00f;

			if(peekq(0x80000000002ED850ULL)==CEX) {c_firmware=4.60f;}
	else	if(peekq(0x80000000002EC5E0ULL)==CEX) {c_firmware=4.55f;}
	else	if(peekq(0x80000000002E9D70ULL)==CEX) {c_firmware=4.53f;}
	else	if(peekq(0x80000000002E9BE0ULL)==CEX) {c_firmware=4.50f;}
	else	if(peekq(0x80000000002EA9B8ULL)==CEX) {c_firmware=4.46f;}
	else	if(peekq(0x80000000002D83D0ULL)==CEX) {c_firmware=3.55f;}
#ifndef COBRA_ONLY
	else	if(peekq(0x8000000000309698ULL)==DEX) {c_firmware=4.50f; dex_mode=2;}
	else	if(peekq(0x8000000000305410ULL)==DEX) {c_firmware=4.46f; dex_mode=2;}
	else	if(peekq(0x8000000000304EF0ULL)==DEX) {c_firmware=4.41f; dex_mode=2;}
	else	if(peekq(0x80000000002EA498ULL)==CEX) {c_firmware=4.41f;}
	else	if(peekq(0x80000000002EA488ULL)==CEX) {c_firmware=4.40f;}
	else	if(peekq(0x80000000002E9F18ULL)==CEX) {c_firmware=4.31f;}
	else	if(peekq(0x8000000000304630ULL)==DEX) {c_firmware=4.30f; dex_mode=2;}
	else	if(peekq(0x80000000002E9F08ULL)==CEX) {c_firmware=4.30f;}
	else	if(peekq(0x80000000002E8610ULL)==CEX) {c_firmware=4.21f;}
	else	if(peekq(0x8000000000302D88ULL)==DEX) {c_firmware=4.21f; dex_mode=2;}
	else	if(peekq(0x80000000002EFE20ULL)==DEX) {c_firmware=3.55f; dex_mode=2;}
	else	if(peekq(0x80000000002CFF98ULL)==CEX) {c_firmware=3.41f;}
	else	if(peekq(0x80000000002E79C8ULL)==DEX) {c_firmware=3.41f; dex_mode=2;}
#endif


#ifdef COBRA_ONLY
	cobra_lib_init();
	cobra_mode=1;
#else
	cobra_mode=0;
#endif

	//pokeq(0x8000000000003560ULL, 0x386000014E800020ULL); // li r3, 0 / blr
	//pokeq(0x8000000000003D90ULL, 0x386000014E800020ULL); // li r3, 0 / blr

	led(YELLOW, BLINK);

	WebmanCfg *webman_config = (WebmanCfg*) wmconfig;
	reset_settings();

	if(webman_config->blind)
		{system_call_8(837, (u64)(char*)"CELL_FS_IOS:BUILTIN_FLSH1", (u64)(char*)"CELL_FS_FAT", (u64)(char*)"/dev_blind", 0, 0, 0, 0, 0);}

		//STANDARD
		BUFFER_SIZE_FTP	= ( 128*1024);
		BUFFER_SIZE		= ( 448*1024);
		BUFFER_SIZE_PSX	= ( 160*1024);
		BUFFER_SIZE_PSP	= (  32*1024);
		BUFFER_SIZE_PS2	= (  64*1024);
		BUFFER_SIZE_DVD	= ( 192*1024);

	if(webman_config->foot==1) //MIN
	{
		BUFFER_SIZE_FTP	= ( 128*1024);
		BUFFER_SIZE		= ( 128*1024);
		BUFFER_SIZE_PSX	= (  32*1024);
		BUFFER_SIZE_PSP	= (  32*1024);
		BUFFER_SIZE_PS2	= (  64*1024);
		BUFFER_SIZE_DVD	= (  64*1024);
	}

	if(webman_config->foot==2) //MAX
	{
		BUFFER_SIZE_FTP	= ( 256*1024);
		BUFFER_SIZE		= ( 640*1024);
		BUFFER_SIZE_PSX	= ( 192*1024);
		BUFFER_SIZE_PSP	= (  64*1024);
		BUFFER_SIZE_PS2	= ( 128*1024);
		BUFFER_SIZE_DVD	= ( 256*1024);
	}

	BUFFER_SIZE_ALL=(BUFFER_SIZE)+(BUFFER_SIZE_PSX)+(BUFFER_SIZE_PSP)+(BUFFER_SIZE_PS2)+(BUFFER_SIZE_DVD);

	init_running=1;
	sys_ppu_thread_t id2;
	sys_ppu_thread_create(&id2, handleclient, (u64)0xC0FEBABE, -0x1d8, 0x20000, 0, "wwwd2");

	if(!webman_config->ftpd)
		sys_ppu_thread_create(&thread_id_ftp, ftpd_thread, NULL, -0x1d8,  0x2000, SYS_PPU_THREAD_CREATE_JOINABLE, THREAD_NAME_FTP);

	led(YELLOW, OFF);
	sys_timer_sleep(5);

#ifdef USE_DEBUG
	u8 d_retries=0;
again_debug:
	debug_s=connect_to_server((char*)"192.168.100.209", 38009);
	if(debug_s<0) {d_retries++; sys_timer_sleep(2); if(d_retries<10) goto again_debug;}
	if(debug_s>=0) ssend(debug_s, "Connected...\r\n");
	sprintf(debug, "FC=%i T0=%i T1=%i\r\n", webman_config->fanc, webman_config->temp0, webman_config->temp1);
	ssend(debug_s, debug);
#endif

	max_temp=0;

	if(webman_config->fanc)
	{
		if(webman_config->temp0==0) max_temp=webman_config->temp1; else max_temp=0;
		fan_control(webman_config->temp0, 0);
	}

    sys_ppu_thread_create(&thread_id_poll, poll_thread, (u64)webman_config->poll, -0x1d8, 0x20000, 0, "poll_thread");

	while(init_running && working) sys_timer_usleep(100000);

	led(GREEN, ON);

//	cellFsUnlink((char*)"/dev_hdd0/tmp/turnoff");

	int list_s;

relisten:
#ifdef USE_DEBUG
	ssend(debug_s, "Listening on port 80...");
#endif
	list_s = slisten(WWWPORT, 8);

	if(list_s<0 && working)
	{
		sys_timer_sleep(2);
		goto relisten;
	}

	if(list_s >= 0)
	{
#ifdef USE_DEBUG
		ssend(debug_s, " OK!\r\n");
#endif
		while(working)
		{
			sys_timer_usleep(10000);
			while(loading_html>2 && working)
			{
#ifdef USE_DEBUG
	sprintf(debug, "THREADS: %i\r\n", loading_html);
	ssend(debug_s, debug);
#endif
				sys_timer_usleep(300000);
			}
			int conn_s;
			if((conn_s = accept(list_s, NULL, NULL)) > 0)
			{
				loading_html++;
				#ifdef USE_DEBUG
				ssend(debug_s, "*** Incomming connection... ");
				#endif
				sys_ppu_thread_t id;
				sys_ppu_thread_create(&id, handleclient, (u64)conn_s, -0x1d8, 0x20000, 0, "wwwd");
			}
			else
				if(sys_net_errno==SYS_NET_EBADF || sys_net_errno==SYS_NET_ENETDOWN)
				{
					sclose(&list_s);
					list_s=-1;
					goto relisten;
				}
		}

		sclose(&list_s);
	}

	sys_ppu_thread_exit(0);
}


int wwwd_start(uint64_t arg)
{
	sys_ppu_thread_create(&thread_id, wwwd_thread, NULL, -0x1d8, 0x4000, SYS_PPU_THREAD_CREATE_JOINABLE, THREAD_NAME);
#ifndef CCAPI
	_sys_ppu_thread_exit(0); // remove for ccapi compatibility
#endif
	return SYS_PRX_RESIDENT;
}

static void wwwd_stop_thread(uint64_t arg)
{
	restore_fan(0);

	uint64_t exit_code;
	working = 0;

	//if (thread_id != (sys_ppu_thread_t)-1)
		sys_ppu_thread_join(thread_id, &exit_code);

	if (thread_id_ftp != (sys_ppu_thread_t)-1)
		sys_ppu_thread_join(thread_id_ftp, &exit_code);

	//if (thread_id_poll != (sys_ppu_thread_t)-1)
		sys_ppu_thread_join(thread_id_poll, &exit_code);

	sys_ppu_thread_exit(0);
}

static void finalize_module(void)
{
	uint64_t meminfo[5];

	sys_prx_id_t prx = prx_get_module_id_by_address(finalize_module);

	meminfo[0] = 0x28;
	meminfo[1] = 2;
	meminfo[3] = 0;

	system_call_3(482, prx, 0, (uint64_t)(uint32_t)meminfo);
}

int wwwd_stop(void)
{
	sys_ppu_thread_t t;
	uint64_t exit_code;

	sys_ppu_thread_create(&t, wwwd_stop_thread, 0, 0, 0x2000, SYS_PPU_THREAD_CREATE_JOINABLE, STOP_THREAD_NAME);
	sys_ppu_thread_join(t, &exit_code);

	finalize_module();
	_sys_ppu_thread_exit(0);
	return SYS_PRX_STOP_OK;
}

#ifndef COBRA_ONLY
void add_to_map(char *path1, char *path2)
{
	if(max_mapped==0) pokeq(MAP_BASE + 0x00, 0x0000000000000000ULL);

	if(max_mapped<8)
	{
		for(u8 n=0; n<max_mapped; n++)
		{
			if(!strcmp(file_to_map[n].src, path1)) goto leave;
		}
		sprintf(file_to_map[max_mapped].src, "%s", path1);
		sprintf(file_to_map[max_mapped].dst, "%s", path2);
		max_mapped++;
	}
leave:
	return;
}

void string_to_lv2(char* path, u64 addr)
{
	u16 len=(strlen(path)+8)&0x7f8;
	if(len<8) len=8;
	if(len>384) len=384;
	u16 len2=strlen(path); if(len2>len) len2=len;

	u8 data2[384];
	u8* data = data2;
	memset(data, 0, 384);
	memcpy(data, path, len2);

	u64 val=0x0000000000000000ULL;
	for(u64 n = 0; n < len; n += 8)
	{
		memcpy(&val, &data[n], 8);
		pokeq(addr+n, val);
	}
}
#endif

void eject_insert(u8 eject, u8 insert)
{
	u8 atapi_cmnd2[56];
	u8* atapi_cmnd = atapi_cmnd2;
	int dev_id;
	memset(atapi_cmnd, 0, 56);

	{system_call_4(600, BDVD_DRIVE, 0, (uint64_t) &dev_id, 0);}

	if(eject)
	{
		atapi_cmnd[0x00]=0x1b;
		atapi_cmnd[0x01]=0x01;
		atapi_cmnd[0x04]=0x02;
		atapi_cmnd[0x23]=0x0c;

		// Eject disc
		{system_call_7(616, dev_id, 1, (uint64_t) atapi_cmnd, 56, NULL, 0, NULL);}
	}

	sys_timer_sleep(2);

	if(insert)
	{
		memset(atapi_cmnd, 0, 56);
		atapi_cmnd[0x00]=0x1b;
		atapi_cmnd[0x01]=0x01;
		atapi_cmnd[0x04]=0x03;
		atapi_cmnd[0x23]=0x0c;

		// Insert disc
		{system_call_7(616, dev_id, 1, (uint64_t) atapi_cmnd, 56, NULL, 0, NULL);}
	}

	{system_call_1(601, dev_id);}
}

#ifdef COBRA_ONLY
int do_umount_iso()
{
	unsigned int real_disctype, effective_disctype, iso_disctype;

	cobra_get_disc_type(&real_disctype, &effective_disctype, &iso_disctype);

	// If there is an effective disc in the system, it must be ejected
	if (effective_disctype != DISC_TYPE_NONE)
	{
		cobra_send_fake_disc_eject_event();
		sys_timer_usleep(4000);
	}

	if (iso_disctype != DISC_TYPE_NONE)
	cobra_umount_disc_image();

	// If there is a real disc in the system, issue an insert event
	if (real_disctype != DISC_TYPE_NONE)
	{
		struct CellFsStat bufc;
		cobra_send_fake_disc_insert_event();
		for(u8 m=0; m<22; m++)
		{
			sys_timer_usleep(4000);

			if(cellFsStat((char*)"/dev_bdvd", &bufc)==CELL_FS_SUCCEEDED) break;
		}
		cobra_disc_auth();
	}
	return 0;
}
#endif

static void mount_with_mm(const char *_path0, u8 do_eject)
{
	//pokeq(0x8000000000003560ULL, 0x386000014E800020ULL); // li r3, 0 / blr
	//pokeq(0x8000000000003560ULL, 0x392000027C0802A6ULL); // li r9, 2 / ...
	//pokeq(0x8000000000003D90ULL, 0x386000014E800020ULL); // li r3, 0 / blr

	char _path[512];
	strcpy(_path, _path0);

#ifdef COBRA_ONLY
	//if(cobra_mode)
	{
		if(do_eject)
		{
			int fd=0;

			if(!strstr(_path0, "/PSPISO/") && !strstr(_path0, "/ISO/"))
			{
				memset(&lastgames, 0, sizeof(_lastgames));
				lastgames.last=250;
				if(cellFsOpen((char*)WMTMP "/last_games.bin", CELL_FS_O_RDONLY, &fd, NULL, 0) == CELL_FS_SUCCEEDED)
				{
					cellFsRead(fd, (void *)&lastgames, sizeof(_lastgames), NULL);
					cellFsClose(fd);
				}

				if(strstr(_path0, "_prev") || strstr(_path0, "_next"))
				{
					if(lastgames.last>(MAX_LAST_GAMES-1)) return;
					if(strstr(_path0, "_prev"))
					{
						if(lastgames.last==0) lastgames.last=(MAX_LAST_GAMES-1); else lastgames.last--;
					}

					if(strstr(_path0, "_next"))
					{
						if(lastgames.last==(MAX_LAST_GAMES-1)) lastgames.last=0; else lastgames.last++;
					}
					if(lastgames.game[lastgames.last][0]!='/') lastgames.last=0;
					if(lastgames.game[lastgames.last][0]!='/' || strlen(lastgames.game[lastgames.last])<7) return;
					strcpy(_path, lastgames.game[lastgames.last]);
				}
				else
				{
					if(lastgames.last==250)
					{
						lastgames.last=0;
						strcpy(lastgames.game[lastgames.last], _path);
					}
					else
					{
						u8 found=0;
						for(u8 n=0;n<MAX_LAST_GAMES;n++)
						{
							if(!strcmp(lastgames.game[n], _path)) {found=1; break;}
						}
						if(!found)
						{
							lastgames.last++;
							if(lastgames.last>(MAX_LAST_GAMES-1)) lastgames.last=0;
							strcpy(lastgames.game[lastgames.last], _path);
						}
					}
				}

				if(cellFsOpen((char*)WMTMP "/last_games.bin", CELL_FS_O_CREAT | CELL_FS_O_TRUNC | CELL_FS_O_WRONLY, &fd, NULL, 0) == CELL_FS_SUCCEEDED)
				{
					u64 written = 0;
					cellFsWrite(fd, (void *)&lastgames, sizeof(_lastgames), &written);
					cellFsClose(fd);
					cellFsChmod((char*)WMTMP "/last_games.bin", 0666);
				}
			}

			if(_path[0]=='_' || strrchr(_path, '/')==NULL) return;

			char path2[512];
			char temp[512];
			sprintf(path2, "\"%s", (strrchr(_path, '/')+1));
			if(strstr(path2, ".ntfs[")) path2[strrchr(path2, '.')-path2]=0;
			if(strrchr(path2, '.')!=NULL) path2[strrchr(path2, '.')-path2]=0;
            sprintf(temp, "\" %s", STR_LOADED2); strcat(path2, temp);
			show_msg(path2);

			if(!strstr(_path, "ntfs["))
			{
				sprintf(path2, WMTMP "/last_game.txt");

				if(cellFsOpen(path2, CELL_FS_O_CREAT | CELL_FS_O_TRUNC | CELL_FS_O_WRONLY, &fd, NULL, 0) == CELL_FS_SUCCEEDED)
				{
					u64 written = 0;
					cellFsWrite(fd, (void *)_path, strlen(_path), &written);
					cellFsClose(fd);
					cellFsChmod(path2, 0666);
				}
			}
		}

		{
			//rawseciso_loaded=0;
			sys_ppu_thread_t t;
			uint64_t exit_code;
			sys_ppu_thread_create(&t, rawseciso_stop_thread, 0, 0, 0x2000, SYS_PPU_THREAD_CREATE_JOINABLE, STOP_THREAD_NAME);
			sys_ppu_thread_join(t, &exit_code);

			//netiso_loaded=0
			sys_ppu_thread_create(&t, netiso_stop_thread, 0, 0, 0x2000, SYS_PPU_THREAD_CREATE_JOINABLE, STOP_THREAD_NAME);
			sys_ppu_thread_join(t, &exit_code);
		}

		while(netiso_loaded)	{sys_timer_usleep(100000);}
		while(rawseciso_loaded)	{sys_timer_usleep(100000);}

		do_umount_iso();
		sys_timer_usleep(4000);
		cobra_send_fake_disc_eject_event();
		sys_timer_usleep(4000);

		if(strstr(_path, "/PS3ISO/") || strstr(_path, "/BDISO/") || strstr(_path, "/DVDISO/") || strstr(_path, "/PS2ISO/") || strstr(_path, "/PSPISO/") || strstr(_path, "/ISO/") || strstr(_path, "/PSXISO/") || strstr(_path, "/PSXGAMES/")
			|| strstr(_path, "/net0") || strstr(_path, "/net1") || strstr(_path, ".ntfs["))
		{
			if(strstr(_path0, "_prev") || strstr(_path0, "_next"))
				sys_timer_sleep(1);
			else
				sys_timer_usleep(50000);

			{sys_map_path((char*)"/dev_bdvd", NULL);}
			{sys_map_path((char*)"/app_home", (char*)"/dev_hdd0/packages");}
			u8 iso_num=1;
			struct CellFsStat s;
			char tmp_iso[4096];
			char iso_list[16][512];
			char *cobra_iso_list[16];
			char path2[512];
			strcpy(iso_list[0], _path);
			cobra_iso_list[0] = (char*)iso_list[0];

			if(strstr(_path, "iso.0") || strstr(_path, "ISO.0"))
			{
				sprintf(path2, "%s", _path);
				path2[strlen(path2)-2]=0;
				for(u8 n=1;n<16;n++)
				{
					sprintf(tmp_iso, "%s.%i", path2, n);
					if(cellFsStat(tmp_iso, &s)==CELL_FS_SUCCEEDED)
					{
						iso_num++;
						strcpy(iso_list[n], tmp_iso);
						cobra_iso_list[n] = (char*)iso_list[n];
					}
					else
						break;

				}
			}

			if(strstr(_path, ".ntfs["))
			{
				int fdw;
				uint64_t msiz = 0;
				if(cellFsOpen(_path, CELL_FS_O_RDONLY, &fdw, 0, 0)==CELL_FS_SUCCEEDED)
				{
					sys_addr_t addr;
					if(sys_memory_allocate(64*1024, SYS_MEMORY_PAGE_SIZE_64K, &addr)==0)
					{
						u8* sprx_data=(u8*)addr;
						cellFsLseek(fdw, 0, CELL_FS_SEEK_SET, &msiz);
						cellFsRead(fdw, sprx_data, (64*1024), &msiz);
						cellFsClose(fdw);

						sys_ppu_thread_create(&thread_id_ntfs, rawseciso_thread, (uint64_t)addr, -0x1d8, 0x2000, SYS_PPU_THREAD_CREATE_JOINABLE, THREAD_NAME_NTFS);

						if(strstr(_path, ".ntfs[PS3ISO]"))
						{
							strcpy(iso_list[15], _path);
							iso_list[15][strlen(iso_list[15])-17]=0;
							strcat(iso_list[15], ".SFO");
							if(cellFsStat(iso_list[15], &s)!=CELL_FS_SUCCEEDED)
							{
								for(u8 n=0;n<10;n++)
								{
									if(cellFsStat((char*)"/dev_bdvd/PS3_GAME/PARAM.SFO", &s)==CELL_FS_SUCCEEDED)
									{
										int fdw;
										uint64_t msiz = 0;
										uint64_t msiz2 = 0;
										if(cellFsOpen((char*)"/dev_bdvd/PS3_GAME/PARAM.SFO", CELL_FS_O_RDONLY, &fdw, 0,0)==CELL_FS_SUCCEEDED)
										{
											cellFsLseek(fdw, 0, CELL_FS_SEEK_SET, &msiz);
											cellFsRead(fdw, (void *)&tmp_iso, 4096, &msiz);
											cellFsClose(fdw);

											if(msiz>256 && cellFsOpen(iso_list[15], CELL_FS_O_CREAT | CELL_FS_O_TRUNC | CELL_FS_O_WRONLY, &fdw, 0,0)==CELL_FS_SUCCEEDED)
											{
												cellFsWrite(fdw, (void *)&tmp_iso, msiz, &msiz2);
												cellFsClose(fdw);
												cellFsChmod(iso_list[15], 0666);

												iso_list[15][strlen(iso_list[15])-4]=0; strcat(iso_list[15], ".PNG");
												if(cellFsStat(iso_list[15], &s)!=CELL_FS_SUCCEEDED)
												{
													if(cellFsOpen((char*)"/dev_bdvd/PS3_GAME/ICON0.PNG", CELL_FS_O_RDONLY, &fdw, 0,0)==CELL_FS_SUCCEEDED)
													{	// copy file
														sys_addr_t buf1=0;
														if(sys_memory_allocate((256*1024), SYS_MEMORY_PAGE_SIZE_64K, &buf1)==0)
														{
															char *tmp_png=(char*)buf1;
															cellFsLseek(fdw, 0, CELL_FS_SEEK_SET, &msiz);
															cellFsRead(fdw, tmp_png, (256*1024), &msiz);
															cellFsClose(fdw);

															if(msiz>256 && cellFsOpen(iso_list[15], CELL_FS_O_CREAT | CELL_FS_O_TRUNC | CELL_FS_O_WRONLY, &fdw, 0,0)==CELL_FS_SUCCEEDED)
															{
																cellFsWrite(fdw, tmp_png, msiz, &msiz2);
																cellFsClose(fdw);
																cellFsChmod(iso_list[15], 0666);
															}
															sys_memory_free(buf1);
														}
													}
												}
											}
										}
										break;
									}
									sys_timer_usleep(500000);
								}
							}
						}

					}
				}
				return;
			}


			if(strstr(_path, "/net0") || strstr(_path, "/net1"))
			{
				sys_addr_t addr;
				if(sys_memory_allocate(64*1024, SYS_MEMORY_PAGE_SIZE_64K, &addr)==0)
				{
					netiso_args *mynet_iso	= (netiso_args*)addr;
					memset(mynet_iso, 0, 64*1024);

					if( (strstr(_path, "/net0") && webman_config->netd0 && webman_config->neth0[0] && webman_config->netp0>0)
						|| (strstr(_path, "/net1") && webman_config->netd1 && webman_config->neth1[0] && webman_config->netp1>0) )
					{
						if(strstr(_path, "/net1"))
						{
							sprintf(mynet_iso->server, "%s", webman_config->neth1);
							mynet_iso->port=webman_config->netp1;
						}

						else
						{
							sprintf(mynet_iso->server, "%s", webman_config->neth0);
							mynet_iso->port=webman_config->netp0;
						}
					}
					else
					{
						sys_memory_free(addr);
						return;
					}

					strcpy(mynet_iso->path, _path+5);
					if(strstr(_path, "/PS3ISO/")) mynet_iso->emu_mode=EMU_PS3;
					else if(strstr(_path, "/BDISO/")) mynet_iso->emu_mode=EMU_BD;
					else if(strstr(_path, "/DVDISO/")) mynet_iso->emu_mode=EMU_DVD;
					else if(strstr(_path, "/PSX"))
					{
						TrackDef tracks[1];
						tracks[0].lba = 0;
						tracks[0].is_audio = 0;
						mynet_iso->emu_mode=EMU_PSX;
						mynet_iso->numtracks=1;
						memcpy(mynet_iso->tracks, tracks, sizeof(TrackDef));
					}
					else if(strstr(_path, "/GAMES/") || strstr(_path, "/GAMEZ/"))
					{
						mynet_iso->emu_mode=EMU_PS3;
						sprintf(mynet_iso->path, "/***PS3***%s", _path+5);
					}
					else
					{
						mynet_iso->emu_mode=EMU_DVD;
						sprintf(mynet_iso->path, "/***DVD***%s", _path+5);
					}

					sys_ppu_thread_create(&thread_id_net, netiso_thread, (uint64_t)addr, -0x1d8, 0x2000, SYS_PPU_THREAD_CREATE_JOINABLE, THREAD_NAME_NET);

					if(mynet_iso->emu_mode==EMU_PS3)
					{
						sprintf(iso_list[15], WMTMP "/%s", (strrchr(_path, '/')+1));
						if(!strstr(mynet_iso->path, "/***PS3***")) iso_list[15][strlen(iso_list[15])-4]=0;
						strcat(iso_list[15], ".SFO");
						if(cellFsStat(iso_list[15], &s)!=CELL_FS_SUCCEEDED)
						{
							//iso_list[15][strlen(iso_list[15])-4]=0; strcat(iso_list[15], ".SFO");
							for(u8 n=0;n<30;n++)
							{
								if(cellFsStat((char*)"/dev_bdvd/PS3_GAME/PARAM.SFO", &s)==CELL_FS_SUCCEEDED)
								{
									int fdw;
									uint64_t msiz = 0;
									uint64_t msiz2 = 0;
									if(cellFsOpen((char*)"/dev_bdvd/PS3_GAME/PARAM.SFO", CELL_FS_O_RDONLY, &fdw, 0,0)==CELL_FS_SUCCEEDED)
									{
										cellFsLseek(fdw, 0, CELL_FS_SEEK_SET, &msiz);
										cellFsRead(fdw, (void *)&tmp_iso, 4096, &msiz);
										cellFsClose(fdw);

										if(msiz>256 && cellFsOpen(iso_list[15], CELL_FS_O_CREAT | CELL_FS_O_TRUNC | CELL_FS_O_WRONLY, &fdw, 0,0)==CELL_FS_SUCCEEDED)
										{
											cellFsWrite(fdw, (void *)&tmp_iso, msiz, &msiz2);
											cellFsClose(fdw);
											cellFsChmod(iso_list[15], 0666);

											iso_list[15][strlen(iso_list[15])-4]=0; strcat(iso_list[15], ".PNG");
											if(cellFsStat(iso_list[15], &s)!=CELL_FS_SUCCEEDED)
											{
												if(cellFsOpen((char*)"/dev_bdvd/PS3_GAME/ICON0.PNG", CELL_FS_O_RDONLY, &fdw, 0,0)==CELL_FS_SUCCEEDED)
												{
													sys_addr_t buf1=0;
													if(sys_memory_allocate((256*1024), SYS_MEMORY_PAGE_SIZE_64K, &buf1)==0)
													{	// copy_file
														char *tmp_png=(char*)buf1;
														cellFsLseek(fdw, 0, CELL_FS_SEEK_SET, &msiz);
														cellFsRead(fdw, tmp_png, (256*1024), &msiz);
														cellFsClose(fdw);

														if(msiz>256 && cellFsOpen(iso_list[15], CELL_FS_O_CREAT | CELL_FS_O_TRUNC | CELL_FS_O_WRONLY, &fdw, 0,0)==CELL_FS_SUCCEEDED)
														{
															cellFsWrite(fdw, tmp_png, msiz, &msiz2);
															cellFsClose(fdw);
															cellFsChmod(iso_list[15], 0666);
														}
														sys_memory_free(buf1);
													}
												}
											}
										}
									}
									break;
								}
								sys_timer_usleep(500000);
							}
						}
					}
				}
				goto patch;
//				return;
			}
			else
			{
				if(strstr(_path, "/PS3ISO/"))
				{
					cobra_mount_ps3_disc_image(cobra_iso_list, iso_num);
					sys_timer_usleep(2500);
					cobra_send_fake_disc_insert_event();

					{
						sprintf(iso_list[15], WMTMP "/%s", (strrchr(_path, '/')+1));
						if(iso_list[15][strlen(iso_list[15])-2]=='.') iso_list[15][strlen(iso_list[15])-2]=0;
						if(iso_list[15][strlen(iso_list[15])-4]=='.') iso_list[15][strlen(iso_list[15])-4]=0;
						strcat(iso_list[15], ".SFO");
						if(cellFsStat(iso_list[15], &s)!=CELL_FS_SUCCEEDED)
						{
							//iso_list[15][strlen(iso_list[15])-4]=0; strcat(iso_list[15], ".SFO");
							for(u8 n=0;n<5;n++)
							{
								if(cellFsStat((char*)"/dev_bdvd/PS3_GAME/PARAM.SFO", &s)==CELL_FS_SUCCEEDED)
								{
									int fdw;
									uint64_t msiz = 0;
									uint64_t msiz2 = 0;
									if(cellFsOpen((char*)"/dev_bdvd/PS3_GAME/PARAM.SFO", CELL_FS_O_RDONLY, &fdw, 0,0)==CELL_FS_SUCCEEDED)
									{
										cellFsLseek(fdw, 0, CELL_FS_SEEK_SET, &msiz);
										cellFsRead(fdw, (void *)&tmp_iso, 4096, &msiz);
										cellFsClose(fdw);

										if(msiz>256 && cellFsOpen(iso_list[15], CELL_FS_O_CREAT | CELL_FS_O_TRUNC | CELL_FS_O_WRONLY, &fdw, 0,0)==CELL_FS_SUCCEEDED)
										{
											cellFsWrite(fdw, (void *)&tmp_iso, msiz, &msiz2);
											cellFsClose(fdw);
											cellFsChmod(iso_list[15], 0666);

											iso_list[15][strlen(iso_list[15])-4]=0; strcat(iso_list[15], ".PNG");
											if(cellFsStat(iso_list[15], &s)!=CELL_FS_SUCCEEDED)
											{
												if(cellFsOpen((char*)"/dev_bdvd/PS3_GAME/ICON0.PNG", CELL_FS_O_RDONLY, &fdw, 0,0)==CELL_FS_SUCCEEDED)
												{	// copy_file
													sys_addr_t buf1=0;
													if(sys_memory_allocate((256*1024), SYS_MEMORY_PAGE_SIZE_64K, &buf1)==0)
													{
														char *tmp_png=(char*)buf1;
														cellFsLseek(fdw, 0, CELL_FS_SEEK_SET, &msiz);
														cellFsRead(fdw, tmp_png, (256*1024), &msiz);
														cellFsClose(fdw);

														if(msiz>256 && cellFsOpen(iso_list[15], CELL_FS_O_CREAT | CELL_FS_O_TRUNC | CELL_FS_O_WRONLY, &fdw, 0,0)==CELL_FS_SUCCEEDED)
														{
															cellFsWrite(fdw, tmp_png, msiz, &msiz2);
															cellFsClose(fdw);
															cellFsChmod(iso_list[15], 0666);
														}
														sys_memory_free(buf1);
													}
												}
											}
										}
									}
									break;
								}
								sys_timer_usleep(500000);
							}
						}
					}
					goto patch;
//					return;
				}
				else if(strstr(_path, "/PSPISO/") || strstr(_path, "/ISO/"))
				{
					cellFsUnlink((char*)"/dev_hdd0/game/PSPC66820/PIC1.PNG");
					cobra_unset_psp_umd();
					int result=cobra_set_psp_umd2(_path, NULL, (char*)"/dev_hdd0/tmp/psp_icon.png", 2);
					if(result==ENOTSUP || result==EABORT)
						return;
					else if(!result)
					{
						cobra_send_fake_disc_insert_event();
						return;
					}
				}
				else if(strstr(_path, "/BDISO/"))
					cobra_mount_bd_disc_image(cobra_iso_list, iso_num);
				else if(strstr(_path, "/DVDISO/"))
					cobra_mount_dvd_disc_image(cobra_iso_list, iso_num);
				else if(strstr(_path, "/PS2ISO/"))
				{
					TrackDef tracks[1];
					tracks[0].lba = 0;
					tracks[0].is_audio = 0;
					cobra_mount_ps2_disc_image(cobra_iso_list, 1, tracks, 1);
					if(webman_config->fanc) fan_control( ((webman_config->ps2temp*255)/100), 0);
				}
				else if(strstr(_path, "/PSXISO/") || strstr(_path, "/PSXGAMES/"))
				{
					if(strstr(_path, ".CUE") || strstr(_path, ".cue"))
					{
						cobra_iso_list[0][strlen(cobra_iso_list[0])-3]='B';
						cobra_iso_list[0][strlen(cobra_iso_list[0])-2]='I';
						cobra_iso_list[0][strlen(cobra_iso_list[0])-1]='N';

						struct CellFsStat s;
						if(cellFsStat(cobra_iso_list[0], &s)!=CELL_FS_SUCCEEDED)
						{
							cobra_iso_list[0][strlen(cobra_iso_list[0])-3]='b';
							cobra_iso_list[0][strlen(cobra_iso_list[0])-2]='i';
							cobra_iso_list[0][strlen(cobra_iso_list[0])-1]='n';
						}
						if(cellFsStat(cobra_iso_list[0], &s)!=CELL_FS_SUCCEEDED)
						{
							cobra_iso_list[0][strlen(cobra_iso_list[0])-3]='I';
							cobra_iso_list[0][strlen(cobra_iso_list[0])-2]='S';
							cobra_iso_list[0][strlen(cobra_iso_list[0])-1]='O';
						}
						if(cellFsStat(cobra_iso_list[0], &s)!=CELL_FS_SUCCEEDED)
						{
							cobra_iso_list[0][strlen(cobra_iso_list[0])-3]='i';
							cobra_iso_list[0][strlen(cobra_iso_list[0])-2]='s';
							cobra_iso_list[0][strlen(cobra_iso_list[0])-1]='o';
						}

						unsigned int num_tracks=0;
						int fdw;
						uint64_t msiz = 0;

						sys_addr_t buf1=0;
						if(sys_memory_allocate(65536, SYS_MEMORY_PAGE_SIZE_64K, &buf1)==0)
						{
							if(cellFsOpen(_path, CELL_FS_O_RDONLY, &fdw, 0, 0)==CELL_FS_SUCCEEDED)
							{
								char *buf=(char*)buf1;
								cellFsLseek(fdw, 0, CELL_FS_SEEK_SET, &msiz);
								cellFsRead(fdw, (void *)buf, 65500, &msiz);
								cellFsClose(fdw);
								if(msiz>10)
								{

									TrackDef tracks[32];
									tracks[0].lba = 0;
									tracks[0].is_audio = 0;

									char tcode[512];
									u8 tmin=0, tsec=0, tfrm=0;
									u8 use_pregap=0;
									u32 lp=0;

									while (lp<msiz)// get_line ( tmp_iso, 512, buf1 ) != NULL )
									{
										u8 line_found=0;
										tmp_iso[0]=0;
										for(u32 l=0; l<511; l++)
										{
											if(l>=msiz) break;
											if(lp<msiz && buf[lp] && buf[lp]!='\n' && buf[lp]!='\r')
											{
												tmp_iso[l]=buf[lp];
												tmp_iso[l+1]=0;
											}
											else
											{
												tmp_iso[l]=0;
											}
											if(buf[lp]=='\n' || buf[lp]=='\r') line_found=1;
											lp++;
											if(buf[lp]=='\n' || buf[lp]=='\r') lp++;

											if(tmp_iso[l]==0) break;
										}

										if(!line_found) break;

										if(strstr(tmp_iso, "PREGAP")) {use_pregap=1; continue;}
										if(!strstr(tmp_iso, "INDEX 01") && !strstr(tmp_iso, "INDEX 1 ")) continue;

										sprintf(tcode, "%s", strrchr(tmp_iso, ' ')+1); tcode[8]=0;
										if(strlen(tcode)!=8 || tcode[2]!=':' || tcode[5]!=':') continue;
										tmin=(tcode[0]-0x30)*10 + (tcode[1]-0x30);
										tsec=(tcode[3]-0x30)*10 + (tcode[4]-0x30);
										tfrm=(tcode[6]-0x30)*10 + (tcode[7]-0x30);
										if(use_pregap && num_tracks) tsec+=2;

										if(num_tracks) tracks[num_tracks].is_audio = 1;
										tracks[num_tracks].lba=(tmin*60 + tsec)*75 + tfrm;

										num_tracks++; if(num_tracks>=32) break;
									}

									if(!num_tracks) num_tracks++;
									cobra_mount_psx_disc_image(cobra_iso_list[0], tracks, num_tracks);
								}
							}
							else
							{
								TrackDef tracks[1];
								tracks[0].lba = 0;
								tracks[0].is_audio = 0;
								cobra_mount_psx_disc_image_iso(cobra_iso_list[0], tracks, 1);
							}

							sys_memory_free(buf1);
						}
					}
					else
					{
						TrackDef tracks[1];
						tracks[0].lba = 0;
						tracks[0].is_audio = 0;
						cobra_mount_psx_disc_image_iso(cobra_iso_list[0], tracks, 1);
					}
				}

				sys_timer_usleep(2500);
				cobra_send_fake_disc_insert_event();
			}
		}
		else
		{
			int special_mode=0;
			cobra_map_game(_path, (char*)"TEST00000", &special_mode);
		}
		return;
	}

#endif

patch:
#ifndef COBRA_ONLY

	max_mapped=0;

	if(c_firmware==0.0f) return;

	//if(!cobra_mode)
	{
        //Remove Lv2 memory protection
		if(c_firmware==4.21f)
		{
			poke_lv1(HV_START_OFFSET_421 +  0, 0x0000000000000001ULL);
			poke_lv1(HV_START_OFFSET_421 +  8, 0xe0d251b556c59f05ULL);
			poke_lv1(HV_START_OFFSET_421 + 16, 0xc232fcad552c80d7ULL);
			poke_lv1(HV_START_OFFSET_421 + 24, 0x65140cd200000000ULL);
		}
		else
		if( (c_firmware==4.30f || c_firmware==4.31f ||
			 c_firmware==4.40f || c_firmware==4.41f || c_firmware==4.46f ||
			 c_firmware==4.50f || c_firmware==4.53f || c_firmware==4.55f) )
		{
			poke_lv1(HV_START_OFFSET_430 +  0, 0x0000000000000001ULL);
			poke_lv1(HV_START_OFFSET_430 +  8, 0xe0d251b556c59f05ULL);
			poke_lv1(HV_START_OFFSET_430 + 16, 0xc232fcad552c80d7ULL);
			poke_lv1(HV_START_OFFSET_430 + 24, 0x65140cd200000000ULL);
		}
        else
		if(c_firmware==4.60f)
		{
			poke_lv1(HV_START_OFFSET_460 +  0, 0x0000000000000001ULL);
			poke_lv1(HV_START_OFFSET_460 +  8, 0xe0d251b556c59f05ULL);
			poke_lv1(HV_START_OFFSET_460 + 16, 0xc232fcad552c80d7ULL);
			poke_lv1(HV_START_OFFSET_460 + 24, 0x65140cd200000000ULL);
		}

		//if(do_eject) eject_insert(1, 0);

		if(c_firmware==4.30f || c_firmware==4.31f || c_firmware==4.40f || c_firmware==4.41f || c_firmware==4.46f ||
           c_firmware==4.50f || c_firmware==4.53f || c_firmware==4.55f || c_firmware==4.60f)
		{	// add and enable lv2 peek/poke + lv1 peek/poke
			pokeq(0x800000000000171CULL,       0x7C0802A6F8010010ULL);
			pokeq(0x800000000000171CULL +   8, 0x396000B644000022ULL);
			pokeq(0x800000000000171CULL +  16, 0x7C832378E8010010ULL);
			pokeq(0x800000000000171CULL +  24, 0x7C0803A64E800020ULL);
			pokeq(0x800000000000171CULL +  32, 0x7C0802A6F8010010ULL);
			pokeq(0x800000000000171CULL +  40, 0x396000B744000022ULL);
			pokeq(0x800000000000171CULL +  48, 0x38600000E8010010ULL);
			pokeq(0x800000000000171CULL +  56, 0x7C0803A64E800020ULL);
			pokeq(0x800000000000171CULL +  64, 0x7C0802A6F8010010ULL);
			pokeq(0x800000000000171CULL +  72, 0x7D4B537844000022ULL);
			pokeq(0x800000000000171CULL +  80, 0xE80100107C0803A6ULL);
			pokeq(0x800000000000171CULL +  88, 0x4E80002080000000ULL);
			pokeq(0x800000000000171CULL +  96, 0x0000170C80000000ULL);
			pokeq(0x800000000000171CULL + 104, 0x0000171480000000ULL);
			pokeq(0x800000000000171CULL + 112, 0x0000171C80000000ULL);
			pokeq(0x800000000000171CULL + 120, 0x0000173C80000000ULL);
			pokeq(0x800000000000171CULL + 128, 0x0000175C00000000ULL);

			if(!dex_mode)
			{
				if(c_firmware==4.30f || c_firmware==4.31f)
				{
					pokeq(0x800000000035DC10ULL, 0x8000000000001778ULL); //sc6
					pokeq(0x800000000035DC18ULL, 0x8000000000001780ULL); //sc7
					pokeq(0x800000000035DC20ULL, 0x8000000000001788ULL); //sc8
					pokeq(0x800000000035DC28ULL, 0x8000000000001790ULL); //sc9
					pokeq(0x800000000035DC30ULL, 0x8000000000001798ULL); //sc10
				}
				else
				if(c_firmware==4.40f || c_firmware==4.41f)
				{
					pokeq(0x800000000035E290ULL, 0x8000000000001778ULL); //sc6
					pokeq(0x800000000035E298ULL, 0x8000000000001780ULL); //sc7
					pokeq(0x800000000035E2A0ULL, 0x8000000000001788ULL); //sc8
					pokeq(0x800000000035E2A8ULL, 0x8000000000001790ULL); //sc9
					pokeq(0x800000000035E2B0ULL, 0x8000000000001798ULL); //sc10
				}
				else
				if(c_firmware==4.46f)
				{
					pokeq(0x800000000035E890ULL, 0x8000000000001778ULL); //sc6
					pokeq(0x800000000035E898ULL, 0x8000000000001780ULL); //sc7
					pokeq(0x800000000035E8A0ULL, 0x8000000000001788ULL); //sc8
					pokeq(0x800000000035E8A8ULL, 0x8000000000001790ULL); //sc9
					pokeq(0x800000000035E8B0ULL, 0x8000000000001798ULL); //sc10
				}
				else
				if(c_firmware==4.50f)
				{
					pokeq(0x800000000035F100ULL, 0x8000000000001778ULL); //sc6
					pokeq(0x800000000035F108ULL, 0x8000000000001780ULL); //sc7
					pokeq(0x800000000035F110ULL, 0x8000000000001788ULL); //sc8
					pokeq(0x800000000035F118ULL, 0x8000000000001790ULL); //sc9
					pokeq(0x800000000035F120ULL, 0x8000000000001798ULL); //sc10
				}
				else
				if(c_firmware==4.53f)
				{
					pokeq(0x800000000035F330ULL, 0x8000000000001778ULL); //sc6
					pokeq(0x800000000035F338ULL, 0x8000000000001780ULL); //sc7
					pokeq(0x800000000035F340ULL, 0x8000000000001788ULL); //sc8
					pokeq(0x800000000035F348ULL, 0x8000000000001790ULL); //sc9
					pokeq(0x800000000035F350ULL, 0x8000000000001798ULL); //sc10
				}
				else
				if(c_firmware==4.55f)
				{
					pokeq(0x80000000003626B0ULL, 0x8000000000001778ULL); //sc6
					pokeq(0x80000000003626B8ULL, 0x8000000000001780ULL); //sc7
					pokeq(0x80000000003626C0ULL, 0x8000000000001788ULL); //sc8
					pokeq(0x80000000003626C8ULL, 0x8000000000001790ULL); //sc9
					pokeq(0x80000000003626D0ULL, 0x8000000000001798ULL); //sc10
				}
				else
				if(c_firmware==4.60f)
				{
					pokeq(0x8000000000363A48ULL, 0x8000000000001778ULL); //sc6  363A18+8*(SC_6)=363A48
					pokeq(0x8000000000363A50ULL, 0x8000000000001780ULL); //sc7
					pokeq(0x8000000000363A58ULL, 0x8000000000001788ULL); //sc8
					pokeq(0x8000000000363A60ULL, 0x8000000000001790ULL); //sc9
					pokeq(0x8000000000363A68ULL, 0x8000000000001798ULL); //sc10
				}
			}

			else // 4.30DEX
			{
				if(c_firmware==4.30f)
				{
					pokeq(0x800000000037C098ULL, 0x8000000000001778ULL); //sc6
					pokeq(0x800000000037C0A0ULL, 0x8000000000001780ULL); //sc7
					pokeq(0x800000000037C0A8ULL, 0x8000000000001788ULL); //sc8
					pokeq(0x800000000037C0B0ULL, 0x8000000000001790ULL); //sc9
					pokeq(0x800000000037C0B8ULL, 0x8000000000001798ULL); //sc10
				}
				else if(c_firmware==4.41f)
				{
					pokeq(0x800000000037CA18ULL, 0x8000000000001778ULL); //sc6
					pokeq(0x800000000037CA20ULL, 0x8000000000001780ULL); //sc7
					pokeq(0x800000000037CA28ULL, 0x8000000000001788ULL); //sc8
					pokeq(0x800000000037CA30ULL, 0x8000000000001790ULL); //sc9
					pokeq(0x800000000037CA38ULL, 0x8000000000001798ULL); //sc10
				}
				else if(c_firmware==4.46f)
				{
					pokeq(0x800000000037D018ULL, 0x8000000000001778ULL); //sc6
					pokeq(0x800000000037D020ULL, 0x8000000000001780ULL); //sc7
					pokeq(0x800000000037D028ULL, 0x8000000000001788ULL); //sc8
					pokeq(0x800000000037D030ULL, 0x8000000000001790ULL); //sc9
					pokeq(0x800000000037D038ULL, 0x8000000000001798ULL); //sc10
				}
				else if(c_firmware==4.50f)
				{
					pokeq(0x8000000000383688ULL, 0x8000000000001778ULL); //sc6
					pokeq(0x8000000000383690ULL, 0x8000000000001780ULL); //sc7
					pokeq(0x8000000000383698ULL, 0x8000000000001788ULL); //sc8
					pokeq(0x80000000003836A0ULL, 0x8000000000001790ULL); //sc9
					pokeq(0x80000000003836A8ULL, 0x8000000000001798ULL); //sc10
				}
			}
		}
	}

	pokeq(0x8000000000000000ULL+MAP_ADDR, 0x0000000000000000ULL);
	pokeq(0x8000000000000008ULL+MAP_ADDR, 0x0000000000000000ULL);

	if(c_firmware==4.21f && !dex_mode)
	{
		SYSCALL_TABLE			= SYSCALL_TABLE_421;
	}
	else
	if(c_firmware==4.30f && !dex_mode)
	{
		SYSCALL_TABLE			= SYSCALL_TABLE_430;
	}
	else
	if(c_firmware==4.30f && dex_mode)
	{
		SYSCALL_TABLE			= SYSCALL_TABLE_430D;
	}
	else
	if(c_firmware==4.31f && !dex_mode)
	{
		SYSCALL_TABLE			= SYSCALL_TABLE_431;
	}
	else
	if(c_firmware==4.40f && !dex_mode)
	{
		SYSCALL_TABLE			= SYSCALL_TABLE_440;
	}
	else
	if(c_firmware==4.41f && !dex_mode)
	{
		SYSCALL_TABLE			= SYSCALL_TABLE_441;
	}
	else
	if(c_firmware==4.41f && dex_mode)
	{
		SYSCALL_TABLE			= SYSCALL_TABLE_441D;
	}
	else
	if(c_firmware==4.46f && !dex_mode)
	{
		SYSCALL_TABLE			= SYSCALL_TABLE_446;
	}
	else
	if(c_firmware==4.50f && !dex_mode)
	{
		SYSCALL_TABLE			= SYSCALL_TABLE_450;
	}
	else
	if(c_firmware==4.53f && !dex_mode)
	{
		SYSCALL_TABLE			= SYSCALL_TABLE_453;
	}
	else
	if(c_firmware==4.55f && !dex_mode)
	{
		SYSCALL_TABLE			= SYSCALL_TABLE_455;
	}
	else
	if(c_firmware==4.60f && !dex_mode)
	{
		SYSCALL_TABLE			= SYSCALL_TABLE_460;
	}
	else
	if(c_firmware==4.50f &&  dex_mode)
	{
		SYSCALL_TABLE			= SYSCALL_TABLE_450D;
	}
	else
	if(c_firmware==4.46f && dex_mode)
	{
		SYSCALL_TABLE			= SYSCALL_TABLE_446D;
	}
	else
	if(c_firmware==4.21f && dex_mode)
	{
		SYSCALL_TABLE			= SYSCALL_TABLE_421D;
	}

	u64 sc_600 = 0;
	u64 sc_604 = 0;
	u64 sc_142 = 0;

	if(!dex_mode)
	{
		if(c_firmware==4.21f)
		{
			pokeq(0x8000000000296264ULL, 0x4E80002038600000ULL );
			pokeq(0x800000000029626CULL, 0x7C6307B44E800020ULL ); // fix 8001003C error
			pokeq(0x8000000000057020ULL, 0x63FF003D60000000ULL ); // fix 8001003D error
			pokeq(0x80000000000570E4ULL, 0x3FE080013BE00000ULL ); // fix 8001003E error

			pokeq(0x8000000000057090ULL, 0x419E00D860000000ULL );
			pokeq(0x8000000000057098ULL, 0x2F84000448000098ULL );
			pokeq(0x800000000005AA54ULL, 0x2F83000060000000ULL ); // fix 80010009 error
			pokeq(0x800000000005AA68ULL, 0x2F83000060000000ULL ); // fix 80010019 error

			sc_600=0x33B2E0;
			sc_604=0x33B448;
			sc_142=0x2FD810;
		}
        else
		if(c_firmware==4.30f)
		{
			pokeq(0x80000000002979D8ULL, 0x4E80002038600000ULL );
			pokeq(0x80000000002979E0ULL, 0x7C6307B44E800020ULL ); // fix 8001003C error
			pokeq(0x8000000000057170ULL, 0x63FF003D60000000ULL ); // fix 8001003D error
			pokeq(0x8000000000057234ULL, 0x3FE080013BE00000ULL ); // fix 8001003E error
			pokeq(0x8000000000057238ULL, 0x3BE000004BFFFF0CULL ); // introduced by me bug

			pokeq(0x80000000000571E0ULL, 0x419E00D860000000ULL );
			pokeq(0x80000000000571E8ULL, 0x2F84000448000098ULL );
			pokeq(0x800000000005ABA4ULL, 0x2F83000060000000ULL );
			pokeq(0x800000000005ABB8ULL, 0x2F83000060000000ULL );

			sc_600=0x33D158; //35EEA0
			sc_604=0x33D2C0; //35EEC0
			sc_142=0x2FF460; //35E050
		}
        else
		if(c_firmware==4.31f)
		{
			pokeq(0x80000000002979E0ULL, 0x4E80002038600000ULL );
			pokeq(0x80000000002979E8ULL, 0x7C6307B44E800020ULL ); // fix 8001003C error
			pokeq(0x8000000000057174ULL, 0x63FF003D60000000ULL ); // fix 8001003D error
			pokeq(0x800000000005723CULL, 0x3FE080013BE00000ULL ); // fix 8001003E error

			pokeq(0x80000000000571E8ULL, 0x600000002F840004ULL );
			pokeq(0x80000000000571F0ULL, 0x48000098E8629870ULL );
			pokeq(0x800000000005ABACULL, 0x60000000E8610188ULL );
			pokeq(0x800000000005ABA0ULL, 0x600000005463063EULL );

			sc_600=0x33D168;
			sc_604=0x33D2D0;
			sc_142=0x2FF470;
		}
        else
		if(c_firmware==4.40f)
		{
			pokeq(0x8000000000296DE8ULL, 0x4E80002038600000ULL );
			pokeq(0x8000000000296DF0ULL, 0x7C6307B44E800020ULL ); // fix 8001003C error
			pokeq(0x80000000000560BCULL, 0x63FF003D60000000ULL ); // fix 8001003D error
			pokeq(0x8000000000056180ULL, 0x3FE080013BE00000ULL ); // fix 8001003E error +C4

			pokeq(0x800000000005612CULL, 0x419E00D860000000ULL );
			pokeq(0x8000000000056134ULL, 0x2F84000448000098ULL );
			pokeq(0x8000000000059AF0ULL, 0x2F83000060000000ULL );
			pokeq(0x8000000000059B04ULL, 0x2F83000060000000ULL );

			sc_600=0x33D720;
			sc_604=0x33D888;
			sc_142=0x2FF9E0;
		}
        else
		if(c_firmware==4.41f)
		{
			pokeq(0x8000000000296DF0ULL, 0x4E80002038600000ULL ); // fix 8001003C error
			pokeq(0x8000000000296DF8ULL, 0x7C6307B44E800020ULL ); // fix 8001003C error
			pokeq(0x80000000000560C0ULL, 0x63FF003D60000000ULL ); // fix 8001003D error
			pokeq(0x8000000000056184ULL, 0x3FE080013BE00000ULL ); // fix 8001003E error

			pokeq(0x8000000000056130ULL, 0x419E00D860000000ULL );
			pokeq(0x8000000000056138ULL, 0x2F84000448000098ULL );
			pokeq(0x8000000000059AF4ULL, 0x2F83000060000000ULL );
			pokeq(0x8000000000059B08ULL, 0x2F83000060000000ULL );

			sc_600=0x33D730;
			sc_604=0x33D898;
			sc_142=0x2FF9F0;
		}
        else
		if(c_firmware==4.46f)
		{
			pokeq(0x8000000000297310ULL, 0x4E80002038600000ULL ); // fix 8001003C error
			pokeq(0x8000000000297318ULL, 0x7C6307B44E800020ULL ); // fix 8001003C error
			pokeq(0x80000000000560C0ULL, 0x63FF003D60000000ULL ); // fix 8001003D error
			pokeq(0x8000000000056184ULL, 0x3FE080013BE00000ULL ); // fix 8001003E error

			pokeq(0x8000000000056130ULL, 0x419E00D860000000ULL );
			pokeq(0x8000000000056138ULL, 0x2F84000448000098ULL );
			pokeq(0x8000000000059AF4ULL, 0x2F83000060000000ULL );
			pokeq(0x8000000000059B08ULL, 0x2F83000060000000ULL );

			sc_600=0x33DD40;
			sc_604=0x33DEA8;
			sc_142=0x2FFF58;
		}
        else
		if(c_firmware==4.50f)
		{
			pokeq(0x800000000026F61CULL, 0x4E80002038600000ULL ); // fix 8001003C error
			pokeq(0x800000000026F624ULL, 0x7C6307B44E800020ULL ); // fix 8001003C error
			pokeq(0x80000000000560BCULL, 0x63FF003D60000000ULL ); // fix 8001003D error
			pokeq(0x8000000000056180ULL, 0x3FE080013BE00000ULL ); // fix 8001003E error

			pokeq(0x800000000005612CULL, 0x419E00D860000000ULL );
			pokeq(0x8000000000056134ULL, 0x2F84000448000098ULL );
			pokeq(0x8000000000059AF0ULL, 0x2F83000060000000ULL );
			pokeq(0x8000000000059B04ULL, 0x2F83000060000000ULL );

			sc_600=0x33C180;
			sc_604=0x33C2E8;
			sc_142=0x302100;
		}
        else
		if(c_firmware==4.53f)
		{
			pokeq(0x800000000026F7F0ULL, 0x4E80002038600000ULL ); // fix 8001003C error
			pokeq(0x800000000026F7F8ULL, 0x7C6307B44E800020ULL ); // fix 8001003C error
			pokeq(0x80000000000560C0ULL, 0x63FF003D60000000ULL ); // fix 8001003D error
			pokeq(0x8000000000056184ULL, 0x3FE080013BE00000ULL ); // fix 8001003E error

			pokeq(0x8000000000056130ULL, 0x419E00D860000000ULL );
			pokeq(0x8000000000056138ULL, 0x2F84000448000098ULL );
			pokeq(0x8000000000059AF4ULL, 0x2F83000060000000ULL );
			pokeq(0x8000000000059B08ULL, 0x2F83000060000000ULL );

			sc_600=0x33C308;
			sc_604=0x33C470;
			sc_142=0x302108;
		}
        else
		if(c_firmware==4.55f)
		{
			pokeq(0x800000000027103CULL, 0x4E80002038600000ULL ); // fix 8001003C error
			pokeq(0x8000000000271044ULL, 0x7C6307B44E800020ULL ); // fix 8001003C error
			pokeq(0x8000000000056380ULL, 0x63FF003D60000000ULL ); // fix 8001003D error
			pokeq(0x8000000000056444ULL, 0x3FE080013BE00000ULL ); // fix 8001003E error

			pokeq(0x80000000000563F0ULL, 0x419E00D860000000ULL );
			pokeq(0x80000000000563F8ULL, 0x2F84000448000098ULL );
			pokeq(0x800000000005A2ECULL, 0x2F83000060000000ULL );
			pokeq(0x800000000005A300ULL, 0x2F83000060000000ULL );

			sc_600=0x33F5C8;
			sc_604=0x33F730;
			sc_142=0x3051D0;
		}
        else
		if(c_firmware==4.60f)
		{
			pokeq(0x80000000002925D8ULL, 0x4E80002038600000ULL ); // fix 8001003C error
			pokeq(0x80000000002925E0ULL, 0x7C6307B44E800020ULL ); // fix 8001003C error
			pokeq(0x8000000000056588ULL, 0x63FF003D60000000ULL ); // fix 8001003D error
			pokeq(0x800000000005664CULL, 0x3FE080013BE00000ULL ); // fix 8001003E error

			pokeq(0x80000000000565F8ULL, 0x419E00D860000000ULL );
			pokeq(0x8000000000056600ULL, 0x2F84000448000098ULL );
			pokeq(0x800000000005A654ULL, 0x2F83000060000000ULL );
			pokeq(0x800000000005A668ULL, 0x2F83000060000000ULL );

			pokeq(0x8000000000055C58ULL, 0x386000004E800020ULL ); // fix 0x8001002B error (found by @smhabib)
			lv2poke32(0x8000000000058DACULL, 0x60000000);         // fix 0x80010017 error (found by @smhabib)

			sc_600=0x340630; //0x363A18 + 600*8 = 00364CD8 -> 80 00 00 00 00 34 06 30
			sc_604=0x340798; //0x363A18 + 604*8 = 00364CF8 -> 80 00 00 00 00 34 07 98
			sc_142=0x306478; //0x363A18 + 142*8 = 00363E88 -> 80 00 00 00 00 30 64 78
		}
    }

	else
	{ //DEX

		if(c_firmware==4.21f)
		{
			pokeq(0x800000000029C8C0ULL, 0x4E80002038600000ULL );
			pokeq(0x800000000029C8C8ULL, 0x7C6307B44E800020ULL ); // fix 8001003C error
			pokeq(0x800000000005A938ULL, 0x63FF003D60000000ULL ); // fix 8001003D error
			pokeq(0x800000000005A9FCULL, 0x3FE080013BE00000ULL ); // fix 8001003E error

			pokeq(0x800000000005A9A8ULL, 0x419E00D860000000ULL );
			pokeq(0x800000000005A9B0ULL, 0x2F84000448000098ULL );
			pokeq(0x800000000005E36CULL, 0x2F83000060000000ULL );
			pokeq(0x800000000005E380ULL, 0x2F83000060000000ULL );

			sc_600=0x3583F8;
			sc_604=0x3584D0;
			sc_142=0x318BA0;
		}
		else
		if(c_firmware==4.30f)
		{
			pokeq(0x800000000029E034ULL, 0x4E80002038600000ULL );
			pokeq(0x800000000029E03CULL, 0x7C6307B44E800020ULL ); // fix 8001003C error
			pokeq(0x800000000005AA88ULL, 0x63FF003D60000000ULL ); // fix 8001003D error
			pokeq(0x800000000005AB4CULL, 0x3FE080013BE00000ULL ); // fix 8001003E error

			pokeq(0x800000000005AAF8ULL, 0x419E00D860000000ULL );
			pokeq(0x800000000005AB00ULL, 0x2F84000448000098ULL );
			pokeq(0x800000000005E4BCULL, 0x2F83000060000000ULL );
			pokeq(0x800000000005E4D0ULL, 0x2F83000060000000ULL );

			sc_600=0x35A220;
			sc_604=0x35A2F8;
			sc_142=0x31A7A0;
		}
		else
		if(c_firmware==4.41f)
		{
			pokeq(0x800000000029D44CULL, 0x4E80002038600000ULL );
			pokeq(0x800000000029D454ULL, 0x7C6307B44E800020ULL ); // fix 8001003C error
			pokeq(0x80000000000599D8ULL, 0x63FF003D60000000ULL ); // fix 8001003D error
			pokeq(0x8000000000059A9CULL, 0x3FE080013BE00000ULL ); // fix 8001003E error

			pokeq(0x8000000000059A48ULL, 0x419E00D860000000ULL );
			pokeq(0x8000000000059A50ULL, 0x2F84000448000098ULL );
			pokeq(0x800000000005D40CULL, 0x2F83000060000000ULL );
			pokeq(0x800000000005D420ULL, 0x2F83000060000000ULL );

			sc_600=0x35AB40;
			sc_604=0x35AC18;
			sc_142=0x31B060;
		}
		else
		if(c_firmware==4.46f)
		{
			pokeq(0x800000000029D96CULL, 0x4E80002038600000ULL ); // fix 8001003C error
			pokeq(0x800000000029D974ULL, 0x7C6307B44E800020ULL ); // fix 8001003C error
			pokeq(0x80000000000599D8ULL, 0x63FF003D60000000ULL ); // fix 8001003D error
			pokeq(0x8000000000059A9CULL, 0x3FE080013BE00000ULL ); // fix 8001003E error

			pokeq(0x8000000000059A48ULL, 0x419E00D860000000ULL );
			pokeq(0x8000000000059A50ULL, 0x2F84000448000098ULL );
			pokeq(0x800000000005D40CULL, 0x2F83000060000000ULL );
			pokeq(0x800000000005D420ULL, 0x2F83000060000000ULL );

			sc_600=0x35B150;
			sc_604=0x35B228;
			sc_142=0x31B5C8;
		}
		else
		if(c_firmware==4.50f)
		{
			pokeq(0x8000000000275D38ULL, 0x4E80002038600000ULL ); // fix 8001003C error
			pokeq(0x8000000000275D40ULL, 0x7C6307B44E800020ULL ); // fix 8001003C error
			pokeq(0x8000000000059A8CULL, 0x63FF003D60000000ULL ); // fix 8001003D error
			pokeq(0x8000000000059B50ULL, 0x3FE080013BE00000ULL ); // fix 8001003E error

			pokeq(0x8000000000059AFCULL, 0x419E00D860000000ULL );
			pokeq(0x8000000000059B04ULL, 0x2F84000448000098ULL );
			pokeq(0x800000000005D4C0ULL, 0x2F83000060000000ULL );
			pokeq(0x800000000005D4D4ULL, 0x2F83000060000000ULL );

			sc_600=0x35EA90;
			sc_604=0x35EB68;
			sc_142=0x322B38;
		}
	}

	if(cobra_mode) return;


	// restore syscall table
	u64 sc_null = peekq(SYSCALL_PTR( 0));
	if(sc_null == peekq(SYSCALL_PTR(79)))
	{
		pokeq(SYSCALL_PTR(  35), sc_null);
		pokeq(SYSCALL_PTR(  36), sc_null);
		pokeq(SYSCALL_PTR(  37), sc_null);
		//pokeq(SYSCALL_PTR(1023), sc_null);

		if(sc_600)
		{
			sc_600|=0x8000000000000000ULL;
			sc_604|=0x8000000000000000ULL;
			sc_142|=0x8000000000000000ULL;

			pokeq(SYSCALL_PTR(600), sc_600); // sys_storage_open 600
			pokeq(SYSCALL_PTR(604), sc_604); // sys_storage_send_device_cmd 604
			pokeq(SYSCALL_PTR(142), sc_142); // sys_timer_sleep 142
		}
	}

	pokeq(0x80000000007FD000ULL, 0);


	u64 base_addr=0;
	u64 open_hook=0;

	if(c_firmware==4.21f && !dex_mode)
	{
		base_addr=0x2D0C98;
		open_hook=0x2C2558;
	}
	else if(c_firmware==4.21f && dex_mode)
	{
		base_addr=0x2EB418;
		open_hook=0x2D9718;
	}
	else if(c_firmware==4.30f && !dex_mode)
	{
		base_addr=0x2D2418;
		open_hook=0x2C3CD4;
	}
	else if(c_firmware==4.30f && dex_mode)
	{
		base_addr=0x2ECB48;
		open_hook=0x2DAE4C;
	}
	else if(c_firmware==4.31f && !dex_mode)
	{
		base_addr=0x2D2428;
		open_hook=0x2C3CE0;
	}
	else if(c_firmware==4.40f && !dex_mode)
	{
		base_addr=0x2D29A8;
		open_hook=0x2C4284;
	}
	else if(c_firmware==4.41f && !dex_mode)
	{
		base_addr=0x2D29B8;
		open_hook=0x2C4290;
	}
	else if(c_firmware==4.41f && dex_mode)
	{
		base_addr=0x2ED418;
		open_hook=0x2DB73C;
	}
	else if(c_firmware==4.46f && !dex_mode)
	{
		base_addr=0x2D2ED8;
		open_hook=0x2C47B0;
	}
	else if(c_firmware==4.50f && !dex_mode)
	{
		base_addr=0x2D4CB8;
		open_hook=0x29DD20;
	}
	else if(c_firmware==4.53f && !dex_mode)
	{
		base_addr=0x2D4E48;
		open_hook=0x29DEF8;
	}
	else if(c_firmware==4.55f && !dex_mode)
	{
		base_addr=0x2D7660;
		open_hook=0x29F748;
	}
	else if(c_firmware==4.60f && !dex_mode)
	{
		base_addr=0x2D88D0;
		open_hook=0x2A02BC;
	}
	else if(c_firmware==4.46f && dex_mode)
	{
		base_addr=0x2ED938;
		open_hook=0x2DBC5C;
	}
	else if(c_firmware==4.50f && dex_mode)
	{
		base_addr=0x2F4778;
		open_hook=0x2B81E8;
	}
	else return;

	base_addr	|=0x8000000000000000ULL;
	open_hook	|=0x8000000000000000ULL;

	// disable mM path table
	pokeq(0x8000000000000000ULL+MAP_ADDR, 0x0000000000000000ULL);
	pokeq(0x8000000000000008ULL+MAP_ADDR, 0x0000000000000000ULL);

	// disable Iris path table
	pokeq(0x80000000007FD000ULL,		  0x0000000000000000ULL);

	// restore hook used by all payloads)
	pokeq(open_hook + 0x00, 0xF821FF617C0802A6ULL);
	pokeq(open_hook + 0x08, 0xFB810080FBA10088ULL);
	pokeq(open_hook + 0x10, 0xFBE10098FB410070ULL);
	pokeq(open_hook + 0x18, 0xFB610078F80100B0ULL);
	pokeq(open_hook + 0x20, 0x7C9C23787C7D1B78ULL);

	// poke mM payload
	pokeq(base_addr + 0x00, 0x7C7D1B783B600001ULL);
	pokeq(base_addr + 0x08, 0x7B7BF806637B0000ULL | MAP_ADDR);
	pokeq(base_addr + 0x10, 0xEB5B00002C1A0000ULL);
	pokeq(base_addr + 0x18, 0x4D820020EBFB0008ULL);
	pokeq(base_addr + 0x20, 0xE8BA00002C050000ULL);
	pokeq(base_addr + 0x28, 0x418200CC7FA3EB78ULL);
	pokeq(base_addr + 0x30, 0xE89A001089640000ULL);
	pokeq(base_addr + 0x38, 0x892300005560063EULL);
	pokeq(base_addr + 0x40, 0x7F895800409E0040ULL);
	pokeq(base_addr + 0x48, 0x2F8000007CA903A6ULL);
	pokeq(base_addr + 0x50, 0x409E002448000030ULL);
	pokeq(base_addr + 0x58, 0x8964000089230000ULL);
	pokeq(base_addr + 0x60, 0x5560063E7F895800ULL);
	pokeq(base_addr + 0x68, 0x2F000000409E0018ULL);
	pokeq(base_addr + 0x70, 0x419A001438630001ULL);
	pokeq(base_addr + 0x78, 0x388400014200FFDCULL);
	pokeq(base_addr + 0x80, 0x4800000C3B5A0020ULL);
	pokeq(base_addr + 0x88, 0x4BFFFF98E89A0018ULL);
	pokeq(base_addr + 0x90, 0x7FE3FB7888040000ULL);
	pokeq(base_addr + 0x98, 0x2F80000098030000ULL);
	pokeq(base_addr + 0xA0, 0x419E00187C691B78ULL);
	pokeq(base_addr + 0xA8, 0x8C0400012F800000ULL);
	pokeq(base_addr + 0xB0, 0x9C090001409EFFF4ULL);
	pokeq(base_addr + 0xB8, 0xE8BA00087C632A14ULL);
	pokeq(base_addr + 0xC0, 0x7FA4EB78E8BA0000ULL);
	pokeq(base_addr + 0xC8, 0x7C842A1488040000ULL);
	pokeq(base_addr + 0xD0, 0x2F80000098030000ULL);
	pokeq(base_addr + 0xD8, 0x419E00187C691B78ULL);
	pokeq(base_addr + 0xE0, 0x8C0400012F800000ULL);
	pokeq(base_addr + 0xE8, 0x9C090001409EFFF4ULL);
	pokeq(base_addr + 0xF0, 0x7FFDFB787FA3EB78ULL);
	pokeq(base_addr + 0xF8, 0x4E8000204D4D504CULL); //blr + "MMPL"

	pokeq(MAP_BASE  + 0x00, 0x0000000000000000ULL);
	pokeq(MAP_BASE  + 0x08, 0x0000000000000000ULL);
	pokeq(MAP_BASE  + 0x10, 0x8000000000000000ULL);
	pokeq(MAP_BASE  + 0x18, 0x8000000000000000ULL);

	pokeq(0x8000000000000000ULL+MAP_ADDR, MAP_BASE);
	pokeq(0x8000000000000008ULL+MAP_ADDR, 0x80000000007FDBE0ULL);

	pokeq(open_hook + 0x20, (0x7C9C237848000001ULL | (base_addr-open_hook-0x24)));


	char path[strlen(_path)+4];
	sprintf(path, "%s", _path);

	char expplg[128];
	char app_sys[128];
	struct CellFsStat buf2;

	sprintf(app_sys, MM_ROOT_STD "/sys");
	if(cellFsStat(app_sys, &buf2)!=CELL_FS_SUCCEEDED)
		sprintf(app_sys, MM_ROOT_STL "/sys");
	if(cellFsStat(app_sys, &buf2)!=CELL_FS_SUCCEEDED)
		sprintf(app_sys, MM_ROOT_SSTL "/sys");

	sprintf(expplg, "%s/ILFS0_000.BIN", app_sys);
	cellFsUnlink(expplg);

	char libfs_new[0x8e07] = {0x53, 0x43, 0x45, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x1C, 0x00, 0x01, 0x00, 0x00, 0x02, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x05, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0xA5, 0xB0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x90, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xD0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0xE0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0xF0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x70, 0x00, 0x00, 0x40, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x04, 0x00, 0x04, 0x00, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7F, 0x45, 0x4C, 0x46, 0x02, 0x02, 0x01, 0x66, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xA4, 0x00, 0x15, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x38, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x40, 0x00, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xF0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xF9, 0xE0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0x48, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0x48, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x01, 0x00, 0x60, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFE, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x17, 0x2C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xA2, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x70, 0x00, 0x00, 0xA4, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x16, 0x70, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x8F, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x70, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x64, 0x09, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x6A, 0xB0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x68, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x73, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1A, 0xE7, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x30, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x62, 0x7C, 0xB1, 0x80, 0x8A, 0xB9, 0x38, 0xE3, 0x2C, 0x8C, 0x09, 0x17, 0x08, 0x72, 0x6A, 0x57, 0x9E, 0x25, 0x86, 0xE4, 0xB6, 0x2D, 0xD5, 0x82, 0x41, 0xC9, 0x68, 0x8F, 0x11, 0xBF, 0xA9, 0x6E, 0xEB, 0x0F, 0xC6, 0xC1, 0x2D, 0x29, 0xD2, 0xC3, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xB2, 0x51, 0xBE, 0x13, 0x27, 0x89, 0xB4, 0x2C, 0xE1, 0x3D, 0x0A, 0x4A, 0x8A, 0x59, 0xCD, 0x61, 0x6B, 0x12, 0x8D, 0xB7, 0xDA, 0x11, 0x5F, 0x38, 0x1F, 0x64, 0x82, 0xE4, 0x2B, 0x69, 0x64, 0x06, 0x02, 0x2E, 0x4A, 0x33, 0xA3, 0xE9, 0xF6, 0xBB, 0xA8, 0x0F, 0x7F, 0x90, 0xC1, 0x8B, 0xA1, 0xB3, 0x68, 0x00, 0x73, 0xF4, 0xBF, 0xF0, 0x91, 0xCE, 0xC4, 0x5F, 0xB8, 0x65, 0xF3, 0x14, 0x68, 0x08, 0x76, 0x66, 0xDA, 0x40, 0xBD, 0xB4, 0x86, 0x48, 0x38, 0xB7, 0x1C, 0x7D, 0xA0, 0xD8, 0x57, 0x3F, 0xCF, 0xF4, 0x54, 0xC9, 0x56, 0x7A, 0x03, 0x7E, 0x58, 0x27, 0xFE, 0x1A, 0x42, 0x88, 0xDA, 0x34, 0xA6, 0x57, 0x1D, 0x1E, 0x83, 0x0D, 0x01, 0x98, 0x17, 0x0F, 0x60, 0x81, 0x32, 0xE8, 0x27, 0xD5, 0x4C, 0x6D, 0x8F, 0xA7, 0xAC, 0x1D, 0xD3, 0xAF, 0xC5, 0xB2, 0xC5, 0x08, 0xAE, 0x19, 0x23, 0xB7, 0xDC, 0xFD, 0x1A, 0xE5, 0xAD, 0xBF, 0xAB, 0xE5, 0x0A, 0x71, 0x11, 0xC7, 0xFA, 0xA9, 0x92, 0x8F, 0x07, 0x66, 0x1D, 0x72, 0xEE, 0x0A, 0x62, 0x29, 0xEF, 0xF9, 0xFC, 0x92, 0x6C, 0x89, 0xBB, 0x67, 0x16, 0x0A, 0x8C, 0x70, 0x5C, 0x23, 0x95, 0x24, 0x32, 0x56, 0x02, 0xC8, 0x5A, 0x2A, 0x45, 0x5E, 0xED, 0x23, 0x25, 0x60, 0x50, 0x48, 0x88, 0xD5, 0x65, 0xF3, 0x93, 0x2C, 0x0C, 0x55, 0xD7, 0x9B, 0xB5, 0x79, 0xC1, 0x64, 0x76, 0xC6, 0xB0, 0xE0, 0x67, 0x43, 0xFC, 0xBB, 0xDC, 0x15, 0x8E, 0x39, 0x12, 0x24, 0x4F, 0x13, 0xAE, 0x80, 0xD6, 0x29, 0x7F, 0x23, 0x9E, 0x2D, 0x7F, 0x3C, 0xAE, 0xD4, 0xD3, 0x0C, 0x44, 0xDA, 0x5B, 0xD1, 0xE8, 0xC1, 0xB9, 0x95, 0x67, 0x0D, 0x4F, 0xE9, 0x51, 0xCF, 0x68, 0xF7, 0xB7, 0xCE, 0x04, 0x71, 0xCB, 0x9D, 0x7C, 0xEC, 0x03, 0x61, 0x86, 0x6C, 0x2D, 0x85, 0xEB, 0x5F, 0x4C, 0x3C, 0xD2, 0x6F, 0xAA, 0x77, 0xA8, 0xA6, 0x1F, 0x0F, 0x66, 0x69, 0xA4, 0xD1, 0x10, 0x0E, 0x25, 0xB9, 0x9B, 0xB3, 0x55, 0x7F, 0x8B, 0x6C, 0xCF, 0xB4, 0xB7, 0xBB, 0xC4, 0x21, 0xDB, 0x90, 0xED, 0x39, 0x2F, 0x10, 0xAC, 0x69, 0xF5, 0x07, 0x49, 0xF0, 0xBD, 0x62, 0x9E, 0xD9, 0xD2, 0x30, 0x48, 0x48, 0xB2, 0xFF, 0xE3, 0x98, 0x6E, 0x29, 0x93, 0x3B, 0x69, 0xED, 0xBF, 0xCB, 0x6F, 0xD4, 0x83, 0x39, 0xDE, 0x6F, 0x16, 0x6B, 0x1F, 0x20, 0x08, 0x74, 0xD6, 0x17, 0x8B, 0x2F, 0x7C, 0x0C, 0x31, 0xA1, 0x9E, 0xAE, 0xCB, 0x1A, 0x9D, 0x5F, 0x52, 0x41, 0x20, 0x76, 0xDF, 0x8B, 0xC7, 0xEA, 0xDB, 0x7E, 0xCD, 0xF6, 0xEF, 0x90, 0x13, 0x82, 0x30, 0x3D, 0xB7, 0x7E, 0x0D, 0xBC, 0xAA, 0x96, 0x8E, 0x6C, 0x35, 0x6D, 0xA5, 0x5B, 0xF7, 0xAA, 0x1C, 0x16, 0xE3, 0xB2, 0x6E, 0xFD, 0xC0, 0x1A, 0x41, 0x87, 0xDF, 0x5C, 0x68, 0x05, 0xDA, 0x65, 0xCB, 0x8B, 0x61, 0x05, 0x70, 0x56, 0x45, 0xBB, 0xDF, 0x87, 0x5C, 0x13, 0x68, 0x26, 0xC3, 0x56, 0xE3, 0x9B, 0xF2, 0x44, 0x0F, 0x2C, 0x78, 0xA0, 0x3C, 0xDF, 0x88, 0xEA, 0xC5, 0x64, 0x22, 0xE0, 0x69, 0x9F, 0xDA, 0xDD, 0x89, 0x50, 0x5F, 0xB9, 0x63, 0x99, 0xFC, 0xB2, 0xCB, 0x82, 0x0B, 0xC5, 0x1C, 0xF1, 0xE5, 0x57, 0x26, 0x3B, 0x6E, 0xF0, 0xFB, 0x90, 0x35, 0xE0, 0xD7, 0x92, 0x0F, 0x6F, 0x08, 0x56, 0xD9, 0x70, 0x27, 0x9B, 0x2E, 0x71, 0xDC, 0x7A, 0x90, 0x8C, 0x91, 0x38, 0x12, 0xD7, 0x23, 0x3E, 0xDA, 0xF9, 0x7D, 0x63, 0x11, 0x31, 0x6C, 0xC1, 0x5F, 0xC3, 0x4C, 0x8F, 0xC0, 0xF1, 0x77, 0x4E, 0x7B, 0x3D, 0x81, 0xCC, 0x46, 0x4F, 0xA7, 0x16, 0x2B, 0x1E, 0x9E, 0x74, 0x83, 0x81, 0xC8, 0x8D, 0x0E, 0x55, 0x22, 0x06, 0x01, 0xA8, 0x22, 0x7E, 0x41, 0x03, 0xAD, 0x71, 0x69, 0x7B, 0x1B, 0x02, 0xD8, 0x7C, 0x9E, 0x1F, 0xF8, 0xCC, 0x80, 0xBE, 0xF6, 0x55, 0x09, 0x21, 0xBA, 0xB7, 0x67, 0x89, 0x76, 0x71, 0x8A, 0x07, 0xEE, 0x1D, 0xF3, 0x38, 0x66, 0xE3, 0x91, 0x99, 0xBA, 0x04, 0x0D, 0x77, 0xDA, 0xF4, 0x4E, 0xB9, 0x24, 0xEE, 0x63, 0x86, 0xB9, 0x7C, 0x05, 0x80, 0x25, 0x5B, 0x0C, 0x0A, 0xF0, 0xF1, 0x50, 0xB7, 0x5C, 0x70, 0xAF, 0xD4, 0x15, 0x1E, 0x09, 0xD5, 0xD9, 0xA1, 0x61, 0x58, 0x67, 0x51, 0x7E, 0x73, 0x11, 0x1A, 0x3B, 0xD7, 0x3B, 0xF9, 0x41, 0x2B, 0xBA, 0x20, 0x91, 0xD2, 0x39, 0x5C, 0x1B, 0x49, 0x2D, 0xE9, 0x10, 0x32, 0x7D, 0x4E, 0x70, 0x7C, 0x24, 0x64, 0xA7, 0x5C, 0x15, 0x64, 0x60, 0x2D, 0xC3, 0xE5, 0xB3, 0x90, 0x3C, 0xDF, 0xAF, 0x32, 0xF5, 0x8B, 0x71, 0xEA, 0x5D, 0xC3, 0x46, 0x64, 0xCC, 0x80, 0x5F, 0x34, 0xB7, 0x58, 0x4B, 0xF9, 0x07, 0x26, 0xB5, 0x95, 0x62, 0xC5, 0x32, 0x38, 0xDA, 0x3B, 0x1F, 0xAD, 0xAF, 0x70, 0xA4, 0xC4, 0x1F, 0x6A, 0xA4, 0x97, 0x1E, 0x5F, 0xBA, 0x71, 0xC5, 0xCE, 0x01, 0x39, 0x8A, 0x48, 0x95, 0xD7, 0x10, 0x80, 0x23, 0x32, 0xDA, 0x1B, 0xB8, 0xA5, 0xD8, 0xE3, 0xFF, 0x7E, 0x38, 0xE3, 0x1A, 0x04, 0x56, 0x92, 0x39, 0x1A, 0x0D, 0x7A, 0xB9, 0xCA, 0xCA, 0xCA, 0x47, 0x9A, 0x3F, 0x3F, 0xEB, 0x79, 0x54, 0x14, 0x55, 0x59, 0x58, 0x5C, 0x7A, 0xC7, 0x8E, 0x5F, 0x71, 0x38, 0xAD, 0xA3, 0xB8, 0xFA, 0x0E, 0x57, 0xA5, 0x7E, 0x58, 0x2F, 0x6B, 0x93, 0x11, 0x8C, 0xF0, 0xB7, 0x84, 0xB6, 0x31, 0xC8, 0x1A, 0x67, 0x0B, 0x4C, 0xDD, 0x6A, 0xA3, 0xC1, 0x38, 0x15, 0x0B, 0x38, 0x2E, 0xD1, 0x61, 0x5E, 0x9A, 0xE8, 0x99, 0x21, 0xF0, 0x5C, 0x15, 0x77, 0x90, 0x7E, 0xCF, 0x91, 0xA4, 0x0E, 0xD1, 0x6B, 0x35, 0x71, 0x33, 0x0F, 0x04, 0xA5, 0xE0, 0xAD, 0x71, 0x84, 0x07, 0x4E, 0x76, 0xB2, 0x0D, 0x71, 0x49, 0x89, 0x1B, 0x94, 0x27, 0xB8, 0x6A, 0xAD, 0x36, 0x2A, 0x61, 0x73, 0x99, 0x41, 0x75, 0x19, 0xCC, 0xA7, 0x42, 0x22, 0xFB, 0xF4, 0xB5, 0xCC, 0x68, 0x06, 0xC7, 0xC1, 0x7E, 0xB3, 0x8D, 0x49, 0xD5, 0x41, 0xA7, 0xEC, 0xB7, 0x07, 0x02, 0x3B, 0xB9, 0xDB, 0xCA, 0x53, 0x04, 0xFB, 0x64, 0xB9, 0xE8, 0x82, 0xF4, 0x3C, 0x58, 0x13, 0x60, 0x0D, 0x97, 0x80, 0x25, 0xF5, 0x13, 0xA7, 0x1B, 0x09, 0x35, 0xE5, 0xDD, 0x9A, 0x56, 0x79, 0xB2, 0xA3, 0xBA, 0xC9, 0xFB, 0xF2, 0xF6, 0x10, 0x47, 0x22, 0xF2, 0xA7, 0x94, 0xA9, 0x06, 0xB8, 0x68, 0x7B, 0xB6, 0x39, 0xF0, 0x57, 0x36, 0x78, 0x1E, 0x8D, 0x06, 0x61, 0xC5, 0x99, 0x1C, 0x9E, 0x13, 0x25, 0x1A, 0x2E, 0x9B, 0xB4, 0x51, 0x9B, 0x52, 0x49, 0xFD, 0x7E, 0x30, 0x55, 0xB0, 0x8D, 0xC6, 0x1C, 0x69, 0xA7, 0x62, 0x26, 0x8F, 0x04, 0x68, 0x0A, 0x07, 0xFB, 0xAA, 0x7A, 0x0B, 0x93, 0xF3, 0x4A, 0xF8, 0xA4, 0x53, 0xC7, 0xC0, 0xBB, 0xEF, 0x26, 0xF7, 0xAB, 0x97, 0x51, 0x63, 0x36, 0x8E, 0xCC, 0x70, 0x36, 0xFD, 0x3E, 0x23, 0xB2, 0x09, 0x37, 0xBD, 0x00, 0x5D, 0x04, 0x81, 0x5D, 0x2F, 0x91, 0x31, 0x45, 0xCD, 0xE7, 0xFB, 0x9D, 0x29, 0xE7, 0x1F, 0xD2, 0xD4, 0x86, 0xF2, 0x01, 0x26, 0xFC, 0x5F, 0x76, 0xE7, 0x03, 0x97, 0x26, 0xE4, 0xE4, 0x41, 0xFC, 0xC6, 0xE6, 0x0C, 0x28, 0xA8, 0x4C, 0xD0, 0x11, 0xDA, 0xA8, 0x90, 0x01, 0x68, 0x7B, 0xDC, 0x41, 0xF7, 0x5D, 0x0F, 0x71, 0xB9, 0x37, 0x40, 0x04, 0x9D, 0xF2, 0x72, 0xA1, 0xAA, 0x66, 0x4E, 0xC2, 0x7C, 0x6F, 0x44, 0x4C, 0x54, 0x37, 0x93, 0x1E, 0x43, 0x05, 0x82, 0xD2, 0x8E, 0x1E, 0xD3, 0x31, 0xED, 0x19, 0x62, 0xF1, 0xE3, 0x92, 0x1E, 0xB7, 0xB1, 0x3F, 0xB0, 0x4B, 0xB9, 0x4A, 0xDC, 0x98, 0x02, 0xF6, 0x3D, 0x09, 0x45, 0x06, 0xDE, 0xD9, 0x08, 0xC3, 0xEC, 0xDD, 0xC8, 0xF9, 0x98, 0xEE, 0x33, 0x50, 0x2B, 0xEF, 0x97, 0x0A, 0xE0, 0xD2, 0xD1, 0x2F, 0xC1, 0xA1, 0x1D, 0x7B, 0x06, 0xE5, 0xD5, 0xCA, 0xFB, 0xD3, 0x00, 0xEA, 0x2E, 0x95, 0x36, 0x79, 0xF1, 0x08, 0x68, 0x53, 0x91, 0x4D, 0xFB, 0xEA, 0x3E, 0x31, 0x7A, 0xB3, 0xE3, 0x41, 0x58, 0xDC, 0xBF, 0x92, 0x45, 0xD6, 0x25, 0xAD, 0xF6, 0x3A, 0x97, 0xE3, 0x7E, 0xD3, 0xB7, 0x0C, 0x87, 0x04, 0x68, 0x05, 0xA5, 0x8A, 0x38, 0xA4, 0x26, 0x11, 0x1B, 0xB9, 0xE5, 0x0C, 0xF0, 0xC7, 0xF3, 0xD4, 0x0E, 0xF8, 0x27, 0x26, 0x38, 0x9E, 0xFF, 0xE2, 0x95, 0x4D, 0xAD, 0x4A, 0xD6, 0xC1, 0x81, 0xFD, 0xD4, 0x1F, 0xF4, 0x1B, 0xEC, 0xD8, 0x6B, 0x8E, 0xE6, 0x7C, 0xB8, 0x04, 0x20, 0x99, 0xC4, 0x37, 0xD9, 0x1A, 0x8D, 0x25, 0xC0, 0xFE, 0x37, 0xEA, 0xB1, 0x6A, 0x11, 0x03, 0xD0, 0xFF, 0xE3, 0xAA, 0x0D, 0xA3, 0xB2, 0x6E, 0xAB, 0x66, 0x3F, 0x5A, 0xEE, 0x34, 0xAA, 0x83, 0xE9, 0x9C, 0xE8, 0x0B, 0x72, 0xF3, 0x37, 0x5F, 0xE1, 0x7A, 0xBF, 0xB9, 0x6C, 0x23, 0xAA, 0xCD, 0x85, 0x04, 0x07, 0x46, 0x02, 0xBD, 0x89, 0x26, 0xAB, 0x47, 0x7F, 0xF3, 0x9C, 0xD0, 0x0A, 0xF4, 0x7A, 0x33, 0x7E, 0x77, 0x84, 0x75, 0x1B, 0xB2, 0xB7, 0x48, 0x68, 0x5F, 0x31, 0xEB, 0x38, 0x4B, 0x39, 0x4B, 0x48, 0xDC, 0x7C, 0xC0, 0x03, 0x0A, 0x98, 0x5A, 0xFA, 0x07, 0xF0, 0xDA, 0xCB, 0xCE, 0x64, 0x11, 0xFB, 0xA2, 0xAB, 0x1C, 0x2B, 0xC4, 0xCD, 0x61, 0x7F, 0xC5, 0x18, 0x2C, 0xE9, 0x55, 0x33, 0xBB, 0x09, 0xCD, 0xC7, 0x81, 0xF7, 0x3B, 0x8D, 0x49, 0xB7, 0x3F, 0xD3, 0xCA, 0xB8, 0x0F, 0xC1, 0x39, 0x17, 0xD5, 0x94, 0x87, 0x1C, 0xB3, 0x1A, 0xEA, 0x07, 0x28, 0x8F, 0xCF, 0x5B, 0xF9, 0xB9, 0xAE, 0x65, 0xE9, 0x08, 0xB3, 0xCF, 0xA6, 0xF6, 0xCF, 0xCF, 0xD2, 0xFB, 0x81, 0xD5, 0x83, 0x3E, 0xC8, 0xB6, 0x7A, 0x4C, 0x22, 0x15, 0x10, 0xD0, 0x39, 0x57, 0xB3, 0x7F, 0x8B, 0x2B, 0x0D, 0xE1, 0xBF, 0xCF, 0xEA, 0x43, 0x6A, 0xD7, 0xB3, 0x3E, 0xBA, 0x37, 0x7B, 0xAE, 0x0B, 0x62, 0x2F, 0xE6, 0x0E, 0x7B, 0x02, 0xA0, 0x54, 0xF8, 0x84, 0x83, 0xA6, 0xA2, 0xE1, 0xD9, 0x20, 0xDB, 0xD2, 0x7C, 0xD9, 0x9F, 0x94, 0xA1, 0x57, 0xC3, 0x64, 0x3E, 0x1B, 0xA4, 0x86, 0x69, 0x67, 0xC2, 0x2B, 0xE0, 0x8A, 0xB6, 0xF6, 0x8F, 0x1F, 0xA2, 0xA3, 0xB9, 0x16, 0x08, 0xA9, 0xAD, 0x9C, 0xD4, 0xFA, 0x24, 0xB3, 0x7B, 0x36, 0xAC, 0xCE, 0xED, 0x35, 0x27, 0x6F, 0x29, 0xA3, 0x68, 0x1C, 0xD9, 0x5F, 0xE0, 0xEC, 0x54, 0x14, 0x03, 0x04, 0xE9, 0x9F, 0x05, 0xD8, 0x3B, 0x47, 0xF8, 0x74, 0x42, 0x76, 0xF0, 0x22, 0x42, 0x89, 0x6E, 0xE6, 0x5B, 0xFE, 0x66, 0x0F, 0xAB, 0xE9, 0x76, 0xA1, 0xCA, 0xB7, 0x23, 0xB9, 0x4E, 0xF0, 0x58, 0xA1, 0x1D, 0x15, 0x48, 0xB1, 0x27, 0x7E, 0x35, 0x28, 0xA6, 0xD6, 0xB5, 0xF2, 0x58, 0x81, 0x92, 0x07, 0xED, 0x0B, 0xA4, 0x50, 0x60, 0xB7, 0xDA, 0x33, 0x5F, 0x18, 0x8F, 0x2D, 0x36, 0x7C, 0xBD, 0x68, 0x6D, 0x11, 0x7F, 0x91, 0xF5, 0x7F, 0x95, 0x25, 0x10, 0x60, 0xB3, 0xCC, 0xC3, 0x3E, 0x42, 0xFB, 0x7D, 0xA9, 0x5E, 0x95, 0x88, 0x25, 0x04, 0x43, 0x6E, 0x67, 0x72, 0x7B, 0x09, 0xF6, 0xD5, 0x8C, 0xCC, 0x76, 0xAC, 0xDB, 0x91, 0x53, 0x57, 0x5B, 0x46, 0x70, 0xFB, 0x21, 0x55, 0x2B, 0x93, 0x23, 0xD7, 0x42, 0x24, 0x11, 0x4B, 0x29, 0xD5, 0x14, 0xC6, 0x5C, 0x26, 0x2B, 0x20, 0x52, 0xF8, 0x11, 0xB9, 0x60, 0x61, 0x13, 0xF6, 0x2A, 0xA3, 0x6D, 0x77, 0xCD, 0x06, 0xE4, 0x11, 0xF4, 0xD1, 0x09, 0x91, 0xEA, 0x06, 0x80, 0x7D, 0xA8, 0x3E, 0xC6, 0xF8, 0x63, 0x2A, 0x1D, 0xFE, 0x06, 0x38, 0x18, 0x94, 0xC7, 0x4E, 0x30, 0x09, 0x4E, 0x71, 0x49, 0xDA, 0x0C, 0x84, 0x7D, 0xDD, 0x7E, 0x0A, 0xEE, 0xE4, 0x70, 0xC0, 0xE0, 0x51, 0x82, 0xD5, 0x63, 0x89, 0xDC, 0xFB, 0x50, 0x63, 0xAD, 0x65, 0x20, 0x78, 0x9A, 0x0A, 0xBF, 0x3B, 0x73, 0x01, 0x39, 0x68, 0x09, 0x82, 0x6E, 0x09, 0x68, 0x4C, 0xB7, 0xC5, 0xD5, 0xF9, 0x85, 0x10, 0x4F, 0xDE, 0x25, 0x71, 0x0C, 0xD4, 0x46, 0x8A, 0xCE, 0xA7, 0x61, 0xCA, 0x9E, 0x8F, 0x6C, 0xD0, 0x4E, 0x9B, 0xFA, 0xD1, 0xAF, 0x29, 0x5F, 0x2B, 0xEF, 0x21, 0xDB, 0xE0, 0x04, 0x89, 0x0F, 0x3D, 0x36, 0x0E, 0x4F, 0xFB, 0x26, 0x00, 0x6B, 0x2D, 0xE1, 0x19, 0x33, 0xB5, 0x8E, 0x51, 0x7D, 0x77, 0x39, 0x19, 0xAE, 0xAE, 0xEB, 0xE3, 0x8C, 0x7A, 0xA4, 0x23, 0x5C, 0xA7, 0x3D, 0xD1, 0x3B, 0x71, 0xEA, 0x5E, 0x4B, 0x05, 0xA0, 0xB1, 0x4D, 0xFF, 0x4A, 0x92, 0xCA, 0xC3, 0x78, 0x24, 0x27, 0x31, 0x9F, 0x1E, 0x5F, 0xD7, 0xEE, 0xDD, 0x61, 0xA2, 0x5C, 0x71, 0x7F, 0xC4, 0x25, 0x06, 0xD4, 0x10, 0x49, 0xA5, 0x04, 0x53, 0x45, 0x82, 0xD3, 0xAC, 0x50, 0xA1, 0x2C, 0x50, 0x1D, 0xB5, 0xC1, 0xC2, 0xBC, 0x9C, 0xD0, 0x97, 0x49, 0x8E, 0x48, 0x3F, 0x2C, 0x4C, 0xAC, 0x06, 0x2F, 0xB7, 0x7B, 0x85, 0xE9, 0xE9, 0x18, 0x2D, 0x74, 0xFE, 0x95, 0x0E, 0x5D, 0xFB, 0x22, 0x9F, 0xCC, 0xD8, 0x6C, 0xDF, 0x67, 0xE4, 0x58, 0x05, 0x72, 0xA1, 0x4B, 0x76, 0xEC, 0x8B, 0x82, 0xF0, 0x1F, 0x5A, 0x30, 0x12, 0xB7, 0x97, 0xDF, 0xAA, 0x14, 0xEE, 0xE1, 0x83, 0x96, 0x83, 0x70, 0x89, 0xB8, 0x8B, 0x1A, 0x85, 0x4C, 0x19, 0xFD, 0xF3, 0xFF, 0xD7, 0xD2, 0x25, 0x1B, 0x6E, 0x00, 0x5E, 0x90, 0x1D, 0xEC, 0xB1, 0x4C, 0xAD, 0xBE, 0x03, 0xBD, 0x7F, 0xB8, 0x83, 0x18, 0x19, 0x64, 0x94, 0x76, 0x7F, 0xAA, 0x94, 0x04, 0x94, 0xE8, 0xC8, 0x6B, 0x88, 0x67, 0xB7, 0xE3, 0xDC, 0x88, 0xE9, 0x68, 0x9B, 0xD4, 0x4A, 0xCA, 0xCB, 0x3D, 0xFA, 0xBD, 0x4B, 0x20, 0xC3, 0xA2, 0x31, 0xD2, 0xAD, 0x77, 0x9D, 0x02, 0xFC, 0x2D, 0x0A, 0x4C, 0x45, 0xA5, 0x2E, 0x5B, 0x0A, 0x7E, 0x79, 0x97, 0x2F, 0xF1, 0xA4, 0xB6, 0x8F, 0xF0, 0xC1, 0x10, 0x9F, 0x3E, 0x69, 0x58, 0x93, 0xA7, 0x5F, 0x8D, 0x44, 0xAA, 0xE8, 0x1E, 0xF2, 0xF2, 0x78, 0x46, 0x71, 0x7F, 0xE1, 0x79, 0x74, 0x82, 0xDA, 0x13, 0xF0, 0xD7, 0xC3, 0x75, 0x1C, 0x2A, 0xE8, 0xBA, 0x53, 0xF8, 0x66, 0x04, 0xD4, 0xE6, 0x47, 0xD3, 0x10, 0xBB, 0x0B, 0x12, 0x09, 0x6A, 0x39, 0x15, 0x7A, 0xBB, 0xEB, 0xA0, 0xA5, 0x5D, 0x3B, 0xCC, 0x55, 0x0E, 0xCB, 0xFD, 0xA6, 0xB2, 0x21, 0xD3, 0x8F, 0x49, 0x5C, 0x0F, 0xA8, 0x94, 0xFF, 0x8B, 0x1D, 0x7C, 0xBF, 0x1F, 0x08, 0xEF, 0x4F, 0x55, 0xA5, 0x81, 0xB2, 0xE7, 0x02, 0x01, 0x21, 0x35, 0x2A, 0x8F, 0x5E, 0xF6, 0x15, 0x7D, 0x29, 0xB8, 0x48, 0xB2, 0xB4, 0x0D, 0x67, 0xB8, 0x00, 0x61, 0xC5, 0xAF, 0x3A, 0x4B, 0xD9, 0x5E, 0xD1, 0x47, 0xA1, 0x6A, 0x22, 0x8D, 0x09, 0x20, 0xC1, 0x80, 0x19, 0x6D, 0x75, 0x3F, 0xC9, 0xA4, 0x9F, 0xBB, 0xC4, 0xD6, 0xFD, 0xBC, 0xF7, 0xB8, 0x47, 0xFC, 0xCE, 0x69, 0x22, 0x80, 0x39, 0xA4, 0xCC, 0xBA, 0xDB, 0x9F, 0x01, 0x30, 0xAB, 0xFC, 0x92, 0xA6, 0xBC, 0x4F, 0x9B, 0x99, 0x69, 0xC5, 0x10, 0x03, 0x26, 0xBE, 0xCA, 0xFD, 0x1B, 0x85, 0x13, 0x3A, 0x4C, 0xE1, 0xBE, 0x19, 0x8D, 0x38, 0xD6, 0x20, 0x89, 0xD4, 0xFE, 0xDA, 0x9A, 0xCD, 0xA8, 0x87, 0x38, 0xB4, 0xE9, 0x15, 0xAB, 0xE9, 0x34, 0xC0, 0x45, 0x36, 0x74, 0xBB, 0xA8, 0xDB, 0xF2, 0x28, 0xBD, 0xC1, 0x98, 0x24, 0x85, 0x35, 0xFE, 0xB6, 0x91, 0xEC, 0x50, 0x76, 0x55, 0xC2, 0x46, 0x7B, 0xD9, 0x96, 0xCE, 0xA9, 0xEE, 0x7B, 0x58, 0x19, 0x4D, 0x8D, 0x53, 0x24, 0x32, 0x00, 0xDA, 0xD6, 0x80, 0xC4, 0xD5, 0x0C, 0x08, 0x99, 0x57, 0x8C, 0xF9, 0xFF, 0xA1, 0x0A, 0xC9, 0xAF, 0x71, 0x72, 0xAC, 0x44, 0x7F, 0xD1, 0x6D, 0x1A, 0x31, 0xF3, 0xB9, 0x50, 0x1F, 0x5C, 0xF6, 0xD7, 0x0F, 0x98, 0x93, 0xDB, 0x64, 0x9D, 0x5C, 0x4B, 0xDF, 0x5E, 0xC0, 0x9D, 0x25, 0xED, 0x1A, 0x77, 0x89, 0xC7, 0x95, 0x8E, 0x89, 0xA2, 0xC1, 0x64, 0xF9, 0x74, 0x8D, 0x8F, 0x01, 0x91, 0x2F, 0x09, 0xD1, 0xD7, 0x5C, 0x1F, 0x56, 0x97, 0x3C, 0x13, 0xE2, 0x55, 0x59, 0x0B, 0x51, 0x0A, 0x92, 0x8F, 0xB1, 0x90, 0x0F, 0xB1, 0xEB, 0x13, 0x37, 0x46, 0x44, 0xC4, 0xA3, 0x1F, 0x37, 0xD2, 0xA4, 0xD4, 0xA1, 0xBD, 0x10, 0xF3, 0x16, 0x7A, 0x00, 0x60, 0x8C, 0x67, 0x43, 0xA3, 0xAB, 0x78, 0xE4, 0xA1, 0xC7, 0x80, 0xED, 0x59, 0xDE, 0xB8, 0x00, 0x8B, 0x5C, 0xE6, 0xEC, 0xE3, 0x4F, 0x3D, 0xA1, 0xC3, 0x65, 0xD5, 0xBA, 0x3D, 0x77, 0x49, 0xA1, 0xF3, 0xC0, 0x6D, 0x3D, 0xAE, 0x53, 0x87, 0x9D, 0x30, 0xB8, 0x31, 0xED, 0x42, 0x9A, 0x8A, 0x64, 0xC5, 0x9F, 0xC4, 0xC2, 0xF5, 0xAD, 0x71, 0xEA, 0xA4, 0x38, 0x69, 0xEA, 0xD4, 0x16, 0xAA, 0xE5, 0xCF, 0xA6, 0x86, 0xBA, 0x76, 0x7D, 0x29, 0x18, 0x21, 0xB7, 0x82, 0x97, 0x84, 0x54, 0x3B, 0x3D, 0x7C, 0xA9, 0xC1, 0xE2, 0x7D, 0x4E, 0xFF, 0x66, 0x53, 0x5F, 0x1D, 0x61, 0x97, 0xAF, 0xD7, 0x45, 0x52, 0x67, 0x54, 0x4B, 0xDD, 0xD5, 0xA3, 0x3D, 0x62, 0xF1, 0xE8, 0x06, 0x88, 0x56, 0x80, 0x6B, 0x9F, 0xCB, 0x53, 0x20, 0x9D, 0x81, 0x8D, 0x37, 0x7F, 0x77, 0xE5, 0x9F, 0xE3, 0x40, 0x81, 0xE8, 0xB3, 0xBB, 0xF0, 0x4B, 0x50, 0xE8, 0x03, 0x1F, 0x9A, 0x6A, 0x59, 0xCD, 0x6B, 0x79, 0x96, 0xE3, 0xC3, 0xD2, 0xCF, 0x16, 0x56, 0xA8, 0x89, 0x9A, 0xC3, 0x6D, 0xA9, 0x37, 0x05, 0x50, 0x1A, 0x69, 0xD1, 0x9A, 0xBC, 0x2B, 0xA1, 0x02, 0x55, 0x1A, 0x03, 0x31, 0x97, 0x6A, 0x3E, 0xA6, 0xE0, 0x4B, 0xCD, 0xDF, 0x91, 0x32, 0x9A, 0xC0, 0xF6, 0xDE, 0x94, 0x34, 0x26, 0x03, 0x0C, 0x52, 0x31, 0xED, 0xB9, 0x39, 0xE3, 0x75, 0xCA, 0x2E, 0x85, 0xA9, 0x61, 0xC3, 0x6F, 0x71, 0x9E, 0xC4, 0x1E, 0xD2, 0xC7, 0x8F, 0xEB, 0x92, 0x69, 0xCD, 0x1D, 0x09, 0x15, 0xA7, 0xA6, 0x68, 0x9D, 0x82, 0x98, 0xE3, 0xE3, 0x87, 0xFD, 0x47, 0x7A, 0x03, 0xCD, 0xAC, 0xBE, 0x54, 0xB5, 0xAE, 0x11, 0xF7, 0x0C, 0xC5, 0x2A, 0x9D, 0x37, 0xA9, 0x42, 0x2A, 0xDE, 0x03, 0x61, 0x02, 0xE2, 0x14, 0x4F, 0x0F, 0xC0, 0x33, 0x30, 0x52, 0xFF, 0x18, 0x0F, 0x72, 0xD0, 0xB8, 0x02, 0x57, 0x1B, 0x87, 0xAD, 0x59, 0x64, 0xDB, 0xDB, 0xB7, 0xA6, 0x58, 0xC2, 0x6E, 0x97, 0x3F, 0xCC, 0xB0, 0x4F, 0x92, 0xB5, 0x2D, 0xAC, 0x13, 0x77, 0x4B, 0xCF, 0xC7, 0xC5, 0x47, 0xFB, 0x61, 0x8B, 0x1A, 0xDA, 0x41, 0x62, 0xFE, 0x93, 0x9A, 0x6E, 0x99, 0xC9, 0xCA, 0xA8, 0x11, 0xD2, 0xB3, 0x73, 0x79, 0xA3, 0x60, 0x72, 0x28, 0x6B, 0x1E, 0x25, 0xCF, 0x98, 0xAC, 0x1F, 0x35, 0x73, 0x82, 0xBB, 0x09, 0xE9, 0xB0, 0xEE, 0xEB, 0xC8, 0xBD, 0x4C, 0x62, 0xF4, 0x5D, 0x3E, 0x28, 0x22, 0xB2, 0x73, 0xFB, 0x92, 0xB5, 0xEE, 0x8E, 0xF4, 0xF8, 0xAD, 0x21, 0xC1, 0x71, 0x60, 0x80, 0x8D, 0x6A, 0x94, 0xDE, 0xB2, 0x2F, 0xAC, 0xB5, 0x3B, 0x88, 0x18, 0x9E, 0x4F, 0x36, 0x1D, 0xDF, 0xC1, 0x22, 0x33, 0xB6, 0x23, 0xDB, 0xDE, 0x05, 0xD2, 0xDC, 0x94, 0x44, 0xD7, 0x56, 0x53, 0x10, 0xC0, 0xBD, 0xA5, 0x77, 0xC8, 0xFA, 0x33, 0x3D, 0x53, 0x3A, 0xCC, 0x17, 0xCA, 0xA7, 0x75, 0x00, 0x14, 0x9C, 0xBC, 0x71, 0x06, 0x4E, 0x3E, 0xC6, 0x0E, 0xEA, 0xC2, 0x0A, 0x0F, 0xFE, 0x88, 0x09, 0x1D, 0x5F, 0xBD, 0xAE, 0x89, 0xE6, 0x4F, 0x11, 0x7E, 0xD1, 0xC3, 0x50, 0x6D, 0x94, 0x0E, 0x46, 0x1B, 0x41, 0xFE, 0x4D, 0xC0, 0x41, 0xC6, 0x17, 0xEA, 0x1D, 0x2E, 0x70, 0xA8, 0x4E, 0x64, 0x9B, 0xC7, 0xC6, 0x62, 0x46, 0xEF, 0x33, 0x5F, 0xCF, 0x55, 0x7F, 0xEC, 0xC5, 0xDD, 0x42, 0xC2, 0x7E, 0xDB, 0x2C, 0x49, 0x46, 0xF0, 0xB8, 0x8B, 0x6C, 0x49, 0x5F, 0x61, 0x9C, 0x0D, 0xDD, 0x4B, 0x8F, 0x24, 0xE4, 0x3D, 0x48, 0x7F, 0x83, 0xF7, 0x71, 0xB0, 0x3B, 0x47, 0x28, 0xCD, 0xFD, 0xB3, 0x48, 0x75, 0x8E, 0x1C, 0x18, 0x8D, 0x46, 0x93, 0xA3, 0xC5, 0x46, 0x21, 0x86, 0xEC, 0xAA, 0xCD, 0x7D, 0x93, 0x4E, 0xC2, 0x8C, 0xD2, 0xC5, 0x9A, 0x9E, 0x53, 0x2F, 0x90, 0x48, 0xD6, 0xD6, 0x71, 0x9D, 0xFA, 0x72, 0x00, 0x24, 0x40, 0x7C, 0x5D, 0x3E, 0x17, 0xFC, 0xCE, 0x40, 0x05, 0x51, 0x3D, 0x2B, 0x08, 0xEA, 0x9C, 0x91, 0xAC, 0xC5, 0x00, 0x32, 0xB9, 0x86, 0x5A, 0xFA, 0x65, 0xE8, 0x7C, 0xFA, 0xB9, 0xB0, 0x18, 0x2A, 0xC2, 0xFC, 0x55, 0x2B, 0xAE, 0x50, 0xAD, 0xA7, 0xF7, 0x1F, 0xF6, 0x06, 0xA8, 0xAC, 0xDC, 0x86, 0x7C, 0x25, 0x05, 0x32, 0x15, 0x88, 0x6A, 0x52, 0x2C, 0xB8, 0x23, 0x49, 0x77, 0xBC, 0x4B, 0x22, 0xD0, 0x63, 0x1C, 0x7F, 0x82, 0xE7, 0x34, 0x0D, 0x88, 0x39, 0xC0, 0xDB, 0xC9, 0x41, 0xFC, 0x1F, 0x37, 0xA9, 0xC4, 0xD5, 0x02, 0x9B, 0xD1, 0x59, 0x00, 0xEB, 0x54, 0x18, 0x72, 0x38, 0x82, 0x5A, 0xBA, 0x48, 0xF1, 0xA3, 0x74, 0xCC, 0x4A, 0x26, 0x7A, 0x15, 0xC8, 0x61, 0x32, 0x46, 0x6B, 0xD0, 0x80, 0x16, 0x68, 0x5C, 0x81, 0x58, 0x18, 0xF1, 0xFA, 0x39, 0xBD, 0x4E, 0x3B, 0xF2, 0x00, 0x90, 0x6B, 0x14, 0xC0, 0xD3, 0x9C, 0x91, 0x0F, 0x1E, 0x05, 0x2E, 0x78, 0x60, 0x3A, 0x39, 0xC2, 0xCE, 0x38, 0xBB, 0x63, 0x48, 0x29, 0x5E, 0x5F, 0x42, 0x81, 0xFC, 0x0A, 0x3C, 0xD6, 0xD9, 0xE0, 0x5A, 0xCD, 0x1A, 0xB4, 0x7E, 0x9B, 0x8B, 0x90, 0x43, 0xE0, 0x09, 0xA8, 0x12, 0x46, 0x7A, 0x90, 0x46, 0x58, 0x78, 0xBB, 0x20, 0x39, 0x78, 0xF5, 0xF5, 0x10, 0x9F, 0x53, 0xDA, 0x6D, 0x39, 0x74, 0xEE, 0x7F, 0xAF, 0x7A, 0x1B, 0xF6, 0x71, 0xA2, 0x73, 0xEE, 0x3E, 0xEA, 0x57, 0xBB, 0x0E, 0xE3, 0xA9, 0xFA, 0x9A, 0x6D, 0x7D, 0xC0, 0xE2, 0x7D, 0x32, 0x85, 0xF9, 0xC9, 0xE0, 0x3F, 0x51, 0xC3, 0xEA, 0x8E, 0xEF, 0xBB, 0x7E, 0x10, 0x1A, 0x25, 0xA4, 0x31, 0xA2, 0xB1, 0xBB, 0xCB, 0x57, 0xD0, 0x88, 0x9B, 0xBD, 0x99, 0xD9, 0x79, 0xA1, 0x92, 0xE1, 0x8B, 0xA9, 0x4A, 0x8A, 0xA5, 0xE4, 0xE4, 0xD6, 0xD5, 0x46, 0x4D, 0x77, 0xC9, 0x30, 0x76, 0x8A, 0x5A, 0xA8, 0x31, 0xA5, 0x38, 0x2E, 0xD1, 0x6F, 0x06, 0xFB, 0xF6, 0xCD, 0x47, 0xBB, 0x0C, 0xB8, 0x13, 0xBA, 0xE6, 0x24, 0x7C, 0x3F, 0x13, 0x79, 0xB3, 0x39, 0xA3, 0x7A, 0x77, 0xB4, 0x84, 0x55, 0xD4, 0x8F, 0x48, 0x28, 0x51, 0xA3, 0x8E, 0x95, 0xD7, 0xCC, 0x1C, 0x75, 0xC2, 0x65, 0x40, 0xA9, 0xF8, 0x1C, 0xD3, 0xEB, 0xCB, 0xF5, 0xF0, 0x9C, 0x67, 0xAF, 0xF1, 0xF1, 0x89, 0xB1, 0xC5, 0x2B, 0x22, 0xD0, 0xF7, 0x57, 0xF1, 0x28, 0xFD, 0x00, 0xC6, 0x7C, 0x9A, 0xF9, 0xB9, 0x90, 0x2F, 0x4C, 0x4E, 0x6D, 0x41, 0xEC, 0x31, 0x60, 0x9E, 0x6A, 0x91, 0xE2, 0xED, 0x56, 0x65, 0x95, 0xE2, 0xFC, 0x62, 0xF9, 0x6F, 0xB1, 0x11, 0x8C, 0x09, 0x4A, 0xC7, 0xB3, 0x6E, 0x3A, 0xE9, 0x7C, 0x46, 0x1F, 0x61, 0xFA, 0xBD, 0x25, 0x0D, 0x15, 0xCA, 0xBE, 0x89, 0xC6, 0xF5, 0xF2, 0x55, 0xAB, 0x2C, 0x82, 0x9E, 0x63, 0xA9, 0xBD, 0x70, 0x52, 0xCB, 0xED, 0xC7, 0x89, 0x9F, 0x27, 0xFE, 0x3D, 0x70, 0x14, 0xDB, 0x7C, 0x4E, 0x4F, 0x4D, 0x35, 0xD3, 0x28, 0xE1, 0xEE, 0x44, 0xA2, 0x1B, 0xF0, 0xEE, 0x3E, 0x1D, 0x69, 0x15, 0xE3, 0xB5, 0x98, 0xD8, 0x8C, 0x05, 0x32, 0x48, 0x4F, 0x76, 0x4C, 0x50, 0x04, 0x3C, 0x10, 0x2F, 0x06, 0x3A, 0x69, 0x20, 0x87, 0xFB, 0xDC, 0x93, 0x2D, 0x1D, 0x2A, 0xE2, 0xB6, 0xBE, 0x7A, 0xBB, 0x34, 0xD0, 0x57, 0x30, 0x6B, 0x40, 0x84, 0x1B, 0x28, 0x4D, 0x6F, 0xFE, 0xE2, 0x00, 0xB8, 0x70, 0x35, 0xDA, 0xED, 0x10, 0xC6, 0x33, 0xB7, 0xFC, 0x49, 0x48, 0x46, 0xD0, 0x01, 0x81, 0x14, 0x6C, 0x4B, 0x2E, 0x5C, 0x58, 0xC5, 0x7E, 0xE0, 0x3A, 0xAA, 0x0D, 0xB9, 0x2E, 0xDD, 0xD5, 0x06, 0x4F, 0x07, 0x36, 0x48, 0xCC, 0x50, 0xFC, 0xE2, 0x8D, 0x96, 0xAD, 0x03, 0xA2, 0xB5, 0x67, 0x36, 0xFD, 0x0E, 0xC6, 0x71, 0x84, 0xCF, 0x83, 0x11, 0xC7, 0x03, 0x57, 0xFA, 0x0B, 0xA8, 0x9B, 0x96, 0x71, 0x65, 0x3B, 0x9F, 0xDA, 0xA8, 0xBC, 0xDA, 0xC0, 0x00, 0xCD, 0x66, 0x5B, 0x92, 0x9B, 0x61, 0xF4, 0x67, 0x7C, 0x53, 0x5D, 0x78, 0x54, 0x15, 0xCB, 0x60, 0x4F, 0xFA, 0xDC, 0xA8, 0xFB, 0xD5, 0x0D, 0xF0, 0x12, 0x03, 0x63, 0x90, 0xE5, 0x2B, 0xA8, 0xAE, 0x2C, 0x6C, 0x09, 0x68, 0x11, 0x05, 0x7A, 0x87, 0x92, 0xA3, 0xAC, 0xAE, 0xB9, 0x98, 0xE3, 0x5C, 0xA1, 0xCD, 0xD7, 0x7E, 0x10, 0x17, 0xDD, 0x09, 0xE4, 0x00, 0x19, 0xEA, 0xC8, 0xC5, 0x8C, 0x5E, 0xAC, 0xAB, 0x7A, 0xAB, 0xB1, 0x4E, 0xE7, 0xF7, 0x53, 0x75, 0x6D, 0x35, 0x2E, 0x14, 0x79, 0x26, 0x37, 0x21, 0x36, 0x22, 0x2B, 0xDB, 0xEA, 0x03, 0x53, 0x8E, 0x7F, 0xC2, 0x03, 0x02, 0x7F, 0xC3, 0x3F, 0x5B, 0x4B, 0xF1, 0xD4, 0x0F, 0xFA, 0xAA, 0xB2, 0xC3, 0x26, 0x44, 0x40, 0xD6, 0x83, 0x98, 0xCA, 0x14, 0x3F, 0x65, 0xC4, 0xFC, 0x28, 0x9F, 0x1E, 0x26, 0xF1, 0xC9, 0x05, 0x3A, 0xC7, 0xBC, 0x7F, 0xE5, 0x36, 0xCC, 0xD9, 0x43, 0x3C, 0x46, 0xC3, 0x77, 0xDE, 0x97, 0x6B, 0xFB, 0xB9, 0x1D, 0x7B, 0xE9, 0x28, 0x96, 0xDA, 0x09, 0x5A, 0x77, 0xFE, 0x77, 0x04, 0xC6, 0x11, 0x51, 0xB9, 0xE1, 0xF7, 0x04, 0xC1, 0xB3, 0x88, 0x0F, 0xED, 0xA3, 0x94, 0x88, 0x28, 0x4B, 0xD9, 0xEA, 0x11, 0xF5, 0x2A, 0x50, 0x18, 0x2D, 0x2B, 0xA3, 0x76, 0x59, 0xD9, 0x09, 0xD6, 0x13, 0x85, 0x38, 0x7A, 0x4F, 0x1A, 0x77, 0xAC, 0x28, 0xF9, 0x4C, 0xA6, 0x4A, 0xAA, 0xFA, 0x79, 0xC5, 0x18, 0x09, 0xC1, 0x89, 0x46, 0x99, 0xE4, 0x6C, 0x00, 0x52, 0x98, 0x28, 0xC1, 0x94, 0x2F, 0x71, 0x20, 0x3A, 0x8B, 0x22, 0xB9, 0x55, 0x33, 0xB6, 0xE6, 0xCE, 0x93, 0xF9, 0x4C, 0xDC, 0x8B, 0x4B, 0x64, 0xF2, 0x2A, 0x47, 0xB8, 0x82, 0x79, 0xCC, 0xE4, 0x6D, 0xD2, 0xCB, 0x2C, 0x6E, 0x0E, 0xEC, 0x55, 0x50, 0x21, 0x84, 0xFE, 0x5B, 0xF7, 0x86, 0x09, 0x6C, 0x2F, 0xB3, 0xE2, 0x07, 0x80, 0xEE, 0xC8, 0x05, 0xB8, 0x9A, 0x14, 0x75, 0xA2, 0xDE, 0x2D, 0x9D, 0xA9, 0xD7, 0x98, 0x6D, 0x58, 0xBF, 0x9C, 0xD3, 0xE8, 0xC3, 0xCC, 0x41, 0x27, 0xEA, 0x62, 0xBB, 0x42, 0x32, 0xB2, 0xAC, 0x35, 0x11, 0x93, 0x58, 0xC4, 0x3B, 0x51, 0x5B, 0x54, 0x4F, 0xEB, 0xA4, 0x4A, 0x44, 0x45, 0x62, 0x22, 0xB1, 0x52, 0x3B, 0x0E, 0xB4, 0x9A, 0x75, 0x76, 0xCA, 0xD8, 0xB1, 0x1C, 0x15, 0xBC, 0xFB, 0x8A, 0x34, 0xB7, 0x59, 0x16, 0x68, 0x6C, 0xD7, 0xF2, 0xF1, 0x67, 0x99, 0x46, 0x07, 0x93, 0xFD, 0x88, 0xA9, 0x90, 0x80, 0x68, 0x1A, 0xAC, 0x69, 0xB5, 0xF3, 0xAE, 0x82, 0xD8, 0x8E, 0x1A, 0x1D, 0xD6, 0xBE, 0x3A, 0xFA, 0x9F, 0x2A, 0xC3, 0xB4, 0x59, 0xC1, 0x38, 0xCF, 0x55, 0xB0, 0x17, 0x6B, 0xB6, 0xA7, 0x25, 0x6D, 0x48, 0x19, 0xCB, 0xE8, 0x2C, 0xFF, 0xB4, 0x4D, 0x98, 0x87, 0xCA, 0xC4, 0x24, 0x31, 0xFB, 0xF6, 0x6D, 0xE5, 0x1B, 0x29, 0xB1, 0x53, 0x61, 0x06, 0x60, 0x5F, 0xAB, 0xE0, 0x49, 0x5F, 0xA2, 0x54, 0xF4, 0xBC, 0xFB, 0x0F, 0xD3, 0xF7, 0xFD, 0xF5, 0x92, 0x8E, 0xF5, 0x00, 0xC4, 0xE3, 0x7A, 0xC6, 0x33, 0xB3, 0xDF, 0xBE, 0x49, 0x85, 0x7B, 0xC4, 0x18, 0x84, 0x37, 0x4A, 0x66, 0x4A, 0xE8, 0x86, 0x74, 0xE7, 0x21, 0x55, 0xD4, 0x6E, 0x4F, 0xAF, 0xAC, 0xB9, 0xC8, 0xCF, 0x5B, 0x9B, 0x06, 0xDA, 0xB2, 0x05, 0x7D, 0xB4, 0x80, 0xF8, 0xB5, 0x6D, 0x0A, 0x10, 0xAD, 0x5C, 0x67, 0x9E, 0x46, 0x40, 0xDD, 0x16, 0xAD, 0xBC, 0xBC, 0x28, 0xF1, 0x0A, 0xD7, 0xEB, 0xD8, 0xB0, 0x95, 0x60, 0x93, 0xAD, 0x7B, 0x37, 0xA4, 0x78, 0x45, 0x72, 0xE8, 0x0D, 0x0D, 0x18, 0x9A, 0x8A, 0x69, 0xA2, 0xF8, 0x05, 0x00, 0x49, 0x28, 0x1C, 0xE2, 0xFB, 0x88, 0xB4, 0x26, 0xF4, 0x02, 0x3E, 0xC2, 0x47, 0xE8, 0xAB, 0x08, 0x49, 0xF0, 0x1A, 0x1A, 0x13, 0x40, 0x2D, 0x5E, 0xF9, 0x64, 0x39, 0x24, 0xCD, 0x99, 0x42, 0x70, 0x5B, 0x0F, 0x85, 0x67, 0x6C, 0x4C, 0xC0, 0xF0, 0x82, 0x91, 0x8D, 0xEF, 0xF2, 0x0C, 0xA0, 0xF4, 0xFE, 0x63, 0x49, 0x4D, 0xA9, 0xA3, 0x2F, 0x33, 0x3E, 0xB8, 0x75, 0xE4, 0x24, 0xB3, 0xB4, 0xE5, 0x34, 0x27, 0x68, 0x49, 0xC5, 0xD9, 0x05, 0x60, 0x17, 0x12, 0x1D, 0xFA, 0x51, 0x7A, 0x78, 0x42, 0x09, 0x21, 0xEA, 0x55, 0x0F, 0x39, 0x9E, 0xBA, 0xD3, 0x12, 0x44, 0x69, 0xA2, 0x6B, 0x07, 0x41, 0xF0, 0xE8, 0x7F, 0x6D, 0xD8, 0xBA, 0x21, 0x76, 0x90, 0xEA, 0xE0, 0xC0, 0xDB, 0x3E, 0x7F, 0xC6, 0xFE, 0xE4, 0xE0, 0xB1, 0x62, 0x1D, 0x23, 0x50, 0x75, 0x45, 0xA4, 0x6E, 0x06, 0x38, 0x86, 0xB1, 0x40, 0x69, 0xAB, 0x3E, 0x3E, 0xAF, 0x24, 0x9E, 0x98, 0x09, 0x88, 0x71, 0x1F, 0x42, 0x2D, 0x59, 0xFB, 0xF2, 0x8A, 0xBD, 0x72, 0xE7, 0x8B, 0x8D, 0x36, 0xD6, 0x3F, 0x83, 0xEC, 0x60, 0xF3, 0xDC, 0x47, 0xC7, 0x8F, 0x47, 0x34, 0xA5, 0xBB, 0x04, 0x0E, 0x87, 0xC0, 0xD4, 0x4A, 0x94, 0xC5, 0xBE, 0x67, 0xAF, 0x1E, 0x77, 0xB3, 0xDB, 0x21, 0x00, 0x48, 0x0D, 0xA5, 0x8C, 0x15, 0x20, 0xF1, 0xAD, 0xA1, 0x31, 0xFD, 0xE2, 0xDE, 0xCE, 0xD3, 0xB3, 0x98, 0x26, 0x86, 0x1E, 0xB2, 0x5A, 0xF3, 0x71, 0x7E, 0xA6, 0x3D, 0xA7, 0xFC, 0x69, 0x6A, 0x43, 0x49, 0x59, 0x54, 0x7B, 0x9D, 0x1B, 0x1B, 0x50, 0xC8, 0x64, 0x62, 0x4E, 0x26, 0x89, 0x40, 0x5B, 0xEA, 0x26, 0x94, 0xFD, 0x6B, 0x48, 0x1C, 0xC2, 0x0F, 0x6C, 0x58, 0x47, 0xE2, 0x3C, 0xC9, 0xD1, 0xCA, 0x14, 0x7F, 0x6D, 0x94, 0x0B, 0xFD, 0xB6, 0xDE, 0xDB, 0x6C, 0x87, 0x7F, 0xA3, 0xBD, 0x19, 0xDC, 0xDD, 0x20, 0x88, 0x6D, 0x65, 0xA2, 0x46, 0x93, 0x2F, 0xA2, 0xA5, 0x9E, 0xA7, 0x56, 0x50, 0x62, 0x16, 0x3E, 0x5E, 0xE5, 0x55, 0x64, 0xD1, 0x70, 0x62, 0xE3, 0xEE, 0x02, 0xC6, 0x51, 0xD3, 0xF8, 0x58, 0x14, 0xE6, 0x99, 0x77, 0x08, 0x1E, 0x32, 0x5E, 0xC3, 0x85, 0xCC, 0x02, 0x60, 0x83, 0xA6, 0xE3, 0xD0, 0xD0, 0x81, 0x00, 0x71, 0x6D, 0xE4, 0x00, 0x73, 0xAF, 0x82, 0x7D, 0x74, 0x42, 0x18, 0x18, 0x69, 0x48, 0xA5, 0xE5, 0x37, 0xAA, 0xEF, 0xE2, 0x79, 0x1E, 0x0F, 0x54, 0x3E, 0x62, 0x99, 0xAE, 0x8E, 0xA9, 0x14, 0x67, 0xC0, 0x80, 0xF9, 0x71, 0xFD, 0xA1, 0xCA, 0x5E, 0x30, 0x4B, 0xA7, 0xBC, 0xBE, 0x08, 0x52, 0x70, 0xE1, 0xBA, 0xFB, 0xD1, 0x97, 0xE6, 0xDA, 0x78, 0x18, 0x14, 0xC2, 0xCE, 0x9F, 0x34, 0x82, 0xC9, 0x79, 0x5D, 0xFE, 0xC3, 0x98, 0x29, 0x1E, 0xEC, 0xB5, 0x23, 0x33, 0x6E, 0xAA, 0xEC, 0x8C, 0x34, 0xC1, 0x04, 0x6D, 0xD8, 0xAD, 0x21, 0xA1, 0x74, 0x32, 0x42, 0x2E, 0x1D, 0xFE, 0xEE, 0xDA, 0x25, 0x3A, 0x11, 0x84, 0x64, 0x36, 0x9B, 0x60, 0x3E, 0xCF, 0x15, 0xEE, 0xDC, 0xD4, 0xDB, 0x10, 0xD8, 0x7F, 0x97, 0x42, 0x5F, 0x85, 0x58, 0x35, 0x2E, 0xD2, 0x88, 0x95, 0x94, 0x86, 0x4B, 0x1A, 0xCA, 0x2A, 0xE4, 0xD3, 0xB7, 0xEF, 0x5A, 0x78, 0xAA, 0x11, 0xB2, 0x66, 0x29, 0xAC, 0x36, 0x2B, 0x29, 0x4C, 0x10, 0x7A, 0x87, 0xAD, 0x40, 0x61, 0x28, 0x73, 0x58, 0x48, 0x66, 0x07, 0xC2, 0xF6, 0x36, 0xDD, 0x9F, 0x32, 0x15, 0x1C, 0xB1, 0x56, 0x6C, 0x81, 0xF4, 0x06, 0x79, 0x2C, 0xD2, 0x5D, 0xB0, 0xAF, 0x4D, 0xFA, 0xC3, 0x21, 0x77, 0x1D, 0x1D, 0x77, 0x26, 0xEF, 0x3E, 0x13, 0x6A, 0x3E, 0x64, 0x79, 0x0A, 0x28, 0xC5, 0x04, 0x0E, 0x18, 0x3A, 0x2B, 0x57, 0x99, 0xF1, 0x03, 0xE1, 0x9E, 0x12, 0xBE, 0x40, 0x63, 0x9E, 0xF1, 0x38, 0x80, 0x4E, 0xEF, 0x58, 0xA1, 0x21, 0xC9, 0x7A, 0xD2, 0xBA, 0xB6, 0x22, 0x30, 0x17, 0x8C, 0xCA, 0x6C, 0x98, 0xF2, 0x2C, 0x39, 0x38, 0x6B, 0xAF, 0x2E, 0x81, 0x7A, 0x74, 0x84, 0xE7, 0xCD, 0x5A, 0x75, 0x53, 0xB3, 0xD7, 0x43, 0x2E, 0x6B, 0x89, 0x66, 0x66, 0x38, 0x05, 0x6D, 0x11, 0x7B, 0x4C, 0xAD, 0xA9, 0x45, 0x9F, 0x6E, 0x74, 0x2C, 0xA3, 0x29, 0xCA, 0x9B, 0x96, 0xDB, 0x0A, 0x8B, 0xDD, 0x83, 0xB6, 0x08, 0x06, 0x39, 0xB6, 0x02, 0xB5, 0xFB, 0x21, 0x97, 0xE9, 0xDB, 0x1A, 0xA9, 0xCC, 0xA3, 0xFE, 0xF1, 0xF7, 0x94, 0x50, 0x67, 0xA7, 0xEE, 0x75, 0x93, 0xB3, 0x59, 0x4B, 0xBF, 0x4E, 0x82, 0xB0, 0xEA, 0xB8, 0x88, 0xBC, 0x6E, 0xF1, 0xB1, 0xB5, 0x13, 0x5B, 0x46, 0xB0, 0xB8, 0xD1, 0x5A, 0x21, 0x37, 0x76, 0x46, 0xE5, 0x19, 0xAB, 0xB8, 0x12, 0xCA, 0x8E, 0x21, 0x94, 0xEA, 0x95, 0x98, 0x98, 0x4D, 0x85, 0xC5, 0x09, 0x70, 0x76, 0xFC, 0x38, 0xB9, 0xD4, 0x11, 0x08, 0xE7, 0x62, 0x41, 0x09, 0x8E, 0xB7, 0x98, 0x4F, 0x9D, 0xC6, 0xEC, 0x59, 0x6B, 0x88, 0x17, 0xCE, 0x73, 0x25, 0xEF, 0xAD, 0x27, 0x10, 0x99, 0x17, 0xF7, 0xCD, 0x84, 0xAD, 0x93, 0x42, 0xFD, 0xF4, 0xFD, 0xCC, 0x22, 0xBF, 0xB6, 0x60, 0x94, 0x8A, 0x78, 0x14, 0xF8, 0x3E, 0xB7, 0x90, 0x40, 0x61, 0xEC, 0xFF, 0x45, 0x29, 0xAB, 0x83, 0xC6, 0xAA, 0x7D, 0x98, 0xE5, 0xDC, 0x32, 0x29, 0xDA, 0x50, 0x10, 0x0A, 0xA3, 0xE4, 0x4E, 0x90, 0xEE, 0x01, 0x69, 0xCD, 0xD4, 0x66, 0x6C, 0x17, 0x59, 0xCA, 0xBF, 0xCF, 0x9D, 0x1C, 0x1F, 0xC8, 0x14, 0x7C, 0x29, 0xCC, 0x5D, 0x0E, 0x3C, 0xFA, 0xC1, 0xFA, 0x34, 0x75, 0x24, 0x29, 0xC0, 0x35, 0x75, 0x3A, 0x31, 0xA6, 0x5F, 0x62, 0x2B, 0xDC, 0x3F, 0x36, 0x94, 0x8C, 0x56, 0x4F, 0x87, 0xF4, 0xD9, 0x1B, 0x71, 0xCB, 0xE5, 0x46, 0xCC, 0xF6, 0xB9, 0x80, 0x6B, 0xE2, 0x22, 0xA2, 0xB4, 0xE8, 0x74, 0x22, 0x3E, 0x5D, 0x7F, 0xA8, 0xA6, 0x6A, 0x5B, 0x7E, 0xC5, 0x1E, 0xB0, 0x87, 0xCC, 0xF5, 0xE5, 0x64, 0x02, 0x45, 0x7B, 0x93, 0x35, 0x03, 0xA8, 0x58, 0x91, 0xB3, 0xB6, 0x85, 0xAD, 0x44, 0x11, 0xC2, 0x94, 0x05, 0xA6, 0x5D, 0x15, 0x80, 0x0F, 0xED, 0x89, 0xFB, 0x01, 0xF7, 0xCC, 0x4A, 0x49, 0xD2, 0xDA, 0xCB, 0x8F, 0xCF, 0xF7, 0xE7, 0xA2, 0x67, 0x67, 0x52, 0xB1, 0x74, 0x26, 0xFB, 0x2A, 0x91, 0x33, 0x4B, 0xD1, 0x9C, 0xB0, 0xB0, 0xFC, 0xE5, 0xBB, 0x28, 0x24, 0xC1, 0xA5, 0xE6, 0x5F, 0x88, 0x28, 0x74, 0xE4, 0x88, 0xDF, 0xB9, 0x4D, 0xC3, 0x34, 0x90, 0x85, 0x0A, 0x3C, 0x90, 0xBA, 0xCE, 0xF3, 0xDE, 0xA4, 0x14, 0xB6, 0xEA, 0x6A, 0xA4, 0x77, 0x64, 0x33, 0x7D, 0x6F, 0x7F, 0xCB, 0x6F, 0x32, 0xB1, 0xE2, 0xC0, 0x54, 0x71, 0x99, 0x55, 0x5F, 0x1B, 0xE2, 0x88, 0x37, 0x99, 0x54, 0xD5, 0xBA, 0x7B, 0x1C, 0x9B, 0xD1, 0x1A, 0x6C, 0x18, 0x4D, 0x82, 0x95, 0x9F, 0xB0, 0xC0, 0x4E, 0xC1, 0x58, 0x51, 0x6D, 0xF3, 0x4D, 0x78, 0x1F, 0x5B, 0x8B, 0x71, 0x91, 0xA4, 0xEC, 0x91, 0xB5, 0x69, 0xCC, 0xB6, 0xED, 0x32, 0x98, 0x78, 0x3F, 0x49, 0xD2, 0x76, 0x9C, 0x0A, 0xBB, 0x71, 0x69, 0x6E, 0x9A, 0x84, 0x41, 0x9F, 0xFE, 0x32, 0x96, 0xC6, 0xCB, 0xCA, 0xCF, 0xB7, 0x67, 0xFF, 0xED, 0x90, 0xDB, 0xA3, 0x7C, 0x0C, 0x13, 0x4A, 0x70, 0xC4, 0xF0, 0x74, 0x19, 0x7E, 0xF8, 0x4C, 0x76, 0xE4, 0x43, 0x31, 0x78, 0xD0, 0xA3, 0x72, 0x8C, 0x98, 0xFC, 0xFC, 0x94, 0xBF, 0x20, 0xF7, 0x1A, 0xFE, 0x22, 0xA2, 0x4A, 0x0D, 0x82, 0x4D, 0xF6, 0xFD, 0x2A, 0xB2, 0x71, 0xB9, 0xA3, 0x07, 0x4E, 0x2B, 0xBD, 0x88, 0xE0, 0x42, 0x28, 0x5A, 0x36, 0xDA, 0x11, 0x8A, 0x43, 0xE0, 0xB5, 0xC6, 0xA0, 0x04, 0x93, 0x07, 0x88, 0x13, 0x7F, 0x79, 0xA9, 0x68, 0x86, 0x76, 0x41, 0xB4, 0x79, 0x56, 0xC1, 0x60, 0x50, 0x5C, 0x27, 0xFC, 0xE9, 0x00, 0x01, 0xCB, 0x78, 0xD8, 0xC5, 0xE8, 0x47, 0xCE, 0x62, 0x6D, 0x37, 0x5E, 0x20, 0x3A, 0xD7, 0x61, 0x24, 0xDC, 0xB4, 0x58, 0x9E, 0xC5, 0x7A, 0xD5, 0xE7, 0xF1, 0x23, 0xBC, 0x92, 0x04, 0x32, 0xFE, 0x56, 0x12, 0x64, 0xC1, 0x97, 0x5F, 0x63, 0x14, 0x38, 0x5B, 0xFE, 0x8B, 0x38, 0x1D, 0x9F, 0x4C, 0x1F, 0xAF, 0x5E, 0x97, 0x36, 0xB8, 0x0F, 0xF8, 0x36, 0x47, 0xFD, 0x77, 0xEB, 0xA2, 0x99, 0x66, 0x14, 0x99, 0x03, 0xBB, 0x55, 0xF9, 0x56, 0x7E, 0x68, 0xAF, 0xF5, 0x35, 0xF0, 0xFB, 0x2A, 0x94, 0x59, 0xAF, 0x8B, 0x41, 0xAF, 0x27, 0xDB, 0xEC, 0x0D, 0x0E, 0x03, 0xEF, 0x12, 0x2A, 0x55, 0x8E, 0x22, 0xD8, 0x27, 0xEF, 0x59, 0x67, 0x73, 0x51, 0x6D, 0x8A, 0xEC, 0x17, 0x90, 0xFC, 0x1B, 0x60, 0x0A, 0xD5, 0x02, 0x7F, 0x19, 0x7B, 0xF8, 0x91, 0x40, 0xF9, 0xB2, 0x08, 0x3A, 0x5B, 0x00, 0x92, 0xE3, 0xF5, 0x3E, 0x36, 0x14, 0x7E, 0xB4, 0x4A, 0x43, 0x48, 0x6F, 0x65, 0x1B, 0xF8, 0x51, 0xEF, 0x24, 0xA2, 0xA1, 0x71, 0x57, 0x40, 0x57, 0x89, 0x66, 0x1F, 0x8B, 0x0D, 0xC6, 0x18, 0x3E, 0xFA, 0xB0, 0xCB, 0x04, 0x77, 0xA1, 0xFF, 0x52, 0xED, 0x34, 0x03, 0x84, 0x8E, 0xAA, 0xC5, 0x44, 0x3D, 0xB5, 0x93, 0xD0, 0x42, 0x31, 0xE5, 0x2A, 0xFA, 0xE3, 0x22, 0x48, 0xD5, 0xB9, 0x3A, 0x51, 0x4D, 0x57, 0x42, 0xF8, 0x8A, 0x41, 0xD6, 0x0D, 0x71, 0xD1, 0x9B, 0x6F, 0xB5, 0x41, 0xF2, 0xDB, 0xBA, 0x96, 0x75, 0x6A, 0x3A, 0x50, 0xAE, 0x03, 0xC8, 0x87, 0x2D, 0x83, 0xD7, 0x70, 0x2D, 0x88, 0xC4, 0xE2, 0xDD, 0x9B, 0xDA, 0xB1, 0x36, 0xF7, 0x60, 0x83, 0x20, 0x86, 0x42, 0xC2, 0xB7, 0x24, 0xF0, 0xBE, 0x95, 0x15, 0x57, 0x5F, 0x74, 0x4B, 0xEE, 0xB1, 0x83, 0x9E, 0x76, 0xF3, 0x26, 0x88, 0xC9, 0x00, 0xB8, 0x32, 0xD0, 0xB6, 0xAD, 0x64, 0xEC, 0x2B, 0x6B, 0xB0, 0x52, 0x96, 0x42, 0x08, 0x05, 0x0E, 0xFA, 0x57, 0x62, 0xE1, 0xB2, 0x4B, 0x1F, 0xFC, 0xD8, 0xD9, 0x4A, 0x63, 0xF4, 0x5C, 0xA3, 0xC0, 0x83, 0x8D, 0x5F, 0x98, 0xB2, 0x6D, 0x90, 0x7D, 0xA2, 0xF5, 0xF6, 0x70, 0x6A, 0xD7, 0x5F, 0x43, 0xE9, 0x75, 0xD7, 0x06, 0xFD, 0x6A, 0xCB, 0x1D, 0xEF, 0x0B, 0xE4, 0xB8, 0x7D, 0x17, 0x47, 0x77, 0xDC, 0x52, 0x70, 0x3B, 0xD2, 0xE8, 0x92, 0x6B, 0xEE, 0x9A, 0x48, 0xA6, 0x07, 0xDD, 0x35, 0x5B, 0x83, 0xCF, 0xA3, 0x48, 0xDC, 0xC3, 0xAB, 0x32, 0xA2, 0x5F, 0x7C, 0xDD, 0x22, 0x44, 0xCE, 0x65, 0x92, 0x63, 0x6B, 0x6B, 0x98, 0x50, 0xED, 0x28, 0xB3, 0x8E, 0x41, 0x34, 0x38, 0xDE, 0x1F, 0x61, 0xF7, 0x37, 0xD2, 0xB3, 0x4C, 0x27, 0xA2, 0x3E, 0x79, 0x1D, 0xBD, 0x10, 0xEF, 0x5F, 0xBB, 0xDD, 0x9B, 0x0D, 0x40, 0xDA, 0x2F, 0xBD, 0x72, 0xA5, 0x28, 0x61, 0x38, 0xE7, 0x96, 0x1C, 0x12, 0xFE, 0x30, 0x32, 0xDB, 0xF8, 0x37, 0x85, 0xF6, 0x99, 0xCC, 0xB6, 0x24, 0x24, 0x3C, 0x7D, 0x98, 0xCA, 0x25, 0xEC, 0x84, 0xFE, 0xF7, 0xC6, 0x03, 0x32, 0x52, 0x1B, 0xC2, 0xF6, 0x55, 0xA5, 0x88, 0xB7, 0x81, 0x59, 0x79, 0x41, 0x5E, 0x55, 0x4C, 0x05, 0x21, 0xE5, 0x1C, 0x98, 0xAE, 0x09, 0x89, 0x6F, 0x1A, 0x27, 0xD7, 0x03, 0x81, 0xCB, 0xA7, 0x0B, 0x01, 0xCE, 0x11, 0xA3, 0x6D, 0xE2, 0x48, 0xF4, 0x78, 0x6F, 0xCF, 0xDA, 0x6E, 0xD8, 0xAE, 0x1B, 0xBC, 0x84, 0xD3, 0x0B, 0xE9, 0x8B, 0xCA, 0x6A, 0xA0, 0xC2, 0x9D, 0x68, 0x3E, 0x6A, 0x3A, 0x61, 0x96, 0x1F, 0x09, 0xD8, 0x6E, 0x2C, 0xBE, 0x87, 0x23, 0x94, 0x26, 0x4F, 0xFA, 0x8D, 0xD7, 0x82, 0x4E, 0x59, 0x38, 0x4B, 0x4D, 0xA0, 0x20, 0x4F, 0x98, 0xEB, 0x47, 0x6F, 0xF3, 0xA2, 0xE6, 0xBD, 0xBE, 0x29, 0xC4, 0x92, 0xD8, 0x37, 0x15, 0xD4, 0x52, 0xC4, 0xF5, 0xD2, 0x70, 0x45, 0xFD, 0xF3, 0xD4, 0x23, 0x4E, 0xB4, 0x29, 0xE4, 0x66, 0xBB, 0x5A, 0x12, 0xF3, 0x25, 0xD2, 0xED, 0x77, 0xB2, 0x51, 0xC8, 0xD3, 0xA1, 0x35, 0x9D, 0xC2, 0xB3, 0x0D, 0x40, 0x12, 0xFC, 0x63, 0x9B, 0xED, 0x46, 0xA9, 0x6A, 0xD0, 0x70, 0x0B, 0xCA, 0xCC, 0xA8, 0xE4, 0x40, 0x04, 0x64, 0xB2, 0x7E, 0x9D, 0x4A, 0xAB, 0x85, 0x29, 0xE6, 0x6D, 0x0E, 0xE7, 0xA0, 0x9A, 0x3A, 0x10, 0x08, 0x87, 0x8E, 0xCD, 0xAB, 0xEE, 0xB1, 0x02, 0x60, 0x37, 0xDD, 0x42, 0x18, 0xCB, 0xB4, 0x6E, 0xB0, 0x31, 0x92, 0x42, 0x3F, 0x14, 0xF9, 0xC8, 0x5F, 0x3C, 0x73, 0x0F, 0x5C, 0x62, 0xCB, 0x60, 0xCF, 0x9B, 0xFE, 0x09, 0xA5, 0x9A, 0xC4, 0x54, 0x3F, 0x93, 0xE8, 0x32, 0x5E, 0xDB, 0xD1, 0x68, 0xB1, 0xAB, 0xC5, 0x09, 0xC7, 0x2E, 0x5A, 0xED, 0x3E, 0x39, 0xD0, 0xCE, 0xBE, 0xC0, 0x06, 0x25, 0x70, 0x2A, 0x79, 0x1C, 0xB2, 0x06, 0x93, 0xB8, 0xE7, 0x64, 0xE8, 0xF2, 0x84, 0x25, 0xB8, 0x33, 0x74, 0xDA, 0x9C, 0x57, 0xCD, 0x90, 0xC8, 0x06, 0xED, 0xCF, 0xDB, 0xF2, 0x97, 0x77, 0x7D, 0xCD, 0xE8, 0xE0, 0x5E, 0x3F, 0x0B, 0x85, 0xAA, 0x51, 0xFC, 0xD4, 0x29, 0xD8, 0xE9, 0xE5, 0x10, 0x21, 0xBD, 0x8C, 0x2C, 0xCB, 0xAC, 0x6F, 0x51, 0x90, 0x70, 0x8B, 0x64, 0x89, 0xEA, 0x23, 0xA8, 0xFD, 0xAA, 0xFA, 0x97, 0xE5, 0x65, 0x03, 0xEB, 0x55, 0x81, 0x97, 0x3D, 0x42, 0xDF, 0x26, 0x12, 0x20, 0xF2, 0xE0, 0xB5, 0x9C, 0x06, 0x79, 0x47, 0x24, 0x81, 0xD4, 0x4B, 0x85, 0xED, 0x2A, 0x80, 0x18, 0xB2, 0x9B, 0x0E, 0x9E, 0x8B, 0xCC, 0x47, 0x42, 0x40, 0x13, 0x24, 0xB6, 0x3B, 0xDC, 0x82, 0x1C, 0x82, 0x1A, 0x17, 0xF6, 0x0F, 0xE1, 0x8D, 0x2A, 0xB3, 0xE9, 0xC3, 0x55, 0x21, 0xCE, 0x3E, 0x82, 0xF5, 0x25, 0xEF, 0x3F, 0xC7, 0x9F, 0x2D, 0xEA, 0xE2, 0xF8, 0xA8, 0x6F, 0x1E, 0xDA, 0x44, 0x34, 0xF0, 0x3D, 0x43, 0x30, 0x6B, 0x3D, 0xD8, 0x6C, 0xE7, 0xD2, 0x0E, 0xA2, 0x5A, 0xEA, 0x77, 0x4B, 0xA4, 0x4F, 0x1E, 0x85, 0xE9, 0x93, 0x75, 0x71, 0xF2, 0xD0, 0xB7, 0x14, 0xF6, 0x45, 0x04, 0x63, 0xA4, 0x5F, 0xC8, 0xE9, 0x6B, 0x08, 0xCE, 0x63, 0x4C, 0x62, 0xA7, 0xB7, 0x10, 0xE0, 0x68, 0xF9, 0x2D, 0x76, 0x9A, 0xB2, 0x02, 0x67, 0xCB, 0x73, 0x54, 0xE4, 0x51, 0x87, 0x8A, 0x64, 0xEC, 0xD5, 0x74, 0x20, 0xD6, 0xEB, 0xF9, 0xC8, 0x87, 0x1B, 0xB2, 0xB7, 0x76, 0xFB, 0xA7, 0x4F, 0xB9, 0xF4, 0x48, 0x68, 0x54, 0x2A, 0xE0, 0xD6, 0xDC, 0x9F, 0xB6, 0xB4, 0x9A, 0x9A, 0x98, 0xF0, 0xE4, 0x2D, 0xC1, 0xF3, 0x27, 0x97, 0x51, 0x6B, 0x05, 0x47, 0x8F, 0xF7, 0x50, 0xE5, 0x5D, 0xB7, 0x23, 0x5E, 0x03, 0x0F, 0x48, 0x59, 0x61, 0x0D, 0x4D, 0x15, 0xD9, 0x9D, 0xB1, 0x8F, 0xA7, 0xBB, 0x73, 0x53, 0xDB, 0x2D, 0xE6, 0x8B, 0x27, 0x52, 0x46, 0xEE, 0x54, 0x61, 0x0B, 0x7D, 0xEA, 0xFB, 0x64, 0x57, 0xDF, 0x3C, 0x5D, 0x26, 0x68, 0x58, 0x9D, 0x45, 0x24, 0x72, 0x59, 0x4D, 0xAC, 0x95, 0x27, 0x10, 0x33, 0x5F, 0x7F, 0x29, 0xDE, 0x98, 0xDA, 0x20, 0x56, 0xB9, 0xEF, 0x14, 0x07, 0x9C, 0xDB, 0x5D, 0xFD, 0x99, 0xA4, 0x05, 0xA2, 0xC5, 0x00, 0xD6, 0x94, 0x9B, 0x8B, 0x95, 0x88, 0x74, 0xE0, 0xF0, 0x25, 0x14, 0x87, 0x18, 0x52, 0xFF, 0x63, 0x08, 0x42, 0x6F, 0x06, 0x44, 0x1E, 0x62, 0xF5, 0xCB, 0xAC, 0x8C, 0xF8, 0x88, 0xE3, 0x70, 0x29, 0x81, 0xCC, 0xD7, 0x98, 0xEA, 0xC7, 0x4B, 0xEA, 0xDF, 0xF6, 0x84, 0x63, 0xAD, 0xB9, 0x7C, 0xFC, 0x64, 0xD0, 0x4A, 0x45, 0x18, 0x32, 0xEA, 0x44, 0x7F, 0x64, 0xF6, 0x2B, 0xD4, 0xCE, 0xF8, 0x2A, 0xB0, 0x84, 0x60, 0x90, 0xC0, 0xB7, 0x47, 0xD9, 0x77, 0xE3, 0x62, 0x17, 0x45, 0xBF, 0x22, 0x54, 0x49, 0x2A, 0x9B, 0xC7, 0xA5, 0x16, 0x3C, 0x85, 0xC8, 0x14, 0x60, 0xCE, 0x38, 0x8D, 0xA3, 0x51, 0x0D, 0x07, 0xCC, 0xB8, 0x29, 0xE6, 0x8C, 0xE0, 0xE0, 0xD6, 0xB7, 0x5E, 0x07, 0x3A, 0xEB, 0x1C, 0xE9, 0xA3, 0x69, 0x06, 0x27, 0x4C, 0xF1, 0x8C, 0xE2, 0x9C, 0xD4, 0x77, 0xAE, 0xE2, 0x12, 0x26, 0x78, 0x58, 0x80, 0xA9, 0x4E, 0x8C, 0x84, 0xAC, 0x87, 0x1B, 0x8F, 0xF9, 0xF4, 0x61, 0x10, 0x6C, 0xA8, 0xA4, 0x83, 0x15, 0x0E, 0x50, 0xF3, 0x71, 0xDF, 0x73, 0xF4, 0x46, 0x9E, 0x66, 0x59, 0x7E, 0x27, 0x40, 0x5E, 0x42, 0x3D, 0x4D, 0x22, 0x3E, 0xC5, 0x94, 0x75, 0xDB, 0x8B, 0x07, 0x2E, 0xB0, 0x74, 0xC1, 0xCE, 0xB3, 0x4B, 0xAE, 0x70, 0x3A, 0xE6, 0x16, 0x16, 0x14, 0xF4, 0xED, 0xBA, 0x14, 0x83, 0x9A, 0x30, 0x90, 0x87, 0xC4, 0x0C, 0xB4, 0xA4, 0x2E, 0x66, 0x62, 0xE4, 0x5D, 0x0C, 0x2E, 0x8F, 0x2B, 0x3C, 0x8E, 0xB1, 0x48, 0xDC, 0xC0, 0x70, 0x10, 0xDC, 0x62, 0xB1, 0x1E, 0xE4, 0x70, 0x7F, 0x41, 0xB1, 0xAC, 0xBE, 0x1D, 0xA2, 0x00, 0x13, 0xD4, 0xBD, 0x95, 0xB0, 0x33, 0xBE, 0xC8, 0x0F, 0x0F, 0xFA, 0x38, 0xA4, 0x5D, 0x63, 0xD9, 0x64, 0x92, 0x6D, 0xEA, 0xF8, 0x31, 0x48, 0x63, 0x29, 0x89, 0xF1, 0x50, 0xAC, 0xBF, 0xFC, 0x8E, 0x77, 0x65, 0xCA, 0x65, 0xEF, 0x65, 0x04, 0x05, 0x3E, 0x6B, 0x09, 0x9A, 0x41, 0x20, 0x71, 0xAF, 0x36, 0x8E, 0x99, 0xC8, 0x1B, 0xF2, 0xEC, 0x8E, 0xB8, 0x6A, 0xBC, 0x89, 0x1B, 0x43, 0x99, 0xF8, 0x28, 0x30, 0xE5, 0x7E, 0xE8, 0x26, 0xFE, 0xBA, 0x21, 0x86, 0x37, 0xC4, 0xAB, 0x1D, 0x88, 0xF7, 0x7F, 0x78, 0x58, 0xE7, 0xA5, 0x02, 0xA6, 0xEC, 0x8B, 0x18, 0x26, 0x8A, 0x15, 0x8F, 0x83, 0xDB, 0x19, 0x35, 0x59, 0xCA, 0xFE, 0x96, 0xBE, 0x61, 0xAE, 0x10, 0x46, 0xA4, 0x54, 0x15, 0xE2, 0x18, 0x74, 0x96, 0x6A, 0x3F, 0x9A, 0x98, 0xE7, 0xE3, 0x19, 0x41, 0x2D, 0x22, 0x1C, 0x14, 0xF3, 0x4F, 0x33, 0xE2, 0x02, 0xD2, 0x44, 0x9E, 0x3C, 0x21, 0x4C, 0x8C, 0x9B, 0x09, 0xF8, 0x44, 0x23, 0x32, 0x7D, 0x26, 0xA6, 0x44, 0x35, 0xFA, 0x45, 0x5F, 0x44, 0xFF, 0x1E, 0x33, 0x58, 0x63, 0xE2, 0x6D, 0xBB, 0x84, 0xD7, 0x68, 0xD8, 0x8C, 0x1E, 0x6B, 0x36, 0x59, 0xCC, 0x6E, 0x59, 0x2B, 0xE2, 0x10, 0x53, 0x3B, 0x05, 0x25, 0xAB, 0x24, 0x41, 0xC1, 0xBD, 0x68, 0xBB, 0xE7, 0x2A, 0xBF, 0x9F, 0x64, 0x7E, 0xC9, 0x21, 0x21, 0x6C, 0x6B, 0x77, 0x27, 0x87, 0x7B, 0xF6, 0xD7, 0x24, 0x68, 0xB4, 0x83, 0xD5, 0xDB, 0x85, 0x41, 0xAA, 0x8A, 0x5E, 0x10, 0x95, 0xED, 0x7A, 0x8F, 0xEB, 0x6F, 0xEF, 0x67, 0xF5, 0xB1, 0x48, 0x1A, 0x99, 0x4C, 0xA7, 0xB3, 0xB0, 0xBE, 0x88, 0x90, 0x1C, 0x8D, 0xD0, 0x4A, 0xEF, 0xA5, 0x86, 0x6F, 0xA6, 0x07, 0x3F, 0x76, 0x1C, 0xCD, 0x88, 0xED, 0x1A, 0x1B, 0x63, 0x5C, 0x84, 0x0F, 0x59, 0xC7, 0x4E, 0x09, 0x86, 0x4B, 0xD7, 0x41, 0x07, 0xAA, 0x83, 0x31, 0x87, 0xBA, 0x27, 0xC0, 0xA5, 0x7D, 0xBC, 0x0A, 0xD6, 0x1A, 0x2A, 0x6F, 0x3E, 0xFA, 0x33, 0x67, 0x17, 0xF8, 0x86, 0x91, 0x22, 0xF2, 0xF8, 0x22, 0x8B, 0xC0, 0x9B, 0xB9, 0xA7, 0xD6, 0xBD, 0xB4, 0xB4, 0xE5, 0x29, 0x7F, 0x00, 0xDF, 0xEC, 0x9B, 0xA1, 0x82, 0xB7, 0x01, 0x6C, 0x9B, 0x95, 0xAD, 0xBB, 0x4B, 0x43, 0x42, 0x78, 0x54, 0x56, 0x79, 0xC6, 0xD7, 0x13, 0x93, 0x06, 0xB2, 0xDB, 0xBB, 0xC0, 0x0A, 0x05, 0xA5, 0xBC, 0x96, 0x45, 0x66, 0xFE, 0x39, 0xD0, 0xB4, 0x55, 0x10, 0x6D, 0x30, 0xAB, 0xE9, 0x49, 0xF6, 0xA7, 0x8B, 0xBD, 0xC5, 0x64, 0xDE, 0x9C, 0xCB, 0x79, 0x10, 0x85, 0xD5, 0xF5, 0xBA, 0x7F, 0x4A, 0xF0, 0x18, 0xDE, 0x15, 0xEC, 0x7F, 0x2D, 0x45, 0x3C, 0xD2, 0x4E, 0x46, 0x19, 0x13, 0xE3, 0x7E, 0x42, 0x7F, 0xBD, 0x78, 0x96, 0x5D, 0x41, 0xA7, 0x87, 0xCD, 0xBD, 0x0A, 0x12, 0x31, 0xDB, 0x37, 0x08, 0xAF, 0x68, 0xE4, 0x7F, 0x79, 0xFE, 0x35, 0x75, 0x14, 0x46, 0x56, 0x7B, 0x12, 0x01, 0xC3, 0x35, 0xA4, 0x88, 0x96, 0x3D, 0xFE, 0x33, 0x04, 0xD6, 0xAE, 0x2B, 0x8D, 0x14, 0xA8, 0x2F, 0x3C, 0x79, 0xA0, 0xC6, 0x9E, 0x05, 0x83, 0xD9, 0x33, 0xE9, 0x1F, 0x70, 0x8A, 0x0F, 0x1A, 0x6D, 0xB0, 0xDB, 0xFE, 0xEE, 0x3E, 0x08, 0xD4, 0xAE, 0x61, 0x07, 0x98, 0x85, 0x97, 0x9A, 0x91, 0x80, 0x29, 0xF9, 0x5F, 0x44, 0x6E, 0x1C, 0xF3, 0x55, 0x7B, 0x5D, 0x47, 0xE9, 0x82, 0xD1, 0x35, 0xAE, 0x1D, 0x54, 0xBB, 0xCF, 0x19, 0x65, 0x0B, 0x5E, 0x46, 0xCC, 0x00, 0x24, 0x96, 0xEB, 0x54, 0xD2, 0xDA, 0xF9, 0xDF, 0x0D, 0x92, 0x5A, 0x99, 0x9F, 0xDF, 0xF5, 0x2E, 0x87, 0x8D, 0x8D, 0x75, 0x0F, 0x63, 0x8B, 0x8C, 0x29, 0x88, 0xD4, 0xEE, 0x31, 0x91, 0x79, 0x14, 0x34, 0xFE, 0xB8, 0xF2, 0x47, 0x2D, 0xA3, 0x6F, 0x39, 0x5D, 0x27, 0x0E, 0x75, 0xFE, 0x1B, 0xEA, 0xFB, 0x6E, 0x08, 0x51, 0xC9, 0x01, 0xD4, 0xA7, 0x64, 0x2F, 0xF6, 0x5C, 0x31, 0xC9, 0x17, 0xC0, 0xE5, 0x5D, 0xF9, 0x12, 0x13, 0xF4, 0x47, 0x35, 0xC3, 0x4B, 0xBF, 0xEB, 0x0F, 0xCB, 0xF6, 0x8D, 0x36, 0x75, 0x85, 0x9A, 0x94, 0x71, 0x8E, 0x59, 0x25, 0xD9, 0xC7, 0xDD, 0x0B, 0x6F, 0x0F, 0xE2, 0xCE, 0x11, 0x49, 0x41, 0xD7, 0xEE, 0x67, 0x7B, 0x0D, 0xB7, 0x34, 0xDD, 0x17, 0xFC, 0xED, 0x57, 0x37, 0xE9, 0x81, 0x58, 0xC9, 0x84, 0x19, 0xFD, 0xAA, 0x43, 0x82, 0x2D, 0x1A, 0x02, 0x6E, 0xA5, 0xDA, 0xE7, 0x92, 0x1D, 0x39, 0xD0, 0x72, 0x00, 0x82, 0x7E, 0xC4, 0x43, 0xD3, 0x07, 0x04, 0xC0, 0x82, 0xBE, 0x0C, 0x3A, 0x93, 0x6F, 0x15, 0x1C, 0x21, 0x4B, 0x0C, 0x6A, 0xC2, 0x80, 0x69, 0xC8, 0x90, 0x35, 0x25, 0x27, 0x3C, 0xD3, 0x17, 0x0C, 0xAA, 0x70, 0xAD, 0x35, 0xEE, 0x80, 0x24, 0x0A, 0xD8, 0xC2, 0xF9, 0x21, 0x42, 0x72, 0xB9, 0x29, 0xC8, 0x0D, 0xBC, 0x7A, 0xEE, 0x6B, 0x02, 0x72, 0x39, 0x33, 0x52, 0xFB, 0x07, 0x21, 0xAC, 0x7D, 0xC0, 0xBA, 0x01, 0x10, 0x6E, 0xEE, 0xC9, 0x7D, 0x96, 0xDA, 0xA3, 0x0A, 0xDD, 0xFC, 0xAD, 0x82, 0x8F, 0x1F, 0xB0, 0xC6, 0x75, 0x4C, 0xE8, 0xC5, 0x47, 0x31, 0x86, 0xA8, 0x76, 0x01, 0x3B, 0x71, 0x7F, 0x1F, 0x92, 0xD0, 0x36, 0x21, 0x74, 0xD6, 0xC9, 0x7A, 0x4F, 0xB2, 0x36, 0xD5, 0xCA, 0x2F, 0x48, 0x36, 0x4A, 0xD4, 0x5F, 0xAD, 0xEA, 0x36, 0x33, 0x03, 0x94, 0xDE, 0x52, 0x69, 0x98, 0x19, 0x13, 0x77, 0x17, 0xEA, 0xD7, 0xBE, 0x32, 0xC3, 0x46, 0x8E, 0x7C, 0xAE, 0xA3, 0x3F, 0x41, 0xF5, 0xA5, 0xAB, 0x85, 0x25, 0x44, 0x0B, 0x48, 0x63, 0x6C, 0xE8, 0x74, 0x4D, 0x28, 0x4B, 0x82, 0x4E, 0x08, 0x9B, 0x2A, 0x7F, 0xB2, 0xCA, 0x5F, 0xEF, 0x7B, 0xF6, 0xA4, 0x80, 0xA1, 0xC0, 0x13, 0x6A, 0x28, 0x7E, 0xCF, 0xF3, 0x94, 0x94, 0x47, 0xC4, 0x0C, 0x05, 0xB2, 0x8D, 0x48, 0xBF, 0xF2, 0xF2, 0x0B, 0x16, 0xBA, 0x7A, 0x3D, 0x3F, 0xCF, 0x13, 0x3A, 0x02, 0xFA, 0xEA, 0xF3, 0x05, 0x8E, 0x19, 0x58, 0x3E, 0xED, 0x42, 0x01, 0x52, 0xEC, 0xAA, 0x92, 0x9A, 0x59, 0xFA, 0x4E, 0xC3, 0xDD, 0x56, 0xDD, 0x3B, 0x80, 0x36, 0x14, 0xC1, 0x00, 0x00, 0x09, 0x52, 0x6C, 0x82, 0xEE, 0x2C, 0xAA, 0xEF, 0x08, 0xCB, 0x76, 0x95, 0x5A, 0x2F, 0xC0, 0xD2, 0x56, 0xB3, 0x45, 0x5B, 0x7F, 0xCB, 0x2F, 0x76, 0xA8, 0xED, 0xB4, 0x45, 0x4F, 0x7D, 0x74, 0x9E, 0x8A, 0x85, 0x45, 0xC8, 0xDC, 0x76, 0xEF, 0x6F, 0xA7, 0x39, 0x01, 0xA3, 0x35, 0x62, 0x26, 0x42, 0x4A, 0xA0, 0x76, 0xAB, 0x15, 0x6E, 0xCB, 0x3A, 0xF8, 0xFE, 0xAA, 0x8B, 0x8C, 0xCB, 0x38, 0x3F, 0x12, 0x35, 0x97, 0x2C, 0x95, 0xCF, 0xE3, 0xDA, 0xF7, 0x7F, 0xDB, 0x1F, 0xC7, 0x58, 0x2F, 0xEE, 0xAF, 0x3F, 0xD4, 0x4A, 0x8A, 0x51, 0xFA, 0xF2, 0x75, 0x77, 0xDA, 0x77, 0xB0, 0xE1, 0x0B, 0xE4, 0xC2, 0x93, 0xC9, 0xD1, 0x24, 0x8B, 0x28, 0x3A, 0x11, 0x7F, 0x3E, 0xBF, 0xC7, 0x79, 0xA0, 0xF2, 0x32, 0x46, 0x0F, 0x12, 0xCA, 0xED, 0x07, 0xC1, 0xB6, 0x35, 0x03, 0x96, 0x11, 0xAF, 0x1D, 0x7F, 0x48, 0x14, 0x98, 0x43, 0xEB, 0x0B, 0x53, 0x11, 0x2C, 0x8E, 0xAE, 0x96, 0x19, 0x1C, 0x18, 0xC5, 0x88, 0xF3, 0x6E, 0x41, 0x84, 0xC6, 0xE7, 0xB4, 0x0C, 0xF9, 0x53, 0x86, 0x11, 0x1E, 0xEF, 0x5F, 0x42, 0x78, 0x88, 0xD3, 0xA2, 0xEE, 0xFD, 0x50, 0xA0, 0x33, 0x91, 0x74, 0x71, 0x74, 0x25, 0x24, 0xDC, 0x74, 0xC3, 0x49, 0x89, 0xB6, 0xF3, 0x42, 0x3D, 0xA8, 0x56, 0xD1, 0xB0, 0x59, 0xEA, 0xB8, 0xA8, 0x43, 0x44, 0x74, 0xF1, 0xED, 0x79, 0xE3, 0x87, 0xCC, 0xC5, 0xA9, 0x98, 0xB4, 0x0A, 0x22, 0x25, 0x83, 0x09, 0x2E, 0x78, 0xAB, 0x50, 0x8A, 0x5C, 0x80, 0xB6, 0x5F, 0xE1, 0x1D, 0x1F, 0xF6, 0x33, 0x02, 0x3E, 0x4A, 0xA1, 0xFC, 0xF6, 0x55, 0x3D, 0x3F, 0x21, 0x34, 0x8F, 0x2A, 0xF4, 0x13, 0x11, 0x92, 0x74, 0x19, 0xAF, 0x80, 0xF4, 0x99, 0x8B, 0x48, 0x65, 0x09, 0x55, 0x5D, 0xC9, 0xA1, 0xC5, 0xA0, 0xD6, 0xD0, 0x20, 0x87, 0x91, 0xC6, 0x6F, 0x89, 0xB0, 0xA3, 0xA2, 0x66, 0x36, 0x59, 0xCC, 0xD2, 0xD0, 0x81, 0x6E, 0x4F, 0x64, 0xAC, 0x4B, 0xAA, 0x4E, 0xF9, 0x9E, 0xC1, 0x57, 0x2A, 0x2F, 0xA8, 0xE8, 0x8B, 0xE3, 0x9C, 0x11, 0x45, 0xF1, 0x65, 0xCB, 0x91, 0x13, 0xF2, 0xC4, 0x7B, 0x62, 0x32, 0x93, 0xF7, 0x25, 0x36, 0x47, 0x1D, 0xA1, 0x02, 0x35, 0x97, 0x80, 0x0C, 0x4D, 0xAB, 0x39, 0x8C, 0x48, 0x71, 0x40, 0x21, 0xA5, 0x59, 0x1C, 0x8D, 0x24, 0x64, 0x6C, 0x34, 0xEA, 0xE6, 0x63, 0x0D, 0x5D, 0xEF, 0x1D, 0xFB, 0x2D, 0x4B, 0xC4, 0xAB, 0xF3, 0x6D, 0x56, 0x3B, 0x8C, 0xDC, 0xFF, 0xED, 0xEA, 0x78, 0xDD, 0x65, 0x94, 0x29, 0x5C, 0x00, 0x92, 0x42, 0xEA, 0xB5, 0x2E, 0xC5, 0x2E, 0xED, 0xF5, 0x85, 0x80, 0x99, 0x3E, 0x91, 0xFC, 0xC7, 0xF1, 0x00, 0xED, 0x01, 0xC0, 0x16, 0xAE, 0x9B, 0x87, 0x9D, 0xB5, 0x48, 0x3A, 0x39, 0x8E, 0x93, 0xC0, 0x9D, 0xFC, 0xDF, 0x2E, 0xBA, 0xE2, 0x5F, 0xCB, 0x54, 0xA9, 0x70, 0xAD, 0xC0, 0xDA, 0xD5, 0xB1, 0x5A, 0xAB, 0x2D, 0xFC, 0x14, 0x84, 0x95, 0xDB, 0xA8, 0x7C, 0x21, 0x8D, 0xFE, 0xED, 0xE2, 0xCE, 0x70, 0x8E, 0x92, 0xFC, 0x10, 0x76, 0x33, 0xB3, 0x4C, 0x1C, 0xAB, 0x21, 0x52, 0x0E, 0xBD, 0x59, 0x91, 0x07, 0x84, 0xF3, 0x88, 0xAB, 0xDD, 0xBD, 0x2A, 0x3A, 0x1D, 0x26, 0xF4, 0x3E, 0x9D, 0x17, 0x62, 0x05, 0xBF, 0xC5, 0x8A, 0x06, 0x0C, 0x7F, 0x2B, 0x48, 0xB5, 0x30, 0x0B, 0x2F, 0x01, 0x8C, 0x55, 0xC9, 0x8F, 0x85, 0x9F, 0x27, 0xCC, 0xAC, 0x1F, 0x32, 0x41, 0xA8, 0x91, 0xBE, 0x71, 0x86, 0x97, 0x7A, 0x7C, 0x3F, 0x5D, 0x2A, 0x97, 0xA2, 0x47, 0x39, 0x65, 0xD8, 0x5C, 0xC6, 0x7E, 0x41, 0x97, 0xA0, 0xDD, 0x49, 0xD3, 0xC7, 0xC8, 0x96, 0x34, 0x51, 0x0D, 0xE2, 0x45, 0x00, 0xC9, 0x54, 0x70, 0x87, 0x6D, 0x4C, 0x7C, 0x9B, 0x50, 0x42, 0xF3, 0x3B, 0x11, 0x92, 0x7C, 0xF2, 0x03, 0x3B, 0x84, 0xD7, 0xBA, 0x8D, 0x35, 0x6F, 0xD4, 0xCD, 0x71, 0xBC, 0xCA, 0xF9, 0x98, 0x83, 0x62, 0x53, 0x34, 0x68, 0xC4, 0x6D, 0x3A, 0x86, 0x55, 0x51, 0x24, 0xA5, 0x37, 0x9C, 0x04, 0x07, 0x66, 0x33, 0x18, 0x54, 0xA7, 0x2C, 0xCF, 0x7B, 0x9A, 0x9C, 0xFF, 0xA0, 0x8D, 0xCC, 0x51, 0x33, 0x56, 0xB8, 0x7B, 0xFD, 0x3F, 0x51, 0x91, 0xCA, 0xEE, 0x66, 0xFE, 0xDB, 0x14, 0xCB, 0x56, 0xA7, 0xE4, 0x7C, 0xC1, 0xCB, 0x49, 0xE2, 0x9F, 0x3B, 0x08, 0x83, 0x5F, 0x01, 0x78, 0x8B, 0x01, 0x77, 0x79, 0x82, 0x85, 0x22, 0x57, 0x46, 0xD8, 0xF7, 0x36, 0xA0, 0x4A, 0xD6, 0x11, 0x4D, 0xFF, 0x3F, 0xCA, 0x74, 0x63, 0x5B, 0x60, 0xFB, 0xD9, 0x20, 0x31, 0x54, 0x32, 0xC8, 0x1A, 0xDA, 0x0B, 0x35, 0xF6, 0xF7, 0xBD, 0xE6, 0x54, 0x64, 0x09, 0xA5, 0x9F, 0xE3, 0x0F, 0x65, 0x82, 0x57, 0xB5, 0x51, 0x64, 0xC5, 0x0C, 0xFF, 0x6C, 0x43, 0xD1, 0xA0, 0x46, 0xDF, 0xDF, 0x53, 0x5F, 0xD8, 0x78, 0x93, 0xE7, 0xB5, 0xC5, 0xFB, 0xD4, 0x43, 0xD7, 0xAC, 0x08, 0x11, 0x0F, 0xC8, 0xC3, 0xFA, 0x8E, 0xB2, 0xD1, 0x73, 0x1A, 0x5A, 0x53, 0xFA, 0x8C, 0x31, 0xF4, 0xC9, 0xF7, 0x47, 0x6A, 0x67, 0x25, 0x86, 0xFE, 0xF7, 0x8D, 0xFE, 0x83, 0x6C, 0x86, 0xFE, 0x7D, 0x49, 0x69, 0x8A, 0xA9, 0xC8, 0x73, 0xE0, 0x2A, 0x29, 0x9D, 0x46, 0xB5, 0x05, 0xCD, 0xDF, 0xEC, 0xDB, 0xEB, 0x91, 0xC2, 0x28, 0xC0, 0xA4, 0x02, 0x8D, 0xC0, 0x8E, 0x2D, 0xED, 0x0E, 0xCD, 0x15, 0x35, 0x8E, 0xD5, 0x9B, 0xB2, 0x24, 0x46, 0x29, 0xE6, 0x7A, 0x40, 0x37, 0x7A, 0x4A, 0x7B, 0xDB, 0xB8, 0xD9, 0x40, 0xF8, 0xD1, 0x47, 0x08, 0xAE, 0x07, 0x68, 0xC6, 0x19, 0xAA, 0xF0, 0xE4, 0x40, 0x49, 0x7E, 0x07, 0x20, 0xF8, 0x00, 0xAF, 0x05, 0xD8, 0xD8, 0x57, 0xE9, 0x5C, 0x64, 0x8D, 0xF1, 0x64, 0x6F, 0x46, 0x2B, 0x4E, 0x12, 0x4B, 0x22, 0xCF, 0x1A, 0xDB, 0x17, 0x84, 0x31, 0x73, 0x8A, 0xA6, 0xF8, 0xA8, 0x03, 0xF7, 0x63, 0x57, 0x4E, 0x6E, 0xBC, 0x03, 0x03, 0xFC, 0x48, 0xE9, 0xA3, 0x79, 0x00, 0x6C, 0x04, 0x65, 0xC6, 0x25, 0x4F, 0x4C, 0x4A, 0xA9, 0x53, 0xA8, 0xE0, 0x63, 0x8C, 0x0B, 0x48, 0x7B, 0x96, 0x1C, 0x18, 0x70, 0xE8, 0x14, 0x10, 0xDE, 0xF0, 0x06, 0x57, 0xAD, 0xE3, 0x14, 0xF5, 0x42, 0x19, 0x92, 0x53, 0x23, 0x4C, 0xDB, 0xC3, 0xDB, 0x34, 0x2D, 0x9D, 0x86, 0x55, 0x71, 0x39, 0xC2, 0xC3, 0xF0, 0x12, 0x6E, 0x01, 0x93, 0xA8, 0x90, 0x2D, 0x49, 0x6D, 0xC9, 0x73, 0x08, 0xED, 0x20, 0x1E, 0x9C, 0xC7, 0xBA, 0x46, 0x75, 0xDF, 0x43, 0xFB, 0x02, 0x5D, 0xD5, 0x51, 0xD4, 0x8B, 0xE7, 0xE5, 0x2B, 0x64, 0x83, 0x9F, 0xB9, 0x38, 0x4F, 0x71, 0x3E, 0x42, 0x2C, 0x8A, 0x6C, 0x49, 0x75, 0xD4, 0xEA, 0x15, 0xA7, 0x19, 0xEE, 0x07, 0x14, 0x4B, 0xCE, 0xD5, 0xAE, 0x2B, 0x5E, 0x01, 0x9A, 0x6A, 0x29, 0xDD, 0xAF, 0x78, 0xB8, 0x3D, 0x61, 0x14, 0xBA, 0x80, 0x7A, 0x24, 0x39, 0xAF, 0x85, 0x43, 0x4C, 0x37, 0x2F, 0x70, 0xA4, 0xA3, 0x35, 0x96, 0x78, 0xBC, 0x73, 0xF9, 0xB3, 0x7C, 0x6A, 0x12, 0x5C, 0x98, 0xC7, 0x0E, 0x81, 0x96, 0x87, 0x65, 0x25, 0x62, 0x95, 0x44, 0x72, 0xFE, 0x61, 0x0E, 0x46, 0xDE, 0xE2, 0xDD, 0x2D, 0xA5, 0x44, 0xA3, 0x76, 0x98, 0xA2, 0xA7, 0xD9, 0x50, 0xF0, 0x98, 0x00, 0x4B, 0x50, 0xEA, 0x5F, 0x5A, 0x3F, 0xEF, 0xBE, 0xDC, 0x82, 0xB0, 0x32, 0x99, 0xF8, 0xAA, 0x1C, 0x48, 0x90, 0x60, 0xE3, 0x48, 0x8B, 0x7B, 0xAF, 0x47, 0x5C, 0x5A, 0x4A, 0x4A, 0x33, 0xE0, 0x26, 0x46, 0x33, 0x2D, 0x67, 0xE0, 0x49, 0x08, 0x24, 0x3F, 0x26, 0xE0, 0x01, 0xA4, 0xAC, 0xA9, 0x44, 0xAC, 0x89, 0x04, 0xBB, 0x27, 0xB4, 0x87, 0x85, 0xE4, 0x31, 0x14, 0x05, 0x82, 0xFF, 0x9B, 0xDA, 0x89, 0xD6, 0x11, 0xED, 0x5E, 0x04, 0x52, 0x4D, 0xF7, 0x33, 0x41, 0x9D, 0x3A, 0xDD, 0x9B, 0x29, 0x97, 0x9C, 0x0F, 0xD8, 0x01, 0x14, 0x68, 0xBA, 0x76, 0x38, 0x8A, 0x72, 0xA5, 0x28, 0xA3, 0xAF, 0xC0, 0xB9, 0x46, 0x9C, 0xDF, 0x52, 0xBD, 0xD2, 0xA2, 0xDE, 0x9D, 0x63, 0x2B, 0x49, 0x4A, 0x11, 0x09, 0x11, 0xA6, 0xA0, 0x2A, 0x2C, 0x2B, 0x67, 0x3B, 0x66, 0x51, 0xF0, 0x17, 0x23, 0xFB, 0xF8, 0xBF, 0x59, 0x9C, 0xFF, 0x2F, 0x7A, 0x5B, 0x36, 0x88, 0x88, 0x07, 0x2E, 0x28, 0xCB, 0x8F, 0xDC, 0x2C, 0x5C, 0x6E, 0xE0, 0x13, 0x99, 0x4F, 0xFD, 0x64, 0x0A, 0x9C, 0x19, 0xB2, 0x91, 0xD6, 0x41, 0x7B, 0x8B, 0x15, 0xF5, 0x0C, 0xDA, 0xE4, 0x21, 0x8F, 0xF6, 0x1B, 0x7F, 0xAA, 0xC9, 0x1C, 0x12, 0x1A, 0x5B, 0xF9, 0x31, 0x13, 0x6F, 0x8B, 0x99, 0x03, 0x3D, 0xCA, 0xEF, 0x71, 0x35, 0x14, 0xCC, 0xD0, 0x4A, 0xB6, 0x6F, 0xD5, 0xC9, 0x6D, 0xB4, 0xE4, 0xE8, 0x5E, 0xB3, 0xED, 0xC0, 0xBB, 0x35, 0x1E, 0x31, 0xCC, 0x82, 0xB2, 0xA1, 0x31, 0xF9, 0x30, 0x64, 0xCE, 0xA4, 0x69, 0xDC, 0xE9, 0x39, 0xAF, 0xD3, 0xE1, 0x84, 0x82, 0x7A, 0x40, 0xB4, 0xE8, 0xC9, 0x32, 0xE3, 0x62, 0x72, 0x3F, 0x71, 0xFE, 0x42, 0xDB, 0xB8, 0x83, 0x5F, 0x97, 0x9C, 0xE1, 0x0D, 0xAA, 0xEC, 0x17, 0x69, 0x03, 0xFC, 0x21, 0x9A, 0x6A, 0x81, 0x81, 0x36, 0xF0, 0x43, 0x36, 0x82, 0xD1, 0x0C, 0x3D, 0xA3, 0xC9, 0xE8, 0x24, 0x84, 0xA9, 0xF5, 0xCC, 0xE5, 0x44, 0x5F, 0x43, 0x49, 0x24, 0x47, 0xB8, 0x81, 0xF4, 0x73, 0x30, 0x7A, 0x72, 0x2E, 0x4F, 0xEE, 0x43, 0x88, 0x55, 0xF0, 0x8E, 0x11, 0xF1, 0x4D, 0xC8, 0xEB, 0x66, 0xFB, 0xD6, 0x18, 0xA3, 0xB0, 0xC4, 0x45, 0x1C, 0x4F, 0x96, 0x91, 0x84, 0x2B, 0x4F, 0x0B, 0x49, 0x08, 0x82, 0xA7, 0x9A, 0xA8, 0xCC, 0x34, 0x03, 0x91, 0x01, 0x15, 0x16, 0x65, 0xFD, 0x3F, 0x40, 0x0C, 0x52, 0x79, 0x1E, 0x1B, 0xC2, 0x1C, 0x8F, 0x89, 0x75, 0x34, 0x1B, 0x19, 0xA5, 0x05, 0x41, 0xDE, 0xEA, 0xAC, 0x75, 0x05, 0x0E, 0x89, 0x80, 0xAA, 0xB5, 0x36, 0xC8, 0x16, 0x5A, 0x32, 0x06, 0x91, 0x9B, 0xB4, 0x50, 0x4E, 0xD3, 0xC8, 0x73, 0xE4, 0xDB, 0x34, 0x42, 0xDF, 0xA6, 0xC0, 0x2F, 0xF8, 0x61, 0x33, 0x96, 0x62, 0x73, 0x41, 0x52, 0xAF, 0xF3, 0xF5, 0x71, 0x59, 0xBF, 0x8B, 0x3D, 0x98, 0x5F, 0xFD, 0xA2, 0x70, 0x24, 0xAC, 0xDA, 0xF9, 0xCE, 0xF1, 0xF3, 0x0D, 0xA0, 0x49, 0x51, 0xFB, 0x55, 0xDE, 0xAE, 0x59, 0x2A, 0x75, 0x0D, 0x1D, 0xE0, 0xA8, 0xCC, 0xCC, 0x34, 0x02, 0x80, 0x0A, 0x9C, 0x51, 0x5C, 0x38, 0x1E, 0xB0, 0x0D, 0x12, 0xF9, 0xC3, 0x59, 0xF6, 0x9F, 0xFC, 0xB8, 0x58, 0x70, 0xCD, 0x3C, 0xE9, 0x1B, 0x95, 0x7E, 0x1B, 0x85, 0xF8, 0x4D, 0x5E, 0x55, 0x8E, 0xDD, 0xB1, 0x2A, 0xB2, 0xB9, 0x28, 0xC5, 0x63, 0x15, 0x25, 0x6B, 0x26, 0x51, 0x5F, 0xB7, 0x42, 0x52, 0xC0, 0x2D, 0x2A, 0x3A, 0xDD, 0xD8, 0xE4, 0xC4, 0xB3, 0xB1, 0x12, 0x9C, 0x5A, 0xBC, 0x17, 0x55, 0xB7, 0x0A, 0x7D, 0x63, 0x4C, 0x4F, 0x5F, 0xB9, 0x6C, 0x7F, 0x0E, 0x6A, 0xA1, 0x1C, 0x4E, 0xC9, 0x82, 0x03, 0xC3, 0x7C, 0xA2, 0x06, 0x72, 0x24, 0xBE, 0x16, 0x4E, 0x39, 0x4A, 0x2C, 0xBE, 0x91, 0x7D, 0xAE, 0xEA, 0x4B, 0x17, 0x7C, 0xD2, 0x24, 0x99, 0x85, 0x62, 0x79, 0xE8, 0x1B, 0xE2, 0xEE, 0x34, 0xFE, 0xCF, 0x16, 0xD2, 0x20, 0x15, 0x88, 0xAE, 0xE1, 0xAD, 0xFD, 0xF1, 0x38, 0x73, 0x39, 0xE8, 0x83, 0x6A, 0x5D, 0xE8, 0xE5, 0x19, 0xCF, 0x98, 0x1D, 0x14, 0x71, 0xCD, 0xCA, 0xB9, 0x38, 0xF8, 0x68, 0x9B, 0xA7, 0xA5, 0x80, 0xD3, 0x90, 0x98, 0x0A, 0x70, 0xC4, 0xBE, 0xCF, 0xE6, 0xCD, 0x1F, 0xD6, 0x83, 0x2B, 0xB3, 0x79, 0xEF, 0xF7, 0xCF, 0xCB, 0x68, 0x73, 0xFB, 0xA6, 0xA1, 0x7F, 0x6F, 0xD1, 0x4E, 0x64, 0x67, 0x2F, 0x00, 0x06, 0xBB, 0xC3, 0xFE, 0x1C, 0xF9, 0x49, 0x49, 0xC4, 0xFB, 0x50, 0xE4, 0xC2, 0xB0, 0xF1, 0x50, 0xAD, 0x1C, 0x0A, 0x3B, 0xA5, 0x3D, 0xD6, 0xAD, 0xF9, 0x1A, 0x36, 0x28, 0x99, 0x78, 0xE9, 0x9B, 0x8D, 0xC9, 0x97, 0x98, 0xD1, 0x71, 0x21, 0x38, 0x49, 0x1C, 0x01, 0x37, 0x23, 0xFF, 0xBE, 0x10, 0xD4, 0x43, 0x83, 0x33, 0x5C, 0xB3, 0x46, 0xF4, 0x3D, 0xC6, 0xA3, 0xB1, 0xC6, 0x7A, 0x73, 0xA9, 0x65, 0xBE, 0x0F, 0x71, 0x78, 0x5C, 0x0C, 0x07, 0x1E, 0xB0, 0x5F, 0x5A, 0x2D, 0x3C, 0xB4, 0x79, 0x0B, 0x1C, 0x65, 0xE8, 0x3F, 0x38, 0xE2, 0xEF, 0x61, 0xC3, 0x73, 0x2B, 0x25, 0xCD, 0x30, 0xB9, 0xA1, 0x78, 0x68, 0x70, 0xB5, 0x0B, 0x7E, 0x93, 0x42, 0x7E, 0x66, 0x39, 0x90, 0xAF, 0x84, 0xB2, 0x36, 0xAE, 0x52, 0x27, 0x54, 0xA6, 0x5F, 0x78, 0x9A, 0x51, 0x9B, 0xA9, 0xAB, 0x69, 0x28, 0x58, 0x8A, 0xBA, 0x24, 0x11, 0xF3, 0x61, 0x2D, 0x9E, 0x11, 0xD0, 0x2C, 0xAD, 0x5B, 0xB1, 0x27, 0x7C, 0xED, 0xDB, 0xBA, 0x81, 0x10, 0x41, 0xE4, 0xC6, 0xE5, 0x15, 0x6D, 0x00, 0x22, 0x8B, 0x13, 0xDA, 0x9B, 0x2C, 0xA4, 0x36, 0x55, 0x7C, 0xA4, 0x0E, 0xF4, 0x09, 0x37, 0xAE, 0xF6, 0x33, 0x36, 0xF1, 0x55, 0xFE, 0x47, 0x3D, 0xA7, 0xC1, 0x22, 0x6F, 0x99, 0x26, 0xD3, 0x9E, 0xE2, 0xD5, 0xCC, 0x9E, 0xA7, 0x32, 0xAA, 0x6F, 0x77, 0x0B, 0x5A, 0x6E, 0x53, 0x8A, 0x33, 0xEF, 0x10, 0x14, 0xB7, 0x15, 0x52, 0x4E, 0xA6, 0xBC, 0x88, 0x1B, 0xEA, 0xDF, 0x9E, 0x45, 0xAB, 0xDE, 0xF8, 0xB9, 0x83, 0xEC, 0x5F, 0x9E, 0xB9, 0x12, 0x71, 0xFE, 0x5F, 0x0D, 0x48, 0xE8, 0x36, 0x23, 0x9E, 0x4D, 0xF4, 0x95, 0x55, 0x5E, 0x03, 0x6A, 0x3B, 0xD4, 0xD4, 0x44, 0xD7, 0x4D, 0x48, 0xBE, 0x70, 0xB5, 0x68, 0x49, 0x12, 0x87, 0xD7, 0x66, 0xD6, 0x50, 0x0B, 0x8D, 0xA9, 0x86, 0x7A, 0x2A, 0x58, 0x34, 0x98, 0x33, 0xCC, 0x3F, 0xCA, 0x36, 0xCE, 0x15, 0x6E, 0x0E, 0xDD, 0xB9, 0xC0, 0x17, 0x9F, 0x4C, 0x56, 0x61, 0x4D, 0x29, 0x84, 0x5F, 0x2E, 0x7A, 0x7C, 0x74, 0xEB, 0x80, 0xAB, 0x16, 0x34, 0xE4, 0x72, 0x6B, 0x6C, 0xEC, 0x8E, 0xE7, 0x3C, 0xEA, 0xE5, 0xC6, 0x8C, 0x34, 0x8E, 0x05, 0xDF, 0x11, 0x46, 0xB1, 0x00, 0xD4, 0x69, 0x11, 0x3B, 0x94, 0x01, 0x47, 0xDA, 0xA0, 0x82, 0x7B, 0x4F, 0x38, 0x1A, 0x34, 0x00, 0x9F, 0x2E, 0x0B, 0xDD, 0x5A, 0x3E, 0xA0, 0xBE, 0x2F, 0x08, 0x75, 0x7E, 0x91, 0x94, 0xCD, 0x32, 0x11, 0x62, 0x73, 0x91, 0xD2, 0x17, 0xB5, 0x60, 0xA5, 0xE6, 0xBD, 0xBB, 0x84, 0x9F, 0x0E, 0xFC, 0xBA, 0xBE, 0x1A, 0xA1, 0x34, 0xDC, 0x49, 0x55, 0x03, 0x97, 0xF4, 0x27, 0xD3, 0x78, 0x91, 0x21, 0x1F, 0xD6, 0xC5, 0x39, 0xEE, 0xA6, 0xF9, 0xDB, 0x94, 0x91, 0x56, 0xEF, 0x7C, 0x1F, 0x91, 0xB2, 0xA1, 0xA0, 0xCB, 0x3C, 0xF7, 0x6A, 0x42, 0x7E, 0x35, 0x57, 0x1A, 0x64, 0x63, 0x9D, 0xF1, 0x65, 0x04, 0x1D, 0x9E, 0xE0, 0xB4, 0xFD, 0x37, 0x4F, 0x0E, 0x22, 0x4C, 0x36, 0xCE, 0x68, 0xE5, 0xD6, 0x77, 0xA3, 0x24, 0xE4, 0xD4, 0xAC, 0x93, 0x2D, 0x62, 0x44, 0x83, 0xFC, 0x82, 0x0D, 0x9D, 0x3B, 0x2A, 0xDA, 0xE4, 0xF7, 0x0F, 0x16, 0x17, 0xB2, 0xD6, 0xD0, 0xF6, 0x0F, 0x98, 0x44, 0xB6, 0x35, 0xE7, 0x6B, 0xC4, 0xBD, 0x52, 0xFC, 0x4D, 0x23, 0xF4, 0xE4, 0x8A, 0x8D, 0x09, 0x12, 0x0E, 0xE7, 0xA4, 0x25, 0x82, 0xA6, 0x0F, 0xF1, 0xD8, 0xD7, 0x32, 0x2A, 0x1F, 0x6D, 0xFD, 0x07, 0x70, 0xD4, 0x6E, 0xB7, 0x5E, 0xCB, 0xD0, 0x00, 0xCB, 0x4A, 0xD1, 0x99, 0x92, 0x42, 0xE7, 0x33, 0x0B, 0x04, 0xA8, 0xA0, 0xA3, 0x44, 0xEA, 0xBA, 0x04, 0x49, 0x63, 0x0A, 0x21, 0x74, 0x5A, 0x48, 0x1E, 0x6E, 0xA0, 0x09, 0x75, 0x9F, 0x7A, 0xFA, 0x50, 0x8E, 0x81, 0x10, 0x72, 0xBE, 0x3F, 0xDE, 0x4F, 0x25, 0xD8, 0xC3, 0x82, 0x8E, 0x12, 0x0A, 0x0A, 0xC0, 0xDE, 0xA5, 0x5A, 0x77, 0x23, 0xB7, 0x27, 0x6F, 0x97, 0xF0, 0xF7, 0x10, 0x90, 0xDF, 0x34, 0x2F, 0x3D, 0x36, 0x39, 0x01, 0x47, 0xFD, 0xA6, 0x91, 0x41, 0x7A, 0x22, 0x73, 0x43, 0x19, 0x65, 0x7E, 0xA2, 0x86, 0x0D, 0xF4, 0x5A, 0xFB, 0xEF, 0x81, 0xEB, 0xF2, 0x26, 0x9F, 0x2E, 0xAB, 0x08, 0x08, 0x88, 0x90, 0x03, 0x4A, 0xC5, 0x14, 0x8A, 0x67, 0xE1, 0x5E, 0x49, 0xFB, 0x9E, 0x1D, 0xAC, 0xB7, 0x1B, 0xF5, 0x24, 0x2D, 0xD0, 0x06, 0x90, 0xD3, 0x3C, 0x83, 0x08, 0xDA, 0xBB, 0x20, 0x36, 0xE2, 0xFA, 0x7A, 0x34, 0xB2, 0xEB, 0x1F, 0x2D, 0x03, 0x0D, 0x7F, 0x77, 0xA6, 0x52, 0xAC, 0x49, 0x28, 0x0D, 0xC4, 0xF1, 0x50, 0xE0, 0x41, 0xAB, 0x7E, 0x02, 0x75, 0x98, 0x05, 0xCF, 0xA0, 0x80, 0x82, 0xC1, 0x1D, 0xE2, 0xEC, 0xD7, 0x69, 0x94, 0x39, 0x7B, 0x67, 0x94, 0xD3, 0x08, 0x0A, 0x55, 0xFA, 0xB3, 0x5C, 0x63, 0x92, 0xE1, 0x73, 0xDE, 0x5E, 0x23, 0x3A, 0x87, 0x09, 0x9F, 0x5E, 0x8E, 0xF5, 0x2F, 0x9A, 0x71, 0xCE, 0x4E, 0x7A, 0xAD, 0x2A, 0x55, 0x04, 0x14, 0xCC, 0x54, 0x68, 0xF9, 0xA9, 0x3E, 0xC6, 0x37, 0x32, 0xD6, 0x02, 0xE8, 0xB8, 0xB9, 0x2F, 0x31, 0x60, 0x09, 0x88, 0xB8, 0x33, 0xCB, 0x32, 0x02, 0x5A, 0x70, 0x8B, 0xDF, 0x0C, 0x3E, 0x5D, 0x70, 0xA8, 0xC2, 0xEF, 0x66, 0x6B, 0xF2, 0x4C, 0x62, 0x13, 0xBD, 0xA5, 0xEB, 0x7A, 0x72, 0x44, 0x2A, 0xAC, 0xA6, 0x3B, 0xE2, 0xD0, 0x55, 0xDF, 0x93, 0x36, 0x69, 0x21, 0x17, 0x6E, 0xA2, 0x10, 0x37, 0xBD, 0x40, 0x7B, 0xEF, 0xAE, 0x11, 0x29, 0x13, 0x7B, 0x43, 0x3E, 0xD4, 0x44, 0xEC, 0x85, 0x38, 0x85, 0xE8, 0xC7, 0xED, 0x38, 0x16, 0x3D, 0xEA, 0xB1, 0xD3, 0xA5, 0xA2, 0x9B, 0x37, 0x16, 0x5E, 0xDF, 0x18, 0x20, 0x4C, 0xD5, 0x1E, 0x3D, 0x22, 0xDC, 0x46, 0x6A, 0x69, 0x55, 0x83, 0x0C, 0x52, 0x7D, 0x7A, 0x54, 0xC5, 0xAE, 0xC5, 0x73, 0xF7, 0x41, 0x6B, 0xEB, 0x67, 0xD7, 0x3D, 0xDF, 0x66, 0xEF, 0x9C, 0x84, 0xA7, 0x6D, 0x3D, 0x77, 0x1B, 0xCE, 0xFC, 0xB7, 0x23, 0x47, 0x17, 0x45, 0x0E, 0x3A, 0x34, 0x3F, 0x27, 0xC7, 0x1D, 0x1D, 0x16, 0x21, 0x4B, 0xB1, 0x85, 0x62, 0x37, 0x9A, 0x6C, 0x8E, 0xF1, 0x23, 0xFC, 0x1A, 0xC1, 0x90, 0xE4, 0x47, 0xBD, 0x6D, 0xD5, 0xA8, 0x68, 0x26, 0x81, 0xCF, 0xEC, 0x76, 0xD9, 0xC6, 0x27, 0x60, 0xFB, 0x1D, 0x3E, 0x1C, 0x25, 0x6F, 0x8A, 0xAB, 0x58, 0x99, 0xD8, 0x0E, 0x5A, 0x7E, 0xC4, 0x5B, 0x64, 0x5E, 0x3F, 0xC0, 0x09, 0xB9, 0xB4, 0x62, 0x8A, 0x75, 0xA8, 0xFA, 0x29, 0x03, 0x2F, 0x19, 0x53, 0x70, 0x8C, 0x8F, 0x45, 0x19, 0x0D, 0x67, 0x9C, 0x3F, 0x80, 0xAE, 0xD2, 0xE1, 0x4D, 0x57, 0x41, 0x15, 0xDA, 0xC6, 0x1B, 0xD0, 0x56, 0x6A, 0x5B, 0x36, 0xEB, 0x69, 0x2B, 0x44, 0xD9, 0x2B, 0x7D, 0x41, 0x74, 0x98, 0x12, 0x93, 0x60, 0xD6, 0xF3, 0xA2, 0x1E, 0x2D, 0xBF, 0xB3, 0x06, 0x3C, 0xCA, 0x38, 0xC6, 0x67, 0xA0, 0x2F, 0xBF, 0x0A, 0x67, 0x23, 0x73, 0x0F, 0xD5, 0x94, 0x21, 0xCE, 0x22, 0x2B, 0xDD, 0xF6, 0x47, 0x12, 0x2E, 0xCB, 0x4B, 0x5C, 0x43, 0x40, 0x28, 0xE8, 0x05, 0xFC, 0xB9, 0xF8, 0x0B, 0x61, 0x94, 0x96, 0x1B, 0x56, 0x67, 0xA9, 0xA3, 0xD6, 0x34, 0x1D, 0x35, 0xB2, 0x20, 0x71, 0x49, 0x40, 0x61, 0x47, 0xB7, 0x38, 0x59, 0x06, 0x68, 0xC9, 0x14, 0x21, 0x72, 0xE2, 0x4E, 0x8D, 0x84, 0x11, 0xDC, 0x3C, 0x84, 0x9E, 0x01, 0xBA, 0xB3, 0xD0, 0x1C, 0x16, 0x5B, 0xF9, 0x73, 0xD0, 0x38, 0x07, 0x66, 0xC7, 0x1A, 0xA4, 0x2C, 0x8F, 0xBC, 0xA2, 0xD2, 0x41, 0x79, 0x9A, 0x99, 0xC9, 0xAE, 0x37, 0xAC, 0xA3, 0x41, 0x72, 0x53, 0x2F, 0x89, 0xED, 0x16, 0xCB, 0x30, 0x51, 0x26, 0x19, 0x73, 0xB1, 0x1A, 0x23, 0x09, 0xF3, 0xCF, 0xAA, 0x0C, 0x6B, 0x30, 0x72, 0x32, 0xA9, 0x20, 0xBA, 0x3C, 0x02, 0x16, 0xDF, 0x81, 0x88, 0x0A, 0xCB, 0x6B, 0x5E, 0xED, 0x38, 0x10, 0x3E, 0xB7, 0xDD, 0x50, 0x15, 0x63, 0x26, 0xD9, 0x2C, 0x02, 0x62, 0xDB, 0x4F, 0xB4, 0x1A, 0xA8, 0x23, 0x3D, 0x93, 0x4E, 0x9D, 0xF9, 0x26, 0xCE, 0xCE, 0x14, 0x07, 0x03, 0x55, 0x7B, 0xAC, 0x10, 0x72, 0xA4, 0x4D, 0xED, 0x2A, 0xF5, 0xB4, 0xF0, 0x26, 0x77, 0x8D, 0xB4, 0x75, 0x93, 0x2C, 0xAA, 0xDD, 0x20, 0xCE, 0x62, 0xCE, 0xDD, 0x97, 0x47, 0x8B, 0x46, 0x6E, 0xA1, 0xDE, 0x20, 0x8C, 0x07, 0xB0, 0x00, 0x0A, 0x99, 0xF2, 0x0A, 0xB7, 0xB1, 0x2B, 0x1C, 0xD7, 0x21, 0x79, 0x23, 0x4D, 0xD2, 0xC4, 0x57, 0x6C, 0xB1, 0xEC, 0xCA, 0x5A, 0x5C, 0x2F, 0x21, 0xD7, 0xCC, 0xEB, 0x93, 0x39, 0xDA, 0x51, 0xE1, 0x00, 0x93, 0x8F, 0xAB, 0xD5, 0xFA, 0xF8, 0x65, 0xC8, 0x46, 0x19, 0x3B, 0xA5, 0xEF, 0x92, 0x43, 0x4E, 0x3B, 0x22, 0x91, 0x50, 0x1E, 0x93, 0xF5, 0x97, 0xDC, 0x7A, 0x4B, 0x85, 0x1A, 0xDE, 0xC5, 0x7A, 0xFF, 0xE4, 0x96, 0x34, 0xBF, 0x4F, 0x09, 0xFA, 0xE3, 0x5E, 0x1A, 0xA2, 0x3D, 0xD1, 0x8B, 0xFF, 0x04, 0x28, 0xDD, 0xC0, 0x30, 0x1D, 0x5B, 0x20, 0x77, 0x8B, 0xDB, 0xDA, 0x7F, 0xE5, 0xA8, 0xD8, 0x88, 0x40, 0xD8, 0xC2, 0xBA, 0x6C, 0x90, 0x05, 0x16, 0x02, 0xD1, 0x22, 0x02, 0xCB, 0xF3, 0x2E, 0xC7, 0x38, 0x27, 0x1A, 0x1B, 0x8E, 0x07, 0xE2, 0xC0, 0x86, 0x88, 0x33, 0xDA, 0x09, 0xC7, 0x2E, 0xAF, 0x49, 0xD9, 0x99, 0xD7, 0xE6, 0x98, 0x8B, 0xA2, 0xAB, 0xB1, 0xB0, 0x9D, 0xC2, 0x63, 0x6C, 0x32, 0x73, 0xF5, 0xFF, 0xC5, 0x31, 0x3E, 0x25, 0xED, 0x44, 0x7E, 0x96, 0x6E, 0xC2, 0x13, 0xF6, 0x9C, 0x15, 0x64, 0x59, 0x8C, 0x9D, 0xBC, 0x5E, 0x18, 0x71, 0x61, 0x0D, 0xA9, 0x0C, 0x9D, 0x37, 0xDF, 0xAA, 0x0B, 0x54, 0x7F, 0x48, 0x1B, 0x2A, 0xBF, 0xF3, 0x99, 0x8E, 0xB9, 0xA8, 0xEA, 0x56, 0xCF, 0xC0, 0x93, 0xA2, 0x44, 0x28, 0xCE, 0xB4, 0x10, 0x9F, 0xF2, 0xA9, 0x24, 0xC6, 0xC5, 0xB2, 0xFD, 0xDD, 0x97, 0x4E, 0x44, 0xA0, 0x14, 0x1A, 0xD7, 0x76, 0xF6, 0x35, 0x5E, 0x6E, 0x17, 0x3F, 0xB1, 0xA8, 0x14, 0xF6, 0x98, 0xD7, 0x28, 0xA8, 0x1D, 0xB9, 0x15, 0x41, 0xDA, 0xE4, 0xB2, 0xBE, 0xF8, 0xCD, 0x27, 0xDB, 0xAC, 0x6A, 0x98, 0x02, 0x2F, 0x4C, 0xD9, 0x04, 0x8F, 0x7E, 0x93, 0x1E, 0x39, 0xEE, 0xCC, 0x78, 0x19, 0xBF, 0xF7, 0xE7, 0x1E, 0xAB, 0x9F, 0xF3, 0x33, 0x2E, 0x76, 0x19, 0x58, 0x65, 0x16, 0x15, 0x12, 0x72, 0x63, 0xFE, 0xCC, 0x7F, 0x87, 0x3A, 0x44, 0x89, 0x5B, 0x2C, 0x9C, 0xEF, 0x4A, 0xEA, 0x0B, 0xC9, 0x39, 0x12, 0x63, 0x57, 0x85, 0x41, 0x72, 0xD0, 0xE5, 0xDF, 0x56, 0xCC, 0xF3, 0x35, 0xF9, 0x4F, 0xDA, 0xDE, 0x91, 0x00, 0xE8, 0x59, 0xDE, 0x23, 0x3F, 0x27, 0xD7, 0xFD, 0x69, 0xAE, 0x4A, 0xD6, 0xFE, 0xBB, 0x42, 0x40, 0x93, 0xFE, 0x3A, 0xE0, 0x76, 0x31, 0x6B, 0xCC, 0xF3, 0x8E, 0x6F, 0x7E, 0x44, 0x57, 0xBE, 0x8A, 0x2D, 0xDA, 0xED, 0x77, 0x03, 0xBC, 0x08, 0x9D, 0x23, 0x13, 0x27, 0x0F, 0x27, 0x13, 0xA1, 0xDA, 0xA1, 0xF4, 0xEB, 0xB7, 0xC4, 0x2C, 0xE0, 0xCE, 0x43, 0xE1, 0xA4, 0x37, 0x22, 0x26, 0x82, 0x03, 0x90, 0x94, 0xDA, 0x36, 0x43, 0xD2, 0x95, 0x6B, 0x49, 0x20, 0x04, 0xAB, 0xDA, 0x99, 0x9F, 0xA2, 0x39, 0x28, 0xAC, 0x9A, 0xF9, 0x6A, 0x12, 0xCD, 0xF7, 0x32, 0x28, 0x7E, 0x0A, 0x53, 0x84, 0x7F, 0xD9, 0x1E, 0xC6, 0x8E, 0x92, 0x10, 0xEF, 0x8E, 0xB4, 0xE3, 0xF2, 0x22, 0xE5, 0xDF, 0x19, 0x11, 0x77, 0xF0, 0xE3, 0x2A, 0x07, 0x6A, 0xCA, 0x3A, 0xB2, 0x06, 0xBD, 0xE2, 0x63, 0xBF, 0xB6, 0xAD, 0x7F, 0x7D, 0xFC, 0xEF, 0x6C, 0x47, 0xED, 0x34, 0x24, 0xE8, 0xBE, 0x66, 0x0C, 0x4C, 0x11, 0xC3, 0xCD, 0x10, 0x61, 0x4B, 0x01, 0x25, 0x2C, 0x12, 0xC1, 0xBB, 0x04, 0x82, 0x81, 0xAB, 0xDB, 0x9E, 0xD9, 0xAB, 0xCF, 0xB5, 0xB5, 0xFA, 0x04, 0xF9, 0x12, 0x33, 0x4C, 0x8A, 0x33, 0x3A, 0x36, 0x9C, 0xB8, 0xD8, 0x12, 0x4B, 0xBA, 0x15, 0x41, 0x2B, 0x92, 0xBE, 0xE0, 0x6D, 0xF8, 0xAD, 0x66, 0x92, 0xBB, 0x19, 0xA5, 0x3D, 0x82, 0x97, 0x35, 0xB3, 0x35, 0x25, 0x14, 0x15, 0x48, 0x07, 0xE6, 0x0F, 0xDB, 0xE4, 0x74, 0x85, 0x17, 0xD7, 0x81, 0xD5, 0x20, 0x49, 0x8C, 0x34, 0x12, 0xBD, 0xE2, 0x52, 0xA7, 0x49, 0xD7, 0x57, 0xFC, 0x6A, 0x79, 0x87, 0xA4, 0xCD, 0x66, 0xE9, 0x98, 0x01, 0x55, 0x04, 0x21, 0xB8, 0xEB, 0xCE, 0x4C, 0x56, 0x2B, 0xE8, 0xC9, 0xCF, 0x9E, 0x7B, 0xB9, 0x33, 0x90, 0x74, 0x32, 0x61, 0x21, 0xFF, 0xB4, 0x92, 0x77, 0xFC, 0x55, 0xA6, 0x0B, 0x26, 0xAC, 0xC9, 0x08, 0x72, 0xB9, 0x21, 0x33, 0x07, 0xAD, 0x3E, 0x67, 0x2A, 0x2D, 0xF6, 0x76, 0xA2, 0xCB, 0x11, 0xBD, 0x7B, 0xD3, 0xBC, 0x24, 0x80, 0xCF, 0x85, 0x54, 0x7B, 0xC3, 0x54, 0x35, 0xAD, 0xA0, 0x8B, 0xF2, 0xF2, 0xDD, 0xBF, 0xEE, 0x19, 0xE9, 0x32, 0x9B, 0xA5, 0x1F, 0xDB, 0x5E, 0xC9, 0x79, 0x88, 0x90, 0x2E, 0x2D, 0xE1, 0x55, 0xFE, 0x6A, 0xAD, 0x10, 0x4C, 0x51, 0xA1, 0xCF, 0xAF, 0x0F, 0x51, 0x72, 0x61, 0x62, 0x73, 0xB7, 0xBD, 0x46, 0xD6, 0x8D, 0xE8, 0x83, 0xED, 0xA8, 0x21, 0x89, 0x24, 0xFC, 0xC8, 0x33, 0x0F, 0x93, 0x60, 0x09, 0x0F, 0xD8, 0x45, 0xCE, 0x2D, 0x4A, 0xCF, 0xFE, 0x38, 0xF9, 0xBF, 0x1F, 0xC8, 0xF0, 0x03, 0x1D, 0x7F, 0x6A, 0x1E, 0xD1, 0x1E, 0xA7, 0xF7, 0xFD, 0x5A, 0xC2, 0xF8, 0x6A, 0xCD, 0xB8, 0x31, 0xC4, 0xE4, 0x6E, 0x54, 0xDB, 0x9A, 0xCE, 0x94, 0xF0, 0x9F, 0xAB, 0x00, 0xF2, 0xE5, 0xC9, 0xD2, 0xF7, 0xDF, 0xEC, 0xE0, 0xA4, 0x18, 0x7C, 0xFC, 0x18, 0x89, 0x9B, 0x20, 0x85, 0x94, 0xA4, 0xBE, 0xCA, 0x52, 0x77, 0xD0, 0x68, 0xD8, 0x66, 0x3D, 0xD5, 0xEC, 0xFB, 0x39, 0x4D, 0x3A, 0x5C, 0x32, 0xE6, 0x6D, 0x3D, 0xAF, 0x65, 0x62, 0x62, 0xEB, 0x61, 0x62, 0xB8, 0x8C, 0x4B, 0x60, 0x08, 0x2E, 0xA7, 0xD9, 0x7D, 0xE9, 0x4E, 0x96, 0x59, 0xB9, 0x1D, 0x48, 0xB7, 0x08, 0x18, 0x63, 0x39, 0xF1, 0x1C, 0xCA, 0xF6, 0xD0, 0x7C, 0x44, 0x31, 0x33, 0xE1, 0xAA, 0x3D, 0x64, 0x35, 0xCB, 0x5C, 0xE5, 0x00, 0x54, 0xDB, 0x5A, 0x74, 0x15, 0xFD, 0x54, 0xAD, 0x0E, 0x3D, 0x9B, 0xED, 0xB1, 0xE4, 0x29, 0x67, 0x26, 0x01, 0xD2, 0xDA, 0x8D, 0xFB, 0xA8, 0x37, 0x22, 0x55, 0xFD, 0xF5, 0xC9, 0x5F, 0xF9, 0xED, 0x85, 0x02, 0x30, 0x90, 0xB2, 0xFC, 0x52, 0xB1, 0x94, 0xE8, 0x6A, 0x45, 0x36, 0x84, 0x84, 0x52, 0x2F, 0xF7, 0xA6, 0x86, 0xD1, 0x91, 0x96, 0x90, 0x4D, 0xB9, 0xCF, 0xF6, 0xC0, 0xC2, 0xFD, 0xA6, 0x87, 0xA3, 0x93, 0x85, 0xCF, 0x96, 0x38, 0xF6, 0xD6, 0xD9, 0x1D, 0x9B, 0x4B, 0x25, 0xAB, 0x5B, 0xF8, 0xF7, 0x38, 0xFD, 0xB8, 0xF7, 0xA7, 0xA1, 0xED, 0x54, 0x50, 0xB3, 0xBC, 0xEC, 0x4D, 0xF5, 0x5F, 0x20, 0x25, 0xC5, 0xBF, 0x42, 0x61, 0x23, 0xCB, 0x51, 0x13, 0xED, 0x62, 0xC8, 0xD3, 0x33, 0x5A, 0x3A, 0x90, 0x82, 0x4C, 0x97, 0x0E, 0xB7, 0xBC, 0x8D, 0x29, 0x87, 0xCD, 0xF1, 0x33, 0x84, 0x5E, 0x7F, 0x7F, 0xFE, 0xFB, 0x70, 0xB5, 0x05, 0xAA, 0x27, 0x88, 0x5F, 0xE0, 0xBD, 0xF4, 0xA8, 0xCD, 0xA0, 0x8D, 0x9C, 0xDF, 0xF7, 0x41, 0xD5, 0x7C, 0x46, 0x9C, 0xA9, 0x2D, 0x7E, 0x48, 0x75, 0xE5, 0x9D, 0x95, 0x8D, 0x67, 0x18, 0x7D, 0x12, 0x15, 0xE1, 0xF0, 0x44, 0x99, 0x11, 0x40, 0x39, 0x9A, 0xA0, 0x0C, 0x17, 0x65, 0x11, 0x59, 0x2A, 0x8B, 0x3A, 0xD6, 0xC0, 0xBD, 0xEF, 0x9A, 0x1C, 0x0D, 0x4C, 0xE0, 0x36, 0x69, 0x4F, 0xF2, 0xF4, 0x18, 0x10, 0xEC, 0x4E, 0x00, 0x60, 0x05, 0xD1, 0x6A, 0xAE, 0x4F, 0x1B, 0x63, 0x35, 0x5E, 0x02, 0xC0, 0x8A, 0x77, 0x26, 0x91, 0xCD, 0x75, 0x4E, 0xBC, 0x2C, 0x4D, 0xF2, 0xE0, 0x56, 0x92, 0x96, 0x23, 0xE5, 0x73, 0x81, 0xE4, 0x0D, 0xAD, 0x63, 0xF0, 0xE7, 0x35, 0x65, 0x05, 0x4D, 0x40, 0xC2, 0xD6, 0xAD, 0x9C, 0x2B, 0x59, 0x77, 0xB9, 0x03, 0x20, 0xE3, 0x96, 0x51, 0x00, 0xD5, 0x64, 0xAF, 0xCB, 0xD9, 0xF6, 0x3E, 0xEA, 0x69, 0x3B, 0x5B, 0x75, 0xBC, 0xCE, 0x0F, 0xB3, 0xFA, 0x39, 0xA2, 0x35, 0x36, 0x1F, 0xC5, 0xD4, 0xB6, 0xB3, 0x7C, 0x66, 0x26, 0xB0, 0xDB, 0x2A, 0x8C, 0x3A, 0xFD, 0x15, 0xC6, 0xF4, 0xFF, 0x32, 0x2C, 0x87, 0xFB, 0x89, 0x5E, 0x65, 0x4E, 0x1C, 0x50, 0xCB, 0x72, 0x19, 0x07, 0x7C, 0x34, 0x6C, 0xBD, 0x19, 0x69, 0xF6, 0x4D, 0xFB, 0x90, 0x7D, 0x84, 0x5B, 0x2F, 0x8D, 0x47, 0x56, 0x85, 0x26, 0xAE, 0x1A, 0xA3, 0x24, 0x44, 0x0A, 0x4A, 0x0E, 0xD7, 0xB5, 0x1A, 0x33, 0x20, 0x70, 0x4B, 0x42, 0xC1, 0x7B, 0x04, 0xE9, 0xCC, 0xAA, 0xDC, 0x6B, 0x94, 0x6E, 0x87, 0x0E, 0x11, 0x76, 0xC1, 0x73, 0x29, 0xD2, 0xA6, 0x12, 0x35, 0x3C, 0x59, 0x74, 0x46, 0x75, 0x80, 0xE3, 0xFE, 0xDF, 0xDF, 0xEF, 0xFB, 0xBC, 0x4E, 0xDB, 0xDF, 0x2B, 0xF2, 0x7A, 0x3F, 0x41, 0x36, 0xAC, 0xC3, 0x96, 0x2B, 0x9F, 0x55, 0xB4, 0x1E, 0xE4, 0x92, 0x10, 0x7A, 0x6B, 0x28, 0xAE, 0xBE, 0x1E, 0x55, 0x34, 0x59, 0x35, 0x33, 0x3C, 0xEB, 0x2C, 0x19, 0xB7, 0xCA, 0x7E, 0x54, 0x50, 0xBD, 0xDA, 0x4D, 0x99, 0x7B, 0xE8, 0x14, 0xC3, 0x28, 0xB1, 0x40, 0x70, 0x7E, 0xDB, 0x79, 0x08, 0x98, 0x69, 0x36, 0xA9, 0x4D, 0x81, 0x2A, 0x7C, 0x5B, 0xCB, 0x7E, 0xB9, 0xC2, 0xB0, 0x8E, 0x92, 0x2B, 0x7E, 0x2E, 0xA4, 0x70, 0x7D, 0xA5, 0x1C, 0xF8, 0x7B, 0x0F, 0xEE, 0x54, 0x6C, 0x37, 0x58, 0x37, 0xF6, 0xED, 0xCE, 0x04, 0x1B, 0x01, 0x9E, 0xDB, 0x79, 0x1F, 0xD1, 0xA6, 0xB6, 0x69, 0xB5, 0x62, 0x3C, 0xE5, 0x56, 0x22, 0xC0, 0x62, 0x10, 0x51, 0xD9, 0xD0, 0xBF, 0x57, 0xF1, 0x71, 0xCD, 0xF2, 0x14, 0x5D, 0x13, 0x27, 0xC4, 0x40, 0xDF, 0xB4, 0xAA, 0xE9, 0xB7, 0x6E, 0xEC, 0xF7, 0x82, 0x72, 0x50, 0x0D, 0x3C, 0x62, 0xAB, 0x22, 0x6B, 0xBF, 0x38, 0xA8, 0x6E, 0x67, 0xD2, 0x92, 0xB0, 0x8E, 0x08, 0x5C, 0x2F, 0x51, 0x34, 0x95, 0x16, 0x63, 0xF9, 0x39, 0x44, 0x23, 0x81, 0xA2, 0xEE, 0x6F, 0xDC, 0xFE, 0xE6, 0x7B, 0x51, 0x35, 0xFD, 0xD6, 0xAC, 0xA3, 0x39, 0xE4, 0x34, 0xDB, 0x37, 0x45, 0xF0, 0x21, 0xDA, 0x02, 0xF2, 0xB9, 0x8B, 0x17, 0xE5, 0xC1, 0x6C, 0x38, 0xEC, 0xA2, 0x23, 0xC8, 0x46, 0x82, 0x01, 0x26, 0x8C, 0xD3, 0xBF, 0x84, 0x63, 0x72, 0x5F, 0xB6, 0x0E, 0x4A, 0xBF, 0xAF, 0x99, 0x2F, 0xFF, 0x9E, 0xDD, 0x88, 0xC0, 0x1B, 0x7E, 0xA6, 0xE5, 0x70, 0x3D, 0x7B, 0x23, 0x85, 0x86, 0xAD, 0x4D, 0xAE, 0x70, 0xBA, 0xD0, 0x3C, 0x09, 0x45, 0x4E, 0xAB, 0x5B, 0xDA, 0xF4, 0x60, 0x1C, 0x0A, 0x0D, 0x61, 0x14, 0xA3, 0x93, 0x45, 0x19, 0xCE, 0xCA, 0x93, 0x1B, 0xB8, 0x86, 0xAC, 0xB8, 0xE6, 0x09, 0x5B, 0x68, 0x86, 0x8D, 0x25, 0x24, 0x09, 0x36, 0x23, 0x09, 0x07, 0x69, 0x47, 0xD9, 0x5A, 0x2D, 0x21, 0x7B, 0x30, 0xE8, 0x64, 0x8A, 0xCB, 0xD2, 0x0C, 0xC1, 0x27, 0x45, 0x7B, 0x5A, 0x2F, 0x4C, 0xFF, 0x92, 0x03, 0x2B, 0x7C, 0x60, 0xFA, 0x87, 0x15, 0x8D, 0x1A, 0x03, 0x3B, 0x9D, 0xCD, 0x16, 0x6C, 0xBD, 0x63, 0x68, 0x1E, 0x08, 0x2B, 0xF8, 0x69, 0x4B, 0x78, 0x3C, 0x0A, 0xE0, 0xBA, 0xD5, 0x74, 0xC3, 0xA0, 0x4B, 0x4E, 0x98, 0x76, 0x10, 0x8B, 0x6E, 0x78, 0xF6, 0xD4, 0xE1, 0xB3, 0xC7, 0xA1, 0xD4, 0x6C, 0xBC, 0x31, 0x24, 0x62, 0x4E, 0x71, 0xFF, 0x96, 0x41, 0x56, 0x25, 0x9F, 0xB1, 0x85, 0x68, 0xD6, 0x2B, 0xFF, 0x7E, 0xE8, 0xF4, 0xBA, 0x6C, 0xA5, 0x65, 0x7E, 0x8C, 0x0D, 0x64, 0x29, 0x67, 0x46, 0xDD, 0x74, 0x53, 0x94, 0xCA, 0x99, 0xF7, 0x19, 0x89, 0xFF, 0x9F, 0x8C, 0x08, 0x0F, 0x69, 0xED, 0x91, 0xEE, 0xB8, 0xEE, 0x1E, 0x60, 0x4D, 0x36, 0x61, 0x70, 0x43, 0x99, 0x8D, 0xBD, 0x98, 0x97, 0xCC, 0x5F, 0xB1, 0xF8, 0xD5, 0x41, 0x0A, 0x9E, 0x6A, 0x3F, 0x3C, 0xB4, 0x9A, 0x86, 0xA9, 0x7C, 0x02, 0xD2, 0x9E, 0x73, 0x37, 0xD8, 0xAB, 0x2B, 0xD8, 0x87, 0xDD, 0x1F, 0xCB, 0x23, 0x9A, 0xD6, 0x6A, 0x7A, 0x8D, 0xC0, 0x12, 0xA6, 0x09, 0xD1, 0x4B, 0x07, 0x9F, 0xF0, 0x73, 0x4B, 0x2A, 0xC8, 0xD8, 0xD8, 0xAA, 0x48, 0x9A, 0x90, 0xD6, 0xC6, 0x36, 0x53, 0xC6, 0x6C, 0x2A, 0xE3, 0x54, 0xA7, 0xB7, 0x27, 0xB8, 0x41, 0xEA, 0xC4, 0xD0, 0xBA, 0x79, 0x00, 0x55, 0xEF, 0x20, 0xCB, 0x61, 0xCE, 0x0B, 0x57, 0x75, 0x0A, 0x90, 0xF8, 0x2C, 0xBB, 0x75, 0xFD, 0x1C, 0x84, 0x34, 0x8D, 0xFE, 0x36, 0xCA, 0xB4, 0xCF, 0x68, 0x14, 0x8E, 0x37, 0x5E, 0x05, 0xBE, 0x38, 0x3C, 0x6B, 0x2E, 0xE2, 0x30, 0x00, 0x5C, 0x22, 0x96, 0x28, 0xF5, 0x8B, 0x06, 0xDF, 0x00, 0x66, 0xCB, 0x87, 0x2D, 0x27, 0x65, 0x50, 0xF3, 0x38, 0x67, 0xF8, 0xE5, 0x42, 0x29, 0xB5, 0x8F, 0xBC, 0x4B, 0x86, 0xCB, 0x6B, 0x3E, 0x73, 0x05, 0xD4, 0x92, 0x3C, 0xBF, 0x86, 0x0F, 0x24, 0x82, 0xFD, 0x34, 0xE4, 0x9B, 0xD5, 0x55, 0x3D, 0xF3, 0xCD, 0x79, 0x51, 0x45, 0x7B, 0xA4, 0xA1, 0xF1, 0x27, 0x49, 0x50, 0x37, 0x6B, 0x53, 0xB4, 0x00, 0x5F, 0xA6, 0xA4, 0xF6, 0x4B, 0x32, 0x46, 0x3B, 0x3F, 0x49, 0xDB, 0x5C, 0xE0, 0x6E, 0x3D, 0xB9, 0xB5, 0xF7, 0x6F, 0x62, 0xC3, 0x5D, 0x9C, 0xC4, 0x53, 0x8D, 0x5D, 0xF9, 0x78, 0x01, 0x16, 0x4B, 0x98, 0xAE, 0x87, 0x1D, 0x01, 0x37, 0xDC, 0x69, 0xD8, 0xDB, 0xF5, 0x68, 0x29, 0x7B, 0xDF, 0xD0, 0xBA, 0x80, 0x94, 0xA4, 0x43, 0x05, 0x9C, 0xCD, 0x52, 0x72, 0xA7, 0x71, 0xFF, 0x95, 0x2A, 0xC0, 0x23, 0xFB, 0xF7, 0x84, 0x59, 0xC1, 0x79, 0xC3, 0xC5, 0x3C, 0x0F, 0xA9, 0xB2, 0x97, 0x44, 0x12, 0x3E, 0x67, 0x17, 0xEC, 0xA5, 0x68, 0xDB, 0xBE, 0x36, 0xCD, 0x5D, 0xBA, 0x20, 0x8B, 0x5C, 0x82, 0x08, 0xB2, 0xB8, 0xE1, 0xD1, 0xF9, 0xFB, 0x55, 0xF5, 0xE4, 0xE5, 0x1E, 0x57, 0xDE, 0x8F, 0x9D, 0x8E, 0x5F, 0x62, 0x6A, 0xCD, 0x26, 0x02, 0xBA, 0x48, 0x5C, 0xA5, 0x50, 0x5F, 0xC0, 0xF2, 0xB2, 0xBD, 0x51, 0x07, 0xAE, 0xE0, 0xCC, 0x39, 0x50, 0xAC, 0x35, 0xAA, 0x4B, 0xB7, 0x4F, 0x19, 0xFA, 0x16, 0x6C, 0xC5, 0x39, 0x93, 0xBE, 0x4C, 0x9A, 0x6E, 0xC2, 0x77, 0x38, 0x33, 0x1A, 0x7F, 0x85, 0x8E, 0x20, 0x2B, 0xDA, 0xE9, 0x48, 0x40, 0xD1, 0xBA, 0x32, 0x8F, 0xFE, 0xC0, 0x10, 0xA8, 0x3E, 0x92, 0x5D, 0x7F, 0x65, 0x27, 0x15, 0x80, 0x49, 0xD5, 0x7D, 0x08, 0x34, 0x10, 0xB3, 0x96, 0x01, 0x6C, 0x90, 0x26, 0xC4, 0x99, 0x41, 0x7F, 0x89, 0x7D, 0xF1, 0x22, 0x37, 0x92, 0x32, 0x95, 0x2C, 0xD2, 0x6B, 0x74, 0x2E, 0xE5, 0x86, 0xB8, 0x5D, 0xE7, 0xAD, 0xF3, 0x2F, 0x72, 0x05, 0xAC, 0xE8, 0x02, 0x03, 0x6F, 0x0D, 0x97, 0xAD, 0x19, 0xFD, 0x0D, 0x6E, 0xEC, 0xF2, 0xC5, 0xC2, 0xBE, 0x12, 0xE0, 0xB9, 0x37, 0x3D, 0xF5, 0x85, 0x81, 0xD3, 0x96, 0x76, 0xD3, 0x17, 0x3F, 0xCF, 0x16, 0xD0, 0x17, 0xC1, 0x4A, 0x53, 0x8F, 0xAC, 0x92, 0x6F, 0x12, 0x0B, 0xA8, 0x9A, 0xB0, 0xED, 0x90, 0xF1, 0xCD, 0xF7, 0xB8, 0x25, 0x16, 0xB2, 0x65, 0x68, 0xBB, 0xA1, 0xBA, 0x94, 0x8B, 0x0F, 0x38, 0x1E, 0xFC, 0x67, 0x58, 0x07, 0xA9, 0x15, 0x2D, 0xD9, 0xEB, 0x28, 0xAB, 0xD1, 0x59, 0x27, 0x35, 0x78, 0x14, 0x6C, 0x3F, 0x8D, 0xCE, 0x36, 0x42, 0xE1, 0xBD, 0x90, 0x7E, 0xBC, 0x36, 0x43, 0x29, 0x14, 0xCF, 0x96, 0xFE, 0xBE, 0x18, 0x98, 0x67, 0x79, 0x97, 0x0D, 0x54, 0x6A, 0xEE, 0x48, 0x1B, 0x3F, 0x36, 0xBA, 0xF6, 0xC5, 0x59, 0xF4, 0x99, 0xB9, 0x75, 0x93, 0xF3, 0x9F, 0x67, 0x9A, 0xBB, 0x52, 0x3A, 0xDE, 0x25, 0x73, 0x97, 0x4A, 0x82, 0x5F, 0x6D, 0xF1, 0xED, 0x8F, 0xBA, 0x76, 0xA0, 0xAA, 0x0A, 0x83, 0xCB, 0x78, 0xF7, 0xDB, 0xDA, 0xD8, 0x57, 0xB7, 0xAD, 0xBA, 0x78, 0xAE, 0x88, 0x48, 0x3A, 0x27, 0xBF, 0x6A, 0x63, 0x5F, 0x2E, 0x6E, 0xBB, 0xD3, 0xAD, 0x3A, 0x53, 0xF6, 0x42, 0x05, 0x9C, 0x59, 0x7D, 0x29, 0xDB, 0xFF, 0x3D, 0x05, 0x50, 0xA3, 0x8C, 0x16, 0xAA, 0xF2, 0xC4, 0x5B, 0xED, 0x64, 0x13, 0x9B, 0xD8, 0x84, 0x21, 0xB0, 0xB9, 0x2F, 0x32, 0x45, 0x6D, 0x7C, 0xFA, 0x0A, 0x7D, 0xC8, 0x43, 0x7E, 0x8F, 0x15, 0x20, 0x9F, 0x4A, 0x0A, 0x80, 0x5A, 0x07, 0xF2, 0x37, 0x50, 0xBB, 0xF9, 0xF3, 0x4D, 0xB7, 0x72, 0x15, 0x8B, 0x2C, 0x36, 0xB4, 0x85, 0xD2, 0xBD, 0x1A, 0xCE, 0xC0, 0x2B, 0xC7, 0x27, 0xA6, 0xB4, 0x79, 0xB8, 0x3C, 0x6A, 0x71, 0x90, 0xBE, 0xAD, 0x15, 0x21, 0x9C, 0x1A, 0xFA, 0xF4, 0xBC, 0x7F, 0xD0, 0x07, 0x0B, 0x0E, 0x5F, 0x94, 0xAA, 0x18, 0x36, 0xAC, 0x84, 0xAA, 0x6E, 0x99, 0x37, 0x76, 0x9A, 0x24, 0xE4, 0x30, 0xF4, 0x40, 0x5C, 0x8B, 0xD3, 0xF7, 0x3C, 0x56, 0x8C, 0xC8, 0x89, 0x89, 0x94, 0x14, 0xE8, 0x05, 0x27, 0x84, 0x72, 0x51, 0xE7, 0x0A, 0xC6, 0x14, 0x53, 0x82, 0x8F, 0xDB, 0x50, 0x1F, 0x64, 0x80, 0xB1, 0xD2, 0xAC, 0x2C, 0xAD, 0x8D, 0xA0, 0x4B, 0x57, 0x40, 0xC4, 0x09, 0x94, 0x35, 0x35, 0x9A, 0xCA, 0xCB, 0x0A, 0x6A, 0xC9, 0x51, 0xCE, 0x53, 0x4B, 0x36, 0xDA, 0xA3, 0x84, 0x4A, 0x46, 0x2B, 0x7C, 0x18, 0x88, 0xD5, 0x1C, 0xC8, 0xA1, 0x52, 0x2D, 0xBD, 0x73, 0x59, 0x66, 0xCE, 0xEC, 0x3E, 0xC8, 0x70, 0xFC, 0xB7, 0xFA, 0xF4, 0x53, 0x3A, 0xD8, 0xFA, 0x61, 0xA7, 0xB6, 0xB0, 0x46, 0x89, 0x87, 0xEC, 0xE7, 0x13, 0x51, 0xEB, 0xCC, 0x45, 0x01, 0x4C, 0xED, 0xC4, 0x07, 0xA7, 0x63, 0xE8, 0x26, 0x9D, 0x82, 0x7E, 0xF8, 0xE7, 0xA3, 0x63, 0x2B, 0xF2, 0x76, 0x39, 0x63, 0x71, 0x79, 0x9E, 0x12, 0x21, 0x9E, 0x17, 0x5F, 0xE5, 0x3F, 0xEA, 0x3C, 0x83, 0x8F, 0x40, 0x65, 0xDA, 0x74, 0xB6, 0x0E, 0x24, 0xA6, 0x12, 0x96, 0xCF, 0x32, 0x01, 0x6E, 0xA1, 0x5D, 0x7A, 0x59, 0x5C, 0x39, 0xD1, 0xD1, 0x29, 0xB8, 0x7E, 0xA9, 0xC0, 0x71, 0xE6, 0xE0, 0x5E, 0x47, 0xA3, 0x13, 0xD8, 0x88, 0xBF, 0xD2, 0x3B, 0x12, 0xBF, 0x27, 0xA5, 0xD8, 0x98, 0xE9, 0x76, 0x97, 0xF9, 0xE0, 0xB9, 0xA0, 0xFC, 0x0E, 0x60, 0xC4, 0xA9, 0x03, 0x02, 0x43, 0x4A, 0x7E, 0xD2, 0x38, 0x75, 0x7C, 0x30, 0x74, 0x67, 0x7B, 0x7A, 0xD6, 0x58, 0x65, 0x4E, 0x05, 0xB4, 0x95, 0xAA, 0x2D, 0x8B, 0x37, 0x70, 0xA2, 0x96, 0x0C, 0x05, 0xCE, 0xBE, 0xE3, 0x88, 0xAF, 0x13, 0x08, 0x81, 0xCB, 0xDA, 0x34, 0xED, 0xD5, 0xDF, 0x26, 0xBE, 0xFB, 0x9B, 0x50, 0x35, 0x91, 0xA6, 0xB6, 0xEF, 0xC2, 0x17, 0x6E, 0x81, 0x81, 0x76, 0x6A, 0x57, 0x82, 0x79, 0x35, 0xDA, 0x1F, 0x2D, 0x3C, 0x8E, 0x2A, 0x5C, 0x22, 0xB5, 0x4F, 0xDA, 0x09, 0xDE, 0x87, 0x30, 0x77, 0xA5, 0x3B, 0x2E, 0x81, 0xF1, 0x13, 0xFA, 0x4F, 0x8F, 0x00, 0xDB, 0xA6, 0x16, 0x2B, 0x21, 0xB5, 0x10, 0xFB, 0xD9, 0xDC, 0xA8, 0x27, 0x05, 0x12, 0x63, 0x83, 0x57, 0xFD, 0xE0, 0x14, 0x0D, 0xB7, 0xFE, 0x65, 0x1F, 0x49, 0x9F, 0x61, 0x16, 0xAA, 0xA0, 0xE1, 0x95, 0x47, 0x9C, 0x10, 0x62, 0xEC, 0xE4, 0xB3, 0x8D, 0xA8, 0xD2, 0xFF, 0x66, 0x10, 0x65, 0x30, 0xE1, 0xA3, 0x89, 0x3F, 0x97, 0xC0, 0x7C, 0x15, 0x14, 0x8E, 0x96, 0x64, 0x48, 0xC1, 0x76, 0xFE, 0xAF, 0xBA, 0x7C, 0x35, 0x14, 0x71, 0xCA, 0x6B, 0x3C, 0xA0, 0x05, 0xE9, 0x41, 0x0C, 0xA3, 0x1D, 0xB7, 0x32, 0x83, 0xAE, 0x4D, 0x51, 0xAA, 0x1C, 0xCF, 0xD7, 0x9E, 0x25, 0x76, 0xFD, 0x43, 0x84, 0xF9, 0xDC, 0x01, 0xED, 0xE9, 0x27, 0xAC, 0x95, 0xE8, 0x79, 0x34, 0xDE, 0xD6, 0x98, 0xA8, 0x49, 0xAA, 0x03, 0x87, 0x6E, 0x06, 0x07, 0xC7, 0xD2, 0xC6, 0x67, 0xCF, 0x9D, 0x70, 0xE0, 0x20, 0x9A, 0x72, 0x66, 0x1D, 0x9A, 0x64, 0x3B, 0x95, 0x41, 0x89, 0xD6, 0x96, 0xBC, 0xF2, 0x3B, 0x32, 0x18, 0x92, 0x19, 0x34, 0x4E, 0x21, 0x86, 0xDB, 0x35, 0xF4, 0x93, 0x91, 0x7A, 0x89, 0x0E, 0x61, 0x6A, 0x83, 0x34, 0xA9, 0x18, 0x5C, 0xAE, 0x01, 0x76, 0x96, 0x95, 0xF4, 0x05, 0xE4, 0xA6, 0x1E, 0x3F, 0x63, 0x69, 0x0C, 0xC9, 0x56, 0x66, 0xB3, 0x04, 0xA3, 0x39, 0x12, 0x77, 0xC2, 0x7D, 0x67, 0x41, 0x00, 0x28, 0xEB, 0x57, 0xB8, 0xF8, 0xCA, 0x2F, 0xE4, 0xD9, 0x0F, 0x9E, 0x34, 0xFF, 0x8C, 0xC8, 0x96, 0xB9, 0x14, 0x55, 0x45, 0x5E, 0x13, 0x11, 0x7B, 0x49, 0xB8, 0x70, 0x21, 0x21, 0xA5, 0x08, 0x94, 0xCF, 0x17, 0xDB, 0xCC, 0xDA, 0xB5, 0x12, 0x79, 0x8D, 0xA6, 0x4D, 0xFE, 0xA5, 0x1B, 0x56, 0x1E, 0xFA, 0x04, 0x82, 0x25, 0x9D, 0xE6, 0x93, 0xD7, 0x7E, 0x72, 0xC1, 0x19, 0x31, 0x08, 0xC4, 0x85, 0xBB, 0x54, 0xDE, 0x36, 0xC9, 0xBE, 0x61, 0x84, 0xB0, 0xC4, 0xDC, 0x8C, 0xCC, 0xF2, 0xEB, 0x12, 0x2A, 0x37, 0x17, 0x23, 0xAB, 0x26, 0xDA, 0x5D, 0x92, 0x9C, 0x63, 0xBF, 0xCB, 0x81, 0xE6, 0x79, 0x80, 0x1C, 0x43, 0xB2, 0x00, 0x99, 0x88, 0xAB, 0x55, 0xC9, 0x23, 0x02, 0x6A, 0x58, 0x59, 0x14, 0xAD, 0xBE, 0xEA, 0x41, 0x94, 0xFD, 0x6B, 0x4E, 0xE5, 0xE7, 0x52, 0x38, 0xC6, 0xCC, 0x0C, 0xE7, 0xE8, 0xAA, 0xFC, 0x41, 0x03, 0x8C, 0x4A, 0x19, 0x67, 0xE5, 0x0E, 0x9D, 0x56, 0xAF, 0xCA, 0x68, 0xA3, 0x19, 0x76, 0x66, 0x62, 0x01, 0x5B, 0xEA, 0xAD, 0x31, 0xEB, 0xEF, 0x5F, 0x93, 0x1B, 0xF1, 0x4C, 0x1A, 0xC8, 0x11, 0x28, 0xA3, 0xA1, 0x30, 0xBE, 0x09, 0x35, 0x22, 0x41, 0xB5, 0x53, 0x77, 0x6C, 0x83, 0xE9, 0xED, 0x51, 0x7C, 0x9A, 0x1A, 0x76, 0x64, 0xC4, 0x4E, 0xE2, 0x16, 0x1C, 0x3E, 0xCE, 0xC2, 0xE1, 0x11, 0x9B, 0xA9, 0x81, 0xE6, 0x86, 0x5D, 0x54, 0x08, 0x28, 0xC2, 0x7A, 0xBE, 0xBF, 0xF9, 0xCF, 0xD0, 0xF1, 0xFF, 0x00, 0x5E, 0x62, 0x64, 0x5F, 0x7D, 0x82, 0x2E, 0xB7, 0xB7, 0xEE, 0x0A, 0xD8, 0x50, 0x35, 0x87, 0xBA, 0x24, 0x32, 0xF9, 0x51, 0xEA, 0x68, 0x34, 0x6F, 0x48, 0x5E, 0x63, 0xA9, 0x17, 0x30, 0x6F, 0x9B, 0xCF, 0x36, 0x2E, 0x1C, 0x2D, 0x6A, 0x5F, 0x11, 0xC5, 0x45, 0x66, 0x82, 0x6A, 0x8B, 0xB1, 0x83, 0x96, 0xE3, 0x30, 0x7E, 0x18, 0x66, 0xB6, 0x98, 0xE3, 0x0E, 0xC2, 0x5D, 0x5A, 0x97, 0x75, 0x56, 0x3C, 0x09, 0xCE, 0xFE, 0x6D, 0x8D, 0x33, 0x36, 0x69, 0xEC, 0x6E, 0x05, 0xA1, 0xAC, 0xF5, 0x02, 0xE3, 0xAE, 0x2F, 0xB9, 0xCD, 0x01, 0x56, 0xD2, 0xC9, 0xEB, 0x81, 0x20, 0x88, 0xF5, 0xD7, 0x8D, 0x84, 0x17, 0xC9, 0xC5, 0x5A, 0x72, 0xC2, 0xB4, 0x0E, 0xC3, 0xDE, 0x3D, 0x7D, 0xC4, 0xD3, 0xB6, 0xB3, 0x33, 0xA8, 0x04, 0x31, 0x70, 0x73, 0x7C, 0xB9, 0xB8, 0xAD, 0xB1, 0x52, 0x5C, 0x60, 0xD1, 0xF9, 0xE8, 0xAF, 0xD6, 0xC4, 0xD1, 0x23, 0xBE, 0xB8, 0x66, 0x92, 0x44, 0x45, 0xC9, 0x91, 0x1B, 0x35, 0xDC, 0x87, 0xF4, 0x48, 0xE3, 0xA2, 0xC3, 0x12, 0x48, 0xF0, 0xF4, 0xE1, 0x3A, 0x57, 0x91, 0xC0, 0xD7, 0xF7, 0x6D, 0x61, 0xD7, 0xAA, 0xB9, 0xD8, 0xFE, 0x54, 0xF1, 0xBA, 0x85, 0xD0, 0xB6, 0xC4, 0x54, 0x5B, 0xEE, 0x4F, 0x25, 0x45, 0xBB, 0x01, 0xAD, 0x5D, 0x7F, 0xF0, 0x40, 0x09, 0xB8, 0xA4, 0xB8, 0x53, 0x14, 0x16, 0xE7, 0x8E, 0xBE, 0x9C, 0x21, 0x95, 0x33, 0x10, 0xEB, 0x4F, 0x88, 0xB4, 0xA7, 0x25, 0x82, 0x53, 0x16, 0xED, 0x92, 0x86, 0x54, 0x00, 0x9B, 0x9B, 0x26, 0x1C, 0xA4, 0x8A, 0x54, 0x68, 0xB7, 0x74, 0xCE, 0x11, 0x1C, 0xFC, 0xE0, 0x54, 0xF3, 0x71, 0x6B, 0x6A, 0xE1, 0x80, 0x41, 0x11, 0x8A, 0x36, 0x01, 0x4F, 0x7F, 0x16, 0x41, 0xEF, 0xA1, 0x9A, 0x5B, 0xAF, 0x2A, 0x69, 0xDC, 0xC1, 0x30, 0x7F, 0x61, 0x0B, 0x8B, 0xFD, 0xDC, 0xA9, 0xDB, 0x43, 0xA1, 0x03, 0x3D, 0xC3, 0x07, 0x31, 0x89, 0xA8, 0x34, 0x00, 0x99, 0xD3, 0xB6, 0xFD, 0xD1, 0x35, 0xEC, 0x5B, 0x3C, 0xF0, 0x97, 0xC2, 0xB4, 0xF7, 0x9A, 0x19, 0x9F, 0xD3, 0x76, 0x75, 0x8F, 0x93, 0x34, 0xC3, 0x37, 0x9A, 0x7C, 0xC1, 0x23, 0x5B, 0x24, 0x62, 0xB8, 0x8F, 0x1E, 0xB0, 0x22, 0xF7, 0xEC, 0x72, 0x12, 0xD1, 0x3D, 0x43, 0x6F, 0x76, 0x98, 0x03, 0x9F, 0x40, 0xB1, 0x93, 0x5F, 0xE7, 0x5C, 0xF2, 0xE2, 0x16, 0x66, 0x22, 0x09, 0x56, 0xEC, 0xD3, 0x09, 0xF0, 0xDF, 0xD8, 0x71, 0x4C, 0x9B, 0x32, 0x50, 0x13, 0x6E, 0xB4, 0x48, 0xC5, 0x8C, 0x0D, 0x27, 0x0F, 0x61, 0x3A, 0x15, 0x9A, 0x9B, 0x22, 0x61, 0x24, 0xB3, 0xFE, 0x77, 0xF1, 0x73, 0x7D, 0xBD, 0x05, 0x1F, 0x61, 0xDF, 0xD3, 0x65, 0x07, 0xA6, 0x6C, 0x90, 0xD0, 0x1B, 0xF7, 0x32, 0xB4, 0x1E, 0x47, 0x74, 0x47, 0x4A, 0x05, 0x6D, 0x38, 0x17, 0x1F, 0xD8, 0x69, 0x55, 0x7C, 0xE3, 0x38, 0x87, 0x33, 0xFA, 0x42, 0x71, 0x2A, 0xFE, 0x17, 0xFA, 0x95, 0x95, 0x03, 0x7F, 0xCC, 0x04, 0xD1, 0x3C, 0x10, 0xE1, 0x84, 0xD7, 0x77, 0xE2, 0xEA, 0x26, 0x6E, 0x58, 0xE5, 0x64, 0xEE, 0xBD, 0xD7, 0x19, 0x01, 0x67, 0x3A, 0xD8, 0x8B, 0x7C, 0x49, 0x19, 0x7F, 0x70, 0x9D, 0xE4, 0x00, 0x98, 0xBB, 0xB2, 0x7E, 0x73, 0xB1, 0xD9, 0xC1, 0x05, 0x68, 0x39, 0x50, 0xF0, 0xFD, 0x1E, 0x77, 0x58, 0x1E, 0x31, 0xF4, 0x61, 0x93, 0xBA, 0x93, 0x72, 0xC5, 0xA8, 0x55, 0x79, 0xE8, 0xB8, 0x56, 0xEC, 0x0A, 0xC2, 0x33, 0x68, 0xCD, 0x41, 0xC2, 0x8E, 0xD7, 0x0F, 0x13, 0xE3, 0xE4, 0x9B, 0x98, 0xFE, 0xA7, 0x37, 0x87, 0x49, 0x21, 0xED, 0xEA, 0xAE, 0x40, 0xCD, 0x9E, 0x40, 0xB6, 0x4E, 0xD6, 0xB3, 0xA7, 0xF4, 0x39, 0xBE, 0x61, 0xD9, 0xB0, 0x1E, 0x71, 0x1A, 0x28, 0xBE, 0x87, 0x55, 0x02, 0x2C, 0x61, 0xB2, 0x73, 0xAF, 0x20, 0x99, 0x65, 0x36, 0xB9, 0xA8, 0x2F, 0xF1, 0x2D, 0x93, 0x21, 0x99, 0x1F, 0x6E, 0xAB, 0x4F, 0x85, 0x82, 0x08, 0x5E, 0xD7, 0x31, 0x58, 0x34, 0xE2, 0xC3, 0xEC, 0xD0, 0xD8, 0xB9, 0xC1, 0x55, 0xB0, 0x0D, 0xD2, 0x74, 0x5E, 0x8B, 0x64, 0x5D, 0x05, 0x84, 0x83, 0x7E, 0x1D, 0x79, 0x68, 0xF5, 0x9D, 0x90, 0xCD, 0xBD, 0x9A, 0x79, 0xF7, 0x09, 0xB4, 0xD9, 0xA5, 0xB4, 0x9F, 0x50, 0x26, 0x4D, 0xA8, 0xD2, 0x49, 0xC4, 0x27, 0x2D, 0xBB, 0x30, 0x24, 0xD9, 0x10, 0x61, 0xD5, 0x70, 0x9C, 0x78, 0xA9, 0xB9, 0xAE, 0x7F, 0x3D, 0x4F, 0xCE, 0x1F, 0x7C, 0x32, 0x98, 0x69, 0x8C, 0xE3, 0x6B, 0xB1, 0xE7, 0x99, 0x11, 0x2C, 0x35, 0xAA, 0x48, 0xF4, 0x15, 0xA5, 0xA3, 0xE4, 0x18, 0xFB, 0x75, 0x92, 0x40, 0xDC, 0xC5, 0x5A, 0x5D, 0x42, 0x36, 0x8E, 0x12, 0x77, 0x8C, 0x40, 0xA5, 0xAB, 0x17, 0x11, 0xF4, 0x24, 0x70, 0x1D, 0x00, 0x68, 0x53, 0xB0, 0xA8, 0x15, 0xCF, 0x48, 0x0C, 0xB9, 0x4E, 0x3D, 0x5B, 0x65, 0x71, 0x03, 0x71, 0x97, 0x22, 0x8A, 0xF0, 0x4C, 0x1E, 0x37, 0xD6, 0x4C, 0x61, 0x8D, 0xA9, 0x8C, 0x5F, 0x76, 0xB3, 0x94, 0xC4, 0xC7, 0xE1, 0x87, 0x6C, 0x32, 0x78, 0x6A, 0x0B, 0x84, 0x8B, 0x1E, 0xBB, 0xC6, 0x3B, 0xB6, 0x4E, 0x02, 0x0F, 0x65, 0x28, 0x27, 0x52, 0x56, 0x43, 0xBA, 0x3E, 0x8E, 0x78, 0xC6, 0x92, 0x9E, 0x94, 0x59, 0x82, 0xA0, 0x8C, 0xCE, 0xDA, 0x33, 0x4B, 0x8A, 0x23, 0x10, 0x73, 0xCF, 0x47, 0x5D, 0xDA, 0x7F, 0x22, 0xAF, 0x2A, 0xD5, 0x85, 0x76, 0xB9, 0xCE, 0x8C, 0xE3, 0xA4, 0xF2, 0x4C, 0xFB, 0x38, 0x17, 0xDE, 0x53, 0xA5, 0xF1, 0x98, 0xF4, 0x7B, 0xD9, 0x29, 0x74, 0x60, 0x7E, 0x34, 0x74, 0xB6, 0xF5, 0xE5, 0x78, 0x30, 0x90, 0xFA, 0xDA, 0xDD, 0x13, 0xB7, 0xF6, 0xE5, 0xCE, 0x05, 0xCC, 0x35, 0x81, 0x60, 0xDE, 0x40, 0x45, 0xEE, 0xD2, 0x4B, 0xBB, 0x26, 0x37, 0x25, 0x27, 0xD4, 0x61, 0x1F, 0x06, 0xE3, 0x10, 0x06, 0xA9, 0x96, 0x0B, 0x34, 0x61, 0x68, 0xE7, 0x72, 0x17, 0x76, 0xA6, 0xA5, 0xC1, 0xD6, 0xEA, 0xE0, 0x4D, 0xE5, 0x1E, 0xB8, 0x5B, 0xA5, 0xB0, 0x9D, 0x96, 0x7C, 0xCF, 0x9A, 0xAA, 0xC6, 0xB7, 0xBF, 0xCA, 0x77, 0x5C, 0x30, 0x42, 0x2D, 0xDB, 0xCF, 0xB1, 0x83, 0x24, 0x39, 0x9A, 0xC1, 0xA7, 0xA6, 0x52, 0xAD, 0x7E, 0x78, 0x05, 0x49, 0x55, 0xCD, 0x31, 0x74, 0x60, 0x21, 0xB4, 0x92, 0x46, 0x22, 0x74, 0xF2, 0x12, 0x44, 0x6E, 0xE4, 0x89, 0xE9, 0xF6, 0x21, 0x9C, 0xBA, 0x2E, 0xAB, 0x50, 0xD2, 0x6F, 0x1B, 0xD9, 0x6D, 0x36, 0x78, 0xA1, 0xB4, 0x52, 0x95, 0xDB, 0xCA, 0x6E, 0x00, 0xCD, 0x20, 0xD6, 0x89, 0x88, 0x04, 0xA6, 0x65, 0x06, 0xE8, 0x13, 0x6A, 0x46, 0x43, 0x5D, 0xC6, 0x53, 0x65, 0x96, 0x03, 0xB3, 0x05, 0x89, 0xB2, 0x2C, 0x23, 0x32, 0xBB, 0x1D, 0x6B, 0x19, 0x82, 0x0B, 0x4A, 0xD0, 0xB2, 0xE2, 0x21, 0xA8, 0x62, 0x03, 0x99, 0x32, 0x8A, 0x06, 0x5F, 0xC0, 0x30, 0x12, 0x2E, 0x4B, 0x30, 0xEB, 0xE3, 0x54, 0x2D, 0xED, 0x3C, 0x9F, 0xB4, 0x7C, 0x23, 0xE7, 0xC1, 0xF6, 0x9F, 0x9A, 0xF1, 0xFC, 0x0D, 0xD4, 0x86, 0x0A, 0x08, 0x4B, 0xF2, 0x8E, 0x88, 0xF2, 0x3E, 0x5F, 0xE1, 0xA9, 0x81, 0x45, 0x7A, 0x2B, 0x87, 0xC8, 0x3B, 0x19, 0x57, 0x3F, 0xB0, 0x66, 0xAF, 0x92, 0xF4, 0x9C, 0xDF, 0xCD, 0x13, 0x71, 0xCB, 0x32, 0xFE, 0x52, 0x54, 0x8A, 0xA2, 0x21, 0x9B, 0xF9, 0xE6, 0x74, 0xCA, 0x2B, 0x3F, 0x83, 0x2A, 0x10, 0xEC, 0x97, 0x2F, 0xD3, 0x94, 0x70, 0x38, 0x5A, 0xC2, 0x93, 0xEC, 0x11, 0x4B, 0xFB, 0x97, 0xB6, 0x0C, 0x27, 0x1E, 0x04, 0xA5, 0x3E, 0x07, 0xEB, 0x2E, 0x85, 0xF4, 0xC3, 0xD1, 0x2C, 0xEF, 0x58, 0x3D, 0x49, 0xC0, 0x6F, 0xA5, 0xAD, 0xF4, 0x37, 0x50, 0x6B, 0x2E, 0x13, 0xDA, 0x94, 0x86, 0x79, 0x51, 0xD4, 0xC1, 0x23, 0x45, 0x2D, 0x1F, 0x58, 0xFA, 0x12, 0xB0, 0x34, 0x50, 0x73, 0x1D, 0x93, 0xF2, 0x63, 0x20, 0xFB, 0x74, 0xC6, 0xE6, 0x8B, 0x35, 0x27, 0xA5, 0x8D, 0xE7, 0xFA, 0x05, 0x81, 0xC5, 0xD7, 0x43, 0x83, 0x9C, 0xD4, 0xD2, 0xBC, 0x68, 0x47, 0x83, 0x54, 0x79, 0xAC, 0xAB, 0x0C, 0x38, 0x71, 0x8B, 0xE3, 0x83, 0xE8, 0xD7, 0xA4, 0xAD, 0xC4, 0xD6, 0xF9, 0x5D, 0x19, 0xBE, 0xE4, 0xD9, 0x36, 0x4D, 0xC6, 0xA5, 0xB3, 0xBF, 0xA2, 0xEC, 0x33, 0xF0, 0xA8, 0xA6, 0x8E, 0x9A, 0xD5, 0x6A, 0x6A, 0xAB, 0xD7, 0x44, 0x29, 0x35, 0x56, 0xDC, 0x1C, 0x36, 0xDB, 0x0B, 0x89, 0xC5, 0x61, 0xD4, 0x99, 0xE3, 0xEB, 0xDB, 0x65, 0x90, 0x83, 0x7D, 0xFE, 0x09, 0xA0, 0xFA, 0x8E, 0x83, 0xC3, 0x4F, 0x64, 0xD3, 0x77, 0x04, 0xAF, 0x96, 0xBF, 0xBE, 0x00, 0xCD, 0xE2, 0x68, 0x62, 0x59, 0xAD, 0xFB, 0x4B, 0x62, 0xF0, 0xB2, 0x35, 0x7F, 0x5E, 0xB4, 0x13, 0xD0, 0x54, 0x65, 0x3A, 0xDD, 0x9C, 0x1A, 0xB5, 0x95, 0x6C, 0xF0, 0x43, 0x97, 0xBB, 0x8E, 0xE4, 0xF9, 0xDE, 0xAB, 0xCF, 0x83, 0x64, 0xB3, 0xBD, 0x2C, 0xA6, 0x91, 0x18, 0xB2, 0x9E, 0xF4, 0x71, 0x6C, 0xF1, 0xD6, 0x91, 0x77, 0xF7, 0xE8, 0x95, 0x15, 0x89, 0xD4, 0xBC, 0xA1, 0xD5, 0x29, 0x99, 0x99, 0x08, 0x29, 0xE7, 0xEB, 0xF3, 0xAA, 0x63, 0x56, 0x2F, 0xA8, 0x1C, 0x6C, 0xFD, 0xCD, 0xC9, 0x1D, 0x3F, 0x54, 0x1D, 0x2C, 0xBA, 0x91, 0x38, 0xF3, 0xCB, 0xA1, 0xA2, 0x43, 0xF1, 0x17, 0x19, 0xC9, 0x23, 0x20, 0x0A, 0xCE, 0x1E, 0x4E, 0x8E, 0x5D, 0x3D, 0x2C, 0x45, 0x82, 0xA1, 0x52, 0x3F, 0xF3, 0x54, 0xC1, 0x24, 0x0F, 0x30, 0x61, 0x72, 0xAC, 0x15, 0x5D, 0x5E, 0x2E, 0x8F, 0xE0, 0xDA, 0x84, 0xE2, 0x7A, 0x36, 0x04, 0xDE, 0x00, 0x71, 0x2E, 0xA6, 0x70, 0x2C, 0xE3, 0xEB, 0x09, 0xFE, 0xD3, 0x85, 0x9D, 0x4C, 0xB7, 0xD4, 0xA6, 0x08, 0x29, 0xAC, 0x68, 0x54, 0x84, 0xC7, 0x70, 0x5B, 0x31, 0x62, 0xDB, 0xF9, 0x43, 0xE0, 0x2F, 0x3B, 0x49, 0x07, 0xE5, 0x67, 0xD3, 0x74, 0x86, 0xFE, 0x78, 0x95, 0xAD, 0x66, 0x8E, 0x2E, 0xBA, 0x34, 0xB4, 0x77, 0xB5, 0xCE, 0x91, 0xF8, 0xCC, 0x80, 0xE5, 0xFE, 0x91, 0x5D, 0x86, 0xB1, 0x65, 0x84, 0xD5, 0xBC, 0xCA, 0xC9, 0x8F, 0x25, 0x9B, 0xAC, 0x18, 0xCA, 0x47, 0x3D, 0xCC, 0xE5, 0xB4, 0xBE, 0x1E, 0x72, 0xC0, 0xF2, 0xA6, 0xAF, 0xF8, 0x1D, 0xB3, 0x52, 0xC9, 0x6D, 0xE5, 0x46, 0x42, 0xB2, 0x81, 0x2A, 0x6E, 0xE8, 0xAA, 0x5C, 0xBD, 0xA3, 0x55, 0x5C, 0x41, 0x12, 0x21, 0x73, 0x63, 0x10, 0x2A, 0x9D, 0x44, 0x70, 0xB0, 0x08, 0xF1, 0xAA, 0x50, 0xF6, 0x2D, 0x4C, 0xB4, 0xC8, 0x30, 0xE2, 0xB1, 0xD3, 0x98, 0xFB, 0xFD, 0xEA, 0x8C, 0x6B, 0x48, 0xF5, 0x87, 0x09, 0xE4, 0x80, 0x12, 0xA5, 0x16, 0x78, 0x02, 0x57, 0x77, 0x4C, 0x5B, 0xA8, 0x77, 0x90, 0x83, 0x8B, 0x60, 0x9E, 0xC9, 0x57, 0xD6, 0x10, 0xE2, 0x16, 0x0D, 0xE7, 0x0B, 0xB3, 0x28, 0x6B, 0x16, 0x36, 0x8E, 0x5B, 0xA6, 0x89, 0x12, 0xF1, 0x35, 0x6F, 0x2A, 0xBD, 0x71, 0x7E, 0x73, 0xD6, 0xE2, 0xEE, 0xE8, 0x44, 0x83, 0xFC, 0x5B, 0xE3, 0xA3, 0xAF, 0xDB, 0xE8, 0x5F, 0x4F, 0xCD, 0x59, 0x50, 0x75, 0x53, 0x7F, 0xE6, 0x59, 0x58, 0x5E, 0xA0, 0xBE, 0x50, 0xF6, 0x6B, 0x50, 0xD1, 0x46, 0x1D, 0x04, 0x97, 0xB6, 0x09, 0xC2, 0xCA, 0xFA, 0x9A, 0xBC, 0x1A, 0x92, 0x0A, 0x30, 0x14, 0xFE, 0x76, 0xE7, 0xF3, 0xA8, 0xEA, 0x74, 0xE8, 0x82, 0xDC, 0x15, 0x12, 0xD2, 0x1E, 0xB9, 0x31, 0xD8, 0x0C, 0xDF, 0x3C, 0x3A, 0x5E, 0xE9, 0x7A, 0x95, 0x00, 0x62, 0x52, 0x9C, 0x8E, 0x08, 0x1E, 0xD7, 0x67, 0x7C, 0x35, 0x48, 0xE0, 0x88, 0xE1, 0xBC, 0xAE, 0xE6, 0xB8, 0xE2, 0xE4, 0xFF, 0xEE, 0x59, 0xDB, 0x35, 0x2B, 0x37, 0x99, 0x95, 0x18, 0xBE, 0x8F, 0x53, 0xB9, 0xA3, 0xB7, 0x55, 0x2C, 0x94, 0x8B, 0x53, 0xCB, 0x0D, 0x8A, 0x89, 0x29, 0xBB, 0x50, 0x7F, 0xAB, 0x32, 0x89, 0xB5, 0x2E, 0x93, 0xF2, 0xFC, 0x64, 0xB8, 0x9B, 0xD0, 0x84, 0x3A, 0x4F, 0x4C, 0x78, 0xC8, 0x6E, 0x32, 0x96, 0x56, 0x29, 0x73, 0xC0, 0x43, 0xE5, 0xC7, 0x5D, 0x0A, 0x3D, 0xEF, 0x9C, 0x8A, 0xCE, 0xA6, 0x9A, 0xB1, 0xA2, 0x8F, 0x4B, 0x95, 0x80, 0x96, 0x8A, 0x59, 0xE1, 0x31, 0xBD, 0x46, 0x95, 0xCD, 0xC3, 0xB3, 0xD7, 0x6D, 0x27, 0xF7, 0xC9, 0x6E, 0xBD, 0x7F, 0xD4, 0x42, 0x5A, 0x7D, 0x6A, 0x0E, 0x60, 0xFC, 0xA7, 0x97, 0x2C, 0x5E, 0x21, 0x3A, 0xBC, 0x0C, 0xF5, 0x56, 0xCC, 0x22, 0x72, 0x23, 0x72, 0x3F, 0xC8, 0xA8, 0xB6, 0x83, 0xD7, 0x5F, 0xA1, 0xBC, 0x7D, 0xA7, 0xBE, 0xD0, 0xCB, 0x49, 0xF8, 0x9D, 0xB0, 0xF2, 0x0D, 0xF8, 0xD6, 0xB3, 0x78, 0x90, 0x38, 0xB1, 0xD2, 0x33, 0xE6, 0x65, 0x8A, 0x19, 0xAD, 0x8D, 0x0C, 0xF5, 0x57, 0xB3, 0x09, 0xC2, 0x68, 0x2D, 0x2B, 0xDC, 0x8A, 0xF5, 0xAF, 0x56, 0xF2, 0x58, 0x75, 0x95, 0x85, 0xC7, 0x8B, 0xE9, 0x40, 0xB3, 0x23, 0xB5, 0xF9, 0xDA, 0x4C, 0x8A, 0x1B, 0xF5, 0x1C, 0x2B, 0x27, 0x8C, 0x39, 0x90, 0xDC, 0xA1, 0x3F, 0xCA, 0xF0, 0x9A, 0xC0, 0xFC, 0x37, 0x83, 0x1D, 0x19, 0xEB, 0xB8, 0xDB, 0xAC, 0x59, 0x5F, 0xCB, 0x22, 0xD8, 0x7B, 0x51, 0xF8, 0x40, 0xB7, 0x99, 0x66, 0xF8, 0x1F, 0x6D, 0x00, 0x5A, 0x20, 0xA2, 0x22, 0xDE, 0x2E, 0xD5, 0xB9, 0xD2, 0xDC, 0xED, 0x64, 0xD9, 0x6A, 0x1F, 0xAD, 0x6E, 0xBF, 0x36, 0xBE, 0x40, 0x40, 0xB1, 0x17, 0x89, 0x56, 0x7F, 0x90, 0x88, 0x5A, 0xFC, 0x9C, 0xEC, 0x9C, 0xCE, 0xCC, 0xA4, 0xDC, 0x57, 0x62, 0x13, 0x36, 0x62, 0x56, 0x06, 0x48, 0x32, 0x94, 0x74, 0x80, 0xAD, 0x11, 0x51, 0xDC, 0x81, 0x8D, 0x55, 0xD6, 0x65, 0xEB, 0x86, 0x72, 0x24, 0x2C, 0xCF, 0x5D, 0xE7, 0x47, 0x39, 0x90, 0xEE, 0x8B, 0x10, 0x94, 0x81, 0xF4, 0x50, 0x85, 0xF2, 0xCC, 0x17, 0xDD, 0x46, 0x17, 0x92, 0xFC, 0x75, 0x6E, 0x62, 0xCA, 0x55, 0xCF, 0xCA, 0xD6, 0xC8, 0x1E, 0x14, 0x62, 0x62, 0x7D, 0x58, 0xA0, 0x16, 0x76, 0xFC, 0x74, 0xAE, 0x71, 0x27, 0x56, 0x53, 0x20, 0x10, 0x22, 0xFB, 0x75, 0x1C, 0x18, 0x11, 0x85, 0x22, 0x6F, 0x82, 0x3B, 0xED, 0x12, 0xDB, 0x0A, 0xCE, 0x35, 0x8F, 0x7B, 0x0D, 0xAE, 0xFF, 0xB4, 0x76, 0x6B, 0xE5, 0x54, 0xEB, 0xA4, 0x08, 0x65, 0xBC, 0xB7, 0x5E, 0x94, 0xD8, 0xF0, 0x4F, 0x9E, 0x5F, 0x55, 0xF9, 0x2C, 0x4D, 0xE4, 0x7F, 0x3E, 0xED, 0x36, 0xF1, 0xAB, 0xB2, 0x7F, 0xCA, 0x7A, 0x78, 0xE7, 0x0F, 0xC5, 0x2B, 0xD2, 0x26, 0x34, 0x89, 0x85, 0x08, 0x5D, 0x4F, 0xCB, 0x96, 0xA0, 0xC5, 0x04, 0x2F, 0x02, 0x49, 0x95, 0xD9, 0xCD, 0x69, 0xCA, 0x03, 0xAD, 0x6C, 0xF4, 0xF4, 0x37, 0xF2, 0x74, 0x5F, 0xEB, 0xDC, 0x64, 0xB3, 0x60, 0x2F, 0xE3, 0x83, 0xE2, 0x50, 0xF1, 0x44, 0x98, 0x48, 0x14, 0x66, 0x40, 0xE7, 0x0A, 0x48, 0x34, 0xDF, 0x8D, 0x8D, 0x83, 0x09, 0x3B, 0xA7, 0x8F, 0x32, 0x13, 0x2C, 0x85, 0xD6, 0xDD, 0x08, 0x09, 0xB6, 0xCB, 0x14, 0x42, 0x43, 0xFC, 0x5E, 0x9A, 0x99, 0x38, 0xE7, 0xA0, 0x75, 0xDA, 0xE9, 0xCA, 0xDE, 0xA7, 0x93, 0xDC, 0x46, 0x36, 0xB0, 0xFE, 0xF7, 0x29, 0x6A, 0xBB, 0x63, 0x15, 0xD3, 0x75, 0xFD, 0x4D, 0x1E, 0xEF, 0x87, 0xD2, 0xFE, 0xA7, 0xC5, 0xAA, 0x19, 0x1B, 0x62, 0x7E, 0x0D, 0x73, 0x5F, 0x32, 0x44, 0xB7, 0xD0, 0x2F, 0xBC, 0xEC, 0x7D, 0x90, 0x75, 0xC8, 0xD8, 0xA5, 0x18, 0x31, 0x7F, 0xD5, 0x5B, 0x79, 0xB7, 0x27, 0x5F, 0xE8, 0xE9, 0x41, 0x48, 0x25, 0xAD, 0x30, 0xD9, 0x6A, 0xAA, 0x3B, 0xC5, 0xB8, 0xCF, 0x98, 0x45, 0x75, 0x3C, 0xE4, 0x51, 0x11, 0x76, 0xEA, 0x22, 0x11, 0x65, 0x6E, 0xEE, 0x95, 0xE4, 0x18, 0x66, 0xBD, 0x6F, 0x93, 0x03, 0x17, 0x4E, 0x0A, 0xC9, 0x3E, 0xA8, 0x6D, 0x43, 0xCB, 0xEF, 0xF1, 0x91, 0x55, 0x49, 0xDC, 0x1E, 0xC3, 0x77, 0xF9, 0xF9, 0x98, 0x3E, 0x6B, 0x24, 0x11, 0xC5, 0x1D, 0xE0, 0x04, 0x36, 0x11, 0xDB, 0xD9, 0x7F, 0x45, 0x58, 0xDA, 0xB9, 0x04, 0xDE, 0xB7, 0x58, 0xFC, 0xAD, 0xEB, 0xD8, 0xE7, 0x17, 0x96, 0xA0, 0x42, 0x1E, 0x82, 0x11, 0x0C, 0xA6, 0x0D, 0x21, 0xEA, 0xB3, 0x0F, 0xAE, 0xC9, 0x38, 0x17, 0xC1, 0x29, 0x03, 0x9D, 0x47, 0xC4, 0x54, 0x19, 0xD3, 0x22, 0x69, 0x50, 0xED, 0x69, 0xC8, 0x34, 0x41, 0xDF, 0xD3, 0xF4, 0x8F, 0xB9, 0x90, 0x7A, 0x8A, 0x77, 0xE5, 0xF2, 0xF1, 0x44, 0xB8, 0x75, 0x4E, 0xF9, 0x70, 0xAE, 0x3C, 0x27, 0x7D, 0x1E, 0x75, 0x05, 0xF4, 0x3E, 0x28, 0xAC, 0xDB, 0xDC, 0x85, 0xAB, 0xB1, 0x0F, 0x4D, 0x31, 0x03, 0x1C, 0x76, 0x31, 0xBF, 0xB7, 0x11, 0xA2, 0x3A, 0xF9, 0x60, 0xC0, 0x8E, 0xE7, 0x1F, 0x0B, 0xE5, 0x39, 0x32, 0x35, 0x9F, 0x46, 0x77, 0x83, 0xCE, 0xBF, 0xEE, 0xD0, 0x8C, 0x91, 0xB6, 0x29, 0x22, 0x5D, 0x58, 0xD0, 0x3B, 0x89, 0x19, 0xA9, 0x6A, 0x4A, 0xEF, 0x5E, 0x64, 0xF0, 0x6C, 0x3F, 0x96, 0x08, 0x5A, 0x35, 0xD0, 0x6E, 0x2A, 0x8E, 0x53, 0x16, 0xFE, 0xC6, 0x3D, 0x67, 0x21, 0x92, 0x9A, 0x25, 0x49, 0xCE, 0xC0, 0xC1, 0x35, 0x7A, 0x7A, 0x0C, 0xF9, 0x17, 0x70, 0xCB, 0x92, 0x19, 0x6B, 0xE7, 0x92, 0x18, 0x7D, 0x74, 0x05, 0xC3, 0x4B, 0x44, 0x76, 0xEC, 0xF8, 0xA8, 0x31, 0xCB, 0x21, 0x4D, 0x86, 0x67, 0x5A, 0x98, 0x45, 0x4A, 0x26, 0xBA, 0x29, 0xDF, 0x89, 0x1C, 0xA4, 0xCF, 0x5E, 0xD9, 0x57, 0x58, 0xF6, 0xCA, 0xBC, 0x50, 0x77, 0x4A, 0xA5, 0x74, 0x53, 0xB1, 0x8F, 0xA0, 0xBA, 0xA2, 0xBE, 0x52, 0xC5, 0xC4, 0x1C, 0xC3, 0xDA, 0x4D, 0x67, 0x36, 0xD9, 0xFC, 0x0F, 0x88, 0x06, 0xBB, 0x52, 0x15, 0xF9, 0x45, 0x2A, 0xDD, 0x2B, 0xF2, 0x3A, 0x8D, 0x15, 0x20, 0x88, 0x10, 0x8D, 0x53, 0xA9, 0xEB, 0xB7, 0x82, 0x9F, 0x85, 0x08, 0x39, 0x86, 0xD4, 0xFD, 0x0D, 0xAC, 0xB0, 0x3A, 0x0F, 0x9D, 0x6D, 0xB1, 0x51, 0x36, 0xE4, 0x7B, 0x6C, 0xC0, 0x5D, 0xC6, 0xA7, 0xB1, 0xD9, 0x6D, 0xF0, 0x75, 0xE1, 0x2D, 0x7C, 0x06, 0x69, 0xE7, 0x6C, 0x6C, 0xA5, 0xBD, 0x5E, 0xAC, 0x73, 0x16, 0x3C, 0xB1, 0x50, 0xAE, 0x3E, 0x40, 0x74, 0xEA, 0x41, 0x39, 0x5F, 0x77, 0x82, 0x73, 0xCC, 0x85, 0x30, 0xDD, 0x0E, 0x89, 0x9D, 0x08, 0x97, 0xBF, 0x9F, 0xCC, 0xC2, 0x64, 0x29, 0x00, 0xA9, 0xA4, 0x87, 0x9D, 0x92, 0xF2, 0x9F, 0x43, 0x27, 0xD2, 0x02, 0x98, 0x54, 0x81, 0x2F, 0x18, 0xFC, 0x36, 0xE3, 0x2C, 0x00, 0xE3, 0xE9, 0x2B, 0x48, 0x97, 0x7A, 0x6D, 0x46, 0xBD, 0x8E, 0x05, 0x91, 0x75, 0x57, 0x1C, 0xFB, 0xC3, 0x3F, 0xC4, 0xC5, 0x71, 0x3D, 0xC8, 0x63, 0x1E, 0xCD, 0x82, 0x38, 0x91, 0xB9, 0x29, 0x67, 0x26, 0x32, 0x13, 0xF0, 0x54, 0x9B, 0x35, 0xD7, 0xB1, 0x8A, 0x1E, 0x87, 0xF2, 0xF6, 0x61, 0x20, 0x9D, 0xAD, 0x89, 0x05, 0xDD, 0x54, 0xED, 0x26, 0x1E, 0xBE, 0xA3, 0x1F, 0x1C, 0x42, 0x09, 0x52, 0x50, 0x10, 0x07, 0x99, 0x67, 0xB5, 0xAE, 0x65, 0xEF, 0x5B, 0x71, 0xE1, 0xDE, 0x8A, 0xF4, 0xB0, 0x16, 0xB0, 0x24, 0x58, 0x6B, 0xF8, 0x21, 0x25, 0xF2, 0x99, 0x79, 0x7C, 0x7F, 0xAA, 0x88, 0x77, 0xB5, 0x91, 0xBD, 0xDF, 0x8D, 0xF6, 0xFA, 0xEB, 0x15, 0x5B, 0x7F, 0x68, 0x9F, 0x0B, 0x99, 0xA8, 0x79, 0x37, 0x28, 0x4B, 0x88, 0xE1, 0xB8, 0x77, 0x7E, 0x4E, 0x45, 0xAB, 0x78, 0x7F, 0x9B, 0x1D, 0x7B, 0x01, 0x33, 0xD0, 0x22, 0x7D, 0x97, 0xDF, 0xBB, 0xA4, 0x41, 0x1C, 0xDD, 0x68, 0x87, 0xE1, 0x22, 0x55, 0x25, 0x05, 0x70, 0x08, 0x45, 0xF4, 0xA8, 0x44, 0xF4, 0xE0, 0x8C, 0xE9, 0x4D, 0xB3, 0x27, 0xCB, 0x01, 0x4D, 0x6D, 0x87, 0x74, 0x65, 0x0F, 0xE0, 0x89, 0xF6, 0xDE, 0xFC, 0x17, 0x15, 0xD2, 0xEF, 0x8A, 0xE5, 0xCD, 0x94, 0xF0, 0xD5, 0xB4, 0x2F, 0xC4, 0xA3, 0x56, 0xA5, 0xB2, 0xED, 0x0B, 0x5B, 0xF1, 0xB7, 0x84, 0xA0, 0x7E, 0x41, 0x6C, 0xDE, 0xAD, 0xF5, 0xCA, 0xCF, 0x92, 0x63, 0xA6, 0x34, 0xE7, 0xCA, 0x3A, 0x81, 0x36, 0xED, 0xED, 0xE8, 0x11, 0x3C, 0x67, 0xC5, 0x4B, 0x26, 0x9D, 0x1F, 0x3E, 0x93, 0xA5, 0xB5, 0xEA, 0xBD, 0x17, 0x39, 0xA7, 0xB9, 0xC4, 0x60, 0xC4, 0xD6, 0x44, 0x6E, 0xE5, 0xF3, 0xB6, 0xC4, 0x6E, 0x9D, 0x6C, 0x0E, 0x6E, 0x33, 0x1A, 0x36, 0xA4, 0xE1, 0x56, 0x38, 0x99, 0x92, 0x63, 0xE2, 0x31, 0xB2, 0xA8, 0x01, 0xDF, 0x55, 0x66, 0x01, 0x8B, 0x97, 0x73, 0x47, 0x1A, 0x05, 0xE1, 0xFA, 0xAE, 0x68, 0x8B, 0x4D, 0xD6, 0x7C, 0x17, 0x70, 0xE4, 0x61, 0xF2, 0xC8, 0x5D, 0xDE, 0xB7, 0x56, 0xF9, 0x89, 0xA7, 0x2D, 0xA7, 0x5C, 0x71, 0x51, 0x01, 0x54, 0xD9, 0x09, 0x24, 0x0C, 0x39, 0x22, 0x21, 0x91, 0x5F, 0x1E, 0x89, 0x54, 0x30, 0x05, 0xBA, 0xAE, 0x41, 0x7B, 0x2B, 0x51, 0xAB, 0xE8, 0x02, 0x73, 0x48, 0x5C, 0x2A, 0x91, 0x0D, 0x35, 0xB5, 0xBF, 0x76, 0x9A, 0x54, 0xC8, 0x61, 0x2F, 0x48, 0x69, 0x2D, 0xDD, 0x8F, 0x1F, 0xCF, 0xA5, 0x76, 0x59, 0x73, 0xB1, 0x9C, 0xA1, 0x53, 0xE0, 0x4D, 0xCA, 0xFD, 0x4B, 0x55, 0x1E, 0x24, 0xFF, 0x22, 0x00, 0xDA, 0xF5, 0x85, 0x98, 0x8B, 0x9D, 0x1B, 0x5A, 0xDF, 0x3E, 0xEA, 0x93, 0x29, 0xF7, 0xA0, 0xCE, 0x27, 0xFC, 0xEA, 0x41, 0x66, 0x66, 0x12, 0xF8, 0x70, 0x91, 0x62, 0x80, 0x1A, 0x8E, 0x97, 0x15, 0xD8, 0x2A, 0x4A, 0x21, 0x4D, 0x21, 0xA0, 0xEE, 0x0E, 0x4E, 0xE1, 0x5A, 0xA8, 0x99, 0xC2, 0x31, 0xC1, 0x46, 0xF6, 0x09, 0xB8, 0x57, 0x85, 0x09, 0x03, 0x43, 0xEC, 0xB2, 0x47, 0x0D, 0x02, 0x37, 0x75, 0x7D, 0xE2, 0x77, 0x30, 0xBA, 0x1B, 0xE1, 0x86, 0x75, 0x5C, 0x28, 0xFF, 0xCF, 0x68, 0x90, 0x5E, 0xFD, 0xFD, 0x6A, 0xBB, 0x35, 0xF7, 0xF4, 0x59, 0xCF, 0x84, 0xCE, 0x30, 0x40, 0xE6, 0xC4, 0x65, 0x90, 0xB2, 0x73, 0x63, 0x4C, 0xE0, 0x17, 0xAC, 0x7C, 0x09, 0x84, 0x26, 0x50, 0x9C, 0x1D, 0x62, 0x3C, 0x4C, 0x0A, 0x60, 0x28, 0x05, 0xC4, 0xE8, 0xC2, 0xDE, 0xB1, 0x62, 0x0B, 0x04, 0x4C, 0x4D, 0xB3, 0xDC, 0xE1, 0xAA, 0xF2, 0x41, 0x62, 0x6E, 0xCC, 0x97, 0xCE, 0xD1, 0xD4, 0xF0, 0x93, 0xD9, 0x01, 0xC0, 0xBC, 0x04, 0x53, 0x0C, 0x9D, 0x14, 0x46, 0xAE, 0x12, 0xEF, 0x4B, 0x05, 0x1B, 0x85, 0x2D, 0x65, 0xC5, 0xBC, 0x59, 0x04, 0xA7, 0x73, 0x15, 0xFE, 0x2F, 0x59, 0xB8, 0x22, 0x82, 0x36, 0x64, 0xD6, 0x0A, 0x9A, 0x48, 0x4A, 0xB5, 0x52, 0x0A, 0x36, 0x07, 0x1B, 0xF0, 0x1B, 0x7A, 0x14, 0xE8, 0x56, 0xC4, 0x35, 0x1C, 0xC1, 0x13, 0x5C, 0x3F, 0xD1, 0x9B, 0xA2, 0x6E, 0xD9, 0x5E, 0x81, 0xA1, 0xFE, 0xBC, 0x27, 0xC4, 0x22, 0x95, 0xC9, 0x53, 0xA4, 0x1F, 0x75, 0xB1, 0x11, 0xA1, 0x30, 0x3E, 0x74, 0xC8, 0x66, 0x31, 0x45, 0x0C, 0x2E, 0xE1, 0x73, 0xCB, 0x19, 0xCC, 0x3A, 0xDA, 0x2A, 0x26, 0xF8, 0xB2, 0x40, 0x46, 0xC5, 0x55, 0xB1, 0x0D, 0x8F, 0xB9, 0x25, 0x66, 0xA6, 0x6E, 0x51, 0xE5, 0xC2, 0x44, 0xF5, 0xDA, 0x08, 0x8C, 0x73, 0xD0, 0x0E, 0x43, 0xDE, 0xAC, 0xB6, 0x8D, 0x1D, 0x4C, 0x62, 0x13, 0xE6, 0x9F, 0x17, 0x89, 0xA9, 0x32, 0xF4, 0xFB, 0xF6, 0x7C, 0x93, 0xA8, 0x40, 0x8F, 0xD1, 0x81, 0x1E, 0x3C, 0x02, 0x5E, 0xDC, 0x01, 0xA5, 0x4A, 0xFE, 0x96, 0x30, 0xCA, 0x59, 0x09, 0xD5, 0x6E, 0xBB, 0x62, 0x80, 0x40, 0xFA, 0xE8, 0xE3, 0x62, 0x07, 0xF4, 0x36, 0x45, 0xF0, 0x1E, 0xCC, 0xA8, 0xB9, 0x36, 0x65, 0xE0, 0x50, 0x01, 0x2A, 0x9B, 0xCF, 0xEA, 0x51, 0x17, 0x3B, 0x1F, 0xD0, 0x66, 0x1F, 0x01, 0x88, 0x3C, 0x21, 0x27, 0xD3, 0x87, 0xD6, 0x20, 0xD6, 0x89, 0x67, 0xD7, 0xF6, 0xA8, 0xFE, 0xF8, 0xDB, 0xDF, 0x5D, 0x3F, 0xEF, 0x05, 0x1F, 0x3E, 0x92, 0x7B, 0x39, 0x46, 0x34, 0x1F, 0x0D, 0x5A, 0x5E, 0x5B, 0x89, 0x1E, 0x2E, 0x1D, 0x77, 0x6F, 0xA1, 0xD7, 0x9B, 0x53, 0xD0, 0x92, 0x71, 0xD1, 0xEA, 0x97, 0x4F, 0x95, 0xB2, 0xD6, 0x47, 0x34, 0x56, 0xA1, 0xD3, 0xA6, 0x01, 0x76, 0xB7, 0xD7, 0x0B, 0x09, 0x71, 0x04, 0xE3, 0x79, 0x76, 0x64, 0xC3, 0x7E, 0x84, 0x78, 0xB5, 0x84, 0x35, 0xF2, 0xFD, 0xF0, 0x49, 0xA0, 0x0D, 0x87, 0xE1, 0x9E, 0xA5, 0xF7, 0x1F, 0x18, 0x63, 0xC9, 0x65, 0x98, 0xC6, 0x96, 0x76, 0x1A, 0xF4, 0x34, 0x09, 0x67, 0xCB, 0x74, 0x02, 0x4A, 0x9F, 0x9F, 0x5A, 0xDD, 0x5D, 0xA9, 0x02, 0xC7, 0x4A, 0x61, 0xD4, 0x46, 0xAA, 0xC3, 0x8F, 0x52, 0xF7, 0x54, 0x39, 0x2A, 0x26, 0xD8, 0xB9, 0xBF, 0x1D, 0x24, 0x40, 0x23, 0xB3, 0xCD, 0xEB, 0x37, 0x42, 0xDB, 0xEF, 0x01, 0x82, 0x9C, 0x95, 0x4C, 0x17, 0x5F, 0x21, 0x2E, 0x70, 0x62, 0x84, 0xCF, 0x08, 0xB1, 0x1A, 0x7E, 0x48, 0x33, 0x2B, 0x1A, 0x3D, 0xE4, 0xD1, 0x84, 0x20, 0x7B, 0xAF, 0xFB, 0x3D, 0xA9, 0x31, 0xBC, 0x36, 0x36, 0x58, 0xC6, 0x5B, 0xCE, 0x87, 0x07, 0xB7, 0x78, 0x01, 0x43, 0x6F, 0x05, 0x1A, 0x3A, 0xFB, 0xBB, 0x90, 0x7F, 0xDA, 0xC9, 0x6F, 0x36, 0x61, 0xCF, 0xDF, 0x48, 0x2D, 0x17, 0x35, 0xA6, 0x94, 0x16, 0xC2, 0xF5, 0xCE, 0x7A, 0x24, 0x81, 0x55, 0x9B, 0x26, 0x63, 0x52, 0x3D, 0x33, 0xEB, 0xF8, 0xFD, 0xC5, 0x0F, 0x8F, 0x0C, 0x03, 0x7C, 0x18, 0xE0, 0x51, 0x80, 0x15, 0xCC, 0x5C, 0xD9, 0x3E, 0xA7, 0xF2, 0xA9, 0x94, 0x9F, 0x3A, 0x39, 0xC8, 0xCE, 0x94, 0xF6, 0xEF, 0x9B, 0xF8, 0x08, 0xE7, 0xD7, 0x3E, 0xA5, 0xAA, 0x64, 0xB5, 0x55, 0xBB, 0x42, 0xC5, 0xDC, 0x1A, 0x6E, 0x80, 0xE4, 0x53, 0x1A, 0x95, 0xB5, 0x91, 0x45, 0x4C, 0xDE, 0x62, 0x3B, 0x09, 0xD9, 0xA8, 0xCC, 0xC6, 0x40, 0xC0, 0x98, 0x44, 0xF2, 0x8E, 0x75, 0x93, 0x35, 0x45, 0xB8, 0xCF, 0xBA, 0xF3, 0x46, 0x73, 0xFC, 0x66, 0xE8, 0xB9, 0x6B, 0xD7, 0x3E, 0x14, 0xB1, 0xD8, 0xA2, 0x52, 0xA3, 0x7A, 0xBB, 0xF7, 0xFE, 0xFF, 0x9E, 0xC5, 0x4A, 0xC1, 0x99, 0xA7, 0x2F, 0x56, 0x72, 0xF6, 0xBC, 0xFB, 0x7F, 0x8B, 0x3B, 0x04, 0x31, 0x93, 0x88, 0x96, 0x34, 0x96, 0xBF, 0xE7, 0x15, 0xC5, 0x99, 0xEA, 0xF8, 0xAA, 0x6E, 0xA4, 0xDA, 0x3D, 0xE3, 0xEF, 0xD8, 0x2A, 0x22, 0xD8, 0x5B, 0xFC, 0x39, 0xDA, 0x3C, 0xF5, 0x41, 0xAA, 0x94, 0x87, 0x20, 0xF1, 0x75, 0x2D, 0x78, 0x3C, 0x5F, 0xDB, 0xE2, 0x2C, 0x12, 0xEA, 0x05, 0x03, 0xE1, 0x24, 0x6E, 0x4C, 0x19, 0x6E, 0xAD, 0xFE, 0xB0, 0x4A, 0xED, 0x30, 0x84, 0x80, 0xD4, 0x5B, 0xFB, 0x3C, 0x88, 0x94, 0x37, 0x75, 0xAB, 0x64, 0x4C, 0x10, 0xA8, 0x18, 0x83, 0x4F, 0x5B, 0x22, 0x79, 0x57, 0x13, 0x7B, 0xFB, 0xB4, 0xFF, 0x06, 0x70, 0xF6, 0xA7, 0xE7, 0xA4, 0xBB, 0xCB, 0x2D, 0xA2, 0x6E, 0xA6, 0x99, 0xA8, 0xFB, 0xA5, 0xCE, 0x39, 0x7C, 0x1E, 0xE8, 0x7A, 0x50, 0x3A, 0x1F, 0xF3, 0xEC, 0xAE, 0x3D, 0xAA, 0xB4, 0xD1, 0x0B, 0x94, 0x47, 0x09, 0x66, 0xBA, 0xBF, 0x58, 0x13, 0x89, 0x3D, 0xA5, 0x5B, 0xF3, 0x38, 0xDB, 0xA4, 0x62, 0x01, 0x61, 0xE4, 0xEA, 0x42, 0x0D, 0x44, 0x26, 0xFA, 0x4F, 0xA7, 0x4F, 0x4C, 0x52, 0x84, 0x9D, 0x07, 0x4C, 0xF3, 0x8F, 0xA1, 0x94, 0x3F, 0xC8, 0x3F, 0x8F, 0xAD, 0x08, 0x1E, 0xEE, 0xDC, 0xF3, 0xCF, 0x09, 0xB3, 0xEA, 0x41, 0x3E, 0x35, 0xC8, 0x50, 0xBE, 0x8C, 0x98, 0x47, 0x20, 0xA7, 0xE2, 0xC0, 0xC6, 0xCD, 0x65, 0x8A, 0xD1, 0xB6, 0xCE, 0xD1, 0x3C, 0xC0, 0xAC, 0x5E, 0x6E, 0x15, 0x11, 0x35, 0x9C, 0xAE, 0xEA, 0x1A, 0x5B, 0xBF, 0x69, 0x51, 0x9C, 0x98, 0xB7, 0x35, 0x5F, 0x71, 0x10, 0x43, 0x67, 0xA1, 0xA0, 0x68, 0xF4, 0x15, 0x38, 0xB0, 0x79, 0xAC, 0x54, 0x99, 0xE2, 0x22, 0x51, 0xBF, 0x0D, 0x02, 0x74, 0xDC, 0xD9, 0x6E, 0xCF, 0x47, 0x80, 0x06, 0xC3, 0x78, 0xAA, 0xD7, 0x37, 0x7F, 0x1E, 0xA1, 0xF8, 0x86, 0xFF, 0xF2, 0xB5, 0xBA, 0x00, 0xA7, 0x5D, 0x0E, 0x45, 0x32, 0x10, 0x1F, 0xAB, 0x4A, 0xE2, 0xAE, 0x28, 0xCA, 0xCE, 0xDE, 0x71, 0xDF, 0xAF, 0x90, 0x3B, 0x49, 0x46, 0xD8, 0x2F, 0xAF, 0x61, 0xAA, 0x6C, 0xED, 0xBA, 0x04, 0x48, 0x46, 0x36, 0x72, 0x6E, 0xAB, 0x51, 0x1A, 0x1C, 0x9C, 0x31, 0x8C, 0x52, 0x0F, 0x89, 0x49, 0xFF, 0xF5, 0x53, 0x7D, 0x20, 0x79, 0x06, 0xB6, 0x0F, 0x8C, 0xCD, 0x64, 0x11, 0x8C, 0x8B, 0x97, 0x4D, 0x96, 0x4C, 0x41, 0x43, 0x2F, 0xCB, 0x27, 0x6C, 0x71, 0xF8, 0x08, 0xA7, 0x85, 0x6F, 0x75, 0xA7, 0xE2, 0x30, 0x79, 0x48, 0x1A, 0x51, 0xDB, 0xAC, 0xF9, 0xAC, 0x86, 0x86, 0xC6, 0x94, 0x5E, 0x99, 0x56, 0xD2, 0x44, 0xB1, 0xAF, 0x7B, 0xC0, 0xB1, 0xB8, 0x56, 0xBA, 0x60, 0xBE, 0x31, 0x16, 0xC2, 0xD1, 0x22, 0xF3, 0xD0, 0xE2, 0x94, 0xB9, 0x8A, 0xA7, 0x08, 0x79, 0x83, 0xD1, 0x35, 0x28, 0xD1, 0xEF, 0x2F, 0x63, 0x5E, 0x61, 0xDC, 0xBB, 0xB8, 0x81, 0x3D, 0xEE, 0xB0, 0x07, 0x38, 0xA0, 0x9D, 0xA0, 0xCF, 0x2E, 0x2E, 0x4B, 0xBD, 0x24, 0x58, 0xE9, 0x99, 0xB6, 0x56, 0xF5, 0x42, 0x12, 0x61, 0xDB, 0xF7, 0x8E, 0xAC, 0xB4, 0x1E, 0x1D, 0x09, 0x6E, 0xD2, 0x40, 0xE8, 0x6B, 0x5E, 0x24, 0xE6, 0xAE, 0x62, 0x40, 0x3A, 0xC4, 0x49, 0xDA, 0x04, 0x09, 0xA6, 0x10, 0xB2, 0x98, 0x75, 0xDA, 0x52, 0xF0, 0x53, 0xCD, 0x7E, 0xD6, 0xC1, 0x00, 0x4F, 0x07, 0x28, 0xAB, 0xB3, 0x8F, 0x6C, 0x12, 0x29, 0xCD, 0x06, 0x55, 0x3A, 0x86, 0xDD, 0xC5, 0xEF, 0x3A, 0x7A, 0x99, 0x67, 0x3E, 0x76, 0xCC, 0xDB, 0x7D, 0xF8, 0x3D, 0xF3, 0x05, 0xE5, 0x39, 0xD9, 0xDC, 0xA7, 0x2E, 0x3F, 0xF7, 0xB6, 0x26, 0x87, 0x91, 0x20, 0x87, 0x41, 0xF8, 0x89, 0x95, 0x20, 0x93, 0x21, 0x34, 0xCF, 0x22, 0xDD, 0x59, 0x4E, 0x88, 0x73, 0xBA, 0x88, 0x09, 0x7B, 0x94, 0xF1, 0x8D, 0x0A, 0x7B, 0xDE, 0x7F, 0x5D, 0xAD, 0xD6, 0xC1, 0x9D, 0x0F, 0x6C, 0x95, 0x02, 0x2C, 0xB7, 0x07, 0xBD, 0xCB, 0xAE, 0xBD, 0x64, 0x8F, 0xAB, 0x24, 0xC5, 0xAA, 0x17, 0x16, 0x2F, 0x26, 0xA4, 0x2E, 0x8A, 0x2D, 0xF9, 0x12, 0xFA, 0x83, 0x08, 0xE4, 0x5D, 0xC8, 0x11, 0xDD, 0xCD, 0x52, 0x5F, 0xC0, 0x82, 0xA3, 0x5D, 0xD9, 0x52, 0xA7, 0x00, 0x0C, 0x78, 0x97, 0xE2, 0x9B, 0x1C, 0xCA, 0x6A, 0x4E, 0x35, 0x5E, 0x11, 0x63, 0xA5, 0x8E, 0xD4, 0x61, 0x2E, 0x11, 0xD7, 0x7B, 0x49, 0xDF, 0x82, 0x84, 0xE2, 0x7E, 0xD7, 0x23, 0x83, 0x47, 0x9A, 0x07, 0xA5, 0x70, 0x62, 0xDE, 0xDD, 0xEB, 0xD6, 0x7D, 0x0E, 0x88, 0x9E, 0x85, 0x84, 0x49, 0x58, 0xD3, 0x24, 0x47, 0xDD, 0x8C, 0x20, 0xDD, 0xE5, 0xD0, 0x42, 0xCB, 0x26, 0xF9, 0x56, 0xA1, 0xCF, 0xF3, 0xA5, 0x3E, 0x1F, 0x43, 0x0F, 0xD4, 0x7A, 0xC4, 0x02, 0x82, 0xFC, 0x41, 0xA1, 0x0F, 0x5F, 0x0F, 0xB7, 0x19, 0xE4, 0xF4, 0xF0, 0x7B, 0xAB, 0x13, 0x73, 0x26, 0x7B, 0x67, 0xE0, 0x06, 0x82, 0x49, 0x86, 0x6E, 0xC4, 0x15, 0xF5, 0xFC, 0xCB, 0x45, 0xB9, 0x81, 0x37, 0xC1, 0x13, 0x78, 0xE9, 0x36, 0xFE, 0x29, 0x89, 0x04, 0xB8, 0xB8, 0x21, 0x47, 0xF1, 0xFE, 0xA6, 0x5B, 0xE8, 0xDB, 0x44, 0x60, 0x1F, 0x6F, 0xE1, 0xC3, 0x5A, 0xC8, 0x02, 0x58, 0xCA, 0x2C, 0x6C, 0x7A, 0x3D, 0x6A, 0x38, 0x80, 0xBE, 0x5D, 0x06, 0x28, 0x02, 0x8A, 0x4A, 0x5A, 0x77, 0x29, 0x65, 0xD6, 0xE2, 0x55, 0x87, 0x83, 0xD0, 0x12, 0x9D, 0x9C, 0xE6, 0xBF, 0x89, 0x88, 0x85, 0x0A, 0x03, 0xD2, 0xDA, 0xA4, 0x3F, 0x39, 0x25, 0xF6, 0xB0, 0x91, 0xB6, 0x82, 0xDD, 0xDB, 0xA8, 0x59, 0x5D, 0x10, 0xC9, 0xB2, 0x7B, 0xAC, 0xF8, 0x96, 0x7D, 0xD2, 0x5B, 0xA4, 0xF7, 0x06, 0x06, 0x2D, 0x11, 0xA1, 0xE8, 0xC7, 0x4F, 0xF8, 0x8C, 0x4F, 0xEE, 0x04, 0xB1, 0x3E, 0x2E, 0xE3, 0x07, 0x2E, 0xDB, 0x46, 0x36, 0xD8, 0xA7, 0x8E, 0x94, 0xEE, 0x7C, 0xAC, 0x3F, 0xD2, 0x77, 0x10, 0x46, 0xBC, 0xA4, 0x76, 0x71, 0xCB, 0xF1, 0x00, 0x76, 0xD0, 0xF9, 0xBB, 0xEC, 0xE0, 0x8D, 0xE1, 0x50, 0xAB, 0xAE, 0x5E, 0xB9, 0xE5, 0x3A, 0xDE, 0xF6, 0xDF, 0x08, 0xDA, 0xF7, 0xE7, 0x4D, 0xBE, 0x36, 0x90, 0x09, 0xB6, 0xE3, 0x0C, 0xEE, 0xE5, 0x2B, 0x4D, 0x74, 0x05, 0x66, 0x94, 0x71, 0xD6, 0x7E, 0x12, 0x2A, 0x2C, 0xBE, 0x5E, 0x36, 0x44, 0x08, 0x5B, 0xFB, 0xBE, 0x12, 0x4A, 0x04, 0x5F, 0xB0, 0x67, 0x54, 0x58, 0x92, 0x31, 0xEA, 0x40, 0xE8, 0x36, 0x39, 0x24, 0x45, 0x65, 0x22, 0x67, 0x6E, 0xE6, 0x8C, 0x29, 0xC3, 0xB3, 0xFE, 0x99, 0xA2, 0xB5, 0x85, 0xCD, 0x20, 0xC0, 0xE1, 0xA4, 0xE3, 0xB9, 0xBC, 0xE6, 0xCB, 0x79, 0xC2, 0x93, 0x5F, 0x22, 0xCC, 0x7A, 0x68, 0x32, 0xD9, 0xE6, 0x86, 0xE4, 0x51, 0x60, 0x42, 0xC5, 0xFB, 0xD5, 0xD8, 0x97, 0x96, 0x1B, 0xE0, 0xB5, 0x9B, 0xF9, 0x3F, 0x63, 0xEE, 0xF0, 0xDD, 0x2E, 0x8D, 0x02, 0xAD, 0x33, 0x1A, 0x22, 0x99, 0xF1, 0x68, 0x33, 0xBD, 0xA9, 0x3D, 0x0D, 0x59, 0x61, 0x22, 0xAC, 0xFB, 0x4B, 0x29, 0xBB, 0x6A, 0x41, 0x3C, 0x0C, 0xB2, 0x0E, 0x08, 0x77, 0xC7, 0x7A, 0x7E, 0xB9, 0xA9, 0x4C, 0xBD, 0xA3, 0x51, 0xCA, 0xB9, 0xE6, 0x06, 0x1C, 0xD9, 0x8D, 0x87, 0x66, 0x8B, 0x07, 0xAD, 0x8F, 0x52, 0xD1, 0xAB, 0xBD, 0x50, 0x42, 0x23, 0xB8, 0x2C, 0x04, 0x70, 0x97, 0x4A, 0x0F, 0x79, 0xA0, 0x4A, 0xDF, 0xC4, 0xB1, 0xC6, 0xA5, 0xC3, 0x1C, 0xA8, 0x32, 0x17, 0xA0, 0xAB, 0xA5, 0xB1, 0xF8, 0x04, 0xB7, 0x73, 0x2F, 0x3B, 0x74, 0xEA, 0x44, 0x76, 0xAB, 0x51, 0xC5, 0xF8, 0xAF, 0x37, 0x75, 0xDE, 0x71, 0x7C, 0x16, 0x6B, 0x95, 0x72, 0xE3, 0x0B, 0x53, 0x00, 0xB4, 0x99, 0x11, 0x7A, 0x67, 0x4E, 0xA8, 0x28, 0x26, 0x7F, 0x66, 0x9B, 0x1A, 0x09, 0x46, 0xF9, 0x22, 0xDB, 0x7A, 0xA6, 0xF2, 0xBA, 0x8D, 0xE9, 0x2E, 0x0B, 0x88, 0x2C, 0x56, 0xC3, 0x72, 0xD4, 0xE4, 0x91, 0x6C, 0x25, 0x2F, 0x55, 0x7F, 0x24, 0xCE, 0xED, 0x1B, 0x1E, 0xA6, 0xDB, 0xB6, 0x9D, 0x79, 0x3F, 0x2A, 0x30, 0x9A, 0xA9, 0xA2, 0x71, 0x29, 0xE9, 0x39, 0xF4, 0xA0, 0x45, 0xF9, 0xF7, 0xCE, 0xAF, 0xEC, 0x78, 0x96, 0x84, 0xCD, 0x4B, 0xAA, 0x0C, 0x8E, 0x65, 0xAB, 0x2B, 0x0B, 0x29, 0x39, 0x2C, 0x76, 0x8D, 0x69, 0x1C, 0x35, 0x31, 0x40, 0x78, 0x74, 0xD9, 0x35, 0xFF, 0xFD, 0x25, 0xD0, 0xEC, 0xF6, 0xC2, 0x0B, 0xA5, 0xF7, 0xB6, 0x74, 0x23, 0xB2, 0x73, 0xCB, 0x2B, 0xBE, 0x26, 0xE4, 0x12, 0x9C, 0xEC, 0x49, 0x18, 0x3B, 0x42, 0x39, 0x48, 0x35, 0x96, 0x4E, 0xC1, 0xB7, 0x96, 0x89, 0x3C, 0x65, 0x4F, 0x44, 0xCE, 0x5A, 0xE3, 0x65, 0xC9, 0xC6, 0x39, 0xF3, 0x80, 0x6B, 0xEF, 0xE9, 0x5A, 0x11, 0x7B, 0xE6, 0xDE, 0xE6, 0x92, 0xFA, 0x8E, 0xE2, 0xBA, 0xCD, 0x8F, 0xDA, 0x56, 0x80, 0xDD, 0x6C, 0x20, 0xB8, 0xC6, 0xBC, 0x0D, 0x20, 0xDE, 0x17, 0xB8, 0x06, 0x68, 0x37, 0x69, 0x9B, 0x4C, 0x7B, 0x7E, 0x45, 0x6C, 0x16, 0x87, 0xE6, 0x12, 0xED, 0x74, 0x18, 0xC3, 0x91, 0x92, 0x24, 0x3A, 0xC4, 0x67, 0x98, 0x22, 0x60, 0xE3, 0x55, 0x2E, 0x2B, 0x39, 0x41, 0x77, 0x3A, 0x0A, 0xF1, 0xBC, 0x4A, 0x97, 0x94, 0xE7, 0x72, 0x54, 0xE5, 0x34, 0x05, 0xF2, 0x00, 0x8F, 0x64, 0x8D, 0x3C, 0x42, 0x9A, 0xB4, 0xA9, 0x86, 0x80, 0xF6, 0x77, 0x89, 0x09, 0x5E, 0x9F, 0x34, 0x9E, 0x1D, 0x83, 0x24, 0x7C, 0xB8, 0xF0, 0x0C, 0x63, 0xBF, 0x4B, 0xCE, 0xDB, 0xCC, 0x84, 0x59, 0x63, 0xFB, 0x49, 0x5B, 0x79, 0x5F, 0xFA, 0x8A, 0x7D, 0xD7, 0x00, 0x01, 0xDA, 0x1D, 0x5A, 0xF0, 0xD4, 0xB0, 0xDC, 0xCC, 0x62, 0xB0, 0x02, 0x75, 0x5F, 0xDC, 0xC9, 0xA7, 0xE0, 0xA7, 0x0D, 0x13, 0x7E, 0xCA, 0xCA, 0x4E, 0x90, 0xF7, 0x0B, 0x91, 0xA2, 0xC0, 0x0A, 0xF0, 0xA7, 0x61, 0x60, 0x8D, 0x8E, 0xB9, 0x4B, 0xA5, 0x7D, 0x09, 0x7F, 0x79, 0x84, 0x7F, 0x43, 0xDB, 0xA7, 0xEF, 0xB5, 0xDA, 0x00, 0xDE, 0xA7, 0x8F, 0xAB, 0x10, 0xD7, 0x8D, 0xB8, 0xD4, 0x14, 0x03, 0x30, 0xA0, 0xDF, 0x27, 0x06, 0x99, 0xD7, 0xE4, 0x87, 0xC2, 0x2D, 0xA1, 0x92, 0x77, 0xFE, 0x00, 0x89, 0x5F, 0x8D, 0x78, 0xCA, 0x1B, 0x4B, 0x61, 0xC7, 0x71, 0xB3, 0x67, 0x34, 0x9C, 0x1D, 0xE1, 0x24, 0x48, 0x4C, 0x13, 0xC7, 0x33, 0x80, 0xC1, 0x5B, 0x14, 0xFE, 0xCF, 0x0C, 0xD1, 0xB6, 0x14, 0x1E, 0xF2, 0x6D, 0x52, 0x50, 0x1C, 0xE2, 0x9E, 0x49, 0x04, 0x47, 0x8C, 0x8A, 0xEE, 0x18, 0x61, 0xE9, 0xF5, 0xA0, 0xB0, 0x6D, 0xC6, 0x6D, 0xA1, 0x40, 0x4B, 0x24, 0x62, 0xA6, 0xC2, 0xC5, 0xD6, 0x31, 0xD0, 0x0C, 0x00, 0x83, 0x92, 0xEE, 0x35, 0xB6, 0x2E, 0xF6, 0x18, 0x0B, 0x56, 0x4C, 0x43, 0x71, 0xFF, 0xB2, 0x97, 0x1F, 0x6B, 0xC1, 0x00, 0x1B, 0x16, 0x32, 0x8E, 0x04, 0x0D, 0xC9, 0x00, 0xB7, 0xC4, 0x79, 0x5E, 0xF2, 0xE7, 0x5E, 0x7E, 0x33, 0xB3, 0xE1, 0x42, 0x75, 0x78, 0xFA, 0x8E, 0x7D, 0xC8, 0xA5, 0xAC, 0x4D, 0x42, 0x8B, 0xDC, 0xD1, 0x25, 0xD0, 0xCD, 0x75, 0xFC, 0xAD, 0xE9, 0x17, 0xDA, 0x76, 0x47, 0x5B, 0x6D, 0xF4, 0xCA, 0x72, 0xB7, 0xB9, 0x97, 0x7A, 0x30, 0xB8, 0xF1, 0xA4, 0x84, 0x2F, 0x01, 0x52, 0xA5, 0x04, 0xF6, 0xD1, 0x59, 0xDD, 0x2E, 0x9F, 0xEC, 0x5F, 0x8E, 0x19, 0x6C, 0x59, 0xC0, 0x52, 0xE4, 0x64, 0x13, 0xAC, 0xFE, 0xF7, 0xE0, 0xD9, 0xCC, 0x30, 0xCC, 0x20, 0xC8, 0xF8, 0xEB, 0xB0, 0x14, 0xEA, 0xF5, 0xC5, 0x69, 0x71, 0x11, 0x60, 0x24, 0xA2, 0xC2, 0x53, 0xB2, 0x10, 0x79, 0x35, 0x98, 0x7E, 0x51, 0x95, 0xE5, 0xB5, 0x57, 0x7F, 0xE4, 0xAB, 0x3E, 0x00, 0x03, 0x6C, 0x25, 0x2F, 0x3A, 0xF4, 0x64, 0x6E, 0x54, 0x8A, 0x28, 0xB7, 0xFF, 0x15, 0xA6, 0x67, 0x19, 0xEE, 0x8C, 0xF8, 0x0F, 0xA1, 0x7C, 0x07, 0xB6, 0x4F, 0xC0, 0xC0, 0x9F, 0xF4, 0x0A, 0xCF, 0x91, 0xF8, 0xB1, 0x66, 0xB4, 0xE9, 0xFB, 0x1C, 0xD1, 0x5B, 0xB9, 0xD0, 0x75, 0x97, 0x3F, 0x27, 0xDE, 0xD4, 0xA7, 0x56, 0x3D, 0x8A, 0x75, 0x2C, 0xA2, 0x90, 0x51, 0xC8, 0x70, 0xF9, 0x29, 0x68, 0xB9, 0x94, 0x5D, 0x85, 0x43, 0x04, 0xB7, 0xC7, 0x76, 0x8A, 0xAB, 0x32, 0x8A, 0x66, 0xBC, 0x31, 0xF5, 0x2B, 0x8A, 0xA2, 0x58, 0xEF, 0x86, 0x6B, 0x04, 0x02, 0x1A, 0x5F, 0x12, 0x45, 0x40, 0xEB, 0xDA, 0x00, 0x67, 0x5D, 0x72, 0xB7, 0x9A, 0x5D, 0x73, 0xE6, 0x10, 0x02, 0x1E, 0x7D, 0xC6, 0x07, 0x77, 0x3E, 0xD4, 0xEF, 0x6E, 0xDC, 0x58, 0x68, 0x25, 0xAB, 0x1A, 0x81, 0x1B, 0x3F, 0x42, 0xE3, 0x18, 0xB2, 0xF8, 0x42, 0x49, 0xBA, 0xE2, 0xFC, 0x79, 0xCD, 0xE8, 0x88, 0x7C, 0x83, 0x69, 0xE1, 0xCF, 0x0C, 0x32, 0xF6, 0xB8, 0x6A, 0x68, 0xBE, 0x9A, 0xDF, 0xB5, 0xC8, 0x9C, 0x67, 0x29, 0xB8, 0xA2, 0xFB, 0x66, 0xEF, 0xB8, 0x8F, 0x50, 0xC7, 0x37, 0x2C, 0x92, 0xC1, 0x0D, 0x24, 0x7D, 0xE6, 0x09, 0xE2, 0x67, 0x16, 0xB6, 0x53, 0xBD, 0x3A, 0x74, 0xA0, 0x82, 0x4E, 0xA2, 0x93, 0xA3, 0xFB, 0x09, 0x79, 0xCA, 0x1B, 0x05, 0x19, 0xF6, 0xDF, 0xF6, 0xDA, 0x1E, 0xF9, 0x4D, 0x0F, 0x74, 0x79, 0x9D, 0xA1, 0x70, 0xE5, 0x57, 0x94, 0xE7, 0xBA, 0xA8, 0xC9, 0x2D, 0x61, 0x05, 0x09, 0x26, 0xE7, 0xA2, 0x5F, 0x09, 0xA0, 0x75, 0xF6, 0x33, 0xEE, 0x80, 0x9F, 0xB7, 0x71, 0x55, 0x5A, 0x3A, 0x10, 0xB6, 0xB0, 0xB0, 0x3F, 0xF4, 0x2F, 0x65, 0x07, 0x66, 0xF5, 0x84, 0x6A, 0x09, 0x12, 0x72, 0x5C, 0xFC, 0xF2, 0x9D, 0x68, 0x34, 0x9C, 0x77, 0xF6, 0xB8, 0xCF, 0x33, 0xFF, 0x6A, 0x91, 0xD6, 0xAE, 0x02, 0xD3, 0xCB, 0x71, 0x84, 0x77, 0x69, 0x40, 0xA0, 0xCC, 0xED, 0xC1, 0x4F, 0x80, 0xE4, 0x14, 0xE8, 0x89, 0xF3, 0xBD, 0xE6, 0x6B, 0xA1, 0x21, 0x78, 0x62, 0x0B, 0xE6, 0xF0, 0xE2, 0x71, 0x03, 0xA2, 0x8B, 0xCF, 0x1A, 0x2E, 0x0A, 0x42, 0x91, 0x36, 0x81, 0x50, 0xED, 0x72, 0x64, 0x22, 0xFB, 0x84, 0x7E, 0x55, 0x05, 0xC2, 0x7A, 0x10, 0xC8, 0x66, 0xF3, 0x6A, 0x1E, 0xD0, 0x0D, 0x49, 0xF1, 0xC9, 0x00, 0x78, 0x2F, 0x03, 0x1A, 0xB0, 0x51, 0xAD, 0x5C, 0x7B, 0x62, 0xE9, 0xC0, 0x3A, 0xE8, 0x4B, 0xD4, 0xDE, 0x5B, 0x2C, 0x94, 0x56, 0x09, 0x02, 0x9F, 0x0B, 0x57, 0xB5, 0xD4, 0x51, 0x12, 0xCB, 0x14, 0xF3, 0x47, 0x4A, 0x6F, 0x2E, 0x34, 0x48, 0x52, 0x64, 0x80, 0x57, 0x23, 0x19, 0xD0, 0xCA, 0xFF, 0xFD, 0xAE, 0xFF, 0x41, 0x8B, 0x0D, 0xDA, 0x18, 0x37, 0x8A, 0x10, 0x2B, 0x9C, 0x4F, 0x96, 0x9E, 0x25, 0x56, 0xDE, 0x41, 0x77, 0x7C, 0xED, 0xB4, 0x9E, 0x54, 0x49, 0x55, 0x65, 0xC0, 0x4B, 0x1D, 0xD6, 0x5A, 0xBF, 0xAC, 0x0D, 0x70, 0xFD, 0x1D, 0x72, 0xF1, 0xC2, 0x03, 0x42, 0xEE, 0xAE, 0xBA, 0xED, 0xE9, 0xD1, 0x45, 0x51, 0x23, 0x24, 0x58, 0x2F, 0xA6, 0xD4, 0x64, 0x27, 0xFC, 0x36, 0xF4, 0xC6, 0x85, 0xC1, 0x42, 0xA5, 0x0E, 0x3D, 0xCA, 0x00, 0x62, 0xC4, 0x83, 0x43, 0x29, 0xE6, 0xCF, 0x92, 0xEA, 0x02, 0x69, 0x25, 0x1F, 0xFC, 0x49, 0x61, 0x58, 0x07, 0x33, 0xC3, 0xF0, 0x38, 0x9A, 0x66, 0xB9, 0x65, 0x64, 0x4A, 0xCB, 0xD1, 0x25, 0x7E, 0x6E, 0x82, 0xEC, 0x3B, 0x6A, 0x90, 0x1D, 0x4E, 0x8C, 0xFA, 0xAF, 0xE7, 0x93, 0x32, 0x12, 0xC9, 0xA8, 0xE8, 0x8E, 0xD6, 0x64, 0x5D, 0x59, 0xA7, 0x8D, 0x20, 0xA1, 0x7A, 0xC2, 0xF1, 0x5C, 0x5B, 0xA1, 0xC8, 0x26, 0xD5, 0x17, 0x5F, 0xF5, 0x37, 0x8D, 0x9C, 0x38, 0xA1, 0x3A, 0x8D, 0xE5, 0x09, 0x79, 0x5D, 0x01, 0xE6, 0x6F, 0xFA, 0xEC, 0xF9, 0x7E, 0xA4, 0xDC, 0xAB, 0x58, 0x96, 0x91, 0xE5, 0xD9, 0x37, 0xE2, 0x67, 0xBE, 0x06, 0x1B, 0xDC, 0xC8, 0x52, 0x08, 0x6C, 0x0E, 0x5E, 0x08, 0x20, 0x57, 0x08, 0x52, 0x6F, 0x91, 0x0E, 0xF8, 0xD6, 0x85, 0x79, 0xC5, 0x48, 0xC6, 0xEA, 0xCD, 0x4E, 0x3A, 0x25, 0x8D, 0x3B, 0xB6, 0x06, 0x4F, 0x77, 0x47, 0xFE, 0xA5, 0x70, 0xD4, 0xF0, 0x0E, 0x0A, 0xEF, 0xFE, 0xBD, 0x1A, 0xFC, 0xB7, 0xD2, 0xCB, 0x64, 0xB4, 0xA3, 0x51, 0xC7, 0xBE, 0x41, 0xCA, 0x61, 0x61, 0x15, 0x46, 0xA6, 0x02, 0xC1, 0x9B, 0x5B, 0x6E, 0xCB, 0xB7, 0x21, 0x30, 0xF9, 0x7A, 0x17, 0x8A, 0x99, 0xB3, 0x2C, 0x2A, 0x11, 0xD1, 0x26, 0x25, 0x75, 0x74, 0x36, 0x49, 0x1E, 0x8F, 0xF1, 0xD3, 0xD9, 0xE8, 0x2F, 0x94, 0xED, 0x06, 0xF7, 0x1B, 0x55, 0x20, 0x19, 0xC5, 0xC7, 0x16, 0x14, 0xC7, 0x75, 0xC8, 0x7F, 0x8F, 0xC5, 0x75, 0x91, 0x64, 0xAB, 0x26, 0xDD, 0xDC, 0x54, 0x41, 0xBF, 0xDA, 0xD1, 0x64, 0x62, 0x63, 0xA3, 0xF2, 0x88, 0xD1, 0x21, 0x08, 0xA7, 0x81, 0xA9, 0x32, 0x71, 0x1A, 0x7E, 0x67, 0x44, 0xC2, 0x6A, 0xAD, 0xE2, 0xE1, 0xAF, 0x18, 0x39, 0x12, 0x26, 0x2D, 0xD9, 0xF8, 0xD8, 0x55, 0x65, 0x7A, 0xE3, 0x32, 0x56, 0xA1, 0x94, 0x13, 0xD0, 0xFC, 0x20, 0x80, 0xC3, 0xFE, 0x05, 0xCA, 0xE4, 0x48, 0xCD, 0x1A, 0x8D, 0x50, 0xEC, 0x8B, 0xFE, 0x71, 0x83, 0x9E, 0xEC, 0x84, 0x87, 0xE8, 0x3A, 0xF7, 0x7A, 0xAC, 0x44, 0x71, 0x78, 0x7F, 0x43, 0x15, 0x45, 0xF7, 0x55, 0x68, 0xE5, 0x64, 0xAD, 0x82, 0x3A, 0x19, 0x71, 0xEC, 0xB5, 0x99, 0xE9, 0x06, 0x9F, 0x25, 0xE9, 0x6C, 0xA7, 0xF5, 0xC7, 0x86, 0xE6, 0xE8, 0xD7, 0x76, 0x4E, 0x29, 0x30, 0x01, 0x7B, 0x86, 0xB5, 0x51, 0x09, 0x39, 0x2F, 0xD9, 0x06, 0x25, 0x33, 0x9B, 0x12, 0x27, 0xF3, 0x26, 0x0D, 0x7D, 0xFF, 0x7A, 0xAD, 0xE8, 0xE7, 0xB8, 0x78, 0x0B, 0x84, 0x69, 0x71, 0x87, 0xAB, 0x8F, 0x51, 0x10, 0x9A, 0xF7, 0x45, 0x79, 0x78, 0x01, 0x96, 0x83, 0x02, 0x95, 0x21, 0x9B, 0x20, 0x65, 0xB4, 0xEC, 0x0C, 0xDC, 0x40, 0xC3, 0x02, 0xBE, 0xE5, 0x79, 0x5F, 0x15, 0x04, 0xC3, 0xF2, 0x5C, 0xC6, 0x61, 0xAE, 0x27, 0xBB, 0x71, 0x8C, 0x0C, 0x51, 0x8F, 0xFD, 0x05, 0xB0, 0x0B, 0x58, 0x2F, 0x7D, 0xBF, 0x8B, 0x46, 0x8B, 0x75, 0x1A, 0xCD, 0x53, 0x9E, 0x99, 0xB6, 0x21, 0xE5, 0xF3, 0x22, 0x54, 0x62, 0x40, 0x97, 0x38, 0x02, 0xE2, 0x68, 0x9E, 0xE2, 0x6E, 0xE8, 0xE1, 0x94, 0xC8, 0x67, 0x26, 0xAF, 0x90, 0x31, 0xCA, 0x29, 0xBA, 0x24, 0xB8, 0x2E, 0x4A, 0x1E, 0x3A, 0x40, 0xE1, 0xE5, 0x1C, 0x7E, 0x3C, 0x5A, 0x10, 0x0D, 0x3B, 0x4A, 0x1C, 0x11, 0xF9, 0xEA, 0xE2, 0x84, 0x56, 0xC2, 0x8E, 0x0A, 0x49, 0xB8, 0x12, 0x46, 0xA3, 0x92, 0x17, 0xC0, 0xA2, 0xB3, 0xC3, 0x0C, 0xFE, 0x56, 0xD6, 0xFA, 0x9C, 0x2D, 0x46, 0x88, 0xDF, 0xF8, 0x2F, 0x54, 0xA4, 0xB4, 0x18, 0x13, 0x13, 0x1E, 0xCC, 0x69, 0x32, 0xCB, 0x48, 0x38, 0x56, 0xC8, 0xF8, 0xF4, 0x14, 0x79, 0x34, 0x31, 0x66, 0x98, 0x03, 0x3E, 0xF9, 0xC0, 0x5C, 0x9A, 0x5A, 0xE1, 0x89, 0x8E, 0x16, 0xA4, 0x40, 0x1E, 0x01, 0xAD, 0xA2, 0xE1, 0xF5, 0x91, 0xA8, 0x93, 0xCB, 0x2E, 0x39, 0xC0, 0xB1, 0xF6, 0xBC, 0x5F, 0xFF, 0x52, 0x2C, 0xF3, 0x15, 0xD6, 0x6C, 0x65, 0xEC, 0xC0, 0x58, 0x7D, 0xEE, 0x4F, 0xDF, 0xC5, 0x96, 0x56, 0xCA, 0x0F, 0xAA, 0xCA, 0xCB, 0xBA, 0x53, 0xF8, 0xB5, 0x65, 0x93, 0x2B, 0xEF, 0xC0, 0x17, 0xD9, 0x1C, 0xE5, 0xDA, 0xA9, 0x5B, 0x06, 0xA2, 0xF0, 0xCB, 0xCA, 0x75, 0x24, 0x38, 0x12, 0xBC, 0xE2, 0xDD, 0x7A, 0xAD, 0xFF, 0x7E, 0x7E, 0x6A, 0x8E, 0x69, 0x38, 0xF0, 0x74, 0xD9, 0x5E, 0x45, 0xBC, 0x5E, 0x57, 0x2D, 0x1E, 0x4E, 0xF8, 0x2F, 0x90, 0xD5, 0xD0, 0x32, 0x38, 0x6F, 0x49, 0x78, 0xEB, 0xC3, 0x05, 0x29, 0xAC, 0x97, 0x1A, 0xF1, 0x12, 0xBA, 0x1C, 0x64, 0x12, 0x9E, 0x1C, 0x5A, 0x23, 0xE1, 0x1A, 0xB3, 0xA7, 0xFA, 0x7D, 0x59, 0xE1, 0x9D, 0xAC, 0x25, 0x30, 0xDA, 0x17, 0x10, 0xB2, 0x5A, 0xC2, 0xE2, 0xA2, 0x1C, 0x3C, 0xAA, 0x5F, 0xFC, 0x35, 0xA8, 0x76, 0x40, 0x9F, 0xFF, 0x90, 0x42, 0xDB, 0x5B, 0xC7, 0x52, 0x60, 0x75, 0xDB, 0x1D, 0x68, 0x10, 0x9D, 0x9F, 0x43, 0xCF, 0x91, 0x3B, 0x3F, 0x03, 0xEE, 0xAD, 0x12, 0x67, 0xCE, 0x56, 0xBF, 0x41, 0x83, 0xBF, 0x32, 0x31, 0xE1, 0xD0, 0x39, 0xE9, 0xCC, 0xAC, 0x5C, 0xFC, 0xB4, 0x72, 0x71, 0xD8, 0x9A, 0xEB, 0x6A, 0xE2, 0x63, 0x45, 0x0D, 0xA5, 0xEF, 0xF3, 0x61, 0x0B, 0x78, 0x2C, 0xD9, 0x68, 0x5D, 0x24, 0x18, 0x1D, 0x7B, 0x54, 0x96, 0xBB, 0xC4, 0xE1, 0x1C, 0x15, 0x35, 0xAF, 0x37, 0x30, 0x1F, 0x96, 0x8A, 0xE8, 0x6B, 0xAA, 0x14, 0xFA, 0x68, 0x6E, 0x9F, 0x5D, 0x6D, 0x2F, 0x65, 0xA3, 0xBE, 0x6C, 0x8C, 0x45, 0x88, 0xE1, 0x5A, 0x39, 0x7F, 0xAA, 0x0B, 0x6C, 0xAF, 0x58, 0xA7, 0xD9, 0x35, 0x43, 0xE7, 0xB8, 0xC3, 0x0A, 0x3C, 0xCE, 0x2B, 0x0B, 0xFE, 0x22, 0xC8, 0x0F, 0xC3, 0x35, 0xDD, 0x0A, 0xA7, 0x81, 0x16, 0xD8, 0xF6, 0x69, 0x13, 0x9B, 0x84, 0x07, 0x34, 0x10, 0x72, 0x94, 0x09, 0xBE, 0x2C, 0x39, 0x19, 0x10, 0xCF, 0x84, 0x19, 0x37, 0x74, 0xB7, 0x8D, 0xE4, 0xE4, 0x4E, 0x18, 0x84, 0x71, 0x4F, 0x21, 0x23, 0x05, 0x88, 0xA3, 0xF9, 0x75, 0x34, 0x55, 0x2B, 0x51, 0x68, 0x1F, 0xAC, 0x4D, 0xA6, 0xF7, 0xB0, 0x8C, 0x24, 0xF6, 0x58, 0x7B, 0x7E, 0xF9, 0xF5, 0x22, 0x60, 0x7D, 0x63, 0x92, 0x46, 0x50, 0x79, 0x21, 0xBF, 0xAD, 0x9A, 0xB7, 0x48, 0x3E, 0x84, 0xAE, 0x35, 0xCF, 0xEC, 0x18, 0x3D, 0xF0, 0x04, 0xA3, 0x61, 0xC0, 0x6A, 0x28, 0x9C, 0x60, 0x11, 0xC9, 0xD2, 0xED, 0x79, 0x25, 0x38, 0xCF, 0x11, 0x76, 0xAB, 0xB3, 0x6B, 0x2D, 0x71, 0x97, 0x32, 0x7B, 0x3B, 0x2B, 0x88, 0xFC, 0x6A, 0x38, 0xC1, 0x61, 0x1C, 0x11, 0xD5, 0xF0, 0x40, 0xAA, 0x29, 0x62, 0x43, 0x2D, 0x1D, 0x08, 0xB4, 0x3C, 0x2C, 0x75, 0xF6, 0x11, 0x68, 0x39, 0x1F, 0x19, 0xF7, 0x6B, 0xD4, 0xCA, 0xDB, 0x9A, 0x82, 0x39, 0x21, 0xB3, 0xA9, 0xAE, 0x94, 0xBF, 0x56, 0x91, 0xE2, 0xCF, 0x2D, 0x66, 0x4E, 0xFB, 0xB1, 0xBB, 0xE4, 0x1B, 0xE2, 0x88, 0x6F, 0xD2, 0x59, 0xE2, 0xD5, 0xCA, 0x1A, 0x9E, 0xEE, 0xD4, 0x68, 0x73, 0xCE, 0xFF, 0x59, 0xB9, 0xA0, 0xBC, 0x47, 0x05, 0xA1, 0x1F, 0x1D, 0x0C, 0x8D, 0x75, 0x0B, 0x77, 0x8F, 0x48, 0xE6, 0x16, 0xA7, 0x97, 0x62, 0xB7, 0x95, 0x4A, 0xCB, 0x90, 0x4B, 0x28, 0x90, 0xA4, 0xD4, 0xC2, 0x4B, 0x8F, 0x91, 0x1A, 0x35, 0x6A, 0x74, 0xEB, 0x84, 0xEB, 0x4C, 0x28, 0xBB, 0x7E, 0xD7, 0x59, 0x65, 0xC0, 0x04, 0x62, 0xCF, 0x8A, 0x10, 0x9B, 0x4A, 0x70, 0xD2, 0xD8, 0x3C, 0x8F, 0x71, 0x5A, 0x5F, 0xA7, 0xE0, 0x2A, 0x05, 0xEC, 0xC6, 0xE2, 0x49, 0xF9, 0xF3, 0x42, 0x45, 0x13, 0xBA, 0xC0, 0x8A, 0x1A, 0x06, 0x52, 0xC3, 0xB1, 0x47, 0x58, 0x1B, 0xB4, 0x12, 0x15, 0x3D, 0x74, 0xCC, 0x15, 0x6C, 0x57, 0x1C, 0xC3, 0xA4, 0xB1, 0x1F, 0xDE, 0x70, 0xA3, 0xAE, 0x38, 0x9F, 0xBB, 0x10, 0x49, 0x66, 0x86, 0x58, 0xE0, 0xD3, 0xE4, 0x36, 0x6A, 0xC9, 0x0B, 0xAF, 0x37, 0x55, 0xA0, 0xAC, 0x2E, 0x3E, 0x2E, 0x30, 0x86, 0xEF, 0x74, 0xBA, 0x5C, 0xF9, 0x09, 0x32, 0xD9, 0x47, 0x27, 0x72, 0xEB, 0x90, 0xB0, 0x01, 0x3F, 0x54, 0x8A, 0xDF, 0xCA, 0x43, 0x59, 0x4A, 0x62, 0x0A, 0xD3, 0x21, 0x10, 0x86, 0x57, 0x0A, 0x24, 0x5D, 0x87, 0xFF, 0xB7, 0x25, 0x13, 0x66, 0xE9, 0xCE, 0x04, 0x5E, 0x7F, 0xB8, 0x0C, 0xC9, 0xF4, 0xAA, 0x3E, 0x75, 0x49, 0x62, 0xA3, 0x0F, 0xC3, 0xFE, 0xAE, 0x6A, 0xC6, 0x9E, 0x43, 0x70, 0x37, 0xA6, 0xC2, 0x6C, 0xED, 0xA4, 0x89, 0xC3, 0xB3, 0xFE, 0xC2, 0x2C, 0x99, 0xF9, 0x4C, 0xF5, 0x87, 0xC9, 0xD4, 0x88, 0xDA, 0x17, 0x87, 0x2B, 0xC7, 0x9D, 0x4E, 0x0B, 0x37, 0xCC, 0x24, 0xE9, 0x3B, 0x83, 0xA8, 0x01, 0xF2, 0x7B, 0x20, 0x2D, 0xCB, 0x65, 0x55, 0x18, 0x14, 0x0A, 0xF1, 0x98, 0xC7, 0xB4, 0xFA, 0xF2, 0x63, 0xDF, 0x3A, 0x0F, 0x49, 0x7B, 0x89, 0x45, 0x94, 0x6C, 0xF6, 0xAC, 0x60, 0x5A, 0x16, 0x1B, 0x29, 0x65, 0x02, 0xBC, 0x65, 0xAD, 0xC3, 0x30, 0xA7, 0x22, 0x9E, 0x20, 0x28, 0x16, 0x22, 0xED, 0xE5, 0xB3, 0xB0, 0xB1, 0xFC, 0xC4, 0xE1, 0xF9, 0xEA, 0x27, 0xA5, 0x4D, 0x17, 0x5F, 0xA2, 0x8D, 0xD7, 0xF5, 0x0C, 0x82, 0x65, 0x68, 0x01, 0x2B, 0x0B, 0xA9, 0x8D, 0x4F, 0x69, 0x8E, 0xA5, 0xC3, 0xF7, 0x98, 0x32, 0x16, 0xAF, 0x09, 0x02, 0xA3, 0xEA, 0x03, 0x70, 0x4D, 0x20, 0x61, 0x13, 0xF0, 0xED, 0x2B, 0x1C, 0x80, 0xB4, 0x99, 0x78, 0xB2, 0x52, 0x1B, 0x48, 0x3E, 0xEE, 0x16, 0xE1, 0x74, 0x57, 0x03, 0x2F, 0x4F, 0x65, 0x94, 0x55, 0x5C, 0x80, 0x81, 0x4D, 0xFC, 0xB6, 0xFF, 0x76, 0xCC, 0x31, 0x16, 0x00, 0xF8, 0xFF, 0x52, 0x5D, 0x62, 0xFF, 0xA1, 0xCE, 0x7B, 0x1D, 0x42, 0x90, 0x73, 0x87, 0x69, 0xEA, 0xBC, 0x9A, 0xD4, 0x84, 0x9E, 0x53, 0x48, 0x76, 0xB2, 0x0C, 0x3D, 0xD3, 0xB4, 0x12, 0x8F, 0x03, 0x3D, 0x3D, 0xFB, 0xA1, 0x97, 0xCA, 0x18, 0xE5, 0x45, 0x14, 0xF0, 0x04, 0x42, 0x23, 0xF5, 0x0B, 0xCD, 0x70, 0xAA, 0x11, 0xA8, 0x3B, 0x48, 0x2C, 0x20, 0x87, 0xE7, 0x27, 0x3D, 0x36, 0x7C, 0x85, 0x03, 0xF3, 0x0D, 0x2E, 0xDB, 0x64, 0xB2, 0xA2, 0x34, 0xB4, 0xAB, 0x8A, 0xA8, 0x4E, 0x1E, 0x97, 0x7E, 0x66, 0x9C, 0x6F, 0x37, 0x04, 0xDA, 0xAD, 0x52, 0x1D, 0xE0, 0xCC, 0x29, 0x05, 0x75, 0xF3, 0x87, 0x69, 0x70, 0x3C, 0x04, 0xA9, 0x03, 0x83, 0x59, 0x04, 0x10, 0xB0, 0xB1, 0x90, 0xDE, 0xC2, 0x1E, 0x27, 0x8D, 0x16, 0x72, 0x7B, 0xA8, 0x1D, 0xF9, 0xAC, 0x2D, 0xDB, 0x7E, 0xCE, 0x0D, 0x51, 0x03, 0xA0, 0x25, 0xAE, 0x21, 0x6E, 0x89, 0x67, 0xF8, 0x99, 0xAF, 0x00, 0x75, 0x70, 0x11, 0x4F, 0x10, 0x84, 0xCC, 0xE5, 0xFB, 0x98, 0xD4, 0x14, 0x77, 0x5D, 0x6E, 0xFC, 0x11, 0xEE, 0xF8, 0x9B, 0xED, 0x9A, 0xCE, 0xD1, 0x7E, 0x27, 0xEE, 0xE1, 0x3A, 0x2E, 0x4F, 0x4B, 0xEF, 0xD1, 0xFF, 0x74, 0x33, 0x97, 0x00, 0x85, 0x4C, 0x07, 0x22, 0x2C, 0xF6, 0x2D, 0xFE, 0x10, 0x3F, 0x7E, 0x19, 0x88, 0x06, 0x9E, 0xBC, 0xAC, 0x20, 0x26, 0x4D, 0x81, 0x1A, 0x67, 0xB7, 0x92, 0x71, 0xE2, 0x6C, 0xF1, 0x06, 0xCC, 0x2F, 0x0F, 0xA3, 0x1C, 0x89, 0x88, 0x78, 0xD1, 0x63, 0x10, 0x45, 0xEF, 0x52, 0xD8, 0xD8, 0x7E, 0x25, 0x39, 0xA5, 0x3C, 0x67, 0xC0, 0x6F, 0xD2, 0x8B, 0xF2, 0x23, 0x38, 0x5B, 0xB9, 0x98, 0xF1, 0x28, 0xDA, 0x52, 0x7C, 0x0A, 0x69, 0xA8, 0xD2, 0xFB, 0x85, 0x69, 0xFA, 0x67, 0xA8, 0x75, 0xE6, 0x9D, 0x69, 0x15, 0x40, 0x85, 0xBC, 0x1D, 0x44, 0x16, 0x8E, 0xD2, 0x9F, 0x35, 0x34, 0xDF, 0x8B, 0x54, 0xA7, 0x14, 0x27, 0xAF, 0xAF, 0x08, 0xA9, 0x4B, 0x53, 0x52, 0x50, 0x24, 0xB5, 0x10, 0x86, 0xA5, 0xEA, 0xE1, 0x46, 0xB8, 0xBB, 0x30, 0xA4, 0x13, 0x07, 0x12, 0x4D, 0xB1, 0xC2, 0x1F, 0x54, 0x85, 0xD6, 0xC6, 0x77, 0xA3, 0xA8, 0x9C, 0x14, 0xAE, 0xED, 0xCB, 0x65, 0x08, 0x2F, 0x48, 0x81, 0x6F, 0x83, 0xE8, 0x43, 0xBA, 0xF1, 0xC2, 0x52, 0xBD, 0x43, 0x18, 0xAB, 0xEA, 0xE4, 0x7F, 0xD9, 0x5D, 0x6B, 0x8A, 0xA8, 0xE2, 0x4B, 0x0D, 0x5D, 0xC1, 0x90, 0x64, 0x18, 0x9B, 0x3E, 0x8D, 0xE4, 0x98, 0x3F, 0x84, 0x4F, 0xFB, 0x68, 0xE4, 0xC9, 0x60, 0x2D, 0xF2, 0x08, 0xF9, 0x83, 0x6E, 0xE5, 0xD9, 0x90, 0x83, 0x26, 0x14, 0x7B, 0xB4, 0xC4, 0xCA, 0xC4, 0xA8, 0x4A, 0x27, 0x74, 0x86, 0x4A, 0xE3, 0x00, 0xD0, 0x74, 0x7D, 0x3A, 0xF2, 0xB8, 0x64, 0x1D, 0xA6, 0xFB, 0xE3, 0x9A, 0xB1, 0x53, 0x52, 0x38, 0x23, 0xE4, 0x6C, 0x67, 0x9E, 0x4A, 0x14, 0xDC, 0x90, 0xEA, 0xFE, 0x1C, 0x32, 0xFB, 0xC9, 0xBF, 0x1C, 0x7F, 0x72, 0xE3, 0x63, 0x66, 0x1E, 0x32, 0x08, 0xFD, 0x6C, 0x19, 0x2F, 0xCC, 0x28, 0x00, 0xBF, 0x8F, 0xA6, 0x6A, 0x8E, 0x72, 0x25, 0xD4, 0x7D, 0x27, 0xE8, 0xA1, 0x98, 0xF8, 0xC8, 0x82, 0xA1, 0x76, 0x27, 0x01, 0xF3, 0x61, 0xB1, 0x5B, 0x04, 0x7D, 0x75, 0xDA, 0x7D, 0xB5, 0x84, 0xF2, 0xF1, 0x1A, 0x6B, 0xDA, 0x34, 0xDB, 0x88, 0x62, 0x8E, 0x9E, 0x76, 0x63, 0x1C, 0x92, 0x5B, 0x4D, 0x74, 0x85, 0x86, 0xC9, 0x58, 0x78, 0xE6, 0xCC, 0x9B, 0x61, 0xA1, 0xF6, 0x3C, 0x78, 0x39, 0x6A, 0xE0, 0xD5, 0xF4, 0xEA, 0xE5, 0xC2, 0x12, 0x98, 0x55, 0x07, 0xDD, 0x0F, 0xD2, 0x17, 0x68, 0x0F, 0xDF, 0x48, 0xC4, 0x82, 0x3F, 0x03, 0x0D, 0xE0, 0x4F, 0x51, 0xA1, 0xDE, 0x89, 0x5C, 0x82, 0x1F, 0x45, 0x74, 0x6A, 0x78, 0x8C, 0x84, 0x19, 0x76, 0x9E, 0x24, 0x1F, 0xD0, 0x76, 0x3B, 0x95, 0x65, 0x42, 0xA5, 0x23, 0xE3, 0xB1, 0x50, 0x5F, 0x55, 0x45, 0x4E, 0xA7, 0xD6, 0x0B, 0x7C, 0x3B, 0x3F, 0x86, 0xFE, 0xCB, 0x34, 0x51, 0x74, 0xEF, 0xAA, 0x1D, 0xFC, 0x12, 0xAA, 0x3F, 0x53, 0x40, 0x72, 0xCF, 0xCC, 0xD8, 0x7E, 0x61, 0x1D, 0x0B, 0x3A, 0xC3, 0xF6, 0xC1, 0xDA, 0x18, 0xAD, 0x96, 0xA6, 0x75, 0xF7, 0x7A, 0x34, 0xA9, 0x77, 0x1E, 0xA6, 0x14, 0xA3, 0x5B, 0x58, 0x7D, 0x83, 0x84, 0xC7, 0x85, 0x3B, 0x6E, 0x89, 0xC7, 0xB5, 0x99, 0xFD, 0xA2, 0xEC, 0xF4, 0x92, 0xC0, 0x40, 0x37, 0xA1, 0x03, 0x7E, 0xE1, 0x83, 0x75, 0xAE, 0xBF, 0x1F, 0xA4, 0xD4, 0x8C, 0x0D, 0x2F, 0x15, 0xE9, 0x05, 0x66, 0xBE, 0xA9, 0xF2, 0x8A, 0x1C, 0xEA, 0x05, 0xA7, 0x1F, 0xBA, 0xDB, 0xD4, 0x3A, 0x09, 0x17, 0xB9, 0xD7, 0x6A, 0xCE, 0x80, 0xC8, 0xBB, 0x04, 0x7E, 0x31, 0xA7, 0x87, 0xC4, 0xAD, 0x6F, 0x3E, 0x88, 0x31, 0x2B, 0xE3, 0x54, 0x79, 0x95, 0xF0, 0xEB, 0x08, 0x4F, 0x51, 0xDC, 0xEF, 0x7D, 0x6F, 0x75, 0x95, 0x00, 0x44, 0xF6, 0x73, 0xFC, 0x63, 0x8F, 0x37, 0x57, 0x21, 0xFB, 0x50, 0x1A, 0x18, 0xED, 0x8B, 0x07, 0x61, 0x95, 0xE7, 0xDD, 0x45, 0x1D, 0x2F, 0xFC, 0x30, 0x9A, 0x60, 0xEE, 0x3E, 0xA4, 0x2F, 0xC6, 0x8A, 0xCB, 0x22, 0x4E, 0xC0, 0xF1, 0x07, 0xD2, 0xB5, 0x82, 0x62, 0xDD, 0x29, 0x12, 0xC3, 0x01, 0x27, 0x35, 0x1E, 0xD2, 0x84, 0xC1, 0x3C, 0x33, 0x18, 0x93, 0xF8, 0xD4, 0x03, 0xBA, 0x9A, 0x02, 0xD3, 0x6D, 0x47, 0x9C, 0x78, 0x11, 0x8C, 0xC5, 0xFF, 0xDA, 0xE6, 0xED, 0x12, 0x7E, 0xAF, 0x8A, 0x2F, 0xA5, 0xC5, 0x02, 0xE9, 0xD0, 0x35, 0x0A, 0x34, 0x53, 0x59, 0x62, 0x0F, 0xCA, 0x16, 0xEE, 0x67, 0x1B, 0xCB, 0x5A, 0x3F, 0x1E, 0xB6, 0xD7, 0xCE, 0x05, 0x0B, 0x73, 0xFD, 0xEE, 0x43, 0x49, 0x64, 0x35, 0x98, 0xB3, 0x57, 0x03, 0xF8, 0xEF, 0x75, 0xAF, 0x09, 0xA8, 0x1B, 0x52, 0xAC, 0xCC, 0xB1, 0xC1, 0x5B, 0x68, 0x7C, 0x65, 0x9D, 0xE2, 0x1F, 0x4D, 0xDD, 0x10, 0xDC, 0x14, 0xF9, 0xED, 0xC8, 0x04, 0xA2, 0xFA, 0x32, 0x70, 0x5B, 0x58, 0x03, 0x12, 0x04, 0xAA, 0xF5, 0xE8, 0x3D, 0x48, 0x97, 0x82, 0x2F, 0xE2, 0x85, 0x2A, 0x5E, 0xD5, 0xAF, 0x7C, 0x54, 0xC8, 0xF8, 0xAC, 0x70, 0x24, 0x16, 0x30, 0xC5, 0x3C, 0x8C, 0x1C, 0x1D, 0x7F, 0x20, 0xC0, 0x84, 0xDF, 0x62, 0x23, 0x61, 0xE7, 0xCD, 0x82, 0x8B, 0x34, 0xC7, 0x28, 0x3C, 0x3E, 0x2A, 0xDC, 0xDE, 0x13, 0x2A, 0xE7, 0x33, 0x54, 0x7A, 0x57, 0x02, 0x2D, 0xBD, 0xE5, 0x16, 0x68, 0x46, 0x2B, 0x12, 0x9E, 0xDA, 0x24, 0xF2, 0x04, 0x95, 0x60, 0xF2, 0x65, 0x63, 0xA6, 0xE2, 0x7F, 0x58, 0x58, 0x69, 0x35, 0xE8, 0xC3, 0xAA, 0xA1, 0x6D, 0x92, 0xC7, 0x49, 0x1D, 0xD7, 0x05, 0xDA, 0x61, 0xE2, 0x04, 0x6B, 0xCE, 0x8D, 0x5D, 0x1E, 0x23, 0x4A, 0xAC, 0xB6, 0x12, 0xA4, 0xCD, 0x65, 0x9D, 0x54, 0xBA, 0x04, 0xAA, 0xAB, 0x32, 0x81, 0x04, 0x53, 0x2A, 0x21, 0xB6, 0x95, 0x04, 0x21, 0x87, 0xEA, 0xB9, 0x86, 0x99, 0xA6, 0xE4, 0xDD, 0x13, 0x41, 0x0F, 0xBB, 0x02, 0xB3, 0xF9, 0xBD, 0x9A, 0xA3, 0x9B, 0x8B, 0x3B, 0x83, 0x31, 0xB3, 0xB7, 0x16, 0xA0, 0xA4, 0x83, 0xED, 0xE9, 0x91, 0x13, 0xDD, 0x17, 0x4F, 0xDA, 0x42, 0x47, 0x32, 0x0A, 0x2C, 0xD1, 0x49, 0xC1, 0x95, 0x41, 0x2F, 0x15, 0xCA, 0xAD, 0xA8, 0xC8, 0x58, 0xEE, 0x64, 0x8F, 0x1B, 0x61, 0xE7, 0xE5, 0x11, 0xD4, 0xC3, 0x3C, 0xFA, 0xEB, 0x77, 0x92, 0x7A, 0xC7, 0xC7, 0xEC, 0xDE, 0x29, 0x38, 0x73, 0xEB, 0x0A, 0x78, 0xCA, 0xCB, 0xD9, 0x51, 0x5B, 0x3C, 0x0E, 0x1E, 0x0D, 0xEF, 0xB2, 0xCF, 0x85, 0x2D, 0xE6, 0x82, 0xF0, 0x15, 0xCA, 0x92, 0xDF, 0x90, 0x28, 0x07, 0x13, 0x90, 0x49, 0x6E, 0x6C, 0xA9, 0x96, 0x68, 0x49, 0x6B, 0x17, 0xF3, 0xDB, 0xDA, 0xF4, 0x0D, 0x5F, 0x8C, 0xCC, 0x7D, 0xA1, 0x17, 0xEF, 0x9D, 0x38, 0xB7, 0xCF, 0xE6, 0xA2, 0xE8, 0x42, 0x54, 0x34, 0x68, 0x56, 0x46, 0x54, 0xD6, 0x30, 0x18, 0xCA, 0xC5, 0x39, 0x53, 0xF6, 0x55, 0xF2, 0xD1, 0x35, 0x83, 0x48, 0xF2, 0x7E, 0xDA, 0xE2, 0x68, 0x02, 0xE6, 0xD5, 0x98, 0x5A, 0x9E, 0x71, 0xA9, 0x75, 0x32, 0x1B, 0x74, 0x9B, 0x42, 0x29, 0xC2, 0x3B, 0x5A, 0xC3, 0xEB, 0xD8, 0x69, 0xC3, 0xEE, 0x78, 0xAD, 0x61, 0x13, 0x9E, 0xB6, 0x68, 0xD2, 0xC9, 0xCD, 0xE6, 0x6F, 0x19, 0x2D, 0xAD, 0x0F, 0xEC, 0x38, 0xF6, 0xE7, 0x91, 0xAD, 0xA5, 0x73, 0xD7, 0x4B, 0x04, 0x31, 0x43, 0xBF, 0xC5, 0x86, 0xB4, 0x6E, 0x86, 0x78, 0x3A, 0x67, 0x80, 0xF1, 0xE5, 0x24, 0x64, 0x2F, 0xDF, 0x2A, 0x09, 0x63, 0x0C, 0xF0, 0x13, 0x8E, 0xA1, 0x87, 0x19, 0x05, 0x4D, 0x60, 0x04, 0x23, 0x9A, 0x69, 0xE9, 0x2B, 0x4C, 0x26, 0x89, 0x63, 0x9F, 0x81, 0x1B, 0x21, 0x63, 0xC3, 0xF7, 0x98, 0xFA, 0xEA, 0x53, 0xF2, 0x3F, 0xD3, 0x6C, 0xD1, 0x26, 0xBB, 0x16, 0xBB, 0x40, 0x9F, 0x71, 0x8D, 0x0D, 0x6D, 0xB5, 0xA7, 0x83, 0x9B, 0x4F, 0x15, 0xCA, 0x71, 0x79, 0x2A, 0x48, 0x55, 0x75, 0xAC, 0x30, 0x0A, 0x7B, 0x47, 0x74, 0xCD, 0x4A, 0x5E, 0xA7, 0x32, 0x8A, 0x19, 0x82, 0x31, 0xF4, 0xA4, 0x42, 0xDF, 0x46, 0x5D, 0x7C, 0xA7, 0x75, 0x38, 0xC0, 0x69, 0x79, 0xCE, 0xCB, 0x9A, 0x9C, 0xAB, 0xBA, 0x32, 0xB0, 0xED, 0x82, 0x52, 0x5A, 0xF8, 0xB1, 0xD1, 0x84, 0x57, 0x48, 0x94, 0x92, 0x16, 0x4C, 0x1D, 0x5A, 0x5A, 0x75, 0x9D, 0x70, 0x4D, 0x26, 0x5E, 0x26, 0x85, 0xD2, 0xDC, 0x9D, 0x98, 0x5F, 0x5D, 0x17, 0x68, 0x1C, 0x0E, 0x8A, 0x1B, 0x02, 0xF5, 0xFB, 0x2E, 0xD4, 0x0C, 0x38, 0x62, 0x60, 0x25, 0x9B, 0x0E, 0x65, 0xDC, 0xE0, 0xD3, 0x2F, 0xE1, 0x2A, 0x85, 0x72, 0xC8, 0xAB, 0x28, 0x53, 0x4E, 0xB0, 0xD3, 0xA9, 0xA7, 0xA5, 0xEC, 0xEF, 0xC9, 0x0F, 0x4A, 0x1C, 0xFB, 0x9C, 0x0B, 0xD8, 0xEF, 0x64, 0xAD, 0xC7, 0xA1, 0xF6, 0x09, 0x28, 0x9D, 0x8A, 0x33, 0xB5, 0x35, 0xA0, 0x13, 0x26, 0x9A, 0x23, 0xD9, 0x01, 0x08, 0x55, 0x6E, 0x60, 0xB5, 0x2C, 0xEF, 0xB7, 0x39, 0x6D, 0xE2, 0x72, 0x87, 0x8E, 0x19, 0x45, 0xAF, 0xDB, 0x85, 0x53, 0x7D, 0x1D, 0x32, 0xE9, 0x7D, 0x84, 0x39, 0xD8, 0x1D, 0x5D, 0xCA, 0x3B, 0xE2, 0xB2, 0x47, 0xD4, 0x2B, 0x7B, 0x57, 0x53, 0x15, 0xB5, 0x8E, 0xA6, 0x43, 0x0C, 0xFA, 0x58, 0x29, 0x3E, 0x27, 0x42, 0xAB, 0x6C, 0xAC, 0x8A, 0x49, 0x58, 0x51, 0xBC, 0x5E, 0x97, 0x02, 0xE4, 0x0D, 0x23, 0x49, 0x48, 0x1A, 0x1A, 0x74, 0x19, 0x93, 0xB5, 0xE9, 0xC6, 0xF7, 0x53, 0x9A, 0x2E, 0x37, 0x8C, 0x6E, 0x8B, 0x27, 0x02, 0xFD, 0x49, 0x65, 0xCF, 0x2C, 0xFD, 0x5E, 0x84, 0xB2, 0x08, 0xAE, 0xE3, 0xB3, 0xB6, 0x70, 0xDB, 0x15, 0x5A, 0xED, 0xB4, 0xFE, 0x14, 0xA0, 0xF3, 0xA0, 0xB5, 0xED, 0x42, 0x08, 0xA1, 0x1E, 0x92, 0xE0, 0xD2, 0x14, 0x88, 0x6D, 0x38, 0x27, 0x73, 0x71, 0x45, 0x4C, 0x5A, 0x6E, 0x75, 0x7C, 0x30, 0x82, 0x30, 0xE4, 0x80, 0xB6, 0x62, 0xB0, 0x88, 0x08, 0x9C, 0x12, 0x75, 0x4F, 0x07, 0x78, 0x74, 0x61, 0x22, 0x1A, 0x74, 0x0B, 0xC6, 0x20, 0x89, 0x50, 0x64, 0x85, 0x90, 0xCD, 0xE2, 0x3F, 0x4C, 0x1A, 0xC5, 0xDD, 0x95, 0xF0, 0x07, 0x20, 0xE8, 0x50, 0x44, 0x70, 0xF7, 0xBD, 0xE9, 0x86, 0x51, 0xF9, 0x01, 0x95, 0xD8, 0x44, 0xF5, 0xD1, 0xCE, 0xA2, 0x41, 0x69, 0xE3, 0xA8, 0x58, 0xA5, 0xAC, 0x57, 0x44, 0x55, 0xD9, 0x1D, 0x77, 0x4D, 0x48, 0xB3, 0xFC, 0xD8, 0x84, 0xFE, 0xAA, 0x63, 0xC5, 0x11, 0xEF, 0x6B, 0x55, 0x98, 0x5C, 0xFB, 0x67, 0xEB, 0x81, 0xD5, 0x56, 0xAE, 0xCA, 0x18, 0x10, 0x2A, 0x76, 0x6E, 0x91, 0x5D, 0x5E, 0xE4, 0x35, 0x57, 0x33, 0xFC, 0x89, 0x30, 0xBD, 0x6D, 0x4A, 0xE0, 0x29, 0xEB, 0xE6, 0x47, 0x64, 0x0E, 0xF8, 0xD9, 0xC6, 0x8A, 0x33, 0x9B, 0xF3, 0xA4, 0x25, 0x17, 0xE4, 0x84, 0x44, 0xA7, 0x04, 0x64, 0xFE, 0xF4, 0x13, 0xAA, 0x96, 0x73, 0xA2, 0x46, 0x0E, 0x61, 0x80, 0xD0, 0x27, 0x92, 0xAD, 0xBE, 0xDD, 0x31, 0x6F, 0x01, 0x56, 0xE6, 0x5F, 0x70, 0x5E, 0x6E, 0x31, 0x42, 0x89, 0xFA, 0xC2, 0x24, 0x47, 0xA1, 0x17, 0x4A, 0x59, 0x45, 0x7A, 0x92, 0xD7, 0x07, 0x22, 0xA1, 0xD8, 0x4B, 0xC2, 0xE7, 0x07, 0x7A, 0x1F, 0x6F, 0xE7, 0x42, 0x53, 0xDD, 0xC7, 0xE2, 0x39, 0x4A, 0x4D, 0x1B, 0x12, 0xDD, 0xFD, 0x58, 0x33, 0x81, 0x8C, 0xA5, 0xF9, 0x43, 0x16, 0x41, 0x52, 0x30, 0x91, 0x0B, 0xBF, 0x0E, 0x31, 0x6D, 0x1A, 0x54, 0x9D, 0x37, 0x68, 0x01, 0x42, 0xFD, 0x42, 0x18, 0x5F, 0x0F, 0x10, 0xB0, 0x80, 0xBD, 0x64, 0x66, 0x0F, 0x45, 0x18, 0x12, 0x2C, 0xF0, 0x9D, 0x35, 0x83, 0xA7, 0x6C, 0x61, 0xA8, 0xDB, 0x1E, 0x5B, 0x22, 0xB9, 0xC5, 0xFF, 0x67, 0x4D, 0x57, 0x6E, 0xBC, 0x11, 0x77, 0x79, 0x04, 0x2A, 0x4C, 0x53, 0x0C, 0x34, 0xB5, 0x9E, 0x67, 0x6D, 0x8A, 0xF0, 0x6C, 0xE8, 0xA8, 0xBC, 0x6A, 0xBA, 0xAC, 0x42, 0x6B, 0x88, 0x81, 0x4F, 0x43, 0x17, 0x7A, 0x06, 0xD5, 0xD7, 0x2D, 0x1B, 0x07, 0x8B, 0xB8, 0x9E, 0x2E, 0xFF, 0x36, 0x0C, 0xE5, 0xFE, 0xBB, 0xDC, 0x7F, 0xC9, 0xC5, 0x93, 0x8B, 0x13, 0x05, 0x2F, 0x1A, 0x1B, 0x37, 0x83, 0xC2, 0x8E, 0x73, 0x1B, 0x23, 0x0D, 0xF4, 0xCE, 0xF2, 0x03, 0x10, 0x60, 0xF8, 0xEF, 0x96, 0x0D, 0xA2, 0x9D, 0x17, 0x65, 0xAB, 0x7F, 0x11, 0xF8, 0x20, 0x63, 0xB2, 0x91, 0xA5, 0xC5, 0xB9, 0xC5, 0x97, 0xE3, 0x29, 0x31, 0x87, 0xE1, 0x3D, 0x30, 0xE5, 0xEA, 0x87, 0x82, 0x3A, 0x0B, 0x66, 0x49, 0x08, 0x23, 0x23, 0xB5, 0xB8, 0x85, 0x4A, 0xBF, 0x31, 0x03, 0x42, 0x12, 0x33, 0xB4, 0x1A, 0xFE, 0xD8, 0x15, 0xDB, 0xBD, 0x74, 0x98, 0x5C, 0x5D, 0x9A, 0x6B, 0x0D, 0x8F, 0x5B, 0x3F, 0xAA, 0xE7, 0x92, 0x71, 0x3F, 0x99, 0x16, 0x47, 0xBF, 0xD6, 0xF2, 0x6F, 0x76, 0xED, 0xF7, 0x9B, 0x24, 0x04, 0x1C, 0x28, 0x8C, 0xC8, 0x6F, 0x05, 0xBC, 0xC4, 0xA1, 0x80, 0xBA, 0x03, 0xB4, 0x13, 0x31, 0x44, 0x51, 0x07, 0x98, 0x7B, 0xAF, 0xFD, 0x2A, 0x95, 0xF0, 0xDF, 0xC5, 0xB9, 0xC0, 0xB4, 0x59, 0x39, 0xB7, 0xD6, 0x3C, 0xB0, 0xCE, 0x90, 0x0E, 0x9A, 0x1A, 0x04, 0x23, 0x2B, 0xEE, 0xE4, 0x13, 0x6C, 0x53, 0xD9, 0x4E, 0x2C, 0xCC, 0x21, 0xEE, 0x9B, 0xBE, 0xE1, 0x11, 0xD6, 0x55, 0xE7, 0x6A, 0xF9, 0xC7, 0x60, 0xEA, 0xF6, 0x1C, 0xB4, 0xF5, 0x65, 0xCA, 0xF4, 0x4C, 0x46, 0x89, 0x29, 0x69, 0xB5, 0x29, 0xFF, 0xA0, 0xD9, 0x2B, 0x50, 0x6D, 0xAE, 0x19, 0x81, 0xA7, 0xA5, 0x09, 0x66, 0x56, 0x0A, 0xB9, 0x9D, 0x6C, 0x30, 0x94, 0x85, 0x81, 0x3B, 0xA1, 0xB1, 0x1D, 0x2B, 0xFF, 0x26, 0xB8, 0x46, 0x91, 0x60, 0x11, 0xC8, 0x2D, 0x85, 0xAA, 0x04, 0xC4, 0x95, 0xE7, 0x16, 0x11, 0x5D, 0x41, 0xBF, 0x41, 0x47, 0xC9, 0x04, 0x2A, 0xB9, 0xCA, 0x0E, 0x25, 0xBE, 0x6E, 0x85, 0xD0, 0x54, 0x26, 0x92, 0xE0, 0xAD, 0x3B, 0xEF, 0x49, 0x36, 0x7E, 0xE7, 0xC2, 0x66, 0x20, 0xFD, 0x6B, 0x36, 0x17, 0x3E, 0xB7, 0x3F, 0x50, 0xB2, 0xC8, 0x91, 0xF3, 0x47, 0xE6, 0xEA, 0x32, 0x5F, 0x88, 0x28, 0xEA, 0x3B, 0xF7, 0xBD, 0x53, 0xAB, 0xB1, 0xE0, 0x99, 0x1E, 0x62, 0xD6, 0x47, 0xFE, 0x6B, 0xF9, 0xA5, 0x9A, 0xF1, 0x30, 0xA3, 0xC7, 0x63, 0x39, 0xBD, 0xAD, 0xB2, 0x1D, 0xBA, 0xC9, 0xCC, 0xF7, 0x6D, 0xD5, 0x42, 0xF2, 0x31, 0x43, 0x3F, 0xD4, 0xEB, 0xB6, 0xC1, 0x06, 0x29, 0x6A, 0xAB, 0x8F, 0x86, 0xE1, 0x72, 0xCA, 0x57, 0x96, 0xF3, 0x50, 0x51, 0x65, 0x37, 0x00, 0x45, 0xFA, 0x29, 0xE9, 0xC2, 0x77, 0xE4, 0x71, 0xCD, 0xBE, 0x52, 0x44, 0xBD, 0xDD, 0x2A, 0x12, 0x08, 0xE7, 0x8E, 0xFD, 0xB8, 0x55, 0x7A, 0x08, 0x7D, 0xAF, 0x29, 0x85, 0x4E, 0xCF, 0xDD, 0x9A, 0x60, 0xC2, 0x6D, 0x8E, 0x71, 0x3E, 0xC1, 0x1E, 0xB9, 0x20, 0xE8, 0x1D, 0xB3, 0x4E, 0x00, 0x04, 0x52, 0x80, 0x4B, 0x02, 0x89, 0x54, 0xA8, 0xE9, 0x08, 0xF6, 0xC8, 0x6D, 0xB8, 0xCE, 0x04, 0x72, 0x06, 0x77, 0x74, 0x44, 0x64, 0x7A, 0xB5, 0x59, 0x75, 0xA7, 0x74, 0xE1, 0x68, 0xA7, 0x24, 0xF7, 0x09, 0xFA, 0x32, 0xC8, 0xE5, 0xA0, 0xB0, 0x99, 0x1E, 0x24, 0xF9, 0x06, 0xFF, 0x2A, 0x88, 0x74, 0x58, 0x86, 0x39, 0x77, 0xA1, 0xD5, 0x83, 0x77, 0x4E, 0x6B, 0x8B, 0xC7, 0x59, 0xC5, 0xF8, 0x50, 0x93, 0xC6, 0x54, 0x5A, 0x82, 0x15, 0xFA, 0xD6, 0xFC, 0xCC, 0x5B, 0x2A, 0x78, 0x7C, 0x5D, 0x7D, 0xD5, 0x31, 0xC6, 0x39, 0xBA, 0x8B, 0xA9, 0x0B, 0x00, 0xBE, 0xD7, 0xA3, 0xBA, 0x19, 0xB1, 0x0D, 0xE0, 0x42, 0x3A, 0x9C, 0x23, 0x6C, 0xE4, 0x32, 0x6D, 0x40, 0x68, 0xE6, 0x39, 0x1A, 0x65, 0x38, 0x3F, 0x14, 0x1F, 0xE3, 0x73, 0x3B, 0x7D, 0x95, 0xE1, 0xE3, 0x5B, 0x28, 0x92, 0x1A, 0x27, 0x24, 0xAC, 0x28, 0x33, 0x1A, 0x89, 0xBF, 0x2D, 0xBF, 0xBC, 0x0A, 0x81, 0x15, 0x6A, 0x86, 0x0C, 0xFE, 0x5E, 0x32, 0xE8, 0x33, 0xBC, 0x27, 0x78, 0x50, 0x8B, 0x48, 0x25, 0xAB, 0xD3, 0x74, 0xDF, 0xEE, 0xA1, 0x94, 0x1E, 0xEB, 0xDF, 0x90, 0xAA, 0xDC, 0x3B, 0x34, 0x8B, 0xF8, 0x6F, 0x94, 0x37, 0x2E, 0x59, 0x38, 0x59, 0x35, 0xA2, 0x07, 0x30, 0x91, 0xCB, 0xA4, 0x1B, 0xE4, 0xAC, 0xF9, 0x9D, 0x9A, 0x02, 0xE2, 0xDF, 0x69, 0x0C, 0x84, 0x7A, 0x7D, 0x79, 0x95, 0xB1, 0xEF, 0x4D, 0xF0, 0xFB, 0xC1, 0xBB, 0x67, 0x07, 0xE0, 0xE6, 0x94, 0xAD, 0xF4, 0x8F, 0xFB, 0x88, 0xD7, 0xB9, 0x9E, 0xBF, 0xF2, 0x5C, 0x50, 0x27, 0xB4, 0x98, 0xF0, 0x26, 0xD2, 0xAD, 0x3A, 0x03, 0x7E, 0xDE, 0x4D, 0xDC, 0xEA, 0x84, 0x5E, 0x8A, 0x71, 0x26, 0x70, 0x02, 0x21, 0x08, 0x4B, 0x27, 0x30, 0xC9, 0xCA, 0x9B, 0x1F, 0xF7, 0x9A, 0xA0, 0xAC, 0x90, 0x96, 0x82, 0xAC, 0x56, 0x5F, 0x79, 0x35, 0x99, 0xD1, 0x5E, 0x16, 0xC5, 0x6F, 0xC4, 0x25, 0xF2, 0x02, 0xA3, 0x92, 0xAB, 0xE3, 0xEA, 0x3C, 0x06, 0x1F, 0x52, 0x6F, 0x8E, 0xB8, 0xAB, 0xE0, 0x4E, 0x9C, 0xA0, 0x90, 0x2D, 0x6A, 0x5E, 0xA8, 0x80, 0x10, 0x78, 0xE7, 0xE7, 0x10, 0x07, 0x25, 0x6C, 0x4B, 0xCA, 0xCD, 0x94, 0xEE, 0xD4, 0xD1, 0xDB, 0xDB, 0x88, 0x9F, 0x7B, 0x84, 0x51, 0xF4, 0x20, 0xC1, 0xCC, 0xBA, 0x61, 0x60, 0xA7, 0x6C, 0x6F, 0xF0, 0xDF, 0x48, 0x54, 0xA1, 0x8C, 0x47, 0x02, 0x92, 0x1D, 0xF2, 0x55, 0x79, 0xC9, 0x94, 0x1E, 0xD5, 0x60, 0x9A, 0x53, 0x86, 0x5E, 0x64, 0x3A, 0xA0, 0x7C, 0xEB, 0xC7, 0x60, 0x6D, 0x86, 0x09, 0xF0, 0x6D, 0xCF, 0x6E, 0xCE, 0x02, 0xB3, 0xCD, 0x38, 0xDB, 0x45, 0xC6, 0x73, 0xBC, 0x85, 0x3D, 0x80, 0x34, 0x79, 0x59, 0xE5, 0x45, 0xDB, 0x1D, 0x81, 0x37, 0xD1, 0x32, 0x0A, 0x2C, 0x49, 0xF2, 0x0A, 0x3F, 0x54, 0xBD, 0x61, 0x47, 0x17, 0xB3, 0x03, 0xEE, 0x7D, 0x25, 0xFC, 0x6E, 0xAC, 0xE9, 0x71, 0x07, 0x97, 0x6B, 0x7E, 0x83, 0x09, 0x52, 0x2D, 0x22, 0xCF, 0x60, 0x7A, 0x7B, 0x92, 0xF4, 0x36, 0xDF, 0x0A, 0x93, 0xD2, 0x95, 0xDE, 0x84, 0x2C, 0x42, 0x6F, 0xA9, 0x9D, 0x33, 0x4D, 0xAA, 0x02, 0x11, 0xE0, 0x01, 0x40, 0x25, 0x31, 0x1B, 0xAB, 0xCC, 0xB7, 0xF3, 0x3C, 0x3B, 0x56, 0xDF, 0xB6, 0x28, 0xCA, 0xB9, 0x12, 0x14, 0x0F, 0x8D, 0x80, 0x66, 0x69, 0x5F, 0x1B, 0xF6, 0x09, 0x48, 0x33, 0xFE, 0xDC, 0x06, 0x05, 0xB0, 0x41, 0x1D, 0x24, 0x38, 0xAC, 0xC9, 0x0B, 0xE6, 0x54, 0x2C, 0x45, 0xE1, 0x15, 0x5C, 0x4D, 0x5E, 0x55, 0x94, 0x38, 0xEE, 0x5C, 0x87, 0x38, 0x76, 0xF9, 0x27, 0x0E, 0x17, 0x5F, 0x17, 0x17, 0xEE, 0x85, 0xE2, 0xA9, 0x5D, 0x91, 0x48, 0xB4, 0x14, 0x51, 0x5A, 0x01, 0xCE, 0x30, 0xBB, 0x2C, 0x0C, 0xC7, 0x7E, 0xFF, 0x67, 0x12, 0x06, 0x5E, 0x4F, 0x96, 0x99, 0x55, 0x76, 0x93, 0x40, 0xEC, 0x23, 0xFF, 0x99, 0xE2, 0xFA, 0xBA, 0xFB, 0x2F, 0xB8, 0x3C, 0x9C, 0xA7, 0xC2, 0x2B, 0xEF, 0xB3, 0xB4, 0xE1, 0x3C, 0x9E, 0xA6, 0x54, 0x89, 0xA6, 0x1B, 0xC7, 0x62, 0x98, 0xB4, 0xCF, 0xD3, 0x34, 0x84, 0x39, 0x9D, 0x06, 0x4F, 0x9A, 0x7D, 0x1D, 0xE8, 0x09, 0xFA, 0x1B, 0x80, 0x82, 0x1B, 0xC2, 0x68, 0x65, 0x51, 0xD0, 0x81, 0x5B, 0xEE, 0xD0, 0x57, 0x94, 0xEA, 0xD2, 0x29, 0x48, 0xFE, 0xCA, 0xEC, 0x3C, 0xC4, 0x85, 0x0C, 0x7B, 0x82, 0xE3, 0xA8, 0x23, 0x3A, 0xB7, 0x3B, 0xD6, 0xED, 0xD8, 0x7C, 0x2D, 0x9B, 0xC9, 0x22, 0x8B, 0x4D, 0x1A, 0x82, 0x64, 0x54, 0x03, 0x68, 0x3C, 0x03, 0x2F, 0x88, 0xD8, 0xE1, 0x91, 0x90, 0x01, 0xF0, 0x82, 0x01, 0xB0, 0x3F, 0x83, 0x96, 0xD0, 0x83, 0x3A, 0xDD, 0x9F, 0xC8, 0x5F, 0x5E, 0x48, 0xE8, 0xF9, 0xEC, 0xCB, 0xC1, 0xD5, 0x31, 0xA5, 0x47, 0x46, 0x1A, 0x23, 0x28, 0x75, 0x56, 0xF0, 0xB7, 0xAD, 0x34, 0xAB, 0x69, 0x63, 0x1C, 0x4E, 0x42, 0x12, 0xE1, 0x43, 0xFA, 0xD1, 0x3F, 0xC8, 0xA0, 0x6E, 0x82, 0xC8, 0x37, 0x8C, 0xCC, 0xBE, 0xEE, 0xE1, 0x73, 0x86, 0xF8, 0x13, 0x25, 0xAB, 0xA5, 0x25, 0x9A, 0x2C, 0x7C, 0xDB, 0x9F, 0xFB, 0x8A, 0xE5, 0x8C, 0xDA, 0x1E, 0xAA, 0xCE, 0x8F, 0x76, 0x40, 0x99, 0x8D, 0x7C, 0xCD, 0x62, 0x28, 0x83, 0x5C, 0x9F, 0x3F, 0x57, 0x74, 0x68, 0x19, 0x2D, 0x8D, 0x19, 0xB5, 0xEE, 0x97, 0xBB, 0xF3, 0x11, 0xAD, 0xB1, 0xC1, 0xFA, 0x36, 0x95, 0xF8, 0xE9, 0xD3, 0x7C, 0x04, 0xDF, 0x3C, 0x5C, 0x7E, 0x63, 0xE4, 0xD6, 0x28, 0x9C, 0x78, 0x30, 0x28, 0x7B, 0xBD, 0x47, 0x98, 0x15, 0xF1, 0xD2, 0x7A, 0xCC, 0xBA, 0x9F, 0xCE, 0x14, 0x48, 0x48, 0x01, 0xBB, 0xE2, 0x8E, 0x35, 0x6A, 0x94, 0x5E, 0xD6, 0x55, 0xBA, 0xDC, 0x85, 0xD8, 0xEE, 0xB5, 0x5D, 0xB6, 0xBF, 0xF8, 0x52, 0xA2, 0x1E, 0xE5, 0xB6, 0x40, 0xB5, 0xCC, 0x85, 0x01, 0xBE, 0x53, 0x92, 0x7E, 0x2C, 0x23, 0xDC, 0x24, 0x82, 0xF0, 0x36, 0x9C, 0x4A, 0x6B, 0x1F, 0xE9, 0xC9, 0x42, 0x27, 0x53, 0x95, 0x47, 0x09, 0x39, 0x44, 0xB5, 0x06, 0xDD, 0x72, 0x18, 0xC6, 0x81, 0xAE, 0x88, 0xB6, 0x61, 0xAB, 0xE4, 0x76, 0x5E, 0x2C, 0xA9, 0xFF, 0xC6, 0x1C, 0x4B, 0xD1, 0xFB, 0xF3, 0x3C, 0x6A, 0xF0, 0x01, 0x07, 0xB8, 0xE7, 0xCF, 0x8A, 0xA7, 0xDB, 0x57, 0x7A, 0x64, 0xE5, 0x4C, 0xC9, 0x26, 0x4E, 0x6B, 0xFC, 0xFE, 0xE3, 0xEF, 0x2B, 0x68, 0xE0, 0x40, 0x44, 0x2C, 0x1F, 0xF2, 0xEE, 0x91, 0xCD, 0x29, 0x94, 0xB3, 0x15, 0x41, 0x7E, 0xF0, 0xAC, 0x49, 0x58, 0x8A, 0x58, 0x88, 0x5B, 0x34, 0x62, 0xAB, 0x53, 0x36, 0xCF, 0xCA, 0x02, 0x9E, 0xBA, 0x43, 0x29, 0xE6, 0xE3, 0xAC, 0xCF, 0x9A, 0x74, 0x24, 0xA6, 0x99, 0x30, 0xE0, 0xC1, 0x17, 0xB8, 0xEC, 0xD5, 0x98, 0x9E, 0x1C, 0x2B, 0xBE, 0x73, 0x09, 0x7A, 0x01, 0x85, 0x12, 0x10, 0x08, 0x59, 0x1C, 0x9B, 0xA1, 0x66, 0xC4, 0x77, 0xF6, 0x8F, 0xFA, 0xC0, 0x94, 0x9C, 0xC1, 0xC1, 0x01, 0x61, 0x0D, 0xA4, 0x37, 0x48, 0x87, 0xDD, 0x00, 0x7E, 0x82, 0xA9, 0xB0, 0x5B, 0xB6, 0x68, 0x8D, 0xC2, 0xEF, 0x6F, 0xAD, 0xAB, 0x36, 0xFD, 0xDC, 0xA1, 0x25, 0x94, 0xC3, 0x6C, 0x5D, 0x3B, 0x64, 0x16, 0x0E, 0x0F, 0x6C, 0xFE, 0x15, 0x23, 0x92, 0x31, 0x34, 0x61, 0xF6, 0xCF, 0x86, 0xC3, 0x73, 0x77, 0x13, 0x0F, 0x6E, 0x9A, 0x88, 0x36, 0x8A, 0xFD, 0xAD, 0xF6, 0xDA, 0x75, 0x16, 0xDA, 0xF4, 0x83, 0x64, 0x63, 0x8A, 0x0B, 0x55, 0xDE, 0x04, 0x41, 0x52, 0xAD, 0xC4, 0x40, 0xEF, 0x8F, 0x05, 0xCD, 0x3B, 0xD8, 0x0E, 0x28, 0x4D, 0xED, 0xE5, 0x4B, 0x51, 0x89, 0xD6, 0x7F, 0x15, 0x04, 0x79, 0x61, 0x00, 0x77, 0x46, 0x0B, 0xD9, 0x71, 0x76, 0xCD, 0x22, 0xA9, 0xAD, 0xD9, 0x75, 0x66, 0xEA, 0xCB, 0x02, 0x3D, 0xC9, 0x6D, 0x99, 0x3A, 0xF5, 0x1F, 0x1D, 0xA9, 0xA4, 0x1A, 0x07, 0x53, 0xA0, 0x33, 0xC1, 0x7C, 0x6B, 0x44, 0xA2, 0x6C, 0x38, 0x37, 0xB4, 0x3D, 0x49, 0xEA, 0x89, 0xBC, 0xBF, 0x64, 0x2D, 0xD0, 0x49, 0x9C, 0xA9, 0x60, 0xF0, 0x9D, 0x72, 0x8D, 0x0F, 0x21, 0x09, 0x44, 0x16, 0x19, 0x30, 0xBF, 0x13, 0xCE, 0xDF, 0x24, 0x5E, 0xC1, 0xA2, 0xEB, 0x3C, 0xE9, 0x6A, 0xA2, 0x22, 0x1B, 0xF2, 0x91, 0xC0, 0x8B, 0x69, 0xBD, 0xBC, 0xAF, 0xDD, 0x01, 0x50, 0xF9, 0x1D, 0x22, 0xA2, 0x7E, 0x04, 0x82, 0x38, 0x50, 0xFB, 0x9E, 0x56, 0x21, 0xB6, 0x33, 0xF4, 0xC9, 0x4E, 0xF5, 0x32, 0x9D, 0x5A, 0xBF, 0x87, 0x4B, 0xBB, 0x74, 0x1C, 0x3D, 0x1A, 0x4D, 0xE7, 0x70, 0xF4, 0xB3, 0xC4, 0xD0, 0xAA, 0x54, 0xBD, 0xA4, 0x58, 0x2F, 0xF5, 0xE6, 0xD6, 0xC3, 0x42, 0xF7, 0x39, 0x58, 0xC9, 0x7C, 0xD3, 0xD8, 0x06, 0x1E, 0x7B, 0xE1, 0x63, 0x31, 0x1C, 0xBB, 0x9B, 0x14, 0x71, 0xB0, 0xE3, 0x66, 0x7D, 0xCB, 0x0E, 0xB4, 0x1A, 0x17, 0x09, 0xA0, 0xCC, 0xB7, 0xD3, 0x6B, 0x7C, 0x50, 0xEF, 0xC6, 0xA2, 0xEE, 0x67, 0x31, 0x8D, 0x3A, 0x35, 0x7B, 0x03, 0x70, 0x36, 0x2C, 0x43, 0xA2, 0x9D, 0x77, 0x33, 0xB0, 0x8A, 0x93, 0xFC, 0x5F, 0x0B, 0xBA, 0xA6, 0x50, 0x9A, 0xC7, 0xF0, 0xA1, 0x9C, 0x67, 0x8E, 0x2C, 0x97, 0x38, 0xD7, 0x89, 0x1E, 0x39, 0x74, 0x72, 0xFE, 0x6E, 0x31, 0x7D, 0x5B, 0x1E, 0x15, 0x2B, 0xBA, 0xCC, 0xCB, 0x3F, 0x3F, 0x2F, 0xDA, 0x8D, 0x67, 0x47, 0x96, 0xAA, 0xD1, 0x78, 0x3E, 0x27, 0x60, 0x47, 0xF5, 0x8B, 0xD8, 0x78, 0x32, 0x0A, 0x21, 0x5D, 0x39, 0xF7, 0x64, 0x88, 0xD0, 0x2E, 0xA5, 0x17, 0x52, 0x92, 0x84, 0x77, 0x1D, 0x3D, 0xDC, 0xC3, 0xDE, 0xE3, 0x6C, 0x72, 0x25, 0xAD, 0x7D, 0x99, 0x17, 0x49, 0x04, 0x97, 0xB7, 0x6C, 0x40, 0x66, 0xFD, 0xF8, 0x7B, 0xE3, 0xAC, 0xC1, 0xD7, 0xEE, 0x31, 0x23, 0xA8, 0xC0, 0x8F, 0xEA, 0xE6, 0x14, 0x3F, 0x0B, 0x80, 0xC6, 0x6A, 0xFF, 0x4B, 0xF6, 0x48, 0xD0, 0xD8, 0x3B, 0x8D, 0xA2, 0xE7, 0xB8, 0x6C, 0x12, 0xBA, 0xE4, 0x19, 0xBA, 0x81, 0xCF, 0xC8, 0xDC, 0xEE, 0x95, 0xD0, 0xBC, 0x41, 0x8F, 0xC7, 0x5A, 0x31, 0x5C, 0x39, 0x50, 0xA4, 0xA4, 0xE4, 0x62, 0xAD, 0x47, 0x8E, 0xF1, 0x8D, 0x77, 0x93, 0xE5, 0x1E, 0x5B, 0xA4, 0x2A, 0x5F, 0x48, 0x46, 0xCE, 0x3A, 0x61, 0xC0, 0xD3, 0x80, 0x0F, 0x5D, 0x1F, 0x54, 0x82, 0x2B, 0xB4, 0xC2, 0xE2, 0x87, 0xE6, 0x86, 0xD5, 0x5D, 0x81, 0x69, 0x01, 0x6A, 0x26, 0xFF, 0x7A, 0x59, 0xE5, 0xA4, 0x28, 0x60, 0x77, 0xC5, 0xA4, 0x94, 0x83, 0xE8, 0x66, 0xFE, 0xA3, 0x2A, 0x35, 0xC5, 0xF2, 0x8E, 0x60, 0xFE, 0x63, 0x50, 0xA6, 0x2A, 0xD0, 0x5F, 0xF3, 0x4E, 0xC8, 0xED, 0x01, 0x01, 0x0D, 0x90, 0xBB, 0x33, 0x0B, 0x21, 0x4A, 0x27, 0x09, 0x9C, 0xC1, 0x51, 0x9F, 0xC3, 0xB3, 0x4A, 0x6B, 0x5C, 0x95, 0x81, 0x43, 0xE7, 0x85, 0x42, 0x35, 0x85, 0x7A, 0x0C, 0x75, 0xCA, 0x77, 0x07, 0xE6, 0x4F, 0x77, 0x94, 0x6B, 0x7C, 0x59, 0x66, 0xE4, 0x55, 0x02, 0xE6, 0x4E, 0x6B, 0x0B, 0x5B, 0x14, 0x31, 0xF1, 0x0E, 0x25, 0x84, 0x16, 0x85, 0x94, 0xF6, 0x15, 0xD8, 0xB5, 0x73, 0x31, 0xA7, 0x88, 0x43, 0x70, 0x0F, 0x79, 0x6D, 0xA5, 0x22, 0x3D, 0x89, 0x17, 0xBA, 0x7C, 0xDC, 0xFF, 0x65, 0x00, 0x9C, 0x74, 0xCD, 0x31, 0x48, 0x08, 0xFC, 0x22, 0x97, 0xBE, 0xAA, 0x9C, 0x25, 0xA7, 0xD8, 0x53, 0x5A, 0xA3, 0xF0, 0xBA, 0x29, 0x55, 0xC1, 0x18, 0x6D, 0x91, 0x75, 0x0F, 0xB1, 0xB9, 0xD8, 0x7B, 0x79, 0x81, 0xB8, 0xEB, 0x8B, 0x93, 0x9F, 0xF8, 0xAA, 0x4C, 0xB4, 0xA0, 0xC2, 0x0B, 0x28, 0xF7, 0x22, 0x43, 0x19, 0xF7, 0x91, 0x6C, 0xB7, 0x07, 0x4D, 0x49, 0xDE, 0x51, 0x75, 0x0E, 0x94, 0xC7, 0xD3, 0xBA, 0xD7, 0x9B, 0xB4, 0xC6, 0x1A, 0x0D, 0xEA, 0xE7, 0x5E, 0xF2, 0x2F, 0x3B, 0xF7, 0x17, 0x9E, 0xF0, 0xE3, 0x99, 0xFB, 0x6D, 0xD8, 0x2E, 0x3B, 0xEE, 0x63, 0x8C, 0x45, 0x58, 0x7A, 0x44, 0x84, 0x71, 0xA5, 0x6F, 0x86, 0x56, 0x86, 0x3C, 0x96, 0x48, 0x77, 0x44, 0x74, 0x2C, 0x76, 0xC3, 0xB3, 0xCB, 0x9C, 0xEF, 0x0A, 0x9B, 0xF1, 0x15, 0xF8, 0x5F, 0x25, 0x44, 0xE4, 0xBD, 0x54, 0xC8, 0x89, 0x11, 0xA6, 0xC7, 0x64, 0x18, 0xE7, 0x10, 0x87, 0x6C, 0x7E, 0x4C, 0xE3, 0x56, 0xA7, 0xFC, 0x98, 0x84, 0x1C, 0xC5, 0x3F, 0x70, 0x3C, 0x11, 0xE3, 0x2F, 0x5E, 0xC6, 0x09, 0xF0, 0x64, 0x4A, 0xAA, 0xDF, 0x45, 0x94, 0x29, 0x5D, 0x7F, 0xE4, 0x41, 0xD0, 0x50, 0x38, 0xB0, 0x03, 0xF3, 0x80, 0xF4, 0xB6, 0x3F, 0x7C, 0x20, 0xFC, 0x53, 0x96, 0x09, 0x7B, 0x2F, 0x74, 0x80, 0x36, 0x7F, 0xA2, 0xDB, 0x18, 0x7D, 0xF3, 0x54, 0xF7, 0xAD, 0x46, 0x7E, 0xFC, 0xD3, 0x62, 0x79, 0x04, 0x77, 0xE1, 0x4F, 0xDA, 0x83, 0x5B, 0xCD, 0x4C, 0x53, 0xC0, 0xF8, 0x06, 0x67, 0x63, 0x33, 0xCD, 0xE0, 0xE2, 0x0F, 0x1B, 0x23, 0x24, 0x59, 0x08, 0xE5, 0xB8, 0x05, 0xF7, 0xD0, 0x17, 0x7D, 0x41, 0xC8, 0xA3, 0xEA, 0x00, 0x8E, 0xCB, 0x78, 0xC5, 0x7D, 0xB4, 0x68, 0x32, 0x10, 0x9D, 0x0F, 0x69, 0xED, 0x5D, 0xDC, 0x19, 0x2C, 0x4D, 0xF0, 0xA5, 0x06, 0xD6, 0xCF, 0x38, 0xD4, 0x40, 0xE1, 0xC1, 0xF9, 0x3A, 0x45, 0x6D, 0x31, 0x49, 0x0C, 0x9B, 0xB7, 0x67, 0x2D, 0x88, 0xD2, 0xBF, 0x1A, 0xDF, 0x12, 0xC8, 0x5E, 0x2D, 0x2D, 0x56, 0xDB, 0x25, 0x39, 0x55, 0x29, 0x50, 0x65, 0x9C, 0x84, 0xEA, 0x1B, 0x98, 0x5C, 0xBD, 0xEC, 0x1E, 0x24, 0x7F, 0xD1, 0x48, 0x19, 0x45, 0xF2, 0x91, 0x89, 0xDB, 0x6A, 0x2D, 0x25, 0x6B, 0xE4, 0x8E, 0xAB, 0x53, 0xB2, 0x40, 0xD5, 0x48, 0x21, 0xCA, 0x78, 0xDA, 0x5D, 0x6D, 0x5C, 0x3D, 0x59, 0xC0, 0x9C, 0x8B, 0x08, 0x58, 0x25, 0x7B, 0xEE, 0x7A, 0xF8, 0x3D, 0x54, 0xAD, 0xDF, 0x3F, 0x3D, 0x41, 0x25, 0xE5, 0x2A, 0xB4, 0xEE, 0xDD, 0x53, 0x1B, 0x66, 0x66, 0xC2, 0x7B, 0x1C, 0x61, 0xB0, 0xC9, 0x8B, 0x17, 0xE2, 0xFC, 0x6D, 0x02, 0x69, 0x68, 0x70, 0xDB, 0x45, 0x56, 0x4F, 0x38, 0x30, 0xB0, 0x16, 0x7F, 0xF3, 0x82, 0x2D, 0x42, 0x57, 0x50, 0x92, 0xB2, 0x2F, 0x74, 0x58, 0x23, 0xCA, 0xA7, 0x3E, 0x53, 0xF6, 0x6A, 0x5B, 0xD7, 0x09, 0xDB, 0x7B, 0xCE, 0x93, 0x0C, 0x53, 0x74, 0x83, 0x70, 0x6E, 0x9C, 0xE0, 0x26, 0xDB, 0x0F, 0x90, 0xB1, 0xC2, 0x42, 0xE4, 0x44, 0x92, 0x10, 0x6C, 0x35, 0xF9, 0x99, 0xC7, 0x68, 0x42, 0xFA, 0xEB, 0x98, 0x8C, 0x0D, 0xD9, 0x58, 0x53, 0x55, 0x20, 0x92, 0x2C, 0xDB, 0x17, 0xA7, 0x65, 0x7F, 0x52, 0x9D, 0xB9, 0x8F, 0x81, 0x7B, 0x3F, 0x28, 0xFB, 0xF2, 0x1E, 0x7A, 0x5C, 0x34, 0x3F, 0xC5, 0x30, 0x7F, 0x54, 0x13, 0x75, 0x58, 0x8A, 0x23, 0x41, 0x62, 0x47, 0xF0, 0xBA, 0x26, 0xAE, 0xA0, 0x48, 0x7F, 0x32, 0x93, 0xEC, 0x3D, 0x83, 0xCE, 0x26, 0x0C, 0xB2, 0x0D, 0x10, 0x3A, 0x54, 0x99, 0xF9, 0x73, 0xFF, 0x76, 0x93, 0x9A, 0x9F, 0xA8, 0xCC, 0x6F, 0x18, 0x26, 0x68, 0x5A, 0xB4, 0xF0, 0x08, 0x7C, 0xA4, 0x84, 0x51, 0xAF, 0x78, 0x77, 0x35, 0x75, 0x87, 0x03, 0x43, 0xE5, 0xB6, 0x03, 0x4A, 0xD8, 0xD8, 0x11, 0xAD, 0x8E, 0x5D, 0xF1, 0x79, 0x57, 0x3B, 0x52, 0x4A, 0x4F, 0xAC, 0xC3, 0x39, 0xC7, 0xE9, 0x6A, 0x54, 0x40, 0x58, 0x79, 0xBB, 0x79, 0xE8, 0x0B, 0xF3, 0xC8, 0x02, 0xCE, 0xCE, 0xD8, 0xE7, 0xD3, 0x20, 0x73, 0xAE, 0x32, 0x92, 0x8F, 0x56, 0xAC, 0xD6, 0x0A, 0x44, 0x7F, 0xFB, 0xE6, 0xFD, 0xCA, 0x3E, 0x7C, 0x4A, 0x6A, 0x88, 0x00, 0x48, 0x72, 0x86, 0x3A, 0x03, 0x4A, 0x85, 0x26, 0x21, 0x0D, 0x06, 0x7B, 0xDE, 0xF4, 0x97, 0xCB, 0xEE, 0x4E, 0xAD, 0x5E, 0x53, 0x5B, 0x2B, 0x75, 0xE1, 0x07, 0xE0, 0x82, 0x90, 0x56, 0x9E, 0x22, 0x1F, 0x88, 0x94, 0xE6, 0x48, 0x8B, 0x7B, 0x48, 0xF5, 0xE6, 0x6E, 0x04, 0x4F, 0xAA, 0xC3, 0x9A, 0xC2, 0xF6, 0x3F, 0xBD, 0x36, 0x70, 0x6F, 0xA4, 0x8B, 0x45, 0x75, 0x04, 0x89, 0x73, 0xFA, 0x62, 0x4B, 0x9B, 0x28, 0x00, 0x59, 0xCC, 0x0D, 0xF6, 0x06, 0x34, 0x6F, 0xED, 0x91, 0xC7, 0x0F, 0x7C, 0x65, 0x77, 0x25, 0xB7, 0xFC, 0xAA, 0x6C, 0xBA, 0x96, 0xEE, 0xD2, 0xEE, 0x41, 0x69, 0xEB, 0xEC, 0x7A, 0x88, 0x3A, 0x07, 0x27, 0xA1, 0x5E, 0x6A, 0x2A, 0xBF, 0x8C, 0x13, 0xCA, 0x1C, 0xBB, 0x59, 0xD6, 0xEF, 0x29, 0xD6, 0xEA, 0x90, 0xDF, 0x75, 0xCD, 0x1F, 0x88, 0xE8, 0x4B, 0x42, 0xAD, 0xD3, 0xB7, 0x8D, 0x5C, 0x14, 0xBA, 0x88, 0x17, 0x27, 0x23, 0x81, 0x18, 0x8B, 0x2A, 0x21, 0x55, 0x03, 0xF0, 0x37, 0xCE, 0x54, 0x87, 0xEC, 0x49, 0x87, 0xA2, 0x4C, 0x82, 0x20, 0x05, 0x8B, 0xD4, 0x5F, 0xA9, 0x2E, 0x97, 0x70, 0x6C, 0x09, 0xFE, 0xDC, 0x1C, 0xA9, 0x9B, 0x16, 0x08, 0x88, 0x5B, 0xFD, 0x9A, 0x05, 0xEE, 0xD7, 0x78, 0x34, 0x4B, 0x29, 0x1D, 0x81, 0xFE, 0x5A, 0xE2, 0x9F, 0xC9, 0x2E, 0xD6, 0xFF, 0x1F, 0x03, 0xFB, 0x3E, 0x63, 0x13, 0x84, 0x7E, 0x99, 0x82, 0x82, 0x68, 0x08, 0x11, 0xD9, 0x07, 0x45, 0x69, 0x0A, 0x1D, 0xF8, 0x48, 0x7D, 0xA1, 0x62, 0xAC, 0xE2, 0x09, 0xAA, 0x79, 0x20, 0x64, 0x2A, 0x85, 0x67, 0x34, 0x78, 0x3E, 0x4D, 0x4C, 0x5E, 0x90, 0x4B, 0xF7, 0x39, 0x6C, 0xE5, 0xDD, 0xA4, 0x8D, 0x63, 0x73, 0xF8, 0x6D, 0x7B, 0xBF, 0xD7, 0x47, 0x6D, 0xC2, 0xF1, 0x83, 0xC0, 0x32, 0x6E, 0x29, 0xB9, 0xD4, 0x10, 0xDD, 0xC5, 0xC9, 0xEE, 0x54, 0x43, 0x42, 0xCA, 0xE6, 0x59, 0xA7, 0xC4, 0x30, 0x86, 0xA6, 0x40, 0x71, 0x6F, 0x96, 0x3D, 0xE2, 0x0D, 0x05, 0x08, 0x78, 0x5C, 0x33, 0x2E, 0x9F, 0x2A, 0x1A, 0x3C, 0x6F, 0xAB, 0x78, 0xE7, 0x1F, 0xA1, 0xC0, 0x44, 0x07, 0x04, 0xCA, 0xD5, 0xA3, 0x16, 0x75, 0xD3, 0x9D, 0x42, 0x4F, 0x26, 0x5D, 0x91, 0x64, 0x53, 0xAC, 0xB7, 0x8C, 0xCF, 0xC7, 0x38, 0xA1, 0xA4, 0x1F, 0x76, 0x7F, 0x25, 0x0D, 0x5A, 0x5B, 0x12, 0x5B, 0xA9, 0xB5, 0x21, 0x31, 0x4E, 0x78, 0x77, 0x3E, 0xF9, 0x60, 0xBC, 0xBC, 0xF0, 0x70, 0x64, 0x03, 0xF7, 0x6C, 0x40, 0x56, 0x74, 0x65, 0x8D, 0xD9, 0xF2, 0xDD, 0xCA, 0xE5, 0x5C, 0xCB, 0x9C, 0x14, 0x8A, 0x3F, 0x29, 0xA1, 0x87, 0xB8, 0x18, 0x54, 0x83, 0x96, 0xA2, 0x2F, 0x98, 0x60, 0x5D, 0x5A, 0x96, 0x1F, 0x60, 0x46, 0x51, 0xFB, 0xFB, 0x6D, 0x04, 0xA7, 0x65, 0xE3, 0xF5, 0x6D, 0x54, 0x07, 0x33, 0x44, 0xC7, 0x90, 0x15, 0x47, 0x72, 0x14, 0x4B, 0xE3, 0x5F, 0x44, 0x73, 0x6D, 0x77, 0x56, 0x97, 0x02, 0x88, 0x24, 0x92, 0x25, 0xE5, 0xD5, 0x3E, 0x17, 0x29, 0x93, 0xD6, 0xC4, 0x11, 0xE1, 0xCD, 0x91, 0x5E, 0x9D, 0xA4, 0x84, 0x10, 0x48, 0x60, 0x60, 0xB8, 0x71, 0x8F, 0x8D, 0xC6, 0x43, 0x14, 0x5E, 0x35, 0xBD, 0x1C, 0x11, 0xBE, 0x46, 0x54, 0xD7, 0xC5, 0xBA, 0xE7, 0xDE, 0x28, 0xCB, 0x71, 0xD4, 0xD2, 0xBD, 0x7D, 0x3C, 0x34, 0xA0, 0x3D, 0x9A, 0x4E, 0x0C, 0x7A, 0x3B, 0x3D, 0x2B, 0x6C, 0x18, 0x8B, 0x04, 0x86, 0x64, 0xB5, 0x17, 0x8E, 0x33, 0xA7, 0xE9, 0x96, 0x6C, 0x8A, 0x52, 0xDC, 0x54, 0xFC, 0x25, 0xCF, 0x62, 0xED, 0x90, 0xD2, 0x14, 0x21, 0x69, 0x0A, 0x91, 0xA2, 0x99, 0xBF, 0xC4, 0x4A, 0x6F, 0x58, 0xAF, 0x27, 0x37, 0xBC, 0x2B, 0x50, 0x9F, 0x09, 0x0F, 0x02, 0x55, 0xB7, 0x8B, 0x3A, 0xD3, 0xC2, 0x49, 0x01, 0x15, 0x74, 0x71, 0xA4, 0x6E, 0xEF, 0xD0, 0x84, 0x13, 0xA4, 0xB5, 0x3C, 0x4E, 0xA6, 0xA3, 0xFA, 0xD7, 0xA7, 0x4D, 0xE0, 0x52, 0x77, 0xCF, 0x89, 0xCE, 0xB5, 0xF8, 0x98, 0x64, 0xC8, 0x5F, 0xA7, 0x23, 0x13, 0x77, 0xC5, 0x18, 0x06, 0x6E, 0xDC, 0x7E, 0x3B, 0xF2, 0xFD, 0xA8, 0x8C, 0x82, 0xF8, 0x0A, 0x2D, 0xE9, 0xFA, 0x57, 0xF6, 0x03, 0x52, 0x00, 0x8D, 0xC2, 0xFF, 0xF6, 0x60, 0x3A, 0xB5, 0xB7, 0x50, 0x9E, 0xA9, 0xBE, 0x8C, 0xB0, 0xE4, 0xB0, 0x16, 0x70, 0x88, 0x55, 0x0B, 0x1B, 0x5E, 0x35, 0xAB, 0x36, 0x54, 0xAD, 0x94, 0x96, 0xC1, 0xE1, 0xD2, 0x7E, 0xA1, 0xE5, 0x42, 0x71, 0xAE, 0x4D, 0x33, 0xDA, 0x82, 0x5E, 0x19, 0xDD, 0xF2, 0x17, 0xD5, 0xEA, 0xAE, 0x66, 0x6C, 0x9F, 0x62, 0x20, 0x74, 0xB1, 0x97, 0xEA, 0x76, 0x6E, 0x62, 0xCA, 0x96, 0xDE, 0x46, 0x9C, 0x27, 0x5D, 0xFF, 0xD9, 0x04, 0xA7, 0x41, 0x6E, 0x30, 0xB6, 0x9C, 0x28, 0x2E, 0x5D, 0x24, 0xC3, 0x51, 0xC8, 0xAB, 0x2E, 0xEB, 0xDE, 0x9F, 0xE3, 0x91, 0xE2, 0xD4, 0xDF, 0xE2, 0x76, 0xC1, 0x25, 0xAE, 0xFC, 0xD6, 0xFA, 0x3C, 0x50, 0xB4, 0xF7, 0xDC, 0x30, 0x5D, 0x33, 0x3C, 0xB6, 0xB2, 0x1B, 0xEC, 0x13, 0xDA, 0x11, 0xBB, 0x7D, 0xDA, 0xD4, 0x02, 0x1C, 0x77, 0x5B, 0x52, 0x0C, 0xA7, 0xA5, 0x2C, 0xA8, 0xC2, 0x73, 0xC0, 0x6E, 0x6A, 0xB6, 0x0F, 0xA5, 0x14, 0xCD, 0x54, 0x4B, 0x8C, 0x82, 0x0D, 0x0C, 0x47, 0xA3, 0xF4, 0x92, 0xC9, 0x6F, 0x66, 0xF3, 0x05, 0x2B, 0xA2, 0x23, 0x5A, 0x19, 0x19, 0xC7, 0x5C, 0x8A, 0x0A, 0xB5, 0x40, 0x4E, 0xEF, 0x2E, 0x06, 0xAA, 0x86, 0x32, 0x89, 0x0E, 0x36, 0x31, 0x7C, 0xA0, 0x64, 0xE6, 0x3C, 0x5A, 0x6D, 0x25, 0xFE, 0xFB, 0xE9, 0x02, 0x99, 0x3F, 0xDD, 0x86, 0x8D, 0xDF, 0x1A, 0x0F, 0x7F, 0xBB, 0xEC, 0xE2, 0x43, 0xEE, 0x32, 0x80, 0x45, 0x65, 0xBC, 0xBF, 0xCC, 0xBF, 0x9F, 0x2B, 0xB0, 0x39, 0xDD, 0x30, 0x18, 0x3F, 0x52, 0xCA, 0x56, 0x59, 0x11, 0x97, 0x1A, 0x75, 0x3B, 0xFE, 0xEC, 0xFE, 0x3A, 0xB6, 0x33, 0x12, 0x05, 0xFA, 0xCF, 0x53, 0x6A, 0x7E, 0x33, 0x69, 0xF4, 0xFC, 0x9C, 0xF3, 0xA6, 0xAC, 0xEF, 0xC7, 0x73, 0x09, 0x2C, 0x31, 0x89, 0x36, 0xE4, 0xAA, 0xD5, 0x77, 0x3C, 0xAA, 0x38, 0x2D, 0x20, 0xA2, 0x54, 0x8F, 0xB6, 0x5B, 0x60, 0x85, 0xF8, 0x6E, 0x67, 0xAF, 0x02, 0xAF, 0xC9, 0x05, 0x88, 0xF1, 0x46, 0x68, 0x00, 0xF8, 0xC8, 0xF0, 0xE2, 0x98, 0x86, 0x2B, 0x67, 0xF3, 0x15, 0x46, 0x61, 0x30, 0xC3, 0x98, 0x39, 0xD4, 0xE9, 0xA0, 0x58, 0x80, 0x8D, 0x2E, 0x58, 0xDC, 0xEE, 0x49, 0x18, 0x38, 0x87, 0x31, 0x8C, 0xC5, 0x2C, 0x16, 0x91, 0x1D, 0xF9, 0x84, 0xCB, 0x98, 0x1E, 0x9E, 0xF1, 0xFB, 0x7C, 0xA6, 0x05, 0x4A, 0xCF, 0x68, 0x98, 0xE5, 0x13, 0x5A, 0x4E, 0xB2, 0x70, 0xF1, 0xE0, 0xB6, 0xC9, 0x33, 0x64, 0x30, 0xAB, 0x50, 0x3F, 0x73, 0xCB, 0xF4, 0x5D, 0x65, 0x8E, 0x6D, 0x70, 0x41, 0x3F, 0xBD, 0x61, 0x0C, 0x4C, 0x45, 0x99, 0xE0, 0x3B, 0xF9, 0x96, 0xD3, 0xF8, 0xBA, 0xED, 0x1B, 0x56, 0x48, 0xD0, 0x77, 0xD2, 0xB1, 0x1A, 0x6C, 0xCB, 0x44, 0x39, 0x75, 0x41, 0x40, 0xA1, 0xAF, 0x53, 0x4D, 0x8C, 0xFE, 0xF5, 0x3B, 0x2B, 0x13, 0x50, 0x16, 0x8C, 0x76, 0x69, 0xEA, 0xDD, 0x6A, 0x8D, 0xA8, 0x88, 0xED, 0xC5, 0x61, 0x73, 0x03, 0x20, 0x95, 0x29, 0xF4, 0x0B, 0x4F, 0x0A, 0x47, 0x90, 0x31, 0xE4, 0x0D, 0x63, 0x2D, 0x62, 0x48, 0xB1, 0x85, 0xAE, 0xAE, 0x58, 0xE1, 0x68, 0xD6, 0xF4, 0x35, 0x2A, 0xB0, 0xF7, 0xED, 0xA8, 0x70, 0x60, 0x1A, 0x9D, 0xE9, 0x5F, 0x5C, 0xD3, 0x2E, 0x4A, 0x26, 0x07, 0xAA, 0x42, 0x5D, 0xA5, 0x4D, 0x09, 0x32, 0x39, 0x32, 0xB5, 0x89, 0xCD, 0x38, 0xC8, 0x01, 0x4D, 0xB4, 0xCB, 0xC3, 0xA4, 0xCE, 0x56, 0x76, 0xF2, 0xA3, 0xD0, 0xF3, 0x9A, 0xF3, 0x05, 0x83, 0x15, 0x19, 0x54, 0xEA, 0x57, 0x8B, 0xF2, 0x56, 0xCD, 0xC1, 0x7F, 0x29, 0xDF, 0x13, 0xC2, 0x24, 0xC1, 0xBE, 0xF2, 0x5C, 0xA1, 0xC4, 0xF5, 0x71, 0x64, 0xDC, 0x61, 0x21, 0x19, 0x8B, 0x6A, 0x06, 0x81, 0xB9, 0x7B, 0xA0, 0x69, 0xA6, 0x67, 0x29, 0x8F, 0x17, 0xA9, 0x1B, 0xF3, 0x82, 0xA3, 0x92, 0x6C, 0x71, 0x76, 0xCB, 0x02, 0x73, 0x4F, 0x79, 0xC3, 0x5E, 0x2C, 0xDE, 0x09, 0xB8, 0xB8, 0xA2, 0x57, 0xE6, 0xFB, 0x53, 0x3D, 0xA9, 0xFE, 0xC4, 0x49, 0x44, 0x83, 0x70, 0xC3, 0xE4, 0xCD, 0x9C, 0xE3, 0xD5, 0xDD, 0xF2, 0x5A, 0xA5, 0x41, 0x1F, 0x79, 0xB9, 0x99, 0xF4, 0x12, 0x7A, 0xC5, 0xA9, 0x09, 0x91, 0xAC, 0x72, 0x1D, 0x5F, 0x12, 0x24, 0x0C, 0xBC, 0xD9, 0x99, 0x8D, 0x01, 0xB4, 0x1F, 0x85, 0x54, 0x7A, 0x8F, 0x55, 0x9B, 0xA3, 0xCF, 0x3A, 0x52, 0x7A, 0x4E, 0xB8, 0x6D, 0x02, 0x33, 0x34, 0xC6, 0xC6, 0xE5, 0x14, 0x4A, 0xEF, 0xBA, 0xE4, 0x66, 0xC1, 0xF4, 0xD4, 0x6A, 0x09, 0x4D, 0xB2, 0x70, 0x3C, 0x6E, 0x5F, 0x6B, 0x42, 0x1F, 0xAB, 0xAA, 0x4D, 0x9C, 0x0F, 0x28, 0xB5, 0x5F, 0x35, 0x7D, 0x2E, 0x7D, 0x24, 0x45, 0x8E, 0xE1, 0x89, 0xD5, 0xA8, 0x53, 0x84, 0xF3, 0x7D, 0x10, 0x7B, 0x59, 0x78, 0x01, 0x92, 0xA5, 0xFE, 0xEB, 0xB1, 0x51, 0xDD, 0xE1, 0x69, 0xC1, 0x61, 0xAA, 0xC0, 0xC3, 0x4F, 0x53, 0xD7, 0x3B, 0xBC, 0xE6, 0x96, 0x9E, 0xEC, 0x9E, 0x10, 0x66, 0xB7, 0xFB, 0x8E, 0xA5, 0x3C, 0xF4, 0xEE, 0xBD, 0x81, 0xC0, 0xD4, 0x30, 0x74, 0xD0, 0x33, 0xC9, 0x44, 0x1B, 0x4D, 0xAC, 0x78, 0xA5, 0xB9, 0x1C, 0x3E, 0x1E, 0xB1, 0x80, 0xF9, 0x30, 0xB7, 0x30, 0x61, 0xED, 0xE4, 0xAD, 0x6F, 0xA6, 0xE9, 0xFF, 0x95, 0x99, 0xC2, 0x0D, 0x45, 0x1C, 0xAF, 0xF4, 0x00, 0xFE, 0xB8, 0x97, 0x23, 0xB2, 0xDD, 0x1D, 0x1F, 0x17, 0x89, 0x32, 0x2B, 0x11, 0xD0, 0x55, 0x51, 0xCC, 0xB2, 0xAB, 0xA6, 0xF9, 0x28, 0x4B, 0x62, 0x96, 0x0E, 0xC5, 0xB2, 0x21, 0xC1, 0xEF, 0x8A, 0xDF, 0x42, 0x36, 0x06, 0x0C, 0xAD, 0xC1, 0xA1, 0xF4, 0xBE, 0xCE, 0xAE, 0x45, 0xFB, 0x47, 0x22, 0xD9, 0xBC, 0x2B, 0x6F, 0xB6, 0xFF, 0x03, 0x38, 0x1B, 0x62, 0x77, 0x88, 0x82, 0x43, 0x40, 0x14, 0x9B, 0xEF, 0x39, 0x52, 0x26, 0xFD, 0xCC, 0x04, 0x34, 0x52, 0xB0, 0x93, 0x5A, 0xB2, 0x96, 0xFE, 0x57, 0x48, 0x24, 0xE8, 0xF4, 0x5A, 0xFD, 0x05, 0x23, 0x49, 0x2B, 0x92, 0x9B, 0x31, 0x25, 0x7B, 0xF0, 0x23, 0xFE, 0xC0, 0x7B, 0x91, 0x29, 0x6E, 0x0C, 0x83, 0x92, 0x1F, 0x9C, 0x5F, 0xA9, 0x51, 0x73, 0x57, 0x0C, 0x0A, 0xDB, 0x0B, 0x5F, 0x0B, 0xD2, 0xAE, 0xC6, 0x93, 0xA3, 0x7F, 0xF3, 0xA9, 0x10, 0xA0, 0xA2, 0x41, 0x72, 0xF8, 0x76, 0xA4, 0x89, 0x7A, 0x7C, 0x5A, 0xB7, 0x8D, 0x7A, 0x47, 0x5F, 0xF0, 0x8E, 0x07, 0x8F, 0x7F, 0xEC, 0xCF, 0xD7, 0xDC, 0x1E, 0xE6, 0x0F, 0x48, 0xD5, 0xED, 0x43, 0x71, 0xCD, 0x53, 0xD0, 0x49, 0xB8, 0xA7, 0xDC, 0xEB, 0xFD, 0x06, 0x4B, 0x5A, 0x2C, 0x25, 0xE8, 0x2F, 0x31, 0x3C, 0x16, 0x72, 0x11, 0x45, 0x58, 0x02, 0x83, 0x1A, 0x47, 0x49, 0x39, 0x9F, 0x1B, 0xCC, 0x1F, 0xE8, 0xC0, 0xB1, 0x03, 0x4C, 0x8B, 0x02, 0xBB, 0x05, 0xA2, 0x56, 0x8F, 0x68, 0x33, 0x78, 0x6E, 0x69, 0x94, 0x9C, 0xFC, 0x64, 0x47, 0x03, 0x94, 0x32, 0xC5, 0x84, 0x4F, 0x40, 0xC7, 0xFB, 0x85, 0xE2, 0xB2, 0x3F, 0xB7, 0x69, 0xEB, 0xBA, 0xC4, 0xA8, 0x87, 0x1B, 0x48, 0xEB, 0x39, 0x30, 0xE5, 0x9D, 0xEA, 0xC8, 0x71, 0x2B, 0x45, 0x90, 0x01, 0x82, 0xB4, 0x68, 0xF5, 0x5F, 0x7B, 0x31, 0xB0, 0x60, 0xDA, 0x0C, 0x02, 0x76, 0xA0, 0xD2, 0x91, 0x1C, 0x55, 0x3A, 0xFF, 0xF0, 0xE8, 0x19, 0xA1, 0x91, 0x89, 0x37, 0x54, 0x51, 0x02, 0xFB, 0xD4, 0xE2, 0xBC, 0xF4, 0x9E, 0x7C, 0x41, 0x7B, 0x39, 0xAE, 0xEC, 0x26, 0x65, 0x70, 0xF5, 0xF0, 0x4B, 0x25, 0xB6, 0x47, 0x0B, 0x8A, 0x1C, 0x02, 0x43, 0xDE, 0xEA, 0x69, 0xF6, 0x13, 0xA9, 0x9D, 0xC2, 0xDD, 0xE2, 0xDE, 0x94, 0xF4, 0x9F, 0x58, 0x17, 0x3D, 0x75, 0xAF, 0x0E, 0xB4, 0x95, 0x9B, 0x87, 0xFB, 0x9F, 0xE5, 0xE8, 0x2C, 0x10, 0x0A, 0x87, 0xB7, 0xAF, 0xB5, 0xA9, 0x7A, 0x1A, 0xBA, 0xDC, 0x72, 0xA0, 0x94, 0x82, 0x77, 0xF7, 0x0C, 0x94, 0x45, 0x74, 0x58, 0x06, 0x98, 0x02, 0x0A, 0x58, 0xA0, 0xDC, 0x5C, 0x37, 0xA1, 0x6E, 0x76, 0x02, 0x18, 0x4D, 0x4B, 0xA0, 0x7C, 0x2C, 0xC8, 0x4E, 0x2B, 0x0C, 0x67, 0x1A, 0x18, 0x43, 0xB5, 0x36, 0xB4, 0x9A, 0x71, 0xFA, 0x5A, 0x27, 0x37, 0x06, 0x61, 0x7A, 0xF2, 0x8D, 0xE9, 0x46, 0x5F, 0xFE, 0xF0, 0x00, 0xD5, 0xA3, 0x2C, 0x97, 0x2E, 0x1F, 0xE4, 0xA4, 0x09, 0x20, 0x96, 0x24, 0x9E, 0xD2, 0x4B, 0xB0, 0xE6, 0x78, 0x3A, 0x63, 0x0D, 0xDD, 0x8F, 0x69, 0x50, 0xDA, 0xB3, 0xE7, 0x59, 0xAF, 0x51, 0x38, 0x20, 0xDA, 0x06, 0x48, 0xB8, 0xE8, 0x17, 0x2D, 0x1D, 0x70, 0x14, 0x66, 0xCE, 0x5C, 0x05, 0xE7, 0x84, 0x81, 0xBF, 0x60, 0x4E, 0xD3, 0x89, 0xC2, 0xF2, 0x41, 0x67, 0xE5, 0x9D, 0x75, 0x8A, 0x48, 0x21, 0x38, 0xB5, 0x1B, 0x13, 0xB4, 0x22, 0x8F, 0x01, 0x46, 0x7C, 0xA7, 0x12, 0xED, 0xA8, 0x1A, 0x31, 0x19, 0x57, 0x27, 0x05, 0x32, 0xF0, 0xF8, 0x92, 0xB3, 0x97, 0x60, 0x5E, 0x06, 0xFC, 0x5A, 0xE1, 0x1D, 0x87, 0x14, 0x3E, 0x3E, 0x6F, 0x47, 0x03, 0xE9, 0x39, 0x5C, 0x4F, 0x6A, 0xD5, 0x12, 0x9B, 0x69, 0x55, 0xA7, 0x6F, 0x07, 0x84, 0x4C, 0x33, 0xA1, 0x97, 0xE7, 0x86, 0xB5, 0x28, 0xAA, 0xFB, 0x9D, 0xA9, 0xB5, 0xB0, 0x15, 0x66, 0x27, 0xCE, 0x9F, 0x8E, 0xD9, 0x27, 0xD2, 0x38, 0xC7, 0xDA, 0x6B, 0xFD, 0xB3, 0x21, 0x1C, 0x11, 0x3E, 0xFE, 0xD4, 0x09, 0x5F, 0xBD, 0xE9, 0xDE, 0xEC, 0x65, 0x40, 0xE6, 0x0B, 0xE9, 0xF9, 0xBD, 0xDA, 0x55, 0x90, 0xBB, 0xE7, 0x21, 0x48, 0x19, 0x5F, 0xBC, 0xF5, 0x57, 0x5E, 0xA6, 0x1C, 0x98, 0xEB, 0x4D, 0xF9, 0x68, 0x7F, 0x08, 0x23, 0x53, 0x73, 0x6F, 0x1F, 0x85, 0x03, 0x85, 0xAA, 0xDE, 0xB0, 0xE6, 0x7A, 0x8E, 0xCB, 0x47, 0x8B, 0x16, 0xD4, 0xFF, 0x92, 0xBA, 0xBA, 0xA8, 0x2D, 0x19, 0x68, 0x88, 0x31, 0x44, 0xA9, 0xB2, 0x82, 0x9D, 0x1E, 0xEA, 0x38, 0xFA, 0xBB, 0xE1, 0x5C, 0x3E, 0x51, 0xD5, 0x32, 0x20, 0xEF, 0xAE, 0xCE, 0xAD, 0x29, 0x2C, 0xC7, 0xD0, 0xAB, 0x62, 0xA5, 0xC8, 0xDC, 0x54, 0x58, 0xBA, 0xBB, 0x52, 0x4F, 0x06, 0x4A, 0x5E, 0xF9, 0x18, 0x15, 0xEB, 0x2E, 0x8B, 0x5D, 0x74, 0x16, 0x4B, 0xE6, 0xC1, 0x36, 0x96, 0xFC, 0x4E, 0xAA, 0xBC, 0x5E, 0xE7, 0x25, 0x2E, 0x94, 0x68, 0xC4, 0x01, 0x45, 0x4A, 0xC1, 0xCC, 0x85, 0xC9, 0xF3, 0xBB, 0xB2, 0xA2, 0xDC, 0x95, 0xD7, 0x79, 0x0D, 0x1C, 0xA6, 0xA7, 0xB4, 0x0F, 0x63, 0x52, 0x18, 0x33, 0x5E, 0xC6, 0x78, 0xE7, 0x1A, 0x1D, 0x41, 0x1F, 0x5D, 0xD5, 0x34, 0xFC, 0xF7, 0x83, 0x6A, 0x74, 0x07, 0x12, 0x73, 0x2E, 0x1D, 0x66, 0x06, 0x54, 0xB9, 0xEC, 0xA6, 0x34, 0xF0, 0x35, 0xA6, 0xB9, 0x13, 0x23, 0xD4, 0xEE, 0x8F, 0xCF, 0x77, 0x93, 0x2C, 0x69, 0x7F, 0xDD, 0x57, 0x75, 0xEE, 0xB0, 0x6C, 0xBA, 0x03, 0x44, 0xFA, 0xF8, 0xF1, 0x1B, 0x14, 0xDA, 0xEE, 0xD8, 0x5D, 0x1F, 0xE0, 0xDE, 0xAE, 0x86, 0xE9, 0x8A, 0xB6, 0x21, 0x19, 0x9D, 0x58, 0xC0, 0x0B, 0x84, 0xF7, 0xF8, 0xBC, 0xBC, 0x71, 0x2B, 0xEC, 0x9B, 0xD1, 0xF7, 0x8F, 0xDD, 0x0A, 0x05, 0xE4, 0x28, 0x93, 0x5C, 0xAD, 0xFB, 0x9E, 0x17, 0x12, 0xCC, 0x44, 0xEC, 0x82, 0x0A, 0xED, 0x8E, 0xE0, 0x01, 0x90, 0x99, 0xE1, 0x6B, 0x97, 0x45, 0xA6, 0x5A, 0x2F, 0x66, 0x92, 0x6F, 0x94, 0x5F, 0x70, 0x2C, 0x2F, 0x87, 0xE9, 0x31, 0xDB, 0x55, 0xEC, 0xB1, 0x1D, 0x94, 0xC2, 0xF7, 0x2D, 0x5A, 0xD5, 0x81, 0x73, 0x70, 0x04, 0x7D, 0xF8, 0x71, 0xBF, 0x9E, 0x7E, 0x0B, 0x6C, 0x2E, 0x07, 0x52, 0xBD, 0xF7, 0x01, 0x3A, 0x99, 0xEF, 0x10, 0x42, 0x0E, 0x27, 0x69, 0x30, 0xD9, 0x74, 0xF5, 0x2B, 0x68, 0x4C, 0x4E, 0xC3, 0x15, 0x3F, 0x8C, 0x62, 0xD5, 0xF1, 0x1A, 0x1E, 0x68, 0xE1, 0x49, 0xDD, 0x28, 0x53, 0x66, 0xCF, 0x42, 0x12, 0x52, 0x7E, 0x29, 0x51, 0x4A, 0x4A, 0x31, 0x9D, 0x9A, 0x34, 0x1F, 0xE6, 0xDE, 0x02, 0x9A, 0x86, 0x18, 0xAF, 0x76, 0x02, 0x8D, 0x0B, 0xC3, 0xA6, 0x40, 0x79, 0xF7, 0x7B, 0x24, 0x6A, 0xC2, 0x07, 0x84, 0xBD, 0x76, 0x10, 0x93, 0x0B, 0x18, 0x75, 0x36, 0x65, 0x38, 0xE6, 0x8E, 0x85, 0x2E, 0x0D, 0x40, 0xC3, 0x5B, 0x23, 0x9F, 0xF4, 0x42, 0x2B, 0x87, 0x2C, 0x46, 0x12, 0x95, 0xC9, 0x35, 0x08, 0xA6, 0x5C, 0x8B, 0xD7, 0x54, 0xB8, 0x47, 0xF3, 0x54, 0xCE, 0x98, 0xE0, 0xDC, 0xE2, 0x5C, 0xA6, 0xBA, 0x00, 0xE9, 0x47, 0x32, 0x6D, 0x57, 0x45, 0x7C, 0xFC, 0x20, 0xC0, 0x6B, 0xE9, 0x02, 0x40, 0x13, 0xA9, 0xB1, 0x06, 0x84, 0xB7, 0xA2, 0x40, 0xC2, 0x23, 0xDD, 0x86, 0x21, 0xE2, 0xF0, 0x3B, 0xB8, 0x3E, 0xE5, 0xDE, 0x23, 0x5D, 0xF0, 0x5F, 0x27, 0x54, 0x7F, 0xA9, 0x43, 0xE0, 0xB9, 0x49, 0x60, 0x97, 0xC5, 0x02, 0x84, 0xF5, 0x9B, 0xF3, 0x6A, 0xF1, 0xD6, 0x69, 0x99, 0x4D, 0xD4, 0x37, 0x6B, 0xC3, 0xA8, 0xE0, 0xF8, 0x9C, 0xF3, 0x8F, 0x50, 0xC4, 0x92, 0x1A, 0x3A, 0xB6, 0xA0, 0x9C, 0x79, 0xD5, 0x4B, 0x6D, 0x4E, 0xC9, 0x3C, 0x9E, 0xAF, 0x09, 0x7C, 0x89, 0xF9, 0x64, 0xC5, 0xD4, 0xE5, 0xD3, 0x0C, 0xFD, 0x06, 0x7B, 0xC1, 0x2F, 0xD1, 0xE0, 0x51, 0xF6, 0x1D, 0xF9, 0xAF, 0xC1, 0x48, 0xEA, 0x1B, 0x3E, 0x8A, 0x57, 0x35, 0x53, 0x74, 0x85, 0x9E, 0x74, 0x7B, 0x85, 0x94, 0xE7, 0x3C, 0xBD, 0x03, 0x82, 0xF9, 0x38, 0xAB, 0xC2, 0xBC, 0x3F, 0x7A, 0x48, 0xAF, 0x45, 0xEF, 0x28, 0x3A, 0x05, 0x98, 0x10, 0x3F, 0xE8, 0x79, 0x3A, 0x0F, 0xB5, 0xD5, 0xE3, 0x64, 0x4B, 0xC9, 0x94, 0xF7, 0x40, 0x58, 0x0C, 0x95, 0xDA, 0x46, 0xE8, 0x9A, 0x05, 0x7B, 0xAE, 0x35, 0xF8, 0xBF, 0xBB, 0xB6, 0x79, 0x89, 0x86, 0x73, 0x0B, 0x9B, 0x41, 0x86, 0x3D, 0x16, 0x32, 0xE1, 0x92, 0xB0, 0x66, 0x84, 0x3F, 0x3A, 0x13, 0x2E, 0x86, 0xA8, 0x02, 0xDD, 0x91, 0xAF, 0xD6, 0xED, 0x5C, 0x64, 0x58, 0x4D, 0x35, 0xB6, 0x1A, 0x0A, 0x77, 0xCC, 0x57, 0x5D, 0xBF, 0x8E, 0x6D, 0xF7, 0xE6, 0xB2, 0xB2, 0x65, 0x65, 0x66, 0xEC, 0x01, 0x69, 0x24, 0x54, 0xF7, 0x6B, 0x67, 0x34, 0xAE, 0x9D, 0x31, 0x92, 0x78, 0x4B, 0x5A, 0x3C, 0x0F, 0xB0, 0x07, 0x7C, 0x47, 0x47, 0xA8, 0xDB, 0xE8, 0x02, 0x9E, 0x64, 0xBE, 0xE8, 0xA9, 0xC2, 0x4E, 0x54, 0x34, 0xA5, 0xE0, 0x01, 0x62, 0x5F, 0x7B, 0x95, 0x45, 0x60, 0xBA, 0x26, 0x65, 0x7E, 0xD3, 0xF1, 0xF2, 0x5E, 0xB4, 0x23, 0x3A, 0xFC, 0xFD, 0x9F, 0xF8, 0xC9, 0x50, 0x5D, 0x29, 0xA8, 0x06, 0x49, 0x3D, 0x62, 0x00, 0xCF, 0x89, 0x7C, 0x8E, 0x2C, 0x8E, 0xE1, 0xFC, 0x6F, 0xE7, 0x28, 0xAA, 0x82, 0xF3, 0x33, 0x5B, 0xAC, 0x2A, 0xA5, 0xC7, 0x78, 0xA8, 0x72, 0x2A, 0x00, 0x1F, 0x15, 0xE1, 0x33, 0x9C, 0x4E, 0xD0, 0x4C, 0x31, 0x6E, 0xB2, 0x5E, 0x4A, 0x4D, 0xDC, 0x98, 0xF3, 0xC5, 0xB3, 0xAE, 0xA7, 0xA6, 0x62, 0xA8, 0xC7, 0x45, 0x1F, 0x11, 0x87, 0xA8, 0xA1, 0xD8, 0xBB, 0x15, 0x98, 0x0B, 0x39, 0x2D, 0x34, 0x15, 0x62, 0xE3, 0x8F, 0xC7, 0x5C, 0xC6, 0xD1, 0x54, 0x4D, 0xBC, 0xD6, 0x6C, 0xCE, 0xEC, 0xC7, 0xAB, 0x35, 0x51, 0xA9, 0x25, 0x3A, 0xDE, 0xF2, 0x97, 0x1F, 0x88, 0x99, 0xB8, 0x51, 0xD1, 0x69, 0x58, 0xD9, 0x9D, 0x7A, 0xB2, 0xD7, 0x05, 0x3C, 0xB7, 0x9D, 0x5B, 0xE0, 0x5A, 0x74, 0xAA, 0x73, 0xE0, 0x16, 0x1B, 0xFF, 0x3F, 0x93, 0x5F, 0xA8, 0xD1, 0x91, 0x22, 0xAA, 0x1B, 0x30, 0x42, 0x21, 0x65, 0x96, 0x41, 0x68, 0xC8, 0x49, 0x53, 0x66, 0xF4, 0xEB, 0xC4, 0xAB, 0xAD, 0xDA, 0x75, 0x1F, 0x01, 0x6E, 0x85, 0xB4, 0xB5, 0x7C, 0x78, 0xE2, 0xFA, 0x74, 0xD2, 0xB5, 0x33, 0xFC, 0xBD, 0x46, 0xD3, 0x04, 0xD2, 0x57, 0x82, 0x1F, 0x30, 0xC0, 0x15, 0x2A, 0x89, 0x4A, 0xF2, 0x74, 0x1B, 0xAA, 0x87, 0xB5, 0xC2, 0xEF, 0x17, 0x35, 0xE8, 0x4C, 0xF5, 0xCF, 0x82, 0x20, 0xE2, 0xA0, 0x95, 0xAA, 0x39, 0x07, 0x43, 0x43, 0xD7, 0xAC, 0x98, 0x99, 0x04, 0x94, 0xDA, 0x7D, 0xC7, 0x85, 0x67, 0x50, 0xF5, 0x74, 0x71, 0x24, 0xC7, 0xA3, 0x87, 0xA2, 0x41, 0x0E, 0xE1, 0xBF, 0x39, 0x20, 0xBA, 0x66, 0xD5, 0x48, 0x88, 0xC0, 0xBC, 0x1D, 0x01, 0x37, 0x67, 0x97, 0x4D, 0x87, 0x1C, 0x5D, 0x98, 0xC4, 0xD3, 0x88, 0x8A, 0xB9, 0x4D, 0x2C, 0x78, 0x1B, 0x68, 0xF7, 0x74, 0xCA, 0xD3, 0xC5, 0x9E, 0xD1, 0x2F, 0xD9, 0xF9, 0x81, 0xE1, 0xD9, 0xC0, 0xC2, 0x1C, 0xC7, 0xB0, 0x2E, 0x8F, 0x36, 0x0D, 0x5C, 0x46, 0x9E, 0x0A, 0x83, 0xC7, 0x68, 0xF3, 0x4C, 0xA3, 0xD8, 0xAC, 0xDC, 0xFE, 0xFC, 0x50, 0x86, 0xE5, 0x04, 0xBA, 0x7F, 0x98, 0x0F, 0x1F, 0x53, 0xB8, 0x29, 0x63, 0xCC, 0x48, 0x75, 0x33, 0x7E, 0xB0, 0xE3, 0xAD, 0x5B, 0x1C, 0x94, 0x37, 0xA7, 0x0B, 0xB9, 0xCB, 0x26, 0x25, 0xB3, 0xAD, 0x1B, 0x3E, 0x01, 0xD8, 0x30, 0xEC, 0x78, 0x9C, 0x78, 0x37, 0x8B, 0x9D, 0xC7, 0xBE, 0xB8, 0x58, 0x37, 0x79, 0x7B, 0x14, 0xC4, 0x75, 0xC7, 0x11, 0xC2, 0x10, 0xD8, 0xA7, 0x35, 0x49, 0x1F, 0x28, 0x3A, 0xCC, 0xD6, 0x17, 0x3E, 0xE1, 0x60, 0x94, 0x5D, 0x06, 0x38, 0x05, 0x07, 0xE6, 0xED, 0x60, 0x5D, 0x40, 0x47, 0x32, 0x04, 0x95, 0x78, 0xC4, 0x95, 0x4E, 0x65, 0xC3, 0x3A, 0x65, 0x44, 0xB4, 0x28, 0xBF, 0x70, 0x22, 0x84, 0xF4, 0x06, 0xA3, 0x97, 0xF5, 0xFC, 0xA6, 0x6C, 0xB9, 0x10, 0xE9, 0xF2, 0x3C, 0x45, 0x97, 0xC0, 0xED, 0x31, 0xD3, 0xD7, 0xCE, 0xEC, 0x1C, 0xDF, 0x2C, 0x16, 0xB1, 0xDB, 0xA5, 0x75, 0x37, 0x29, 0xE9, 0x97, 0xA4, 0x31, 0x12, 0xF5, 0xA1, 0x95, 0x2E, 0x72, 0x14, 0x62, 0xE7, 0x58, 0x0E, 0xD9, 0x7E, 0xFA, 0x47, 0x1E, 0xFF, 0xB1, 0x3A, 0x49, 0x50, 0xB1, 0xC3, 0x52, 0x1E, 0x8C, 0x7A, 0x7A, 0x06, 0x0B, 0x6D, 0x43, 0xA9, 0x06, 0x92, 0xC8, 0x77, 0x9E, 0x1E, 0x32, 0xF1, 0x10, 0x05, 0xD6, 0x8E, 0x0C, 0x6B, 0x0F, 0x8C, 0x12, 0xE3, 0xA9, 0x97, 0xF8, 0x54, 0x70, 0x9A, 0x72, 0x88, 0x18, 0x23, 0xC8, 0xFB, 0xCD, 0x50, 0xAA, 0xB3, 0x32, 0xA5, 0x5C, 0xF8, 0x54, 0xCC, 0xE1, 0x9F, 0x38, 0x82, 0x1B, 0x0D, 0x16, 0x09, 0xAE, 0x24, 0x56, 0x17, 0xA5, 0x09, 0xE0, 0x7D, 0x66, 0xC9, 0x0F, 0xCB, 0x74, 0xC3, 0x9D, 0x7A, 0xE1, 0xAA, 0xB5, 0x4C, 0x31, 0x27, 0x6D, 0x0D, 0xD6, 0xD9, 0x7F, 0x80, 0xE3, 0x4E, 0xB9, 0x51, 0xDE, 0xB0, 0xBB, 0xE4, 0xC5, 0x37, 0x7F, 0x9E, 0xD3, 0x27, 0xBA, 0x5C, 0x77, 0x3B, 0x65, 0x1F, 0x89, 0xD0, 0x6C, 0x74, 0x72, 0x81, 0xEC, 0x38, 0xD4, 0xB1, 0xE9, 0xBB, 0xBC, 0x95, 0xEE, 0xCF, 0x0C, 0xE2, 0xE0, 0xBD, 0xCE, 0x28, 0x7E, 0x73, 0x05, 0x2B, 0x11, 0x76, 0x8A, 0x16, 0xBF, 0x6A, 0x3E, 0x02, 0x57, 0x26, 0x19, 0x8A, 0x80, 0x63, 0x58, 0x1D, 0xA1, 0x83, 0xAB, 0x5B, 0xB0, 0xD8, 0xF4, 0xBE, 0x04, 0xE7, 0x49, 0xD6, 0x22, 0xDF, 0xF3, 0xD5, 0x23, 0xBA, 0x7F, 0x2E, 0x3C, 0xB8, 0x64, 0xDB, 0x57, 0xA2, 0x48, 0x3C, 0xBC, 0xC4, 0xFA, 0x5C, 0xC4, 0xB5, 0xD8, 0xD0, 0x3C, 0xCB, 0xC9, 0x9A, 0xFC, 0x7A, 0x96, 0x11, 0x54, 0xA0, 0xCA, 0x7C, 0x30, 0x7A, 0xF4, 0xFD, 0xEB, 0x78, 0x28, 0xB5, 0x78, 0x64, 0xA1, 0x0E, 0x64, 0x41, 0x52, 0x3B, 0x4F, 0x5B, 0xCA, 0x60, 0x99, 0x2F, 0x45, 0xD5, 0x14, 0x04, 0xF3, 0x52, 0xEC, 0xCC, 0x52, 0x61, 0xD1, 0x10, 0xC2, 0xA8, 0x40, 0x17, 0xE9, 0xC3, 0x0C, 0x7D, 0x70, 0xA3, 0x7B, 0x65, 0xD6, 0xB7, 0xB0, 0x1B, 0xDF, 0x63, 0x3F, 0x1D, 0x37, 0xBE, 0x2B, 0xC7, 0x09, 0x54, 0xF7, 0xBA, 0x4F, 0xB0, 0x29, 0x08, 0x99, 0x1A, 0xDD, 0xD4, 0xAD, 0x37, 0x85, 0xDB, 0xA8, 0xF8, 0xFE, 0xFC, 0x6C, 0x08, 0x34, 0x90, 0x3C, 0xED, 0xC2, 0x36, 0xB0, 0x2C, 0x04, 0x9B, 0xC1, 0xB0, 0x6C, 0xEF, 0xA3, 0x58, 0x36, 0x12, 0x9D, 0x1A, 0xFA, 0x29, 0xFE, 0x63, 0xA7, 0x8D, 0xC1, 0xE5, 0x78, 0xD1, 0x46, 0x10, 0x71, 0xC9, 0x53, 0xBE, 0x89, 0x8B, 0x70, 0x64, 0xDC, 0xF5, 0x45, 0x1E, 0x88, 0x0A, 0xCC, 0x5C, 0x25, 0xE5, 0x95, 0x48, 0x7F, 0xB9, 0xC3, 0x5B, 0x9E, 0xB8, 0x81, 0x9F, 0xD9, 0x08, 0xF0, 0xBE, 0xC4, 0xB8, 0x8B, 0x59, 0xAA, 0xAB, 0x01, 0x9D, 0x17, 0x9C, 0x50, 0xC5, 0x30, 0x96, 0x46, 0xDC, 0x53, 0xBB, 0x47, 0xD1, 0xA5, 0xFF, 0x3C, 0x73, 0xD1, 0xF5, 0x38, 0x04, 0xBB, 0x7F, 0xD0, 0x17, 0x4A, 0xA7, 0x4F, 0xAE, 0x95, 0x46, 0x9B, 0xA8, 0x74, 0x2D, 0x4F, 0xF1, 0x49, 0xDF, 0x9D, 0xEB, 0xCF, 0x6C, 0xDC, 0x68, 0x59, 0x1A, 0x4D, 0x43, 0x53, 0x17, 0x37, 0x8D, 0x0B, 0x62, 0x93, 0x4F, 0x35, 0xC0, 0x3D, 0x2C, 0x37, 0x76, 0x87, 0x60, 0x7E, 0x97, 0xD1, 0x99, 0x4C, 0x56, 0x14, 0x88, 0xD2, 0xE9, 0xD4, 0x28, 0xE9, 0xF9, 0x10, 0xE4, 0x6E, 0x87, 0x85, 0x9B, 0xCD, 0x79, 0xEA, 0x6F, 0x3C, 0x0D, 0xAA, 0xF8, 0x9B, 0x2D, 0xBF, 0x67, 0x1C, 0xE5, 0x45, 0x01, 0x5B, 0xFE, 0x93, 0xFC, 0x07, 0x1E, 0x69, 0x99, 0x7F, 0xE0, 0x2A, 0x5F, 0xB4, 0x28, 0x4D, 0x83, 0xE1, 0xC8, 0x29, 0x77, 0xF1, 0x3E, 0x70, 0x94, 0xB0, 0xD9, 0x32, 0xBC, 0x65, 0x7A, 0xF4, 0x6B, 0x7A, 0xC0, 0x5F, 0xD2, 0xD2, 0x16, 0xA7, 0x70, 0xA2, 0x02, 0x99, 0x35, 0xE6, 0x25, 0x28, 0x13, 0xDF, 0x9C, 0x3D, 0x88, 0x25, 0x07, 0xAD, 0x25, 0x91, 0xC6, 0xAE, 0x25, 0xE9, 0xD0, 0x62, 0x55, 0xCF, 0x00, 0x77, 0x48, 0x8F, 0x3C, 0xD5, 0x6D, 0x25, 0x63, 0xFA, 0xC0, 0x9C, 0xC6, 0x4B, 0x4F, 0xB3, 0x05, 0x97, 0x01, 0x8F, 0x53, 0xD9, 0x61, 0x28, 0x17, 0xCD, 0xA5, 0x4B, 0xE2, 0xCA, 0x85, 0xC7, 0x0C, 0xBB, 0xB2, 0x6E, 0x29, 0xD4, 0x39, 0x4E, 0x37, 0x01, 0x2D, 0x5F, 0x2C, 0xB5, 0x6D, 0x85, 0x4B, 0x7F, 0xE1, 0xB5, 0xAB, 0xE2, 0xC6, 0x19, 0x12, 0x33, 0xD2, 0x01, 0x42, 0x1D, 0xB2, 0x9A, 0x52, 0x2B, 0x16, 0x93, 0x8D, 0x89, 0xD4, 0x8D, 0xF3, 0xEF, 0x26, 0xA7, 0xAB, 0x36, 0xF2, 0x99, 0x84, 0x06, 0xEB, 0xEB, 0xB9, 0x9C, 0xE8, 0x74, 0x02, 0xDA, 0x05, 0x7C, 0xF4, 0xC0, 0x78, 0xE1, 0x0F, 0xE1, 0xFC, 0x6D, 0x51, 0xA1, 0xBC, 0xCE, 0x7B, 0x67, 0x0A, 0xEB, 0xC2, 0x24, 0x2E, 0xD4, 0x31, 0x8C, 0x05, 0xEC, 0x26, 0xC8, 0xF7, 0x76, 0x93, 0xAA, 0xBE, 0xC0, 0x79, 0xDF, 0xDC, 0x5B, 0xC1, 0x33, 0x00, 0x31, 0x0B, 0x0D, 0xC8, 0x51, 0x11, 0xBE, 0x29, 0xBF, 0xD9, 0x7D, 0x6E, 0x50, 0xEC, 0x2D, 0xA0, 0x07, 0x1A, 0x89, 0xF6, 0xBC, 0x55, 0x18, 0x6E, 0x9D, 0xBF, 0x7E, 0x7D, 0xE0, 0xB6, 0x7D, 0x54, 0xA8, 0xF2, 0xE4, 0x38, 0xDD, 0xA9, 0xC6, 0xFF, 0xC1, 0x54, 0x18, 0xE4, 0x52, 0x99, 0x6A, 0x6E, 0x36, 0xF8, 0x72, 0x21, 0xCE, 0x36, 0x6C, 0xC3, 0xFA, 0x58, 0x8E, 0x62, 0xEF, 0xA5, 0xD3, 0x08, 0x27, 0x10, 0xC3, 0xEF, 0x54, 0x6F, 0x9A, 0xEB, 0x13, 0x36, 0xA9, 0x45, 0x0B, 0x4C, 0x30, 0x00, 0xD8, 0xA0, 0x51, 0x51, 0x68, 0xBA, 0xAA, 0x7C, 0x19, 0xAF, 0x2C, 0xAB, 0x8E, 0xE4, 0xA4, 0x7E, 0x9A, 0x63, 0x9D, 0x4F, 0x84, 0x5C, 0x61, 0x83, 0x2B, 0x5A, 0x55, 0x8B, 0x54, 0xA6, 0x17, 0xFB, 0xD1, 0x46, 0xA9, 0xB7, 0xAC, 0x3B, 0x67, 0x4E, 0x48, 0xDF, 0xD6, 0x4A, 0x7F, 0x15, 0xDA, 0xED, 0x14, 0xFC, 0x55, 0x77, 0x42, 0x1D, 0xA5, 0x83, 0xE4, 0x7C, 0xDF, 0xD5, 0xC6, 0xE9, 0x7E, 0xD4, 0xD7, 0x13, 0x5C, 0x2B, 0xCA, 0x08, 0xE9, 0xD6, 0xA6, 0x20, 0xDE, 0xF4, 0xD3, 0x83, 0x1D, 0x9C, 0x9B, 0x35, 0x64, 0x38, 0x3B, 0x89, 0xF9, 0xCC, 0xA8, 0xD0, 0x9F, 0x9D, 0xCA, 0x06, 0xA2, 0xD3, 0xD0, 0x37, 0xFC, 0xAE, 0xE9, 0x8F, 0x6A, 0x10, 0xBB, 0xBA, 0x05, 0x71, 0xF0, 0x7D, 0xD2, 0x63, 0x86, 0xDE, 0xF3, 0x07, 0x12, 0x36, 0xE8, 0x9F, 0xE5, 0x6A, 0x15, 0xDD, 0x76, 0x60, 0xE1, 0x63, 0xCF, 0x1A, 0xC0, 0x12, 0xD9, 0x94, 0xF4, 0x54, 0x56, 0x25, 0x09, 0x33, 0xB6, 0x03, 0xE5, 0x98, 0x87, 0x94, 0x7D, 0x7D, 0x68, 0x47, 0xE9, 0x3F, 0xFA, 0x5A, 0x74, 0xB7, 0xC8, 0x4D, 0x0A, 0x65, 0x9C, 0x5F, 0xE9, 0xA0, 0x74, 0x3A, 0x9A, 0x40, 0x40, 0xAF, 0xB1, 0x6F, 0x4A, 0x4A, 0x0C, 0x62, 0x79, 0x3F, 0x92, 0x82, 0xC5, 0x6D, 0xE0, 0x0E, 0x8D, 0xAF, 0x6F, 0x4A, 0xFA, 0x93, 0x1D, 0xDB, 0x30, 0x31, 0xA9, 0xED, 0xA2, 0xE6, 0x29, 0xCC, 0x33, 0xA4, 0xD1, 0x32, 0x78, 0x90, 0x8C, 0x0B, 0xAD, 0x5F, 0x92, 0x3B, 0x0D, 0x64, 0xC3, 0x05, 0x1B, 0x38, 0xAF, 0xDB, 0x1A, 0x2D, 0x92, 0xAF, 0xF2, 0x13, 0x65, 0xA9, 0x88, 0x53, 0x62, 0x9C, 0x78, 0x74, 0xA3, 0x97, 0xBA, 0x13, 0x54, 0x36, 0x95, 0x3B, 0x47, 0xB7, 0x8E, 0x0B, 0x71, 0x97, 0xF1, 0x72, 0x77, 0x72, 0x17, 0xC4, 0xE5, 0x18, 0x2A, 0xDC, 0xD0, 0x31, 0x63, 0xD5, 0xBF, 0x27, 0x61, 0x71, 0x8D, 0xC1, 0x05, 0x9E, 0xF6, 0xFD, 0xCF, 0xA3, 0x59, 0xFB, 0x82, 0x3E, 0xC7, 0xAF, 0xED, 0xE1, 0x14, 0xF5, 0xAA, 0x89, 0xA8, 0xDC, 0x86, 0xAA, 0x9B, 0x00, 0x1D, 0x04, 0x79, 0xFA, 0xC0, 0xD0, 0x82, 0xAA, 0xB5, 0xD6, 0x2F, 0x67, 0x4F, 0xDE, 0x62, 0x9A, 0xAF, 0xE7, 0x55, 0x72, 0x61, 0x60, 0xD7, 0x54, 0x4D, 0xF7, 0x6D, 0x35, 0x0E, 0x05, 0x26, 0x56, 0x39, 0xE2, 0xA7, 0xB4, 0xC7, 0xE4, 0xAE, 0x8D, 0x29, 0x6C, 0x7A, 0xD5, 0x8E, 0x7D, 0x60, 0x97, 0xB4, 0x56, 0xAB, 0x02, 0x53, 0xDE, 0xEB, 0xF9, 0xCF, 0x63, 0x7A, 0x8A, 0xCF, 0xB5, 0x06, 0x8C, 0x1D, 0xBD, 0x86, 0x1E, 0x44, 0x5F, 0x55, 0xC9, 0xD9, 0xF7, 0xF2, 0xA9, 0x5C, 0x0D, 0xC4, 0x57, 0x82, 0x01, 0x32, 0x81, 0xB8, 0x96, 0xDC, 0x14, 0xD8, 0x21, 0x65, 0x22, 0x84, 0x82, 0x1E, 0xD6, 0x52, 0xA8, 0x5F, 0x28, 0x64, 0x3A, 0x42, 0x51, 0x74, 0x2D, 0xD9, 0x6F, 0x8E, 0x62, 0xC1, 0x1B, 0x27, 0x7A, 0x35, 0xE7, 0xE5, 0xA1, 0xA7, 0xC9, 0x15, 0xC3, 0x1E, 0x5B, 0x65, 0xEF, 0x13, 0x11, 0x8B, 0x50, 0x20, 0xE8, 0x1A, 0x6C, 0x84, 0xA0, 0x68, 0x05, 0x7C, 0xC6, 0xB8, 0xB2, 0x2B, 0x8B, 0x0E, 0x51, 0xC6, 0xA7, 0x64, 0xD9, 0x91, 0xB1, 0x8F, 0x7A, 0xFE, 0x2E, 0x08, 0x51, 0xA6, 0xC9, 0xB1, 0x7D, 0xD9, 0xBA, 0x77, 0xAC, 0x75, 0x92, 0x47, 0x79, 0x78, 0x0B, 0xC0, 0x30, 0x0D, 0xE9, 0x2B, 0xCE, 0x08, 0x9D, 0xAC, 0x98, 0xF2, 0xE8, 0xD9, 0xB5, 0x97, 0x40, 0x54, 0x20, 0xD7, 0xFB, 0x8A, 0x3B, 0xF7, 0x71, 0xA9, 0xAE, 0xA7, 0x35, 0xA0, 0x2B, 0xE5, 0xD3, 0x2C, 0x1C, 0x0D, 0x11, 0xBF, 0x80, 0x51, 0xCB, 0xFE, 0x07, 0xE4, 0xD2, 0xE5, 0xFA, 0xF8, 0xB2, 0x99, 0xEF, 0x12, 0x1E, 0x5A, 0xAB, 0x83, 0x33, 0x55, 0x91, 0x9E, 0xB8, 0x0C, 0x23, 0x20, 0x22, 0xA2, 0xA2, 0x37, 0x79, 0x72, 0xC0, 0xF6, 0x2E, 0xFE, 0x61, 0x83, 0xF5, 0x04, 0x67, 0xB3, 0x68, 0xEA, 0x95, 0x93, 0xFF, 0x82, 0xA5, 0xC5, 0x48, 0x7B, 0xEF, 0x63, 0xD1, 0xE1, 0x15, 0x1E, 0x5D, 0xB8, 0xFC, 0x17, 0x6F, 0xBB, 0xD7, 0xB2, 0x9E, 0xA0, 0x26, 0x46, 0x1B, 0xD8, 0x8E, 0x1D, 0x62, 0x20, 0x9D, 0x75, 0xCA, 0xE2, 0x58, 0x1C, 0x46, 0xFE, 0xB8, 0xEB, 0x2B, 0x38, 0xE8, 0x84, 0x1C, 0xA3, 0xEA, 0x94, 0x1C, 0x4C, 0x23, 0x89, 0x69, 0x3B, 0xD7, 0x49, 0xAC, 0xE2, 0xC5, 0xF6, 0xEB, 0xFD, 0xDB, 0x3A, 0xF9, 0x5C, 0x6C, 0xBE, 0x0E, 0xDC, 0xC1, 0xC3, 0xBA, 0x44, 0xF9, 0xEA, 0xD9, 0x53, 0x89, 0xA7, 0xFA, 0x42, 0xFE, 0x02, 0x64, 0xFE, 0x5B, 0x24, 0xB5, 0x08, 0xCE, 0x04, 0x14, 0x12, 0x15, 0x9E, 0x3D, 0x5E, 0x4C, 0x16, 0xFD, 0x39, 0x8A, 0x76, 0xB2, 0xC5, 0xCC, 0xDD, 0xA5, 0x2D, 0x4A, 0xB0, 0xCE, 0xF9, 0x81, 0xCC, 0x4E, 0xF9, 0x11, 0x60, 0xA6, 0x7E, 0x4A, 0x72, 0xFD, 0x12, 0x1B, 0xAC, 0x1B, 0xCE, 0x23, 0x46, 0x42, 0xB8, 0x0F, 0xA2, 0x6E, 0xCC, 0x06, 0xE8, 0xA2, 0x6A, 0xC3, 0xBE, 0x30, 0x6C, 0x40, 0x2C, 0x5F, 0x74, 0x54, 0x22, 0xE9, 0x1D, 0xC1, 0x5A, 0x54, 0x6E, 0x9A, 0x0B, 0xC7, 0x53, 0x71, 0x43, 0x41, 0x56, 0xF5, 0xF2, 0xFC, 0xEF, 0x0B, 0x0A, 0x12, 0x02, 0x28, 0xBC, 0x17, 0x09, 0xB0, 0xFA, 0x23, 0x8C, 0xF3, 0x39, 0x12, 0xAC, 0xB0, 0x02, 0xAE, 0x6F, 0x23, 0x3A, 0xBE, 0x76, 0xE3, 0x1F, 0x25, 0xFD, 0xEB, 0x5C, 0x8E, 0xFB, 0xC9, 0xCF, 0x2A, 0x02, 0x68, 0x12, 0xC0, 0x43, 0xEA, 0x58, 0x94, 0xCF, 0xA3, 0x96, 0x74, 0xAE, 0xC1, 0xBA, 0xB1, 0x05, 0x4D, 0x74, 0x4B, 0x12, 0xE9, 0x1B, 0xF5, 0xB3, 0x85, 0xBE, 0xC3, 0x8D, 0xB1, 0x9C, 0xFA, 0xBB, 0x9B, 0x15, 0x5A, 0xCB, 0x3C, 0x1F, 0x8D, 0xD1, 0x8D, 0xD5, 0x99, 0x52, 0xB2, 0xCC, 0xBB, 0x68, 0xC8, 0x70, 0x09, 0xB3, 0x50, 0xAE, 0xD3, 0xF9, 0xBF, 0xDF, 0x58, 0xFF, 0x50, 0x8E, 0xBC, 0x02, 0xF6, 0xAF, 0x99, 0x09, 0xE2, 0xB7, 0x0E, 0xE6, 0xAE, 0xEA, 0x39, 0x31, 0x9D, 0xF4, 0x5B, 0x9C, 0xC4, 0x59, 0xC6, 0xBC, 0x3F, 0x7A, 0x48, 0xAF, 0x45, 0xEF, 0x28, 0x2D, 0x78, 0x96, 0x2B, 0x0B, 0x09, 0x50, 0x9F, 0xA5, 0x22, 0x00, 0xC5, 0x50, 0x6F, 0x71, 0x4A, 0x23, 0xE9, 0x97, 0x55, 0x0D, 0x0D, 0xE2, 0x69, 0x92, 0xDF, 0x20, 0x11, 0xCA, 0x8D, 0x63, 0x89, 0x7B, 0xF4, 0xD6, 0xB6, 0x3D, 0x63, 0xBE, 0xB4, 0x46, 0xD7, 0xD1, 0x6D, 0x71, 0x27, 0xCF, 0x91, 0x59, 0x77, 0x24, 0x1F, 0xC6, 0x3A, 0x1F, 0xD1, 0x4F, 0x73, 0x3C, 0x7C, 0x40, 0xEB, 0x24, 0x66, 0xE5, 0xBA, 0x5F, 0xA0, 0xF0, 0x32, 0xAB, 0xF9, 0x4A, 0x00, 0x10, 0xD1, 0x3B, 0xF7, 0x83, 0x54, 0x2C, 0x6A, 0x50, 0xB7, 0x4E, 0x53, 0xCF, 0x8A, 0xB3, 0x08, 0x44, 0x08, 0x0E, 0x39, 0xBE, 0xAD, 0x75, 0xB5, 0x92, 0x96, 0xBA, 0xA5, 0xAF, 0x88, 0xA1, 0x60, 0x75, 0x2A, 0x01, 0x0B, 0xEB, 0xDA, 0xA6, 0xCB, 0xC8, 0xE0, 0x0C, 0x92, 0xA0, 0x6E, 0xEA, 0x4C, 0x28, 0x25, 0x24, 0xCC, 0xE2, 0x6C, 0xCC, 0x92, 0xD6, 0x5D, 0x08, 0xCB, 0xC3, 0x98, 0xDB, 0x5A, 0xE4, 0xA7, 0x15, 0x87, 0xAA, 0xC2, 0xF5, 0xDD, 0xCF, 0xDA, 0xEF, 0x29, 0x6D, 0x0C, 0x0F, 0xEC, 0xD0, 0x65, 0x97, 0xED, 0x4A, 0x9B, 0x79, 0x47, 0x5A, 0x4A, 0x14, 0x25, 0x6B, 0xF1, 0xE6, 0x4B, 0xA8, 0xCF, 0xCD, 0xC1, 0x6F, 0xBD, 0x92, 0x70, 0xA9, 0x18, 0xB9, 0x35, 0xF6, 0x1F, 0xFB, 0x91, 0x1B, 0xD4, 0x08, 0x0F, 0xC2, 0x27, 0x59, 0xB3, 0xBC, 0x25, 0x5B, 0x29, 0x83, 0xA5, 0x7B, 0xB1, 0x5E, 0x87, 0x65, 0x3B, 0xAA, 0x2F, 0xCA, 0x8F, 0x87, 0xB3, 0xAE, 0x9D, 0x18, 0x54, 0xE9, 0xFA, 0x09, 0xDF, 0x95, 0xC9, 0x8B, 0xA6, 0xE7, 0xC3, 0xEA, 0xA7, 0xB7, 0xC4, 0x3A, 0xFF, 0x12, 0x11, 0x57, 0x8E, 0xEA, 0x75, 0x29, 0xFF, 0xBF, 0x22, 0x80, 0x4E, 0x82, 0x2F, 0xBE, 0xAA, 0x7D, 0x37, 0xF1, 0xEF, 0x51, 0x32, 0x26, 0x64, 0xFC, 0x4A, 0xBB, 0xB3, 0x28, 0x4C, 0xBE, 0x6F, 0xCB, 0xB4, 0x3B, 0x1E, 0x07, 0xFD, 0x98, 0x23, 0x92, 0x92, 0x45, 0xD9, 0xF5, 0x42, 0x2E, 0xCC, 0xE3, 0xE7, 0xF6, 0xBE, 0x4A, 0x11, 0xA6, 0x0C, 0xC1, 0x73, 0xF5, 0x0F, 0x0C, 0x64, 0x81, 0x4E, 0x9F, 0xD6, 0x8B, 0xAC, 0x34, 0x75, 0xFC, 0x04, 0xD6, 0xAC, 0x5D, 0x2D, 0xC3, 0xC0, 0x91, 0x04, 0xEE, 0xAD, 0x29, 0x2E, 0xFF, 0x16, 0xEB, 0x83, 0xA4, 0x0C, 0xB5, 0xEB, 0x66, 0x47, 0xED, 0x1A, 0x86, 0x2C, 0x66, 0xC6, 0xDF, 0x6A, 0x20, 0x91, 0x3E, 0x34, 0x6E, 0xDD, 0xB9, 0x0C, 0xD1, 0x23, 0x84, 0x55, 0x48, 0x3B, 0x5F, 0x04, 0xE7, 0x56, 0x3E, 0x49, 0xC2, 0x6A, 0xD0, 0x51, 0xDD, 0xAB, 0xE0, 0xF1, 0x1B, 0xBE, 0xA3, 0x8A, 0x09, 0x40, 0x91, 0x13, 0xFC, 0xA5, 0x81, 0x3F, 0x24, 0x3F, 0x2A, 0xB6, 0x20, 0xCD, 0x18, 0x35, 0x3C, 0xCA, 0x43, 0xD0, 0x76, 0x8F, 0x84, 0xBF, 0x79, 0xE4, 0xAD, 0x6D, 0x79, 0xBA, 0x63, 0xE5, 0xE8, 0xC2, 0xD1, 0xFB, 0x5C, 0x62, 0x5E, 0xAA, 0x8E, 0x35, 0xDF, 0x3C, 0x72, 0x11, 0x93, 0x86, 0x25, 0xA4, 0xEB, 0x91, 0x44, 0xB0, 0x19, 0x70, 0x3F, 0x78, 0x38, 0xBA, 0x45, 0x8C, 0x99, 0x57, 0xE0, 0xE0, 0xD3, 0x1B, 0x6F, 0x4B, 0xC6, 0xC5, 0xD4, 0x7D, 0x45, 0xCF, 0x19, 0xD0, 0x27, 0x9D, 0x2D, 0x10, 0x02, 0x31, 0xB3, 0xB4, 0x01, 0xAA, 0x19, 0x57, 0x61, 0x6D, 0xDB, 0x42, 0x5C, 0xAB, 0xE5, 0x49, 0xBE, 0xC1, 0x67, 0x7D, 0x99, 0xA4, 0xF3, 0xC1, 0x5E, 0x13, 0x36, 0x12, 0x02, 0xE5, 0x92, 0x94, 0xED, 0x6F, 0x49, 0x43, 0x89, 0xFA, 0x6E, 0x07, 0x32, 0x78, 0x57, 0x6E, 0x86, 0xDC, 0xAD, 0x26, 0x55, 0x33, 0xC9, 0x5F, 0xA0, 0x98, 0x43, 0x6C, 0xB3, 0x44, 0xFE, 0xA7, 0xE7, 0x26, 0xFB, 0xCC, 0x5E, 0x83, 0x3D, 0x69, 0xEC, 0x42, 0x98, 0xC6, 0xEA, 0x0A, 0x15, 0x07, 0x7F, 0x56, 0x5A, 0x84, 0x2D, 0x29, 0x87, 0x4A, 0xF6, 0x85, 0x10, 0x87, 0x39, 0x72, 0x70, 0x99, 0x95, 0x6F, 0x04, 0x25, 0xF2, 0x7A, 0x46, 0x1F, 0xE8, 0xA8, 0x0A, 0x4C, 0xC7, 0xBD, 0x0C, 0x29, 0x29, 0x4A, 0x55, 0x9F, 0x51, 0xDD, 0x2A, 0x5B, 0x77, 0x96, 0x4A, 0x9C, 0x02, 0x44, 0x15, 0x9F, 0xBE, 0x1C, 0x21, 0xD4, 0x06, 0x79, 0x1B, 0xB9, 0x4C, 0xBE, 0x0C, 0x1C, 0xA8, 0x10, 0xBE, 0x67, 0xD4, 0xEA, 0xC2, 0xF3, 0xE7, 0xA6, 0x59, 0xC5, 0xC6, 0xA2, 0xD1, 0x44, 0x9A, 0x07, 0xEE, 0xED, 0x2F, 0x59, 0x5F, 0xA4, 0x6F, 0x87, 0xA1, 0x75, 0xF3, 0xDD, 0xFD, 0x16, 0x9C, 0x52, 0x32, 0x8B, 0x0B, 0x4C, 0x76, 0x8B, 0x9C, 0xB7, 0x5E, 0x1F, 0x4F, 0xA4, 0x36, 0x1A, 0x52, 0xAC, 0x0E, 0xB6, 0x47, 0x3C, 0x2D, 0x83, 0xE3, 0x98, 0x5A, 0x3C, 0xCD, 0xA7, 0xA3, 0x75, 0x59, 0x84, 0xD9, 0x80, 0xAA, 0x9A, 0x63, 0x83, 0x3E, 0x13, 0xAA, 0x3A, 0x4B, 0x02, 0xEF, 0x58, 0xA1, 0x03, 0xE5, 0x76, 0xB7, 0x20, 0x67, 0xDE, 0xF1, 0x76, 0x88, 0xB5, 0xAA, 0x41, 0xB2, 0x88, 0xD1, 0xBD, 0xB7, 0x0C, 0xBB, 0x87, 0xD7, 0xD5, 0x44, 0x46, 0x12, 0x18, 0x0E, 0x21, 0xE6, 0x44, 0xF5, 0x4A, 0x6A, 0x38, 0x4E, 0x28, 0xF1, 0x8D, 0xAA, 0x49, 0x3E, 0x97, 0xB7, 0xA8, 0x49, 0x16, 0xC2, 0x5A, 0x3E, 0x3E, 0x7C, 0x4A, 0xA4, 0x8B, 0x48, 0xCF, 0x86, 0xAB, 0x63, 0x13, 0x2E, 0x35, 0xA7, 0x99, 0x13, 0x1E, 0x0F, 0xDB, 0xFB, 0x7B, 0x58, 0xA5, 0x06, 0x50, 0x82, 0x62, 0x42, 0x4A, 0xF2, 0x77, 0xDB, 0x78, 0x2C, 0x9F, 0x0A, 0x8C, 0x8C, 0xCD, 0xAD, 0xCC, 0x38, 0x99, 0xCA, 0x6D, 0xD2, 0x75, 0xEE, 0xC6, 0x10, 0x1B, 0xBC, 0xCE, 0x6B, 0x73, 0x63, 0x31, 0x45, 0x83, 0x6B, 0x94, 0x23, 0x35, 0xA3, 0x31, 0x21, 0x8C, 0x26, 0xCE, 0xA5, 0x14, 0x64, 0xFA, 0xAF, 0x4F, 0x77, 0x41, 0xA6, 0x3E, 0x4C, 0x84, 0x26, 0xCF, 0x3B, 0x04, 0xF8, 0x7A, 0xEC, 0xDB, 0xFE, 0x10, 0x2B, 0xD9, 0xCA, 0x3F, 0x5A, 0x07, 0x61, 0x2F, 0x5C, 0x32, 0x1F, 0xC7, 0xC8, 0xD3, 0x61, 0xC8, 0x66, 0xC9, 0xAB, 0x03, 0xDA, 0x82, 0x98, 0x33, 0x20, 0x97, 0x62, 0x38, 0x54, 0x7D, 0xF4, 0x70, 0x53, 0x7F, 0x85, 0xA0, 0xA0, 0xC7, 0x16, 0x45, 0x7E, 0x60, 0xE9, 0xA6, 0xA5, 0x2C, 0x76, 0x67, 0xEA, 0x0B, 0xF9, 0xA9, 0xFC, 0xCC, 0x96, 0x0D, 0xA3, 0xA3, 0xC2, 0x45, 0xD2, 0x82, 0x1D, 0xDC, 0x50, 0x4E, 0xC2, 0xFD, 0x4D, 0xEA, 0xD7, 0x3D, 0x00, 0x94, 0x88, 0x02, 0x17, 0x22, 0xD7, 0xFF, 0x81, 0x97, 0xD3, 0xA4, 0x61, 0xA7, 0x69, 0x4D, 0x52, 0x25, 0x0E, 0x69, 0xA4, 0x71, 0x8A, 0xA1, 0x3C, 0xB0, 0xC7, 0x05, 0x60, 0xBF, 0x18, 0xA0, 0xCD, 0x3A, 0x68, 0x4A, 0xE2, 0xF6, 0xD8, 0x97, 0xAE, 0x01, 0xAC, 0x3D, 0xA6, 0x09, 0xFF, 0xBD, 0x4E, 0x07, 0x50, 0xFA, 0x10, 0x53, 0x29, 0x76, 0x0B, 0x7A, 0x8E, 0x19, 0xCE, 0x38, 0x54, 0x62, 0xD5, 0x36, 0x1F, 0x01, 0xA5, 0x8C, 0xD2, 0x78, 0x17, 0xFE, 0x99, 0xCE, 0x71, 0x01, 0x4A, 0x53, 0x4F, 0xF2, 0xDC, 0xC0, 0x7B, 0x57, 0x36, 0x79, 0x9F, 0xCB, 0x8A, 0x25, 0xDF, 0x52, 0xF6, 0xD8, 0x63, 0xFF, 0x13, 0x8E, 0x78, 0xDA, 0x43, 0xCB, 0x3C, 0xFC, 0xA7, 0x33, 0xB1, 0xB4, 0x9B, 0x84, 0x83, 0xCD, 0x20, 0x96, 0x8F, 0x82, 0xB3, 0xA4, 0x11, 0x32, 0xF5, 0x9A, 0x40, 0xC6, 0x1D, 0xCA, 0x44, 0x60, 0xB2, 0xF1, 0x2B, 0x74, 0xFD, 0xDC, 0xB9, 0xA5, 0x85, 0xFB, 0x8E, 0x9D, 0xA9, 0xC5, 0x5A, 0xF8, 0x17, 0x77, 0x58, 0x20, 0x73, 0x39, 0x0A, 0xA8, 0x11, 0xA6, 0xC4, 0x76, 0x0D, 0xD6, 0x2F, 0x49, 0x85, 0x17, 0x1E, 0x3C, 0xD9, 0x3E, 0xE1, 0xBD, 0x70, 0x34, 0x2F, 0xE1, 0x62, 0x15, 0x98, 0xB5, 0xFE, 0xC1, 0x53, 0xED, 0xFA, 0x47, 0x1E, 0x1F, 0x05, 0xA5, 0x59, 0x41, 0xBB, 0xDD, 0x92, 0xD1, 0x8C, 0x7B, 0x16, 0xBA, 0x40, 0x56, 0x68, 0xE5, 0x5F, 0x1C, 0xCD, 0xA7, 0x3A, 0x99, 0x61, 0x4E, 0xC9, 0x17, 0xCE, 0x15, 0xD0, 0x22, 0xCF, 0x1E, 0xF5, 0xAC, 0x5D, 0x79, 0xFD, 0xB1, 0x20, 0x88, 0x50, 0xDA, 0xDF, 0x8E, 0xEE, 0x0D, 0x44, 0x47, 0xAA, 0xD5, 0x30, 0xC9, 0x6C, 0x7F, 0x17, 0xA4, 0xF4, 0x5E, 0x0D, 0xF6, 0xD2, 0xAD, 0x12, 0x22, 0x90, 0x82, 0x00, 0x70, 0xAF, 0xAF, 0x51, 0x21, 0xD3, 0xAC, 0x53, 0xB4, 0x0F, 0xA2, 0x43, 0x4B, 0x1B, 0xBD, 0x09, 0x24, 0x22, 0xFB, 0xBC, 0x3B, 0x71, 0x25, 0xED, 0xA2, 0x4D, 0xB3, 0xE8, 0xD3, 0x0E, 0xE0, 0xDB, 0x87, 0x92, 0xBC, 0xDF, 0xDD, 0xF8, 0x2B, 0x66, 0xC8, 0xF4, 0x64, 0xB8, 0x31, 0x1B, 0x59, 0x42, 0x0B, 0xC4, 0xBE, 0xEC, 0xDE, 0xC3, 0x81, 0x15, 0x2C, 0xBF, 0xAF, 0x08, 0x9C, 0x49, 0x1A, 0x37, 0x1A, 0x1F, 0x9A, 0x85, 0x7B, 0xF2, 0xF9, 0xE0, 0x66, 0x29, 0xAE, 0xD0, 0x2C, 0xDA, 0x62, 0xCA, 0x4B, 0xCF, 0x61, 0x2B, 0x70, 0xA9, 0x4E, 0x2B, 0x80, 0xEE, 0x87, 0xCA, 0xB7, 0x10, 0xE0, 0x4B, 0x18, 0x70, 0xCE, 0xF4, 0xEB, 0x44, 0xE2, 0x7B, 0xEA, 0xB0, 0x78, 0x58, 0x4B, 0xFD, 0xB1, 0x19, 0xA3, 0x1F, 0x38, 0x42, 0x4A, 0x62, 0x9F, 0xC6, 0xB2, 0x58, 0xA5, 0x43, 0xE4, 0xB0, 0x81, 0x22, 0x25, 0x8A, 0x46, 0x9C, 0x16, 0x6C, 0xA1, 0x47, 0x2C, 0x80, 0x07, 0xE9, 0x4A, 0x3E, 0xAB, 0x9D, 0xF5, 0x28, 0x7C, 0x1A, 0xBD, 0xF3, 0x77, 0x16, 0x10, 0x38, 0xC4, 0x90, 0x84, 0x14, 0x1A, 0xCA, 0xC3, 0x10, 0x65, 0x5B, 0xF4, 0x0A, 0x97, 0x1B, 0xBF, 0x44, 0xC5, 0x70, 0x22, 0xCB, 0x8B, 0xCE, 0xCE, 0x25, 0x48, 0x0E, 0xBF, 0x86, 0x1E, 0x3B, 0xBB, 0x19, 0x7F, 0xBA, 0xB3, 0xC2, 0xEA, 0x0F, 0xB0, 0xEB, 0x05, 0x07, 0xF0, 0x83, 0xC7, 0xF4, 0x00, 0x04, 0x47, 0xAA, 0x0C, 0xBA, 0xB1, 0x16, 0x20, 0x33, 0xB8, 0xFB, 0x03, 0xAD, 0x19, 0x26, 0xFE, 0xD8, 0x0D, 0xC0, 0x94, 0x46, 0x3F, 0xF0, 0x64, 0x60, 0x4D, 0x32, 0xBA, 0xAF, 0x62, 0x6D, 0xE3, 0xA8, 0x5D, 0x6B, 0xDD, 0x29, 0x63, 0x11, 0x59, 0x96, 0x49, 0x36, 0x32, 0x6E, 0x5D, 0xBA, 0x83, 0xB3, 0x3F, 0x2F, 0x5A, 0x9D, 0x0A, 0xE5, 0x85, 0xF9, 0x0D, 0x51, 0x76, 0x4D, 0xD3, 0x1E, 0x72, 0xA3, 0xF8, 0xC3, 0x87, 0x52, 0x3C, 0x7D, 0xF5, 0x48, 0x9B, 0xC6, 0x82, 0x38, 0x8F, 0xAD, 0xC8, 0x56, 0xD4, 0x92, 0x6F, 0xDD, 0xF0, 0xA6, 0xD1, 0xFC, 0x55, 0x71, 0x31, 0x67, 0xC3, 0xB0, 0xF0, 0xFE, 0x71, 0x3B, 0xF8, 0x8C, 0xC2, 0x6D, 0xBE, 0x7B, 0x27, 0xC6, 0x0D, 0x09, 0x5A, 0x15, 0x78, 0x70, 0xB6, 0x85, 0xD6, 0xCF, 0x05, 0x2B, 0xEE, 0x89, 0x07, 0xC0, 0x2A, 0xD1, 0x40, 0x87, 0x6C, 0xD5, 0x1F, 0xD4, 0x75, 0x43, 0x8E, 0x99, 0x06, 0xB3, 0x68, 0x42, 0x18, 0xB5, 0xE6, 0x9D, 0xF2, 0xD1, 0xE9, 0xBE, 0x50, 0x8B, 0xA9, 0xBA, 0xFE, 0xF7, 0x96, 0xE4, 0xF6, 0x60, 0xDA, 0x3D, 0x9B, 0x08, 0xCA, 0x6A, 0xFC, 0x3A, 0x73, 0x86, 0xA4, 0xC3, 0x40, 0x7A, 0xBA, 0xFC, 0x8B, 0x4C, 0x1B, 0xF4, 0xDB, 0x34, 0xE6, 0xE6, 0xEB, 0x62, 0x48, 0xE9, 0xE0, 0xBC, 0xAD, 0xA6, 0xC4, 0xDC, 0x8E, 0x35, 0x96, 0xE2, 0x96, 0x12, 0x82, 0xA5, 0x40, 0x88, 0x75, 0xD1, 0x7E, 0x35, 0xDB, 0xBE, 0x7B, 0x44, 0xCA, 0x94, 0x91, 0x28, 0x73, 0x7D, 0x0E, 0x2D, 0xB6, 0x1D, 0xFA, 0x47, 0xAF, 0xC4, 0xCB, 0x52, 0x69, 0x6C, 0x35, 0x71, 0x17, 0x61, 0x97, 0x7E, 0x79, 0x48, 0xF5, 0x76, 0x12, 0xD3, 0xED, 0x13, 0x98, 0x12, 0xBC, 0x7B, 0xB7, 0x37, 0x1A, 0x8C, 0xEE, 0x90, 0x70, 0x3E, 0x5F, 0x4D, 0x5A, 0xC6, 0xE7, 0x96, 0xA8, 0x0F, 0x2D, 0x79, 0xA6, 0x77, 0x26, 0x0B, 0x26, 0xB1, 0xC0, 0x40, 0x0C, 0xCD, 0x9E, 0x4A, 0x7B, 0x38, 0x2D, 0x49, 0x13, 0x3B, 0xA1, 0x6C, 0x44, 0x52, 0x87, 0xBA, 0x78, 0x45, 0x3D, 0xA8, 0x02, 0xFF, 0x78, 0x07, 0xA3, 0x76, 0xED, 0x67, 0xDE, 0x39, 0xA1, 0x6A, 0x29, 0xDA, 0xAB, 0x0E, 0xA5, 0xB8, 0x77, 0xA3, 0x80, 0x4E, 0x0B, 0x23, 0x2B, 0x24, 0x2B, 0x0B, 0x9A, 0xE2, 0xB3, 0x18, 0x3D, 0xF5, 0xAB, 0xB3, 0xD9, 0x60, 0x67, 0xD9, 0x8A, 0xF9, 0xB7, 0xD9, 0x07, 0xB1, 0x7C, 0xE3, 0x7D, 0x39, 0x95, 0x2A, 0x63, 0x9A, 0xFB, 0x0C, 0xE9, 0x24, 0x7E, 0x75, 0x22, 0x0E, 0x50, 0x9E, 0x46, 0xA8, 0x81, 0x31, 0xA0, 0x74, 0xA6, 0x1F, 0xA6, 0xA3, 0x44, 0x2D, 0x00, 0x5C, 0xA5, 0x75, 0x22, 0x1C, 0xE3, 0xD1, 0xD2, 0x78, 0x30, 0xCE, 0x9D, 0xD2, 0x91, 0x21, 0x83, 0x7A, 0xFD, 0xDE, 0xDE, 0x70, 0xC6, 0x07, 0x61, 0xE6, 0xC9, 0x96, 0x8E, 0x92, 0xB8, 0x71, 0x11, 0x2E, 0x11, 0x59, 0x1F, 0x06, 0x98, 0xF8, 0x68, 0xD1, 0x59, 0xD3, 0xEB, 0xF9, 0xA9, 0x63, 0x21, 0xFD, 0x01, 0xD0, 0x0C, 0xF8, 0xA2, 0xEA, 0x95, 0x86, 0xA4, 0x89, 0x34, 0x4F, 0x44, 0x17, 0x52, 0x5D, 0x6B, 0x73, 0xC7, 0x99, 0x58, 0x9B, 0x9D, 0x9F, 0x2F, 0x0D, 0x9F, 0x94, 0xFE, 0x52, 0x66, 0x9B, 0x41, 0xC7, 0x95, 0x62, 0xFC, 0xDB, 0xC9, 0xB2, 0x9F, 0x6B, 0x5B, 0x4B, 0x21, 0xC4, 0xDE, 0x53, 0x76, 0x37, 0xD4, 0x11, 0x3D, 0x8A, 0x60, 0xA0, 0xC1, 0x5E, 0xDC, 0xEF, 0x83, 0xEC, 0xBB, 0xFE, 0xB2, 0xEA, 0x1D, 0x4C, 0x94, 0xE8, 0x92, 0x1A, 0x6E, 0x9C, 0x52, 0x95, 0xB8, 0x1F, 0x5D, 0x55, 0x22, 0x83, 0x22, 0xD6, 0x11, 0xE1, 0x4B, 0xD5, 0x6A, 0xD0, 0xF0, 0x00, 0x0F, 0x51, 0x40, 0x69, 0x18, 0xC2, 0xB5, 0xAB, 0xEA, 0x05, 0x07, 0x07, 0x75, 0x34, 0x76, 0xB0, 0xDE, 0x7B, 0x0B, 0x19, 0xDB, 0x10, 0xFC, 0xE8, 0xAB, 0x32, 0x24, 0x2E, 0x46, 0x07, 0xE4, 0xE3, 0x9B, 0x2D, 0xD9, 0x96, 0x3A, 0xE7, 0x09, 0x8D, 0x90, 0x21, 0xE3, 0x7B, 0xD3, 0xCE, 0xC0, 0x08, 0xC9, 0x6A, 0xD1, 0xAF, 0x53, 0xA8, 0x2A, 0xCD, 0x26, 0xD3, 0x89, 0x3E, 0x28, 0x12, 0x75, 0xAF, 0xF3, 0x58, 0x8E, 0x30, 0xA6, 0x97, 0xB6, 0x12, 0x8C, 0xA3, 0xA5, 0x99, 0xB3, 0x4B, 0x32, 0x02, 0xDB, 0x83, 0x4C, 0x17, 0x5C, 0x2A, 0x24, 0xD9, 0xCB, 0xA6, 0x74, 0x41, 0xCD, 0xEA, 0xEF, 0x74, 0xE2, 0xB6, 0x19, 0xF0, 0x15, 0x7A, 0x49, 0x8F, 0x20, 0xB4, 0x13, 0x02, 0xA3, 0x7A, 0x0D, 0x06, 0xD7, 0x96, 0x85, 0xBC, 0xBB, 0xC3, 0x82, 0x48, 0xF3, 0x02, 0x85, 0xCF, 0x0A, 0xA2, 0x89, 0xF1, 0xF9, 0x10, 0xFD, 0x65, 0xC8, 0x64, 0x6B, 0x2E, 0x56, 0x48, 0x37, 0x4F, 0x87, 0xE0, 0x22, 0x54, 0xD9, 0xF6, 0xAA, 0x5B, 0x78, 0x91, 0xAB, 0x9A, 0x4C, 0x72, 0xFD, 0x28, 0xE5, 0x7A, 0xED, 0x7B, 0x86, 0x9D, 0xEF, 0xE8, 0x9A, 0x89, 0x22, 0x39, 0x74, 0x54, 0xEB, 0x34, 0xC7, 0x30, 0xF6, 0x2D, 0x69, 0x96, 0x43, 0x79, 0x21, 0x77, 0x3C, 0x0F, 0xDD, 0x9B, 0x61, 0x6E, 0x1E, 0xDF, 0xDF, 0xE8, 0x13, 0x19, 0x19, 0x1C, 0x77, 0x74, 0x02, 0xD7, 0xF4, 0x4B, 0x1A, 0x58, 0x0D, 0xF7, 0x58, 0x78, 0x70, 0x74, 0x56, 0x41, 0x0F, 0x3C, 0xD7, 0x95, 0xE3, 0xB2, 0xAB, 0x4D, 0x90, 0x80, 0x5A, 0x42, 0x8C, 0x58, 0x89, 0x20, 0x9E, 0x6D, 0xEF, 0x8E, 0xF2, 0x17, 0xCE, 0xCE, 0x9E, 0xBF, 0xD3, 0x5B, 0xAA, 0x42, 0x54, 0x40, 0x5B, 0x51, 0x26, 0xE6, 0x03, 0x59, 0x2E, 0x43, 0x7D, 0xBA, 0x5C, 0xAB, 0x18, 0xB5, 0x05, 0x47, 0x8C, 0x53, 0xD6, 0x48, 0x01, 0x0B, 0xBD, 0xE8, 0x2F, 0xAD, 0x47, 0x4E, 0xD4, 0xA2, 0xFE, 0xDC, 0xB0, 0xBE, 0x22, 0x2E, 0x6F, 0x6A, 0x4C, 0x9E, 0x56, 0x5A, 0x7F, 0x2D, 0x70, 0xEC, 0x52, 0x3D, 0x75, 0xDF, 0xED, 0xA0, 0xBE, 0x40, 0xDC, 0xE1, 0x8C, 0x0D, 0x8A, 0xA0, 0x7B, 0xA8, 0x96, 0x7B, 0x78, 0x8E, 0x05, 0x31, 0x43, 0xA6, 0x6E, 0xC5, 0xA8, 0xB6, 0x2F, 0xBA, 0x0E, 0xFB, 0xE3, 0x59, 0x6B, 0x64, 0x67, 0x13, 0x81, 0x69, 0x98, 0x5A, 0x0B, 0x4F, 0xBE, 0x02, 0x29, 0x00, 0xD6, 0x4E, 0x66, 0xF5, 0xC1, 0xEA, 0x86, 0x57, 0x15, 0xAA, 0xEE, 0x17, 0x99, 0xFD, 0x7E, 0x77, 0x49, 0x46, 0xE0, 0xA3, 0x19, 0x21, 0x5B, 0xC7, 0xAD, 0xC1, 0xFA, 0x05, 0x4A, 0x58, 0x15, 0x02, 0x4A, 0x8A, 0x24, 0xD3, 0x2F, 0x2C, 0x3E, 0xA7, 0x48, 0xED, 0xAB, 0x5D, 0xF8, 0x53, 0x79, 0xE6, 0xA1, 0x29, 0x19, 0x0A, 0xBD, 0x8F, 0x4C, 0xA2, 0xCB, 0x01, 0x4B, 0x56, 0x6A, 0x1A, 0x7B, 0xB7, 0x9A, 0xCD, 0x9B, 0xA7, 0xE5, 0x51, 0xAD, 0x88, 0xFB, 0x21, 0x07, 0x8E, 0x9D, 0xEF, 0x08, 0xFE, 0xB3, 0x33, 0x2E, 0x7F, 0x7F, 0x6E, 0x50, 0x84, 0xF5, 0x08, 0xEE, 0x7B, 0x47, 0x80, 0x52, 0x80, 0xB9, 0x3F, 0xAB, 0x27, 0xBA, 0x07, 0xB4, 0x1E, 0xE1, 0x6D, 0x7C, 0x61, 0x69, 0x31, 0x49, 0x2D, 0xF9, 0xD2, 0x97, 0x76, 0x06, 0x9E, 0xB1, 0x7C, 0x1C, 0x49, 0xCB, 0x11, 0x85, 0x1E, 0x1F, 0x74, 0x51, 0x4A, 0xB6, 0x5D, 0x1C, 0xEA, 0x40, 0x74, 0x7C, 0x87, 0x2B, 0x9C, 0x61, 0xCB, 0x3E, 0x0A, 0x47, 0x43, 0xAA, 0x63, 0x44, 0xE1, 0x80, 0x43, 0xC8, 0x7C, 0x09, 0x38, 0xBE, 0x6E, 0xD8, 0x8D, 0x8E, 0xCE, 0x1C, 0xC1, 0x8F, 0x4A, 0x78, 0x0B, 0xC5, 0x9E, 0xC0, 0xDA, 0x4C, 0x7A, 0xC0, 0x59, 0x87, 0x1C, 0xB4, 0xFB, 0xF2, 0x38, 0x94, 0xA4, 0x11, 0x81, 0x32, 0x35, 0xB8, 0x3E, 0x3E, 0xCA, 0x07, 0x4A, 0x4A, 0x96, 0x1B, 0x56, 0x49, 0x46, 0x1A, 0x6E, 0xC4, 0x62, 0x35, 0xDB, 0x06, 0xBE, 0x80, 0xD0, 0x05, 0xDC, 0x6B, 0x38, 0xE1, 0x2F, 0x94, 0x82, 0x73, 0x35, 0x64, 0x74, 0x79, 0x9E, 0x98, 0xF4, 0x4E, 0x63, 0x1A, 0x7A, 0xBE, 0x46, 0x7D, 0x34, 0x9C, 0x00, 0xED, 0x02, 0x18, 0xA4, 0x2B, 0x1A, 0x37, 0x96, 0x15, 0x12, 0x0F, 0x21, 0xB3, 0x32, 0x0C, 0xD8, 0xF7, 0xAF, 0x83, 0xB2, 0x41, 0x85, 0xE6, 0xD3, 0xB4, 0x6C, 0xCD, 0xBA, 0xE4, 0x55, 0x53, 0x31, 0xE8, 0xD4, 0xC0, 0xCE, 0x9B, 0xD9, 0xEB, 0x6B, 0x5A, 0x2B, 0xA4, 0x3F, 0xA1, 0xF5, 0x55, 0xBA, 0xC4, 0x83, 0x50, 0x86, 0x1C, 0x56, 0x9B, 0x3D, 0xE6, 0x54, 0xEB, 0x02, 0xAA, 0xF5, 0xDD, 0xE0, 0x50, 0x9B, 0xB1, 0xA0, 0xF1, 0xCB, 0xF4, 0xBC, 0x5D, 0x66, 0xB5, 0x24, 0x69, 0xD4, 0x8B, 0x98, 0xAE, 0xDD, 0x62, 0xA7, 0x21, 0xDA, 0xA1, 0xA0, 0xC3, 0xBD, 0x06, 0x6B, 0xCC, 0xD6, 0x1D, 0xE3, 0xF4, 0xA3, 0x7F, 0xE2, 0x82, 0x97, 0x4C, 0x95, 0xC9, 0x36, 0xD7, 0xE3, 0x63, 0x1D, 0xB9, 0x7A, 0x9F, 0xE2, 0x3A, 0xF6, 0xCE, 0x8B, 0xAA, 0x7D, 0x4A, 0xCA, 0xAE, 0x88, 0xF6, 0x5A, 0x98, 0xA5, 0x51, 0x13, 0xB1, 0x97, 0xC4, 0xA6, 0x32, 0xBF, 0x69, 0x28, 0x92, 0x16, 0x69, 0x62, 0x86, 0xAC, 0x11, 0x59, 0x33, 0x7F, 0x29, 0x9B, 0x88, 0x8A, 0x49, 0xDB, 0x21, 0xCD, 0x60, 0x8F, 0xB2, 0x73, 0x25, 0x48, 0x18, 0x9B, 0x53, 0xD7, 0x31, 0xE4, 0xFF, 0x15, 0xDE, 0x7F, 0x4E, 0x6C, 0x48, 0x19, 0x0F, 0x78, 0xEA, 0x9B, 0x7A, 0x42, 0x33, 0x49, 0x8A, 0xC3, 0x56, 0x9D, 0xFF, 0xE1, 0x73, 0xA9, 0x38, 0xCD, 0x75, 0x30, 0xD2, 0x44, 0xC0, 0x1F, 0x38, 0x17, 0xBD, 0x32, 0xD4, 0x9E, 0x47, 0x5E, 0xCE, 0x1B, 0x71, 0xA1, 0x25, 0x91, 0x2B, 0x12, 0xF2, 0x7A, 0xD0, 0xD5, 0x26, 0xFE, 0x74, 0x5A, 0x30, 0x1A, 0x59, 0x0E, 0xA6, 0xBA, 0xB9, 0x3E, 0x77, 0xDB, 0x6E, 0x06, 0x63, 0x42, 0x52, 0xC1, 0x25, 0x18, 0x32, 0xF9, 0x73, 0x51, 0x72, 0x4D, 0xE3, 0x2F, 0x6E, 0xCE, 0xAE, 0xDC, 0x9E, 0xD1, 0xD5, 0x9D, 0xC1, 0xAB, 0xA3, 0x67, 0xA2, 0xF7, 0x6E, 0x67, 0xFB, 0x30, 0x5B, 0x6A, 0x3A, 0xE2, 0xD7, 0xBD, 0x59, 0xE9, 0xFF, 0xE1, 0x27, 0x88, 0x8C, 0x41, 0x41, 0xEC, 0x99, 0x4A, 0x0E, 0x1B, 0x54, 0xAC, 0x8F, 0x9A, 0x9B, 0x51, 0xA4, 0xC3, 0x63, 0xD2, 0xD5, 0x53, 0xA7, 0x2F, 0xCC, 0xEA, 0xE6, 0x5E, 0x79, 0x55, 0xA5, 0x8C, 0x98, 0x5D, 0x48, 0xD5, 0x0B, 0xB0, 0x62, 0x1B, 0x51, 0x30, 0xE9, 0x9D, 0x10, 0x41, 0x0B, 0x11, 0x45, 0xD0, 0xD5, 0x51, 0x0B, 0xDD, 0xB5, 0xD4, 0x9B, 0x03, 0xAB, 0xDD, 0xC9, 0x1B, 0x4B, 0x19, 0xCF, 0xE8, 0xF7, 0x87, 0xF6, 0xBE, 0xAB, 0xFE, 0x1C, 0x85, 0xFA, 0xF3, 0x2F, 0xDB, 0xDF, 0x4B, 0x5F, 0xE0, 0x8C, 0x8E, 0x93, 0xBB, 0x12, 0x4E, 0x41, 0xED, 0xEB, 0x87, 0x7D, 0x3C, 0x45, 0x2E, 0x96, 0xB6, 0xE3, 0x57, 0xF3, 0xD3, 0xDE, 0xB6, 0xE6, 0xA5, 0x86, 0xAF, 0x9B, 0x6E, 0xA9, 0xF9, 0xBB, 0xB5, 0xD0, 0xF0, 0xA5, 0x3F, 0x5B, 0xFD, 0x6F, 0x38, 0xA1, 0xFB, 0xD6, 0xEF, 0x59, 0x20, 0x9F, 0x2D, 0x5A, 0x6B, 0x6A, 0xFC, 0x01, 0xA9, 0x9A, 0x8D, 0xDA, 0xB8, 0xF4, 0x2D, 0xAB, 0x8D, 0x4F, 0x8B, 0x83, 0xA2, 0x1E, 0xBF, 0x53, 0x11, 0xE2, 0xBF, 0x8B, 0x73, 0x9B, 0xC1, 0x5B, 0x27, 0xBB, 0xC0, 0xE8, 0x2C, 0xDB, 0x95, 0x24, 0x68, 0xC4, 0x09, 0x8A, 0xDE, 0xBA, 0x8B, 0xF9, 0x2D, 0xF7, 0x2B, 0x03, 0x3A, 0x7F, 0xB1, 0xB3, 0x91, 0x8A, 0x37, 0xD8, 0xD5, 0xEF, 0x92, 0xBD, 0x32, 0x97, 0x19, 0xCE, 0x9C, 0x02, 0x1F, 0x86, 0xE2, 0x22, 0xC7, 0x1E, 0x59, 0xE2, 0x49, 0x91, 0x31, 0x9F, 0xFB, 0xAE, 0xD6, 0xCD, 0x59, 0x98, 0xE5, 0x02, 0xDF, 0xDF, 0xEE, 0xB7, 0x73, 0xCD, 0x13, 0xD1, 0xFB, 0x3F, 0xA7, 0x1E, 0xF3, 0xFC, 0x6C, 0x5C, 0x14, 0x74, 0xC6, 0xE7, 0x1E, 0x7C, 0x56, 0x51, 0x42, 0xE3, 0x55, 0x3F, 0xAC, 0x6F, 0x37, 0x1D, 0x3D, 0x39, 0x77, 0x4A, 0x8F, 0x35, 0xE4, 0xA9, 0x08, 0x3E, 0x81, 0x6E, 0x1C, 0x09, 0xE9, 0xC7, 0x48, 0x89, 0x1B, 0xB9, 0x70, 0x02, 0x17, 0x0D, 0x83, 0xEA, 0x88, 0x20, 0x1F, 0xFC, 0xF1, 0x4C, 0x3B, 0x18, 0xD6, 0x1A, 0x5F, 0xE2, 0x69, 0x1E, 0xD8, 0xBB, 0x39, 0xE4, 0xE5, 0x0E, 0x14, 0xB1, 0x64, 0x4F, 0x9E, 0x91, 0x57, 0x6B, 0xCE, 0x79, 0x55, 0x71, 0xFA, 0x7B, 0x5A, 0x55, 0xE6, 0x02, 0x11, 0xFA, 0xAA, 0xC4, 0xBF, 0x2E, 0xAE, 0xB2, 0xBB, 0x20, 0xA1, 0x37, 0x79, 0x9C, 0x5F, 0x1A, 0x0E, 0x5D, 0xA6, 0xD0, 0x9A, 0x85, 0x4D, 0x24, 0xE5, 0xFD, 0x33, 0x67, 0x74, 0x97, 0x4D, 0x68, 0x92, 0x4A, 0xEA, 0xC9, 0x2C, 0x51, 0xBD, 0x10, 0x43, 0xF5, 0x2B, 0x51, 0xB9, 0xED, 0x51, 0x74, 0x52, 0x38, 0xE6, 0xB6, 0xE1, 0xBA, 0x26, 0x6B, 0x80, 0xF8, 0xA9, 0xBF, 0xB3, 0x8E, 0x8F, 0x23, 0x7C, 0xB7, 0xE5, 0xBE, 0xD5, 0x22, 0xA5, 0xEF, 0xC9, 0xA9, 0xB9, 0xD1, 0x58, 0xCC, 0x41, 0x35, 0x52, 0xE1, 0x24, 0xF7, 0x46, 0xFA, 0xE1, 0x0F, 0x72, 0x26, 0x6B, 0x11, 0x53, 0x2F, 0x1F, 0xC6, 0x54, 0x20, 0xBF, 0x1D, 0xE4, 0xEB, 0x82, 0x76, 0x7F, 0x83, 0xC6, 0x03, 0xCD, 0x18, 0x9B, 0x9C, 0x23, 0x5A, 0xA2, 0xD4, 0xB9, 0x7B, 0x15, 0xC4, 0x15, 0x56, 0x9E, 0xE3, 0x26, 0x77, 0xD7, 0x1B, 0xDE, 0x7E, 0xE8, 0xA9, 0xF7, 0x0E, 0x9E, 0x07, 0xAA, 0x70, 0x3F, 0xDD, 0xF5, 0x77, 0x74, 0x2B, 0xED, 0x93, 0x58, 0x82, 0x50, 0xD2, 0x50, 0xB4, 0x54, 0xA3, 0xA3, 0xC8, 0x4A, 0xC3, 0x89, 0x26, 0x04, 0x22, 0xAC, 0xF3, 0x6B, 0x4E, 0xE5, 0x77, 0x56, 0x3A, 0x52, 0xF5, 0x15, 0x74, 0x58, 0xD0, 0xA0, 0x48, 0xDD, 0x23, 0x97, 0x13, 0x80, 0x21, 0x51, 0xA5, 0xE0, 0x53, 0x20, 0xAB, 0x1A, 0xC0, 0x83, 0xBE, 0x2A, 0xA1, 0xED, 0x6D, 0xE2, 0x50, 0x51, 0xAD, 0xF6, 0x16, 0xDC, 0xFC, 0xDC, 0x04, 0x3A, 0x5C, 0x89, 0x33, 0xBC, 0xC5, 0xE9, 0x11, 0x2D, 0x51, 0x39, 0x49, 0x4F, 0x43, 0x1D, 0x3A, 0x09, 0xFD, 0x76, 0x1F, 0xEF, 0x0D, 0xE1, 0x80, 0x97, 0x4F, 0xAE, 0xF6, 0x53, 0xCD, 0x98, 0x63, 0xAE, 0x26, 0xB8, 0xEE, 0xEC, 0xDB, 0xC6, 0x6A, 0xF3, 0xCC, 0x5C, 0x53, 0x61, 0xDF, 0x7D, 0x0A, 0x0D, 0xB6, 0xF6, 0xBA, 0xC7, 0x61, 0xFC, 0xFA, 0xEC, 0x70, 0x52, 0x71, 0x6C, 0x2F, 0xD7, 0x7D, 0x5F, 0xE8, 0x03, 0x40, 0x9A, 0xD9, 0x04, 0x15, 0x50, 0x86, 0x9B, 0xE0, 0x16, 0x77, 0x81, 0x65, 0x17, 0x44, 0xF9, 0xAB, 0x75, 0x1E, 0x98, 0x3B, 0xD4, 0x06, 0x89, 0x03, 0x21, 0x39, 0xBA, 0x63, 0xEA, 0x1F, 0x8C, 0x7C, 0xE4, 0x63, 0x59, 0x9C, 0x28, 0x43, 0x22, 0x5D, 0xD5, 0x0A, 0x47, 0x61, 0xA0, 0x28, 0x36, 0xE7, 0x04, 0xF5, 0xEE, 0xED, 0x41, 0x6C, 0x96, 0x60, 0x9A, 0x61, 0x18, 0xF7, 0x11, 0x1B, 0x03, 0xFC, 0xFF, 0xFF, 0x64, 0xE8, 0xAF, 0x3B, 0x96, 0x18, 0x63, 0xDA, 0xDF, 0xBA, 0xA2, 0x6D, 0x5D, 0x54, 0x6D, 0x39, 0x8F, 0x78, 0x9D, 0x17, 0x29, 0xFE, 0xA5, 0xEB, 0x04, 0x2A, 0xC9, 0x66, 0xF8, 0x32, 0x22, 0x8E, 0x7D, 0x50, 0x2C, 0xCA, 0x7E, 0x29, 0x59, 0x51, 0x2B, 0x40, 0xBF, 0x17, 0x1D, 0x43, 0xCC, 0x3C, 0xF5, 0x4B, 0x60, 0xEF, 0xE8, 0xA9, 0x75, 0x14, 0x72, 0x59, 0x2E, 0xC9, 0x9E, 0xA1, 0x43, 0x99, 0xA8, 0xA7, 0x74, 0x9D, 0x92, 0xA2, 0x4F, 0x18, 0xA5, 0xC0, 0xBF, 0x15, 0xF0, 0x18, 0x41, 0xDD, 0x77, 0x60, 0x2F, 0x74, 0xA2, 0xB6, 0x03, 0x5F, 0x45, 0x50, 0x4C, 0xA9, 0xFD, 0xDC, 0x72, 0x6A, 0x7C, 0x85, 0x84, 0x77, 0x58, 0xE6, 0x39, 0xA9, 0x4F, 0xA2, 0x3C, 0x7D, 0xA3, 0xEC, 0x23, 0x6B, 0x44, 0xA5, 0xF3, 0x46, 0xEE, 0x83, 0xF3, 0xA5, 0x42, 0x50, 0x3F, 0x6D, 0x92, 0x37, 0x58, 0x93, 0xE0, 0x11, 0x5E, 0x35, 0xC3, 0xFB, 0x58, 0xEC, 0xC3, 0xA8, 0x7E, 0x6F, 0xB8, 0xD6, 0xF9, 0x89, 0x3B, 0x3B, 0x62, 0x7A, 0x7D, 0xD6, 0xFD, 0xF2, 0x5B, 0x83, 0x7D, 0xDF, 0x6A, 0x2A, 0x32, 0x97, 0x20, 0x2E, 0x75, 0xF0, 0x07, 0x1C, 0x68, 0xF5, 0x85, 0x6B, 0xFE, 0xC1, 0x1D, 0xC5, 0x13, 0xE4, 0xAD, 0x44, 0x67, 0xED, 0x64, 0x77, 0xFA, 0x2D, 0x8D, 0x96, 0xF9, 0xE9, 0x49, 0xED, 0x48, 0x32, 0xFB, 0x42, 0x81, 0xA6, 0xF8, 0x22, 0x22, 0xBD, 0x88, 0x9D, 0xDE, 0xA4, 0xDF, 0xCC, 0x51, 0x06, 0xB1, 0x7C, 0x12, 0xEF, 0xA7, 0x1C, 0x35, 0x8B, 0x6E, 0x61, 0x74, 0x38, 0x21, 0xA6, 0x1A, 0x77, 0xB9, 0xCC, 0x27, 0x1A, 0xFA, 0xEB, 0xE5, 0x65, 0x49, 0x6F, 0x3E, 0x2D, 0xF7, 0xEC, 0xFE, 0x2A, 0xFB, 0x70, 0x37, 0xFD, 0x64, 0x68, 0x81, 0x59, 0xB6, 0x30, 0x9A, 0x3C, 0xE0, 0x31, 0x32, 0xE8, 0x69, 0x50, 0x9A, 0x15, 0x7C, 0x37, 0xFB, 0x07, 0xAC, 0xF2, 0xB4, 0xD9, 0xB7, 0xCF, 0x69, 0x82, 0xF8, 0x6A, 0xFB, 0xDA, 0x9F, 0x6D, 0xE7, 0x44, 0x40, 0x2C, 0x1E, 0xA2, 0xF4, 0xB8, 0x9E, 0xEA, 0x07, 0xD0, 0xE9, 0x70, 0xDC, 0x3E, 0xDC, 0x1A, 0x7C, 0xDD, 0xAD, 0xFB, 0xFC, 0x04, 0x18, 0xCA, 0x18, 0x4B, 0xFE, 0x4D, 0x74, 0x87, 0x4A, 0x73, 0xCD, 0x6C, 0x51, 0xE7, 0xAA, 0xE0, 0xD8, 0x20, 0xB7, 0x87, 0x2C, 0x2B, 0xF4, 0x7E, 0x4C, 0x87, 0x5C, 0xDC, 0x4D, 0x36, 0xC3, 0x0E, 0x17, 0xBD, 0x8C, 0x73, 0x73, 0x57, 0x96, 0x86, 0x19, 0x71, 0x17, 0x5C, 0x52, 0xE6, 0xC5, 0x50, 0x89, 0x06, 0x5B, 0x37, 0xF0, 0xDE, 0x03, 0xEF, 0xD2, 0x5B, 0xC3, 0x3F, 0xB4, 0x6A, 0x34, 0x9B, 0xC2, 0xBD, 0xC6, 0x93, 0x23, 0x8F, 0x3A, 0xB8, 0xA9, 0xDF, 0x36, 0x6C, 0x96, 0xB2, 0xA3, 0xB7, 0xCC, 0x14, 0x1D, 0xDD, 0x7C, 0xAF, 0x4B, 0xA3, 0x50, 0xFE, 0xC6, 0xD6, 0x22, 0xED, 0x49, 0x42, 0x0A, 0x9B, 0xCF, 0xC4, 0x2B, 0x24, 0xF2, 0xB8, 0x53, 0x7F, 0x67, 0x0A, 0x67, 0xC3, 0x5B, 0x48, 0xE6, 0x88, 0x88, 0x98, 0x19, 0xE7, 0x2F, 0xB5, 0x47, 0xCB, 0x22, 0xFC, 0x3F, 0x50, 0xE0, 0x69, 0x31, 0x94, 0x16, 0xD6, 0x94, 0xBE, 0xA2, 0x9C, 0xEA, 0xB2, 0xDA, 0xDB, 0x70, 0xC0, 0xE4, 0x57, 0xBA, 0xD2, 0x1B, 0x97, 0x44, 0xFF, 0x8E, 0xB8, 0xAD, 0x26, 0xBE, 0xE9, 0x99, 0x5A, 0xD7, 0xC7, 0x1D, 0xD0, 0x3C, 0xC2, 0xD0, 0x3B, 0x40, 0x29, 0x72, 0xB9, 0x70, 0xCE, 0xC8, 0xCB, 0xCD, 0x2C, 0xF9, 0x5E, 0x76, 0x25, 0x7D, 0x56, 0x65, 0x6D, 0x02, 0xBF, 0x88, 0x57, 0x8D, 0x81, 0xEC, 0x64, 0xB4, 0x10, 0xAE, 0x38, 0xAE, 0x08, 0x78, 0x3F, 0xED, 0x4C, 0x0C, 0x8B, 0x24, 0x54, 0x3F, 0x17, 0x0B, 0x26, 0x6C, 0x22, 0x6D, 0x47, 0xFE, 0xF9, 0x4F, 0xFB, 0x71, 0x0F, 0xA2, 0xBE, 0xC8, 0x83, 0x6C, 0x35, 0x10, 0xB6, 0xB9, 0xE8, 0xE4, 0xF2, 0x34, 0xC0, 0x8F, 0x48, 0x7E, 0x02, 0x5F, 0xA2, 0x8A, 0xC2, 0x74, 0xC1, 0x39, 0xD5, 0xD4, 0x39, 0xE0, 0xE2, 0x05, 0x43, 0x08, 0x0C, 0x17, 0x63, 0x9C, 0x78, 0xC8, 0x93, 0xC6, 0x8E, 0x44, 0xD5, 0x42, 0x4A, 0x94, 0x2C, 0x86, 0x7E, 0x3C, 0xF2, 0xDA, 0x84, 0x3E, 0x02, 0x26, 0xAD, 0xA1, 0x87, 0x05, 0x9D, 0xE1, 0x11, 0x77, 0x36, 0x67, 0xC7, 0xC9, 0xB5, 0xAE, 0xBB, 0xF4, 0xDF, 0x0B, 0x4A, 0xD5, 0xF3, 0x32, 0x3F, 0xD2, 0x6B, 0x94, 0xC1, 0xFA, 0x1D, 0x61, 0x31, 0xC0, 0x93, 0x57, 0x92, 0x65, 0x84, 0xD0, 0x3F, 0x41, 0x82, 0x98, 0x36, 0xC6, 0x37, 0x74, 0x4A, 0xEF, 0x14, 0xA2, 0x48, 0x57, 0xE5, 0x64, 0x38, 0x2F, 0xE4, 0xE5, 0x1A, 0x9A, 0xA1, 0xC1, 0xDF, 0x1F, 0x84, 0xB2, 0x48, 0x85, 0x67, 0x8A, 0x6B, 0xB4, 0xA2, 0xF5, 0xD5, 0x54, 0x67, 0xB5, 0x2D, 0x79, 0xC9, 0xD6, 0xD1, 0xB9, 0x98, 0x19, 0xB3, 0xB4, 0x95, 0xEB, 0x49, 0x25, 0x1D, 0x4B, 0x3C, 0x00, 0x41, 0x2E, 0xF8, 0xBE, 0x0F, 0x9A, 0xD1, 0x83, 0x14, 0x0A, 0x7F, 0x40, 0xD1, 0x2A, 0x97, 0xD6, 0x24, 0x4D, 0x7B, 0xD9, 0x04, 0x4C, 0x8E, 0x49, 0x24, 0x4E, 0xC1, 0x4B, 0x04, 0x47, 0xED, 0x2C, 0xDD, 0x43, 0x30, 0x34, 0x0B, 0x81, 0xD5, 0xC8, 0xCC, 0xFD, 0x72, 0xDF, 0x67, 0x50, 0x92, 0x42, 0xD0, 0xC6, 0x53, 0x50, 0xD3, 0x65, 0xF0, 0x6A, 0xF6, 0xC8, 0x66, 0xEE, 0x26, 0x54, 0x18, 0xB7, 0x1F, 0xFC, 0x7A, 0x5A, 0xD2, 0x32, 0xD9, 0xB2, 0x14, 0x71, 0x75, 0x43, 0x90, 0x9C, 0xC3, 0x65, 0x3A, 0xDE, 0xDD, 0x6A, 0x31, 0xB1, 0x36, 0xD8, 0x81, 0x36, 0xAC, 0xEE, 0x3D, 0xA4, 0xA0, 0x50, 0xEF, 0x3E, 0x8C, 0x3C, 0x74, 0xC0, 0x95, 0x87, 0x06, 0x30, 0x15, 0x75, 0xC5, 0x85, 0xB7, 0x6B, 0xD2, 0xE8, 0x15, 0x57, 0xBD, 0x83, 0xE1, 0x2A, 0x9E, 0x1F, 0x32, 0x10, 0x99, 0xF0, 0x1A, 0xB7, 0x28, 0x71, 0x58, 0x9F, 0x95, 0xEC, 0xC8, 0xE3, 0x36, 0x5C, 0x13, 0x68, 0xD7, 0xB6, 0x43, 0x5F, 0x75, 0xCE, 0x69, 0xA1, 0x96, 0xAC, 0xFA, 0x3D, 0x78, 0x3E, 0x84, 0xFE, 0xC5, 0x95, 0xD6, 0x18, 0x53, 0xFF, 0xD6, 0x20, 0x78, 0x98, 0x25, 0xD5, 0x91, 0x83, 0x1F, 0xE8, 0xD1, 0x11, 0x4E, 0x18, 0xC3, 0xBD, 0x68, 0x51, 0x3E, 0x31, 0x11, 0xB8, 0xE1, 0x70, 0xE3, 0x28, 0xB0, 0x23, 0x52, 0x6A, 0x90, 0x1E, 0x00, 0xF2, 0x34, 0x61, 0x61, 0x52, 0x5C, 0x4B, 0xA6, 0x7A, 0xC5, 0x99, 0x0F, 0x78, 0x03, 0x14, 0xEC, 0xDE, 0x03, 0x1E, 0xB6, 0x32, 0x67, 0x0E, 0xCC, 0x5C, 0x51, 0xC1, 0xEE, 0x7B, 0x09, 0x4A, 0x0E, 0x2B, 0x44, 0x2E, 0xC3, 0x73, 0xCB, 0x4D, 0xFC, 0x92, 0x61, 0x19, 0x8C, 0x70, 0xB7, 0xF5, 0x6E, 0xFC, 0x0D, 0xDF, 0x66, 0x41, 0x42, 0x13, 0x46, 0x96, 0xD2, 0xDA, 0x3A, 0x1A, 0x06, 0xE1, 0xE9, 0x7B, 0xEC, 0xED, 0xF6, 0x96, 0x9F, 0x8E, 0x8C, 0x5B, 0xA2, 0x01, 0x51, 0x9B, 0x6F, 0xD9, 0xE3, 0x80, 0x91, 0x4C, 0x70, 0x23, 0xB6, 0xFF, 0x57, 0x44, 0xC0, 0xDB, 0xD3, 0x46, 0x2F, 0x83, 0xC7, 0xFD, 0x6E, 0xA5, 0xA0, 0x9F, 0x50, 0x1E, 0x6F, 0x8B, 0xB0, 0x31, 0x69, 0xAD, 0x22, 0x62, 0x56, 0x89, 0xB2, 0x73, 0x16, 0xDF, 0xD3, 0xC7, 0x45, 0x67, 0x31, 0x19, 0xBD, 0x19, 0xEB, 0x3C, 0xDF, 0x80, 0x2C, 0xCC, 0x47, 0x41, 0x39, 0x6D, 0x36, 0x0C, 0xEC, 0x3F, 0xAA, 0x03, 0xF4, 0xA1, 0x82, 0xE3, 0x98, 0xA2, 0x0A, 0xD7, 0x9F, 0x08, 0x24, 0x53, 0xE8, 0x9E, 0x76, 0xFC, 0xBF, 0x16, 0xB4, 0x18, 0xC7, 0x0C, 0x9A, 0xE8, 0x63, 0x2E, 0x82, 0x1B, 0xF7, 0x83, 0x49, 0xDB, 0x1B, 0x62, 0x0C, 0x36, 0xBD, 0x28, 0xBF, 0x15, 0x47, 0x45, 0x1F, 0x23, 0x13, 0xD2, 0x78, 0x1B, 0xAC, 0xBA, 0xF2, 0x28, 0xAA, 0xD7, 0xA1, 0x72, 0x06, 0x64, 0x86, 0xEF, 0xF0, 0x2D, 0x78, 0x1C, 0xAF, 0x93, 0xE5, 0x6C, 0x97, 0xAF, 0xC0, 0xD4, 0xD9, 0x66, 0xDA, 0x55, 0xBD, 0xCB, 0x22, 0xDF, 0x5E, 0x45, 0xD2, 0x0D, 0xAE, 0xED, 0xCD, 0xDA, 0x50, 0x39, 0x56, 0xB6, 0xA5, 0xFB, 0x29, 0xDD, 0x16, 0xC1, 0xCF, 0xF1, 0x6E, 0x53, 0x4B, 0x33, 0xC6, 0x01, 0x31, 0x27, 0x9F, 0x7D, 0x3A, 0x7E, 0xC0, 0x0B, 0xEF, 0x72, 0x6D, 0xEA, 0xAF, 0x11, 0xB1, 0x04, 0x75, 0xE0, 0x20, 0x12, 0x50, 0xAF, 0x1E, 0x92, 0xF2, 0x54, 0xBE, 0xF2, 0x73, 0xE5, 0x7B, 0x64, 0x38, 0x02, 0x59, 0xD7, 0xB7, 0x91, 0xEF, 0x69, 0x39, 0xAE, 0x50, 0xCC, 0xCD, 0xA7, 0x41, 0x78, 0x48, 0x7D, 0x74, 0xC3, 0x81, 0xD3, 0xB1, 0x9E, 0x34, 0x66, 0xFD, 0x53, 0xB8, 0xF5, 0xC5, 0x99, 0x5A, 0x38, 0xB6, 0xEB, 0x9F, 0x64, 0xF0, 0x79, 0x9E, 0x2B, 0xCB, 0x53, 0xB5, 0x16, 0x3B, 0xF6, 0x5F, 0xE7, 0x13, 0x0D, 0xD4, 0x5B, 0xFF, 0x14, 0x9F, 0xBD, 0x77, 0xA2, 0x31, 0x40, 0x1C, 0xAB, 0x47, 0x05, 0x8C, 0x42, 0xF1, 0xBB, 0xF0, 0x08, 0xC0, 0xCD, 0xAD, 0x8E, 0x5E, 0x99, 0x72, 0xE1, 0xE1, 0x13, 0xF0, 0x09, 0x11, 0xFD, 0xCB, 0xA3, 0xEE, 0x34, 0xA0, 0x35, 0x05, 0x3D, 0x0F, 0x5D, 0xBD, 0xD2, 0x1A, 0x2D, 0xB5, 0xE5, 0x54, 0x75, 0x3B, 0x0F, 0xDC, 0x51, 0x57, 0x3C, 0xE0, 0x86, 0x29, 0x60, 0x85, 0x91, 0x1F, 0x7C, 0x26, 0x20, 0x13, 0x90, 0x9F, 0xC6, 0xB4, 0x11, 0x7D, 0x8D, 0x74, 0xF7, 0x1F, 0xB4, 0x0C, 0xD6, 0x0C, 0xF2, 0xA8, 0x51, 0xDF, 0x09, 0x7F, 0x89, 0x26, 0x60, 0x58, 0x09, 0xC1, 0xF4, 0xC1, 0xBE, 0x05, 0xEF, 0x45, 0xE6, 0x4A, 0x59, 0xF3, 0xC4, 0x7F, 0x0C, 0x91, 0xBD, 0xAF, 0x1A, 0x38, 0x15, 0x9D, 0x91, 0xA4, 0x34, 0x02, 0xC7, 0xE8, 0x29, 0xCB, 0x93, 0x59, 0x7C, 0x98, 0x0E, 0x0D, 0xCA, 0x8A, 0x96, 0xFF, 0x18, 0x4C, 0xB2, 0xCE, 0x65, 0xA2, 0xC5, 0x6C, 0xA9, 0x3A, 0xF5, 0xFC, 0xB3, 0xEB, 0x77, 0x33, 0xFD, 0xB7, 0xF3, 0x1B, 0xA5, 0xA3, 0xA0, 0x01, 0xE9, 0x66, 0x0A, 0xB9, 0x89, 0x6C, 0x31, 0x40, 0x48, 0x46, 0xEC, 0xA2, 0x3F, 0x20, 0x43, 0x27, 0x92, 0x41, 0x72, 0x20, 0x48, 0x8B, 0xB2, 0x49, 0x21, 0xD2, 0x1F, 0xA6, 0xA0, 0x88, 0x03, 0x09, 0xCA, 0xF3, 0x16, 0xB8, 0x15, 0xC9, 0x4E, 0xE6, 0x22, 0xAC, 0xF7, 0x10, 0xF6, 0x1D, 0x23, 0xDF, 0xC9, 0xC6, 0x24, 0x3E, 0xE2, 0xB4, 0x1B, 0xA9, 0x5F, 0x4A, 0xDB, 0xCA, 0x13, 0x15, 0x9A, 0x56, 0xF0, 0x0B, 0x86, 0x1F, 0x2F, 0x48, 0xF2, 0x86, 0xA6, 0x0C, 0x96, 0x9C, 0xDA, 0xF2, 0x39, 0x81, 0x07, 0xE5, 0x28, 0x1F, 0x96, 0x93, 0x4F, 0x63, 0xEE, 0xE4, 0xF6, 0x6C, 0xCD, 0x84, 0xE3, 0x15, 0xFD, 0x4E, 0x80, 0xBD, 0x73, 0x1E, 0x6F, 0xD6, 0x5B, 0xCF, 0x9D, 0xAA, 0x0F, 0xE0, 0x14, 0x4C, 0x56, 0xDD, 0x4C, 0x3A, 0xCA, 0x51, 0x98, 0x19, 0x93, 0x42, 0xCE, 0x96, 0x9E, 0xE5, 0xE3, 0xAE, 0x47, 0x06, 0xBF, 0xB8, 0xAE, 0xDF, 0xD2, 0x6E, 0x5D, 0x9B, 0x78, 0xF5, 0xD0, 0x73, 0x4C, 0xA7, 0x78, 0x25, 0x65, 0x6E, 0x2E, 0x15, 0xE8, 0x82, 0xB8, 0xE7, 0x6E, 0x0F, 0xEC, 0x39, 0x56, 0xFA, 0xB8, 0x36, 0xD6, 0xC6, 0xBC, 0x37, 0xD9, 0xFD, 0x8A, 0x9D, 0xA9, 0x73, 0x2F, 0x61, 0xF2, 0xB6, 0xB1, 0x5D, 0xDB, 0x72, 0xFE, 0x39, 0x99, 0x34, 0x1F, 0xFF, 0x85, 0x45, 0x50, 0x2F, 0x74, 0x17, 0x3E, 0x9F, 0x31, 0x0A, 0xDA, 0x5F, 0x59, 0x29, 0xEC, 0xF7, 0xC4, 0x4F, 0x83, 0x63, 0x3B, 0xB3, 0x9A, 0xC4, 0xE8, 0xD2, 0x7D, 0x18, 0x6D, 0xDE, 0x96, 0x20, 0xC4, 0xDB, 0x3D, 0xF9, 0x22, 0xC2, 0x7E, 0xC1, 0xE1, 0x82, 0xA6, 0xD8, 0xDF, 0x3D, 0xBD, 0xC8, 0x38, 0xB8, 0x58, 0x8C, 0xF6, 0xDA, 0x05, 0xDA, 0x73, 0xC6, 0xD2, 0xC2, 0x5C, 0x0E, 0xE3, 0xB1, 0x49, 0xFD, 0xAE, 0x41, 0x8F, 0x69, 0xC2, 0xCE, 0xF0, 0xAC, 0xA6, 0xD1, 0xB3, 0xB9, 0xD2, 0x5B, 0xCC, 0xF9, 0xAC, 0xAE, 0x51, 0xA4, 0x0A, 0xA1, 0x45, 0x41, 0x3C, 0x9B, 0x93, 0x5A, 0xF3, 0xC8, 0x5D, 0x1D, 0x4A, 0x17, 0x81, 0xF4, 0x13, 0xB4, 0x81, 0x6B, 0x84, 0xD9, 0x86, 0xE8, 0xCD, 0xB2, 0x84, 0x2E, 0xCB, 0xCC, 0x39, 0x97, 0x54, 0x1D, 0xFA, 0x26, 0x5E, 0x88, 0x09, 0xC2, 0xDE, 0x0A, 0x05, 0xBD, 0xCC, 0xB8, 0x99, 0xCD, 0xC9, 0xC5, 0x88, 0xCA, 0x9D, 0xA3, 0x96, 0xF1, 0x6D, 0xCE, 0xC2, 0x25, 0xFC, 0x69, 0x05, 0x79, 0x67, 0xAD, 0xF1, 0xE9, 0xA9, 0x15, 0x7E, 0xFA, 0x12, 0xB3, 0x9C, 0x8E, 0x36, 0xF8, 0x05, 0xFA, 0xE1, 0x03, 0x64, 0x85, 0x05, 0x98, 0x30, 0xF3, 0xB9, 0x6F, 0xD0, 0xF5, 0x11, 0x5F, 0x4D, 0x1A, 0xDF, 0x1F, 0xA4, 0x58, 0x84, 0xA2, 0x87, 0x7A, 0x6F, 0x51, 0xFC, 0x9F, 0x5A, 0x05, 0xE7, 0x57, 0xE1, 0x0A, 0x42, 0x04, 0xD4, 0x2A, 0xEE, 0x87, 0x11, 0x76, 0x0D, 0x54, 0xBA, 0x3E, 0x6D, 0x0B, 0x62, 0x85, 0xD1, 0xFB, 0xC6, 0x0C, 0x89, 0xD2, 0x47, 0xFE, 0xDF, 0xD1, 0x09, 0x17, 0x46, 0xC3, 0x6D, 0x03, 0xAD, 0xD3, 0xE4, 0xAD, 0x18, 0x64, 0xCE, 0x4E, 0x44, 0x77, 0x16, 0x5E, 0x08, 0x94, 0x89, 0x78, 0x15, 0x8B, 0xBC, 0x62, 0x6F, 0xAE, 0x23, 0x57, 0x49, 0x1D, 0x7C, 0x57, 0x6B, 0x73, 0xD4, 0x9E, 0xB8, 0xEB, 0x7D, 0x38, 0x61, 0x1C, 0xE6, 0xAC, 0x37, 0x69, 0x49, 0x4B, 0xA6, 0xFA, 0xFC, 0x75, 0xC9, 0x2D, 0xCE, 0xBE, 0x91, 0xB5, 0x6D, 0x47, 0xF0, 0x37, 0xC1, 0xC7, 0xE4, 0x2B, 0xCE, 0xE3, 0xD4, 0x6B, 0x55, 0xEB, 0xD0, 0xEC, 0x6F, 0x48, 0x2B, 0x81, 0x2B, 0xA0, 0x52, 0x6C, 0xFA, 0x69, 0x31, 0xBB, 0x4B, 0x6C, 0x39, 0xA1, 0x6C, 0x50, 0x7C, 0x41, 0xDB, 0x45, 0xAE, 0x6E, 0xB0, 0xBE, 0xE4, 0xFC, 0x57, 0x03, 0xDA, 0xE0, 0x32, 0x4A, 0x4C, 0x27, 0x99, 0xB2, 0xE5, 0x5C, 0xA0, 0x79, 0x87, 0xA9, 0x87, 0x5D, 0xEC, 0x14, 0xBA, 0x66, 0x0C, 0xDF, 0x0C, 0xA4, 0xD3, 0xA0, 0x32, 0x5D, 0x05, 0x6D, 0x60, 0x9D, 0x8F, 0x6C, 0x16, 0x29, 0x1D, 0x12, 0x20, 0x74, 0xEC, 0x5C, 0xFD, 0xA7, 0x5E, 0x4D, 0x85, 0x25, 0xBE, 0x1F, 0x77, 0xAF, 0x53, 0xFA, 0x16, 0xE2, 0xCF, 0xA8, 0xF1, 0x2A, 0xA1, 0x0C, 0x65, 0xBC, 0xED, 0xD3, 0xB5, 0xBC, 0x3B, 0x15, 0x41, 0x2A, 0xB6, 0x83, 0x0C, 0x64, 0x80, 0x7A, 0x7B, 0x8E, 0x8E, 0x3C, 0x17, 0xFA, 0xAF, 0xBE, 0xB3, 0xB9, 0x6E, 0xC7, 0x21, 0xDC, 0xE7, 0xB1, 0x4D, 0x41, 0xA8, 0xAC, 0x12, 0xD2, 0x78, 0x7A, 0x18, 0x8C, 0xBB, 0xED, 0x06, 0x96, 0xA1, 0x7A, 0x56, 0xFA, 0xFB, 0x63, 0x43, 0x78, 0xC1, 0x4F, 0xCF, 0x45, 0x00, 0x29, 0x73, 0x53, 0xCD, 0x09, 0x6F, 0x3B, 0x36, 0x79, 0xAA, 0x86, 0x9D, 0xA6, 0xD2, 0xEF, 0x9A, 0x8F, 0x61, 0x68, 0x1D, 0x90, 0x12, 0x73, 0x6B, 0x15, 0x70, 0xED, 0x49, 0xB1, 0x99, 0x66, 0x47, 0x0D, 0xF5, 0x03, 0x9F, 0x0B, 0x7B, 0xCD, 0x0F, 0x3D, 0x07, 0x2F, 0x7C, 0x63, 0xB4, 0xD9, 0x83, 0xBD, 0x43, 0x07, 0x7D, 0xFD, 0xBE, 0x7D, 0x9D, 0x2E, 0x21, 0x1F, 0xC6, 0x65, 0xF0, 0xCE, 0xAD, 0xC5, 0x45, 0x7E, 0x4D, 0x44, 0x54, 0x39, 0xE6, 0xE3, 0x0F, 0x40, 0x50, 0x3C, 0x25, 0x83, 0x52, 0xBB, 0x96, 0x15, 0xF4, 0x4C, 0x92, 0xCC, 0x10, 0x57, 0xF4, 0x8A, 0x04, 0x01, 0xDB, 0x54, 0xDF, 0x9E, 0xFA, 0x98, 0x94, 0x30, 0x33, 0xB9, 0x36, 0xDF, 0x60, 0x09, 0x4F, 0x45, 0xC4, 0x07, 0xF0, 0xA0, 0x28, 0xB1, 0x47, 0x22, 0x7B, 0xF0, 0x9C, 0x23, 0x12, 0x88, 0x3D, 0x70, 0xD4, 0x88, 0xEB, 0x67, 0x84, 0xB3, 0xFE, 0x5D, 0x43, 0xE6, 0x58, 0x5E, 0x18, 0x1B, 0x8A, 0x00, 0xB3, 0x1C, 0x1D, 0x44, 0x30, 0x18, 0x7D, 0xCC, 0x87, 0x92, 0xE0, 0x8A, 0xFF, 0xC4, 0x3C, 0x3C, 0xA7, 0x6B, 0xF5, 0xDC, 0xF4, 0xCC, 0xD0, 0xEA, 0xFF, 0x68, 0xE5, 0xD0, 0xBE, 0x9A, 0x90, 0x8B, 0x49, 0x11, 0x05, 0x29, 0xBC, 0x1D, 0xC6, 0xA5, 0x73, 0xC2, 0x08, 0x7E, 0x91, 0x71, 0xF9, 0x53, 0x36, 0x44, 0x5E, 0x52, 0xF3, 0x2F, 0xC7, 0x1F, 0xB1, 0xA5, 0xD6, 0xCC, 0x9C, 0x07, 0x90, 0x03, 0x8A, 0x70, 0x63, 0x43, 0x61, 0x8C, 0x96, 0x76, 0x75, 0xD2, 0x0A, 0xEF, 0xA4, 0x8E, 0x7E, 0x12, 0x60, 0xE3, 0x47, 0x49, 0x99, 0x75, 0x73, 0xA4, 0xFC, 0xF4, 0x47, 0x4C, 0x87, 0x83, 0x54, 0xA9, 0x2B, 0xC6, 0x32, 0x02, 0xD9, 0x76, 0x80, 0x0C, 0x71, 0x3E, 0x80, 0xF6, 0x58, 0xEC, 0xB4, 0x5C, 0xDA, 0x25, 0xA4, 0x79, 0xBD, 0xDC, 0x40, 0xD3, 0x10, 0x04, 0x33, 0xED, 0xF1, 0xCC, 0x55, 0x43, 0x0A, 0x89, 0x45, 0xC6, 0x29, 0xBB, 0xD6, 0x45, 0x8E, 0x2B, 0x04, 0x75, 0x09, 0xDC, 0xF0, 0x47, 0xFB, 0x98, 0xA9, 0xF1, 0xF9, 0x19, 0xA2, 0xE0, 0x7B, 0xB4, 0xA4, 0x40, 0x45, 0x2F, 0xE3, 0xBC, 0x66, 0xCC, 0x2F, 0x84, 0xD9, 0x94, 0x61, 0xBE, 0x2A, 0x6E, 0x47, 0xA6, 0x1A, 0x1D, 0xBF, 0x08, 0x7E, 0xC8, 0xF5, 0x53, 0xA6, 0x56, 0x18, 0x40, 0xED, 0x0F, 0xCA, 0xCF, 0x7E, 0xB7, 0x2B, 0x3B, 0xB7, 0xE8, 0xCD, 0x16, 0x96, 0x8A, 0x5E, 0x07, 0x36, 0x29, 0x27, 0x33, 0x8C, 0xBB, 0x62, 0x4C, 0x7D, 0x25, 0x8F, 0x19, 0xEF, 0xCE, 0xC0, 0xAE, 0xD4, 0x5D, 0x76, 0xD7, 0x7D, 0xB1, 0xDC, 0x58, 0x7F, 0xA6, 0x48, 0x97, 0xEC, 0xCA, 0x13, 0x1B, 0x51, 0xDC, 0x9A, 0xA2, 0x93, 0x66, 0x80, 0xC8, 0x49, 0xEE, 0x14, 0x67, 0xFB, 0xBE, 0x32, 0x69, 0x2F, 0xB2, 0x0D, 0x16, 0x17, 0x57, 0x78, 0x5B, 0x59, 0x19, 0x5E, 0x74, 0x88, 0xFF, 0x55, 0x80, 0xB2, 0x0F, 0x7F, 0xCF, 0xAE, 0xBC, 0x1A, 0x52, 0xD2, 0xA7, 0xDD, 0xC9, 0x64, 0x59, 0x60, 0x19, 0xA5, 0x7C, 0x84, 0x79, 0xEF, 0x4F, 0x36, 0xEA, 0x46, 0xC0, 0xA3, 0x88, 0xAC, 0xB0, 0x22, 0xE1, 0x42, 0x5D, 0x29, 0x27, 0x74, 0x06, 0x29, 0x08, 0x11, 0xF1, 0xA0, 0xF9, 0x6C, 0x98, 0x9D, 0x54, 0x2A, 0x89, 0x1E, 0x17, 0x7E, 0x4B, 0x37, 0xDC, 0xDB, 0x01, 0x24, 0x5E, 0xED, 0x13, 0x81, 0xAE, 0x5D, 0xD1, 0xA1, 0xEA, 0xC5, 0xCC, 0xA8, 0x42, 0x56, 0x1B, 0x70, 0x4C, 0xA5, 0xAB, 0x54, 0x2C, 0xA2, 0x29, 0xE1, 0xA3, 0x40, 0xA1, 0xB3, 0xA6, 0xD8, 0xB4, 0x1D, 0x0F, 0xD4, 0xBA, 0x33, 0x67, 0x13, 0x74, 0x93, 0x60, 0x19, 0x09, 0x8E, 0x0F, 0x82, 0xDE, 0x4F, 0x29, 0x00, 0xD2, 0xE9, 0x2D, 0x1F, 0x5C, 0x1C, 0x47, 0x02, 0x33, 0x01, 0xD8, 0x41, 0xCA, 0xC0, 0x21, 0xD4, 0xDF, 0x89, 0x59, 0xEA, 0x73, 0x36, 0x7F, 0x1A, 0x11, 0x2A, 0xFB, 0x47, 0x2A, 0xD2, 0xCF, 0xAA, 0xA5, 0xC6, 0x0D, 0x93, 0x73, 0x4D, 0x48, 0xA5, 0x0B, 0x6B, 0xA6, 0xFB, 0x6E, 0x0F, 0x45, 0xD3, 0x4E, 0xD5, 0x80, 0x80, 0x27, 0x9D, 0x94, 0xFF, 0x51, 0x70, 0xEC, 0x7D, 0xD5, 0x48, 0xBC, 0x99, 0x7A, 0x4D, 0x2E, 0x24, 0xA9, 0x1B, 0x08, 0xCA, 0xEA, 0x9B, 0x7C, 0xF5, 0xDF, 0x6A, 0xD9, 0x32, 0x35, 0xD9, 0x0B, 0xB3, 0xE3, 0x16, 0x49, 0x1D, 0xF5, 0xBA, 0x4E, 0xFC, 0x59, 0xE7, 0x70, 0x38, 0xD7, 0xBC, 0xCF, 0x7A, 0x15, 0x74, 0xEF, 0x66, 0x04, 0x87, 0x67, 0xE4, 0xD5, 0x56, 0x45, 0xD2, 0x23, 0x41, 0x1E, 0x3F, 0xB6, 0x5C, 0x9A, 0x08, 0xFA, 0xF6, 0xD8, 0xB1, 0x72, 0x82, 0x9D, 0x19, 0xDF, 0x54, 0xBB, 0xA3, 0x18, 0x5A, 0xE9, 0x2C, 0xBD, 0xE7, 0xD5, 0x32, 0x28, 0x62, 0xF0, 0x4A, 0x5D, 0x25, 0x64, 0x20, 0x27, 0xA1, 0xD4, 0x48, 0x12, 0x48, 0x45, 0xEB, 0xB1, 0x84, 0x24, 0xFE, 0x6C, 0xA4, 0x77, 0x20, 0xF3, 0x95, 0x2B, 0xAB, 0x95, 0xA6, 0x77, 0x3E, 0x8E, 0x86, 0x7A, 0x78, 0x07, 0xA4, 0x42, 0x80, 0x89, 0x1A, 0x9B, 0x74, 0x57, 0x36, 0xED, 0x45, 0x6E, 0xAC, 0x59, 0x99, 0x0F, 0x3F, 0x90, 0xC7, 0xC2, 0xC7, 0x9E, 0x20, 0x83, 0x2F, 0x67, 0x9D, 0x4B, 0x02, 0x75, 0x12, 0x49, 0xEA, 0x57, 0xF1, 0x59, 0x32, 0xC6, 0x3C, 0x7A, 0x70, 0x02, 0x74, 0xA2, 0xE3, 0xB4, 0xB4, 0xCE, 0xB8, 0x90, 0x53, 0x2C, 0x95, 0x8E, 0x89, 0x2E, 0xEA, 0xE2, 0x25, 0xB9, 0xCF, 0xCC, 0xBE, 0x94, 0x11, 0xAB, 0x54, 0x24, 0xF3, 0x5A, 0x8D, 0x74, 0x64, 0xBC, 0xBA, 0x4D, 0x7E, 0xE3, 0x87, 0xFC, 0x05, 0xEE, 0xEB, 0x7A, 0x1D, 0x10, 0xE9, 0x96, 0xE3, 0x1B, 0x5E, 0xFD, 0x9C, 0x8C, 0xD6, 0xE5, 0xD1, 0x38, 0x02, 0x97, 0xA2, 0xF7, 0x03, 0x14, 0xDF, 0xF1, 0x8E, 0xA3, 0x5A, 0x79, 0x55, 0x89, 0x73, 0x13, 0x4A, 0x46, 0x38, 0xB9, 0x59, 0x99, 0x49, 0x40, 0x37, 0x75, 0x19, 0xA6, 0xC2, 0x40, 0x6D, 0x2E, 0x08, 0xC1, 0x8C, 0xBA, 0xD0, 0x85, 0xC7, 0xC5, 0xC5, 0xCA, 0xE5, 0x30, 0x26, 0x24, 0xC4, 0x0A, 0x31, 0xED, 0x41, 0x81, 0x56, 0xF1, 0xE9, 0x91, 0x5F, 0x8C, 0x79, 0xDE, 0x39, 0xF4, 0x9B, 0x3A, 0x1F, 0x2E, 0x03, 0x58, 0x5F, 0x72, 0x27, 0x77, 0xB9, 0xF8, 0x32, 0xF4, 0x09, 0x6D, 0x46, 0x28, 0xCE, 0x18, 0xA7, 0x3D, 0xFC, 0x50, 0xB0, 0x56, 0x61, 0x15, 0x1D, 0xF7, 0xF6, 0x1F, 0x8A, 0xAA, 0x95, 0x62, 0x42, 0xFD, 0xB8, 0x0A, 0xDB, 0x11, 0xC8, 0x2F, 0x91, 0x15, 0x3B, 0xA9, 0x65, 0xDD, 0x5A, 0xFD, 0x0A, 0xAF, 0x7F, 0x20, 0x90, 0xAC, 0x6A, 0x2A, 0x0A, 0x39, 0x5A, 0x19, 0x19, 0x45, 0x15, 0x78, 0xD1, 0x57, 0x16, 0x8B, 0x75, 0x3B, 0x3F, 0x3F, 0x9C, 0xB1, 0x39, 0x2D, 0x7B, 0x87, 0x4D, 0xB5, 0xFC, 0x9D, 0x17, 0xD7, 0x50, 0x71, 0xA9, 0xEF, 0xE0, 0x46, 0x23, 0x1D, 0xB5, 0xA0, 0x82, 0xE8, 0xA4, 0x0E, 0xAE, 0x74, 0x8A, 0x64, 0xA0, 0x35, 0x28, 0x99, 0x8D, 0xF1, 0xD5, 0x55, 0xC3, 0x13, 0x57, 0xAD, 0xE2, 0x7F, 0x8C, 0x62, 0x3D, 0x4C, 0xD1, 0xAB, 0xCD, 0x92, 0x3C, 0xBA, 0x9A, 0xDA, 0xAC, 0x13, 0xB0, 0xE0, 0x40, 0x59, 0x5E, 0x72, 0x3D, 0x15, 0x9F, 0xE4, 0x27, 0x78, 0xC9, 0x9E, 0x89, 0xF5, 0x8E, 0x41, 0x7F, 0xA4, 0xEE, 0x75, 0x1F, 0x6E, 0xCA, 0xA8, 0x3A, 0xCD, 0x5A, 0xAF, 0x17, 0xF0, 0x38, 0xCB, 0xA8, 0x9E, 0xC8, 0x72, 0x4D, 0x2C, 0x0B, 0x36, 0x1A, 0xE9, 0x7F, 0xE1, 0xA6, 0xFB, 0xD5, 0xAB, 0x95, 0xFC, 0x88, 0x2E, 0x81, 0x9C, 0xDD, 0x8B, 0xC4, 0xE3, 0xEB, 0x30, 0x1D, 0xC1, 0xB3, 0xC9, 0x57, 0xAE, 0x48, 0x3C, 0x02, 0xDB, 0xF5, 0x98, 0xD3, 0xDC, 0x49, 0xDA, 0x22, 0x02, 0xA3, 0xED, 0xC9, 0x9F, 0xCE, 0x39, 0xDF, 0x7A, 0xC1, 0x30, 0xAB, 0xC3, 0x97, 0xDE, 0x8F, 0x07, 0x57, 0xCB, 0x22, 0xCD, 0xCD, 0x87, 0x7B, 0x61, 0x75, 0x3A, 0xE0, 0xCE, 0x17, 0x50, 0xCC, 0x40, 0x7F, 0x36, 0xBF, 0xC3, 0x0B, 0xB2, 0xE3, 0x24, 0x06, 0xBB, 0x2D, 0xFA, 0x09, 0x21, 0x3A, 0x6C, 0xEE, 0x7E, 0xAB, 0xFF, 0x4F, 0xBD, 0xFD, 0xA7, 0xF9, 0x2C, 0x3F, 0xF5, 0x52, 0xF1, 0xB4, 0x43, 0xBE, 0x94, 0x9A, 0x6B, 0x82, 0x9D, 0x10, 0xDB, 0xE6, 0xAA, 0xF2, 0xF6, 0xCA, 0x28, 0xE7, 0x34, 0x40, 0x10, 0xDB, 0x2F, 0xA2, 0x56, 0x0F, 0x90, 0x8A, 0xA9, 0xA1, 0x01, 0x64, 0x1D, 0x42, 0xD6, 0xE7, 0xD4, 0xD7, 0x87, 0xB0, 0x04, 0xFA, 0x9A, 0x40, 0x6D, 0x97, 0xB9, 0xA1, 0x79, 0x69, 0x4D, 0x9A, 0xEC, 0x3F, 0x85, 0x48, 0xFF, 0x24, 0x1F, 0xE9, 0xB0, 0x50, 0x08, 0x5F, 0x72, 0x6A, 0xA3, 0x76, 0xE5, 0x8D, 0x38, 0x5A, 0x44, 0x8C, 0xFD, 0x42, 0x2C, 0x79, 0x5D, 0x0D, 0x82, 0x0F, 0x8C, 0x6F, 0x44, 0x83, 0xB3, 0xD8, 0x6D, 0x0B, 0x0B, 0xE1, 0xB7, 0xD0, 0x53, 0xC5, 0x07, 0xF1, 0x81, 0x7C, 0x59, 0x25, 0x28, 0xAF, 0xDC, 0x6D, 0x4F, 0x46, 0x34, 0xDC, 0x9C, 0x99, 0x8F, 0xEA, 0x7A, 0x3F, 0x59, 0xA6, 0xA7, 0x80, 0x3E, 0xD1, 0xDF, 0xA3, 0xAD, 0x74, 0xBD, 0xC0, 0x5F, 0xAC, 0x9A, 0x31, 0xF9, 0x16, 0xC5, 0x71, 0x00, 0xB9, 0xC3, 0x99, 0x69, 0x4D, 0x8A, 0x3C, 0x88, 0x6C, 0x81, 0x98, 0x48, 0xA9, 0x59, 0xC1, 0x0D, 0x47, 0x01, 0xE6, 0x4E, 0xEA, 0xBA, 0x26, 0xB0, 0xB5, 0x8B, 0x0C, 0x08, 0xF7, 0xEB, 0x5B, 0x00, 0x68, 0xA6, 0x18, 0xE1, 0x8F, 0x03, 0x56, 0xC8, 0x47, 0xCC, 0xDC, 0xE6, 0x05, 0x14, 0xCD, 0x5E, 0xCF, 0x48, 0xF6, 0xD1, 0xAD, 0x80, 0xE3, 0x97, 0xE6, 0x47, 0x5F, 0x9F, 0x82, 0x97, 0x9A, 0x42, 0xF2, 0x5A, 0x22, 0x13, 0x6A, 0x0C, 0xDF, 0x4F, 0x16, 0x36, 0xE5, 0x96, 0x70, 0xCC, 0xE0, 0x53, 0x40, 0x42, 0x28, 0xCE, 0x02, 0x00, 0xC1, 0xD4, 0xE9, 0xE4, 0x18, 0x15, 0x7C, 0xD7, 0x22, 0xC1, 0x4B, 0xC8, 0xA2, 0x54, 0x9D, 0x39, 0x88, 0x6B, 0xDA, 0x25, 0xEF, 0x68, 0x4A, 0xF6, 0x50, 0x03, 0x12, 0xA0, 0x31, 0x15, 0x3F, 0xDC, 0xC4, 0x09, 0x4F, 0x02, 0x7E, 0x07, 0x8F, 0xBC, 0x6F, 0xA1, 0xD9, 0xEF, 0xEE, 0xF2, 0x9E, 0x17, 0xEE, 0x12, 0xC6, 0xDD, 0x2B, 0x0D, 0x01, 0xAE, 0x83, 0x9A, 0xE3, 0x06, 0xEB, 0x37, 0xF6, 0xFE, 0xB2, 0x93, 0x52, 0x4E, 0x5F, 0x66, 0x44, 0x56, 0x80, 0xE3, 0x3D, 0x2F, 0x48, 0xF5, 0x10, 0xD7, 0xAE, 0xC7, 0x19, 0xC3, 0xD4, 0x7C, 0x25, 0x8C, 0x2F, 0x30, 0xC8, 0xD2, 0xB0, 0xBF, 0x06, 0xAB, 0x7B, 0x30, 0x80, 0xAA, 0x86, 0x9A, 0x7D, 0x87, 0x73, 0x6F, 0x82, 0x7A, 0x9B, 0xD5, 0x7D, 0x66, 0x90, 0x8F, 0xCC, 0xA9, 0x57, 0xF8, 0x89, 0xB2, 0x64, 0x53, 0x73, 0xED, 0x11, 0x2F, 0xE3, 0x96, 0x93, 0xA3, 0x54, 0x5B, 0x80, 0x7D, 0x63, 0x4C, 0xFE, 0x7E, 0x3D, 0x42, 0x17, 0xC5, 0x94, 0x54, 0x75, 0x0A, 0x17, 0x08, 0x2B, 0x8F, 0xF8, 0x90, 0xA8, 0x25, 0x8A, 0x1D, 0xBB, 0x5B, 0x02, 0x14, 0x68, 0x4E, 0xE9, 0x38, 0x23, 0x97, 0x69, 0xF3, 0x1A, 0xE6, 0xA5, 0x5A, 0x99, 0x44, 0xB8, 0x6C, 0x42, 0x28, 0xDD, 0x51, 0x7D, 0x88, 0xC0, 0x87, 0x2F, 0xB6, 0xBE, 0xA1, 0xF2, 0x32, 0x08, 0x3F, 0xA0};

	if(c_firmware>=4.20f && c_firmware<=4.60f)
	{
		if(strstr(_path, "/dev_usb"))
		{
			u8 usbd=(_path[8]-0x30)*100 + (_path[9]-0x30)*10 + _path[10]-0x30;
			if(usbd<=127)
			{
				char aiodata[128];
				sprintf(aiodata, "%s/AIOMOD.BIN", app_sys);
				if(cellFsStat(aiodata, &buf2)==CELL_FS_SUCCEEDED)
				{
					int fdw;
					uint64_t msiz = 0;
					if(cellFsOpen(aiodata, CELL_FS_O_RDONLY, &fdw, 0,0)==CELL_FS_SUCCEEDED)
					{
						cellFsLseek(fdw, 0x20+(usbd*0x8e07), CELL_FS_SEEK_SET, &msiz);
						cellFsRead(fdw, (void *)&libfs_new, 0x8e07, &msiz);
						cellFsClose(fdw);

						if(msiz==0x8e07 && cellFsOpen(expplg, CELL_FS_O_CREAT | CELL_FS_O_TRUNC | CELL_FS_O_WRONLY, &fdw, 0,0)==CELL_FS_SUCCEEDED)
						{
							cellFsWrite(fdw, (void *)&libfs_new, 0x8e07, &msiz);
							cellFsClose(fdw);
							cellFsChmod(expplg, 0666);
						}
					}
				}
			}
		}
		else
		{
			int fdw;
			uint64_t msiz = 0;
			if(cellFsOpen(expplg, CELL_FS_O_CREAT | CELL_FS_O_TRUNC | CELL_FS_O_WRONLY, &fdw, 0,0)==CELL_FS_SUCCEEDED)
			{
				cellFsWrite(fdw, (void *)&libfs_new, 0x8e07, &msiz);
				cellFsClose(fdw);
				cellFsChmod(expplg, 0666);
			}
		}
	}

	if(do_eject)
	{
		add_to_map((char*)"/dev_bdvd", path);

		char path2[strlen(_path)+24];

		sprintf(path2, "%s/PS3_GAME", _path);
		add_to_map((char*)"/app_home/PS3_GAME", path2);

		sprintf(path2, "%s/PS3_GAME/USRDIR", _path);
		add_to_map((char*)"/app_home/USRDIR", path2);

		sprintf(path2, "%s/PS3_GAME/USRDIR/", _path);
		add_to_map((char*)"/app_home/", path2);
	}

	add_to_map((char*)"/app_home", path);

	if(c_firmware==4.21f)
		sprintf(expplg, "%s/IEXP0_420.BIN", app_sys);
	else if(c_firmware==4.30f || c_firmware==4.31f)
		sprintf(expplg, "%s/IEXP0_430.BIN", app_sys);
	else if(c_firmware==4.40f || c_firmware==4.41f)
		sprintf(expplg, "%s/IEXP0_440.BIN", app_sys);
	else if(c_firmware==4.46f)
		sprintf(expplg, "%s/IEXP0_446.BIN", app_sys);
	else if(c_firmware==4.50f || c_firmware==4.53f || c_firmware==4.55f)
		sprintf(expplg, "%s/IEXP0_450.BIN", app_sys);
	else if(c_firmware==4.60f)
		sprintf(expplg, "%s/IEXP0_460.BIN", app_sys);
	else
        sprintf(expplg, "%s/none", app_sys);

	if(do_eject && cellFsStat(expplg, &buf2)==CELL_FS_SUCCEEDED)
		add_to_map( (char*)"/dev_flash/vsh/module/explore_plugin.sprx", expplg);


	sprintf(expplg, "%s/ILFS0_000.BIN", app_sys);
	if(do_eject && cellFsStat(expplg, &buf2)==CELL_FS_SUCCEEDED)
		add_to_map((char*) "/dev_flash/sys/external/libfs.sprx", expplg);

	u64 map_data  = (MAP_BASE);
	u64 map_paths = (MAP_BASE) + (max_mapped+1) * 0x20;

	for(u8 n=0; n<32; n++)
	{
		pokeq(map_data + (n*32) + 0x00, 0);
		pokeq(map_data + (n*32) + 0x08, 0);
		pokeq(map_data + (n*32) + 0x10, 0);
		pokeq(map_data + (n*32) + 0x18, 0);
	}

	if(!max_mapped) return;

	for(u8 n=0; n<max_mapped; n++)
	{
		if(map_paths>0x80000000007FE800ULL) break;
		pokeq(map_data + (n*32) + 0x10, map_paths);
		string_to_lv2(file_to_map[n].src, map_paths);
		map_paths+= (strlen(file_to_map[n].src)+8)&0x7f8;

		pokeq(map_data + (n*32) + 0x18, map_paths);
		string_to_lv2(file_to_map[n].dst, map_paths);
		map_paths+= (strlen(file_to_map[n].dst)+8)&0x7f8;

		pokeq(map_data + (n*32) + 0x08, strlen(file_to_map[n].dst));
		pokeq(map_data + (n*32) + 0x00, strlen(file_to_map[n].src));
	}

	if(cellFsStat((char*)"/dev_bdvd", &buf2)==CELL_FS_SUCCEEDED)
		sys_timer_sleep(2);

	//if(do_eject) eject_insert(0, 1);
#endif
	max_mapped=0;
}
